#!/vlt/JAN2006/gnu/bin/perl


use Getopt::Std;
getopts('mhv');

$mirror = 0;

if(defined $opt_m){
#    print "-s $opt_m\n";
    $mirror = 1;
}
$verbose=0;

if(defined $opt_v){
  $verbose=1;
}

if(defined $opt_h){
    print "\nUsage: gerber2eps -m -v infile [ outfile ]\n\n";
    print "DESCRIPTION:\n";
    print "gerber2eps converts Gerber 274X Artwork (.art) files to\n";
    print "encapsulated Postscript (.epsf) format.\n"; 
    print "gerber2eps is only tested with Cadence, Cadstar and Protel Artwork files !!!\n\n";
    print "-m creates a mirrored picture\n\n"; 
    print "-v creates more messages\n";
    exit 1;
}

# Say Hello:

print "This is Andy's Gerber274X to EPS Utility, Ver. 0.00\n";


# Open Input- and Output-Files
 
unless ($#ARGV > -1)               # Identify arguments
{
    print "Error: No Input File specified!\n";
    exit 1;
}

$infile = $ARGV[0];                  # Try to open Input
unless(open(IN,"<".$infile)){
    print "Input-File $infile is empty, I will try $infile".".art!\n";
    $infile = $infile.".art";          # forgotten .art ending ?
    unless(open(IN,"<".$infile)){
	print "Error: Input-File $infile is empty, too!\n";
	exit 1;
    }
}	  

@intext = <IN>;
close(IN);
print "Input-File:  $infile \n";
$infiletypeOK = 0;
foreach(@intext){
    if($_ =~ m|^.*%FS|){
	$infiletypeOK = 1;
    }
}
if($infiletypeOK == 1){	
    print "Input-File is Gerber 274X File!\n";
    # print "i.e. I found a line matching ...%FS...\n";
}
else{
    print "Error: Input-File seems not to be a Gerber 274X  File!\n";
    exit 1;
}

if($#ARGV > 0){$outfile = $ARGV[1];}
else{$outfile = $infile.".epsf";}
unless(open(OUT,">".$outfile)){
    die "Error: Cannot open Output-File!\n";
}
print "Output-File: $outfile \n";
$warnings = 0;                           # count warnings
$aperture_warnings = 0;                          # count warnings
# Initialize Postscript-File

print OUT "%!PS-Adobe-2.0 EPSF-2.0\n";
print OUT "%%Title: EPS-File converted from Gerber 274X Format\n";
print OUT "%%Creator: Andy's gerber2eps Utility, Ver. 0.00\n";
print OUT "%%For: $ENV{USER}"."@"."$ENV{HOST}\n";
print OUT "%%Pages: 0\n";
print OUT "%%BoundingBox: -30 -50 560 820\n";
print OUT "%%DocumentFonts: (atend)\n";
print OUT "%%EndComments\n";
print OUT "% Input-File: $ENV{PWD}/$infile\n";
print OUT "\n";


print OUT "%%EndProlog\n";
print OUT "\n";

# Start with real paintings

$Fonts = "Courier";

$bbdefined = 0;
$xmax = 0;                       # We want to track the Bounding Box
$xmin = 0;                       # these vars are updated in every drawing 
$ymax = 0;                       # operation
$ymin = 0;


# First, we look for the units, scale of the main aperture, and
# translation of the main aperture!
# All comments of the input file are collected at the beginning

$dec_style=0;
$coord_mode=0;
$seq_number=0;
$prep_func_code=0;
$x_digits_l=0;
$x_digits_t=0;
$y_digits_l=0;
$y_digits_t=0;
$z_digits_l=0;
$z_digits_t=0;
$draft_code=0;
$misc_code=0;

$image_pol=1;
$image_rot=0;

$units = 72 / 25.4;
$unitname = "xx";

$ascale=1.0;
$bscale=1.0;
$aoffset=0.0;
$boffset=0.0;
$line = 0;

while($line < $#intext + 1){
  if($intext[$line]=~ m|^G04|){         # it's a comment-line!
    &comment;
  }
  else{
    if($intext[$line]=~ m|^G01\*|){        
#     print "Puuhh: We luckily have linear interpolation\n";
    }
    if($intext[$line]=~ m|^G75\*|){        
#      print "Hrrgh: They threaten us with Circular Interpolation!\n";
    }
    if($intext[$line]=~ m|.*FS([NLTD])([AI])(N(\d*))?(G(\d*))?X(\d)(\d)Y(\d)(\d)(Z(\d)(\d))?(D(\d*))?(M(\d*))?.*|){    # Embedded Format statement
      $dec_style=$1;
      $coord_mode=$2;
#	    $seq_number=$4;
#	    $prep_func_code=$6;
      $x_digits_l=$7;
      $x_digits_t=$8;
      $y_digits_l=$9;
      $y_digits_t=$10;
#	    $z_digits_l=$12;
#	    $z_digits_t=$13;
#	    $draft_code=$15;
#	    $misc_code=$17;
      print "Decimal style is: $dec_style\n";
      print "Coordinate mode is: $coord_mode\n";
      print "Sequence number is: $seq_number\n";
      print "Preparatory function code is: $prep_func_code\n";
      print "Format of X input data is: $x_digits_l.$x_digits_t\n";
      print "Format of Y input data is: $y_digits_l.$y_digits_t\n";
      print "Format of Z input data is: $z_digits_l.$z_digits_t\n";
      print "Draft code is: $draft_code\n";
      print "Misc code is: $misc_code\n";
      ($x_digits_l==0) && ($x_digits_t==0) && die "no x_digits defined\n";
      ($y_digits_l==0) && ($y_digits_t==0) && die "no y_digits defined\n";
    }
    if($intext[$line]=~ m|.*IP.*|){    # define units, scaling the painting
      if($intext[$line]=~ m|.*IR(\d*(\.\d*)?).*|){
	$image_rot=$1;
	print "Image Rotation is $image_rot Degrees\n";
	print OUT "% Painting is rotated by $image_rot Degrees\n";
	print OUT "$image_rot rotate\n";
      }
      if($intext[$line]=~ m|.*IPPOS.*|){
	$image_pol=1;
	print "Image polarity is positive\n";
      }
      elsif($intext[$line]=~ m|.*IPNEG.*|){
	$image_pol=-1;
	print "Image polarity is negative\n";
      }
    }		
    if($intext[$line]=~ m|.*MO.*|){    # define units, scaling the painting
      if($unitname == "xx"){
	if($intext[$line]=~ m|.*MOMM.*|){    # size(\d*.\d*)
	  $units = 72 / 25.4;
	  $unitname = "mm";
	  print "MO says: Units are Millimeters\n";
	  print OUT "% MO says: Units are Millimeters\n";
	}
	elsif($intext[$line]=~ m|.*MOIN|){
	  $units = 72;
	  $unitname = "inch";
	  print "MO says: Units are Inches\n";
	  print OUT "% MO says: Units are Inches\n";
	}
	print OUT "$units $units scale\n";
      }
      else {
	die "Multiple Definition of Units!\n";
      }
    }
    if($intext[$line]=~ m|^G70|){    # define units, scaling the painting
      if($unitname == "xx"){
	$units = 72;
	$unitname = "inch";
	print "G70 says: Units are Inches\n";
	print OUT "% G70 says: Units are Inches\n";
	print OUT "$units $units scale\n";
      }
      else {
	die "Multiple Definition of Units!\n";
      }
    }
    if($intext[$line]=~ m|^G71|){    # define units, scaling the painting
      if($unitname == "xx"){
	$units = 72 / 25.4;
	$unitname = "mm";
	print "G71 says: Units are Millimeters\n";
	print OUT "% G71 says: Units are Millimeters\n";
	print OUT "$units $units scale\n";
      }
      else {
	die "Multiple Definition of Units!\n";
      }
    }
    if($intext[$line]=~ m|.*MIA(\d)B(\d).*|){ #Mirror
      if($1==1||$2==1) {
	if ($mirror == 1) {
	  $mirror = 0;
	  print "WARNING: Picture is mirrored twice, i.e. it is not mirrored!\n";
	  $warnings++;
	}
	else {
	  $mirror=1;
	}
      }
    }
    if($intext[$line]=~ m|.*SFA(\d*.\d*)B(\d*.\d*).*|){ #Scale of the painting
      $ascale=$1;
      $bscale=$2;
      print "painting is scaled with a: $ascale and b: $bscale\n";
    }
    if($intext[$line]=~ m|.*OFA(\d*.\d*)B(\d*.\d*).*|){ #Scale of the painting
      $aoffset=$1;
      $boffset=$2;
      print "painting is offset by  $aoffset $boffset\n";
    }
  }
  $line++;
}
      if ($mirror==1) {
	print "painting is mirrored!\n";
	print OUT "% painting is mirrored!\n";
	$ascale=-$ascale;
      }
      else {
	print "painting is not mirrored!\n";
	print OUT "% painting is not mirrored!\n";
      } 
      print OUT "% Painting is scaled with a: $ascale and b: $bscale\n";
      print OUT "$ascale $bscale scale\n";
      print OUT "% painting is offset by  $aoffset $boffset\n";
      print OUT "$aoffset $boffset translate\n";

# now start with the real things

$linewidth{"Null"} = 0;
$bbadd{"Null"} = 0;
$actualaper = "Null";
$current_pol=1;
$circ_mode360=0;
$draw_now=0;

print OUT "% round the linecaps\n";
print OUT "1 setlinecap\n";
print OUT "1 setlinejoin\n";

$line = 0;
$xcoord = 0;    # because some DPF codes like DRAW only update new
$ycoord = 0;    # coordinates, we have to save the actual point


while($line < $#intext + 1){
  if($intext[$line]=~ m|^G04.*| ||
     $intext[$line]=~ m|^G01\*.*| ||   # Comments
     $intext[$line]=~ m|^G70\*.*| ||   # 
     $intext[$line]=~ m|^G71\*.*| ||   # It's one of the first lines
     $intext[$line]=~ m|^.*%FS.*| ||   # don't know what this means
     $intext[$line]=~ m|^.*%MO.*| ||
     $intext[$line]=~ m|^%.*IP.*| ||
     $intext[$line]=~ m|^%SFA.*| ||
     $intext[$line]=~ m|^%OFA.*| ||
     $intext[$line]=~ m|^%MIA.*| ||
     $intext[$line]=~ m|^\*\s*$| ||           #comment or initial string
     $intext[$line]=~ m|^\s*$|){           #comment or initial string
  }
  elsif($intext[$line]=~ m|^G75.*|){#Enable 360deg circular interpolation
    $circ_mode360=1;
    #      print "G75: Switching to 360deg Mode\n";
  }
  elsif($intext[$line]=~ m|^G74.*|){#Disable 360deg circular interpolation
    $circ_mode360=0;
    #      print "G74: Switching off 360deg Mode\n";
  }
  elsif($intext[$line]=~ m|^%LN(\w+)|){  #a layer name, cry out!
    print "\n Processing layer $1\n";
    print OUT "%Processing layer $1\n";
  }
  elsif($intext[$line]=~ m|^%LPC.*|){  #what follows is negative
    print "\n We are now drawing clearances\n";
    print OUT "% We are now drawing clearances\n";
    $current_pol=-1;
    if($image_pol * $current_pol < 0){
      print OUT "1 setgray\n";
    }
    else{
      print OUT "0 setgray\n";
    }
  }
  elsif($intext[$line]=~ m|^%LPD.*|){  #what follows is positive
    print "\n We are now drawing dark stuff\n";
    print OUT "% We are now drawing dark stuff\n";
    $current_pol=1;
    if($image_pol * $current_pol < 0){
      print OUT "1 setgray\n";
    }
    else{
      print OUT "0 setgray\n";
    }
  }
  elsif($intext[$line]=~ m|^%AM.*|){  # a new aperture macro is defined !
    $tmp = $intext[$line];
    while(!($intext[$line]=~ m|^.*%\s*$|)){ # macro def can be more
      chop($tmp);                 # than one line, but is definitely
                                  #ended by *%
      $tmp = join(' ',$tmp,$intext[$line + 1]); 
#      	    print "TMPjoin = $tmp%\n";
      $line++;
    }
    &macrodef($tmp);
  }
  elsif($intext[$line]=~ m|^%ADD\d+.*|){  # a new aperture is defined !
    &aperdef($intext[$line]);
  }
  elsif($intext[$line]=~ m|^(G54)?(D(\d+)).*|&&$3>9){ # a aperture is selected
    $actualaper=$2;
    print "\nActual aperture is $actualaper: ";
    print OUT "$linewidth{$actualaper} setlinewidth\n";
  }
  elsif($intext[$line]=~ m|^G36.*|){     #a polygon drawing, we have to split
    $tmp = $intext[$line];
    while(!($intext[$line + 1]=~ m|^G37.*|)){ # macro def can be more
      chop($tmp);                 # than one line, but is definitely
                                  #ended by G37
      $tmp = join(' ',$tmp,$intext[$line + 1]); 
      #	    print "TMPjoinpoly = $tmp%\n";
      $line++;
    }
    $line++;
#    $tmp =~ s/G\d+//g;
# 	print "TMPfinalpoly = $tmp%\n";
    &polygon($tmp);
  }
  
  
  elsif($intext[$line]=~ m|^(G01)?X?[\.\-\d]*Y?[\.\-\d]*D01\*| ||
       ($intext[$line]=~ m|^(G01)?X?[\.\-\d]*Y?[\.\-\d]*\*| && $draw_now == 1)){
    $tmp = $intext[$line];              # it's a  draw
    $tmp =~ s/G01//g;
    $tmp =~ s/D01//g;
    &draw($tmp,1);
    $draw_now=1;
  }
  elsif($intext[$line]=~ m|^(G01)?X?[\.\-\d]*Y?[\.\-\d]*D02\*| ||
	($intext[$line]=~ m|^(G01)?X?[\.\-\d]*Y?[\.\-\d]*\*| && $draw_now == 0)){  
    $tmp = $intext[$line];             # it's a move without draw
    $tmp =~ s/G01//g;
    $tmp =~ s/D02//g;
    &move($tmp,1);
    $draw_now=0;
  }
  elsif($intext[$line]=~ m|.*D03\*|){  # it's a flash
    $tmp = $intext[$line];
    $tmp =~ s/G01//g;
    $tmp =~ s/D03//g;
    &flash($tmp);
  }
  elsif($intext[$line]=~ m|^G02.*D01\*| || 
       ($intext[$line]=~ m|^G02.*| && $draw_now == 1)){  # it's a clockwise arc
    $tmp = $intext[$line];
    $tmp =~ s/G02//g;
    $tmp =~ s/D01//g;
    &carc($tmp,1);
    $draw_now=1;
  }
  elsif($intext[$line]=~ m|^G03.*D01\*| || 
($intext[$line]=~ m|^G03.*| && $draw_now == 1)){  # it's a counterclock arc
    $tmp = $intext[$line];
    $tmp =~ s/G03//g;
    $tmp =~ s/D01//g;
    &ccarc($tmp,1);
    $draw_now=1;
  }
  elsif($intext[$line]=~ m|M02.*|){  # it's the end
    print "\nEnd designator M02 reached\n";
    print OUT "% End designator M02 reached\n";
  }
  else{
    if(draw_now==0){
      print "\nWARNING (main): I cannot handle: $intext[$line] Last action was not drawing!\n";
    }
    else {
      print "\nWARNING (main): I cannot handle: $intext[$line] Last action was drawing!\n";
    }
    $warnings++;
  }
  $line++;
}



$BBx = $xmin * $units;                       # calculate the Bounding Box
$BBy = $ymin * $units;
$BBw = ($xmax - $xmin) * $units;
$BBh = ($ymax - $ymin) * $units; 

print OUT "showpage\n";

print OUT "%%Trailer\n";
$BBurx=$BBw+$BBx;
$BBury=$BBh+$BBy;
if ($mirror==1) {
$xxx=$BBx;
$BBx=-$BBurx;
$BBurx=-$xxx;
}

print OUT "%%BoundingBox: $BBx $BBy $BBurx $BBury\n";
print OUT "%%DocumentFonts: $Fonts\n";

close(OUT);
if($aperture_warnings > 1){
    print "\nWARNING: I found $aperture_warnings Apertures with Size 0!\nProbably caused by an undefined Linewidth for Text!\n\n";
}
elsif($aperture_warnings == 1){
    print "\nWARNING: I found $aperture_warnings Aperture with Size 0!\nProbably caused by an undefined Linewidth for Text!\n\n";
}
$warnings=$warnings+$aperture_warnings;
if($warnings <1){
    print "\nConversion succeeded!\n\n";
}
elsif($warnings == 1){
    print "\nSorry, there was $warnings warning!\n\n";
}   
else{
    print "\nSorry, there were $warnings warnings!\n\n";
}   

print "\ngerber2eps.perl end\n";


# Subroutines for parsing

# treat comments

sub comment
{
    $string = $intext[$line];
    chop($string);
    print OUT "%$string\n";
}

sub xcoord2real
{
  local($input)=@_;
  local($output)=0;
  local($divider)=1.0;
#  print "xcoord2real got: $input\n";
  if($dec_style eq "D"){
    $output =  $input;
    print "xcoord2real D input: $input returned value $output\n";
  }
  elsif($dec_style eq "L" || $dec_style eq "N"){   #leading or no zeroes are suppressed
    $output=$input/exp($x_digits_t*log(10)); 
    #	print "xcoord2real L input: $input returned value $output\n";
  }
  elsif($dec_style eq "T"){   #trailing zeroes are suppressed
    #	chop($input);
    if($input =~ m|-\d+|){ # take care of negative numbers
      $output=$input/exp((length($input)-1-$x_digits_l)*log(10)); 
    }
    else {
      $output=$input/exp((length($input)-$x_digits_l)*log(10)); 
    }    
#    print "xcoord2real T input: $input returned value $output\n";
  }
  $output;
}

sub ycoord2real
{
  local($input)=@_;
  local($output)=0;
  local($divider)=1.0;
  #    print "ycoord2real got: $input\n";
  if($dec_style eq "D"){ #do nothing
    $output =  $input;
    print "ycoord2real D input: $input returned value $output\n";
  }
  elsif($dec_style eq "L" || $dec_style eq "N"){   #leading or no zeroes are suppressed
    $output=$input/exp($y_digits_t*log(10)); 
    #	print "ycoord2real L input: $input returned value $output\n";
  }
  elsif($dec_style eq "T"){   #trailing zeroes are suppressed
    #	chop($input);
    if($input =~ m|-\d+|){ # take care of negative numbers
      $output=$input/exp((length($input)-1-$x_digits_l)*log(10)); 
    }
    else {
      $output=$input/exp((length($input)-$y_digits_l)*log(10)); 
    }
    #	print "ycoord2real T input: $input returned value $output\n";
  }
  $output;
}
	
	
	

# treat aperture definitions

sub macrodef
{
    local($text) = @_;
    local ($rot, $vertices, $coords, $macrotype, @tmp, $vertex, $x_y, @x, @y);
    local($xrel, $yrel);
    print "macrodef text = $text";
    if ($text =~ m|^%AM(\w+)\*\s*(\d+),.*|) {
	$macroname = $1;                      # it's a macro 
	$macrotype=$2;                            
        $linewidth{$macroname} = 0;
	if($macrotype == 4){  # it's an outline with max 50 vertices
	    if($text =~ m|^%AM(\w+)\*\s*(\d+),(\d+),(\d+),(.*)|){
		$vertices=$4;
		$coords=$5;
		$coords =~ s/\s//g;
#		print "coords = $coords\n";
		print "Treating an outline-macro $macroname with $vertices vertices\n";
		print OUT "% Macro $macroname with $vertices vertices\n";
		print OUT "/$macroname { moveto\n"; # save origin 
		@tmp=split(/,/,$coords);                   # on stack
		$vertex=0;
		$x_y=0;
		foreach(@tmp){
		    if($_ =~ m|(-?\d*(\.\d*)?).*$|){
#			print "matched text is $1\n";
			if($vertex <= $vertices){
			    if($x_y == 0){
				$x[$vertex] = $1;
				$x_y =1;
#				print "x$vertex = $x[$vertex]\n";
			    }
			    else{
				$y[$vertex] = $1;
#				print "y$vertex = $y[$vertex]\n";
				$x_y=0;
				$vertex++;
			    }
			}
			else{
			    $rot = $1;
#			    print "rot = $rot\n";
			}
		    }
		}
		print OUT "$rot rotate\n";
		print OUT "$x[0] $y[0] rmoveto\n";
		$vertex = 1;
		while($vertex <= $vertices){
		    $xrel= $x[$vertex]-$x[$vertex-1];
		    $yrel= $y[$vertex]-$y[$vertex-1];
		    print OUT "$xrel $yrel rlineto\n";
		    $vertex++;
		}
		print OUT "closepath\nfill\n";
		$rot=-$rot;
		print OUT "$rot rotate \n";
		print OUT "} def\n";
		$vertex=0;
		$bbadd{$macroname} = 0;
		while($vertex <= $vertices){
		    if(abs($x[$vertex]>$bbadd{$macroname})){
			$bbadd{$macroname}=abs($x[$vertex]);
		    }
		    elsif(abs($y[$vertex]>$bbadd{$macroname})){
			$bbadd{$macroname}=abs($y[$vertex]);
		    }
		    $vertex++;
		}

	    }	    
	    else{
		die "Outline-Macro $macroname is confusing \n";
	    }

	}
	else{
	    die "Macro $macroname is not a known type \n";
	}
    }
    else{
	die "macrodef was not correctly called with string:\n $text\n";
    }
}

  
sub aperdef
{
    local($text) = @_;
    local($r, $apername, $diameter, $macroname, $w, $h, $w2, $h2);

    if($text =~ m|^%AD(D\d+)C,(-?\d*(\.\d*)?)|){     # it's a circle
	$apername = $1;
	$diameter = $2;
	print "\n$apername is a circle with $diameter$unitname diameter";
	print OUT "% $apername is a circle with $diameter$unitname diameter\n";
	$r = $diameter /2;
	print OUT "/$apername { 2 copy\n$r 0 360 arc\n";
      	print OUT "closepath\nfill\n";
	if($text =~ m|^%AD(D\d+)C,(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)|){
	  $holex=$4;
	  $holey=$6;
	  print "\n  $apername has a rectangular hole with dimension $holex$unitname x $holey$unitname";
	  if($image_pol * $current_pol < 0){
	    print OUT "0 setgray\n";
	  }
	  else{
	    print OUT "1 setgray\n";
	  }
	  $holex_2=$holex/2.0;
	  $holey_2=$holey/2.0;
	  print OUT "moveto\n";
	  print OUT "-$holex_2 -$holey_2 rmoveto\n";
	  print OUT "$holex 0 rlineto\n";
	  print OUT "0 $holey rlineto\n";
	  print OUT "-$holex 0 rlineto\n";
	  print OUT "closepath\nfill\n";
	  if($image_pol * $current_pol < 0){
	    print OUT "1 setgray\n";
	  }
	  else{
	    print OUT "0 setgray\n";
	  }
	}
	elsif($text =~ m|^%AD(D\d+)C,(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)|){
	  $holediameter=$4;
	  print "\n  $apername has a round hole with $holediameter$unitname diameter";
	  if($image_pol * $current_pol < 0){
	    print OUT "0 setgray\n";
	  }
	  else{
	    print OUT "1 setgray\n";
	  }
	  $holer = $holediameter /2;
	  print OUT "$holer 0 360 arc\n";
	  print OUT "closepath\nfill\n";
	  if($image_pol * $current_pol < 0){
	    print OUT "1 setgray\n";
	  }
	  else{
	    print OUT "0 setgray\n";
	  }
	}
	else {
	  print OUT "pop pop\n";
	}
	print OUT "} def\n";
	$linewidth{$apername} = $diameter;
	if ($linewidth{$apername}==0) {
	  print "\nWARNING: This aperture has diameter 0!\n";
	  $aperture_warnings++;
	}
	$bbadd{$apername} = $diameter / 2;
	$apertype{$apername} = "circle";
    }
    elsif($text =~ m|^%AD(D\d+)R,(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?).*|){ 
	$apername = $1;                            # it's a rectangle
	$w = $2;
	$h = $4; 
	$w2 = $w / 2;
	$h2 = $h / 2;
	print "\n$apername is a rect $w x $h$unitname";
	print OUT "% $apername is a rect $w x $h$unitname\n";
	print OUT "/$apername { 2 copy moveto\n";
	print OUT "-$w2 -$h2 rmoveto\n";
	print OUT "$w 0 rlineto\n";
	print OUT "0 $h rlineto\n";
	print OUT "-$w 0 rlineto\n";
	print OUT "0 -$h rlineto\n";
	print OUT "closepath\nfill\n";
	if($text =~ m|^%AD(D\d+)R,(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)|){
	  $holex=$6;
	  $holey=$8;
	  print "\n  $apername has a rectangular hole with dimension $holex$unitname x $holey$unitname";
	  if($image_pol * $current_pol < 0){
	    print OUT "0 setgray\n";
	  }
	  else{
	    print OUT "1 setgray\n";
	  }
	  $holex_2=$holex/2.0;
	  $holey_2=$holey/2.0;
	  print OUT "moveto\n";
	  print OUT "-$holex_2 -$holey_2 rmoveto\n";
	  print OUT "$holex 0 rlineto\n";
	  print OUT "0 $holey rlineto\n";
	  print OUT "-$holex 0 rlineto\n";
	  print OUT "closepath\nfill\n";
	  if($image_pol * $current_pol < 0){
	    print OUT "1 setgray\n";
	  }
	  else{
	    print OUT "0 setgray\n";
	  }
	}
	elsif($text =~ m|^%AD(D\d+)R,(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)|){
	  $holediameter=$6;
	  print "\n  $apername has a round hole with $holediameter$unitname diameter";
	  if($image_pol * $current_pol < 0){
	    print OUT "0 setgray\n";
	  }
	  else{
	    print OUT "1 setgray\n";
	  }
	  $holer = $holediameter /2;
	  print OUT "$holer 0 360 arc\n";
	  print OUT "closepath\nfill\n";
	  if($image_pol * $current_pol < 0){
	    print OUT "1 setgray\n";
	  }
	  else{
	    print OUT "0 setgray\n";
	  }
	}
	else {
	  print OUT "pop pop\n";
	}
         print OUT "} def\n";
	$linewidth{$apername} = $h;
	if ($linewidth{$apername}==0) {
	  print "\nWARNING: This aperture has diameter 0!";
	  $aperture_warnings++;
	}
	$bbadd{$apername} = sqrt($w * $w + $h * $h)/2;
	$apertype{$apername} = "rect";
	if($w>0.0){
	  if(abs($w-$h)/$w < 0.001){
	    $apertype{$apername} = "square";
#	    print "  Hey! This rectangle is actually a square\n";
	  }
	}
	elsif($w==0 && $h==0){
	    $apertype{$apername} = "square";
	}
#	print "linewidth ($apername) = $linewidth{$apername}$unitname\n";
#	print "bbadd ($apername) = $bbadd{$apername}$unitname\n";
    }
    elsif($text =~ m|^%AD(D\d+)O,(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?).*|){ 
	$apername = $1;                                      # it's a oblong
	$w = $2;
	$h = $4; 
	$w2 = $w / 2 - $h/2;
	$w3 = $w-$h;
	$h2 = $h / 2 - $w/2;
	$h3=$h-$w;
	print "\n$apername is a oblong $w x $h$unitname";
	print OUT "% $apername is a oblong $w x $h$unitname\n";
	print OUT "/$apername { 2 copy moveto\n";
	if($w >$h){                               # oblong is horizontal
	  print OUT "gsave\n";
	  print OUT "$h setlinewidth\n";
	  print OUT "-$w2 0 rmoveto\n";
	  print OUT "$w3 0 rlineto\n";
	  print OUT "stroke\n";
	  print OUT "grestore\n";
	  print OUT "newpath\n";
	  if($text =~ m|^%AD(D\d+)O,(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)|){
	    $holex=$6;
	    $holey=$8;
	    print "\n  $apername has a rectangular hole with dimension $holex$unitname x $holey$unitname";
	    if($image_pol * $current_pol < 0){
	      print OUT "0 setgray\n";
	    }
	    else{
	      print OUT "1 setgray\n";
	    }
	    $holex_2=$holex/2.0;
	    $holey_2=$holey/2.0;
	    print OUT "moveto\n";
	    print OUT "-$holex_2 -$holey_2 rmoveto\n";
	    print OUT "$holex 0 rlineto\n";
	    print OUT "0 $holey rlineto\n";
	    print OUT "-$holex 0 rlineto\n";
	    print OUT "closepath\nfill\n";
	    if($image_pol * $current_pol < 0){
	      print OUT "1 setgray\n";
	    }
	    else{
	      print OUT "0 setgray\n";
	    }
	  }
	  elsif($text =~ m|^%AD(D\d+)O,(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)|){
	    $holediameter=$6;
	    print "\n  $apername has a round hole with $holediameter$unitname diameter";
	    if($image_pol * $current_pol < 0){
	      print OUT "0 setgray\n";
	    }
	    else{
	      print OUT "1 setgray\n";
	    }
	    $holer = $holediameter /2;
	    print OUT "$holer 0 360 arc\n";
	    print OUT "closepath\nfill\n";
	    if($image_pol * $current_pol < 0){
	      print OUT "1 setgray\n";
	    }
	    else{
	      print OUT "0 setgray\n";
	    }
	  }
	  else {
	    print OUT "pop pop\n";
	  }
	  print OUT "} def\n";
	  $linewidth{$apername} = $h;
	  if ($linewidth{$apername}==0) {
	    print "\n WARNING: This aperture has diameter 0!";
	    $aperture_warnings++;
	  }
	  $bbadd{$apername} = $w;
	}
	else{                                     # oblong is vertical
	  print OUT "gsave\n";
	  print OUT "$w setlinewidth\n";
	  print OUT "0 -$h2 rmoveto\n";
	  print OUT "0 $h3 rlineto\n";
	  print OUT "stroke\n";
	  print OUT "grestore\n";
	  print OUT "newpath\n";
	  if($text =~ m|^%AD(D\d+)O,(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)|){
	    $holex=$6;
	    $holey=$8;
	    print "\n  $apername has a rectangular hole with dimension $holex$unitname x $holey$unitname";
	    if($image_pol * $current_pol < 0){
	      print OUT "0 setgray\n";
	    }
	    else{
	      print OUT "1 setgray\n";
	    }
	    $holex_2=$holex/2.0;
	    $holey_2=$holey/2.0;
	    print OUT "moveto\n";
	    print OUT "-$holex_2 -$holey_2 rmoveto\n";
	    print OUT "$holex 0 rlineto\n";
	    print OUT "0 $holey rlineto\n";
	    print OUT "-$holex 0 rlineto\n";
	    print OUT "closepath\nfill\n";
	    if($image_pol * $current_pol < 0){
	      print OUT "1 setgray\n";
	    }
	    else{
	      print OUT "0 setgray\n";
	    }
	  }
	  elsif($text =~ m|^%AD(D\d+)O,(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)X(-?\d*(\.\d*)?)|){
	    $holediameter=$6;
	    print "\n  $apername has a round hole with $holediameter$unitname diameter";
	    if($image_pol * $current_pol < 0){
	      print OUT "0 setgray\n";
	    }
	    else{
	      print OUT "1 setgray\n";
	    }
	    $holer = $holediameter /2;
	    print OUT "$holer 0 360 arc\n";
	    print OUT "closepath\nfill\n";
	    if($image_pol * $current_pol < 0){
	      print OUT "1 setgray\n";
	    }
	    else{
	      print OUT "0 setgray\n";
	    }
	  }
	  else {
	    print OUT "pop pop\n";
	  }
	  print OUT "} def\n";
	  $linewidth{$apername} = $w;
	  if ($linewidth{$apername}==0) {
	    print "\nWARNING: This aperture has diameter 0!\n";
	    $aperture_warnings++;
	  }
	  $bbadd{$apername} = $h;
	}
	$apertype{$apername} = "oblong";
#	print "linewidth ($apername) = $linewidth{$apername}$unitname\n";
#	print "bbadd ($apername) = $bbadd{$apername}$unitname\n";
      }
    elsif($text =~ m|^%AD(D\d+)(\w+)\*|){         # it's a macro
      $apername = $1;
      $macroname = $2;
      print "\n $apername is the macro $macroname";
      print OUT "% $apername is the macro $macroname\n";
      print OUT "/$apername { $macroname } def\n";
      $linewidth{$apername} = 0;	
      $bbadd{$apername} = $bbadd{$macroname};
      $apertype{$apername} = "macro";
    }
    else{
      print "\nWARNING (aperdef): I cannot handle: $text !\n";
      $warnings++;
    }
  }


# Moving, Drawing, Flashing, and Circling

sub move
{
    local($text, $stroke) = @_;

#    print "movestring is:$text\n";    
    if($text=~ m|^X(-?\d*(\.\d*)?).*\*|){
	$xcoord = &xcoord2real($1);
    }
    if($text=~ m|^.*Y(-?\d*(\.\d*)?)\*|){
	$ycoord = &ycoord2real($1);
    }
    # in polygons, we cannot move before every line 
    # so we have to do it here
    if($stroke==0){
      print OUT "$xcoord $ycoord moveto\n";
    }
    &updateBBwAper($xcoord, $ycoord);
}

sub draw
{
  if($verbose){
    print "d";
  }
  local($text, $stroke) = @_;

#    print "drawstring is:$text\n";    
#    print "$stroke = Actual Aperture is $actualaper of Type |$apertype{$actualaper}|\n";    
    
  if($apertype{$actualaper} eq "circle" || $apertype{$actualaper} eq "square" || stroke==0){
    if($stroke==1){
      print OUT "$xcoord $ycoord moveto\n";
    }
    if($apertype{$actualaper} eq "square" && $stroke==1){
      print OUT "$xcoord $ycoord $actualaper\n";
      print OUT "$xcoord $ycoord moveto\n";
    }
    if($text=~ m|^X(-?\d*(\.\d*)?).*\*|){
	$xcoord = &xcoord2real($1);
    }
    if($text=~ m|^.*Y(-?\d*(\.\d*)?)\*|){
	$ycoord = &ycoord2real($1);
    }
    print OUT "$xcoord $ycoord lineto\n";
    if($stroke==1){
      print OUT "stroke\n";
    }
    if($apertype{$actualaper} eq "square" && $stroke==1){
      print OUT "$xcoord $ycoord $actualaper\n";
#      print "Crazy, I'm drawing with a square aperture!\n";
    }
  }
  else {
    print "\nERROR: draw: can't draw with aperture $actualaper of type $apertype{$actualaper}\n!";
    $warnings++;
    if($text=~ m|^X(-?\d*(\.\d*)?).*|){
      $xcoord = &xcoord2real($1);
    }
    if($text=~ m|^[X\d\.\-]*Y(-?\d*(\.\d*)?).*|){
      $ycoord =  &ycoord2real($1);
    }
  }	
  &updateBBwAper($xcoord, $ycoord);
}


sub carc
  {
  if($verbose){
    print "a";
  }
    local($text,$stroke) = @_;
    local ($oldx, $oldy, $midx, $midy, $radius, $startangle, $stopangle,);
    local ($icoord, $jcoord);
#   print "carcstring is:$text\n";    
    
    if($apertype{$actualaper} eq "circle" || stroke==0){
#	print OUT "% I'm in carc carcstring is:$text\n";
	$oldx=$xcoord;
	$oldy=$ycoord;
	if($text=~ m|^\*?X(-?\d*(\.\d*)?).*\*|){
	  $xcoord = &xcoord2real($1);
	}
	if($text=~ m|^.*Y(-?\d*(\.\d*)?).*\*|){
	  $ycoord = &ycoord2real($1);
	}
	if($text=~ m|^.*I(-?\d*(\.\d*)?).*\*|){
	  $icoord = &xcoord2real($1);
	}
	else {
	  $icoord = 0;  
	}
	if($text=~ m|^.*J(-?\d*(\.\d*)?).*\*|){
	  $jcoord = &xcoord2real($1);
	}
	else {
	  $jcoord = 0;  
	}
	if($icoord==0 && $jcoord==0){
	  print "\nWARNING: carc: $text did not match, I and J not found\n";
	  $warnings++;
	  return;
	}
	if($circ_mode360=0){
	  if($iccord < 0 || $jcoord < 0){
	    print "\nERROR: In 1 quadrant mode, I and J shouldn't be negative\n";
	    $warnings++;
	    return;
	  }
	  else {
	    print OUT " \n We are in 1 quadrant (<90 deg) clockwise Mode\n";
	    print OUT "\n Untested Feature: Please Report to Andy.\n";
	  }
	  # We are drawing only Circles with 90 deg
	  if($xcoord > $oldx && $ycoord > $oldy){
	    $midx=$oldx+$icoord;
	    $midy=$oldy-$jcoord;
	    $radius=$icoord;
	  }
	  elsif($xcoord > $oldx && $ycoord < $oldy){
	    $midx=$oldx-$icoord;
	    $midy=$oldy-$jcoord;
	    $radius=$jcoord;
	  }
	  elsif($xcoord < $oldx && $ycoord < $oldy){
	    $midx=$oldx-$icoord;
	    $midy=$oldy+$jcoord;
	    $radius=$icoord;
	  }
	  elsif($xcoord < $oldx && $ycoord > $oldy){
	    $midx=$oldx+$icoord;
	    $midy=$oldy+$jcoord;
	    $radius=$jcoord;
	  }
	}
	else {
#	    print "\n We are in 360 deg clockwise Mode\n";
	    $midx=$oldx+$icoord;
	    $midy=$oldy+$jcoord;
	    $radius=sqrt($icoord*$icoord+$jcoord*$jcoord);
	}
	$startangle = 57.2957795131 * atan2($oldy-$midy,$oldx-$midx);
	$stopangle  = 57.2957795131 * atan2($ycoord-$midy,$xcoord-$midx);
	if(abs($startangle-$stopangle)<0.01){
	  $startangle=360; # full circle
	  $stopangle=0;
	}
	if($stroke==1){
#	  print OUT "$oldx $oldy moveto\n";
	}
	if($apertype{$actualaper} eq "square" && stroke==1){
	    print OUT "$oldx $oldy $actualaper\n";
#	    print OUT "$oldx $oldy moveto\n";
	}
	print OUT "$midx $midy $radius $startangle $stopangle arcn\n";
	if($stroke==1){
	  print OUT "stroke\n";
	}
     	if($apertype{$actualaper} eq "square" && stroke==1){
	    print OUT "$xcoord $ycoord $actualaper\n";
	  }
	&updateBBwAper($midx+$radius, $midy+$radius);
	&updateBBwAper($midx-$radius, $midy-$radius);
      }
    else {
      print "WARNING: carc: can't draw with aperture $actualaper of type $apertype{$actualaper}\n!";
      $warnings++;
      if($text=~ m|^(X(-?\d*(\.\d*)?))?(Y(-?\d*(\.\d*)?))?.*|){
      $xcoord = &xcoord2real($2);
      $ycoord = &ycoord2real($5);
    }
      &updateBBwAper($xcoord, $ycoord);
    }	
  }

sub ccarc
  {
  if($verbose){
    print "A";
  }
    local($text, $stroke) = @_;
    local ($oldx, $oldy, $midx, $midy, $radius, $startangle, $stopangle,);
    local ($icoord, $jcoord);
    print OUT "% ccarcstring is $apertype{$actualaper} $actualaper, $stroke:$text\n ";    
    
    if($apertype{$actualaper} eq "circle" || stroke ==0){
#	print "% \nI'm in ccarc carcstring is:$text\n";
	$oldx=$xcoord;
	$oldy=$ycoord;
	if($text=~ m|^\*?X(-?\d*(\.\d*)?).*\*|){
	  $xcoord = &xcoord2real($1);
	}
	if($text=~ m|^.*Y(-?\d*(\.\d*)?).*\*|){
	  $ycoord = &ycoord2real($1);
	}
	if($text=~ m|^.*I(-?\d*(\.\d*)?).*\*|){
	  $icoord = &xcoord2real($1);
	}
	else {
	  $icoord = 0;  
	}
	if($text=~ m|^.*J(-?\d*(\.\d*)?).*\*|){
	  $jcoord = &xcoord2real($1);
	}
	else {
	  $jcoord = 0;  
	}
	if($icoord==0 && $jcoord==0){
	  print "\nWARNING: carc: $text did not match, I and J not found\n";
	  $warnings++;
	  return;
	}
	if($circ_mode360=0){
	  if($iccord < 0 || $jcoord < 0){
	    print "\nERROR: In 1 quadrant mode, I and J shouldn't be negative\n";
	    $warnings++;
	    return;
	  }
	  else {
	    print "\nWe are in 1 quadrant (<90 deg) counterclockwise Mode\n";
	    print "\nUntested Feature: Please Report to Andy.\n";
	  }
	  # We are drawing only Circles with 90 deg
	  if($xcoord > $oldx && $ycoord > $oldy){
	    $midx=$oldx-$icoord;
	    $midy=$oldy+$jcoord;
	    $radius=$jcoord;
	  }
	  elsif($xcoord > $oldx && $ycoord < $oldy){
	    $midx=$oldx+$icoord;
	    $midy=$oldy+$jcoord;
	    $radius=$icoord;
	  }
	  elsif($xcoord < $oldx && $ycoord < $oldy){
	    $midx=$oldx+$icoord;
	    $midy=$oldy-$jcoord;
	    $radius=$jcoord;
	  }
	  elsif($xcoord < $oldx && $ycoord > $oldy){
	    $midx=$oldx-$icoord;
	    $midy=$oldy-$jcoord;
	    $radius=$icoord;
	  }
	}
	else {
#	    print "We are in 360 deg clockwise Mode\n";
	    $midx=$oldx+$icoord;
	    $midy=$oldy+$jcoord;
	    $radius=sqrt($icoord**2+$jcoord**2);
	}
	$startangle = 57.2957795131 * atan2($oldy-$midy,$oldx-$midx);
	$stopangle  = 57.2957795131 * atan2($ycoord-$midy,$xcoord-$midx);
	if($stroke==1){
#	  print OUT "$oldx $oldy moveto\n";
	}
	if($apertype{$actualaper} eq "square" && stroke==1){
	    print OUT "$oldx $oldy $actualaper\n";
#	    print OUT "$oldx $oldy moveto\n";
	}
	if(abs($startangle-$stopangle)<0.01){
	  $startangle=0;
	  $stopangle=360;
	}
    print OUT "$midx $midy $radius $startangle $stopangle arc\n";
    if($stroke==1){
      print OUT "stroke\n";
    }
    if($apertype{$actualaper} eq "square" && stroke==1){
	  print OUT "$xcoord $ycoord $actualaper\n";
	  }
    &updateBBwAper($midx+$radius, $midy+$radius);
    &updateBBwAper($midx-$radius, $midy-$radius);
  }
else {
  print "\nWARNING: ccarc: can't draw with aperture $actualaper of type $apertype{$actualaper}\n!";
      $warnings++;
      if($text=~ m|^(X(-?\d*(\.\d*)?))?(Y(-?\d*(\.\d*)?))?.*|){
      $xcoord = &xcoord2real($2);
      $ycoord = &ycoord2real($5);
    }
      &updateBBwAper($xcoord, $ycoord);
    }	
}

sub polygon
{
  if($verbose){
    print "p";
  }
    local($text) = @_;
    local(@tmp);
    $text=~ s/\s//g;
#    print "polygonstring is:$text\n";
    print OUT "% Here comes a polygon!\n";   
    @tmp=split(/\*/,$text);
    foreach(@tmp){
      $_=join('',$_,"*");
#      print OUT "% polytmp = $_\n";
      if($_=~ m|^G75.*|){
	$circ_mode360=1;
      }
      elsif($_=~ m|^G74.*|){
	$circ_mode360=0;
      }
      elsif($_=~ m|^G36.*|){
	# do nothing, but do not complain
      }
      elsif($_=~ m|^G37.*|){
	# do nothing, but do not complain
      }
      elsif($_=~ m|^(G01)?X?[\.\-\d]*Y?[\.\-\d]*D02\*| ||
	    ($_=~ m|^(G01)?X?[\.\-\d]*Y?[\.\-\d]*\*| && $draw_now == 0)){  
	# it's a move without draw
	$_ =~ s/G01//g;
	$_ =~ s/D02//g;
#	print OUT "% string an move = $_\n";
	&move($_,0);
	$draw_now=0;
      }
      elsif($_=~ m|^(G01)?X?[\.\-\d]*Y?[\.\-\d]*D01\*| ||
	    ($_=~ m|^(G01)?X?[\.\-\d]*Y?[\.\-\d]*\*| && $draw_now == 1)){
	# it's a  draw
	$_ =~ s/G01//g;
	$_ =~ s/D01//g;
#	print OUT "% string an draw = $_\n";
	&draw($_,0);
	$draw_now=1;
      }
      elsif($_=~ m|^G02.*D01\*| || 
	    ($_=~ m|^G02.*| && $draw_now == 1)){  # it's a clockwise arc
	$_ =~ s/G02//g;
	$_ =~ s/D01//g;
#	print OUT "% string an carc = $_\n";
	&carc($_,0);
	$draw_now=1;
      }
      elsif($_=~ m|^G03.*D01\*| || 
	    ($_=~ m|^G03.*| && $draw_now == 1)){  # it's a counterclock arc
	$_ =~ s/G03//g;
	$_ =~ s/D01//g;
#	print OUT "% string an ccarc = $_\n";
	&ccarc($_,0);
	$draw_now=1;
      }
     else {
	print "\nWARNING: polygon: Could not handle: $_ \n";
	$warnings++;
      }
    }
    print OUT "closepath\nfill\n";
}
	    
	    




sub flash
{
  if($verbose){
    print "f";
  }
    local($text) = @_;
#    print "flashstring is:$text\n";    

    if($text=~ m|^X(-?\d*(\.\d*)?).*|){
	$xcoord = &xcoord2real($1);
    }
    if($text=~ m|^[X\d\.\-]*Y(-?\d*(\.\d*)?).*|){
	$ycoord =  &ycoord2real($1);
    }
    print OUT "$xcoord $ycoord $actualaper\n";

    &updateBBwAper($xcoord, $ycoord);
}

# some handy subs for updating the BB values



sub updateBBwAper
{
    local($x, $y) = @_;
    if($x - $bbadd{$actualaper} < $xmin){
	$xmin = $x - $bbadd{$actualaper};
    }
    elsif($x + $bbadd{$actualaper} > $xmax){
	$xmax = $x + $bbadd{$actualaper};
    }
    if($y - $bbadd{$actualaper} < $ymin){
	$ymin = $y - $bbadd{$actualaper};
    }
    elsif($y + $bbadd{$actualaper} > $ymax){
	$ymax = $y + $bbadd{$actualaper};
    }
    if($bbdefined == 0){
      if($xmin == 0){
	$xmin=$xmax;
      }
      else {
	$xmax=$xmin;
      }
      if($ymin == 0){
	$ymin=$ymax;
      }
      else {
	$ymax=$ymin;
      }
#      print "initial xmin = $xmin xmax = $xmax ymin = $ymin ymax = $ymax\n";

      $bbdefined = 1;
    }


#    print "xmin = $xmin xmax = $xmax ymin = $ymin ymax = $ymax\n";
}

sub updateBB
{
    local($x, $y) = @_;
    if($x < $xmin){
	$xmin = $x;
    }
    elsif($x > $xmax){
	$xmax = $x;
    }
    if($y < $ymin){
	$ymin = $y;
    }
    elsif($y > $ymax){
	$ymax = $y;
    }
    if($bbdefined == 0){
      if($xmin == 0){
	$xmin=$xmax;
      }
      else {
	$xmax=$xmin;
      }
      if($ymin == 0){
	$ymin=$ymax;
      }
      else {
	$ymax=$ymin;
      }
      $bbdefined = 1;
#      print "initial xmin = $xmin xmax = $xmax ymin = $ymin ymax = $ymax\n";
    }
}
