#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: GraphFeatures_plugin.tcl 215449 2011-06-05 05:50:41Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2011/05/31  PPRS-23355: Added debugging
# pbaksai   2010/11/17  Renamed from PE_SkyCat_plugin. tcl to
#                       GraphFeatures_plugin.tcl
# hummel    1999/08/13  Created
# A.Brighton 20 Jan 98  Created
#
#*******************************************************************************
#   NAME
#       GraphFeatures_plugin.tcl - Graphics features plug-in script
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#       This is a plugin file for skycat. It is used to add graphics
#       features to skycat at startup. To use this file for skycat, set the
#       environment variable SKYCAT_PLUGIN to the path name of this file.
# 
#       This file must define at least one tcl proc: SkyCat_plugin, which
#       takes one argument: the name of the top level skycat widget.  
#       See the documentation, man pages and Itcl widget source code for details
#
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#-------------------------------------------------------------------------------
#
puts stdout "Loading the graphics features plugin...."

# This proc is required. It will be called once for each skycat
# instance. The parameter is the name ("$this") of the SkyCat
# class object.

proc SkyCat_plugin {this} {
    global env
    global graphics_plugin_initialized_

    set debug_ 0
    if { [info exists env(FIMS_DEBUG)] } {
        if { [string compare -nocase -length 3 $env(FIMS_DEBUG) "off"] != 0 } {
            set debug_ 1
        }
    }
    if { $debug_ } { puts "SkyCat_plugin $this (GF)" }

    # get the toplevel widget name
    set w [utilNamespaceTail $this]

    # call a proc to add items to the Graphics menu.
    # Note that the directory containing this file is automatically
    # appended to the Tcl auto_path, so we can just call a proc
    # as long as there is a tclIndex file in this directory.
    add_graphics_features $w

    # Should this code run only once
    if { ![info exists graphics_plugin_initialized_] } {
        set graphics_plugin_initialized_ 1
    } else {
        if { $debug_ } { puts "graphics_plugin_initialized_: already >$graphics_plugin_initialized_<" }
    }

}

#
#___oOo___
