#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fims.tcl 257795 2014-07-28 14:28:30Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2014/07/25  CCB-000579: Support for 64 bits and portability.
#                       - Adjusted to new product package structure.
# pbaksai   2012/11/28  PPRS-48245: Quit Mode bug
#                       - Fixed bug in destructor that tried to use SP object
#                         before it was created and outside a catch.
#                       - Added creation/reusage of SP object in constructor.
#                       - Improved status panel behavior.
#                       - Removed useless big_status_panel method.
# pbaksai   2012/11/28  PPRS-48244: Empty checksum
#                       - Temporarily removed CHECKSUM keywords from p_focf, 
#                         p_targ files as well.
# pbaksai   2012/11/03  PPRS-24729: Bug fixing
#                       - Temporarily removed CHECKSUM keyword from gerber files
#                       - Fixed minor syntax errors.
# pbaksai   2012/11/01  PPRS-23355: Updated for new product package structure
#                       - Use global variable skycat_version
# pbaksai   2012/09/22  PPRS-23355: Updated for new product package structure
#                       - Files in FIMSROOT/lib/fors moved to FIMSROOT/config
# pbaksai   2012/01/31  PPRS-24729: Fixed format of reference pixel position.
# pbaksai   2011/11/22  PPRS-24729: Fixed problem opening status panel a second
#                                   time.
# pbaksai   2011/11/20  PPRS-24729: Improved usability of pick object for center
#                                   slit option.
# pbaksai   2011/11/19  PPRS-24729: Fixed mistake with EQUINOX, EPOCH usage
# pbaksai   2011/11/19  PPRS-24729: Fixed WCS handling in frameless mode
# pbaksai   2011/11/16  PPRS-24729: Fixed flipped mask when abs(PA) > 90.0 
# pbaksai   2011/06/05  PPRS-24729: Adjusted focal field limits per mode
# pbaksai   2011/05/31  PPRS-24729: Fixed reduced focal field calculation
# pbaksai   2010/12/03  PPRS-23355: More uses of interactive option
#                                   Register autoslit method in the header of
#                                   setup files.
#                                   Register FITS directory path and filename
#                                   in separate lines of the setup file headers.
#                                   Generate a pure Gerber file ready to be
#                                   loaded in CircuitCam for inspection.
# pbaksai   2010/11/16  PPRS-23355: Added interactive option
#                                   save_TPX method returns filename
# pbaksai   2010/11/08  PPRS-24729: Fixed load_*_TPX for autopos case with no
#                                   mask reset.
# pbaksai   2010/04/10  PPRS-35032: Fixed format of .p_targ file for tcl llength
#                                   Checked all coordinate conversions.
#                                   Fixed bug with slit width in IMG mode
# pbaksai   2010/04/08  PPRS-34948: Fixed reported bug in ref. slit coords.
# pbaksai   2010/03/23  PPRS-24729: Fixed reported bug and feature
#                                   - Fixed bug in code for MOS reference target
#                                   - Do not reset mask if loading from AutoPos
# pbaksai   2010/01/24  PPRS-24729: Fixed bugs in code for curved slits
# pbaksai   2009/11/21  PPRS-33492: Added dimensional units to wave limits
#                                   output.
# pbaksai   2009/11/20  PPRS-24729: Fixed bug in dispersion frame calculation
# pbaksai   2009/09/13  PPRS-27204: HIT-MS implementation
# pbaksai   2009/08/17  PPRS-27204: HIT-MS implementation
# pbaksai   2009/08/13  PPRS-27204: HIT-MS implementation
# pbaksai   2009/07/28  PPRS-27204: HIT-MS implementation
# pbaksai   2009/07/19  PPRS-27204: HIT-MS implementation
# pbaksai   2009/06/30  PPRS-27204: HIT-MS implementation
# pbaksai   2009/06/16  PPRS-27204: HIT-MS implementation
# pbaksai   2009/06/12  PPRS-27204: HIT-MS implementation
# pbaksai   2009/06/02  PPRS-27204: HIT-MS implementation
# pbaksai   2008/12/09  PPRS-29765: Corrections
# pbaksai   2008/11/25  PPRS-29438: Fixed typo
# pbaksai   2008/07/20  PPRS-27337: Added camera option configuration for FORS3
# pbaksai   2008/06/23  PPRS-27204: Menu reengineering, HIT-MS support
# pbaksai   2008/06/19  PPRS-27204: Menu item index names
# pbaksai   2007/12/02  PPRS-24729: Fixed more bugs found
#                       - Initialization of mw_list variable
#                       - Destroy slit parameter editor panels left around this
#                         mask when resetting the mask
# pbaksai   2007/11/22  PPRS-24729: Fixed reported bugs and others found by me
#                       - FIMS does not load the FORS1/2 image that has been
#                         mosaiced manually
#                       - Impossible to re-load previously created and saved
#                         FIMS setup file
#                       - long INS.MASK.NAID
#                       - FIMS creates mask file names with forbidden characters
#                       -  Project name could start with numbers, saving the
#                          mask with file names such as mxu2.42.0123pbak.fims
#                          Then, next exposure counter would be wrongly 124 and
#                          not 43 
#                       - Slit offsets were not saved in mask setup file
#                         mxu2.<exp count>.<fn>.fims
#                         In the future must be saved in .p_focf when FORS2
#                         understands this keyword.
#                       - Slit parameters editor panel presented incorrect title
#                         and fields if another slit was clicked without closing
#                         the panel
#                       - After using the MOS mode, and returning to MXU, if you
#                         click inside the mask rectangle, a syntax error occurs
#                         while trying to move the mxu slit which is not allowed
#                       - Host name definition required to run FIMS
#                       - Checksum not computed
# pbaksai   2007/04/03  PPRS-23198: Fixed typos
# pbaksai   2006/12/15  Improved tracer, modified bonsai logic, fixed typos
# mkiekebu  2006-12-05  FORS1 CCD upgrade:
#                       - updated proc ccd_init
#                       - updated proc wave_limit
#                       - updated load_ccd_window
#                       - updated load_fieldstop
#                       - updated set_scale
# pbaksai   2006/03/21  Added support for tracer, slit offset, slit delete
# mkiekebu  2004/06/17  Accept float for grism wavelenght according to the new
#                       ISF config
# mkiekebu  2004/06/14  Fixed problem with dialog windows (RH9.0)
# pbaksai   2004/06/12  PPRS-13135:
#                       - FORS1 moved from UT1 to UT2
#                       - FORS2 moved from UT4 to UT1
#                       - changed focscale_UTx values
# tsz/pbg   2003/11/13  PPRS-10127: Remove slit if out of range
# aaguayo     28/10/03  PPRS-10127: Max number of mxu slits updated to 1000
# aaguayo     28/10/03  PPRS-10127: distortion factors for FORS1 read from the
#                                   fits headers
# aaguayo     28/10/03  PPRS-10127: Included 'SIMPLE' option
# aaguayo     28/10/03  PPRS-10127: refresh of the ref-slit plots after toggling
#                                   labels on/off
# aaguayo     28/10/03  PPRS-10127: Edit button removed while saving
# aaguayo     05/12/02  PPRS-7840: De-Select mxu slit when clicking 2-butt on
#                                  label
# aaguayo     05/12/02  PPRS-7840: commented plot section about 1000 Pixel area
#                                  for mxu
# aaguayo     05/12/02  PPRS-7840: commented workaround 13. Sept 2000 WH
#                                  (FORS1 dist)
# aaguayo     05/12/02  PPRS-7840: changed Y pos offset of GRIS_1400V to 111.4
#                                  pixel
# aaguayo     05/12/02  PPRS-7840: changed Y pos offset of GRIS_600RI to 272.4
#                                  pixel
# mkiekebu  2002/10/05  Restored changes on load_mxu_TPX removed previous
#                       version
# aaguayo     07/06/01  Removed changes in save_TPX, load_TPX and load_mxu_TPX
#                       that came up from v2.31 to go back to v 2.29
# mkiekebu    24/03/02  look for CHIP.ID instead of CHIP.NAME in ablage
# mkiekebu    13/12/01  removed some print out
# mkiekebu    07/11/01  fixed rotation of wlength limits lines
# mkiekebu    03/11/01  Added fixed reference slit for MXU masks
# mkiekebu    30/10/01  Fix problem with merged image (bonsai effect)
# mkiekebu    20/07/01  FORS 2 CCD Upgrade
# aaguayo     07/05/01  modified load_mxu_TPX to avoid positioning of Slits
#                       outside the focal field
# aaguayo     05/05/01  modified save_TPX and load_TPX methods to avoid refstars
#                       in the forbidden zone
# mkiekebu    31/03/01  removed bug (last modification)
# aaguayo     14/03/01  fims 2.28 - (use of SKYCAT_VER variable)
# mkiekebu    29/12/00  Added dialog info for GRIS_600RI
# aaguayo     14/12/00  expiration date checking... feature disabled again
# aaguayo     14/12/00  INS.FIMS.NAME changed to
#                       filename_skycatversion_fimsversion
# aaguayo     12/12/00  Added info_dialog about not allowed CURVED & CIRCLE
#                       reference slits
# aaguayo     12/12/00  shape="none" not supported for MXU mode ->
#                       widget disabled
# aaguayo     12/12/00  expiration date checking re-activated
# aaguayo     12/12/00  expiration date checking put at the begining of working
# W.Hummel   27.Oct 97  copied from version 0.9
# W.Hummel   15.Nov 97  permanently progressing
# W.Hummel   12.May 98  use with skycat V2.0.13
# W.Hummel   25.May 98  V0.9.5
# W.Hummel   25.May 98  V0.9.6, SCT-2.0.15
# W.Hummel   21.Jul 98  allow to move the mask out of the frame
# W.Hummel   22.Jul 98  allow to load a mask, moved oput of the frame
# W.Hummel   30.Jul 98  bug in LSS, new modes IMS,PIMS,HIT
# W.Hummel   sep/oct98  FORS1 Commissioning at paranal
# W.Hummel   17.Nov 98  Preparing for FORS1 COMM2
# wgaessle    14/01/99  puts -nonewline for p_*
#                       asking for slit 10
#		        - search for wgaessle for the bigger changes -
# wgaessle    14/01/99  added INS.FOCF.* to *foc files
# W.Hummel    15/01/99  - FORS optical correction applied,
#                         if fits image from FORS
#                       - IPOL dither now more accurate 22.73'' and 11.36''
# W.Hummel    15/01/99  - slit 10 is drawn if applied
# W.Hummel    15/01/99  - first modifications for FORS2
# W.Hummel    18/01/99  changed default TSF name
# W.Hummel    19/01/99  collimator no longer hardcoded but read from isf
# W.Hummel    20/01/99  grism,filter,lmin,lmax configured via load-mask
# W.Hummel    22/01/99  FORS1,FOSR2 dont interfere anymore
# W.Hummel    07/04/99  preparation for keyword change WID --> WIDTH
# W.Hummel    14/04/99  start with echelle spectroscopy = ECHS
# W.Hummel    27/07/99  version 1.6 goes with skycat 2.4.3 !!!!!!!!
# W.Hummel    30/08/99  MOD for position angle corrected
#                       save lss reference target and slit target exchanged
# W.Hummel    01/09/99  curved MXU slit
#
#*******************************************************************************
#   NAME
#   FORS - Tcl class
#
#   NAMESPACE
#       fors
#
#   SYNOPSIS
#       FORS my_mask $skycat_ $canvas_ $image_ $menu_ [options]
#
#   PARENT CLASS
#       none
#
#   DESCRIPTION
#       [incr]tcl/tk class containing full graphical handling
#       of all VLT-FORS1/2 instrumental modes.
#       This class is plugged into the ESO SkyCat Tool.
#       Following modes are supported:
#       FORS1           FORS2
#       ----------------------
#       MOS             MOS
#       IMG             IMG
#       LSS             LSS
#       HITS            HITS
#       IPOL            HIT-MS
#       PMOS            MXU
#       ----------------------
#       PMOS     : Polarimetric MOS (only even MOS slit blades in use)
#       IMG      : Imaging (obscure bright targets with MOS slit blades)
#       IPOL (SR): Polarimetric IMG (only even slit blades can be used)
#       IPOL (HR): Imaging polarimetry, a particular mask
#       LSS      : Long slit spectroscopy, a particular mask
#       HITS     : High time resolution spectroscopy, a LSS mask with a decker
#       HIT-MS   : High time resolution multi-shift spectroscopy
#       MXU      : Mask exchange unit (FORS2 only) create slits
#       ECHS     : Echelle spectroscopy, MOS 10 and MOS 9 at a fixed position
#       There are also 2 fims internal submodes
#       IMS      : Imaging spectroscopy as a submode of MOS
#                  = slitless spectroscopy
#       PIMS     : Polarimetric imaging spectroscopy as a submode of PMOS
#                  = slitless spectropolarimetry
#
#       Some Methods of this class call the AstroCat class of SkyCat
#       There is also one external binary (AP) for automatic mask
#       adjustment.
#       Only one instance of the class FORS is allowed.
#       Start with FORSInit. Methods for basic motions
#       like shift rotate and move slit blades available
#
#   OPTIONS
#       -instrument
#           name of the instrument: FORS1, FORS2, FORS2+1
#       -modus
#           instrumental modi can be one of
#           mos pmos img ipol lss hits hitms
#       -long_width
#           instrument part: slit width for LSS and HITS, can be
#           0.3 0.4 0.5 0.7 1.0 1.6 2.0 ( 2.5 LSS only)
#       -slit_width
#           instrument part: MOS slit width in arcsec, default 1
#       -slit_length
#           layout option: MOS slit length
#       -slit_offset
#           layout option: MXU straight slit center offset
#       -slit_shape
#           layout option: MXU slit shape
#       -collimat
#           instrument part: COLL_SR+1 or COLL_HR+2
#       -filter
#           instrument part: default none
#       -grism
#           instrument part: default none
#       -camera
#           instrument part: default U
#       -rotangle
#           rotation step in degrees/click, default 5
#       -colorset
#           layout option: color of the mask, default Yellow/Blue
#       -graytype
#           layout option for transparency, default 00
#       -badpixelmap
#           flag to show the bad pixel map
#       -label
#           flag to show the labels of the mask
#       -pics
#           flag to center slit on target, using PickObject
#       -debug
#           flag to print detailed messages
#       -verbosity
#           flag to print detailed messages
#       -interactive
#           flag to disable interactions with user when in remote control
#       -width
#           the pixel width of the underlying frame,
#           default is [$image_ width]
#       -height
#           the pixel height of the underlying frame,
#           default is [$image_ height]
#       -fors_frame
#           flag to use image scale from the fits header keyword CDELT1
#           option to apply optical corr to MOS slit positions
#           option to apply optical corr to reference target
#       -file_name
#           project filename
#       -calib_passwd
#       -int_dir
#       -isf_dir
#       -log_dir
#       -set_dir
#       -prep_dir
#       -isf_file
#
#   PUBLIC METHODS
#       load_ccd_window {}
#            load CCD (master and slave) dimensions from ccdsize array depending 
#            on selected camera CCD
#       load_TPX { filename }
#           calls either load_mos_TPX or load_img_TPX
#       save_TPX {}
#           calls either save_mos_TPX, save_img_TPX or save_lss_TPX
#       update_scale {}
#           called when the zoom button is pushed
#       wave_config_mask { four }
#           called by wave_length_CB to assign L_MIN and L_MAX. It is also
#           called by filter and grism public variables initialization
#       init_mask_world2 {}
#          most important, most complex and most sensistive Method
#          determines the initial mask position and the complete
#          layout of the mask.
#          extrapolates WCS to areas out of the underlying fits frame, that
#          the mask can also be plotted outside the fits frame
#       reset_mask {}
#          called by the rest mask button
#          call init_mask_world2
#       update_mask {}
#          rescales mask when the 'Z;oom or 'z'oom button is pressed
#       update_and_replot {}
#          calls update_scale and plot
#       update_color {}
#          updates color table according to Color and Transparency options
#       update_xform_coeff { {of_whom ""} }
#
#       plot_mask {}
#          plot the mask
#       edit_ref_targ { c }
#
#       help {}
#           maintenance
#       open_blades { {resol SR} }
#
#       close_blades { {resol SR} }
#
#       adjust_slit_length {}
#          handles virtual MOS slit lengths
#       noars {}
#
#       noas {}
#
#       AutoPos { kind }
#           automatic positioning
#       set_wcs { in }
#           frameless mode
#       new_wcs {}
#           frameless mode
#       hitms_delete_slit { w sx sy }
#
#       mxu_delete_slit { w sx sy }
#
#       mxu_delete_all_slits { {keep_rs 0} }
#
#       mxu_ref_slit { w sx sy on }
#
#       hitms_config_slit { w sx sy }
#
#       mos_config_slit { w sx sy }
#
#       mxu_config_slit { w sx sy }
#
#       config_hitms { my wiofpo }
#
#       config_mos { my width }
#
#       config_mxu { my wileofpo }
#
#       wave_length {}
#           EntryWidget for user defined wavelength range
#       wave_length_CB  { lmin lmax } {
#           called by the wave_length EntryWidget, adds more parameters
#           before calling wave_config_mask
#       select_mxu_name  {}
#       
#       select_set_dir  {}
#
#       select_filename  {}
#           get user defined project name
#       get_calib_passwd  {}
#
#       test_optic_distor_FORS {}
#           maintenance
#       show_status {}
#
#       demo {}
#          called by demo button
#
#   PUBLIC DATA MEMBERS
#       instrument   FORS2
#       modus        mos
#
#       slit_width   1.0
#       slit_length  real
#       slit_offset  0.0
#       slit_shape   STRAIGHT
#       long_width   0.5
#       collimat     none
#       filter       none
#       grism        none
#       camera       U
#
#       rotangle     5
#       colorset     Yellow/Blue
#       graytype     0
#       badpixelmap  0
#       label        1
#       pics         0
#       debug        0
#       verbosity    0
#
#       width        200
#       height       200
#       fors_frame   1
#
#       int_dir  $env(FIMSROOT)
#       isf_dir  $env(ISF_DIR)
#       log_dir  $env(LOG_DIR)
#       set_dir  $env(SET_DIR)
#       prep_dir $env(PREP_DIR)
#       isf_file INS.isf
#
#       calib_passwd ""
#       file_name    ""
#
#   PROTECTED METHODS
#
#   PROTECTED DATA MEMBERS
#
#   PRIVATE METHODS
#       init {}
#       init_ccd_params { {ccd "FORS-MIT"} }
#       load_instrument {}
#           read the FORS.isf and configures the class.
#           e.g. reads the currently available grisms
#       load_BPM {}
#           load bad pixel map table
#       load_hitms_TPX { filename }
#           load target acquisition file for HIT-MS,
#           plot and reset the mask
#           filename is <mode>.<ec>.<projectname>.fims
#       load_img_TPX { filename }
#           load target acquisition file for IMG and IPOL,
#           plot and reset the mask
#           filename is <mode>.<ec>.<projectname>.fims
#       load_mos_TPX { filename }
#           load target acquisition file for MOS and PMOS,
#           plot and reset the mask
#           filename is <mode>.<ec>.<projectname>.fims
#       load_mxu_FAD { filename }
#           load standard field aperture definition file
#       load_mxu_TPX { filename }
#           load target acquisition file for MXU,
#           plot and reset the mask
#           filename is <mode>.<ec>.<projectname>.fims
#       save_hitms_TPX { filename }
#           save target acquisition file for HIT-MS mode
#       save_hits_TPX { filename }
#           save target acquisition file for HITS mode
#       save_img_TPX { filename }
#           save target acquisition file for IMG and IPOL mode
#       save_lss_TPX { filename }
#           save target acquisition file for LSS mode
#       save_mos_TPX { filename }
#           save target acquisition file for MOS and PMOS mode
#       save_mxu_TPX { filename }
#           save target acquisition file for MXU mode
#       prepare_mxu_for_save {}
#
#       save_mxu_MSF { filename }
#           save mask file for MXU mode
#       save_mxu_ADF { filename }
#           save aperture definition file for MXU mode
#       save_mxu_RS274X { filename }
#
#       save_mxu_RS274X_forPS { filename }
#
#       wave_limit {}
#           sets the wavelength limits of the spectral range
#       calculate_focalfield_frame { {resol SR} }
#           calculate focalfield and CCD frames
#       calculate_dispersion_frame { {resol SR} }
#           calculate reduced frame according to wavelength dispersion
#       calculate_slits { {resol SR} }
#           calculate blade positions according to modus
#       set_scale { flag }
#
#       align_mask { w sx sy }
#
#       update_mask_na {}
#
#       update_mask_id {}
#
#       update_mask_naid {}
#
#       update_labels {}
#
#       update_bez_oc { i }
#
#       update_BPM {}
#
#       get_cat {}
#           get the currently active catalogue
#       get_ec { mode }
#           scans the setup file directory and gets new exposure counter
#       get_fits_name {}
#           fits file name
#       extra_lss { a b }
#           handles reference target and slit target for LSS
#       pick_star {}
#
#       star_picked { b }
#           called by pick_star to assign pick variables
#       echs_pos {}
#
#       echs_targ {}
#
#       plot_tcs_label {}
#           plot the labels with telescope pointing on the canvas
#       plot_ins_label {}
#           plot the labels with filter and grism setup
#       plot_mos_dispersion_label { oriA oriB ori1 ori19 } {
#
#       plot_BPM {}
#
#       plot_ref_star_area_ccd2 {}
#
#       plot_ref_star_area { mos10flag }
#
#       fix_mask_slit  { mod cf }
#
#       reset_slits {}
#           called by rotate to deselect all slits
#           we also account for OptiCorr and DiffRefrac in HITS
#       move_slit { w sx sy {on 1} {click 1} }
#           move one single slit in MOS and PMOS
#           (MOS,PMOS)
#       move_slit_blade { w sx sy but }
#           move either MOS blade A or MOS blade B to cursor position
#           (IMG, IPOL-SR, MOS-slitless, PMOS-slitless)
#       erase {}
#            delete objects on canvas via tags
#       mark { w xs ys }
#           called when clicked on the green circle to store current mask
#           position
#       rotate { w x y dir}
#           rotate the whole mask
#       shift { w xs ys }
#           shift the whole mask, called when B1 move
#       warn {}
#
#       catalog { a_c }
#           compare slit position with local catalog
#       calc_ctom {}
#           canvas to focal field mm conversion
#       calc_zemax { my x y }
#           apply correction of optical distortion of FORS1 to MOS slits
#       calc_zemax_ref { a d }
#           apply correction of optical distortion of FORS1
#           to reference targets
#       calc_oc { x_mm x_pix my a2 d2 a3 d3 }
#           calculate empirical optical distortion curve
#           (empirical distortion model no longer used)
#       calc_diff_refrac { my a d }
#           apply optical correction due to differential refraction
#       calc_dither { c frac }
#           gets the offset for IPOL
#       dither_pol { w c frac }
#           handles the offset for IPOL
#       bind_align {}
#
#       bind_frame {}
#           bindings for green, white and gray circle
#       bind_echs {}
#
#       bind_hitms {}
#
#       unbind_hitms {}
#
#       bind_img {}
#           bindings for IMG and IPOL slit blades
#       bind_ipol {}
#           bindings for IPOL mask
#       bind_mos {}
#           bindings for MOS and PMOS slit blades pairs
#       bind_mxu {}
#
#       unbind_mxu {}
#
#       flip_echs {}
#
#       shift_cms { w sx sy }
#
#       mark_cms { w xs ys }
#
#       end_cms { w sx sy }
#
#       size_CS { my }
#
#       mxu_new_slit { click x y px py wid len off pa shape name ff max {ref 0} {mmff 0} }
#
#       hitms_create_slit { w sx sy }
#
#       mxu_create_slit { w sx sy }
#
#       mxu_create_refslit {}
#
#       select_cat { sc }
#           if more than one catalog is in use
#       od_F { x y }
#           empirical optical correction, no longer used
#       p0 { y }
#           empirical optical correction, no longer used
#       p1 { y }
#           empirical optical correction, no longer used
#       p2 { y }
#           empirical optical correction, no longer used
#       p3 { y }
#           empirical optical correction, no longer used
#       p4 { y }
#           empirical optical correction, no longer used
#       ablage_SR { r }
#           theoretical optical distortion
#       ablage_HR { r }
#           theoretical optical distortion
#       test_ablage { }
#           maintenance
#       small_status_panel {}
#
#       open_log {}
#
#       ldelete_all { w_list value }
#
#       ldelete { list value }
#
#       string_exchange { input old new }
#
#       DDMMSS2deg { ddmmss }
#           conversion routine
#       F2G { value }
#
#       pick_technique {}
#           Explains pick object technique for slit center option.
#
#   PRIVATE DATA MEMBERS
#       lochost           // $env(HOST)
#       f_v s_v           // fims and skycat release number
#       lfId              // logfile id
#       m_image
#       m_canvas
#       m_skycat          //  local name for image, canvas, skycat and rtd
#       canvscale         // scale (zoom) of the world/canvas
#       EQX               // equinox
#       s_slit 4          // boundary errors in slits_lengths  [mm]
#       world_x, world_y  // the actual position of the mask in world units
#       wx,wy             // the initial position of the mask in world units
#       mos               // the cordinates of the MOS mask, a big matrix
#       A_disp            // the grism-filter-dispersion array
#       good              // index of current filter-grism set
#       targw             // the world coordinates of the 19 positioned slits
#       abs_ang_rad 0.0   // absolute angle of the mask in rad with respect
#                         // to the WCS
#       x_ccd 300         // size of the ccd
#       ccx, ccy          // the  center of the mask (TCS position on canvas)
#       fr_x,fr_y         // initial canvas positon of mask = canvas center
#       worldscale 1.7    // scale pixel/''
#       oldpos            // old position for shifting
#       speclen 50.0      // the length of the spectrum on the CCD in arcsec
#       specmin 50.0
#       specmax 50.0
#       L_MIN,L_MAX       // user defined wavelength range
#                         // FORS configuration constants
#       XSRMAX,YSRMAX,XHRMAX,YHRMAX,XSRMIN,YSRMIN,XHRMIN,YHRMIN
#       mosslitmin,mosslitmax
#       htslitxmin,htslitymin,htslitxmax,htslitymax
#       longslitxmin,longslitymin,longslitxmax,longslitymax
#       polarimslitxmin,polarimslitymin,polarimslitxmax,polarimslitymax
#       long_number
#       mask
#       slit_coll
#       gris_range
#       filt_range
#       osfi_range
#       exposure_counter 1
#       trapa gray00      // transparanecy of slit blades string
#                         // VLT-U1 focal scale in arcsec/mm
#                         // there is rho=0.5280 theta -1.1288E-6 theta*theta
#                         // we use 1/0.5280 [arcsec/mm]
#                         // x [mm] *$focscale = y [arcsec]  (mtoas)
#       focscale 1.893939394
#                         // scale on the CCD  [mm/arcsec]
#                         // y [arcsec] * CCDscale = x [mm] (astom)
#                         // this can be modified:
#                         // take the Focus value from FORS.isf
#                         // nominal values
#       CCDscale_standard 0.2 // arcsec/pixel
#       CCDscale_highres  0.1 // arcsec/pixel
#       PIX              0.024 // pixel size in mm
#       atd 2.7777778E-4  // numeric arcmin_to_degree
#       trigo 1
#       ctom              //  numeric canvas_to_mm
#       color1 red        // fill colors
#       color2 yellow
#       color3 red        // outline colors
#       color4 yellow
#                         // Pick values
#       star_ix, star_iy, star_ra, star_de
#                         // LSS reference and slit object
#       lss_ref_ra, lss_ref_de, lss_slit_ra, lss_slit_de
#                         // fits scale
#       fits_x_scale, fits_y_scale
#       WP                // angle between WCS and pixel coords
#       man_WP            // interactive input of WP
#       CSO               // coord-system-orientation = 1/-1
#       CRO               // signum(CROTA2) needed for init_mask
#       PAM               // position angle of the mask
#       tcs_pam           // = -PAM for tcs_label
#       mask_initialized  // set by init_mask_world2
#       mountangle        // no comment
#
#   FILES
#       fims.tcl
#       fims_menu.tcl
#       mxu_class.tcl
#       SkyCat_plugin.tcl
#       SkySearch_plugin.tcl
#
#   ENVIRONMENT
#       ESO-RTD (Real-Time-Display),
#       ESO-DMD-SkyCat Tool
#       fims calls Methods of the original
#       ESO-SkyCat code, which are modified for
#       fims purposes:
#       RtdImageTrans: Methods rotate, update_trans and flip
#       AstroCat:      Method remove_FORS
#
#   COMMANDS
#       FORSInit { skycat_ canvas_ image_ menu_ instrument modus
#                  rot col tra BPM lbl cnt ctrl
#
#   RETURN VALUES
#
#   CAUTIONS
#       Use FIMS V1.2.9 only with skycat V>2.1.3
#
#   EXAMPLES
#
#   SEE ALSO
#
#
#         1) The Focal Field Orientation of FORS
#
#            This diagram shows the focal field of FORS1/2
#            together with the orientation of several coordinate
#            systems, like pixel (x,y), WCS (RA,DEC) and the fims
#            internal indexes of the large mask matrix
#            mos(slitnumber,cornerindex)
#
#            Focal field as viewed from bottom to top
#            Field as it appears in the final fits frame
#            and in FIMS
#
#
#
#         mos(0,a)          ^           mos(0,b)
#         (white circle)    |
#                           |
#                       del,DEC>0
#            Instrument Connection Point (ICP)
#                  IPOL-mask/MXU-mask
#                       North (ff)
#                        MOS 1
#                     y<0 (YSRMIN)
#                           |
#                           |     P.A.
#                           |    /
#                           |---/
#                           |  /
#                           | /
#          East  (ff)       |/        West (ff)
#    <------Mos-A-----------+---------Mos-B--------->
#           x<0 (XSRMIN)    |         x>0 (XSRMAX)
#          (blue spectrum)  |         (red spectrum)
#          alp,RA>0         |         alp,RA<0
#                           |
#                           |
#                        del,DEC<0
#                      (coll-drive)
#                        LSS-mask
#                       South (ff)
#                         MOS 19
#                     y>0 (YSRMAX)
#                           |
#                           |
#         mos(0,d)          V            mos(0,c)
#        (gray circle=
#        CCD-reference pixel)
#        pixel=1,1
#
#
#        2) orientation of the internal mask array
#
#         focfield A = mos(0,a) = (-x,+y)  (N,E) (white circle)
#                  B = mos(0,b) = (+x,+y)  (N,W)
#                  C = mos(0,c) = (+x,-y)  (S,W)
#                  D = mos(0,d) = (-x,-y)  (S,E) (gray circle
#                                                 = CCD reference pixel)
#
#
#        3) orientation of the wavelength borders
#
#        If 'label'-button == on, the user defined wavelength
#        borders are drawn on the canvas.
#        CCD frames  STANDARD RESOLUTION, high resolution
#        e.g. mos(0,gr) is the high res mask SW wavelength border
#
#                            (-y,N)
#                            (slit 1)
#                A      Ar          Br       B
#                +------+-----------+--------+
#                |e     |er         |fr      |f
#                |      |           |        |
#                |      |           |        |
#                |      |           |        |
#        (-x,E)  |      |           |        | (x,W, MOS B)
#        (MOS A) |      |           |        |
#                |      |           |        |
#                |      |           |        |
#                |h     |hr         |gr      |g
#                +------+-----------+--------+
#                D      Dr          Cr       C
#
#                          (y,S, slit 19)
#
#        e.g. mos(0,br) is the north red wavelength border line
#        for standard resolution collimator modus
#        wavelength labels at Cr,Dr and hr,gr
#
#        4) orientation of the individual slit blades
#
#        coords of a mos slit blade  mos($i,ax) ...
#
#            a              b        c               d
#            ---------------+        +----------------
#                           |  N,-y  |
#           (A, East,-x)    |        | (B, West, +x)
#                           |  S,+y  |
#            ---------------+        +----------------
#            e              f        g               h
#
#        e.g. mos(7,h) is the north east corner of slit 7
#
#        5)  coords of a long-slit and ipol stripe
#
#
#             a d
#             +-+
#             | |          a                   d
#             | |          +-------------------+
#             | |  LSS     |                   |  IPOL
#             | |          +-------------------+
#             | |          e                   h
#             +-+
#             e h
#
#          LSS/HITS orientation position angle=0
#          --------------------------------
#          9 long slits in north-south direction
#          and two pinholes :  P=big pinhole, p=small pinhole
#          The big pinhole becomes visible in HITS
#          together with the 0.3'' HIT slit.
#          The small pinhole becomes visible together with
#          the 0.4'' HIT slit. The 0.4'' HT slit is not supported
#          pinhole is not supported (neither by P2PP nor by FIMS)
#
#
#                              north          pixel 2000,2000
#                    +-------------------------+
#                    |  1.6   0.5   0.4   1.3  | <-slit width in arcsec
#                    |     \    |   |    /     |
#        internal    |      |   |   |   |      |           +-----> x
#        slit        |    | | | | | | | | |    |           |
#        number----> |    9 7 5 3 1 2 4 6 8    |           |
#             east   |    | | | |P|p| | | |    |  west    y|
#                    |    | | | | | | | | |    |           V
#                    |    | | | | | | | | |    |
#                    |    |   |   |   |   |    |
#                    |   /    /   |   \    \   |
#                    |2.5  1.0   0.3   0.7  2.0| <-slit width in arcsec
#                    +-------------------------+
#            pixel 1,1
#
#
#             long slit decker
#             ----------------
#                              north          pixel 2000,2000
#                    +-------------------------+
#                    |       +-+               |
#                    |       | | <-- LSS       |
#                    |       | |               |           +-----> x
#                    |       | |               |           |
#                    |       | |               |           |
#                    |       | |               |  west    y|
#                    |       | |               |           V
#                    |       | |               |
#                    |       | |  +-+          |
#                    |       | |  | | <-- HITS |
#                    |       +-+  +-+          |
#                    +-------------------------+
#            pixel 1,1
#
#                    The x-distance between HITS decker and LSS decker
#                    is half the distance between two LSS slits.
#                    If HITS decker enables 0.3'' slit, the LSS decker
#                    will enble the big pinhole
#
#
#        6) miscellaneous: orientation of the FORS focal field
#           with respect to other coord systems
#
#                               (Taltal)
#                                   s
#        geographic :  (Pacific)  w   e (Andes)
#                                   n
#                             (Antofagasta)
#
#
#                              ICP/IPOL
#        instrument :       MOS-A   MOS-B
#                              COLL/LSS
#
#                                #1
#        mask units :       MOS-A   MOS-B
#                                #19
#
#                                (-y)
#        cartes. focal field   (-x) (x)
#                                 (y)
#
#                                 n
#        sky in foc field        e w
#        ----------------         s
#
#                              del/DEC>0
#        aequat coords    alp/RA>0    alp/RA<0
#                              del/DEC<0
#
#
#   BUGS
#       none known...aaaaaaha ha ha ha ha.......
#
#-------------------------------------------------------------------------------
#        1         2         3         4         5         6         7         8
#23456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
#
itk::usual FORS {}

namespace eval fors {

class FORS {

#------------------------------------
constructor { ms mc mi mb args } {
#------------------------------------
    tracer ENTERING " of $this"
    global env
    global ::m_i;                   # External menu item index array
    global ::skycat_version

    if { [info exists env(HOST)] } {
        set lochost $env(HOST)
    }
    set m_skycat $ms;               # SkyCat pointer
    set m_canvas $mc;               # Canvas pointer
    set m_image $mi;                # Image pointer
    set m_mb $mb;                   # External menu button pointer

    set f_v [eval fims_version];    # fims version
    # set s_v [eval skycat_version];  # skycat version
    set s_v $skycat_version;        # skycat version
    set tcl_precision 18;

    # open a log-file to write some info
    # the instrument modus is not known yet
    #
    set log_file $log_dir
    append log_file "/startup.log"
    set lfId [open $log_file w]

    set mask_initialized 0

    # We need to configure -instrument, -camera and -modus options now
    eval configure $args

    # Ensure it is a boolean
    if { [$this cget -debug] != 0 } {
        eval configure { -debug 1 }
    }
    if { [$this cget -interactive] != 1 } {
        eval configure { -interactive 0 }
    }

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Some of the options will need to be configured again when the
    # Config/Layout menu exists.
    set cam [$this cget -camera]
    set rota [$this cget -rotangle]
    set color [$this cget -colorset]
    set trans [$this cget -graytype]
    set bpm [$this cget -badpixelmap]
    set lbl [$this cget -label]
    set cnt [$this cget -pics]

    puts stdout "\nNEW MASK --> SkyCat V$s_v -->  FIMS V$f_v for $instrument"

    # Configure some dynamic parts on the menu
    # Save button label and check
    $m_mb entryconfigure $m_i(SaveSetup) -label "Save '[$this cget -modus]'."
    if { [string compare $s_v $env(SKYCAT_VER)] } {
        $m_mb entryconfigure $m_i(SaveSetup) -state disabled
        info_dialog "SkyCat Version Error:\n\
                     \n\
                     You cannot \[Save\] !" .skycat1
    }
    # Autopos methods
    build_fors_menu_AutoSlit $m_skycat [$this cget -instrument] [$this cget -modus]
    # Reference with MOS10 or not
    build_fors_menu_RefTargets $m_skycat [$this cget -instrument] [$this cget -modus]
    # Layout options
    build_fors_menu_Layout $m_skycat [$this cget -instrument] [$this cget -modus]

    #Init CCD parameters
    init_ccd_params

    # Load FORS.isf, FORS1.isf or FORS2.isf
    # Configure grism requires dispersion table
    load_instrument

    # From here on collimat, filter, grism names are known

    # Collimator
    build_fors_menu_Collimator $m_skycat [$this cget -instrument] [$this cget -modus] $co_range
    # Camera
    build_fors_menu_Camera $m_skycat [$this cget -instrument] [$this cget -modus] $cam_range
    # Order separation filter
    build_fors_menu_Filter $m_skycat [$this cget -instrument] [$this cget -modus] $filt_range
    # Grism
    build_fors_menu_Grism $m_skycat [$this cget -instrument] [$this cget -modus] $gris_range

    # Configure defaults for some public attributes that modify the menu state
    eval configure { \
                     -slit_shape STRAIGHT \
                     -camera $cam \
                     -rotangle $rota \
                     -colorset $color \
                     -graytype $trans \
                     -badpixelmap $bpm \
                     -label $lbl \
                     -pics $cnt \
                   }

    # Configure the menu buttons according to ISF
    # Camera
    if { [string compare [$this cget -instrument] "FORS1"] == 0 } {
        eval configure { -camera [lindex $cam_range 0] }
    } elseif { [string compare [$this cget -instrument] "FORS2"] == 0 } {
        eval configure { -camera [lindex $cam_range 0] }
    } else {
        # Camera code (B or R) comes from user command line
        # Just keep it
    }
    # Once the Camera is defined, can set the CCD size
    load_ccd_window

    # Configure some special cases defaults
    set mii [lsearch -exact $filt_range none]
    # Filter decade 0,1...4,5...
    set sm [expr {$mii / 10}]
    # Index from 0 to 9 for each menu item of the current decade
    set smi [expr {$mii % 10}]
    if { $sm == 0 } {
        $m_mb.layout.fi invoke $m_i(Layout_Filter_m${sm}_$smi)
    } else {
        $m_mb.layout.fi.m$sm invoke $m_i(Layout_Filter_m${sm}_$smi)
    }

    switch -exact -- [$this cget -modus] {
        hitms {
            $m_mb.layout.sw invoke $m_i(Layout_SlitWidth_22_0)
            $m_mb.layout.sl invoke $m_i(Layout_SlitLength_5_0)
            $m_mb.layout.gr invoke $m_i(Layout_Grism_XGRIS_600B+92)
            eval configure { \
                             -long_width 0.5 \
                 }            
        }
        hits {
            $m_mb.layout.sw invoke $m_i(Layout_SlitWidth_1_0)
            $m_mb.layout.sl invoke $m_i(Layout_SlitLength_real)
            $m_mb.layout.gr invoke $m_i(Layout_Grism_none)
            eval configure { \
                             -long_width 0.7 \
                 }            
        }
        img {
            $m_mb.layout.sl invoke $m_i(Layout_SlitLength_real)
            $m_mb.layout.gr invoke $m_i(Layout_Grism_none)
            eval configure { \
                             -long_width 0.5 \
                 }            
        }
        default {
            $m_mb.layout.sw invoke $m_i(Layout_SlitWidth_1_0)
            $m_mb.layout.sl invoke $m_i(Layout_SlitLength_real)
            $m_mb.layout.gr invoke $m_i(Layout_Grism_none)
            eval configure { \
                             -long_width 0.5 \
                 }            
        }
    }

    # Default Standard Resolution collimator
    # We configure default collimator through the menu because
    # the default name depends on the instrument. The menu was
    # dinamically defined from load_instrument
    $m_mb.layout.sh invoke $m_i(Layout_Collimator_SR) 

    # The FIMS internal exposure counter will be part of the
    # setup files .p_targ .p_focf and .fims
    #
    set exposure_counter [get_ec [$this cget -modus]]

    # The initial project name is emty
    #
    $this configure -file_name ""

    # Read the equinox (2000 or 1950) from fits frame
    # Read WCS at reference pixel
    #
    set world_x [$m_image fits get CRVAL1]
    set world_y [$m_image fits get CRVAL2]
    set EQX     [$m_image fits get EQUINOX]
    if { $debug } {
        puts stdout "constructor: Any image WCS?"
        puts stdout "             world_x = >$world_x< (CRVAL1) \[degrees\]"
        puts stdout "             world_y = >$world_y< (CRVAL2) \[degrees\]"
    } else {
        puts stdout "WCS: world_x = $world_x   world_y = $world_y"
    }

    # Frameless mode, at least no image is loaded
    set frameless 0
    set wcsset [$m_image wcsset]
    if { $world_x == "" || $world_y == "" } {
        puts stdout "Frameless mode..."
        set frameless 1
        set wx    [lindex $wcsset 0]
        set wy    [lindex $wcsset 1]
        set fr_x  [lindex $wcsset 3]
        set fr_y  [lindex $wcsset 4]
        set fr_nx [lindex $wcsset 5]
        set fr_ny [lindex $wcsset 6]
        set EQX   [lindex $wcsset 8]
        if { $debug && $verbose } {
            puts stdout "          fr_x  = $fr_x (wcsset 3)"
            puts stdout "          fr_y  = $fr_y (wcsset 4)"
            puts stdout "          fr_nx = $fr_nx (wcsset 5)"
            puts stdout "          fr_ny = $fr_ny (wcsset 6)"
            puts stdout "             wx = >$wx< (wcsset 0)"
            puts stdout "             wy = >$wy< (wcsset 1)"
            puts stdout "            EQX = >$EQX< (wcsset 8)"
        }
        if { [catch {
                  $m_image convert coords $wx $wy "wcs $EQX" \
                                          world_x world_y "deg $EQX"
              } result] } {
            info_dialog "No Frame Loaded\n\No Catalog Loaded" .skycat1
            set world_x 0.0
            set world_y 0.0
        }
        set ccx $fr_x
        set ccy $fr_y
        set wx $world_x
        set wy $world_y
    } else {
        puts stdout "Image mode..."
        # If reference pixel is outside the image
        # mask is centered to frame (pixel) center
        # This can happen when using midas: EXTRACT/IMA
        #
        # default telecope pointing == mask center
        # == FITS header WCS reference pixel for WCS
        #
        set fr_x  [$m_image fits get CRPIX1]
        set fr_y  [$m_image fits get CRPIX2]
        set fr_nx [$m_image fits get NAXIS1]
        set fr_ny [$m_image fits get NAXIS2]
        if { $debug && $verbose } {
            puts stdout "          fr_x  = $fr_x (CRPIX1)"
            puts stdout "          fr_y  = $fr_y (CRPIX2)"
            puts stdout "          fr_nx = $fr_nx (NAXIS1)"
            puts stdout "          fr_ny = $fr_ny (NAXIS2)"
        }
        if { $fr_x > $fr_nx ||
             $fr_y > $fr_ny } {
            puts stdout [format "Warning: Reference pixel outside frame:     X = %8.2f       Y = %8.2f" $fr_x $fr_y]
            puts stdout [format "                                 Frame  Width = %6d  Height = %6d" $width $height]
            # Reference pixel outside frame, take frame center
            set fr_x [expr {$width/2}]
            set fr_y [expr {$height/2}]
            puts stdout [format "         Reference pixel changed to:        X = %8.2f       Y = %8.2f" $fr_x $fr_y]
            # Recentering to frame center pixel
            if { [catch {
                      $m_image convert coords $fr_x $fr_y image world_x world_y "deg $EQX"
                  } result] } {
                if { $debug } {
                    puts stdout "             Frame center (fr_x,fr_y)"
                    puts stdout "             Error converting ($fr_x,$fr_y) image to (world_x,world_y) \[deg\])"
                    puts stdout "             $result" 
                }
            } else {
                puts stdout [format "                     recentering to   world_x = %8.2f world_y = $world_y" $world_x $world_y]
            }
        }
        set ccx $fr_x
        set ccy $fr_y
        set wx $world_x
        set wy $world_y
        if { $debug && $verbose } {
            puts stdout "             wx = >$wx< (CRVAL1)"
            puts stdout "             wy = >$wy< (CRVAL2)"
            puts stdout "            EQX = >$EQX< (EQUINOX)"
        }
    }
    if { $debug } {
        puts stdout "        world_x = >$world_x< \[degrees\]"
        puts stdout "        world_y = >$world_y< \[degrees\]"
        puts stdout "            ccx = >$ccx< \[pix\]"
        puts stdout "            ccy = >$ccy< \[pix\]"
    }

    # Create an instance of the reference target class
    #
    fors::REF_TARG C_ref_targ $m_canvas \
                              $m_image \
                              -eqx $EQX \
                              -debug [$this cget -debug] \
                              -verbosity [$this cget -verbosity] \
                              -interactive [$this cget -interactive]

    # Initial mask scale taken from fits header if fits from FORS itself
    #
    # For the status panel
    set sp_instrume [$m_image fits get INSTRUME]
    set sp_origin   [$m_image fits get ORIGIN]
    if { [string match *FORS* $sp_instrume] } {
        # Reset external radio button
	$this configure -fors_frame 1
        if { [string compare ESO $sp_origin] } {
            puts stdout "Warning: FITS file observed with FORS but processed by [$m_image fits get ORIGIN]"
        }
    } else {
	$this configure -fors_frame 0
    }

    # Copy the current image scale to mxu class
    #
    if { [string compare [$this cget -modus] hitms] == 0 ||
         [string compare [$this cget -modus] mxu] == 0 } {
        fors::MXU_SLIT::fc $focscale
        fors::MXU_SLIT::label [$this cget -label]
        fors::MXU_SLIT::transparency [$this cget -graytype]
    }

    # mxu  : has internal mmfocalfield <-> canvas conversion
    # echs : requires explicitly mm <-> canvas conversion
    # Create an instance of the coord conversion class
    # MM - canvas coords
    # badpixelmap requires conversion for all modi
    fors::CONV_MC C_mc -eqx $EQX

    # Create an instance of the coord conversion class
    # WCS - canvas coords
    fors::CONV_WC C_wc -m_image $m_image -eqx $EQX

    # Create a mask at the initial WCS position with default parameters
    #
    init_mask_world2

    # Plot the mask on the canvas
    #
    plot_mask

    # Start the basic bindings : move and rotate the mask
    #
    bind_frame

    # Update canvas scale
    #
    update_scale

    if { [$this cget -badpixelmap] } {
        update_BPM
    }

    # Open a  new log file
    open_log

    # Expiration checking in the begining of a session
    # if { [expire_date] } {
    #     warn
    # }

    # Do not load refslits for hitms
    if { [string match "*mxu*" [$this cget -modus]] } {
        mxu_create_refslit
    }
    puts $lfId  "Mask initialized"
    puts stdout "Mask initialized"
    tracer MSG "[tracer TAB  ]Mask Initialized"

    # Create an instance of the Status Panel
    if { [catch {fors::StatusPanel SP -fors_class $this}] } {
        SP configure -fors_class $this
    }
    
    set last_action "Enter [string toupper [$this cget -modus]]"
    catch {SP update}
    tracer LEAVING "end of $this"
}

#-----------
destructor {
#-----------
    tracer ENTERING " of $this"
    global imageName
    global ::m_i

    # delete drawings from the canvas
    erase
    if { $debug } {
        puts stdout "Quit [string toupper [$this cget -modus]] Mode"
    }
    puts $lfId ">>> Quit Mode <<<"
    # Close the log file
    catch { close $lfId }

    # Delete object instances
    catch { delete object C_ref_targ }
    catch { delete object C_wc }
    catch { delete object C_mc }

    # Delete all instances of MXU slit
    mxu_delete_all_slits

    $m_mb entryconfigure $m_i(SaveSetup) -label "Save Setup"
    set last_action "Quit [string toupper [$this cget -modus]]"

    # Update Status Panel
    catch {SP update}
    catch {SP configure -fors_class ""}

    if { $debug } {
        puts stdout "Mask $this destroyed"
    }
    tracer LEAVING "end of $this"
}

#========================
private method init {} {
#========================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    tracer LEAVING
}

#===================================================
private method init_ccd_params { {ccd "FORS-MIT"}} {
#===================================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"]
    if { $binx == 1 } {
        set PIX 0.015
        set CCDscale_standard 0.125062
        set CCDscale_highres 0.0625313
    } else {
        set PIX 0.030
        set CCDscale_standard 0.250124
        set CCDscale_highres  0.1250626
    }
    tracer LEAVING
}

#==================================
private method load_instrument {} {
#==================================
# Open the instrument summary file, reads available
# configuration and set relevant variables.
# ISF file format is a PAF and must follow certain rules:
# - Avoid comments not starting at col 1
# - One keyword per line
# - Line ends with semicolon
# - All keyword names in upper case
# - All keyword values enclosed in quotes
# - Keyword sequence numbers without leading zeroes
# - 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    puts stdout "Executing load_instrument ..."

    set pattern1 *.VALUE*
    set pattern2 *.RANGE*
    set pattern3 GRIS*.FILT*
    set pattern4 EX.COMB.*
    set pattern5 CCD.*

    set co_range      "";  # collimator range
    set cam_range     "";  # camera range
    set filt_range    "";  # filter range
    set fwhm_range    "";  # FWHM of filtrange
    set lamcent_range "";  # lam0 of filtrange
    set osfi_range    "";  # OS filter range
    set gris_range    "";  # grism range (normal dispersion)
    set egris_range   "";  # echelle grism range
    set xgris_range   "";  # cross disperser grism range
    set slit_range    "";  # long slit range
    set htslit_range  "";  # HITS slit range
    set echelle_range "";  # echelle range
    set ph_range      "";  # pinhole range

    set line_no 0;  # File line counter
    set osf_cnt 0;  # Order separation filter counter
    set cam_tmp ""; # Camera temporary range, which translates CAMERA+3 to B and CAMERA+8 to R
    set disp_string ""
    set nf_refslit 0

    if { [file exists $isf_file] & [file readable $isf_file] } {
        set fileId [open $isf_file r]
        puts stdout "load_instrument $isf_file"
        puts $lfId  "load_instrument $isf_file"
        # as long as there are lines in the file
        while { [gets $fileId line_org] >= 0 } {
            if { [string first # $line_org] == 0 } {
                # ignore lines that begin with comment character '#'
                # puts stdout $line_org
                incr line_no
            } else {
                if { [string match *# $line_org] } {
                    # Trim the line to the right of the '#'
                    set line_org [string range $line_org 0 [expr {[string first # $line_org] - 1}]]
                }
                incr line_no
                set kwd UNDEFINED.KEYWORD
                set nbr 0
                set val 0
                #----------------------------
                # PART-1 load mask dimensions
                #----------------------------
                if { [string match $pattern1 $line_org] == 1 } {
                    # A line like *.VALUE
                    scan [string trim [lrange [split $line_org .] 0 0] \[{}\]] "%s" kwd
                    set nbr [string trimleft [string toupper $kwd] \[ABCDEFGHIJKLMNOPQRSTUVWXYZ\]]
                    if { [scan [string trim [lrange [split $line_org \"] 1 1] \[{}\]] "%f" val] == -1 } {
                        puts "Error in $isf_file, line $line_no:"
                        puts "$kwd value is not enclosed in quotes"
                    }
                    set val [format "%.4f" $val]
                    switch -glob -- $line_org {
                        *MOSSLIT* {
                            # MOS keyword
                            switch -glob -- $kwd {
                                *MIN* {
                                    set mosslitmin($nbr) $val
                                }
                                *MAX* {
                                    set mosslitmax($nbr) $val
                                }
                            }
                        }
                        *LONGSLIT* {
                            # LSS keyword
                            switch -glob -- $kwd {
                                *XMIN* {
                                    set longslitxmin($nbr) $val
                                }
                                *YMIN* {
                                    set longslitymin($nbr) $val
                                }
                                *XMAX* {
                                    set longslitxmax($nbr) $val
                                }
                                *YMAX* {
                                    set longslitymax($nbr) $val
                                }
                            }
                        }
                        *HTSLIT* {
                            # HITS keyword
                            switch -glob -- $kwd {
                                *XMIN* {
                                    set htslitxmin($nbr) $val
                                }
                                *YMIN* {
                                    set htslitymin($nbr) $val
                                }
                                *XMAX* {
                                    set htslitxmax($nbr) $val
                                }
                                *YMAX* {
                                    set htslitymax($nbr) $val
                                }
                            }
                        }
                        *POLARIMSLIT* {
                            # POL keyword
                            switch -glob -- $kwd {
                                *XMIN* {
                                    set polarimslitxmin($nbr) $val
                                }
                                *YMIN* {
                                    set polarimslitymin($nbr) $val
                                }
                                *XMAX* {
                                    set polarimslitxmax($nbr) $val
                                }
                                *YMAX* {
                                    set polarimslitymax($nbr) $val
                                }
                            }
                        }
                        *FOCFIELD* {
                            # Focal field keyword
                            # These are intervals, meaning they have two floats: min and max
                            set min 0
                            set max 0
                            scan [string trim [lrange [split $line_org \"] 1 1] \[{}\]] "%f %f" min max
                            set min [format "%.4f" $min]
                            set max [format "%.4f" $max]
                            if { $min > $max } {
                                set val $min
                                set min $max
                                set max $val
                            }
                            switch -glob -- $kwd {
                                *XRANGESR* {
                                    set XSRMIN $min
                                    set XSRMAX $max
                                }
                                *YRANGESR* {
                                    set YSRMIN $min
                                    set YSRMAX $max
                                }
                                *XRANGEHR* {
                                    set XHRMIN $min
                                    set XHRMAX $max
                                }
                                *YRANGEHR* {
                                    set YHRMIN $min
                                    set YHRMAX $max
                                }
                            }
                        }
                        FIELDXRANGE* -
                        FIELDYRANGE* {
                            # Field stop keyword
                            # These are intervals, meaning they have two floats: min and max
                            set min 0
                            set max 0
                            scan [string trim [lrange [split $line_org \"] 1 1] \[{}\]] "%f %f" min max
                            set min [format "%.4f" $min]
                            set max [format "%.4f" $max]
                            if { $min > $max } {
                                set val $min
                                set min $max
                                set max $val
                            }
                            switch -glob -- $kwd {
                                *X* {
                                    set FIELDSTOP_XMIN $min
                                    set FIELDSTOP_XMAX $max
                                }
                                *Y* {
                                    set FIELDSTOP_YMIN $min
                                    set FIELDSTOP_YMAX $max
                                }
                            }

                        }
                        *PINH* {
                            # Pin holes keyword
                            switch -glob -- $kwd {
                                *XCENT* {
                                    set pinhxcent($nbr) $val
                                }
                                *YCENT* {
                                    set pinhycent($nbr) $val
                                }
                            }
                        }
                    }
                }

                #-------------------------
                # PART-2 load setup ranges
                #-------------------------
                if { [string match $pattern2 $line_org] == 1 } {
                    # A line like *.RANGE: CCD.*.RANGE will be ignored here and processed in PART-4
                    scan [string trim [lrange [split $line_org .] 0 0] \[{}\]] "%s" kwd
                    set range_string [string trim [lrange [split $line_org \"] 1 1] \[{}\]]
                    if { [string length $range_string] == 0 } {
                        puts "Error in $isf_file, line $line_no:"
                        puts "$kwd range is not enclosed in quotes"
                        set range_string UNDEFINED
                    }
                    switch -glob -- $line_org {
                        *FILT* {
                            set filt_range [ldelete_all $range_string ""]
                        }
                        *FWHM* {
                            set fwhm_range [ldelete_all $range_string ""]
                        }
                        *LAMCENT* {
                            set lamcent_range [ldelete_all $range_string ""]
                        }
                        EGRIS* {
                            set egris_range [ldelete_all $range_string ""]
                        }
                        XGRIS* {
                            set xgris_range [ldelete_all $range_string ""]
                        }
                        GRIS* {
                            set gris_range [ldelete_all $range_string ""]
                        }
                        CAM* {
                            set cam_range [ldelete $range_string ""]
                            if { [string match "*3*" $cam_range] } {
                                lappend cam_tmp B
                            }
                            if { [string match "*8*" $cam_range] } {
                                lappend cam_tmp R
                            }
                        }
                        *HTSLIT* {
                            set htslit_range [ldelete $range_string ""]
                        }
                        *SLIT* {
                            set slit_range [ldelete $range_string ""]
                        }
                        *PINH* {
                            set ph_range [ldelete $range_string ""]
                        }
                        *COLL* {
                            set co_range [ldelete $range_string ""]
                        }
                        MXU* {
                            # MXU reference slits
                            set len [llength $range_string]
                            switch -glob -- $kwd {
                                *REFTP* {
                                    set nf_refslit $len
                                    for { set i 0 } { $i < $len } { incr i } {
                                        set mxureftp($i) [lindex $range_string $i]
                                    }
                                }
                                *REFSX* {
                                    for { set i 0 } { $i < $len } { incr i } {
                                        set mxurefsx($i) [lindex $range_string $i]
                                    }
                                }
                                *REFSY* {
                                    for { set i 0 } { $i < $len } { incr i } {
                                        set mxurefsy($i) [lindex $range_string $i]
                                    }
                                }
                                *REFWD* {
                                    for { set i 0 } { $i < $len } { incr i } {
                                        set mxurefwd($i) [lindex $range_string $i]
                                    }
                                }
                                *REFLE* {
                                    for { set i 0 } { $i < $len } { incr i } {
                                        set mxurefle($i) [lindex $range_string $i]
                                        set mxurefof($i) 0
                                    }
                                }
                                *REFRO* {
                                    for { set i 0 } { $i < $len } { incr i } {
                                        set mxurefro($i) [lindex $range_string $i]
                                    }
                                }
                            }
                        }
                    }
                }
                #-----------------------------
                # PART-3 load dispersion array
                #-----------------------------
                if { [string match $pattern3 $line_org] == 1 } {
                    # A line like GRIS1.FILT1
                    scan [string trim [lrange [split $line_org .] 0 0] \[{}\]] "%s" kwd
                    set nbr [string trimleft [string toupper $kwd] \[ABCDEFGHIJKLMNOPQRSTUVWXYZ\]]
                    set row_string [string trim [lrange [split $line_org \"] 1 1] \[{}\]]
                    if { [string length $row_string] == 0 } {
                        puts "Error in $isf_file, line $line_no:"
                        puts "$kwd range is not enclosed in quotes"
                        set row_string "UNDEFINED 0.0 0 0 0"
                    }
                    lassign $row_string l_filt l_disp l_cen l_min l_max l_ang
                    incr osf_cnt
                    set A_disp(0) $osf_cnt
                    set A_disp($osf_cnt,grism) [lindex $gris_range [expr {$nbr - 1}]]
                    set A_disp($osf_cnt,filter) $l_filt
                    set A_disp($osf_cnt,disp) $l_disp
                    set A_disp($osf_cnt,cen) $l_cen
                    set A_disp($osf_cnt,min) $l_min
                    set A_disp($osf_cnt,max) $l_max
                    set A_disp($osf_cnt,ang) $l_ang
                    append disp_string [format "%-2d   " $osf_cnt]
                    append disp_string [format "%-16s   " $A_disp($osf_cnt,grism)]
                    append disp_string [format "%-16s   " $A_disp($osf_cnt,filter)]
                    append disp_string [format "%6.2f   " $A_disp($osf_cnt,disp)]
                    append disp_string [format "%8.2f   " $A_disp($osf_cnt,cen)]
                    append disp_string [format "%8.2f   " $A_disp($osf_cnt,min)]
                    append disp_string [format "%8.2f   " $A_disp($osf_cnt,max)]
                    append disp_string [format "%8.2f   \n" [expr {$A_disp($osf_cnt,ang)%360}]]
                    # append this order separation filter to osfi_range in case it is not yet in it
                    if { [string match *$A_disp($osf_cnt,filter)* $osfi_range] == 0 } {
                        append osfi_range " $A_disp($osf_cnt,filter)"
                    }


                }
                if { [string match $pattern4 $line_org] == 1 } {
                    # A line like EX.COMB.1
                    set row_string [string trim [lrange [split $line_org \"] 1 1] \[{}\]]
                    if { [string length $row_string] == 0 } {
                        puts "Error in $isf_file, line $line_no:"
                        puts "$kwd range is not enclosed in quotes"
                        set row_string UNDEFINED
                    }
                    append echelle_range "$row_string "
                }
                #---------------------------
                # PART-4 load ccd dimensions
                #---------------------------
                if { [string match $pattern5 $line_org] == 1 } {
                    # First character of kwd is the camera code: B or R
                    scan [string trim [lrange [split $line_org .] 1 1] \[{}\]] "%s" kwd
                    set cam [csubstr $kwd 0 1]
                    if { [string length $cam] == 0 } {
                        set cam [lindex $cam_tmp 0]
                    }
                    # These are intervals, meaning they have two floats: min and max
                    set min 0
                    set max 0
                    scan [string trim [lrange [split $line_org \"] 1 1] \[{}\]] "%f %f" min max
                    set min [format "%.4f" $min]
                    set max [format "%.4f" $max]
                    if { $min > $max } {
                        set val $min
                        set min $max
                        set max $val
                    }
                    switch -glob -- $kwd {
                        *SLAVE_XSR* {
                            set ccdsize($cam,SLAVE_XSR_MIN) $min
                            set ccdsize($cam,SLAVE_XSR_MAX) $max
                        }
                        *MASTER_XSR* {
                            set ccdsize($cam,MASTER_XSR_MIN) $min
                            set ccdsize($cam,MASTER_XSR_MAX) $max
                        }
                        *SLAVE_YSR* {
                            set ccdsize($cam,SLAVE_YSR_MIN) $min
                            set ccdsize($cam,SLAVE_YSR_MAX) $max
                        }
                        *MASTER_YSR* {
                            set ccdsize($cam,MASTER_YSR_MIN) $min
                            set ccdsize($cam,MASTER_YSR_MAX) $max
                        }
                        *SLAVE_XHR* {
                            set ccdsize($cam,SLAVE_XHR_MIN) $min
                            set ccdsize($cam,SLAVE_XHR_MAX) $max
                        }
                        *MASTER_XHR* {
                            set ccdsize($cam,MASTER_XHR_MIN) $min
                            set ccdsize($cam,MASTER_XHR_MAX) $max
                        }
                        *SLAVE_YHR* {
                            set ccdsize($cam,SLAVE_YHR_MIN) $min
                            set ccdsize($cam,SLAVE_YHR_MAX) $max
                        }
                        *MASTER_YHR* {
                            set ccdsize($cam,MASTER_YHR_MIN) $min
                            set ccdsize($cam,MASTER_YHR_MAX) $max
                        }
                        *GAP_SR* {
                            set CCDGAP_SR $min
                            puts $lfId "CCD SR Gap: $CCDGAP_SR"
                        }
                        *GAP_HR* {
                            set CCDGAP_HR $min
                            puts $lfId "CCD HR Gap: $CCDGAP_HR"
                        }
                    }
                }
            }
        }
        puts $lfId  "Closed $isf_file"
        puts stdout "Closed $isf_file"
        close $fileId

        # Post Processing
        # Change camera range to the one letter code (B R)
        set cam_range $cam_tmp

        # LSS slits only for the 'all' option
        set longslitxmin(0) $longslitxmin(9)
        set longslitxmax(0) $longslitxmax(9)
        set longslitymin(0) $longslitymin(1)
        set longslitymax(0) $longslitymax(1)

        # MOS slits 1 and 19
        # don't show the real dimension of MOS1 and MOS19 but only
        # the fractional part which is inside the focal field
        #
        # set mosslitmin(1)  $YSRMIN
        # set mosslitmax(19) $YSRMAX

        # HITS slits only for the 'all' option
        set htslitxmin(0) $htslitxmin(8)
        set htslitxmax(0) $htslitxmax(8)
        set htslitymin(0) $htslitymin(1) 
        set htslitymax(0) $htslitymax(1)
        # only for the long_number=1,2
        # pinholes hardcoded
        # big pinhole
        set htslitxmin(9) [expr {$pinhxcent(1) - 0.95 / $focscale}]
        set htslitxmax(9) [expr {$pinhxcent(1) + 0.95 / $focscale}]
        set htslitymin(9) [expr {$pinhycent(1) - 0.95 / $focscale}]
        set htslitymax(9) [expr {$pinhycent(1) + 0.95 / $focscale}]
        # small pinhole
        set htslitxmin(0) [expr {$pinhxcent(2) - 0.02 / $focscale}]
        set htslitxmax(0) [expr {$pinhxcent(2) + 0.02 / $focscale}]
        set htslitymin(0) [expr {$pinhycent(2) - 0.02 / $focscale}]
        set htslitymax(0) [expr {$pinhycent(2) + 0.02 / $focscale}]

        # FOCFIELD checks and report
        # XSRMIN-MAX are the optical focal field limits
        # XBLADEA, XBLADEB are the park positions of the MOS
        #                  are the min/max  of INS.MOS.POS
        # The park positions must be lower than the limit switches.
        # The nominal park positions are +-123 mm
        #
        set XBLADEA -123.0; # MOSA (mountangle 0=west, 180=east)
        set XBLADEB +123.0; # MOSB (mountangle 0=east, 180=west)
        if { $XHRMAX > $XSRMAX } {
            puts stdout "Warning:$myfile: XHRMAX is greater than XSRMAX"
        }

        # OSF: Order Separation Filters
        incr osf_cnt
        set A_disp(0)              $osf_cnt
        set A_disp($osf_cnt,grism)  none
        set A_disp($osf_cnt,filter) none
        set A_disp($osf_cnt,disp)     5.00
        set A_disp($osf_cnt,cen)    500
        set A_disp($osf_cnt,min)    500
        set A_disp($osf_cnt,max)    500
        set A_disp($osf_cnt,ang)      0
        append disp_string [format "%-2d   "  $osf_cnt]
        append disp_string [format "%-16s   " $A_disp($osf_cnt,grism)]
        append disp_string [format "%-16s   " $A_disp($osf_cnt,filter)]
        append disp_string [format "%6.2f   " $A_disp($osf_cnt,disp)]
        append disp_string [format "%8.2f   " $A_disp($osf_cnt,cen)]
        append disp_string [format "%8.2f   " $A_disp($osf_cnt,min)]
        append disp_string [format "%8.2f   " $A_disp($osf_cnt,max)]
        append disp_string [format "%8.2f   \n" $A_disp($osf_cnt,ang)]
    } else {
        puts $lfId  "Warning:$isf_file: does not exist or has no read permission"
        puts stdout "Warning:$isf_file: does not exist or has no read permission"
    }

    # Report loaded values and ranges
    puts $lfId "XSRMIN: $XSRMIN | XSRMAX: $XSRMAX | XHRMIN: $XHRMIN | XHRMAX: $XHRMAX"
    puts $lfId "YSRMIN: $YSRMIN | YSRMAX: $YSRMAX | YHRMIN: $YHRMIN | YHRMAX: $YHRMAX"
    puts $lfId "$osf_cnt gris-filt combinations read from $isf_file\n"
    puts $lfId "$disp_string\n"

    puts $lfId "     co_range = $co_range"
    puts $lfId "    cam_range = $cam_range"
    puts $lfId "   filt_range = $filt_range"
    puts $lfId "   fwhm_range = $fwhm_range"
    puts $lfId " lamcentrange = $lamcent_range"

    # last filter 'none' has not entries in fwhm and lamcent
    set a [expr {[llength $filt_range] - 1}]
    for { set i 0 } { $i < $a } { incr i 1 } {
        puts $lfId [format "%2d %18s %8.3f %8.3f" \
                            $i [lindex $filt_range $i] \
                               [lindex $lamcent_range $i] \
                               [lindex $fwhm_range $i]]
    }

    puts $lfId "   osfi_range = $osfi_range"
    puts $lfId "  Xgris_range = $xgris_range"
    puts $lfId "  Egris_range = $egris_range"
    puts $lfId "   gris_range = $gris_range"
    puts $lfId "   slit_range = $slit_range"
    puts $lfId " htslit_range = $htslit_range"
    puts $lfId "echelle_range = $echelle_range"
    puts $lfId "     ph_range = $ph_range"

    puts $lfId "FIELD STOP XMIN: $FIELDSTOP_XMIN"
    puts $lfId "FIELD STOP XMAX: $FIELDSTOP_XMAX"
    puts $lfId "FIELD STOP YMIN: $FIELDSTOP_YMIN"
    puts $lfId "FIELD STOP YMAX: $FIELDSTOP_YMAX"
    tracer LEAVING
}

#============================
private method load_BPM { } {
#============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set filename [format "F%1dBadPixelMap.asc" $insId]
    set myfile $int_dir/../config/$filename
    # puts stdout $myfile

    set i 0
    if { [file exists $myfile] & [file readable $myfile] } {
        set fileId [open $myfile r]
        puts $lfId "FORS::load_BPM  $fileId"
        # puts stdout "FORS::load_BPM  $fileId"

	# read FORS CCD reference pixel
	gets $fileId line_org
	set rpx [lindex $line_org 0]; set rpy [lindex $line_org 1]
        # puts stdout "$rpx $rpy"

        if { [string match *HR* [$this cget -collimat]] } {
            set ls $CCDscale_highres
        } else {
            set ls $CCDscale_standard
        }

        # as long as there are lines in the file
        while { [gets $fileId line_org] >= 0 } {
            if { [string match #* $line_org] } {
                # ignore comment lines
                # puts stdout $line_org
            } else {
                # now no comment lines
                incr i +1
                # badpixel - reference pixel to mm focal plane
                # (0.2 ''/pixel) / (1.89 ''/mm) (SR)
                # (0.1 ''/pixel) / (1.89 ''/mm) (HR)
                set BPM($i,xmin) [expr ([lindex $line_org 0]-$rpx) * \
                                                $ls/$focscale]
                set BPM($i,xmax) [expr ([lindex $line_org 1]-$rpx) * \
                                                $ls/$focscale]
                set BPM($i,ymin) [expr ([lindex $line_org 2]-$rpy) * \
                                               -$ls/$focscale]
                set BPM($i,ymax) [expr ([lindex $line_org 3]-$rpy) * \
                                               -$ls/$focscale]
                # puts stdout [format "%2d %8.3f %8.3f %8.3f %8.3f" \
                #                     $i  $BPM($i,xmin) $BPM($i,xmax) \
                #                         $BPM($i,ymin) $BPM($i,ymax)]
            }
        }
    }
    set BPM_counter $i
    # puts stdout "load $BPM_counter bad pixel areas"
    tracer LEAVING
}

#==================================
public method load_ccd_window { } {
#==================================
# Loading the CCD dimensions for Master and Slave from ccdsize array
# depending on the selected camera

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set cam [$this cget -camera]
    set CCDMASTER_XSRMIN  $ccdsize($cam,MASTER_XSR_MIN)
    set CCDMASTER_XSRMAX  $ccdsize($cam,MASTER_XSR_MAX)
    set CCDSLAVE_XSRMIN   $ccdsize($cam,SLAVE_XSR_MIN)
    set CCDSLAVE_XSRMAX   $ccdsize($cam,SLAVE_XSR_MAX)
    set CCDMASTER_YSRMIN  $ccdsize($cam,MASTER_YSR_MIN)
    set CCDMASTER_YSRMAX  $ccdsize($cam,MASTER_YSR_MAX)
    set CCDSLAVE_YSRMIN   $ccdsize($cam,SLAVE_YSR_MIN)
    set CCDSLAVE_YSRMAX   $ccdsize($cam,SLAVE_YSR_MAX)
    set CCDMASTER_XHRMIN  $ccdsize($cam,MASTER_XHR_MIN)
    set CCDMASTER_XHRMAX  $ccdsize($cam,MASTER_XHR_MAX)
    set CCDSLAVE_XHRMIN   $ccdsize($cam,SLAVE_XHR_MIN)
    set CCDSLAVE_XHRMAX   $ccdsize($cam,SLAVE_XHR_MAX)
    set CCDMASTER_YHRMIN  $ccdsize($cam,MASTER_YHR_MIN)
    set CCDMASTER_YHRMAX  $ccdsize($cam,MASTER_YHR_MAX)
    set CCDSLAVE_YHRMIN   $ccdsize($cam,SLAVE_YHR_MIN)
    set CCDSLAVE_YHRMAX   $ccdsize($cam,SLAVE_YHR_MAX)

    puts $lfId "Master SR XMIN: $CCDMASTER_XSRMIN"
    puts $lfId "Master SR XMAX: $CCDMASTER_XSRMAX"

    puts $lfId "Slave SR XMIN: $CCDSLAVE_XSRMIN"
    puts $lfId "Slave SR XMAX: $CCDSLAVE_XSRMAX"

    puts $lfId "Master SR YMIN: $CCDMASTER_YSRMIN"
    puts $lfId "Master SR YMAX: $CCDMASTER_YSRMAX"

    puts $lfId "Slave SR YMIN: $CCDSLAVE_YSRMIN"
    puts $lfId "Slave SR YMAX: $CCDSLAVE_YSRMAX"

    puts $lfId "Master HR XMIN: $CCDMASTER_XHRMIN"
    puts $lfId "Master HR XMAX: $CCDMASTER_XHRMAX"

    puts $lfId "Slave HR XMIN: $CCDSLAVE_XHRMIN"
    puts $lfId "Slave HR XMAX: $CCDSLAVE_XHRMAX"

    puts $lfId "Master HR YMIN: $CCDMASTER_YHRMIN"
    puts $lfId "Master HR YMAX: $CCDMASTER_YHRMAX"

    puts $lfId "Slave HR YMIN: $CCDSLAVE_YHRMIN"
    puts $lfId "Slave HR YMAX: $CCDSLAVE_YHRMAX"
    tracer LEAVING
}


#====================================
public method load_TPX { filename } {
#====================================
# The central method to load a .fims file.
# If in MOS or PMOS the -slit_width == open,
# load an ims.fims file

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    switch -exact -- [$this cget -modus] {
        echs {
            load_mos_TPX $filename
        }
        mos -
        pmos {
            if { [string compare [$this cget -slit_width] open] == 0 } {
                load_img_TPX $filename
            } else {
                load_mos_TPX $filename
            }
        }
        hits -
        img -
        ipol -
        lss {
            load_img_TPX $filename
        }
        hitms {
            load_hitms_TPX $filename
        }
        mxu {
            load_mxu_TPX $filename
        }
    }

    # Number of reference targets
    set sp_nort [C_ref_targ nort]

    # Each load_*_TPX method called above should call reset_mask which resets
    # AutoposMethod to ""
    append AutoposMethod "configured "

    # For status panel
    set last_action "Load SetUp"
    catch {SP update}

    # Verify reference_targets
    set fail_flag [edit_ref_targ verify_load]
    if { $fail_flag } {
        if { [$this cget -interactive] } {
            info_dialog "There are reference targets in the forbidden zone!!!." .skycat1
        }
    }
    tracer LEAVING
}

#===========================================
private method load_hitms_TPX { filename } {
#===========================================
# Reading a hitms2.<ec><pn>.fims file
# The input file must be a setup file like 'hitms2.12.project.fims'
# Slits are given in wcs

    tracer ENTERING
    global ::m_i

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # For the coordinate transformation matrix
    C_wc coeff_wc $wx $wy

    # --- Open filename ---------
    if { $filename == "" } {
        # No filename means load result from AutoPos, written to
        # .fims/LOG/<modus><1|2>.<ec>.autopos.fims
        set myfile $log_dir
        append myfile "/[$this cget -modus]${insId}.$exposure_counter.auto_pos.fims"
        if { $debug && $verbose } {
            puts stdout "load_hitms_TPX: No filename, loading AutoPos TPX from $myfile"
        }
        set auto_flag 1
    } else {
        set myfile $set_dir
        if { $debug && $verbose } {
            puts stdout "load_hitms_TPX: Loading TPX file >$filename< from $set_dir"
        }
        if { $filename != 0 } {
            append myfile "/$filename"
        } else {
            # filename is zero, call dialog
            global .fs
            catch { destroy .fs  }
            set myfile [filename_dialog $set_dir "[$this cget -modus]${insId}.*.fims"]
        }
        set auto_flag 0
    }

    # If file is a fors2 standard-mask aperture definition file (FAD)
    # b must be 'fad'
    set a [lindex [split $myfile "/"] [expr {[llength [split $myfile "/"]] - 1}]]
    set b [lindex [split $a "."] [expr {[llength [split $a "."]] - 1}]]
    if { [string compare $b fad] == 0 } {
        load_mxu_FAD $myfile
        tracer LEAVING FAD
        return
    }

    set tel_kwd TEL
    set ins_kwd INS
    set seq_kwd SEQ.PREP
    set f_cam 0
    set f_gris 0
    set f_filt 0
    set f_slit_width 0
    set f_coll 0
    set f_lmin 0
    set L_prop M_
    set L_mid 900000
    set lPA 0
    set lty ""
    set lwx 000000.00
    set lwy 000000.00
    set epo_flag 0
    set equ_flag 0

    # ------- Read TCS keywords -------------------------
    # Step 1: Load opt1.targ, read only TEL pointing.
    #         Shift and rotate according to these values.
    # ---------------------------------------------------
    if { [file exists $myfile] & [file readable $myfile] } {
        if { $debug && $verbose } {
            puts stdout "load_hitms_TPX: Opening TPX file >$myfile<"
        }
	set fileId [open $myfile r]
        if { $debug && $verbose } {
	    puts stdout "load_hitms_TPX: File opened with fileId >$fileId<"
        }
	puts $lfId "load_hitms_TPX: File opened with fileId >$fileId<"
        
	# As long as there are lines in the file...
	while { [gets $fileId line_org] >= 0 } {
	    if { [string match #* $line_org] } {
		if { $debug && $verbose } {
                    puts stdout $line_org
                }
	    } else {
                if { [string match *$tel_kwd* $line_org] == 1 } {
		    # TEL
		    switch -glob -- $line_org {
			*ALPHA* {
                            set lwx [lindex $line_org 1]
                            if { $debug && $verbose } {
                                 puts stdout "TEL.TARG.ALPHA = $lwx"
                            }
                        }
			*DELTA* {
                            set lwy [lindex $line_org 1]
                            if { $debug && $verbose } {
                                 puts stdout "TEL.TARG.DELTA = $lwy"
                            }
                        }
			*ROT.OFF* {
                            set lPA [lindex $line_org 1]
                            if { $debug && $verbose } {
                                 puts stdout "TEL.ROT.OFFANGLE = $lPA"
                            }
                        }
			*EPOCH* {
                            set EQX [lindex $line_org 1]
                            set epo_flag 1
                            if { $debug && $verbose } {
                                 puts stdout "TEL.TARG.EPOCH = $EQX"
                            }
                        }
			*EQUINOX* {
                            set EQX [lindex $line_org 1]
                            set equ_flag 1
                            if { $debug && $verbose } {
                                 puts stdout "TEL.TARG.EQUINOX = $EQX"
                            }
                        }
			*TYPE* {
                            set lty [lindex $line_org 1]
                            if { $debug && $verbose } {
                                 puts stdout "TEL.TARG.TYPE = $lty"
                            }
                        }
		    }
		}

		# --- by the way, read SEQ.PREP keywords ---
		if { [string match *$seq_kwd* $line_org] == 1 } {
                    if { [string match *CAM* $line_org] == 1 } {
                        # the camera
                        set l_cam [lindex $line_org 1]
                        set f_cam 1
                        if { $debug && $verbose } {
                             puts stdout "SEQ.PREP.CAM = $l_cam"
                        }
                    }
		    if { [string match *COLL* $line_org] } {
			# the collimator
			set l_coll [lindex $line_org 1]
			set f_coll 1
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.COLL = $l_coll"
                        }
		    }
		    if { [string match *GRIS* $line_org] } {
			# the grism
			set l_gris [lindex $line_org 1]
			set f_gris 1
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.GRISM = $l_gris"
                        }
		    }
		    if { [string match *LMIN* $line_org] } {
			# the user defined wavelength region
			set L_MIN  [lindex $line_org 1]
			set f_lmin 1
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.LMIN = $L_MIN"
                        }
		    }
		    if { [string match *LMAX* $line_org] } {
			# the user defined wavelength region
			set L_MAX [lindex $line_org 1]
			set f_lmin 1
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.LMAX = $L_MAX"
                        }
		    }
		    if { [string match *FILT* $line_org] } {
			# the os filter
			set l_filt [lindex $line_org 1]
			set f_filt 1
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.FILT = $l_filt"
                        }
		    }
		    if { [string match *MASK_NA* $line_org] } {
			# the eso allocated proposal id
			set L_prop [lindex $line_org 1]
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.MASK_NA = $L_prop"
                        }
		    }
		    if { [string match *MASK_ID* $line_org] } {
			# the mask id is either 900000 for science (sky) masks
			# or 900001 <= id <= 900100 for technical masks
			# science masks will get their id at paranal
			set L_mid [lindex $line_org 1]
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.MASK_ID = $L_mid"
                        }
			# We apply standard mask mode a little bit later
			# after reset_mask, and after every slit has been created.
			# Look for 'L_mid'
		    }
		}
	    }
	}
	close $fileId
    } else {
        if { $debug && $verbose } {
            puts stdout "load_hitms_TPX: Could not open TPX file >$myfile<"
        }
    }

    # Some tests for backfward compatibility
    if { $lty == "" } {
        puts stdout "\n->->-> Warning TEL.TARG.TYPE not defined"
    }
    if { !$equ_flag } {
        puts stdout "\n->->-> Warning TEL.TARG.EQUINOX not found"
    }
    if { $epo_flag }  {
        puts stdout "\n->->-> Warning TEL.TARG.EQUINOX defined via TEL.TARG.EPOCH = $EQX"
    }

    # Set local position angle (mask rotation)
    # rotator angle = - position angle of mask
    set lPA [expr {$lPA * (-1.0)}]
    set lPA [expr {fmod($lPA,360)}]

    # Configure instrument due to SEQ.PREP keywords if necessary
    if { $f_cam } {
        # Change only if new cam != old cam
        if { [string compare [$this cget -camera] $l_cam] != 0 } {
            $this configure -camera $l_cam
        }
    }

    if { $f_lmin } {
        # The f_lmin flags changes in L_MIN and/or L_MAX
        wave_limit
    }

    if { $f_gris } {
	# If the grism defined in the .fims file is different from the current one
	# Change only if new grism != old grism
	if { [string compare [$this cget -grism] $l_gris] != 0 } {
            $this configure -grism $l_gris
        }
    }

    if { $f_filt } {
	# Change only if new filt != old filt
	if { [string compare [$this cget -filter] $l_filt] } {
            $this configure -filter $l_filt
        }
    }

    if { $f_coll } {
	# Change only if new coll != old coll
	if { [string compare [$this cget -collimat] $l_coll] } {
            $this configure -collimat $l_coll
	}
    }

    set ff [list $CCDMASTER_XSRMIN $CCDMASTER_XSRMAX $CCDMASTER_YSRMIN $CCDSLAVE_YSRMAX]
    if { [string match *HR* [$this cget -collimat]] } {
        set ff [list $CCDMASTER_XHRMIN $CCDMASTER_XHRMAX $CCDMASTER_YHRMIN $CCDSLAVE_YHRMAX]
    }

    if { $auto_flag == 1 } {
        if { $debug && $verbose } {
            puts stdout "load_hitms_TPX: AutoPos, mask not reset..."
        }
    } else {
        if { $debug && $verbose } {
            puts stdout "load_hitms_TPX: Resetting Mask..."
        }
        reset_mask

    }

    fors::MXU_SLIT::set_MASK_NAID [list $L_prop $L_mid]
    if { $debug && $verbose } {
        puts stdout "load_hitms_TPX: Mask ID [fors::MXU_SLIT::get_MASK_NAID]"
    }

    if { [catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas mx my screen } result] } {
        if { $debug && $verbose } {
            puts stdout "load_hitms_TPX: Mask center (mos(0,x0),mos(0,y0))"
            puts stdout "                Error converting ($mos(0,x0),$mos(0,y0)) canvas to (mx,my) screen coord."
            puts stdout "                $result"
        }
    } else {
        if { $debug && $verbose } {
            puts stdout "load_hitms_TPX: Mark mask at ($mos(0,x0), $mos(0,y0)) \[canvas\] = ($mx, $my) \[screen\]"
        }
        mark $m_canvas $mx $my
    }
    set a1 [expr {15.0 * [DDMMSS2deg $lwx]}]
    set d1 [DDMMSS2deg $lwy]
    if { $debug && $verbose } {
        puts stdout [format "load_hitms_TPX: deg $EQX = %10.3f %10.3f" $a1 $d1]
    }
    # wcs to canvas via bonsai matrix
    set dum [C_wc w2c $a1 $d1]
    set cx [lindex $dum 0]
    set cy [lindex $dum 1]
    if { $debug && $verbose } {
        puts stdout [format "                canvas = %10.3f %10.3f" $cx $cy]
    }

    if { $auto_flag == 0 } {
        # Shift mask
        if { [catch { $m_image convert coords $cx $cy canvas nwx nwy screen } result] } {
            if { $debug && $verbose } {
                puts stdout "load_hitms_TPX: Mask center (cx,cy)"
                puts stdout "                Error converting ($cx,$cy) canvas to (nwx,nwy) screen coord."
                puts stdout "                $result"
            }
        } else {
            if { $debug && $verbose } {
                puts stdout "load_hitms_TPX: Shift mask to ($cx, $cy) \[canvas\] = ($nwx, $nwy) \[screen\]"
            }
            $this shift $m_canvas $nwx $nwy
        }

        # Rotate mask
        # In reset_mask->init_mask_world2, abs_ang_rad is set to 0
        set rota [$this cget -rotangle]
        $this configure -rotangle [expr {$lPA * $CSO}]
        if { $debug && $verbose } {
            puts stdout "load_hitms_TPX: Rotate mask to $lPA"
        }
        # In rotate abs_ang_rad is set in rad back again
        $this rotate $m_canvas $mos(0,x0) $mos(0,y0) 1

        # Restore default value
        $this configure -rotangle $rota
    }

    # ------- Read INS keywords -------
    # Step 2: Load the same file again,
    #         and now create mxu slits.
    # ---------------------------------
    set lshap ""
    set lwd 0.0
    set lle 0.0
    set lro 0.0
    set lof 0.0

    set loc_x1 ""
    set loc_x2 ""
    set loc_x3 ""
    set loc_y1 ""
    set loc_y2 ""
    set loc_y3 ""
    set loc_REF 0

    set lmod [$this cget -modus]
    set LMOD [string toupper $lmod]
    if { $debug && $verbose } {
        puts stdout "load_hitms_TPX: create $LMOD slits"
    }
    # Reset targets and reference targets
    # check this:
    for { set i 1 } { $i <= 40 } { incr i 1 } {
        set targw($i,f) 0
    }

    if { [file exists $myfile] & [file readable $myfile] } {
        if { $debug && $verbose } {
            puts stdout "load_hitms_TPX: Opening TPX file >$myfile<"
        }
	set fileId [open $myfile r]
        if { $debug && $verbose } {
	    puts stdout "load_hitms_TPX: File opened with fileId >$fileId<"
        }
	puts $lfId "load_hitms_TPX: File opened with fileId >$fileId<"
        # As long as there are lines in the file...
        while { [gets $fileId line_org] >= 0 } {
            if { $debug && $verbose } {
                # List the file content
                puts stdout $line_org
            }
            if { [string match *$ins_kwd* $line_org] == 1 } {
                # INS
                switch -glob -- $line_org {
                    *NUMBER* {
                        set s_c [lindex $line_org 1]
                        if { $debug && $verbose } {
                            puts stdout "$s_c $LMOD slits in file"
                        }
                    }
                    *NAME* {
                         set lna [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_name $lna
                         if { $debug && $verbose } {
                             puts stdout "------N-$key-$loc_name-"
                         }
                    }
                    *REF* {
                        set lref [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        if { $lref } {
                            set loc_REF 1
                        }
                        if { $debug && $verbose } {
                            puts stdout "------R-$key-$loc_REF-"
                        }
                    }
                    *SHAPE* {
                        set lshap [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_SHAP $lshap
                        if { $debug && $verbose } {
                            puts stdout "------S-$key-$loc_SHAP-"
                        }
                    }
                    *WID* {
                        set lwd [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_W $lwd
                        if { $debug && $verbose } {
                            puts stdout "------W-$key-$loc_W-"
                        }
                    }
                    *LEN* {
                         set lle [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_LEN $lle
                         if { $debug && $verbose } {
                             puts stdout "------L-$key-$loc_LEN-"
                         }
                    }
                    *ROT* {
                        set lro [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_RO $lro
                        if { $debug && $verbose } {
                            puts stdout "------R-$key-$loc_RO-"
                        }
                    }
                    *OFF* {
                        set lof [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_OF $lof
                        if { $debug && $verbose } {
                            puts stdout "------O-$key-$loc_OF-"
                        }
                    }
                    *ALPHA* {
                        set lwx [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_RA [expr {15.0 * [DDMMSS2deg $lwx]}]
                        if { $debug && $verbose } {
                            puts stdout "------A-$key-$loc_RA-"
                        }
                        # Science slit numbers start with 101,
                        # reference targets have no names
                        if { $key < 100 } {
                           # reference target
                           set targw($key,f) 1
                           set targw($key,a) $loc_RA
                        }
                    }
                    *DELTA* {
                        set lwy [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_DEC [DDMMSS2deg $lwy]
                        if { $debug && $verbose } {
                            puts stdout "------D-$key-$loc_DEC-"
                        }
                        # Reference targets have no names
                        if { $key < 100 } {
                            # Reference target
                            set targw($key,f) 1
                            set targw($key,d) $loc_DEC
                        } else {
                            # We assume that 'DELTA' is the last keyword for the
                            # slit read from the .fims file
                            # We can therefore create the slit here and now
                            # the next line read from the '.fims' will contain
                            # information for the next (=another) HIT-MS slit
                            if { [catch { $m_image convert coords $loc_RA $loc_DEC "deg $EQX" x y canvas } result] } {
                                if { $debug && $verbose } {
                                    puts stdout "load_hitms_TPX: Slit $loc_name"
                                    puts stdout "                Error converting ($loc_RA,$loc_DEC) \[deg\] to (x,y) canvas coord."
                                    puts stdout "                $result"
                                }
                            }
                            if { [catch { $m_image convert coords $loc_RA $loc_DEC "deg $EQX" px py image } result] } {
                                if { $debug && $verbose } {
                                    puts stdout "load_hitms_TPX: Slit $loc_name"
                                    puts stdout "                Error converting ($loc_RA,$loc_DEC) \[deg\] to (px,py) image coord."
                                    puts stdout "                $result"
                                }
                            }
                            #
                            # Checking that slit is well positioned (inside the focal field)
                            set inside_mask_flag 0; # default outside the focal field
                            set a [$m_canvas find overlapping \
	                          [expr {$x - 1}] [expr {$y - 1}] \
	                          [expr {$x + 1}] [expr {$y + 1}]]
                            foreach c $a {
	                        set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
	                        if { [string compare [lindex $local_tags 0] A] == 0 } {
		                    set inside_mask_flag 1
                                }
                            }
	                    if { $inside_mask_flag } {
                                if { $debug && $verbose } {
	                            puts stdout "                Slit $loc_name is inside"
                                }
	                    } else {
	                        # info_dialog "Slit $loc_name is outside!!! \n This will skipped !!"
                                if { $debug && $verbose } {
	                            puts stdout "                Slit $loc_name is outside!!!"
	                            puts stdout "                This one will skipped!!"
                                }
	                        continue
 	                    }
                            # Configure the slit length before, so that the
                            # hmslit_max check doesn't fail.
                            eval configure { -slit_length $lle }
                            # The number of hitms slits can be 1 or 2
                            # depending on the slit length.
                            set sl [$this cget -slit_length]
                            switch $sl {
                                5.0 {
                                    set hmslit_max 2
                                }
                                10.0 {
                                    set hmslit_max 1
                                }
                                default {
                                    set hmslit_max 1
                                }
                            }
                            # For HIT-MS, the reference slit is the same as
                            # the target slit
                            mxu_new_slit 0 $x $y $px $py $lwd $lle $lof $lro $lshap $loc_name $ff $hmslit_max $loc_REF 0
                            set noms [fors::MXU_SLIT::nos]
                            if { [$this noas] >= $hmslit_max } {
                                $m_mb.layout.sl entryconfigure $m_i(Layout_SlitLength_10_0) -state disabled
                            }
                            if { [string match *CURVED* $lshap] } {
                                C_mxus$noms move_curve [list $loc_x1 $loc_y1 $loc_x2 $loc_y2 $loc_x3 $loc_y3]
                                # Correct the loaded bezier points for optical distortion
                                update_bez_oc $noms
                            }
                            # For the next slit
                            set loc_REF 0
                        }
                    }
                    *X1* {
                        set x1 [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_x1 $x1
                        if { $debug && $verbose } {
                            puts stdout "------X-$key-$x1-"
                        }
                    }
                    *X2* {
                         set x2 [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_x2 $x2
                         if { $debug && $verbose } {
                             puts stdout "------X-$key-$x2-"
                         }
                    }
                    *X3* {
                         set x3 [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_x3 $x3
                         if { $debug && $verbose } {
                             puts stdout "------X-$key-$x3-"
                         }
                    }
                    *Y1* {
                         set y1 [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_y1 $y1
                         if { $debug && $verbose } {
                             puts stdout "------Y-$key-$y1-"
                         }
                    }
                    *Y2* {
                         set y2 [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_y2 $y2
                         if { $debug && $verbose } {
                             puts stdout "------Y-$key-$y2"
                         }
                    }
                    *Y3* {
                         set y3 [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_y3 $y3
                         if { $debug && $verbose } {
                             puts stdout "------Y-$key-$y3-"
                         }
                    }
                    default {
                         puts stdout "------X->$line_org"
                    }
	        }
            }
        }
        close $fileId

        set s_c 0
        set result {ok}
        # for { set i 1 } { $i < 20 } { incr i 1 } {
        #     puts stdout [format "%2d %10s %10.3f %10.3f" \
        #                          $i $targw($i,f) $targw($i,a) $targw($i,d)]
        # }

        # --------- reference targets
        C_ref_targ reset
        for { set i 31 } { $i <= 40 } { incr i 1 } {
            if { $targw($i,f) } {
                # puts stdout "reference target : $i  $targw($i,a) $targw($i,d)"
                if { [catch { $m_image convert coords $targw($i,a) $targw($i,d) "deg $EQX" x y "wcs $EQX" } result] } {
                    if { $debug && $verbose } {
                        puts stdout "load_hitms_TPX: Ref slit $i"
                        puts stdout "                Error converting ($targw($i,a),$targw($i,d)) \[deg\] to (x,y) WCS coord."
                        puts stdout "                $result"
                    }
                }
                catch {
                    C_ref_targ put $x $y
                    C_ref_targ plot $mos(0,r)
                }
            }
        }

        if { [C_ref_targ nort] } {
            puts stdout "[C_ref_targ nort] Reference Targets found"
        } else {
            puts stdout "\n->->-> Warning No Reference Targets found"
        }

        # C_ref_targ show
        # --------- reference targets end

        # Change to standard mask modus
        # You loaded a science setup base on a technical mask
        if { $L_mid > 900000 && $L_mid <= 900100 } {
            fors::MXU_SLIT::set_mmff 1
            puts stdout "tech_mask = >[fors::MXU_SLIT::get_mmff]<"
        }

        # targw locally as deg, but when move_slit is called it is
        # in wcs again
        # for { set i 1 } { $i < 20 } { incr i 1 } {
        #     if { $targw($i,f) != 0 } {
        #         # move_slit requires screen coords
        #         $m_image convert coords $targw($i,a) $targw($i,d) \
        #                                 "deg $EQX" mx my screen
        #         puts stdout [format "%2d-->%s (%11.7f,%11.7f)(deg $EQX) \
        #                              (%8.3f,%8.3f)(screen) " $i \
        #                              $targw($i,f) $targw($i,a) \
        #                              $targw($i,d) $mx $my \
        #                     ]
        #         set targw($i,f) 0
        #         # select,  not manually
        #         if { [$this move_slit $m_canvas $mx $my 1 0] != "" } {
        #             incr s_c
        #             # puts stdout $s_c
        #         }
        #     }
        # }

        puts stdout "+------------------------------+"
        puts stdout [format "%2d $LMOD slits positioned" [fors::MXU_SLIT::nos]]
        puts stdout "+------------------------------+"
        update_and_replot
    }
    set last [string length $myfile]
    set first [expr {[string last "/" $myfile] + 1}]
    set last_load_fn [string range $myfile $first [expr {$last - 6}]]

    $this configure -file_name [lindex [split $last_load_fn "."] 2]

    catch { SP update }
    tracer LEAVING
}

#=========================================
private method load_img_TPX { filename } {
#=========================================
# Open setup file, move and rotate mask and
# position slity blades according to the setup
# reading a img1.<ec>.<pn>.fims file
# IMG : MOS slit blades in mm focal plane
# LSS :
# HITMS:
# HITS:
# More information in load_mos_TPX
# puts stdout "load_img_TPX >$filename<"

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    C_wc coeff_wc $wx $wy

    set myfile $set_dir

    if { $debug && $verbose } {
        puts stdout "load_img_TPX: Loading TPX file >$filename< from $set_dir"
    }
        
    if { $filename == "" } {
        append myfile "/A"
        append myfile [$this cget -modus]
        append myfile $exposure_counter
        append myfile ".targ"
    } else {
        if { $filename } {
            append myfile "/$filename"
        } else {
            global .fs
            catch { destroy .fs }
            set myfile [filename_dialog $set_dir "[$this cget -modus]${insId}.*.fims"]
        }
    }

    set keyword TEL
    set keyw2   INS
    set f_cam 0
    set f_coll 0
    set f_gris 0
    set f_filt 0
    set f_slit_width 0
    set f_lmin 0
    set lty ""
    set lwx 000000.00
    set lwy 000000.00
    set epo_flag 0
    set equ_flag 0

    # -----------------------------------------------------
    # Step 1 : Load opt1.targ, read only TEL pointing.
    #          Shift and rotate according to these values.
    # -----------------------------------------------------
    if { [file exists $myfile] & [file readable $myfile] } {
        if { $debug && $verbose } {
            puts stdout "load_img_TPX: Opening TPX file >$myfile<"
        }
        set fileId [open $myfile r]
        if { $debug && $verbose } {
	    puts stdout "load_img_TPX: File opened with fileId >$fileId<"
        }
	puts $lfId "load_img_TPX: File opened with fileId >$fileId<"

        # As long as there are lines in the file...
        while { [gets $fileId line_org] >= 0 } {
            if { [string match #* $line_org] } {
                puts stdout $line_org
            } else {
                if { [string match *$keyword* $line_org] == 1 } {
                    # TEL
                    switch -glob -- $line_org {
                        *ALPHA* {
                            set lwx [lindex $line_org 1]
                        }
                        *DELTA* {
                            set lwy [lindex $line_org 1]
                        }
                        *ROT.OFF* {
                            set lPA [lindex $line_org 1]
                        }
                        *EPOCH* {
                            set EQX [lindex $line_org 1]
                            set epo_flag 1
                        }
                        *EQUINOX* {
                            set EQX [lindex $line_org 1]
                            set equ_flag 1
                        }
                        *TYPE* {
                            set lty [lindex $line_org 1]
                        }
                    }
                }
                if { [string match *$keyw2* $line_org] == 1 } {
                    # puts stdout $line_org
                    # INS (SLIT or PINH)
                    switch -glob -- $line_org {
                        *PINH* {
                            # in case it is  lss or hits
                            # configure slit width before reset_mask
                            set key "INS.SLIT.NAID       lSlit0_3arcsec"
                            set key [split $key " "]
                            set ll [expr {[llength $key] - 1}]
                            # puts stdout "[$this cget -modus] [lindex $key 0] [lindex $key $ll]"
                            # config_mask "[$this cget -modus] [lindex $key 0] [lindex $key $ll]"
                            # $m_mb.layout.ht invoke *0.3*
                            $this configure -long_width 0.3
                        }
                        *SLIT* {
                            set key $line_org
                            set key [split $key " "]
                            set ll [expr {[llength $key] - 1}]
                            set l_slit [lindex $key $ll]
                            # puts stdout ">$l_slit<"
                            switch -exact -- $l_slit {
                                lSlit0_3arcsec {
                                    set sl 0.3
                                }
                                htSlit0_4arcsec -
                                lSlit0_4arcsec {
                                    set sl 0.4
                                }
                                htSlit0_5arcsec -
                                lSlit0_5arcsec {
                                    set sl 0.5
                                }
                                htSlit0_7arcsec -
                                lSlit0_7arcsec {
                                    set sl 0.7
                                }
                                htSlit1_0arcsec -
                                lSlit1_0arcsec {
                                    set sl 1.0
                                }
                                htSlit1_3arcsec -
                                lSlit1_3arcsec {
                                    set sl 1.3
                                }
                                htSlit1_6arcsec -
                                lSlit1_6arcsec {
                                    set sl 1.6
                                }
                                htSlit2_0arcsec -
                                lSlit2_0arcsec {
                                    set sl 2.0
                                }
                                lSlit2_5arcsec {
                                    set sl 2.5
                                }
                                default {
                                    set sl 1.0
                                }
                            }
                            # puts stdout ">$l_slit<  >$sl<"
                            # puts stdout ">[$this cget -long_width]<  >$sl<"
                            if { [string compare [$this cget -long_width] $sl] } {
                                if { [string index $l_slit 0] == "h" } {
                                    # puts stdout "$m_mb.layout.ht invoke *$sl*"
                                    # $m_mb.layout.ht invoke *$sl*
                                    $this configure -long_width $sl
                                    # puts stdout "$m_mb.layout.ht invoke *$sl*"
                                } else {
                                    # puts stdout "$m_mb.layout.ls invoke *$sl*"
                                    # $m_mb.layout.ls invoke *$sl*
                                    $this configure -long_width $sl
                                    # puts stdout "$m_mb.layout.ls invoke *$sl*"
                                }
                            }
                        }
                    }
                }
                # pinhole slit end

                # step 2 -------------------------------------
                # --- by the way read SEQ.PREP keywords ---------
                if { [string match *SEQ.PREP* $line_org] == 1 } {
                    if { [string match *CAM* $line_org] == 1 } {
                        # the camera
                        set l_cam [lindex $line_org 1]
                        set f_cam 1
                    }
                    if { [string match *COLL* $line_org] == 1 } {
                        # the collimator
                        set l_coll [lindex $line_org 1]
                        set f_coll 1
                    }
                    if { [string match *GRIS* $line_org] == 1 } {
                        # the grism
                        set l_gris [lindex $line_org 1]
                        set f_gris 1
                    }
                    if { [string match *FILT* $line_org] == 1 } {
                        # the os filter
                        set l_filt [lindex $line_org 1]
                        set f_filt 1
                    }
                    if { [string match *WID* $line_org] == 1 } {
                        # the common slit_width
                        set l_slit_width [lindex $line_org 1]
                        set f_slit_width 1
                    }
                    if { [string match *LMIN* $line_org] == 1 } {
                        # the user defined wavelength region
                        set L_MIN  [lindex $line_org 1]
                        set f_lmin 1
                    }
                    if { [string match *LMAX* $line_org] == 1 } {
                        # the user defined wavelength region
                        set L_MAX [lindex $line_org 1]
                        set f_lmin 1
                    }
                }
            }
        }
        close $fileId
    } else {
        if { $debug && $verbose } {
            puts stdout "load_img_TPX: Could not open TPX file >$myfile<"
        }
    }

    # Some tests for backward compatibility
    if { $lty == "" } {
        puts stdout "\n->->-> Warning TEL.TARG.TYPE not defined"
    }
    if { !$equ_flag } {
        puts stdout "\n->->-> Warning TEL.TARG.EQUINOX not found"
    }
    if { $epo_flag } {
        puts stdout "\n->->-> Warning TEL.TARG.EQUINOX defined via TEL.TARG.EPOCH = $EQX"
    }

    # Set local position angle (mask rotation)
    # rotator angle = - position angle of mask
    set lPA [expr {$lPA * (-1.0)}]
    set lPA [expr {fmod($lPA,360)}]

    # puts stdout [format "load_img_TPX : HHMMSS.xxx %13.5f ->  \
    #                      %10.6f deg"  $lwx [expr {15.0 * [DDMMSS2deg $lwx]}]]
    # puts stdout [format "load_img_TPX : DDMMSS.xxx %13.5f ->  \
    #                      %10.6f deg"  $lwy [DDMMSS2deg  $lwy]]
    # puts stdout [format "load_img_TPX : P.A. (M-W) %13.5f deg" $lPA]

    set mask_initialized 0; # assume mask not yet reset
                            # this is for only one reset_mask when more than one
                            # instrument config has been changed

    # Configure instrument due to SEQ.PREP keywords if nessecary
    if { $f_cam } {
        # Change only if new cam != old cam
        if { [string compare [$this cget -camera] $l_cam] != 0 } {
            $this configure -camera $l_cam
        }
    }

    if { $f_lmin } {
        # The f_lmin flags changes in L_MIN and/or L_MAX
        wave_limit
    }

    if { $f_slit_width } {
        # Change only if new slit != old slit
        if { [string compare [$this cget -slit_width] $l_slit_width] != 0 } {
            $this configure -slit_width $l_slit_width
        }
    }

    if { $f_gris } {
        # Change only if new grism != old grism
        if { [string compare [$this cget -grism] $l_gris] != 0 } {
            $this configure -grism  $l_gris
        }
    }

    if { $f_filt } {
        # Change only if new filt != old filt
        if { [string compare [$this cget -filter] $l_filt] != 0 } {
            $this configure -filter $l_filt
        }
    }

    if { $f_coll } {
        # Change only if new coll != old coll
        if { [string compare [$this cget -collimat] $l_coll] != 0 } {
            $this configure -collimat $l_coll
        }
    }

    if { $mask_initialized == 0 } {
        if { $debug && $verbose } {
            puts stdout "load_img_TPX: Resetting Mask..."
        }
        reset_mask
    }

    if { [catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas mx my screen } result] } {
        if { $debug && $verbose } {
            puts stdout "load_img_TPX: Mask center (mos(0,x0),mos(0,y0))"
            puts stdout "              Error converting ($mos(0,x0),$mos(0,y0)) canvas to (mx,my) screen coord."
            puts stdout "              $result"
        }
    } else {
        if { $debug && $verbose } {
            puts stdout "load_img_TPX: Mark mask at ($mos(0,x0), $mos(0,y0)) \[canvas\] = ($mx, $my) \[screen\]"
        }
        mark $m_canvas $mx $my
    }
    set a1 [expr {15.0 * [DDMMSS2deg $lwx]}]
    set d1 [DDMMSS2deg $lwy]
    if { $debug && $verbose } {
        puts stdout [format "load_img_TPX: deg $EQX = %10.3f %10.3f" $a1 $d1]
    }
    # wcs to canvas via bonsai matrix
    set dum [C_wc w2c $a1 $d1]
    set cx [lindex $dum 0]
    set cy [lindex $dum 1]
    if { $debug && $verbose } {
        puts stdout [format "              canvas = %10.3f %10.3f" $cx $cy]
    }

    # Shift mask
    if { [catch { $m_image convert coords $cx $cy canvas nwx nwy screen } result] } {
        if { $debug && $verbose } {
            puts stdout "load_img_TPX: Mask center (cx,cy)"
            puts stdout "              Error converting ($cx,$cy) canvas to (nwx,nwy) screen coord."
            puts stdout "              $result"
        }
    } else {
        if { $debug && $verbose } {
            puts stdout "load_img_TPX: Shift mask to ($cx, $cy) \[canvas\] = ($nwx, $nwy) \[screen\]"
        }
        shift $m_canvas $nwx $nwy
    }

    # Rotate mask
    # In reset_mask->init_mask_world2, abs_ang_rad is set to 0
    set rota [$this cget -rotangle]
    $this configure -rotangle [expr {$lPA * $CSO}]
    set abs_ang_rad 0
    set PAM 0
    if { $debug && $verbose } {
        puts stdout "load_img_TPX: Rotate mask to $lPA"
    }
    # In rotate abs_ang_rad is set in rad back again
    $this rotate $m_canvas $mos(0,x0) $mos(0,y0) 1

    # Restore default value
    $this configure -rotangle $rota

    # we expect the file cointains either
    # INS.MOS{i}.POS and INS.MOS{i}.WID keywords for img, ipol modes
    # or INS.SLIT.NAID (lss,hits) or INS.PINH.NAID (hits) keywords

    # mask is now at the right place with the right position angle
    # --------------------------------
    # step 3: load the same file again, read INS.MOS{i}.POS and INS.MOS{i}.WID
    #         and adjust mos-slits

    # what is this ?
    for { set i 31 } { $i <= 40 } { incr i 1 } {
        set targw($i,f) 0
    }

    if { [file exists $myfile] & [file readable $myfile] } {
        set fileId [open $myfile r]
        puts $lfId "FORS::load_TARG  $fileId"
        # As long as there are lines in the file...
        while { [gets $fileId line_org] >= 0 } {
            # puts stdout $line_org
            if { [string match *$keyw2* $line_org] == 1 } {
                # INS
                switch -glob -- $line_org {
                    *POS* {
                        set lpo [lindex $line_org 1]
                        # IMG  slit center in mm
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "MOS"]
                        # puts stdout "------P-$key-$lpo mm-"
                        set t(po,$key) $lpo
                    }
                    *WID* {
                        set lwi [lindex $line_org 1]
                        # slit width in mm
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "MOS"]
                        # puts stdout "------W-$key-$lwi mm-"
                        set t(wi,$key) $lwi
                    }
                    *SET* {
                        set lwt [lindex $line_org 1]
                        # this is for the blades on non-default positions
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "MOS"]
                        set targw($key,f) 1
                        # puts stdout "------T-$key-$targw($key,f)-"
                    }
                    *ALPHA* {
                        set lwx [lindex $line_org 1]
                        # this is for the reference targets
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set targw($key,a) [expr {15.0 * [DDMMSS2deg $lwx]}]
                        set targw($key,f) 1
                        # puts stdout "------A-$key-$targw($key,a)-"
                    }
                    *DELTA* {
                        set lwy [lindex $line_org 1]
                        # this is for the reference targets
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set targw($key,d) [DDMMSS2deg $lwy]
                        set targw($key,f) 1
                        # puts stdout "------D-$key-$targw($key,d)-"
                    }
                    default {
                        # puts stdout "------X-"
                    }
                }
            }
        }
        close $fileId

        # --------- reference targets
        C_ref_targ reset

        # normal case
        # IMG, IPOL, IMS, PIMS, HITS, LSS2, LSS1
        for { set i 31 } { $i <= 40 } { incr i 1 } {
            if { $targw($i,f) } {
                if { [catch { $m_image convert coords $targw($i,a) $targw($i,d) "deg $EQX" x y "wcs $EQX" } result] } {
                    if { $debug && $verbose } {
                        puts stdout "load_img_TPX: Ref slit $i"
                        puts stdout "              Error converting ($targw($i,a),$targw($i,d)) \[deg\] to (x,y) WCS coord."
                        puts stdout "              $result"
                    }
                }
                catch {
                    C_ref_targ put $x $y
                    C_ref_targ plot $mos(0,r); # delete all and plot all
                }
            }
        }

        if { [C_ref_targ nort] } {
            puts stdout "[C_ref_targ nort] Reference Targets found"
        } else {
            puts stdout "\n->->-> Warning No Reference Targets found"
        }
        # --------- reference targets end

        # puts stdout "reftargs end"

        # lets calculate the new MOS slit blade pos.
        # (POS-WID/2)/(XSRMAX-XSRMIN)=(f-e)/(h-e)=(b-a)/(d-a)
        # and so on
        set fux [expr {$XBLADEB - $XBLADEA}]

        if { [string compare img [$this cget -modus]] == 0 } {
            if { $mos(10,xh) == $mos(10,xe) } {
                # north to right or left
                for { set i 1 } { $i < 20 } { incr i 1 } {
                    set mos($i,yf) [expr {($t(po,$i) - $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          * ($mos($i,yh) - $mos($i,ye)) + $mos($i,ye)}]
                    set mos($i,yb) [expr {($t(po,$i)-$t(wi,$i)/2.0-$XBLADEA)/$fux \
                                          * ($mos($i,yd) - $mos($i,ya)) + $mos($i,ya)}]
                    set mos($i,yg) [expr {($t(po,$i) + $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          * ($mos($i,yh) - $mos($i,ye)) + $mos($i,ye)}]
                    set mos($i,yc) [expr {($t(po,$i) + $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          * ($mos($i,yd) - $mos($i,ya)) + $mos($i,ya)}]
                }
            } elseif { $mos(10,yh) == $mos(10,ye) } {
                for { set i 1 } { $i < 20 } { incr i 1 } {
                    set mos($i,xf) [expr {($t(po,$i) - $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          * ($mos($i,xh) - $mos($i,xe)) + $mos($i,xe)}]
                    set mos($i,xb) [expr {($t(po,$i) - $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          * ($mos($i,xd) - $mos($i,xa)) + $mos($i,xa)}]
                    set mos($i,xg) [expr {($t(po,$i) + $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          * ($mos($i,xh) - $mos($i,xe)) + $mos($i,xe)}]
                    set mos($i,xc) [expr {($t(po,$i) + $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          * ($mos($i,xd) - $mos($i,xa)) + $mos($i,xa)}]
                }
            } else {
                for { set i 1 } { $i < 20 } { incr i 1 } {
                    set mos($i,xf) [expr {($t(po,$i) - $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          *($mos($i,xh) - $mos($i,xe)) + $mos($i,xe)}]
                    set mos($i,yf) [expr {($t(po,$i) - $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          *($mos($i,yh) - $mos($i,ye)) + $mos($i,ye)}]
                    set mos($i,xb) [expr {($t(po,$i) - $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          *($mos($i,xd) - $mos($i,xa)) + $mos($i,xa)}]
                    set mos($i,yb) [expr {($t(po,$i) - $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          *($mos($i,yd) - $mos($i,ya)) + $mos($i,ya)}]
                    set mos($i,xg) [expr {($t(po,$i) + $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          *($mos($i,xh) - $mos($i,xe)) + $mos($i,xe)}]
                    set mos($i,yg) [expr {($t(po,$i) + $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          *($mos($i,yh) - $mos($i,ye)) + $mos($i,ye)}]
                    set mos($i,xc) [expr {($t(po,$i) + $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          *($mos($i,xd) - $mos($i,xa)) + $mos($i,xa)}]
                    set mos($i,yc) [expr {($t(po,$i) + $t(wi,$i) / 2.0 - $XBLADEA) / $fux \
                                          *($mos($i,yd) - $mos($i,ya)) + $mos($i,ya)}]
                }
            }
        }
        update_and_replot
    }
    set last [string length $myfile]
    set first [expr {[string last "/" $myfile] + 1}]
    set last_load_fn [string range $myfile $first [expr {$last - 6}]]
    $this configure -file_name [lindex [split $last_load_fn "."] 2]
    catch {SP update}
    tracer LEAVING
}

#=========================================
private method load_mos_TPX { filename } {
#=========================================
# Open setupfile (e.g.  'mos2.<ec>.<pn>.fims') and
# read the setup file, configure values and plot the
# mask at the proper position.
# Slit positions are given in wcs
# filename = ""     : use default filename=auto_pos (used by AutoPos)
# filename = 0      : from filename_dialog widget
# filename = string : take this

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Initialize the scales for coord transformation
    #
    C_wc coeff_wc $wx $wy

    # --- Open filename ---------
    #
    if { $filename == "" } {
        # No filename means load result from AutoPos, written to
        # .fims/LOG/(p)mos(1/2).<ec>.autopos.fims
        # We load auto_pos from LOG dir, to be independent from user-defined
        # set_dir
        set myfile $log_dir
        append myfile "/[$this cget -modus]${insId}.$exposure_counter.auto_pos.fims"
        if { $debug && $verbose } {
            puts stdout "load_mos_TPX: No filename, loading AutoPos TPX from $myfile"
        }
        set auto_flag 1
    } else {
        set myfile $set_dir
        if { $debug && $verbose } {
            puts stdout "load_mxu_TPX: Loading TPX file >$filename< from $set_dir"
        }
        if { $filename } {
            append myfile "/$filename"
        } else {
            # filename is zero, call dialog
            global .fs
            catch { destroy .fs }
            set myfile [filename_dialog $set_dir "[$this cget -modus]${insId}.*.fims"]
        }
        set auto_flag 0
    }

    set keyword TEL
    set keyw2   INS
    set f_cam 0
    set f_gris 0
    set f_filt 0
    set f_slit_width 0
    set f_coll 0
    set f_lmin 0
    set lty ""
    set lwx 000000.00
    set lwy 000000.00
    set epo_flag 0
    set equ_flag 0

    # ------- Read TCS keywords -------------------------
    # Step 1: Load opt1.targ, read only TEL pointing.
    #         Shift and rotate according to these values.
    # ---------------------------------------------------
    if { [file exists $myfile] & [file readable $myfile] } {
        if { $debug && $verbose } {
            puts stdout "load_mos_TPX: Opening TPX file >$myfile<"
        }
        set fileId [open $myfile r]
        if { $debug && $verbose } {
	    puts stdout "load_mos_TPX: File opened with fileId >$fileId<"
        }
	puts $lfId "load_mos_TPX: File opened with fileId >$fileId<"
        
        # As long as there are lines in the file...
        while { [gets $fileId line_org] >= 0 } {
            if { [string match #* $line_org] } {
		if { $debug && $verbose } {
                    puts stdout $line_org
                }
            } else {
                if { [string match *$keyword* $line_org] == 1 } {
                    # TEL
                    switch -glob -- $line_org {
                        *ALPHA* {
                            set lwx [lindex $line_org 1]
                        }
                        *DELTA* {
                            set lwy [lindex $line_org 1]
                        }
                        *ROT.OFF* {
                            set lPA [lindex $line_org 1]
                        }
                        *EPOCH* {
                            set EQX [lindex $line_org 1]
                            set epo_flag 1
                        }
                        *EQUINOX* {
                            set EQX [lindex $line_org 1]
                            set equ_flag 1
                        }
                        *TYPE* {
                            set lty [lindex $line_org 1]
                        }
                    }
                }

                # --- by the way read SEQ.PREP keywords ---------
                if { [string match *SEQ.PREP* $line_org] == 1 } {
                    if { [string match *CAM* $line_org] == 1 } {
                        # the camera
                        set l_cam [lindex $line_org 1]
                        set f_cam 1
                    }
                    if { [string match *COLL* $line_org] } {
                        # the collimator
                        set l_coll [lindex $line_org 1]
                        set f_coll 1
                    }
                    if { [string match *GRIS* $line_org] } {
                        # the grism
                        set l_gris [lindex $line_org 1]
                        set f_gris 1
                    }
                    if { [string match *LMIN* $line_org] } {
                        # the user defined wavelength region
                        set L_MIN  [lindex $line_org 1]
                        set f_lmin 1
                    }
                    if { [string match *LMAX* $line_org] } {
                        # the user defined wavelength region
                        set L_MAX [lindex $line_org 1]
                        set f_lmin 1
                    }
                    if { [string match *FILT* $line_org] } {
                        # the os filter
                        set l_filt [lindex $line_org 1]
                        set f_filt 1
                    }
                    if { [string match *MTALI* $line_org] } {
                        # the 'alignment with mos slit 10' option
                        puts stdout "SEQ.PREP.MTALI = [lindex $line_org 1]"
                        C_ref_targ configure -mos10ali [lindex $line_org 1]
                        # for statuspanel
                        set sp_mtali [lindex $line_org 1]
                        catch {SP update}
                    }
                    if { [string match *ECHS9* $line_org] } {
                        # the echs9 flag
                        puts stdout "SEQ.PREP.ECHS9 = [lindex $line_org 1]"
                        set l_echs9 [lindex $line_org 1]
                    }
                }
            }
        }
        close $fileId
    } else {
        if { $debug && $verbose } {
            puts stdout "load_mos_TPX: Could not open TPX file >$myfile<"
        }
    }

    # Some tests for backfward compatibility
    if { $lty == "" } {
        puts stdout "\n->->-> Warning TEL.TARG.TYPE not defined"
    }
    if { !$equ_flag } {
        puts stdout "\n->->-> Warning TEL.TARG.EQUINOX not found"
    }
    if { $epo_flag } {
        puts stdout "\n->->-> Warning TEL.TARG.EQUINOX defined via TEL.TARG.EPOCH = $EQX"
    }

    # Set local position angle (mask rotation)
    # rotator angle = - position angle of mask
    set lPA [expr {$lPA * (-1.0)}]
    set lPA [expr {fmod($lPA,360)}]

    # Configure instrument due to SEQ.PREP keywords if nessecary
    if { $f_cam } {
        # Change only if new cam != old cam
        if { [string compare [$this cget -camera] $l_cam] != 0 } {
            $this configure -camera $l_cam
        }
    }

    if { $f_lmin } {
        # The f_lmin flags changes in L_MIN and/or L_MAX
        wave_limit
    }

    # Individual slit width from .fims used
    # default MOS slit width not overwritten
    # 19/2/99 WH
    if { [string match echs [$this cget -modus]] } {
        # echs_flag = current status, l_echs9 = just loaded
        # puts stdout "fims echs9=$echs_flag, load_echs9=$l_echs9"
        if { $echs_flag != $l_echs9 } {
            flip_echs
            # puts stdout "fims echs9=$echs_flag, load_echs9=$l_echs9"
        }
    }

    if { $f_gris } {
        # Change only if new grism != old grism
        if { [string compare [$this cget -grism] $l_gris] != 0 } {
            $this configure -grism $l_gris
        }
    }

    if { $f_filt } {
        # Change only if new filt != old filt
        if { [string compare [$this cget -filter] $l_filt] } {
            $this configure -filter $l_filt
        }
    }

    if { $f_coll } {
        # Change only if new coll != old coll
        if { [string compare [$this cget -collimat] $l_coll] } {
            $this configure -collimat $l_coll
        }
    }

    if { $auto_flag == 1 } {
        if { $debug && $verbose } {
            puts stdout "load_mos_TPX: AutoPos, mask not reset..."
        }
    } else {
        if { $debug && $verbose } {
            puts stdout "load_mos_TPX: Resetting Mask..."
        }
        reset_mask
    }

    if { [catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas mx my screen } result] } {
        if { $debug && $verbose } {
            puts stdout "load_mos_TPX: Mask center (mos(0,x0),mos(0,y0))"
            puts stdout "              Error converting ($mos(0,x0),$mos(0,y0)) canvas to (mx,my) screen coord."
            puts stdout "              $result"
        }
    } else {
        if { $debug && $verbose } {
            puts stdout "load_mos_TPX: Mark mask at ($mos(0,x0), $mos(0,y0)) \[canvas\] = ($mx, $my) \[screen\]"
        }
        mark $m_canvas $mx $my
    }
    set a1 [expr {15.0 * [DDMMSS2deg $lwx]}]
    set d1 [DDMMSS2deg $lwy]
    if { $debug && $verbose } {
        puts stdout [format "load_mos_TPX: deg $EQX = %10.3f %10.3f" $a1 $d1]
    }
    # wcs to canvas via bonsai matrix
    set dum [C_wc w2c $a1 $d1]
    set cx [lindex $dum 0]
    set cy [lindex $dum 1]
    if { $debug && $verbose } {
        puts stdout [format "              canvas = %10.3f %10.3f" $cx $cy]
    }

    if { $auto_flag == 0 } {
        # Shift mask
        if { [catch { $m_image convert coords $cx $cy canvas nwx nwy screen } result] } {
            if { $debug && $verbose } {
                puts stdout "load_mos_TPX: Mask center (cx,cy)"
                puts stdout "              Error converting ($cx,$cy) canvas to (nwx,nwy) screen coord."
                puts stdout "              $result"
            }
        } else {
            if { $debug && $verbose } {
                puts stdout "load_mos_TPX: Shift mask to ($cx, $cy) \[canvas\] = ($nwx, $nwy) \[screen\]"
            }
            $this shift $m_canvas $nwx $nwy
        }

        # Rotate mask
        # In reset_mask->init_mask_world2, abs_ang_rad is set to 0
        set rota [$this cget -rotangle]
        $this configure -rotangle [expr {$lPA * $CSO}]
        set abs_ang_rad 0
        set PAM 0
        if { $debug && $verbose } {
            puts stdout "load_mos_TPX: Rotate mask to $lPA"
        }
        # In rotate abs_ang_rad is set in rad back again
        $this rotate $m_canvas $mos(0,x0) $mos(0,y0) 1

        # Restore default value
        $this configure -rotangle $rota 
    }

    # ------- Read INS keywords ----------------
    # Step 2: Load the same file again, read INS
    #         and adjust mos-slits
    # ------------------------------------------

    # Reset targets and reference targets
    # puts stdout "load_mos_TPX : Move Slitlets"
    for { set i 1 } { $i <= 40 } { incr i 1 } {
        set targw($i,f) 0
    }

    if { [file exists $myfile] & [file readable $myfile] } {
        set fileId [open $myfile r]
        puts $lfId "FORS::load_TARG  $fileId"
        # As long as there are lines in the file...
        while { [gets $fileId line_org] >= 0 } {
            # puts stdout $line_org
            if { [string match *$keyw2* $line_org] == 1 } {
                # INS
                switch -glob -- $line_org {
                    *ALPHA* {
                        set lwx [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set targw($key,a) [expr {15.0 * [DDMMSS2deg $lwx]}]
                        # reference targets have no names
                        if { $key > 30 } {
                            set targw($key,f) 1
                        }
                        # puts stdout "------A-$key-$targw($key,a)-"
                    }
                    *DELTA* {
                        set lwy [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set targw($key,d) [DDMMSS2deg $lwy]
                        # reference targets have no names
                        if { $key > 30 } {
                            set targw($key,f) 1
                        }
                        # puts stdout "------D-$key-$targw($key,d)-"
                    }
                    *WID* {
                        set lwd [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set mos($key,b) $lwd
                        # for ECHS, slit 9 should have the same
                        # slit width as slit 10
                        set l_key $key
                    }
                    *NAME* {
                        set lna [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set targw($key,f) $lna
                        # puts stdout "------N-$key-$targw($key,f)-"
                    }
                    *MOS* {
                        # this keyword should be read before
                        # INS.TARG, here only verification
                        # buts since it appears after INS.TARG
                        # and we do not want to read the file a third time
                        # we use it only for verification reasons
                        set lta [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "MOS"]
                        if { $targw($key,f) == 0 } {
                            puts stdout "Warning: opt1.targ: INS.MOS >><< INS.TARG"
                        }
                        if { $key != $lta } {
                            puts stdout "Warning: opt1.targ: INS.MOS >> << INS.MOS"
                        }
                        # puts stdout "------M-$key-$lta-"
                    }
                    default {
                        # puts stdout "------X->$line_org<"
                    }
                }
            }
        }
        close $fileId
        set s_c 0
        set result {ok}
        # for { set i 1 } { $i < 20 } { incr i 1 } {
        #     puts stdout [format "%2d %10s %10.3f %10.3f" \
        #                          $i $targw($i,f) $targw($i,a) $targw($i,d)]
        # }

        # --------- reference targets
        C_ref_targ reset
        for { set i 31 } { $i <= 40 } { incr i 1 } {
            if { $targw($i,f) } {
                if { [catch { $m_image convert coords $targw($i,a) $targw($i,d) "deg $EQX" x y "wcs $EQX" } result] } {
                    if { $debug && $verbose } {
                        puts stdout "load_mos_TPX: Ref slit $i"
                        puts stdout "              Error converting ($targw($i,a),$targw($i,d)) \[deg\] to (x,y) WCS coord."
                        puts stdout "              $result"
                    }
                }
                catch {
                    C_ref_targ put $x $y
                    C_ref_targ plot $mos(0,r)
                }
            }
        }

        if { [C_ref_targ nort] } {
            puts stdout "[C_ref_targ nort] Reference Targets found"
        } else {
            puts stdout "\n->->-> Warning No Reference Targets found"
        }

        # C_ref_targ show
        # --------- reference targets end

        if { [string compare [$this cget -modus] echs] == 0 } {
            # for echs we make the slit width common to the last
            # specified slit width (= mos 10 in any case)
            if { [catch { $m_mb.layout.sw invoke *$lwd* } result] } {
                puts stdout "$result \n Slit_Width Error $lsw"
                $this configure -slit_width $lwd
            } else {
                # puts stdout "$m_mb.layout.sw invoke *$lwd*"
            }
        }
        # targw locally as deg, but when move_slit is called it is
        # in wcs again
        for { set i 1 } { $i < 20 } { incr i 1 } {
            if { $targw($i,f) != 0 } {
                # the move_slit requires screen coords
                if { [catch { $m_image convert coords $targw($i,a) $targw($i,d) "deg $EQX" mx my screen } result] } {
                    if { $debug && $verbose } {
                        puts stdout "load_mos_TPX: Blade $i"
                        puts stdout "              Error converting ($targw($i,a),$targw($i,d)) \[deg\] to (mx,my) screen coord."
                        puts stdout "              $result"
                    }
                } else {
                    puts stdout [format "%2d-->%s (%11.7f,%11.7f)(deg $EQX) (%8.3f,%8.3f)(screen) "\
                                        $i $targw($i,f) $targw($i,a) $targw($i,d) $mx $my\
                                ]
                    set targw($i,f) 0
                    # select, not manually
                    set a [$this move_slit $m_canvas $mx $my 1 0]
                    if { $a != "" } {
                        incr s_c
                    }
                }
            }
        }
        puts stdout "+----------------------------+"
        puts stdout [format "%2d MOS slits positioned" $s_c]
        puts stdout "+----------------------------+"
    }

    set last [string length $myfile]
    set first [expr {[string last "/" $myfile] + 1}]
    set last_load_fn [string range $myfile $first [expr {$last - 6}]]

    $this configure -file_name [lindex [split $last_load_fn "."] 2]

    catch {SP update }
    tracer LEAVING
}

#=========================================
private method load_mxu_FAD { filename } {
#=========================================
# reading a mxu2.<MaskId-900000>.standard.fad file
# the input file must be a fors aperture definition file
# like $FIMSROOT/config/mxu2.01.standard.fad
# such files define one of the 100 MXU standard masks
# permanently located on Paranal.
# these masks need not to be manufactured, they are already there
#

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    puts stdout "In load_mxu_FAD"
    set a [split $filename "/"]
    set b [lindex $a [expr {[llength $a] - 1}]]
    set c [split $b "."]
    set d [string trimleft [lindex $c 1] 0]
    set fn_maskid [expr {900000 + $d}]
    puts stdout "orig filename = $b"
    puts stdout "mode          = [lindex $c 0]"
    puts stdout "mask Id index = [lindex $c 1]"
    puts stdout "mask Id       = $fn_maskid"
    puts stdout "orig format   = [lindex $c 2]"
    puts stdout "orig .ext     = [lindex $c 3]\n\n"

    if { [file exists $filename] & [file readable $filename] } {
        set fileId [open $filename r]
        puts $lfId "FORS::load_mxu_FAD  $fileId"
        puts stdout "FORS::load_mxu_FAD  $fileId"
        # set some flags
        set i_flag 0;  # first line must be 'FORS2'
        set lcounter 0; # line counter after 'FORS2'

        # adapt collimator, used to create mxu slit
        set ff [list $XSRMIN $XSRMAX $YSRMIN $YSRMAX]
        if { [string match *HR* [$this cget -collimat]] } {
            set ff [list $XHRMIN $XHRMAX $YHRMIN $YHRMAX]
        }

        reset_mask

        # extent the functionality of the playground
        # center/slit now for the whole mask
        # $m_canvas addtag ali withtag mxu

        # as long as there are lines in the file
        while { [gets $fileId line_org] >= 0 } {
            if { [string match #* $line_org] } {
                # this is an optional comment line
                # do nothing
                puts stdout "comment> $line_org"
            } else {
                # puts stdout ">$i_flag< >$lcounter< >>>$line_org"
                if { $i_flag } {
                    # we count lines after FORS2 string
                    incr lcounter 1
                    switch -exact -- $lcounter {
                        1 {
                            # we expect the checksum
                            # this checksum will be used as the
                            # the program id for standard masks
                            set checksum $line_org
                            puts stdout "checksum=$checksum"
                        }
                        2 {
                            # we expect real mask id
                            set maskid $line_org
                            puts stdout "maskid  =$maskid"
                            if { $maskid != $fn_maskid } {
                                puts stdout "MASK_NAID inconsistency"
                            } else {
                                fors::MXU_SLIT::set_MASK_NAID "$checksum $maskid"
                                puts stdout "maskid =[fors::MXU_SLIT::get_MASK_NAID]"
                            }
                        }
                        3 {
                            # we expect number of STRAIGHT slits
                            set straight $line_org
                            puts stdout "number of STRAIGHT=$straight"
                        }
                        4 {
                            # we expect number of CURVED slits
                            set curved  $line_org
                            puts stdout "number of CURVED  =$curved"
                        }
                        5 {
                            # we expect number of CIRCLE holes
                            set circle $line_org
                            puts stdout "number of CIRCLE  =$circle"
                        }
                        default {
                            # now we get the apertures
                            # we make life easy. Instead of counting
                            # the slit numbers we decide via the number of
                            # parameters in line_org
                            #       3 parameters => CIRCLE
                            #  4 or 5 parameters => STRAIGHT
                            # 5 to 12 parameters => CURVED
                            set np [llength [ldelete_all [split $line_org " "] ""]]
                            puts stdout "linecounter=[expr {$lcounter - 5}] nofpar=$np"

                            switch -exact -- $np {
                                0 {
                                    # empty line
                                    puts stdout "empty line, do nothing"
                                }
                                3 {
                                    # this must be a circle aperture
                                    $this configure -slit_shape CIRCLE
                                    set b [ldelete_all [split $line_org " "] ""]
                                    set lx [lindex $b 0]
                                    set ly [lindex $b 1]
                                    set ld [lindex $b 2]
                                    set lle $ld
                                    set lro 0.0
                                    set lof 0.0
                                    puts stdout "CIRCLE with x,y=($lx $ly) a,b=($ld,$lle) mm"
                                }
                                4 -
                                5 {
                                    # this must be a box aperture
                                    $this configure -slit_shape STRAIGHT
                                    set b [ldelete_all [split $line_org " "] ""]
                                    set lx [lindex $b 0]
                                    set ly [lindex $b 1]
                                    set ld [lindex $b 2]
                                    set lle [lindex $b 3]
                                    set lro [lindex $b 4]
                                    set lof 0.0
                                    puts stdout "STRAIGHT with x,y=($lx,$ly) l,w=($ld,$lle) >$lro< mm"
                                }
                                6 -
                                7 -
                                8 -
                                9 -
                               10 -
                               11 -
                               12 {
                                    # this must be a curved aperture
                                    # $this configure -slit_shape CURVED
                                    puts stdout "CURVED   with x,y=($lx,$ly) l,w=($ld,$lle) >$lro< mm"
                                }
                                default {
                                    puts stdout " wrong format in $filename"
                                }
                            }
                            if { $np > 2 } {
                                # If it was not an empty line
                                # If the number of parameters was at least 3
                                puts stdout "$lx $ly $EQX $ld $lle $lro $ff [$this cget -slit_shape]"
                                if { [fors::MXU_SLIT::nos] == $mnomxus } {
                                    info_dialog "Sorry, cannot handle more than\n $mnomxus MXU slits" .skycat1
                                    tracer LEAVING 1
                                    return
                                }
                                mxu_new_slit 0 $lx $ly $lx $ly $ld $lle $lof $lro [$this cget -slit_shape] aperture $ff $mnomxus 0 1
                            }
                        }
                    }

                } else {
                    if { [string match *FORS2* $line_org] } {
                        set i_flag 1
                        puts stdout "we found the FORS2 string"
                    }
                }
            }
        }
    }
    close $fileId

    update_and_replot

    append AoutoposMethod "FAD_"

    set last [string length $filename]
    set first [expr {[string last "/" $filename] + 1}]
    set last_load_fn [string range $filename $first [expr {$last - 6}]]

    $this configure -file_name [lindex [split $last_load_fn "."] 2]

    catch {SP update }
    tracer LEAVING 2
}

#=========================================
private method load_mxu_TPX { filename } {
#=========================================
# Reading a mxu2.<ec><pn>.fims file
# The input file must be a setup file like 'mxu2.12.project.fims'
# Slits are given in wcs

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # For the coordinate transformation matrix
    C_wc coeff_wc $wx $wy

    # --- Open filename ---------
    if { $filename == "" } {
        # No filename means load result from AutoPos, written to
        # .fims/LOG/<modus><1|2>.<ec>.autopos.fims
        set myfile $log_dir
        append myfile "/[$this cget -modus]${insId}.$exposure_counter.auto_pos.fims"
        if { $debug && $verbose } {
            puts stdout "load_mxu_TPX: No filename, loading AutoPos TPX from $myfile"
        }
        set auto_flag 1
    } else {
        set myfile $set_dir
        if { $debug && $verbose } {
            puts stdout "load_mxu_TPX: Loading TPX file >$filename< from $set_dir"
        }
        if { $filename != 0 } {
            append myfile "/$filename"
        } else {
            # filename is zero, call dialog
            global .fs
            catch { destroy .fs  }
            set myfile [filename_dialog $set_dir "[$this cget -modus]${insId}.*.fims"]
        }
        set auto_flag 0
    }

    # If file is a fors2 standard-mask aperture definition file (FAD)
    # b must be 'fad'
    set a [lindex [split $myfile "/"] [expr {[llength [split $myfile "/"]] - 1}]]
    set b [lindex [split $a "."] [expr {[llength [split $a "."]] - 1}]]
    if { [string compare $b fad] == 0 } {
        load_mxu_FAD $myfile
        tracer LEAVING FAD
        return
    }

    set tel_kwd TEL
    set ins_kwd INS
    set seq_kwd SEQ.PREP
    set f_cam 0
    set f_gris 0
    set f_filt 0
    set f_slit_width 0
    set f_coll 0
    set f_lmin 0
    set L_prop M_
    set L_mid 900000
    set lPA 0
    set lty ""
    set lwx 000000.00
    set lwy 000000.00
    set epo_flag 0
    set equ_flag 0

    # ------ Read TCS keywords ---------------------------
    # Step 1 : Load opt1.targ, read only TEL pointing.
    #          Shift and rotate according to these values.
    # ----------------------------------------------------
    if { [file exists $myfile] & [file readable $myfile] } {
        if { $debug && $verbose } {
            puts stdout "load_mxu_TPX: Opening TPX file >$myfile<"
        }
	set fileId [open $myfile r]
        if { $debug && $verbose } {
	    puts stdout "load_mxu_TPX: File opened with fileId >$fileId<"
        }
	puts $lfId "load_mxu_TPX: File opened with fileId >$fileId<"
        
	# As long as there are lines in the file...
	while { [gets $fileId line_org] >= 0 } {
	    if { [string match #* $line_org] } {
		if { $debug && $verbose } {
                    puts stdout $line_org
                }
	    } else {
                if { [string match *$tel_kwd* $line_org] == 1 } {
		    # TEL
		    switch -glob -- $line_org {
			*ALPHA* {
                            set lwx [lindex $line_org 1]
                            if { $debug && $verbose } {
                                 puts stdout "TEL.TARG.ALPHA = $lwx"
                            }
                        }
			*DELTA* {
                            set lwy [lindex $line_org 1]
                            if { $debug && $verbose } {
                                 puts stdout "TEL.TARG.DELTA = $lwy"
                            }
                        }
			*ROT.OFF* {
                            set lPA [lindex $line_org 1]
                            if { $debug && $verbose } {
                                 puts stdout "TEL.ROT.OFFANGLE = $lPA"
                            }
                        }
			*EPOCH* {
                            set EQX [lindex $line_org 1]
                            set epo_flag 1
                            if { $debug && $verbose } {
                                 puts stdout "TEL.TARG.EPOCH = $EQX"
                            }
                        }
			*EQUINOX* {
                            set EQX [lindex $line_org 1]
                            set equ_flag 1
                            if { $debug && $verbose } {
                                 puts stdout "TEL.TARG.EQUINOX = $EQX"
                            }
                        }
			*TYPE* {
                            set lty [lindex $line_org 1]
                            if { $debug && $verbose } {
                                 puts stdout "TEL.TARG.TYPE = $lty"
                            }
                        }
		    }
		}

		# --- by the way, read SEQ.PREP keywords ---
		if { [string match *$seq_kwd* $line_org] == 1 } {
                    if { [string match *CAM* $line_org] == 1 } {
                        # the camera
                        set l_cam [lindex $line_org 1]
                        set f_cam 1
                        if { $debug && $verbose } {
                             puts stdout "SEQ.PREP.CAM = $l_cam"
                        }
                    }
		    if { [string match *COLL* $line_org] } {
			# the collimator
			set l_coll [lindex $line_org 1]
			set f_coll 1
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.COLL = $l_coll"
                        }
		    }
		    if { [string match *GRIS* $line_org] } {
			# the grism
			set l_gris [lindex $line_org 1]
			set f_gris 1
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.GRISM = $l_gris"
                        }
		    }
		    if { [string match *LMIN* $line_org] } {
			# the user defined wavelength region
			set L_MIN  [lindex $line_org 1]
			set f_lmin 1
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.LMIN = $L_MIN"
                        }
		    }
		    if { [string match *LMAX* $line_org] } {
			# the user defined wavelength region
			set L_MAX [lindex $line_org 1]
			set f_lmin 1
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.LMAX = $L_MAX"
                        }
		    }
		    if { [string match *FILT* $line_org] } {
			# the os filter
			set l_filt [lindex $line_org 1]
			set f_filt 1
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.FILT = $l_filt"
                        }
		    }
		    if { [string match *MASK_NA* $line_org] } {
			# the eso allocated proposal id
			set L_prop [lindex $line_org 1]
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.MASK_NA = $L_prop"
                        }
		    }
		    if { [string match *MASK_ID* $line_org] } {
			# the mask id is either 900000 for science (sky) masks
			# or 900001 <= id <= 900100 for technical masks
			# science masks will get their id at paranal
			set L_mid [lindex $line_org 1]
			if { $debug && $verbose } {
                            puts stdout "SEQ.PREP.MASK_ID = $L_mid"
                        }
			# We apply standard mask mode a little bit later
			# after reset_mask, and after every slit has been created.
			# Look for 'L_mid'
		    }
		}

	    }
	}
	close $fileId
    } else {
        if { $debug && $verbose } {
            puts stdout "load_mxu_TPX: Could not open TPX file >$myfile<"
        }
    }

    # Some tests for backfward compatibility
    if { $lty == "" } {
        puts stdout "\n->->-> Warning TEL.TARG.TYPE not defined"
    }
    if { !$equ_flag } {
        puts stdout "\n->->-> Warning TEL.TARG.EQUINOX not found"
    }
    if { $epo_flag }  {
        puts stdout "\n->->-> Warning TEL.TARG.EQUINOX defined via TEL.TARG.EPOCH = $EQX"
    }

    # Set local position angle (mask rotation)
    # rotator angle = - position angle of mask
    set lPA [expr {$lPA * (-1)}]
    set lPA [expr {fmod($lPA,360)}]

    # 5.11.98 I see this becomes very circumstancy
    #  --the SkyCat_plugin routines must become part of class FORS
    #  --standard highres must become COLL_SR+1 ...
    #  --read COLL_SR+1 GRIS_... from ISF
    #  --

    # Configure instrument due to SEQ.PREP keywords if necessary
    if { $f_cam } {
        # Change only if new cam != old cam
        if { [string compare [$this cget -camera] $l_cam] != 0 } {
            $this configure -camera $l_cam
        }
    }

    if { $f_lmin } {
        # The f_lmin flags changes in L_MIN and/or L_MAX
        wave_limit
    }

    if { $f_gris } {
	# If the grism defined in the .fims file is different from the current one
	# Change only if new grism != old grism
	if { [string compare [$this cget -grism] $l_gris] != 0 } {
            $this configure -grism $l_gris
        }
    }

    if { $f_filt } {
	# Change only if new filt != old filt
	if { [string compare [$this cget -filter] $l_filt] } {
            $this configure -filter $l_filt
        }
    }

    if { $f_coll } {
	# Change only if new coll != old coll
	if { [string compare [$this cget -collimat] $l_coll] } {
            $this configure -collimat $l_coll
	}
    }

    set ff [list $CCDMASTER_XSRMIN $CCDMASTER_XSRMAX $CCDMASTER_YSRMIN $CCDSLAVE_YSRMAX]
    if { [string match *HR* [$this cget -collimat]] } {
        set ff [list $CCDMASTER_XHRMIN $CCDMASTER_XHRMAX $CCDMASTER_YHRMIN $CCDSLAVE_YHRMAX]
    }

    if { $auto_flag == 1 } {
        if { $debug && $verbose } {
            puts stdout "load_mxu_TPX: AutoPos, mask not reset..."
        }
    } else {
        if { $debug && $verbose } {
            puts stdout "load_mxu_TPX: Resetting Mask..."
        }
        reset_mask
    }

    fors::MXU_SLIT::set_MASK_NAID [list $L_prop $L_mid]
    if { $debug && $verbose } {
        puts stdout "load_mxu_TPX: Mask ID [fors::MXU_SLIT::get_MASK_NAID]"
    }

    if { [catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas mx my screen } result] } {
        if { $debug && $verbose } {
            puts stdout "load_mxu_TPX: Mask center (mos(0,x0),mos(0,y0))"
            puts stdout "              Error converting ($mos(0,x0),$mos(0,y0)) canvas to (mx,my) screen coord."
            puts stdout "              $result"
        }
    } else {
        if { $debug && $verbose } {
            puts stdout "load_mxu_TPX: Mark mask at ($mos(0,x0), $mos(0,y0)) \[canvas\] = ($mx, $my) \[screen\]"
        }
        mark $m_canvas $mx $my
    }
    set a1 [expr {15.0 * [DDMMSS2deg $lwx]}]
    set d1 [DDMMSS2deg $lwy]
    if { $debug && $verbose } {
        puts stdout [format "load_mxu_TPX: deg $EQX = %10.3f %10.3f" $a1 $d1]
    }
    # wcs to canvas via bonsai matrix
    set dum [C_wc w2c $a1 $d1]
    set cx [lindex $dum 0]
    set cy [lindex $dum 1]
    if { $debug && $verbose } {
        puts stdout [format "              canvas = %10.3f %10.3f" $cx $cy]
    }

    if { $auto_flag == 0 } {
        # Shift mask
        if { [catch { $m_image convert coords $cx $cy canvas nwx nwy screen } result] } {
            if { $debug && $verbose } {
                puts stdout "load_mxu_TPX: Mask center (cx,cy)"
                puts stdout "              Error converting ($cx,$cy) canvas to (nwx,nwy) screen coord."
                puts stdout "              $result"
            }
        } else {
            if { $debug && $verbose } {
                puts stdout "load_mxu_TPX: Shift mask to ($cx, $cy) \[canvas\] = ($nwx, $nwy) \[screen\]"
            }
            $this shift $m_canvas $nwx $nwy
        }

        # Rotate mask
        # In reset_mask->init_mask_world2, abs_ang_rad is set to 0
        set rota [$this cget -rotangle]
        $this configure -rotangle [expr {$lPA * $CSO}]
        if { $debug && $verbose } {
            puts stdout "load_mxu_TPX: Rotate mask to $lPA"
        }
        $this rotate $m_canvas $mos(0,x0) $mos(0,y0) 1
        # In rotate abs_ang_rad is set in rad back again

        # Restore default value
        $this configure -rotangle $rota
    }

    # ------ Read INS keywords --------
    # step 2: Load the same file again,
    #         and now create mxu slits
    # ---------------------------------
    set lshap ""
    set lwd 0.0
    set lle 0.0
    set lro 0.0
    set lof 0.0

    set loc_x1 ""
    set loc_x2 ""
    set loc_x3 ""
    set loc_y1 ""
    set loc_y2 ""
    set loc_y3 ""
    set loc_REF 0

    set lmod [$this cget -modus]
    set LMOD [string toupper $lmod]
    if { $debug && $verbose } {
        puts stdout "load_mxu_TPX: create $LMOD slits"
    }
    # Reset targets and reference targets
    # check this:
    for { set i 1 } { $i <= 40 } { incr i 1 } {
        set targw($i,f) 0
    }

    if { [file exists $myfile] & [file readable $myfile] } {
        if { $debug && $verbose } {
            puts stdout "load_mxu_TPX: Opening TPX file >$myfile<"
        }
	set fileId [open $myfile r]
        if { $debug && $verbose } {
	    puts stdout "load_mxu_TPX: File opened with fileId >$fileId<"
        }
	puts $lfId "load_mxu_TPX: File opened with fileId >$fileId<"
        # As long as there are lines in the file
        while { [gets $fileId line_org] >= 0 } {
            if { $debug && $verbose } {
                # List the file content
                puts stdout $line_org
            }
            if { [string match *$ins_kwd* $line_org] == 1 } {
                # INS
                switch -glob -- $line_org {
                    *NUMBER* {
                        set s_c [lindex $line_org 1]
                        if { $debug && $verbose } {
                            puts stdout "$s_c $LMOD slits in file"
                        }
                    }
                    *NAME* {
                         set lna [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_name $lna
                         if { $debug && $verbose } {
                             puts stdout "------N-$key-$loc_name-"
                         }
                    }
                    *REF* {
                        set lref [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        if { $lref } {
                            set loc_REF 1
                        }
                        if { $debug && $verbose } {
                            puts stdout "------R-$key-$loc_REF-"
                        }
                    }
                    *SHAPE* {
                        set lshap [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_SHAP $lshap
                        if { $debug && $verbose } {
                            puts stdout "------S-$key-$loc_SHAP-"
                        }
                    }
                    *WID* {
                        set lwd [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_W $lwd
                        if { $debug && $verbose } {
                            puts stdout "------W-$key-$loc_W-"
                        }
                    }
                    *LEN* {
                         set lle [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_LEN $lle
                         if { $debug && $verbose } {
                             puts stdout "------L-$key-$loc_LEN-"
                         }
                    }
                    *ROT* {
                        set lro [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_RO $lro
                        if { $debug && $verbose } {
                            puts stdout "------R-$key-$loc_RO-"
                        }
                    }
                    *OFF* {
                        set lof [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_OF $lof
                        if { $debug && $verbose } {
                            puts stdout "------O-$key-$loc_OF-"
                        }
                    }
                    *X1* {
                        set x1 [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_x1 $x1
                        if { $debug && $verbose } {
                            puts stdout "------X-$key-$x1-"
                        }
                    }
                    *X2* {
                         set x2 [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_x2 $x2
                         if { $debug && $verbose } {
                             puts stdout "------X-$key-$x2-"
                         }
                    }
                    *X3* {
                         set x3 [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_x3 $x3
                         if { $debug && $verbose } {
                             puts stdout "------X-$key-$x3-"
                         }
                    }
                    *Y1* {
                         set y1 [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_y1 $y1
                         if { $debug && $verbose } {
                             puts stdout "------Y-$key-$y1-"
                         }
                    }
                    *Y2* {
                         set y2 [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_y2 $y2
                         if { $debug && $verbose } {
                             puts stdout "------Y-$key-$y2"
                         }
                    }
                    *Y3* {
                         set y3 [lindex $line_org 1]
                         set key [lindex $line_org 0]
                         set key [split $key "."]
                         set key [lindex $key 1]
                         set key [string trimleft $key "TARG"]
                         set loc_y3 $y3
                         if { $debug && $verbose } {
                             puts stdout "------Y-$key-$y3-"
                         }
                    }
                    *ALPHA* {
                        set lwx [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_RA [expr {15.0 * [DDMMSS2deg $lwx]}]
                        if { $debug && $verbose } {
                            puts stdout "------A-$key-$loc_RA-"
                        }
                        # Science slit numbers start with 101,
                        # reference targets have no names
                        if { $key < 100 } {
                           # reference target
                           set targw($key,f) 1
                           set targw($key,a) $loc_RA
                        }
                    }
                    *DELTA* {
                        set lwy [lindex $line_org 1]
                        set key [lindex $line_org 0]
                        set key [split $key "."]
                        set key [lindex $key 1]
                        set key [string trimleft $key "TARG"]
                        set loc_DEC [DDMMSS2deg $lwy]
                        if { $debug && $verbose } {
                            puts stdout "------D-$key-$loc_DEC-"
                        }
                        # Reference targets have no names
                        if { $key < 100 } {
                            # Reference target
                            set targw($key,f) 1
                            set targw($key,d) $loc_DEC
                        } else {
                            # We assume that 'DELTA' is the last keyword for the
                            # slit read from the .fims file
                            # We can therefore create the slit here and now
                            # the next line read from the '.fims' will contain
                            # information for the next (=another) MXU slit
                            if { [catch { $m_image convert coords $loc_RA $loc_DEC "deg $EQX" x y canvas } result] } {
                                if { $debug && $verbose } {
                                    puts stdout "load_mxu_TPX: Slit $loc_name"
                                    puts stdout "              Error converting ($loc_RA,$loc_DEC) \[deg\] to (x,y) canvas coord."
                                    puts stdout "              $result"
                                }
                            }
                            if { [catch { $m_image convert coords $loc_RA $loc_DEC "deg $EQX" px py image } result] } {
                                if { $debug && $verbose } {
                                    puts stdout "load_mxu_TPX: Slit $loc_name"
                                    puts stdout "              Error converting ($loc_RA,$loc_DEC) \[deg\] to (px,py) image coord."
                                    puts stdout "              $result"
                                }
                            }

                            # This is necessary for calc_zemax
                            set targw(1,f) 1
                            set targw(1,x) $px
                            set targw(1,y) $py
                            if { [catch { $m_image convert coords $px $py image targw(1,a) targw(1,d) "wcs $EQX" } result] } {
                                if { $debug && $verbose } {
                                    puts stdout "load_mxu_TPX: For calc_zemax"
                                    puts stdout "              Error converting ($px,$py) image to (targw(1,a),targw(1,d)) WCS coord."
                                    puts stdout "              $result"
                                }
                            }

                            #
                            # Checking that slit is well positioned (inside the focal field)
                            set inside_mask_flag 0
                            set a [$m_canvas find overlapping \
	                          [expr {$x - 1}] [expr {$y - 1}] \
	                          [expr {$x + 1}] [expr {$y + 1}]]

                            foreach c $a {
	                        set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
	                        if { [string compare [lindex $local_tags 0] A] == 0 } {
		                    set inside_mask_flag 1
                                }
                            }
	                    if { $inside_mask_flag } {
	                        # puts stdout "inside $loc_name"
	                    } else {
	                        # info_dialog "The mask $loc_name is outside!!! \n This will skipped !!"
	                        puts stdout "The mask $loc_name is outside!!! \n This will skipped !!"
	                        continue
 	                    }
	                    if { $loc_REF == 0 } {
                                mxu_new_slit 0 $x $y $px $py $lwd $lle $lof $lro $lshap $loc_name $ff $mnomxus 0 0
                                set noms [fors::MXU_SLIT::nos]
                                if { [string match *CURVED* $lshap] } {
                                    if { $debug && $verbose } {
	                                puts stdout "load_mxu_TPX: "
                                        puts stdout "              loc_x1 = >$loc_x1<"
                                        puts stdout "              loc_y1 = >$loc_y1<"
                                        puts stdout "              loc_x2 = >$loc_x2<"
                                        puts stdout "              loc_y2 = >$loc_y2<"
                                        puts stdout "              loc_x3 = >$loc_x3<"
                                        puts stdout "              loc_y3 = >$loc_y3<"
                                    }
                                    C_mxus$noms move_curve [list $loc_x1 $loc_y1 $loc_x2 $loc_y2 $loc_x3 $loc_y3]
                                    update_bez_oc $noms
                                }
	                    }
                            # For the next slit
                            set loc_REF 0
                        }
                    }
                    default {
                         puts stdout "------X->$line_org"
                    }
	        }
            }
        }
        close $fileId

        set s_c 0
        set result {ok}
        # for { set i 1 } { $i < 20 } { incr i 1 } {
        #     puts stdout [format "%2d %10s %10.3f %10.3f" \
        #                          $i $targw($i,f) $targw($i,a) $targw($i,d)]
        # }

        # --------- reference targets
        C_ref_targ reset
        for { set i 31 } { $i <= 40 } { incr i 1 } {
            if { $targw($i,f) } {
                # puts stdout "reference target : $i  $targw($i,a) $targw($i,d)"
                if { [catch { $m_image convert coords $targw($i,a) $targw($i,d) "deg $EQX" x y "wcs $EQX" } result] } {
                    if { $debug && $verbose } {
                        puts stdout "load_mxu_TPX: Ref slit $i"
                        puts stdout "              Error converting ($targw($i,a),$targw($i,d)) \[deg\] to (x,y) WCS coord."
                        puts stdout "              $result"
                    }
                }
                catch {
                    C_ref_targ put $x $y
                    C_ref_targ plot $mos(0,r)
                }
            }
        }

        if { [C_ref_targ nort] } {
            puts stdout "[C_ref_targ nort] Reference Targets found"
        } else {
            puts stdout "\n->->-> Warning No Reference Targets found"
        }

        # C_ref_targ show
        # --------- reference targets end

        # Change to standard mask modus
        # You loaded a science setup base on a technical mask
        if { $L_mid > 900000 && $L_mid <= 900100 } {
            fors::MXU_SLIT::set_mmff 1
            puts stdout "tech_mask = >[fors::MXU_SLIT::get_mmff]<"
        }

        # targw locally as deg, but when move_slit is called it is
        # in wcs again
        # for { set i 1 } { $i < 20 } { incr i 1 } {
        #     if { $targw($i,f) != 0 } {
        #         # move_slit requires screen coords
        #         $m_image convert coords $targw($i,a) $targw($i,d) \
        #                                 "deg $EQX" mx my screen
        #         puts stdout [format "%2d-->%s (%11.7f,%11.7f)(deg $EQX) \
        #                              (%8.3f,%8.3f)(screen) " $i \
        #                              $targw($i,f) $targw($i,a) \
        #                              $targw($i,d) $mx $my \
        #                     ]
        #         set targw($i,f) 0
        #         # select,  not manually
        #         if { [$this move_slit $m_canvas $mx $my 1 0] != "" } {
        #             incr s_c
        #             # puts stdout $s_c
        #         }
        #     }
        # }

        puts stdout "+------------------------------+"
        puts stdout [format "%2d $LMOD slits positioned" [fors::MXU_SLIT::nos]]
        puts stdout "+------------------------------+"
        update_and_replot
    }
    set last [string length $myfile]
    set first [expr {[string last "/" $myfile] + 1}]
    set last_load_fn [string range $myfile $first [expr {$last - 6}]]

    $this configure -file_name [lindex [split $last_load_fn "."] 2]

    catch { SP update }
    tracer LEAVING
}

#===========================
public method save_TPX { } {
#===========================
# Central function for saving a fims setup file
# For multi object spectroscopic modes MOS, PMOS, HIT-MS and MXU
# a catalog can be selected. Slit positions will be compared
# with catalog positons and the catalog target name will be stored
# together with the slit position information.

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Initial file  name without extension: e.g. 'mos1.12.project'
    set mod "[$this cget -modus]"
    set M [string toupper $mod]
    set filename ${mod}${insId}.${exposure_counter}.[$this cget -file_name]

    # For mxu check first the number of reference slits
    # Do not check hitms
    if { [string match mxu [$this cget -modus]] } {
        set noms [fors::MXU_SLIT::nos]
        set sp_rsc [$this noars]
        if { $noms > 0 } {
            # For statuspanel
            catch {SP update}
            if { $sp_rsc < 3 } {
                info_dialog "Less than 3 MXU Reference Slits found\n\
                            Verification Failed !\n\
                            Setup not Saved !\n\n\
                            Click on the Slit Number Label\n\
                            (e.g. #4) close to the Slit to select\n\
                            as Reference Slit.\n\
                            LeftMouseBut=ON, MidMouseBut=OFF" .skycat1
                tracer LEAVING LT3
                return
            } else {
                # info_dialog "$sp_rsc MXU Reference Slits found\n\
                #              Verification Ok\n\
                #              Reason : $sp_rsc > 2"
	        puts "$sp_rsc MXU Reference Slits found - Verification Ok"
            }
        }
    }

    # For all modes, check the number of reference targets
    set text "Save  : $filename\n\n"
    set fail_flag [edit_ref_targ verify]
    if { $fail_flag } {
        append text "Verify: Failed ! \n\nPress  : \[Cancel\]"
        set deff 1
    } else {
        append text "Verify: Ok \n\nPress  : \[Ok\]"
        set deff 0
    }

    if { [$this cget -interactive] } {
        set wid .skycat1.save
        catch {destroy $wid}
        set w [DialogWidget $wid \
                            -title "Verification" \
                            -text $text \
                            -bitmap warning \
                            -default $deff \
                            -buttons { Ok Cancel } \
              ]

        # AAG - PPRS 10127
        # Edit was removed because of potential problems in long
        # filenames
        set result [$w activate]

        # activate expiration_date checking
        # set warn_flag [expire_date]
        if { $warn_flag && $result == 0 } {
            warn
        }

        if { $result == 1 } {
            tracer LEAVING CANCEL
            return
        }
    } else {
        puts stdout "Verification:\n $text"
    }

    puts stdout "\n Store acquisition in --> $set_dir/$filename"

    # Prepare a common header for output setup files
    set file_header ""
    append file_header "# created by SKYCAT-TOOL V$s_v -- FIMS-plugin V$f_v\n"
    append file_header "# orig filename  : $set_dir/$filename.%s\n"
    append file_header "# user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost\n"
    append file_header "# fits file dir  : [file dirname [[.skycat1 component image] cget -file]]\n"
    append file_header "# fits file name : [$this get_fits_name].fits\n"
    append file_header "# fits instrument: [$m_image fits get INSTRUME]\n"
    append file_header "# fits origin    : [$m_image fits get ORIGIN]\n"
    append file_header "# apply optidist : [$this cget -fors_frame]\n"
    append file_header "# fits object    : [$m_image fits get OBJECT]\n"

    set paf_header ""
    append paf_header "PAF.HDR.START; # Marks start of header\n"
    append paf_header "PAF.TYPE          \"Instrument Setup\" ; # Type of parameter file\n"
    append paf_header "PAF.ID            \"FORS2_$mod\" ; # Parameter file ID\n"
    append paf_header "PAF.NAME          \"$filename.p_%s\" ; # Parameter file NAME\n"
    append paf_header "PAF.DESC          \"FORS %s Mask Setup File\" ; # Purpose\n"
    append paf_header "PAF.CRTE.NAME     \"FIMS $f_v\" ; # SW version date UT\n"
    append paf_header "PAF.CRTE.DAYTIM   \"[exec date -u +%Y-%m-%dT%H:%M:%S]\" ; # Date and time of creation\n"
    append paf_header "PAF.LCHG.NAME     \"\" ; # Who did last change\n"
    append paf_header "PAF.LCHG.DAYTIM   \"\" ; # Date and time of last change\n"
    # append paf_header "PAF.CHCK.NAME     \"\" ; # Appl. checking par. file\n"
    # append paf_header "PAF.CHCK.DAYTIM   \"\" ; # Date and time of lastcheck\n"
    # append paf_header "PAF.CHCK.CHECKSUM \"\" ; # Parameter file checksum\n"
    append paf_header "PAF.HDR.END; # Marks end of header\n\n"

    # Find catalog name
    switch -exact -- [$this cget -modus] {
        echs -
        hitms -
        mos  -
        mxu  -
        pmos {
            set sc [get_cat]
            if { [llength $sc] == 0 } {
                puts stdout "No catalog active"
                set CatFileName no_catalog
            } else {
                catch { catalog $sc }
            }
        }
    }
    if { [string length $AutoposMethod] == 0 } {
        set AutoposMethod "manual "
    }
    append file_header "# catalog name   : $CatFileName\n"
    append file_header "# autoslit method: $AutoposMethod\n"

    # Save setup files according to mode
    set save_mode ""
    switch -exact -- [$this cget -modus] {
        echs -
        mos  {
            if { [string compare [$this cget -slit_width] open] == 0 } {
                set filename ims${insId}.$exposure_counter.[$this cget -file_name]
                set units "mm of the focal field "
                set save_mode img
            } else {
                set units "alp/del"
                set save_mode mos
            }
        }
        pmos {
            if { [string compare [$this cget -slit_width] open] == 0 } {
                set filename pims${insId}.$exposure_counter.[$this cget -file_name]
                set units "mm of the focal field "
                set save_mode img
            } else {
                set units "alp/del"
                set save_mode mos
            }
        }
        img -
        ipol {
            set units "mm of the focal field "
            set save_mode img
        }
        hits -
        lss {
            if { [string compare [$this cget -long_width] all] == 0 } {
                info_dialog "You cannot save with\nSlit width = all" .skycat1
                tracer LEAVING 3
                return
            }
            # CAUTION CAUTION CAUTION
            # -----------------------
            # 8.Jul.99 : lss2 will use new alignment
            #            lss1 will still use the old alignment
            # use new alignment for lss1 when FORS1-OS has been
            # updated ( ?? april 2000 ?? )
            # if { $insId == 1 } {
            #     # old alignment for FORS1 lss
            #     save_lss_TPX $filename
            # } else {
            #     # new alignment for FORS2 lss
            #     save_hits_TPX $filename
            # }
            set save_mode hits
        }
        hitms -
        mxu   {
            # saving files for the mxu
            # --------------------------
            # we save 4 different formats
            # adf is the aperture definition file (no longer used)
            # msf is the machine slit file (no longer used)
            # rs274x is the gerberX file (to be included in OBs)
            # TPX creates the well known fims file (to be used by FIMS)
            #
            # focal field coords of FORS2 (X,Y) have to be translated to
            # MMU working table coords. The Transfornation is
            # X_mmu = Y_ff
            # Y_mmu = -X_ff
            # why ?
            # 1)
            #    looking down (as a photon) we see a left-hand xy-system
            #    in the FORS2 focalplane
            #    while the mmu working table used a right-hand system.
            #    therefore x -> -x  y -> y
            # 2)
            #    for mechanical reasons it is more suitable to cut
            #    the mask from the invar sheet when rotated by 90 degr
            #    therefore y -> x, x -> y
            # standard masks have not been target of
            # correction of the optical distortion
            # we only change RA,DEC but not mmff values
            #
	    fors::MXU_SLIT::set_mmff 1
            if { [fors::MXU_SLIT::get_mmff] } {
                # Prepare standard mask to be saved
                # the wcs of all slits (in mm) have to be determined
                puts stdout "Standard mask: prepare save"
                # Update canvas--mmff conversion
                fors::MXU_SLIT::coeff $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                                      $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                                      $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
                set noms [fors::MXU_SLIT::nos]
                if { $noms > 0 } {
                    for { set i 1 } { $i <= $noms } { incr i 1 } {
			set targa [C_mxus$i get_RA]
			set targd [C_mxus$i get_DEC]
                        set refIn [C_mxus$i check_range]
                        if { $debug } {
                            puts stdout "save_TPX: Slit $i"
                            puts stdout "          RA = $targa"
                            puts stdout "         DEC = $targd"
                        }
                        if { $debug && $verbose } {
                            puts stdout "       refIn = $refIn"
                        }

		        if { !$refIn } {
                            if { $debug } {
                                # PPRS-10127
			        puts stdout "Slit $i is out of range: Remove slit and continue"
                            }
			    # TSZ remove slit:
                            set text "Slit $i at ($targa, $targd) \n\
                                      is out of range!!! \n\
                                      This reference slit will be ignored !! \n\
                                      You may want to move the mask such that \n\
                                      the reference slits are within the pre-image \n \n \
                                      PRESS OK"
                            if { [$this cget -interactive] } {
                                info_dialog $text .skycat1
                            } else {
                                puts stdout $text
                            }
			    C_mxus$i erase
			    C_mxus$i configure -ON 0
			    continue
		        } else {
                            if { $debug } {
			        puts stdout "In range: $i"
                            }
			    set targw(1,a) $targa
			    set targw(1,d) $targd
			    if { [catch { $m_image convert coords $targw(1,a) $targw(1,d) "wcs $EQX" \
			                                          targw(1,x)  targw(1,y) image } result] } {
                                if { $debug && $verbose } {
                                    puts stdout "save_TPX: Slit $i (targw(1,a),targw(1,d))"
                                    puts stdout "          Error converting ($targw(1,a),$targw(1,d)) WCS to (targw(1,x),targw(1,y)) image coord."
                                    puts stdout "          $result"
                                }
                            }
			    calc_zemax 1 $targw(1,x) $targw(1,y)
			    # targw(1,cx) and targw(1,cy) are not used
			    C_mxus$i put_OC  [list $targw(1,ca) $targw(1,cd)]
			    # Apply correction of differential refraction (10.5.99)
			    # Input slit position, corrected for optical distortion
			    if { $dire_flag } {
			        calc_diff_refrac 1 $targw(1,ca) $targw(1,cd)
			        C_mxus$i put_OC [list $targw(1,ca) $targw(1,cd)]
			    }
		        }
		    }
		}
	    }
	    fors::MXU_SLIT::set_mmff 0
	    # Special curved slit handling
            prepare_mxu_for_save
            # puts stdout "->ADF begin"
            # save_mxu_ADF $filename
            # puts stdout "->ADF end MSF begin"
            # save_mxu_MSF $filename
            # puts stdout "->MSF end GBR begin"
            # gerber file format for MASKID > 900100 can be saved
            # without maintenance password
            # handle mask ID <> 900100
            # 15.54.2000 greetings to jason and the DMD
            # agreement: FIMS must generate a
            # INS.MASK.NA = Mdddcccccccc
            # INS.MASK.ID = 9ddddd
            # INS.MASK.NAID = $INS.MASK.NA+$INS.MAK.ID
            # with c = single car and d = single digit
            # for technical masks the old concept is retained
            # if technical mask take naid from mxu class
            # if sky mask create new naid
            if { [fors::MXU_SLIT::get_mmff] } {
                set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
                puts stdout "SAVE_TPX: technical mask >$PM<"
            } else {
                update_mask_id
                set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
                puts stdout "SAVE_TPX: sky       mask >$PM<"
            }
            # new concept: the user cannot specify mask ids,
            # only technical masks have default mask id
            set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
            set a [lindex $PM 1];
            # must be 900000 (default sky mask) or
            # 900001-900100 (technical mask)
            if { 900000 < $a && $a < 900101 } {
                # technical mask
                # if there is no ins.mask.naid, the mask
                # is just newly defined by the staff astronomer
                # otherwise, if the ins.mask.naid is known
                # since it has been read from the fad, do not
                # create a gerber file since this mask already
                # exists on Paranal and is available
                if { [string compare [lindex $PM 0] "create"] == 0 } {
                    # maintenenace branch, mask not yet produced
                    set wahrheit [get_calib_passwd]
                    if { !$wahrheit } {
                        puts stdout "Password ERROR: NO Gerber file created"
                    } else {
                        # password ok, save gerber file of technical mask
                        save_mxu_RS274X $filename
                        puts stdout "gerber file saved with mask id "
                        save_mxu_RS274X_forPS $filename
                        # and save it as a postscript file
                        if { [catch {[exec $int_dir/bin/gerber2eps.perl $log_dir/$filename]} result] } {
                            puts stdout "Gerb2PS: $result"
                            set a [exec rm $log_dir/$filename]
                        } else {
                            # the perl script always gives a message
                        }
                    }
                } else {
                    # save a setup with a technical mask
                    # no gerber file required
                    # no postscript file required
                    puts stdout "Save technical mask without Gerber file"
                }
            } else {
                # science mask with default mask id
                # outside 900000 .. 900100
                save_mxu_RS274X $filename
                puts stdout "gerber file saved with mask id "
                save_mxu_RS274X_forPS $filename
                # in case of problems with perl, go on !
                if { [catch {[exec $int_dir/bin/gerber2eps.perl $log_dir/$filename]} result] } {
                    puts stdout "Gerb2PS: $result"
                    set a [exec rm $log_dir/$filename]
                } else {
                    # the perl script always gives a message
                }
            }
            append file_header "# INS.MASK.NA    : [lindex $PM 0]\n"
            append file_header "# INS.MASK.ID    : [lindex $PM 1]\n"
            append file_header "# INS.MASK.NAID  : [lindex $PM 0]+[lindex $PM 1]\n"
            set units "alp/del"
            # TPX must be the last, since it
            # increments the exposure counter
            if { [string compare [$this cget -modus] hitms] == 0 } {
                set save_mode hitms
            } else {
                set save_mode mxu
            }
        }
    }
    append file_header "# target setup for [$this cget -instrument]-$M\n"
    append file_header "# values in $units\n"
    append file_header "# --------------------------------------------\n\n"
    if { [string length $save_mode] > 0 } {
        save_${save_mode}_TPX $filename
    } else {
        if { $debug } {
            puts stdout "save_TPX: No save_<save_mode>_TPX executed!"
        }
    }

    set last_action "Save SetUp"
    set last_save_fn $filename
    catch { SP update }
    # Open a new log file
    open_log
    tracer LEAVING
    return "$filename"
}

#=========================================
private method save_hitms_TPX {filename} {
#=========================================
# open the *.obx file
# saving current parameters of one exposure
# modus hitms
# we save 5 !! files
# <mode>.<expo_count>.<fits_name>.fims   -> SET
# <mode>.<expo_count>.<fits_name>.targ   -> LOG
# <mode>.<expo_count>.<fits_name>.focf   -> LOG
# <mode>.<expo_count>.<fits_name>.p_targ -> SET (one line file)
# <mode>.<expo_count>.<fits_name>.p_focf -> SET (PAF header file)
# .fims   contains all specified keywords, its the
#         file to be loaded when [Load Mask]
# .focf   contains only the INS.FOC.SETUP keyword in one line
#         to be inserted in an obd file (e.g. via emacs)
# .targ   contains only the INS.TARG.SETUP keyword ...
#         and the INS.MSKREADY.NAID keyword
# .p_focf contains only the value of the INS.FOC.SETUP
#         keyword to be inserted to an obx file by p2pp
#         and the INS.MASK.NAID
# .p_targ contains only the value of the INS.TARG.SETUP keyword ...
# save results as one long string

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Focal and target setup as one long string
    set string_foc  ""
    set string_targ ""

    # Before we do anything we show the status of all (1 or 2) hitms slits
    puts stdout "\n\nsave_hitms_TPX"
    puts stdout "save_hitms_TPX: >[fors::MXU_SLIT::get_MASK_NAID]<"
    set noms [fors::MXU_SLIT::nos]
    if { $noms > 0 } {
        for { set i 1 } { $i <= $noms } { incr i 1 } {
             if { [C_mxus$i cget -ON] } {
                 puts stdout [format "#%2d %1d %14s %5.2f %5.2f %5.2f %5.2f %8s %8s %8s %8s %8s" \
                                     $i \
                                     [C_mxus$i cget -ON         ] \
                                     [C_mxus$i cget -target_name] \
                                     [C_mxus$i cget -slit_width ] \
                                     [C_mxus$i cget -slit_length] \
                                     [C_mxus$i cget -slit_offset] \
                                     [C_mxus$i cget -pos_ang    ] \
                                     [C_mxus$i cget -shape      ] \
                                     [C_mxus$i get_RA           ] \
                                     [C_mxus$i get_DEC          ] \
                                     [C_mxus$i get_OC_RA        ] \
                                     [C_mxus$i get_OC_DEC       ] \
                             ]
             }
        }
    }

    set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]

    # Apply optical correction if fits-image from FORS
    set corr_flag [$this cget -fors_frame]

    #-------------
    # .fims header
    #-------------
    # Open .fims setup file and write header
    set myfile "$set_dir/$filename.fims"
    puts stdout   "# orig filename  : $myfile"
    set fileId [open $myfile w]; # fims file
    set header [format $file_header fims]
    puts $fileId $header
    # Log file entry
    puts $lfId "FORS::save_hitms_TPX:  $fileId $myfile"; # log file

    # Technical mask flag
    set tm [fors::MXU_SLIT::get_mmff]
    # Science mask flag
    set sm [expr ! $tm]

    if { $sm } {
        # We copy the INS.FIMS.NAME again to find it as the first chars in KE
        # java-pippi
        # Particular Pipeline issue
        append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s\n" \
                                  $filename $s_v [lindex $f_v 0] \
                          ]

        # PAF header for p_focf file
        # --------------------------
        # The hitms2.p_focf differs from all other p_focf files of the
        # other FORS instrumental modi
        append string_foc [format $paf_header focf HIT-MS]
        append string_foc $header

    }

    # Particular Pipeline issue
    append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s" \
                              $filename $s_v [lindex $f_v 0] \
                      ]
    append string_targ [format " INS.FIMS.NAME  %-1s.p_targ_%1s_%-1s " \
                               $filename $s_v [lindex $f_v 0] \
                       ]

    if { $sm } {
        append string_foc "\n"
        # append string_targ "\n"
    }
    # What does this mean? Technical mask p_focf p_targ consist of one string,
    # sky masks follow the PAF convention

    #-------------------
    #-------- TCS ------
    #-------------------
    if { [catch { $m_image convert coords $world_x $world_y "deg $EQX" dummy_x dummy_y "wcs $EQX" } result] } {
        if { $debug && $verbose } {
            puts stdout "save_hitms_TPX: (world_x,world_y)"
            puts stdout "                Error converting ($world_x,$world_y) \[deg\] to (dummy_x,dummy_y) WCS coord."
            puts stdout "                $result"
        }
        set dummy_x 99:99:99.99
        set dummy_y 99:99:99.99
    }

    lassign [split $dummy_x :] d1 d2 d3
    puts $fileId       [format "TEL.TARG.ALPHA  %02s%02s%06.3f" $d1 $d2 $d3]
    append string_foc  [format "INS.FOCF.ALPHA  %02s%02s%06.3f" $d1 $d2 $d3]
    append string_targ [format " TEL.TARG.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
    if { $sm } {
        append string_foc "\n"; # append string_targ "\n"
    }
    lassign [split $dummy_y :] d1 d2 d3
    puts $fileId       [format "TEL.TARG.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
    append string_foc  [format "INS.FOCF.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
    append string_targ [format " TEL.TARG.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    if { $sm } {
        append string_foc "\n"; # append string_targ "\n"
    }
    puts $fileId       [format "TEL.ROT.OFFANGLE  %6.3f" [expr {$PAM * (-1)}]]
    append string_foc  [format "INS.FOCF.OFFANGLE  %6.3f" [expr {$PAM * (-1)}]]
    append string_targ [format " TEL.ROT.OFFANGLE  %6.3f " [expr {$PAM * (-1)}]]
    if { $sm } {
        append string_foc "\n"; # append string_targ "\n"
    }
    # puts $fileId       [format "TEL.TARG.EPOCH  %6.1f" $EQX]
    # append string_targ [format " TEL.TARG.EPOCH  %6.1f " $EQX]
    puts $fileId       [format "TEL.TARG.EQUINOX  %6.1f" $EQX]
    append string_targ [format " TEL.TARG.EQUINOX  %6.1f " $EQX]
    if { $sm } {
        append string_foc "\n"; # append string_targ "\n"
    }
    puts $fileId       [format "TEL.TARG.TYPE  COORDINATE"]
    append string_targ [format " TEL.TARG.TYPE  COORDINATE "]
    if { $sm } {
        # append string_targ "\n"
    }

    # User-defined program ID and MASK.NAID
    #
    # if { [lindex $PM 1] > 900000 }
    append string_foc  "INS.MASK.NAID  [lindex $PM 0]+[lindex $PM 1]"
    append string_targ " INS.MSKREADY.NAID  [lindex $PM 0]+[lindex $PM 1] "
    if { $sm } {
        append string_foc  "\n"
        # append string_targ "\n"
        append string_foc  "TPL.FILE.DIRNAME  \$INS_ROOT/SYSTEM/COMMON/SETUPFILES/INS"
        # append string_targ " TPL.FILE.DIRNAME  \$INS_ROOT/SYSTEM/COMMON/SETUPFILES/INS "
        append string_foc  "\n"
        # append string_targ "\n"
    }

    # Save number of slits
    set noms [fors::MXU_SLIT::nos]
    set s_c [$this noas]
    set sp_rsc [$this noars]
    puts $fileId [format "INS.SLIT.NUMBER  %d" $s_c]

    # If no reference slits are defined by the user,
    # take the first (up to) ten
    if { $sp_rsc == 0 } {
        # No reference slit specified
        puts stdout "$sp_rsc reference slits specified by user"
        set r_c 0
        for { set i 1 } { $i <= $noms } { incr i 1 } {
            if { $r_c < 10 } {
                if { [C_mxus$i cget -ON] } {
                    if { [string match *STRAIGHT* [C_mxus$i cget -shape]] } {
                        incr r_c 1
                        C_mxus$i configure -ref_slit 1
                    }
                }
            }
        }
	set sp_rsc $r_c
        puts stdout "$sp_rsc reference slits specified by FIMS"
    }
    catch {SP update}
    puts $fileId [format "INS.RSLIT.NUMBER  %d" $sp_rsc]

    # ---------------------
    # --- MXU slit loop ---
    # ---------------------
    set s_c 100;   # science slit counter offset ?
    set r_c 1;     # reference slit counter offset
    if { $noms > 0 } {
        for { set i 1 } { $i <= $noms } { incr i 1 } {
            # if slit is specified
            if { [C_mxus$i cget -ON] } {
                incr s_c 1
                # -1----------- NAME
                puts $fileId [format "INS.TARG$s_c.NAME  %19s" [C_mxus$i cget -target_name]]
                # To appear later in the fits file header of the FORS2 image
                append string_foc [format "INS.TARG$s_c.NAME  %19s " [C_mxus$i cget -target_name]]
                if { $sm } {
                    append string_foc "\n"
                }

                # -2----------- Reference slits
                if { [C_mxus$i cget -ref_slit] } {
	            append string_foc [format "INS.SLIT%1d.REF  %1d " $r_c $s_c]
                    if { $sm } {
                        append string_foc "\n"
                    }
	            puts $fileId [format "INS.SLIT%1d.REF  %1d" $r_c $s_c]
                    incr r_c 1
                }

                # -3----------- SLIT DIMENSIONS
                set cu_width  [C_mxus$i cget -slit_width ]
                set cu_length [C_mxus$i cget -slit_length]
                set cu_offset [C_mxus$i cget -slit_offset]
                set cu_posang [C_mxus$i cget -pos_ang    ]
                set cu_shape  [C_mxus$i cget -shape      ]

                puts $fileId [format "INS.TARG$s_c.SHAPE %19s" $cu_shape]
                puts $fileId [format "INS.TARG$s_c.WID  %3.1f" $cu_width]
                puts $fileId [format "INS.TARG$s_c.LEN  %3.1f" $cu_length]

                # Slit position angle only for STRAIGHT
                if { [string match *STRAIGHT* $cu_shape] } {
                    puts $fileId [format "INS.TARG$s_c.ROT  %3.1f" $cu_posang]
                }

                # slit offset only for STRAIGHT
                if { [string match *STRAIGHT* $cu_shape] } {
                    # puts stdout [format "INS.TARG$s_c.OFF    %3.1f" $cu_offset]
                    # PBG: Slit offset deactivated
                    # PBG: The keyword in this file allows fims to redraw the slit offset
                    # PBG: puts $fileId [format "INS.TARG$s_c.OFF    %3.1f" $cu_offset]
                }

                append string_foc [format "INS.TARG$s_c.SHAPE %19s " $cu_shape]
                if { $sm } {
                    append string_foc "\n"
                }
                append string_foc [format "INS.TARG$s_c.WID  %3.1f " $cu_width]
                if { $sm } {
                    append string_foc "\n"
                }
                append string_foc [format "INS.TARG$s_c.LEN  %3.1f " $cu_length]
                if { $sm } {
                    append string_foc "\n"
                }

                # Slit position angle only for STRAIGHT
                if { [string match *STRAIGHT* $cu_shape] } {
                    append string_foc [format "INS.TARG$s_c.ROT  %3.1f " $cu_posang]
                    if { $sm } {
                        append string_foc "\n"
                    }
                }

                # slit offset only for STRAIGHT
                if { [string match *STRAIGHT* $cu_shape] } {
                    # append string_targ [format " INS.TARG$s_c.OFF  %3.1f " $cu_offset]
                    # PBG: Slit offset deactivated
                    # PBG: Only when FORS2 understands the setup of this keyword: 
                    # PBG: append string_foc  [format "INS.TARG$s_c.OFF  %3.1f " $cu_offset]
                    # if { $sm } {
                    #     append string_foc "\n"
                    # }
                }

                # 3 bezier points for slit shape = CURVED
                if { [string match *CURVED* $cu_shape] } {
                    set bez_list [C_mxus$i get_curve_mm]
                    # The middle slit curve
                    set x1 [expr {([lindex $bez_list 0] + [lindex $bez_list  6]) * 0.5}]
                    set y1 [expr {([lindex $bez_list 1] + [lindex $bez_list  7]) * 0.5}]
                    set x2 [expr {([lindex $bez_list 2] + [lindex $bez_list  8]) * 0.5}]
                    set y2 [expr {([lindex $bez_list 3] + [lindex $bez_list  9]) * 0.5}]
                    set x3 [expr {([lindex $bez_list 4] + [lindex $bez_list 10]) * 0.5}]
                    set y3 [expr {([lindex $bez_list 5] + [lindex $bez_list 11]) * 0.5}]

                    puts $fileId [format "INS.TARG$s_c.X1   %3.1f" $x1]
                    puts $fileId [format "INS.TARG$s_c.Y1   %3.1f" $y1]
                    puts $fileId [format "INS.TARG$s_c.X2   %3.1f" $x2]
                    puts $fileId [format "INS.TARG$s_c.Y2   %3.1f" $y2]
                    puts $fileId [format "INS.TARG$s_c.X3   %3.1f" $x3]
                    puts $fileId [format "INS.TARG$s_c.Y3   %3.1f" $y3]

                    append string_foc [format "INS.TARG$s_c.X1   %3.1f " $x1]
                    if { $sm } {
                        append string_foc "\n"
                    }
                    append string_foc [format "INS.TARG$s_c.Y1   %3.1f " $y1]
                    if { $sm } {
                        append string_foc "\n"
                    }
                    append string_foc [format "INS.TARG$s_c.X2   %3.1f " $x2]
                    if { $sm } {
                        append string_foc "\n"
                    }
                    append string_foc [format "INS.TARG$s_c.Y2   %3.1f " $y2]
                    if { $sm } {
                        append string_foc "\n"
                    }
                    append string_foc [format "INS.TARG$s_c.X3   %3.1f " $x3]
                    if { $sm } {
                        append string_foc "\n"
                    }
                    append string_foc [format "INS.TARG$s_c.Y3   %3.1f " $y3]
                    if { $sm } {
                        append string_foc "\n"
                    }
                }
                # Curved slits will not be used for the alignment

                # Save uncorrected values to .fims file to avoid
                # iterative optical correction when load/save/load/save ..
                lassign [split [C_mxus$i get_RA] :] a1 a2 a3
                lassign [split [C_mxus$i get_DEC] :] d1 d2 d3
                puts $fileId [format "INS.TARG$s_c.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
                puts $fileId [format "INS.TARG$s_c.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

                if { $corr_flag } {
                    lassign [split [C_mxus$i get_OC_RA ] :] a1 a2 a3
                    lassign [split [C_mxus$i get_OC_DEC] :] d1 d2 d3
                }
                append string_foc [format "INS.TARG$s_c.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
                if { $sm } {
                    append string_foc "\n"
                }
                append string_foc [format "INS.TARG$s_c.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
                if { $sm } {
                    append string_foc "\n"
                }
            }
        }
    }

    #----------------------------
    # Reference objects are known
    #----------------------------
    # Total numer of ref objects
    puts stdout "[C_ref_targ nort] Reference Objects selected"

    # Reference Objects already in correct coords
    # Save reference objects to setup file
    for { set j 1 } { $j <= [C_ref_targ nort] } { incr j 1 } {
        set z [expr {$j + 30}]; # the INS.TARG$z. number

        lassign [split [C_ref_targ get $j a] :] a1 a2 a3
        puts $fileId [format "INS.TARG$z.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]

        lassign [split [C_ref_targ get $j d] :] d1 d2 d3
        puts $fileId [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

        if { $corr_flag } {
            set new_ref [calc_zemax_ref [C_ref_targ get $j a] [C_ref_targ get $j d]]
            lassign [split $new_ref " "] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
            # Apply differential refraction
            if { $dire_flag } {
                set new_ref [calc_diff_refrac 0 $a0 $d0]
                lassign [split $new_ref " "] a0 d0
                puts stdout "new REF COORDS : >$a0<  >$d0<"
            }
            lassign [split $a0 :] a1 a2 a3
            lassign [split $d0 :] d1 d2 d3
        }
        append string_targ [format " INS.TARG$z.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
        if { $sm } {
            # append string_targ "\n"
        }
        append string_targ [format " INS.TARG$z.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
        if { $sm } {
            # append string_targ "\n"
        }
    }

    #----------------------
    # Reference targets end
    #----------------------

    #-------------------------------
    #--- Write SEQ.PREP keywords ---
    #-------------------------------
    # Save also grism, OS filter and collimator
    #------------------------------------------
    puts $fileId [format "SEQ.PREP.LMIN  %8.3f" $L_MIN]
    puts $fileId [format "SEQ.PREP.LMAX  %8.3f" $L_MAX]
    puts $fileId [format "SEQ.PREP.GRIS  %s" [$this cget -grism]]
    puts $fileId [format "SEQ.PREP.FILT  %s" [$this cget -filter]]
    puts $fileId [format "SEQ.PREP.MASK_NA  %s" [lindex $PM 0]]
    puts $fileId [format "SEQ.PREP.MASK_ID  %s" [lindex $PM 1]]
    puts $fileId [format "SEQ.PREP.COLL  %s" [$this cget -collimat]]
    puts $fileId [format "SEQ.PREP.CAM   %s" [$this cget -camera]]
    append string_targ [format " INS.COLL.NAID  %s " [$this cget -collimat]]
    append string_targ [format " SEQ.CCD  %s " [$this cget -camera]]
    close $fileId

    #-----------------------------------------------------------------------
    #--- Save p_... files to be used by the P2PP FORS instrument package ---
    #-----------------------------------------------------------------------
    # Save the same stuff again as one string to a
    # file to be called by the P2PP tool
    set myfile $set_dir/$filename.p_targ
    #                             ------
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_targ
    close $fileId
    # catch { [exec pafchkCreate $myfile] } result

    set myfile $set_dir/$filename.p_focf
    #                             ------
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_foc
    close $fileId
    catch { [exec pafchkCreate $myfile] } result

    incr exposure_counter 1
    tracer LEAVING
}

#==========================================
private method save_hits_TPX { filename } {
#==========================================
# hits and lss use this branch
#          ---
# saving current parameters of one exposure
# we save 5 !! files
# <mode><instrument number>.<expo_count>.<fits_name>.fims -> SET
# <mode><instrument number>.<expo_count>.<fits_name>.targ -> LOG
# <mode><instrument number>.<expo_count>.<fits_name>.focf -> LOG
# <mode><instrument number>.<expo_count>.<fits_name>.p_targ -> SET
# <mode><instrument number>.<expo_count>.<fits_name>.p_forc -> SET
# e.g. hits2.15.ngc0815.fims is a local save by fims
#      this file can be loaded again when [Load Mask]
# .focf contains the same as p_focf but with
#       carriage returns for better reading
# .targ contains the same as p_targ but with carrage
#       returns for better reading
# .p_focf contains only the value of the INS.FOC.SETUP
#         keyword to be inserted to an obx file by p2pp
# .p_targ contains only the value of the INS.TARG.SETUP keyword ...
# ipol.<ec>.<fits_name>.offs -> LOG

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Save focal and target setup as one long string
    set string_foc  ""
    set string_targ ""

    # Apply optical correction if fits-image from FORS
    set corr_flag [$this cget -fors_frame]

    #-------------
    # .fims header
    #-------------
    # Open .fims setup file and write header
    set myfile "$set_dir/$filename.fims"
    puts stdout  "# orig filename  : $myfile"
    set fileId [open $myfile w]
    set header [format $file_header fims]
    puts $fileId $header
    # Log file entry
    puts $lfId "FORS::save_hits_TPX:  $fileId $myfile"

    # Particular ESO-Pipeline issue
    append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s " \
                      $filename $s_v [lindex $f_v 0]]
    append string_targ [format "INS.FIMS.NAME  %-1s.p_targ_%1s_%-1s " \
                       $filename $s_v [lindex $f_v 0]]

    #-------------------
    #-------- TCS ------
    #-------------------
    if { [catch { $m_image convert coords $world_x $world_y "deg $EQX" dummy_x dummy_y "wcs $EQX" } result] } {
        if { $debug && $verbose } {
            puts stdout "save_hits_TPX: (world_x,world_y)"
            puts stdout "               Error converting ($world_x,$world_y ) \[deg\] to (dummy_x,dummy_y) WCS coord."
            puts stdout "               $result"
        }
        set dummy_x 99:99:99.99
        set dummy_y 99:99:99.99
    }

    lassign [split $dummy_x :] d1 d2 d3
    puts $fileId [format "TEL.TARG.ALPHA  %02s%02s%06.3f" $d1 $d2 $d3]
    lassign [split $dummy_y :] d1 d2 d3
    puts $fileId [format "TEL.TARG.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
    if { [string match lss [$this cget -modus]] } {
        fix_mask_slit lss $corr_flag
    } else {
        fix_mask_slit hits $corr_flag
    }

    if { $corr_flag } {
        lassign [split $targw(2,ca) :] d1 d2 d3
        append string_foc   [format "INS.FOCF.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
        append string_targ  [format "TEL.TARG.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
        lassign [split $targw(2,cd) :] d1 d2 d3
        append string_foc   [format "INS.FOCF.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
        append string_targ  [format "TEL.TARG.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    } else {
        $m_image convert coords $world_x $world_y "deg $EQX" \
                                 dummy_x dummy_y  "wcs $EQX"
        lassign [split $dummy_x :] d1 d2 d3
        append string_foc   [format "INS.FOCF.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
        append string_targ  [format "TEL.TARG.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
        lassign [split $dummy_y :] d1 d2 d3
        append string_foc   [format "INS.FOCF.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
        append string_targ  [format "TEL.TARG.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    }

    # The rest is independent on correction
    puts $fileId [format "TEL.ROT.OFFANGLE  %6.3f" [expr {$PAM * (-1)}]]
    append string_foc  [format "INS.FOCF.OFFANGLE  %6.3f " [expr {$PAM * (-1)}]]
    append string_targ [format "TEL.ROT.OFFANGLE  %6.3f " [expr {$PAM * (-1)}]]
    # puts $fileId [format "TEL.TARG.EPOCH  %6.1f" $EQX]
    # append string_targ [format "TEL.TARG.EPOCH  %6.1f " $EQX]
    puts $fileId [format "TEL.TARG.EQUINOX  %6.1f" $EQX]
    append string_targ [format "TEL.TARG.EQUINOX  %6.1f " $EQX]
    puts $fileId [format "TEL.TARG.TYPE  COORDINATE"]
    append string_targ [format "TEL.TARG.TYPE  COORDINATE "]

    # slit or ht slit or pinhole
    # --------------------------
    # slit width string
    set dummy [$this cget -long_width]

    # ht-slit  or lss slit
    if { [string match lss [$this cget -modus]] } {
        set ins_long "lSlit"
        append ins_long [string index $dummy 0] "_" [string index $dummy 2] "arcsec"
    } else {
        set ins_long "htSlit"
        if { [string compare $dummy PH] == 0 } {
            # extra case: if htSlit0_3arcsec is choosen,
            #             take pinh0_95arcsec
            set ins_long [lindex $ph_range 0]
        } else {
            append ins_long [string index $dummy 0] "_" [string index $dummy 2] "arcsec"
        }
    }

    if { [string match *pinh* $ins_long] } {
        # Extra case
        puts $fileId [format "INS.PINH.NAID  %s" $ins_long]
        append string_foc [format "INS.PINH.NAID  %s " $ins_long]
    } else {
        # Normal case
        puts $fileId [format "INS.SLIT.NAID  %s" $ins_long]
        append string_foc [format "INS.SLIT.NAID  %s " $ins_long]
    }

    # SCIENCE TARGET
    # --------------
    if { [string length $targw(10,a)] == 0 } {
        plot_ref_star_area 0
        set stop 1
        info_dialog "Select the Science Target" .skycat1
        EnterObject .eo \
                    -title {Science Target} \
                    -image .skycat1.image \
                    -labels {ra dec} \
                    -command [code $this extra_lss 1]
        tkwait window .eo
        # wcs EQX
        set targw(10,a) $lss_slit_ra
        set targw(10,d) $lss_slit_de
        # delete cyan(-kali) area
        $m_canvas delete rtz
        set reftarg_zone 0
        set stop 0
    }

    # For LSS without pics and HITS, science target default targ10
    lassign [split $targw(10,d) :] d1 d2 d3
    lassign [split $targw(10,a) :] a1 a2 a3
    # puts stdout  [format "INS.TARG1.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
    puts $fileId [format "INS.TARG1.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
    append string_foc [format "INS.TARG1.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
    # puts stdout  [format "INS.TARG1.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
    puts $fileId [format "INS.TARG1.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
    append string_foc [format "INS.TARG1.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    puts $fileId [format "INS.TARG1.NAME  %20s" NN]
    append string_foc [format "INS.TARG1.NAME  %20s " NN]
    # Plot a red circle around the science target
    if { [catch { $m_image convert coords $targw(10,a) $targw(10,d) "wcs $EQX" x y canvas } result] } {
        if { $debug && $verbose } {
            puts stdout "save_hits_TPX: (targw(10,a),targw(10,d))"
            puts stdout "               Error converting ($targw(10,a),$targw(10,d)) WCS to (x,y) canvas coord."
            puts stdout "               $result"
        }
    } else {
        $m_canvas create oval [expr {$x - $mos(0,r)}] [expr {$y - $mos(0,r)}] \
                              [expr {$x + $mos(0,r)}] [expr {$y + $mos(0,r)}] \
                             -outline red -tag { A }
    }

    if { $corr_flag } {
        if { [catch { $m_image convert coords $targw(10,a) $targw(10,d) "wcs $EQX" targw(10,x) targw(10,y) image } result] } {
            if { $debug && $verbose } {
                puts stdout "save_hits_TPX: Slit 10 OC (targw(10,a),targw(10,d))"
                puts stdout "               Error converting ($targw(10,a),$targw(10,d)) WCS to (targw(10,x),targw(10,y)) image coord."
                puts stdout "               $result"
            }
            lassign [split $targw(10,a) :] a1 a2 a3
            lassign [split $targw(10,d) :] d1 d2 d3
        } else {
            calc_zemax 10 $targw(10,x) $targw(10,y)
            lassign [split $targw(10,ca) :] a1 a2 a3
            lassign [split $targw(10,cd) :] d1 d2 d3
        }
    }
    append string_targ [format "INS.TARG1.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
    append string_targ [format "INS.TARG1.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    append string_targ [format "INS.TARG1.NAME  %20s " NN]

    # ---------------------------
    # Reference objects are known
    # ---------------------------
    # Total numer of ref objects
    puts stdout "[C_ref_targ nort] Reference Objects selected"

    # Reference Objects already in correct coords
    # Save reference objects to setup file
    for { set j 1 } { $j <= [C_ref_targ nort] } { incr j 1 } {
        set z [expr {$j + 30}]; # the INS.TARG$z. number
        lassign [split [C_ref_targ get $j a] :] a1 a2 a3
        # puts stdout  [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3]
        puts $fileId [format "INS.TARG$z.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
        lassign [split [C_ref_targ get $j d] :] d1 d2 d3
        # puts stdout  [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
        puts $fileId [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
        if { [catch { $m_image convert coords [C_ref_targ get $j a] [C_ref_targ get $j d] "wcs $EQX" x y image } result] } {
            if { $debug && $verbose } {
                puts stdout "save_hits_TPX: Ref. object $j"
                puts stdout "               Error converting ([C_ref_targ get $j a],[C_ref_targ get $j d]) WCS to (x,y) image coord."
                puts stdout "               $result"
            }
        }
        # puts stdout "$j   $x $y"
        if { $corr_flag } {
            set new_ref [calc_zemax_ref [C_ref_targ get $j a] \
                        [C_ref_targ get $j d]]
            lassign [split $new_ref " "] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
            # apply differential refraction
            if { $dire_flag } {
                set new_ref [calc_diff_refrac 0 $a0 $d0]
                lassign [split $new_ref " "] a0 d0
                puts stdout "new REF COORDS : >$a0<  >$d0<"
            }
            lassign [split $a0 :] a1 a2 a3
            lassign [split $d0 :] d1 d2 d3
        }
        append string_targ [format "INS.TARG$z.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
        append string_targ [format "INS.TARG$z.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    }

    #----------------------
    # Reference targets end
    #----------------------
    # if { [C_ref_targ nort] > 0 } {
    #     if there is an alignment specify alignment slit
    #     append string_targ "  INS.MOS10.POS 0.000   INS.MOS10.WID  0.5"
    # }

    #-------------------------------
    #--- Write SEQ.PREP keywords ---
    #-------------------------------
    # Save also grism, OS filter and collimator
    #------------------------------------------
    puts $fileId [format "SEQ.PREP.LMIN  %8.3f" $L_MIN]
    puts $fileId [format "SEQ.PREP.LMAX  %8.3f" $L_MAX]
    puts $fileId [format "SEQ.PREP.GRIS  %10s" [$this cget -grism]]
    puts $fileId [format "SEQ.PREP.FILT  %10s" [$this cget -filter]]
    puts $fileId [format "SEQ.PREP.COLL  [$this cget -collimat]"]
    puts $fileId [format "SEQ.PREP.CAM   [$this cget -camera]"]
    append string_targ "SEQ.CCD  [$this cget -camera] "
    append string_targ "INS.COLL.NAID  [$this cget -collimat] "
    close $fileId

    #-----------------------------------------------------------------------
    #--- Save p_... files to be used by the P2PP FORS instrument package ---
    #-----------------------------------------------------------------------
    # Save the same stuff again as one string to a
    # file to be called by P2PP tool
    set myfile $set_dir/$filename.p_targ
    #                             ------
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_targ
    close $fileId

    set myfile $set_dir/$filename.p_focf
    #                             ------
    set fileId [open $myfile w]
    # Special behaviour when saving an empty string to a file
    # the file  will have 1 char and P2PP will complain
    # therefore dont copy empty string to file
    if { [string length $string_foc] > 0 } {
        puts -nonewline $fileId $string_foc
    }
    close $fileId

    # Save the same stuff again but in a more readable format
    # into the .fims/LOG directory (only for maintenance)
    set s2t [ldelete_all [split $string_targ " "]  ""]
    set number [llength $s2t]
    for { set i 0 } { $i < $number } { incr i 2 } {
         set s2t [lreplace $s2t $i $i [lrange $s2t $i [expr {$i + 1}]]]
    }
    for { set i [expr {$number - 1}] } { $i > 0 } { incr i -2 } {
         set s2t [lreplace $s2t $i $i]
    }
    set string2_targ [join $s2t "\n"]
    set myfile $log_dir/$filename.targ
    #                             ----
    set fileId [open $myfile w]
    puts $fileId $string2_targ
    close $fileId

    set s2f [ldelete_all [split $string_foc " "]  ""]
    set number [llength $s2f]
    for { set i 0 } { $i < $number } { incr i 2 } {
        set s2f [lreplace $s2f $i $i [lrange $s2f $i [expr {$i + 1}]]]
    }
    for { set i [expr {$number - 1}] } { $i > 0 } { incr i -2 } {
        set s2f [lreplace $s2f $i $i]
    }
    set string2_foc [join $s2f "\n"]
    set myfile $log_dir/$filename.focf
    #                             ----
    set fileId [open $myfile w]
    puts $fileId $string2_foc
    close $fileId
    incr exposure_counter 1
    tracer LEAVING
}

#=========================================
private method save_img_TPX { filename } {
#=========================================
# open the *.obx file
# saving current parameters of one exposure
# we save 5 !! files
# <mode>.<expo_count>.<fits_name>.fims -> SET
# <mode>.<expo_count>.<fits_name>.targ -> LOG
# <mode>.<expo_count>.<fits_name>.focf -> LOG
# <mode>.<expo_count>.<fits_name>.p_targ -> SET
# <mode>.<expo_count>.<fits_name>.p_forc -> SET
# .fims contains all specified keywords, its the
#       file to be loaded when [Load Mask]
# .focf contains only the INS.FOC.SETUP keyword in one line
#       to be inserted in an obd file (e.g. via emacs)
# .targ contains only the INS.TARG.SETUP keyword ...
# .p_focf contains only the value of the INS.FOC.SETUP
#         keyword to be inserted to an obx file by p2pp
# .p_targ contains only the value of the INS.TARG.SETUP keyword ...
# if IPOL save also the offset file
# ipol.<ec>.<fits_name>.offs -> LOG
# 18.11.99 bug detected: INS.MASK.NAID polarimMask not saved to p_focf
#          in IPOL-HR

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Focal and target setup as one long string
    set string_foc  ""
    set string_targ ""
    
    # Empirical optical distortion of FORS1
    set corr_flag [$this cget -fors_frame]

    #-------------
    # .fims header
    #-------------
    # Open .fims setup file and write header
    set myfile "$set_dir/$filename.fims"
    puts stdout  "# orig filename  : $myfile"
    set fileId [open $myfile w]
    set header [format $file_header fims]
    puts $fileId $header
    # Log file entry
    puts $lfId "FORS::save_img_TPX:  $fileId $myfile"

    # Particular Pipeline issue
    append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s " \
                      $filename $s_v [lindex $f_v 0]]
    append string_targ [format "INS.FIMS.NAME  %-1s.p_targ_%1s_%-1s " \
                       $filename $s_v [lindex $f_v 0]]

    #-------------------
    #-------- TCS ------
    #-------------------
    if { [catch { $m_image convert coords $world_x $world_y "deg $EQX" dummy_x dummy_y "wcs $EQX" } result] } {
        if { $debug && $verbose } {
            puts stdout "save_img_TPX: (world_x,world_y)"
            puts stdout "              Error converting ($world_x,$world_y) \[deg\] to (dummy_x,dummy_y) WCS coord."
            puts stdout "              $result"
        }
        set dummy_x 99:99:99.99
        set dummy_y 99:99:99.99
    }

    lassign [split $dummy_x :] d1 d2 d3
    puts $fileId [format "TEL.TARG.ALPHA  %02s%02s%06.3f" $d1 $d2 $d3]
    lassign [split $dummy_y :] d1 d2 d3
    puts $fileId [format "TEL.TARG.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

    # Here a special feature for IPOL dither
    # When an offset of 1 MOS stripe width is choosen
    # -> 2 exposures 1 offsets
    # When an offset of 2/3 MOS stripe width is choosen
    # -> 3 exposures 2 offsets
    # Change TEL keyword for *.targ (not for *.fims)
    if { [string compare [$this cget -modus] "ipol"] == 0 } {
        # save the offset file
        switch -exact -- $dith_num {
            1 {
                # one offset
                # calculate the offset
                set result [calc_dither 1 1.0]
                set off_a [lindex $result 2]
                set off_d [lindex $result 3]
            }
            2 {
                # move the mask back and calculate the offset
                # the offset will be carried out two times
                #
                set result [calc_dither -1 2.0/3.0]
                set off_a [expr {(-1.0) * [lindex $result 2]}]
                set off_d [expr {(-1.0) * [lindex $result 3]}]
                set a0 [lindex $result 0]; # new TEL
                set d0 [lindex $result 1]; # new TEL
                # overwrite default TEL
                # $m_image convert coords $a0 $d0 "deg $EQX" \
                #                         dummy_x dummy_y "wcs $EQX"
            }
            default {
                puts stdout "Warning: IPOL OFFSET"
            }
        }
        # this is no longer necessary since the templates use
        # TEL.TARG.OFFSETY
        if { [string compare [$this cget -modus] "ipol"] == 0 } {
            if { [string match *HR*  [$this cget -collimat]] } {
                # one could also ask for 'if { $mask == pol }
                append string_foc "INS.MASK.NAID polarimMask "
            }
        }
        # open offset file and save offset
        # set offfile "$log_dir/$filename.offs"
        # set offId [open $offfile w]
        # puts $offId [format \
        # "SEQ.OFF.NUMBER  %1d  SEQ.OFF.ALPHA  %8.5f SEQ.OFF.DELTA  %8.5f" \
        # $dith_num $off_a $off_d]
        # close $offId
        # info_dialog  [format "IPOL offset (for P2PP):\n\n\
        #               SEQ.OFF.NUMBER  %1d \n\
        #               SEQ.OFF.ALPHA  %8.5fs\n\
        #               SEQ.OFF.DELTA  %8.5f''\n\n\
        #               written to\n\
        #               %s/\n%s" \
        #               $dith_num $off_a $off_d \
        #               $log_dir $filename.offs]
    }
    # Take modified TEL in case of 2/3 offset IPOL
    lassign [split $dummy_x :] d1 d2 d3
    append string_targ [format "TEL.TARG.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
    lassign [split $dummy_y :] d1 d2 d3
    append string_targ [format "TEL.TARG.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]

    # The rest is  independent on IPOL
    puts $fileId [format "TEL.ROT.OFFANGLE  %6.3f" [expr {$PAM * (-1)}]]
    append string_targ [format "TEL.ROT.OFFANGLE  %6.3f " [expr {$PAM * (-1)}]]
    # puts $fileId [format "TEL.TARG.EPOCH  %6.1f" $EQX]
    # append string_targ [format "TEL.TARG.EPOCH  %6.1f " $EQX]
    puts $fileId [format "TEL.TARG.EQUINOX  %6.1f" $EQX]
    append string_targ [format "TEL.TARG.EQUINOX  %6.1f " $EQX]
    puts $fileId [format "TEL.TARG.TYPE  COORDINATE"]
    append string_targ [format "TEL.TARG.TYPE  COORDINATE "]

    # This is not for IPOL hr and PIMS hr (mask=pol), but for
    # IMG hr/sr, IPOLsr IMS sh/hr PIMS sr
    if { [string compare $mask "pol"] } {
        set frac [expr {$XBLADEA / ($XBLADEA - $XBLADEB)}]
        for { set i 1 } { $i < 20 } { incr i 1 } {
            # calculate canvas back to mm
            # ctom
            # update width
            set xdum [expr {$mos($i,xb) - $mos($i,xc)}]
            set ydum [expr {$mos($i,yb) - $mos($i,yc)}]
            $m_image convert dist $xdum $ydum canvas x y "deg $EQX"
            set mos($i,b) [expr {3600.0 * sqrt($x*$x + $y*$y)}]

            # update middel position
            set px [expr {$mos($i,xb) + ($mos($i,xc) - $mos($i,xb)) / 2.0}]
            set py [expr {$mos($i,yb) + ($mos($i,yc) - $mos($i,yb)) / 2.0}]
            # puts stdout "#$i- $mos($i,xb) < $px < $mos($i,xc) || \
            #              #$i- $mos($i,yb) < $py < $mos($i,yc)"
            # $m_image convert coords $xdum $ydum canvas px py "deg $EQX"
            # x=zero position
            # puts stdout "$XSRMIN/$XSRMAX/$frac"
            set ax [expr {$mos($i,xa) + ($mos($i,xd) - $mos($i,xa)) * $frac}]
            set ay [expr {$mos($i,ya) + ($mos($i,yd) - $mos($i,ya)) * $frac}]
            # puts stdout "#$i- $mos($i,xa) < $ax < $mos($i,xd) || \
            #              #$i- $mos($i,ya) < $ay < $mos($i,yd)"
            $m_image convert dist [expr {$px - $ax}] [expr {$py - $ay}] canvas a d "deg $EQX"
            set dist_arc [expr {3600.0 * sqrt($a*$a+$d*$d)}]

            # since we are only dealing with distances, we still
            # need the sign +-
            # positiv if c-axe < b-axis
            set cax [expr {sqrt(($mos($i,xc)-$ax)*($mos($i,xc)-$ax)+\
                                ($mos($i,yc)-$ay)*($mos($i,yc)-$ay))}]
            set bax [expr {sqrt(($mos($i,xb)-$ax)*($mos($i,xb)-$ax)+\
                                ($mos($i,yb)-$ay)*($mos($i,yb)-$ay))}]
            if { $bax < $cax } {
                set sign 1
            } else {
                set sign -1
            }
            set dist_mm [expr {$sign * $dist_arc / $focscale}]
            if { $dist_mm < -105.0 } {
                set dist_mm -117; set mos($i,b) [expr {6.0 * $focscale}]
            }
            if { $dist_mm >  105.0 } {
                set dist_mm +117; set mos($i,b) [expr {6.0 * $focscale}]
            }
            puts $fileId [format "INS.MOS$i.POS  %06.3f" $dist_mm]
            append string_foc  [format "INS.MOS$i.POS  %06.3f " $dist_mm]
            set dummy  [expr {$mos($i,b) / $focscale}]
            if { $dummy > 240.0 } {
                set dummy 234
            }
            if { $dummy < 6.0  } {
                set dummy 6
            }
            puts $fileId [format "INS.MOS$i.WID  %05.3f" $dummy]
            append string_foc [format "INS.MOS$i.WID  %05.3f " $dummy]
            # puts stdout "save_img_TPX: POS=$dist_arc ''  POS=$dist_mm WID=$dummy"
            # -----------------------------------------------------------
            #   since 08/04/99 new DICD INS.MOS$i.WID --> INS.MOS$i.WIDTH
            # -----------------------------------------------------------
            # puts $fileId [format "INS.MOS$i.WIDTH  %05.3f" $dummy]
            # append string_foc [format "INS.MOS$i.WIDTH  %05.3f " $dummy]
            # puts stdout "save_img_TPX: POS=$dist_arc ''  POS=$dist_mm WIDTH=$dummy"
            # save if the blade has been moved
            # this is a good information when loading
            # the setup again
            if { $targw($i,f) != 0 } {
                puts $fileId [format "INS.MOS$i.SET  %2d" $i]
            }
        }
    }

    # ---------------------------
    # Reference objects are known
    # ---------------------------
    # Total numer of ref objects
    puts stdout "[C_ref_targ nort] Reference Objects selected"

    # Reference Objects already in correct coords
    # Save reference objects to setup file
    for { set j 1 } { $j <= [C_ref_targ nort] } { incr j 1 } {
         set z [expr {$j + 30}]; # the INS.TARG$z. number
         lassign [split [C_ref_targ get $j a] :] a1 a2 a3
         # puts stdout  [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3]
         puts $fileId [format "INS.TARG$z.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
         lassign [split [C_ref_targ get $j d] :] d1 d2 d3
         # puts stdout  [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
         puts $fileId [format "INS.TARG$z.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
         if { $corr_flag } {
             set new_ref [calc_zemax_ref [C_ref_targ get $j a] [C_ref_targ get $j d]]
             lassign [split $new_ref " "] a0 d0
             puts stdout "new REF COORDS : >$a0<  >$d0<"
             # apply differential refraction
             if { $dire_flag } {
                 set new_ref [calc_diff_refrac 0 $a0 $d0]
                 lassign [split $new_ref " "] a0 d0
                 puts stdout "new REF COORDS : >$a0<  >$d0<"
             }
             lassign [split $a0 :] a1 a2 a3
             lassign [split $d0 :] d1 d2 d3
         }
         append string_targ [format "INS.TARG$z.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
         append string_targ [format "INS.TARG$z.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    }

    #----------------------
    # Reference targets end
    #----------------------
    # if { [C_ref_targ nort] > 0 } {
    #     if there is an alignment specify alignment slit
    #     append string_targ "  INS.MOS10.POS 0.000   INS.MOS10.WID  0.5"
    # }

    #-------------------------------
    #--- Write SEQ.PREP keywords ---
    #-------------------------------
    # Save also grism, OS filter and collimator
    # ims or pims
    #------------------------------------------
    if { [string compare [$this cget -slit_width] open] == 0 } {
         puts $fileId [format "SEQ.PREP.LMIN  %8.3f" $L_MIN]
         puts $fileId [format "SEQ.PREP.LMAX  %8.3f" $L_MAX]
         puts $fileId [format "SEQ.PREP.GRIS  %10s" [$this cget -grism]]
         puts $fileId [format "SEQ.PREP.FILT  %10s" [$this cget -filter]]
    }
    puts $fileId [format "SEQ.PREP.COLL  [$this cget -collimat]"]
    puts $fileId [format "SEQ.PREP.CAM   [$this cget -camera]"]
    append string_targ "SEQ.CCD  [$this cget -camera] "
    append string_targ "INS.COLL.NAID  [$this cget -collimat] "
    close $fileId

    #-----------------------------------------------------------------------
    #--- Save p_... files to be used by the P2PP FORS instrument package ---
    #-----------------------------------------------------------------------
    # Save the same stuff again as one string to a
    # file to be called by P2PP tool
    set myfile $set_dir/$filename.p_targ
    #                             ------
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_targ
    close $fileId

    set myfile $set_dir/$filename.p_focf
    #                             ------
    set fileId [open $myfile w]
    # Special behaviour when saving an empty string to a file
    # the file  will have 1 char and P2PP will complain
    # therefore dont copy empty string to file
    if { [string length $string_foc] > 0 } {
        puts -nonewline $fileId $string_foc
    }
    close $fileId

    # Save the same stuff again but in a more readable format
    # into the .fims/LOG directory (only for maintenance)
    set s2t [ldelete_all [split $string_targ " "]  ""]
    set number [llength $s2t]
    for {set i 0} {$i < $number} { incr i 2 } {
        set s2t [lreplace $s2t $i $i [lrange $s2t $i [expr {$i + 1}]]]
    }
    for { set i [expr {$number - 1}] } { $i > 0 } { incr i -2 } {
        set s2t [lreplace $s2t $i $i]
    }
    set string2_targ [join $s2t "\n"]

    set myfile $log_dir/$filename.targ
    #                             ----
    set fileId [open $myfile w]
    puts $fileId $string2_targ
    close $fileId

    set s2f [ldelete_all [split $string_foc " "]  ""]
    set number [llength $s2f]
    for { set i 0 } { $i < $number } { incr i 2 } {
        set s2f [lreplace $s2f $i $i [lrange $s2f $i [expr {$i + 1}]]]
    }
    for { set i [expr {$number - 1}] } { $i > 0 } { incr i -2 } {
        set s2f [lreplace $s2f $i $i]
    }
    set string2_foc [join $s2f "\n"]
    set myfile $log_dir/$filename.focf
    #                             ----
    set fileId [open $myfile w]
    puts $fileId $string2_foc
    close $fileId
    incr exposure_counter 1
    tracer LEAVING
}

#=========================================
private method save_lss_TPX { filename } {
#=========================================
# open the *.obx file
# saving current parameters of one exposure
# modus lss
# we save 5 !! files
# <mode>.<expo_count>.<fits_name>.fims
# <mode>.<expo_count>.<fits_name>.targ
# <mode>.<expo_count>.<fits_name>.focf
# <mode>.<expo_count>.<fits_name>.p_fims
# <mode>.<expo_count>.<fits_name>.p_fims
# .fims contains all specified keywords, its the
#       file to be loaded when [Load Mask]
# .focf contains only the INS.FOC.SETUP keyword in one line
#       to be inserted in an obd file (e.g. via emacs)
# .targ contains only the INS.TARG.SETUP keyword ...
# .p_focf contains only the value of the INS.FOC.SETUP
#         keyword to be inserted to an obx file by p2pp
# .p_targ conatins only the value of the INS.TARG.SETUP keyword ...

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Focal and target setup as one long string
    set string_foc  ""
    set string_targ ""

    # Apply optical correction if fits-image from fors
    set corr_flag [$this cget -fors_frame]

    C_ref_targ reset
    set stop 1
    plot_ref_star_area 0

    info_dialog "Select the Slit Position (#1)" .skycat1
    EnterObject .eo \
                -title {Slit Position (#1)} \
                -image .skycat1.image \
                -labels {ra dec} \
                -command [code $this extra_lss 1]
    tkwait window .eo
    C_ref_targ put $lss_slit_ra $lss_slit_de
    C_ref_targ show
    C_ref_targ plot $mos(0,r)

    info_dialog "Select the Reference Object (#2)" .skycat1
    EnterObject .eo \
                -title {Reference Object (#2)} \
                -image .skycat1.image \
                -labels {ra dec} \
                -command [code $this extra_lss 2]
    tkwait window .eo
    C_ref_targ put $lss_ref_ra $lss_ref_de
    C_ref_targ show
    C_ref_targ plot $mos(0,r)

    # Delete cyan forbidden area
    $m_canvas delete rtz
    set reftarg_zone 0
    set stop 0

    #-------------
    # .fims header
    #-------------
    # Open .fims setup file and write header
    set myfile "$set_dir/$filename.fims"
    puts stdout  "# orig filename  : $myfile"
    set fileId [open $myfile w]
    set header [format $file_header fims]
    puts $fileId $header
    # Log file entry
    puts $lfId "FORS::save_lss_TPX:  $fileId $myfile"

    # Particular Pipeline issue
    append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s " \
                              $filename $s_v [lindex $f_v 0] \
                      ]
    append string_targ [format "INS.FIMS.NAME  %-1s.p_targ_%1s_%-1s " \
                               $filename $s_v [lindex $f_v 0] \
                       ]

    #-------------------
    #-------- TCS ------
    #-------------------
    if { [catch { $m_image convert coords $world_x $world_y "deg $EQX" dummy_x dummy_y "wcs $EQX" } result] } {
        if { $debug && $verbose } {
            puts stdout "save_lss_TPX: (world_x,world_y)"
            puts stdout "              Error converting ($world_x,$world_y) \[deg\] to (dummy_x,dummy_y) WCS coord."
            puts stdout "              $result"
        }
        set dummy_x 99:99:99.99
        set dummy_y 99:99:99.99
    }

    lassign [split $dummy_x :] d1 d2 d3
    puts $fileId [format "TEL.TARG.ALPHA  %02s%02s%06.3f" $d1 $d2 $d3]
    append string_targ [format "TEL.TARG.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
    lassign [split $dummy_y :] d1 d2 d3
    puts $fileId [format "TEL.TARG.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
    append string_targ [format "TEL.TARG.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    puts $fileId [format "TEL.ROT.OFFANGLE  %6.3f" [expr {$PAM * (-1)}]]
    append string_targ  [format "TEL.ROT.OFFANGLE  %6.3f " [expr {$PAM * (-1)}]]
    # puts $fileId [format "TEL.TARG.EPOCH  %6.1f" $EQX]
    # append string_targ  [format "TEL.TARG.EPOCH  %6.1f " $EQX]
    puts $fileId [format "TEL.TARG.EQUINOX  %6.1f" $EQX]
    append string_targ  [format "TEL.TARG.EQUINOX  %6.1f " $EQX]
    puts $fileId [format "TEL.TARG.TYPE  COORDINATE"]
    append string_targ  [format "TEL.TARG.TYPE  COORDINATE "]

    # COLL
    # ----
    append string_targ [format "INS.COLL.NAID  %s " [$this cget -collimat]]

    # SLIT
    # ----
    set dummy [$this cget -long_width]
    set ins_long "lSlit";

    # Pinhole only available under save_hits_TPX
    append ins_long [string index $dummy 0] "_" [string index $dummy 2] "arcsec"
    puts  $fileId [format "INS.SLIT.NAID  %s" $ins_long]
    append string_foc [format "INS.SLIT.NAID  %s " $ins_long]

    # Reference and slit target
    # -------------------------
    # puts stdout "$lss_slit_ra \n$lss_slit_de \n$lss_ref_ra \n$lss_ref_de"
    # TARGET 1 is SLIT OBJECT
    lassign [split $lss_slit_ra :] a1 a2 a3
    # puts stdout  [format "INS.TARG1.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
    puts $fileId [format "INS.TARG1.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
    lassign [split $lss_slit_de :] d1 d2 d3
    # puts stdout  [format "INS.TARG1.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
    puts $fileId [format "INS.TARG1.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

    if { $corr_flag } {
        set new_ref [calc_zemax_ref $lss_slit_ra $lss_slit_de]
        lassign [split $new_ref " "] a0 d0
        puts stdout "new REF COORDS : >$a0<  >$d0<"
        # apply differential refraction
        if { $dire_flag } {
            set new_ref [calc_diff_refrac 0 $a0 $d0]
            lassign [split $new_ref " "] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
        }
        lassign [split $a0 :] a1 a2 a3
        lassign [split $d0 :] d1 d2 d3
    }
    append string_targ [format "INS.TARG1.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
    append string_targ [format "INS.TARG1.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]

    # TARGET 2 is REFERENCE OBJECT
    lassign [split $lss_ref_ra :] a1 a2 a3
    # puts stdout [format "INS.TARG2.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3]
    puts $fileId [format "INS.TARG2.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
    lassign [split $lss_ref_de :] d1 d2 d3
    # puts stdout  [format "INS.TARG2.DELTA   %02s%02s%06.3f" $d1 $d2 $d3]
    puts $fileId [format "INS.TARG2.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
    if { $corr_flag } {
        # Apply optical correction to reference targets
        set new_ref [calc_zemax_ref $lss_ref_ra $lss_ref_de]
        lassign [split $new_ref " "] a0 d0
        puts stdout "new REF COORDS : >$a0<  >$d0<"
        # Apply differential refraction
        if { $dire_flag } {
            set new_ref [calc_diff_refrac 0 $a0 $d0]
            lassign [split $new_ref " "] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
        }
        lassign [split $a0 :] a1 a2 a3
        lassign [split $d0 :] d1 d2 d3
    }
    append string_targ [format "INS.TARG2.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
    append string_targ [format "INS.TARG2.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]

    #-------------------------------
    #--- Write SEQ.PREP keywords ---
    #-------------------------------
    # Save also grism, OS filter and collimator
    #------------------------------------------
    puts $fileId [format "SEQ.PREP.LMIN  %8.3f" $L_MIN]
    puts $fileId [format "SEQ.PREP.LMAX  %8.3f" $L_MAX]
    puts $fileId [format "SEQ.PREP.GRIS  %10s" [$this cget -grism]]
    puts $fileId [format "SEQ.PREP.FILT  %10s" [$this cget -filter]]
    puts $fileId [format "SEQ.PREP.COLL  %s" [$this cget -collimat]]
    puts $fileId [format "SEQ.PREP.CAM   %s" [$this cget -camera]]
    append string_targ [format "SEQ.CCD  %s " [$this cget -camera]]
    close $fileId

    #-----------------------------------------------------------------------
    #--- Save p_... files to be used by the P2PP FORS instrument package ---
    #-----------------------------------------------------------------------
    # Save the same stuff again as one string to a
    # file to be called by p2pp
    set myfile $set_dir/$filename.p_targ
    #                             ------
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_targ
    close $fileId

    set myfile $set_dir/$filename.p_focf
    #                             ------
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_foc
    close $fileId

    # Save the same stuff again but in a more readable format
    # into the .fims/LOG directory (only for maintenance)
    set s2t [ldelete_all [split $string_targ " "]  ""]
    set number [llength $s2t]
    for { set i 0 } { $i < $number } { incr i 2 } {
        set s2t [lreplace $s2t $i $i [lrange $s2t $i [expr {$i + 1}]]]
    }
    for { set i [expr {$number - 1}] } { $i > 0 } { incr i -2 } {
        set s2t [lreplace $s2t $i $i]
    }
    set string2_targ [join $s2t "\n"]

    set myfile $log_dir/$filename.targ
    #                             ----
    set fileId [open $myfile w]
    puts $fileId $string2_targ
    close $fileId

    set s2f [ldelete_all [split $string_foc " "]  ""]
    set number [llength $s2f]
    for { set i 0 } { $i < $number } { incr i 2 } {
        set s2f [lreplace $s2f $i $i [lrange $s2f $i [expr {$i + 1}]]]
    }
    for { set i [expr {$number - 1}] } { $i > 0 } { incr i -2 } {
         set s2f [lreplace $s2f $i $i]
    }
    set string2_foc [join $s2f "\n"]

    set myfile $log_dir/$filename.focf
    #                             ----
    set fileId [open $myfile w]
    puts $fileId $string2_foc
    close $fileId
    incr exposure_counter 1
    tracer LEAVING
}

#=======================================
private method save_mos_TPX {filename} {
#=======================================
# open the *.obx file
# saving current parameters of one exposure
# modus either mos or pmos or echs
# we save 5 !! files
# <mode>.<expo_count>.<fits_name>.p_fims -> SET
# <mode>.<expo_count>.<fits_name>.p_fims -> SET
# <mode>.<expo_count>.<fits_name>.fims   -> SET
# <mode>.<expo_count>.<fits_name>.targ   -> LOG
# <mode>.<expo_count>.<fits_name>.focf   -> LOG
# .fims contains all specified keywords, its the
#       file to be loaded when [Load Mask]
# .focf contains only the INS.FOC.SETUP keyword in one line
#       to be inserted in an obd file (e.g. via emacs)
# .targ contains only the INS.TARG.SETUP keyword ...
# .p_fof contains only the value of the INS.FOC.SETUP
#         keyword to be inserted to an obx file by p2pp
# .p_targ contains only the value of the INS.TARG.SETUP keyword ...
# 18/01/99 mos-> mos1 or mos2
# save results as one long string for the p_focf and p_targ

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Focal and target setup as one long string
    set string_foc  ""
    set string_targ ""

    # Apply optical correction if FITS frame from FORS
    set corr_flag [$this cget -fors_frame]

    # Ask if slit 10 for alignment
    #
    # set slit_flag 0
    # set w [DialogWidget .corr \
    #           -text "MOS 10 slit alignment : [C_ref_targ cget -mos10ali]" \
    #           -bitmap warning \
    #           -buttons {ON OFF}]
    # if { [$w activate] } {
    #     puts stdout "OFF"
    #     set slit_flag 0
    #     C_ref_targ configure -mos10ali OFF
    # } else {
    #     # puts stdout "ON"
    #     set slit_flag 1
    #     C_ref_targ configure -mos10ali ON
    # }

    # # for status panel
    # set sp_mtali [C_ref_targ cget -mos10ali]
    # catch {SP update}

    #-------------
    # .fims header
    #-------------
    # Open .fims setup file and write header
    set myfile "$set_dir/$filename.fims"
    puts stdout  "# orig filename  : $myfile"
    set fileId [open $myfile w]
    set header [format $file_header fims]
    puts $fileId $header
    # Log file entry
    puts $lfId "FORS::save_mos_TPX:  $fileId $myfile"

    # Particular Pipeline issue
    append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s " \
                              $filename $s_v [lindex $f_v 0] \
                      ]
    append string_targ [format "INS.FIMS.NAME %-1s.p_targ_%1s_%-1s " \
                               $filename $s_v [lindex $f_v 0] \
                       ]

    #-------------------
    #-------- TCS ------
    #-------------------
    if { [catch { $m_image convert coords $world_x $world_y "deg $EQX" dummy_x dummy_y "wcs $EQX" } result] } {
        if { $debug && $verbose } {
            puts stdout "save_mos_TPX: (world_x,world_y)"
            puts stdout "              Error converting ($world_x,$world_y) \[deg\] to (dummy_x,dummy_y) WCS coord."
            puts stdout "              $result"
        }
        set dummy_x 99:99:99.99
        set dummy_y 99:99:99.99
    }

    lassign [split $dummy_x :] d1 d2 d3
    puts $fileId [format "TEL.TARG.ALPHA    %02s%02s%06.3f" $d1 $d2 $d3]
    lassign [split $dummy_y :] d1 d2 d3
    puts $fileId [format "TEL.TARG.DELTA    %02s%02s%06.3f" $d1 $d2 $d3]

    if { $corr_flag && [string match echs [$this cget -modus]] } {
        # Used corrected pointing for ECHS on fors images
        lassign [split $targw(2,ca) :] d1 d2 d3
        append string_foc  [format " INS.FOCF.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
        append string_targ [format " TEL.TARG.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
        lassign [split $targw(2,cd) :] d1 d2 d3
        append string_foc  [format " INS.FOCF.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
        append string_targ [format " TEL.TARG.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    } else {
        # Use real pointing for MOS
        $m_image convert coords $world_x $world_y "deg $EQX" \
                                dummy_x dummy_y "wcs $EQX"
        lassign [split $dummy_x :] d1 d2 d3
        append string_foc  [format " INS.FOCF.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
        append string_targ [format " TEL.TARG.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
        lassign [split $dummy_y :] d1 d2 d3
        append string_foc  [format " INS.FOCF.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
        append string_targ [format " TEL.TARG.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    }

    puts $fileId [format "TEL.ROT.OFFANGLE  %6.3f" [expr {$PAM * (-1)}]]
    append string_foc  [format " INS.FOCF.OFFANGLE  %6.3f " [expr {$PAM * (-1)}]]
    append string_targ [format " TEL.ROT.OFFANGLE  %6.3f " [expr {$PAM * (-1)}]]
    # puts $fileId [format "TEL.TARG.EPOCH  %6.1f" $EQX]
    # append string_targ [format " TEL.TARG.EPOCH  %6.1f " $EQX]
    puts $fileId [format "TEL.TARG.EQUINOX  %6.1f " $EQX]
    puts $fileId [format " TEL.TARG.TYPE  COORDINATE "]
    append string_targ [format " TEL.TARG.EQUINOX  %6.1f " $EQX]
    append string_targ [format " TEL.TARG.TYPE  COORDINATE "]

    append string_targ [format " INS.COLL.NAID  %s " [$this cget -collimat]]

    #---------------------
    #--- MOS slit loop ---
    #---------------------
    for { set i 1} {$i < 20 } { incr i 1 } {
        # targw($i,f) is either
        # 0 = not even moved
        # 1 = moved, did not found entry in AstroCat widget
        # string = the target id of  AstroCat local_list
        # 3a target name
        #
        if { [string compare $targw($i,f) "0"] == 0 } {
            # do nothing
            # slit is not defined
        } elseif { [string compare $targw($i,f) "1"] == 0 } {
            puts $fileId [format "INS.TARG$i.NAME  %19s" NN]
            append string_foc [format " INS.TARG$i.NAME  %19s " NN]
        } else {
            set targw($i,f) [string_exchange $targw($i,f) " " "_"]
            puts $fileId [format "INS.TARG$i.NAME  %19s" $targw($i,f)]
            append string_foc [format " INS.TARG$i.NAME  %19s " $targw($i,f)]
        }
        # 3b MOS slit width
        #
        if { [string compare $targw($i,f) "0"] } {
            # only f!=0 targets should contain valid a,d
            puts $fileId [format "INS.TARG$i.WID  %3.1f" $mos($i,b)]
            append string_foc [format " INS.TARG$i.WID  %3.1f " $mos($i,b)]
            # --------------------------------------------
            # 08/04/99 INS.MOS$i.WID  -->  INS.MOS$i.WIDTH
            # --------------------------------------------
            # puts $fileId [format "INS.TARG$i.WIDTH  %3.1f" $mos($i,b)]
            # append string_foc [format " INS.TARG$i.WIDTH  %3.1f " $mos($i,b)]
            # 3c target position inside the MOS slit
            #
            # puts stdout "$targw($i,a) $targw($i,d) $targw($i,f)"
            # save uncorrected values to .fims file to avoid
            # iterative optical correction when load/save/load/save ..
            lassign [split $targw($i,d) :] d1 d2 d3
            lassign [split $targw($i,a) :] a1 a2 a3
            # puts stdout  [format "INS.TARG$i.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
            puts $fileId [format "INS.TARG$i.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
            # puts stdout [format "INS.TARG$i.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
            puts $fileId [format "INS.TARG$i.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
            if { $corr_flag } {
                lassign [split $targw($i,ca) :] a1 a2 a3
                lassign [split $targw($i,cd) :] d1 d2 d3
            }
            append string_foc [format " INS.TARG$i.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
            append string_foc [format " INS.TARG$i.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
            # save targets in CCD-pixel format
            # if { $corr_flag } {
            #     puts stdout  [format "INS.TARG$i.X  %05.3f" $targw($i,cx)]
            #     puts $fileId [format "INS.TARG$i.X  %05.3f" $targw($i,cx)]
            # } else {
            #      uts stdout  [format "INS.TARG$i.X  %05.3f" $targw($i,x)]
            #      uts $fileId [format "INS.TARG$i.X  %05.3f" $targw($i,x)]
            # }
            # puts stdout  [format "INS.TARG$i.Y  %05.3f" $targw($i,y)]
            # puts $fileId [format "INS.TARG$i.Y  %05.3f" $targw($i,y)]
            # if { $corr_flag } {
            #     puts stdout [format "INS.TARG$i.ALPHA  %02s%02s%06.3f \
            #                          INS.TARG$i.DELTA  %02s%02s%06.3f \
            #                          INS.TARG$i.X  %05.3f \
            #                          INS.TARG$i.Y  %05.3f" \
            #                          $a1 $a2 $a3 $d1 $d2 $d3 $targw($i,cx) $targw($i,y)]
            # } else {
            #     puts stdout [format "INS.TARG$i.ALPHA  %02s%02s%06.3f \
            #                          INS.TARG$i.DELTA  %02s%02s%06.3f \
            #                          INS.TARG$i.X  %05.3f \
            #                          INS.TARG$i.Y  %05.3f" \
            #                          $a1 $a2 $a3 $d1 $d2 $d3 $targw($i,x) $targw($i,y)]
            # }
        }
    }

    #-------------------------------
    #--- Slit number association ---
    #-------------------------------
    for { set j 1 } { $j < 20 } { incr j 1 } {
        # assign only slits, which have been moved
        if { [string compare $targw($j,f) "0"] } {
            puts $fileId [format "INS.MOS$j.TARG  %3d" $j]
            append string_foc [format " INS.MOS$j.TARG  %3d " $j]
        }
    }

    #----------------------------
    # Reference objects are known
    #----------------------------
    # Total numer of ref objects
    puts stdout "[C_ref_targ nort] Reference Objects selected"

    # Reference Objects already in correct coords
    # Save reference objects to .fims setup file
    for { set j 1 } { $j <= [C_ref_targ nort] } { incr j 1 } {
        set z [expr {$j + 30}]; # the INS.TARG$z. number
        lassign [split [C_ref_targ get $j a] :] a1 a2 a3
        # puts stdout [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3]
        puts $fileId [format "INS.TARG$z.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]

        lassign [split [C_ref_targ get $j d] :] d1 d2 d3
        # puts stdout [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
        puts $fileId [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

        if { $corr_flag } {
            # correct reference targets for zemax and diff refrac
            set new_ref [calc_zemax_ref [C_ref_targ get $j a] [C_ref_targ get $j d]]
            lassign [split $new_ref " "] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
            # apply differential refraction
            if { $dire_flag } {
                set new_ref [calc_diff_refrac 0 $a0 $d0]
                lassign [split $new_ref " "] a0 d0
                puts stdout "new REF COORDS : >$a0<  >$d0<"
            }
            lassign [split $a0 :] a1 a2 a3
            lassign [split $d0 :] d1 d2 d3
        }
        append string_targ [format " INS.TARG$z.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
        append string_targ [format " INS.TARG$z.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    }

    #----------------------
    # Reference targets end
    #----------------------
    if { [C_ref_targ cget -mos10ali] } {
        if { [C_ref_targ nort] > 0 } {
            # if there is an alignment specify alignment slit
            append string_targ " INS.MOS10.POS  0.000  INS.MOS10.WID  0.5 "
            # --------------------------------------------
            # 08/04/99 INS.MOS$i.WID  -->  INS.MOS$i.WIDTH
            # --------------------------------------------
            # append string_targ " INS.MOS10.POS  0.000  INS.MOS10.WIDTH  0.5 "
        }
    }

    #-------------------------------
    #--- Write SEQ.PREP keywords ---
    #-------------------------------
    # Save also grism, OS filter and collimator
    #------------------------------------------
    puts $fileId [format "SEQ.PREP.LMIN   %8.3f" $L_MIN]
    puts $fileId [format "SEQ.PREP.LMAX   %8.3f" $L_MAX]
    puts $fileId [format "SEQ.PREP.GRIS   %s" [$this cget -grism]]
    puts $fileId [format "SEQ.PREP.FILT   %s" [$this cget -filter]]
    puts $fileId [format "SEQ.PREP.MTALI  %s" [C_ref_targ cget -mos10ali]]
    puts $fileId [format "SEQ.PREP.COLL   %s" [$this cget -collimat]]
    puts $fileId [format "SEQ.PREP.CAM    %s" [$this cget -camera]]
    append string_targ [format " SEQ.CCD  %s " [$this cget -camera]]
    if { [string match echs [$this cget -modus]] } {
        puts $fileId [format "SEQ.PREP.ECHS9  %1d" $echs_flag]
    }
    close $fileId

    #-----------------------------------------------------------------------
    #--- Save p_... files to be used by the P2PP FORS instrument package ---
    #-----------------------------------------------------------------------
    # Save the same stuff again as one string to a
    # file to be called by P2PP tool
    set myfile $set_dir/$filename.p_targ
    #                             ------
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_targ
    close $fileId

    set myfile $set_dir/$filename.p_focf
    #                             ------
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_foc
    close $fileId

    # Save the same stuff again but in a more readable format
    # into the .fims/LOG directory (only for maintenance)
    set s2t [ldelete_all [split $string_targ " "]  ""]
    set number [llength $s2t]
    for { set i 0 } { $i < $number } { incr i 2 } {
        set s2t [lreplace $s2t $i $i [lrange $s2t $i [expr {$i + 1}]]]
    }
    for { set i [expr {$number - 1}] } { $i > 0 } { incr i -2 } {
        set s2t [lreplace $s2t $i $i]
    }
    set string2_targ [join $s2t "\n"]

    set myfile $log_dir/$filename.targ
    #                             ----
    set fileId [open $myfile w]
    puts $fileId $string2_targ
    close $fileId

    set s2f [ldelete_all [split $string_foc " "]  ""]
    set number [llength $s2f]
    for { set i 0 } { $i < $number } { incr i 2 } {
        set s2f [lreplace $s2f $i $i [lrange $s2f $i [expr {$i + 1}]]]
    }
    for { set i [expr {$number - 1}] } { $i > 0 } { incr i -2 } {
        set s2f [lreplace $s2f $i $i]
    }
    set string2_foc [join $s2f "\n"]
    set myfile $log_dir/$filename.focf
    #                             ----
    set fileId [open $myfile w]
    puts $fileId $string2_foc
    close $fileId
    incr exposure_counter 1
    tracer LEAVING
}

#=======================================
private method save_mxu_TPX {filename} {
#=======================================
# open the *.obx file
# saving current parameters of one exposure
# modus mxu
# some changes of the output files after
# VIC-ESO meeting in 12/99
# we save 5 !! files
# <mode>.<expo_count>.<fits_name>.fims   -> SET
# <mode>.<expo_count>.<fits_name>.targ   -> LOG
# <mode>.<expo_count>.<fits_name>.focf   -> LOG
# <mode>.<expo_count>.<fits_name>.p_targ -> SET (PAF header file)
# <mode>.<expo_count>.<fits_name>.p_focf -> SET (PAF header file)
# .fims contains all specified keywords, its the
#       file to be loaded when [Load Mask]
# .focf contains only the INS.FOC.SETUP keyword in one line
#       to be inserted in an obd file (e.g. via emacs)
# .targ contains only the INS.TARG.SETUP keyword ...
#  and the INS.MSKREADY.NAID keyword
# .p_focf contains only the value of the INS.FOC.SETUP
#         keyword to be inserted to an obx file by p2pp
#         and the INS.MASK.NAID
# .p_targ contains only the value of the INS.TARG.SETUP keyword ...
# 18/01/99 mos-> mos1 or mos2
# save results as one long string

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Focal and target setup as one long string
    set string_foc  ""
    set string_targ ""

    # Before we do anything we show the status of all mxu slits
    puts stdout "\n\nsave_mxu_TPX"
    puts stdout "save_mxu_TPX: >[fors::MXU_SLIT::get_MASK_NAID]<"
    set noms [fors::MXU_SLIT::nos]
    if { $noms > 0 } {
        for { set i 1 } { $i <= $noms } { incr i 1 } {
             if { [C_mxus$i cget -ON] } {
                 puts stdout [format "#%2d %1d %14s %5.2f %5.2f %5.2f %5.2f %8s %8s %8s %8s %8s" \
                                      $i \
                                      [C_mxus$i cget -ON         ] \
                                      [C_mxus$i cget -target_name] \
                                      [C_mxus$i cget -slit_width ] \
                                      [C_mxus$i cget -slit_length] \
                                      [C_mxus$i cget -slit_offset] \
                                      [C_mxus$i cget -pos_ang    ] \
                                      [C_mxus$i cget -shape      ] \
                                      [C_mxus$i get_RA           ] \
                                      [C_mxus$i get_DEC          ] \
                                      [C_mxus$i get_OC_RA        ] \
                                      [C_mxus$i get_OC_DEC       ] \
                             ]
             }
        }
    }

    set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]

    # Apply optical correction if fits-image from FORS
    set corr_flag [$this cget -fors_frame]

    # workaround 13. Sept 2000 WH
    # FORS1 images are aliens for MXU
    # if { [string compare [$m_image fits get INSTRUME] "FORS1"] == 0 } {
    #     set corr_flag 0
    # }

    #-------------
    # .fims header
    #-------------
    # Open .fims setup file and write header
    set myfile "$set_dir/$filename.fims"
    puts stdout   "# orig filename  : $myfile"
    set fileId [open $myfile w]
    set header [format $file_header fims]
    puts $fileId $header
    # Log file entry
    puts $lfId "FORS::save_mxu_TPX:  $fileId $myfile"

    # Technical mask flag
    set tm [fors::MXU_SLIT::get_mmff]
    # Science mask flag
    set sm [expr ! $tm]

    if { $sm } {
        # We copy the INS.FIMS.NAME again to find it as the first chars in KE
        # java-pippi
        # particular Pipeline issue
        append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s\n" \
                                  $filename $s_v [lindex $f_v 0] \
                          ]
        append string_targ [format "INS.FIMS.NAME  %-1s.p_targ_%1s_%-1s\n" \
                                   $filename $s_v [lindex $f_v 0] \
                           ]

        # PAF header for p_targ file
        # --------------------------
        # the mxu2.p_targ differs from all other p_targ files of the
        # other FORS instrumental modi
        append string_targ [format $paf_header targ MXU]
        append string_targ $header

        # PAF header for p_focf file
        # --------------------------
        # the mxu2.p_focf differs from all other p_focf files of the
        # other FORS instrumental modi
        append string_foc [format $paf_header focf MXU]
        append string_foc $header

        append string_foc  "TPL.FILE.DIRNAME  \$INS_ROOT/SYSTEM/COMMON/SETUPFILES/INS "
        append string_targ "TPL.FILE.DIRNAME  \$INS_ROOT/SYSTEM/COMMON/SETUPFILES/INS "
        append string_foc  "\n"
        append string_targ "\n"
    }

    # Particular Pipeline issue
    append string_foc [format "INS.FIMS.NAME  %-1s.p_focf_%1s_%-1s " \
                              $filename $s_v [lindex $f_v 0] \
                      ]
    append string_targ [format "INS.FIMS.NAME  %-1s.p_targ_%1s_%-1s " \
                               $filename $s_v [lindex $f_v 0] \
                       ]

    if { $sm } {
        append string_foc "\n"
        append string_targ "\n"
    }
    # What does this mean? Technical mask p_focf p_targ consist of one string,
    # sky masks follow the PAF convention

    #-------------------
    #-------- TCS ------
    #-------------------
    if { [catch { $m_image convert coords $world_x $world_y "deg $EQX" dummy_x dummy_y "wcs $EQX" } result] } {
        if { $debug && $verbose } {
            puts stdout "save_mxu_TPX: (world_x,world_y)"
            puts stdout "              Error converting ($world_x,$world_y) \[deg\] to (dummy_x,dummy_y) WCS coord."
            puts stdout "              $result"
        }
        set dummy_x 99:99:99.99
        set dummy_y 99:99:99.99
    }

    lassign [split $dummy_x :] d1 d2 d3
    puts $fileId        [format "TEL.TARG.ALPHA  %02s%02s%06.3f" $d1 $d2 $d3]
    append string_foc   [format "INS.FOCF.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
    append string_targ  [format "TEL.TARG.ALPHA  %02s%02s%06.3f " $d1 $d2 $d3]
    if { $sm } {
        append string_foc "\n"
        append string_targ "\n"
    }
    lassign [split $dummy_y :] d1 d2 d3
    puts $fileId        [format "TEL.TARG.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
    append string_foc   [format "INS.FOCF.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    append string_targ  [format "TEL.TARG.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
    if { $sm } {
        append string_foc "\n"
        append string_targ "\n"
    }
    puts $fileId        [format "TEL.ROT.OFFANGLE  %6.3f" [expr {$PAM * (-1)}]]
    append string_foc   [format "INS.FOCF.OFFANGLE  %6.3f " [expr {$PAM * (-1)}]]
    append string_targ  [format "TEL.ROT.OFFANGLE  %6.3f " [expr {$PAM * (-1)}]]
    if { $sm } {
        append string_foc "\n"
        append string_targ "\n"
    }
    # puts $fileId        [format "TEL.TARG.EPOCH  %6.1f" $EQX]
    # append string_targ  [format "TEL.TARG.EPOCH  %6.1f " $EQX]
    puts $fileId        [format "TEL.TARG.EQUINOX  %6.1f" $EQX]
    append string_targ  [format "TEL.TARG.EQUINOX  %6.1f " $EQX]
    if { $sm } {
        append string_targ "\n"
    }
    puts $fileId        [format "TEL.TARG.TYPE  COORDINATE"]
    append string_targ  [format "TEL.TARG.TYPE  COORDINATE "]
    if { $sm } {
        append string_targ "\n"
    }

    append string_targ [format "INS.COLL.NAID  %s " [$this cget -collimat]]
    if { $sm } {
        append string_targ "\n"
    }

    # User-defined program ID and MASK.NAID
    #
    # if { [lindex $PM 1] > 900000 }
    append string_foc  "INS.MASK.NAID  [lindex $PM 0]+[lindex $PM 1] "
    append string_targ "INS.MSKREADY.NAID  [lindex $PM 0]+[lindex $PM 1] "
    if { $sm } {
        append string_foc  "\n"
        append string_targ "\n"
    }

    # Save number of slits
    set noms [fors::MXU_SLIT::nos]
    set s_c [$this noas]
    set sp_rsc [$this noars]
    puts $fileId [format "INS.SLIT.NUMBER  %d" $s_c]

    # If no reference slits are defined by the user,
    # take the first (up to) ten
    if { $sp_rsc == 0 } {
        # No reference slit specified
        puts stdout "$sp_rsc reference slits specified by user"
        set r_c 0
        for { set i 1 } { $i <= $noms } { incr i 1 } {
            if { $r_c < 10 } {
                if { [C_mxus$i cget -ON] } {
                    if { [string match *STRAIGHT* [C_mxus$i cget -shape]] } {
                        incr r_c 1
                        C_mxus$i configure -ref_slit 1
                    }
                }
            }
        }
	set sp_rsc $r_c
        puts stdout "$sp_rsc reference slits specified by FIMS"
    }
    catch {SP update}
    puts $fileId [format "INS.RSLIT.NUMBER  %d" $sp_rsc]

    # ---------------------
    # --- MXU slit loop ---
    # ---------------------
    set s_c 100;   # science slit counter offset ?
    set r_c 1;     # reference slit counter offset
    if { $noms > 0 } {
        for { set i 1 } { $i <= $noms } { incr i 1 } {
            # if slit is specified
            if { [C_mxus$i cget -ON] } {
                incr s_c 1
                # -1----------- NAME
                puts $fileId [format "INS.TARG$s_c.NAME  %19s" [C_mxus$i cget -target_name]]
                # puts stdout [format "INS.TARG$s_c.NAME  %19s" [C_mxus$i cget -target_name]]
                # to appear later in the fits file header of the FORS2 image
                # append string_targ [format "INS.TARG$s_c.NAME  %19s\n" [C_mxus$i cget -target_name]]
                if { $sm } {
                    # Empty line to separate target keyword groups
                    append string_foc "\n"
                }
                append string_foc [format "INS.TARG$s_c.NAME  %19s " [C_mxus$i cget -target_name]]
                if { $sm } {
                    append string_foc "\n"
                }

                # -2----------- Reference slits
                if { [C_mxus$i cget -ref_slit] } {
	            # append string_targ [format "INS.SLIT%1d.REF  %1d\n" $r_c $s_c]
	            append string_foc [format "INS.SLIT%1d.REF  %1d " $r_c $s_c]
                    if { $sm } {
                        append string_foc "\n"
                    }
	            puts $fileId [format "INS.SLIT%1d.REF  %1d" $r_c $s_c]
                    incr r_c 1
                }

                # -3----------- SLIT DIMENSIONS
                set cu_width  [C_mxus$i cget -slit_width ]
                set cu_length [C_mxus$i cget -slit_length]
                set cu_offset [C_mxus$i cget -slit_offset]
                set cu_posang [C_mxus$i cget -pos_ang    ]
                set cu_shape  [C_mxus$i cget -shape      ]

                # puts stdout [format "INS.TARG$s_c.SHAPE  %19s"  $cu_shape]
                puts $fileId [format "INS.TARG$s_c.SHAPE  %19s"  $cu_shape]
                # puts stdout [format "INS.TARG$s_c.WID  %3.1f" $cu_width]
                puts $fileId [format "INS.TARG$s_c.WID  %3.1f" $cu_width]
                # --------------------------------------------
                # 08/04/99 INS.MOS$i.WID  -->  INS.MOS$i.WIDTH
                # --------------------------------------------
                # puts stdout [format "INS.TARG$s_c.WIDTH  %3.1f" $cu_width]
                # puts $fileId [format "INS.TARG$s_c.WIDTH  %3.1f" $cu_width]

                # puts stdout [format "INS.TARG$s_c.LEN    %3.1f" $cu_length]
                puts $fileId [format "INS.TARG$s_c.LEN  %3.1f" $cu_length]

                # slit position angle only for STRAIGHT
                if { [string match *STRAIGHT* $cu_shape] } {
                    # puts stdout [format "INS.TARG$s_c.ROT  %3.1f" $cu_posang]
                    puts $fileId [format "INS.TARG$s_c.ROT  %3.1f" $cu_posang]
                }

                # slit offset only for STRAIGHT
                if { [string match *STRAIGHT* $cu_shape] } {
                    # puts stdout  [format "INS.TARG$s_c.OFF  %3.1f" $cu_offset]
                    # PBG: Slit offset deactivated
                    # PBG: The keyword in this file allows fims to redraw the slit offset
                    # PBG: puts $fileId [format "INS.TARG$s_c.OFF  %3.1f" $cu_offset]
                }

                append string_foc  [format "INS.TARG$s_c.SHAPE  %19s "  $cu_shape]
                if { $sm } {
                    append string_foc "\n"
                }
                append string_foc  [format "INS.TARG$s_c.WID  %3.1f " $cu_width]
                if { $sm } {
                    append string_foc "\n"
                }
                append string_foc  [format "INS.TARG$s_c.LEN  %3.1f " $cu_length]
                if { $sm } {
                    append string_foc "\n"
                }
                # append string_targ [format "INS.TARG$s_c.SHAPE  %19s\n"  $cu_shape]
                # append string_targ [format "INS.TARG$s_c.WID  %3.1f\n" $cu_width]
                # append string_targ [format "INS.TARG$s_c.LEN  %3.1f\n" $cu_length]

                # --------------------------------------------
                # 08/04/99 INS.MOS$i.WID  -->  INS.MOS$i.WIDTH
                # --------------------------------------------
                # append string_targ [format "INS.TARG$s_c.WIDTH  %3.1f\n" $cu_width]
                # append string_foc  [format "INS.TARG$s_c.WIDTH  %3.1f\n" $cu_width]

                # slit position angle only for STRAIGHT
                if { [string match *STRAIGHT* $cu_shape] } {
                    # append string_targ [format "INS.TARG$s_c.ROT  %3.1f\n" $cu_posang]
                    append string_foc  [format "INS.TARG$s_c.ROT  %3.1f " $cu_posang]
                    if { $sm } {
                        append string_foc "\n"
                    }
                }

                # slit offset only for STRAIGHT
                if { [string match *STRAIGHT* $cu_shape] } {
                    # append string_targ [format "INS.TARG$s_c.OFF  %3.1f\n" $cu_offset]
                    # PBG: Slit offset deactivated
                    # PBG: Only when FORS2 understands the setup of this keyword: 
                    # PBG:  append string_foc  [format "INS.TARG$s_c.OFF  %3.1f " $cu_offset]
                    # if { $sm } {
                    #     append string_foc "\n"
                    # }
                }

                # 3 bezier points for slit shape = CURVED
                if { [string match *CURVED* $cu_shape] } {
                    set bez_list [C_mxus$i get_curve_mm]
                    # the middle slit curve
                    set x1 [expr {([lindex $bez_list 0] + [lindex $bez_list  6]) * 0.5}]
                    set y1 [expr {([lindex $bez_list 1] + [lindex $bez_list  7]) * 0.5}]
                    set x2 [expr {([lindex $bez_list 2] + [lindex $bez_list  8]) * 0.5}]
                    set y2 [expr {([lindex $bez_list 3] + [lindex $bez_list  9]) * 0.5}]
                    set x3 [expr {([lindex $bez_list 4] + [lindex $bez_list 10]) * 0.5}]
                    set y3 [expr {([lindex $bez_list 5] + [lindex $bez_list 11]) * 0.5}]

                    puts $fileId [format "INS.TARG$s_c.X1   %3.1f" $x1]
                    puts $fileId [format "INS.TARG$s_c.Y1   %3.1f" $y1]
                    puts $fileId [format "INS.TARG$s_c.X2   %3.1f" $x2]
                    puts $fileId [format "INS.TARG$s_c.Y2   %3.1f" $y2]
                    puts $fileId [format "INS.TARG$s_c.X3   %3.1f" $x3]
                    puts $fileId [format "INS.TARG$s_c.Y3   %3.1f" $y3]

                    # append string_targ [format "INS.TARG$s_c.X1   %3.1f\n" $x1]
                    # append string_targ [format "INS.TARG$s_c.Y1   %3.1f\n" $y1]
                    # append string_targ [format "INS.TARG$s_c.X2   %3.1f\n" $x2]
                    # append string_targ [format "INS.TARG$s_c.Y2   %3.1f\n" $y2]
                    # append string_targ [format "INS.TARG$s_c.X3   %3.1f\n" $x3]
                    # append string_targ [format "INS.TARG$s_c.Y3   %3.1f\n" $y3]

                    append string_foc  [format "INS.TARG$s_c.X1   %3.1f " $x1]
                    if { $sm } {
                        append string_foc "\n"
                    }
                    append string_foc  [format "INS.TARG$s_c.Y1   %3.1f " $y1]
                    if { $sm } {
                        append string_foc "\n"
                    }
                    append string_foc  [format "INS.TARG$s_c.X2   %3.1f " $x2]
                    if { $sm } {
                        append string_foc "\n"
                    }
                    append string_foc  [format "INS.TARG$s_c.Y2   %3.1f " $y2]
                    if { $sm } {
                        append string_foc "\n"
                    }
                    append string_foc  [format "INS.TARG$s_c.X3   %3.1f " $x3]
                    if { $sm } {
                        append string_foc "\n"
                    }
                    append string_foc  [format "INS.TARG$s_c.Y3   %3.1f " $y3]
                    if { $sm } {
                        append string_foc "\n"
                    }
                }
                # curved slits will not be used for the alignment

                # puts stdout "$targw($i,a) $targw($i,d) $targw($i,f)"
                # save uncorrected values to .fims file to avoid
                # iterative optical correction when load/save/load/save ..
                lassign [split [C_mxus$i get_RA ] :] a1 a2 a3
                lassign [split [C_mxus$i get_DEC] :] d1 d2 d3
                # puts stdout [format "INS.TARG$s_c.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
                puts $fileId [format "INS.TARG$s_c.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]
                # puts stdout  [format "INS.TARG$s_c.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
                puts $fileId [format "INS.TARG$s_c.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

                if { $corr_flag } {
                    lassign [split [C_mxus$i get_OC_RA ] :] a1 a2 a3
                    lassign [split [C_mxus$i get_OC_DEC] :] d1 d2 d3
                }
                # append string_targ [format "INS.TARG$s_c.ALPHA  %02s%02s%06.3f\n" $a1 $a2 $a3]
                # append string_targ [format "INS.TARG$s_c.DELTA  %02s%02s%06.3f\n" $d1 $d2 $d3]
                append string_foc  [format "INS.TARG$s_c.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
                if { $sm } {
                    append string_foc "\n"
                }
                append string_foc  [format "INS.TARG$s_c.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
                if { $sm } {
                    append string_foc "\n"
                }

                # save targets in CCD-pixel format
                # if { $corr_flag } {
                #     puts stdout  [format "INS.TARG$s_c.X   %05.3f" $targw($i,cx)]
                #     puts $fileId [format "INS.TARG$s_c.X   %05.3f" $targw($i,cx)]
                # } else {
                #     puts stdout  [format "INS.TARG$s_c.X   %05.3f" $targw($i,x)]
                #     puts $fileId [format "INS.TARG$s_c.X   %05.3f" $targw($i,x)]
                # }
                # puts stdout  [format "INS.TARG$s_c.Y   %05.3f" $targw($i,y)]
                # puts $fileId [format "INS.TARG$s_c.Y   %05.3f" $targw($i,y)]

                # if { $corr_flag } {
                #     puts stdout [format "INS.TARG$i.ALPHA  %02s%02s%06.3f \
                #                          INS.TARG$i.DELTA  %02s%02s%06.3f \
                #                          INS.TARG$i.X  %05.3f \
                #                          INS.TARG$i.Y  %05.3f" \
                #                          $a1 $a2 $a3 $d1 $d2 $d3 $targw($i,cx) $targw($i,y) \
                #                 ]
                # } else {
                #     puts stdout [format "INS.TARG$i.ALPHA  %02s%02s%06.3f \
                #                          INS.TARG$i.DELTA  %02s%02s%06.3f \
                #                          INS.TARG$i.X  %05.3f  \
                #                          INS.TARG$i.Y  %05.3f" \
                #                          $a1 $a2 $a3 $d1 $d2 $d3 $targw($i,x) $targw($i,y) \
                #                 ]
                # }
            }
        }
    }

    #----------------------------
    # Reference objects are known
    #----------------------------
    # Total numer of ref objects
    puts stdout "[C_ref_targ nort] Reference Objects selected"

    # Reference Objects already in correct coords
    # Save reference objects to setup file
    for { set j 1 } { $j <= [C_ref_targ nort] } { incr j 1 } {
        set z [expr {$j + 30}]; # the INS.TARG$z. number

        lassign [split [C_ref_targ get $j a] :] a1 a2 a3
        # puts stdout  [format "INS.TARG$z.ALPHA   %02s%02s%06.3f" $a1 $a2 $a3]
        puts $fileId [format "INS.TARG$z.ALPHA  %02s%02s%06.3f" $a1 $a2 $a3]

        lassign [split [C_ref_targ get $j d] :] d1 d2 d3
        # puts stdout  [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]
        puts $fileId [format "INS.TARG$z.DELTA  %02s%02s%06.3f" $d1 $d2 $d3]

        if { $corr_flag } {
            set new_ref [calc_zemax_ref [C_ref_targ get $j a] [C_ref_targ get $j d]]
            lassign [split $new_ref " "] a0 d0
            puts stdout "new REF COORDS : >$a0<  >$d0<"
            # apply differential refraction
            if { $dire_flag } {
                set new_ref [calc_diff_refrac 0 $a0 $d0]
                lassign [split $new_ref " "] a0 d0
                puts stdout "new REF COORDS : >$a0<  >$d0<"
            }
            lassign [split $a0 :] a1 a2 a3
            lassign [split $d0 :] d1 d2 d3
        }
        append string_targ [format "INS.TARG$z.ALPHA  %02s%02s%06.3f " $a1 $a2 $a3]
        if { $sm } {
            append string_targ "\n"
        }
        append string_targ [format "INS.TARG$z.DELTA  %02s%02s%06.3f " $d1 $d2 $d3]
        if { $sm } {
            append string_targ "\n"
        }
        # append string_foc  [format "INS.TARG$z.ALPHA  %02s%02s%06.3f\n" $a1 $a2 $a3]
        # append string_foc  [format "INS.TARG$z.DELTA  %02s%02s%06.3f\n" $d1 $d2 $d3]
    }

    #----------------------
    # Reference targets end
    #----------------------

    #-------------------------------
    #--- Write SEQ.PREP keywords ---
    #-------------------------------
    # save also grism, OS filter and collimator
    puts $fileId [format "SEQ.PREP.LMIN   %8.3f" $L_MIN]
    puts $fileId [format "SEQ.PREP.LMAX   %8.3f" $L_MAX]
    puts $fileId [format "SEQ.PREP.GRIS   %s" [$this cget -grism]]
    puts $fileId [format "SEQ.PREP.FILT   %s" [$this cget -filter]]
    puts $fileId [format "SEQ.PREP.MASK_NA  %s" [lindex $PM 0]]
    puts $fileId [format "SEQ.PREP.MASK_ID  %s" [lindex $PM 1]]
    puts $fileId [format "SEQ.PREP.COLL  %s" [$this cget -collimat]]
    puts $fileId [format "SEQ.PREP.CAM  %s" [$this cget -camera]]
    append string_targ [format "SEQ.CCD  %s " [$this cget -camera]]
    if { $sm } {
        append string_targ "\n"
    }

    close $fileId

    #-----------------------------------------------------------------------
    #--- Save p_... files to be used by the P2PP FORS instrument package ---
    #-----------------------------------------------------------------------
    # Save the same stuff again as one string to a
    # file to be called by the P2PP tool
    set myfile $set_dir/$filename.p_targ
    #                             ------
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_targ
    close $fileId
    catch { [exec pafchkCreate $myfile] } result

    set myfile $set_dir/$filename.p_focf
    #                             ------
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_foc
    close $fileId
    catch { [exec pafchkCreate $myfile] } result

    # Save the same stuff again but in a more readable format
    # into the .fims/LOG directory (only for maintenance)
    # set s2t [ldelete_all [split $string_targ " "]  ""]
    # set number [llength $s2t]
    # for { set i 0 } { $i < $number } { incr i 2 } {
    #     set s2t [lreplace $s2t $i $i [lrange $s2t $i [expr {$i + 1}]]]
    # }
    # for { set i [expr {$number - 1}] } { $i > 0 } { incr i -2 } {
    #     set s2t [lreplace $s2t $i $i]
    # }
    # set string2_targ [join $s2t "\n"]

    # set myfile $log_dir/$filename.targ
    #                               ----
    # set fileId [open $myfile w]
    # puts $fileId $string2_targ
    # close $fileId

    # set s2f [ldelete_all [split $string_foc " "]  ""]
    # set number [llength $s2f]
    # for { set i 0 } { $i < $number } { incr i 2 } {
    #     set s2f [lreplace $s2f $i $i [lrange $s2f $i [expr {$i + 1}]]]
    # }
    # for { set i [expr {$number - 1}] } { $i > 0 } { incr i -2 } {
    #     set s2f [lreplace $s2f $i $i]
    # }
    # set string2_foc [join $s2f "\n"]

    # set myfile $log_dir/$filename.focf
    #                               ----
    # set fileId [open $myfile w]
    # puts $fileId $string2_foc
    # close $fileId

    incr exposure_counter 1
    tracer LEAVING
    # select_file $myfile
}

#=======================================
private method prepare_mxu_for_save {} {
#=======================================
#  update slits of type CURVED

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set noms [fors::MXU_SLIT::nos]
    if { $noms > 0 } {
        for { set i 1 } { $i <= $noms } { incr i 1 } {
            if { [C_mxus$i cget -ON] } {
                if { [string match *CURVED* [C_mxus$i cget -shape]] } {
                    # we determine the curved slitcenter, apply correction of
                    # of optical distortion and save correct values to mxu slit
                    # this point has only meaning for DRS
                    set dummy [C_mxus$i update_curved_center]
                    if { [catch { $m_image convert coords [lindex $dummy 0] [lindex $dummy 1] canvas \
                                                          targw(1,x) targw(1,y) image \
                          } result] } {
                        if { $debug && $verbose } {
                            puts stdout "prepare_mxu_for_save: Curved slit $i center"
                            puts stdout "                      Error converting ([lindex $dummy 0],[lindex $dummy 1]) canvas to (targw(1,x),targw(1,y)) image coord."
                            puts stdout "                      $result"
                        }
                    } else {
                        set targw(1,a) [C_mxus$i get_RA]
                        set targw(1,d) [C_mxus$i get_DEC]
                        calc_zemax 1 $targw(1,x) $targw(1,y)
                        # targw(1,cx) and targw(1,cy) is not used
                        C_mxus$i put_OC  [list $targw(1,ca) $targw(1,cd)]
                        update_bez_oc $i
                    }
                    # update the three Bezier points
                    # get points in [canvas]
                    # set dummy [C_mxus$i get_curve_ca]
                    # set targw(1,a) [C_mxus$i get_RA]
                    # set targw(1,d) [C_mxus$i get_DEC]

                    # $m_image convert coords [lindex $dummy 0] [lindex $dummy 1] canvas \
                    #                         targw(1,x) targw(1,y) image
                    # calc_zemax 1 $targw(1,x) $targw(1,y)
                    # C_mxus$i put_b_oc 1 [list $targw(1,cx) $targw(1,cy)]
                    #
                    # $m_image convert coords [lindex $dummy 2] [lindex $dummy 3] canvas \
                    #                         targw(1,x) targw(1,y) image
                    # calc_zemax 1 $targw(1,x) $targw(1,y)
                    # C_mxus$i put_b_oc 2 [list $targw(1,cx) $targw(1,cy)]
                    #
                    # $m_image convert coords [lindex $dummy 4] [lindex $dummy 5] canvas \
                    #                         targw(1,x) targw(1,y) image
                    # calc_zemax 1 $targw(1,x) $targw(1,y)
                    # C_mxus$i put_b_oc 3 [list $targw(1,cx) $targw(1,cy)]
                    #
                    # $m_image convert coords [lindex $dummy 6] [lindex $dummy 7] canvas \
                    #                         targw(1,x) targw(1,y) image
                    # calc_zemax 1 $targw(1,x) $targw(1,y)
                    # C_mxus$i put_b_oc 4 [list $targw(1,cx) $targw(1,cy)]
                    #
                    # $m_image convert coords [lindex $dummy 8] [lindex $dummy 9] canvas \
                    #                         targw(1,x) targw(1,y) image
                    # calc_zemax 1 $targw(1,x) $targw(1,y)
                    # C_mxus$i put_b_oc 5 [list $targw(1,cx) $targw(1,cy)]
                    #
                    # $m_image convert coords [lindex $dummy 10] [lindex $dummy 11] canvas \
                    #                         targw(1,x) targw(1,y) image
                    # calc_zemax 1 $targw(1,x) $targw(1,y)
                    # C_mxus$i put_b_oc 6 [list $targw(1,cx) $targw(1,cy)]
                }
            }
        }
    }
    tracer LEAVING
}

#=======================================
private method save_mxu_MSF {filename} {
#=======================================
# saving mask setup for MXU in MachineSlitFile format

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set string_msf  ""
    # msf_header
    # see VLT-TRE-VIRG-1416-0057 05/02/99 Issue 1 Rev 1, page 33
    # ---------------------------------------------------------
    set corr_flag [$this cget -fors_frame]

    # <><><><><><><><><><><><><><><><><><><><><>
    # workaround 13. Sept 2000 WH
    # FORS1 images are aliens for MXU
    # if { [string compare [$m_image fits get INSTRUME] "FORS1"] == 0 } {
    #     set corr_flag 0
    # }
    # <><><><><><><><><><><><><><><><><><><><><>

    set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]

    append string_msf [format "FORS2\n"]
    append string_msf [format "0\n"]
    append string_msf [format "%6d\n" [lindex $PM 1]]

    # determine number of rectangular and curved slits
    set STRA_sc 0;   # slit counter STRAIGHT
    set CURV_sc 0;   # slit counter CURVED
    set CIRC_sc 0;   # slit counter CIRCLE
    set noms [fors::MXU_SLIT::nos]
    if { $noms > 0 } {
        for { set i 1 } { $i <= $noms } { incr i 1 } {
            # if slit has is specified
            if { [C_mxus$i cget -ON] } {
                set loc_shap [C_mxus$i cget -shape]
                # count the number of slits
                switch -exact -- $loc_shap {
                    STRAIGHT {
                        incr STRA_sc 1
                    }
                    CURVED {
                        incr CURV_sc 1
                    }
                    CIRCLE {
                        incr CIRC_sc 1
                    }
                }
            }
        }
        append string_msf [format "%d\n" $STRA_sc]
        append string_msf [format "%d\n" $CURV_sc]
        append string_msf [format "%d\n" $CIRC_sc]

        if { $noms > 0 } {
            # first the straight slits
            # -------------------------
            for { set i 1 } { $i <= $noms } { incr i 1 } {
                if { [C_mxus$i cget -ON] } {
                    if { [string match *STRAIGHT* [C_mxus$i cget -shape]] } {
                        if { $corr_flag } {
                            # set slit [C_mxus$i get_oc_slit]
                            set slit [C_mxus$i get_oc_slit_mmu]
                        } else {
                            # set slit [C_mxus$i get_slit]
                            set slit [C_mxus$i get_slit_mmu]
                        }
                        append string_msf [format "%9.3f   " [lindex $slit 0]]
                        append string_msf [format "%9.3f   " [lindex $slit 1]]
                        append string_msf [format "%9.3f   " [lindex $slit 2]]
                        append string_msf [format "%9.3f   " [lindex $slit 3]]
                        append string_msf [format "%9.3f\n"  [C_mxus$i cget -pos_ang]]
                    }
                }
            }
            # now the curved  slits
            # --------------------
            for { set i 1 } { $i <= $noms } { incr i 1 } {
                if { [C_mxus$i cget -ON] } {
                    if { [string match *CURVED* [C_mxus$i cget -shape]] } {
                        if { $corr_flag } {
                            # set slit [C_mxus$i get_oc_slit]
                            set slit [C_mxus$i get_oc_slit_mmu]
                        } else {
                            # set slit [C_mxus$i get_slit]
                            set slit [C_mxus$i get_slit_mmu]
                        }
                        append string_msf [format "%9.3f   "   [lindex $slit 0]]
                        append string_msf [format "%9.3f   \n" [lindex $slit 1]]
                    }
                }
            }
            # now the circle slits
            # --------------------
            for { set i 1 } { $i <= $noms } { incr i 1 } {
                 if { [C_mxus$i cget -ON] } {
                     if { [string match *CIRCLE* [C_mxus$i cget -shape]] } {
                         if { $corr_flag } {
                             # set slit [C_mxus$i get_oc_slit]
                             set slit [C_mxus$i get_oc_slit_mmu]
                         } else {
                             # set slit [C_mxus$i get_slit]
                             set slit [C_mxus$i get_slit_mmu]
                         }
                         append string_msf [format "%9.3f   "   [lindex $slit 0]]
                         append string_msf [format "%9.3f   "   [lindex $slit 1]]
                         append string_msf [format "%9.3f   \n" [lindex $slit 2]]
                     }
                 }
            }
        }
    }
    #  SAVE  --- MSF ---
    set myfile $log_dir/$filename.fad
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_msf
    close $fileId
    tracer LEAVING
}

#=========================================
private method save_mxu_ADF { filename } {
#=========================================
# saving mask setup for MXU in ApertureDefinitionFile format

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set string_adf  ""
    set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]

    # adf_header
    # see VLT-TRE-VIRG-1416-0058 02/07/98 Issu1 Rev 0, page 120
    # ---------------------------------------------------------
    set dump "user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost"
    # apply optical correction if fits-image from fors
    set corr_flag [$this cget -fors_frame]

    # <><><><><><><><><><><><><><><><><><><><><>
    # workaround 13. Sept 2000 WH
    # FORS1 images are aliens for MXU
    # if { [string compare [$m_image fits get INSTRUME] "FORS1"] == 0 } {
    #     set corr_flag 0
    # }
    # <><><><><><><><><><><><><><><><><><><><><>

    append string_adf [format "TYPE        MOS\n"]
    append string_adf [format "VERSION     1\n"]
    append string_adf [format "MASK NA     %30s\n" [lindex $PM 0]]
    append string_adf [format "NAME        %50s\n"  $dump]
    append string_adf [format "MASK ID     %6d\n" [lindex $PM 1]]
    append string_adf [format "MASK NAID   %30s\n" [lindex $PM 0]+$exposure_counter]
    append string_adf [format "QUAD        %d\n" 0]
    append string_adf [format "PLATESCALE  %10.8f\n" $focscale]
    append string_adf [format "REF NO      %d\n" 0]
    append string_adf [format "SLIT NO     %d\n" [fors::MXU_SLIT::nos]]

    # -----------------
    # --- slit loop ---
    # -----------------
    set slit_c 0
    set noms [fors::MXU_SLIT::nos]
    if { $noms > 0 } {
        for { set i 1 } { $i <= $noms } { incr i 1 } {
            # if slit has is specified
            if { [C_mxus$i cget -ON] } {
                incr slit_c 1;   # count only the non-deleted slits
                puts stdout  [format "INS.TARG$i.NAME  %19s" [C_mxus$i cget -target_name]]
                append string_adf [format "SLIT%d ID          %d\n" $slit_c $i]
                append string_adf [format "SLIT%d NAME  %19s\n" $slit_c [C_mxus$i cget -target_name]]
                append string_adf [format "SLIT%d TYPE  %19s\n" $slit_c STRAIGHT]
                # slit dimensions and wcs
                if { $corr_flag } {
                    # set slit [C_mxus$i get_oc_slit]
                    set slit [C_mxus$i get_oc_slit_mmu]
                    lassign [split [C_mxus$i get_OC_RA ] :] a1 a2 a3
                    lassign [split [C_mxus$i get_OC_DEC] :] d1 d2 d3
                } else {
                    # set slit [C_mxus$i get_slit]
                    set slit [C_mxus$i get_slit_mmu]
                    lassign [split [C_mxus$i get_RA ] :] a1 a2 a3
                    lassign [split [C_mxus$i get_DEC] :] d1 d2 d3
                }

                append string_adf [format "SLIT%d X      %5.3f\n" $slit_c [lindex $slit 0]]
                append string_adf [format "SLIT%d Y      %5.3f\n" $slit_c [lindex $slit 1]]
                append string_adf [format "SLIT%d DIMX   %5.3f\n" $slit_c [lindex $slit 2]]
                append string_adf [format "SLIT%d DIMY   %5.3f\n" $slit_c [lindex $slit 3]]
                append string_adf [format "SLIT%d OBJ NO %d\n" $slit_c 1]
                append string_adf [format "SLIT%d OBJ X  %5.3f\n" $slit_c [lindex $slit 0]]
                append string_adf [format "SLIT%d OBJ Y  %5.3f\n" $slit_c [lindex $slit 1]]
                append string_adf [format "SLIT%d OBJ RA %02s%02s%06.3f\n" $slit_c $a1 $a2 $a3]
                append string_adf [format "SLIT%d OBJ DEC %02s%02s%06.3f\n" $slit_c $d1 $d2 $d3]
                append string_adf [format "SLIT%d OBJ LWIDTH     %5.3f\n" $slit_c 0.815]
            }
        }
    }

    #  SAVE  --- ADF ---
    set myfile $log_dir/$filename.adf
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_adf
    close $fileId
    tracer LEAVING
}

#============================================
private method save_mxu_RS274X { filename } {
#============================================
# Saving mask setup for MXU as Gerber format
# The gerber file has been testet and improved
# in June 1999 with the LPKF SW and Stencil laser
# cutting machine in Milano
#
# The empirical offset will be applied to the contour only
# not to the slit positions

# Offset in focal field mm units
# 00/06/99 W.Hummel initial offset 0.000 0.000 mm
# 23/07/99 W.Hummel   first offset 0.353 1.398 mm

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set FF_X 0.353; # empirical offset of the mask rim to coincide slits
    set FF_Y 1.398; # empirical offset of the mask rim to coincide slits
    set MM_X $FF_Y;                   # offset in mm of the mmu working table
    set MM_Y [expr {(-1.0) * $FF_X}]; # offset in mm of the mmu working table

    # We create two files, one for p2pp and for direct run
    set string_p ""
    set string_g ""
    # With two different separators
    # set S "?"
    set S "\n"
    set C "\n"

    set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
    set noms [fors::MXU_SLIT::nos]

    # --------------------------
    # PAF header for .p_gbr file
    # --------------------------
    append string_g "PAF.HDR.START; # Marks start of header\n"
    append string_g "PAF.TYPE          \"Mask Manufacuring Unit Executable\"; # Type of parameter file\n"
    append string_g "PAF.ID            \"FORS2-[string toupper [$this cget -modus]]\"; # Parameter file ID\n"
    append string_g "PAF.NAME          \"$filename.p_gbr\"; # Parameter file NAME\n"
    append string_g "PAF.DESC          \"Gerber file, executable for MMU\"; # Purpose\n"
    append string_g "PAF.CRTE.NAME     \"FIMS $f_v\"; # SW version date UT\n"
    append string_g "PAF.CRTE.DAYTIM   \"[exec date -u +%Y-%m-%dT%H:%M:%S]\"; # Date and time of creation\n"
    append string_g "PAF.LCHG.NAME     \"\"; # Who did last change\n"
    append string_g "PAF.LCHG.DAYTIM   \"\"; # Date and time of last change\n"
    # append string_g "PAF.CHCK.NAME     \"\"; # Appl. checking par. file\n"
    # append string_g "PAF.CHCK.DAYTIM   \"\"; # Date and time of lastcheck\n"
    # append string_g "PAF.CHCK.CHECKSUM \"\"; # Parameter file checksum\n"
    append string_g "PAF.HDR.END; # Marks end of header\n\n"
    append string_g "TPL.FILE.DIRNAME    \$INS_ROOT/SYSTEM/COMMON/SETUPFILES/INS\n"


    # -----------
    # info header
    # -----------
    set a "G04: created by SKYCAT-TOOL V$s_v -- FIMS-plugin V$f_v*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: orig filename  : $filename*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: fits file dir  : [file dirname [[.skycat1 component image] cget -file]]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: fits file name : [$this get_fits_name].fits*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: fits instrument: [$m_image fits get INSTRUME]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: fits origin    : [$m_image fits get ORIGIN]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: apply optidist : [$this cget -fors_frame]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: fits object    : [$m_image fits get OBJECT]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: catalog name   : $CatFileName*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: autoslit method: $AutoposMethod*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: INS.MASK.NA    : [lindex $PM 0]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: INS.MASK.ID    : [lindex $PM 1]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: INS.MASK.NAID  : [lindex $PM 0]+[lindex $PM 1]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: RS-274X for    : [$this cget -instrument]-[string toupper [$this cget -modus]]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04:  --------------------------------------------*"
    append string_g $a$C; append string_p $a$S;

    # -------------
    # gerberX setup
    # -------------
    set a "%IN FORS2_MXU_200x260_[lindex $PM 1]*%"
    append string_g $a$C; append string_p $a$S;
    set a "%ASAXBY*%"
    append string_g $a$C; append string_p $a$S;
    set a "%FSLAX33Y33*%"
    append string_g $a$C; append string_p $a$S;
    set a "%MOMM*%"
    append string_g $a$C; append string_p $a$S;
    # set a "%OFA0B0*%"
    # append string_g $a$C; append string_p $a$S;
    set a "%SFA1.0B1.0*%"
    append string_g $a$C; append string_p $a$S;
    set a "%IOA0B0*%"
    append string_g $a$C; append string_p $a$S;
    # set a "%IPPOS*%"
    # append string_g $a$C; append string_p $a$S;
    set a "%IR0*%"
    append string_g $a$C; append string_p $a$S;

    # ----------------
    # define apertures
    # ----------------
    set a "G04______________DEFINE APERTURES*"
    append string_g $a$C; append string_p $a$S;
    set a "%ADD10C,0.0*%"
    append string_g $a$C; append string_p $a$S;
    for { set i 1 } { $i <= $noms } { incr i 1 } {
        if { [C_mxus$i cget -ON] } {
            # [string length [C_mxus$i get_add_mmu]]
            set a [C_mxus$i get_add_mmu]
            # puts stdout $a
            append string_g $a$C; append string_p $a$S;
        }
    }

    # -----
    # slits
    # -----
    set a "G04_______________SLIT_CUTTING*"
    append string_g $a$C; append string_p $a$S;
    set a "%LNSLITS*%"
    append string_g $a$C; append string_p $a$S;
    set a "G90*";      # once more absolute
    append string_g $a$C; append string_p $a$S;
    set a "G54D10*";   # small aperture
    append string_g $a$C; append string_p $a$S;
    set a "D02*";      # close shutter
    append string_g $a$C; append string_p $a$S;

    # we ask all slits on their size (width, length) and
    # orientation (posang). Each rectangular slit with
    # zero pos_angle will get its own aperture definition
    set ad_counter 10
    set noms [fors::MXU_SLIT::nos]
    if { $noms > 0 } {
        for { set i 1 } { $i <= $noms } { incr i 1 } {
            # if slit has is specified
            if { [C_mxus$i cget -ON] } {
                set loc_shape [C_mxus$i cget -shape]
                switch -exact -- $loc_shape {
                    STRAIGHT {
                        # we have a non-deleted rectangular slit
                        if { [fors::MXU_SLIT::get_mmff] } {
                            # technical mask production,
                            # do not use OC values
                            set slit [C_mxus$i get_slit_mmu]
                        } else {
                            set slit [C_mxus$i get_oc_slit_mmu]
                        }
                        set lw [F2G [lindex $slit 3]]
                        set ll [F2G [lindex $slit 2]]; #slit dimensions
                        set lang [F2G [C_mxus$i cget -pos_ang]]
                        if { [string length [C_mxus$i get_add_mmu]] } {
                            # slit not rotated, use flash command
                            set a "G04________SLIT$i RECT $lw $ll $lang *"
                            append string_g $a$C; append string_p $a$S;
                            set a G54D[expr {$i + $ad_counter}]*
                            append string_g $a$C; append string_p $a$S;
                            set a "X[F2G [lindex $slit 0]]"
                            append a "Y[F2G [lindex $slit 1]]D03*"
                            append string_g $a$C; append string_p $a$S;
                        } else {
                            # slit rotated to parallelogram, use individual contour
                            if { [fors::MXU_SLIT::get_mmff] } {
                                # technical mask production,
                                # do not use OC values
                                set slit_dim [C_mxus$i get_slit_contour_mmu]
                            } else {
                                set slit_dim [C_mxus$i get_oc_slit_contour_mmu]
                            }
                            # puts stdout ">$slit_dim<"
                            # convert 123.456789 to 123456
                            set a "G04________SLIT$i PARA $lw $ll $lang *"
                            append string_g $a$C; append string_p $a$S;
                            set a "G54D10*"
                            append string_g $a$C; append string_p $a$S;
                            set a "X[F2G [lindex $slit_dim 0]]"
                            puts stdout [lindex $slit_dim 1]
                            append a "Y[F2G [lindex $slit_dim 1]]D02*"
                            append string_g $a$C; append string_p $a$S;
                            set a "X[F2G [lindex $slit_dim 2]]"
                            append a "Y[F2G [lindex $slit_dim 3]]D01*"
                            append string_g $a$C; append string_p $a$S;
                            set a "X[F2G [lindex $slit_dim 4]]"
                            append a "Y[F2G [lindex $slit_dim 5]]D01*"
                            append string_g $a$C; append string_p $a$S;
                            set a "X[F2G [lindex $slit_dim 6]]"
                            append a "Y[F2G [lindex $slit_dim 7]]D01*"
                            append string_g $a$C; append string_p $a$S;
                            set a "X[F2G [lindex $slit_dim 0]]"
                            append a "Y[F2G [lindex $slit_dim 1]]D01*"
                            append string_g $a$C; append string_p $a$S;
                        }
                    }
                    CIRCLE {
                        # we have a non-deleted circle aperture
                        if { [fors::MXU_SLIT::get_mmff] } {
                            # technical mask production,
                            # do not use OC values
                            set slit [C_mxus$i get_slit_mmu]
                        } else {
                            set slit [C_mxus$i get_oc_slit_mmu]
                        }
                        set lw [F2G [lindex $slit 3]]
                        # use flash command
                        set a "G04________SLIT$i CIRC $lw *"
                        append string_g $a$C; append string_p $a$S;
                        set a G54D[expr {$i + $ad_counter}]*
                        append string_g $a$C; append string_p $a$S;
                        set a "X[F2G [lindex $slit 0]]"
                        append a "Y[F2G [lindex $slit 1]]D03*"
                        append string_g $a$C; append string_p $a$S;
                    }
                    CURVED {
                        # we have a non-deleted curved slit
                        if { [fors::MXU_SLIT::get_mmff] } {
                            puts stdout "Error: CURVED in standard mask !!!"
                        }
                        set slit_dim [C_mxus$i get_b_oc_mmu]
                        # puts stdout ">$slit_dim<"
                        set lw [C_mxus$i cget -slit_width]
                        set a "G04________SLIT$i CURV $lw *"
                        append string_g $a$C; append string_p $a$S;
                        set a "G54D10*"
                        append string_g $a$C; append string_p $a$S;
                        #
                        # index of the six bezier points
                        #
                        #    0,1 +----+ 6,7
                        #         \    \
                        #          \    \
                        #           \    \
                        #           /    /  2,3 +     + 8,9
                        #      4,5 +----+ 10,11
                        #
                        # flash the first point
                        # convert 123.456789 to 123456
                        set a "X[F2G [lindex $slit_dim 0]]"
                        append a "Y[F2G [lindex $slit_dim 1]]D02*"
                        append string_g $a$C; append string_p $a$S;
                        # number of polygon steps for a curved slit
                        set res 200
                        #   1 pixel = 0.2 '' = 0.1 mm = 100 mu
                        # 200 pixel = 40''   = 20  mm = 20000 mu (maximum slit length)
                        # red Bezier
                        set x1 [lindex $slit_dim 0]
                        set y1 [lindex $slit_dim 1]
                        set x2 [lindex $slit_dim 2]
                        set y2 [lindex $slit_dim 3]
                        set x3 [lindex $slit_dim 4]
                        set y3 [lindex $slit_dim 5]

                        set x0 $x1
                        set y0 $y1
                        for { set j 0 } { $j <= $res } { incr j 1 } {
                            set t [expr {(1.0 * $j) / (1.0 * $res)}]
                            set xt [expr {(1-$t) * (1-$t) * $x1 + 2 * (1-$t) * $t * $x2 + $t * $t * $x3}]
                            set yt [expr {(1-$t) * (1-$t) * $y1 + 2 * (1-$t) * $t * $y2 + $t * $t * $y3}]
                            set a "X[F2G $xt]"
                            append a "Y[F2G $yt]D01*"
                            append string_g $a$C; append string_p $a$S;
                            set x0 $xt
                            set y0 $yt
                        }
                        # blue Bezier
                        # draw in opposite direction
                        set x1 [lindex $slit_dim 10]
                        set y1 [lindex $slit_dim 11]
                        set x2 [lindex $slit_dim 8]
                        set y2 [lindex $slit_dim 9]
                        set x3 [lindex $slit_dim 6]
                        set y3 [lindex $slit_dim 7]

                        # draw from red contour to blue contour
                        set a "X[F2G $x1]"
                        append a "Y[F2G $y1]D01*"
                        append string_g $a$C; append string_p $a$S;
                        set x0 $x1
                        set y0 $y1
                        for { set j 0 } { $j <= $res } { incr j 1 } {
                            set t [expr {(1.0 * $j) / (1.0 * $res)}]
                            set xt [expr {(1-$t) * (1-$t) * $x1 + 2 * (1-$t) * $t * $x2 + $t * $t * $x3}]
                            set yt [expr {(1-$t) * (1-$t) * $y1 + 2 * (1-$t) * $t * $y2 + $t * $t * $y3}]
                            set a "X[F2G $xt]"
                            append a "Y[F2G $yt]D01*"
                            append string_g $a$C; append string_p $a$S;
                            set x0 $xt
                            set y0 $yt
                        }
                        # draw blue conour back to red contour and end
                        set x1 [lindex $slit_dim 0]
                        set y1 [lindex $slit_dim 1]
                        set a "X[F2G $x1]"
                        append a "Y[F2G $y1]D01*"
                        append string_g $a$C; append string_p $a$S;
                    }
                }
            }
        }
    }

    # -------
    # mask id
    # -------
    #
    # W A R N I N G
    # -------------
    #
    # This part of the code exists as a copy in the
    # m2g script to insert the mask id to science masks
    # as part of the science opartion on Paranal
    # any change in this part of the code must be
    # mirrored in the m2g script

    set maskid [lindex $PM 1]

    # new concept since 12/1999:
    # sky masks contain no mask id
    # technical masks contain mask id

    # include MASD.ID only to technical masks
    # if {$maskid > 900000 && $maskid <= 900100}
    if { $noms > 1000 } {
        puts stdout "Error: number of apertures collides with maskid"
    }
    # this is a technical mask
    # create gerber file with mask id
    set a "G04______________MASK_ID_CUTTING $maskid*"
    append string_g $a$C; append string_p $a$S;
    # this defines a new layer for the LPKF SW
    # MASK ID will be cut faster
    set a "%LNBARCODE*%"
    append string_g $a$C; append string_p $a$S;

    # philosophy of mask id creation
    #
    #  --   this is a '8' consisting of 7 bars.
    # |  |  each bar is l x b large (length x width=breite).
    #  --   the full digit is 2s x 4s large, where
    # |  |  2s is a little bit larger than l in order to
    #  --   avoid overlapping bars.
    #       You can change the total size of the MASKID digit
    #       by parameter s. The digit is 2*s x 4*s mm large.
    #       You can change the layout of the digit with
    #       parameter l.
    #       Note that l must be smaller than 2*s in any case.

    # size of the digit
    set s  3.0;   # digit size = 2s x 4s
    set l  4.0;   # length of a bar, l < 2s !!
    set b  [expr {(2.0 * $s - $l) / 4.0}]; # width of a bar
    set ss [expr {$s * 2.0}]
    if { $l >= $ss } {
        puts stdout "ERROR: MASK_ID font size"
    }
    # for mmu working table switch l and b
    # set dummy $l;
    # set l $b;
    # set b $dummy

    puts -nonewline stdout "MASK_ID = "

    for { set k 0 } { $k <= 5 } { incr k 1 } {
        # central position of the digit in the focal field
        # set x0 [expr {-75 + $k * 15}]
        # set y0 -140
        # central position  on the mmu working table
        set x0 -140
        set y0 [expr {75 - $k * 15}]
        # select the digit
        set ziffer [string index $maskid $k]
        # puts stdout ">>$ziffer<<"
        # we paint digits directly in mmu working table coords
        switch -exact -- $ziffer {
            0 {
                # puts "$k th number is a 0"
                puts -nonewline stdout  0
                set a "%AMZERO*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                               21,1,\$2,\$1,\$3+$ss,\$4,0*\
                               21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                               21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                               21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                               21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
                append string_g $a$C; append string_p $a$S;
                set a "%ADD1000ZERO,$l\X$b\X0X0%"
                append string_g $a$C; append string_p $a$S;
                set a "G54D1000*"
                append string_g $a$C; append string_p $a$S;
                set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                append string_g $a$C; append string_p $a$S;
            }
            1 {
                # puts stdout "$k th number is a 1"
                puts -nonewline stdout  1
                set a "%AMONE*21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                              21,1,\$1,\$2,\$3-$s,\$4-$s,0*%"
                append string_g $a$C; append string_p $a$S;
                set a "%ADD1001ONE,$l\X$b\X0X0%"
                append string_g $a$C; append string_p $a$S;
                set a "G54D1001*"
                append string_g $a$C; append string_p $a$S;
                set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                append string_g $a$C; append string_p $a$S;
            }
            2 {
                # puts stdout "$k th number is a 2"
                puts -nonewline stdout  2
                set a "%AMTWO*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                              21,1,\$2,\$1,\$3+$ss,\$4,0*\
                              21,1,\$2,\$1,\$3,\$4,0*\
                              21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                              21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
                append string_g $a$C; append string_p $a$S;
                set a "%ADD1002TWO,$l\X$b\X0X0%"
                append string_g $a$C; append string_p $a$S;
                set a "G54D1002*"
                append string_g $a$C; append string_p $a$S;
                set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                append string_g $a$C; append string_p $a$S;
            }
            3 {
                # puts stdout "$k th number is a 3"
                puts -nonewline stdout  3
                set a "%AMTHREE*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                                21,1,\$2,\$1,\$3+$ss,\$4,0*\
                                21,1,\$2,\$1,\$3,\$4,0*\
                                21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                                21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
                append string_g $a$C; append string_p $a$S;
                set a "%ADD1003THREE,$l\X$b\X0X0%"
                append string_g $a$C; append string_p $a$S;
                set a "G54D1003*"
                append string_g $a$C; append string_p $a$S;
                set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                append string_g $a$C; append string_p $a$S;
            }
            4 {
                # puts stdout "$k th number is a 4"
                puts -nonewline stdout  4
                set a "%AMFOUR*21,1,\$2,\$1,\$3,\$4,0*\
                               21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                               21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                               21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
                append string_g $a$C; append string_p $a$S;
                set a "%ADD1004FOUR,$l\X$b\X0X0%"
                append string_g $a$C; append string_p $a$S;
                set a "G54D1004*"
                append string_g $a$C; append string_p $a$S;
                set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                append string_g $a$C; append string_p $a$S;
            }
            5 {
                # puts stdout "$k th number is a 5"
                puts -nonewline stdout  5
                set a "%AMFIVE*21,1,\$2,\$1,\$3,\$4,0*\
                               21,1,\$2,\$1,\$3+$ss,\$4,0*\
                               21,1,\$2,\$1,\$3-$ss,\$4,0*\
                               21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                               21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
                append string_g $a$C; append string_p $a$S;
                set a "%ADD1005FIVE,$l\X$b\X0X0%"
                append string_g $a$C; append string_p $a$S;
                set a "G54D1005*"
                append string_g $a$C; append string_p $a$S;
                set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                append string_g $a$C; append string_p $a$S;
            }
            6 {
                # puts stdout "$k th number is a 6"
                puts -nonewline stdout  6
                set a "%AMSIX*21,1,\$2,\$1,\$3,\$4,0*\
                              21,1,\$2,\$1,\$3+$ss,\$4,0*\
                              21,1,\$2,\$1,\$3-$ss,\$4,0*\
                              21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                              21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                              21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
                append string_g $a$C; append string_p $a$S;
                set a "%ADD1006SIX,$l\X$b\X0X0%"
                append string_g $a$C; append string_p $a$S;
                set a "G54D1006*"
                append string_g $a$C; append string_p $a$S;
                set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                append string_g $a$C; append string_p $a$S;
            }
            7 {
                # puts stdout "$k th number is a 7"
                puts -nonewline stdout  7
                set a "%AMSEVEN*21,1,\$2,\$1,\$3+$ss,\$4,0*\
                                21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                                21,1,\$1,\$2,\$3-$s,\$4-$s,0*%"
                append string_g $a$C; append string_p $a$S;
                set a "%ADD1007SEVEN,$l\X$b\X0X0%"
                append string_g $a$C; append string_p $a$S;
                set a "G54D1007*"
                append string_g $a$C; append string_p $a$S;
                set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                append string_g $a$C; append string_p $a$S;
            }
            8 {
                # puts stdout "$k th number is a 8"
                puts -nonewline stdout  8
                set a "%AMEIGHT*21,1,\$2,\$1,\$3,\$4,0*\
                                21,1,\$2,\$1,\$3+$ss,\$4,0*\
                                21,1,\$2,\$1,\$3-$ss,\$4,0*\
                                21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                                21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                                21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                                21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
                append string_g $a$C; append string_p $a$S;
                set a "%ADD1008EIGHT,$l\X$b\X0X0%"
                append string_g $a$C; append string_p $a$S;
                set a "G54D1008*"
                append string_g $a$C; append string_p $a$S;
                set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                append string_g $a$C; append string_p $a$S;
            }
            9 {
                # puts stdout "$k th number is a 9"
                puts -nonewline stdout  9
                set a "%AMNINE*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                               21,1,\$2,\$1,\$3,\$4,0*\
                               21,1,\$2,\$1,\$3+$ss,\$4,0*\
                               21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                               21,1,\$1,\$2,\$3+$s,\$4+$s,0*\
                               21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
                append string_g $a$C; append string_p $a$S;
                set a "%ADD1009NINE,$l\X$b\X0X0%"
                append string_g $a$C; append string_p $a$S;
                set a "G54D1009*"
                append string_g $a$C; append string_p $a$S;
                set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                append string_g $a$C; append string_p $a$S;
            }
            A {
                puts stdout "$k th number is a A, but is not yet supported"
            }
            default {
                puts stdout "something wrong "
            }
        }
    }

    # end of : include mask id only for science masks
    puts stdout "\n"

    # ------------
    # mask contour
    # ------------
    # View A:
    # You will hold it in your hand  in such a way
    #
    #  (94.9,126)    o     (-95,126)
    #          +-----------+C               FF         MMU
    #         / D           \
    #       E+               +B              ^ Y          ^ X
    #        |        _      |               |            |
    #        |       |_      |o              |            |
    #        |       |       |               +---X   <----+
    #        |               |                        Y
    #        |               |o
    #        |---------------|    (-126)
    #        | 987654        |
    #       F+---------------+G  (-203)
    #                        |
    #                        +A = starting point for cutting
    #
    # o = the three frame pins
    #
    # View B: flip view A at the FG-axis
    # You prepare it with fims in an orientation as viewed
    # from bottom to top
    #
    #                        +A = starting point
    #                        |
    # (-203)F+---------------+G
    #        |68L924         |
    # (-126) |---------------|    (y=-126)
    #        |    North      |o
    #        |               |                X        Y
    #   East |       |_      |West        +--->    <---+
    #        |       |_      |            |            |
    #        |               |o           |            |
    #       E+               +B           V Y          V X
    #         \ D   South   /
    #          +-----------+C                 FF         MMU
    #  (-95,126)      o    (94.9,126)
    #
    #
    #  View C: = view A rotated by 90deg
    #  It will be cut on the MMU working table like this
    #
    #      F                E        ^           +---->Y
    #      +--+-------------+        |Y          |
    #      |_o|              \       |           |
    #      |oo|               +D     |           |X
    #      |  |    _ _        |      +---> X
    #      |  |     | |       |o
    #      |  |               |
    #      |  |               +C     MMU          FF
    #      |  |              /
    #   A+-+--+-------------+B
    #      G    o         o
    #
    # contours are hard-coded, fix-points at BC and AB
    # x=94.9mm  means 0.2 mm space for thermal contraction of the frame
    # focal field ends in -Y at C=-126mm but there is still about 75mm
    # space available in -Y for mask ID cutting

    set a "G04______________CONTOUR_CUTTING*"
    append string_g $a$C; append string_p $a$S;
    set a "%LNCONTOUR*%"
    append string_g $a$C; append string_p $a$S;
    set a "G90*";                 # absolute
    append string_g $a$C; append string_p $a$S;
    set a "G54D10*";              # circle apertur
    append string_g $a$C; append string_p $a$S;
    # round corners
    # at B and E, we reduce the plate width by 2mm
    # on 2cm in y

    # in focal field coords
    # apply FF_X, FF_Y
    # set a "X-095000Y-164000D02*";            # A = starting point
    # set a "X-095000Y106000D01*";             # B
    # set a "X-093000Y126000D01*";             # C
    # set a "X092900Y126000D01*";              # D
    # set a "X094900Y106000D01*";              # E
    # set a "X094900Y-162000D01*";             # F
    # set a "X-095000Y-162000D01*";            # G

    # in mmu working table coords, x=y, y=x

    # original contours, no offset applied
    # set a "X-164000Y-095000D02*";            # A = starting point
    # append string_g $a$C; append string_p $a$S;
    # set a "X106000Y-095000D01*";             # B
    # append string_g $a$C; append string_p $a$S;
    # set a "X126000Y-093000D01*";             # C
    # append string_g $a$C; append string_p $a$S;
    # set a "X126000Y092900D01*";              # D
    # append string_g $a$C; append string_p $a$S;
    # set a "X106000Y094900D01*";              # E
    # append string_g $a$C; append string_p $a$S;
    # set a "X-162000Y094900D01*";             # F
    # append string_g $a$C; append string_p $a$S;
    # set a "X-162000Y-095000D01*";            # G
    # append string_g $a$C; append string_p $a$S;

    # apply MM_X, MM_Y
    set a "X[F2G [expr {-164.000 + $MM_X}]]Y[F2G [expr {-095.000 + $MM_Y}]]D02*"; # A
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr {106.000 + $MM_X}]]Y[F2G [expr {-095.000 + $MM_Y}]]D01*"; # B
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr {126.000 + $MM_X}]]Y[F2G [expr {-093.000 + $MM_Y}]]D01*"; # C
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr {126.000 + $MM_X}]]Y[F2G [expr {092.900 + $MM_Y}]]D01*"; # D
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr {106.000 + $MM_X}]]Y[F2G [expr {094.900 + $MM_Y}]]D01*"; # E
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr {-162.000 + $MM_X}]]Y[F2G [expr {094.900 + $MM_Y}]]D01*"; # F
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr {-162.000 + $MM_X}]]Y[F2G [expr {-095.000 + $MM_Y}]]D01*"; # G
    append string_g $a$C; append string_p $a$S;
    # end
    set a "D02*"
    append string_g $a$C; append string_p $a$S;
    set a "M02*"
    append string_g $a$C; append string_p $a$S;

    # SAVE  --- gerber format --- hitherto used for P2PP
    # set myfile $log_dir/$filename.p_gbr
    # set fileId [open $myfile w]
    # puts -nonewline $fileId $string_p
    # close $fileId
    # we will close this file soon

    # Later at Paranal use UNIX command
    #
    # tr -s '?' '\012' < mxu2.60.test_0.p_gbr > mxu2.60.test_0.gbr
    #
    # to connvert '?' to CR again
    #
    # Conversion to DOS readable files:
    # insert a Ctrl-M, use Ctrl-V Ctrl-M in vi for the script:
    # sed -e 's/$/^M/' $1 > $2

    # SAVE  --- gerber format --- for LPKF-SW
    set myfile $set_dir/$filename.p_gbr
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_g
    close $fileId
    catch { [exec pafchkCreate $myfile] } result
    # SAVE  --- gerber format --- for CircuitCam-SW
    set myfile $set_dir/$filename.gbr
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_p
    close $fileId
    tracer LEAVING
}

#=================================================
private method save_mxu_RS274X_forPS  {filename} {
#=================================================
# Saving mask setup for MXU as Gerber format
# The gerber file has been testet and improved
# in June 1999 with the LPKF SW and Stencil laser
# cutting machine in Milano
#
# The empirical offset will be applied to the contour only
# not to the slit positions
# Offset in focal field mm units
# 00/06/99 W.Hummel initial offset 0.000 0.000 mm
# 23/07/99 W.Hummel   first offset 0.353 1.398 mm

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set FF_X 0.353; # empirical offset of the mask rim to coincide slits
    set FF_Y 1.398; # empirical offset of the mask rim to coincide slits
    # set MM_X $FF_Y;                   # offset in mm of the mmu working table
    # set MM_Y [expr {(-1.0) * $FF_X}]; # offset in mm of the mmu working table
    set MM_X $FF_X
    set MM_Y $FF_Y;
    set OFFX 140.0
    set OFFY 200.0
    set MM_X [expr {$MM_X + $OFFX}]; # special offset to create gerberfiel with
                                     # all posotions x,y>0
    set MM_Y [expr {$MM_Y + $OFFY}]; # special offset to create gerberfiel with
                                     # all posotions x,y>0

    # We create two files, one for p2pp and for direct run
    set string_p ""
    set string_g ""
    # With two different separators
    set S "?"
    set C "\n"

    set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
    set noms [fors::MXU_SLIT::nos]

    # --------------------------
    # PAF header for .p_gbr file
    # --------------------------
    append string_g "G04: PAF.HDR.START;                            # Marks start of header\n"
    append string_g "G04: PAF.TYPE          \"Mask Manufacuring Unit Executable\"; # Type of parameter file\n"
    append string_g "G04: PAF.ID            \"FORS2-[string toupper [$this cget -modus]]\"; # Parameter file ID\n"
    append string_g "G04: PAF.NAME          \"\$INS_ROOT/SYSTEM/COMMON/SETUPFILES/$filename.p_gbr\"; # Parameter file NAME\n"
    append string_g "G04: PAF.DESC          \"gerber file, executable for MMU\"; # Purpose\n"
    append string_g "G04: PAF.CRTE.NAME     \"FIMS $f_v\"; # SW version date UT\n"
    append string_g "G04: PAF.CRTE.DAYTIM   \"[exec date -u +%Y-%m-%dT%H:%M:%S]\"; # Date and time of creation\n"
    append string_g "G04: PAF.LCHG.NAME     \"\";                     # Who did last change\n"
    append string_g "G04: PAF.LCHG.DAYTIM   \"\";                     # Date and time of last change\n"
    # append string_g "G04: PAF.CHCK.NAME     \"\";                     # Appl. checking par. file\n"
    # append string_g "G04: PAF.CHCK.DAYTIM   \"\";                     # Date and time of lastcheck\n"
    # append string_g "G04: PAF.CHCK.CHECKSUM \"\";                     # Parameter file checksum\n"
    append string_g "G04: PAF.HDR.END;                              # Marks end of header\n\n"

    # -----------
    # info header
    # -----------
    set a "G04: created by SKYCAT-TOOL V$s_v -- FIMS-plugin V$f_v*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: user-date-host : [exec whoami] [exec date -u +%Y-%m-%dT%H:%M:%S] $lochost*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: orig filename  : $filename*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: fits file dir  : [file dirname [[.skycat1 component image] cget -file]]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: fits file name : [$this get_fits_name].fits*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: fits instrument: [$m_image fits get INSTRUME]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: fits origin    : [$m_image fits get ORIGIN]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: apply optidist : [$this cget -fors_frame]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: fits object    : [$m_image fits get OBJECT]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: catalog name   : $CatFileName*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: autoslit method: $AutoposMethod*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: INS.MASK.NA    : [lindex $PM 0]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: INS.MASK.ID    : [lindex $PM 1]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: INS.MASK.NAID  : [lindex $PM 0]+[lindex $PM 1]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04: RS-274X for    : [$this cget -instrument]-[string toupper [$this cget -modus]]*"
    append string_g $a$C; append string_p $a$S;
    set a "G04:  --------------------------------------------*"
    append string_g $a$C; append string_p $a$S;

    # -------------
    # gerberX setup
    # -------------
    set a "G04: %IN FORS2_MXU_200x260_[lindex $PM 1]*%"
    append string_g $a$C; append string_p $a$S;
    set a "G04: %ASAYBX*%"
    append string_g $a$C; append string_p $a$S;
    set a "%FSLAX33Y33*%"
    append string_g $a$C; append string_p $a$S;
    set a "%MOMM*%"
    append string_g $a$C; append string_p $a$S;
    # set a "%OFA0B0*%"
    # append string_g $a$C; append string_p $a$S;
    set a "%SFA0.8B0.8*%"
    append string_g $a$C; append string_p $a$S;
    set a "G04: %IOA0B0*%"
    append string_g $a$C; append string_p $a$S;
    # set a "%IPPOS*%"
    # append string_g $a$C; append string_p $a$S;
    set a "G04: %IR0*%"
    append string_g $a$C; append string_p $a$S;

    # define apertures
    set a "G04______________DEFINE APERTURES*"
    append string_g $a$C; append string_p $a$S;
    set a "%ADD10C,0.0*%"
    append string_g $a$C; append string_p $a$S;
    for { set i 1 } { $i <= $noms } { incr i 1 } {
        if { [C_mxus$i cget -ON] } {
            # [string length [C_mxus$i get_add]]
            set a [C_mxus$i get_add]
            # puts stdout $a
            append string_g $a$C; append string_p $a$S;
        }
    }

    # -----
    # slits
    # -----
    set a "G04_______________SLIT_CUTTING*"
    append string_g $a$C; append string_p $a$S;
    set a "%LNSLITS*%"
    append string_g $a$C; append string_p $a$S;


    set a "G04: G90*";      # once more absolute
    append string_g $a$C; append string_p $a$S;
    set a "G54D10*";   # small aperture
    append string_g $a$C; append string_p $a$S;
    set a "D02*";      # close shutter
    append string_g $a$C; append string_p $a$S;

    # we ask all slits on their size (width, length) and
    # orientation (posang). Each rectangular slit with
    # zero pos_angle will get its own aperture definition
    set ad_counter 10
    set noms [fors::MXU_SLIT::nos]
    if { $noms > 0 } {
        for { set i 1 } { $i <= $noms } { incr i 1 } {
            # if slit has is specified
            if { [C_mxus$i cget -ON] } {
                set loc_shape [C_mxus$i cget -shape]
                switch -exact -- $loc_shape {
                    STRAIGHT {
                        # we have a non-deleted rectangular slit
                        if { [fors::MXU_SLIT::get_mmff] } {
                            # technical mask production,
                            # do not use OC values
                            set slit [C_mxus$i get_slit]
                        } else {
                            set slit [C_mxus$i get_oc_slit]
                        }
                        set lw [F2G [lindex $slit 3]]
                        set ll [F2G [lindex $slit 2]]; #slit dimensions
                        set lang [F2G [C_mxus$i cget -pos_ang]]
                        # Use individual contour
                        if { [fors::MXU_SLIT::get_mmff] } {
                            # technical mask production,
                            # do not use OC values
                            set slit_dim [C_mxus$i get_slit_contour]
                        } else {
                            set slit_dim [C_mxus$i get_oc_slit_contour]
                        }
                        # puts stdout ">$slit_dim<"
                        # convert 123.456789 to 123456
                        set a "G04________SLIT$i PARA $lw $ll $lang *"
                        append string_g $a$C; append string_p $a$S;
                        set a "G54D10*"
                        append string_g $a$C; append string_p $a$S;
                        set a "X[F2G [expr {[lindex $slit_dim 0] + $OFFX}]]"
                        # puts stdout [lindex $slit_dim 1]
                        append a "Y[F2G [expr {[lindex $slit_dim 1] + $OFFY}]]D02*"
                        append string_g $a$C; append string_p $a$S;
                        set a "X[F2G [expr {[lindex $slit_dim 2] + $OFFX}]]"
                        append a "Y[F2G [expr {[lindex $slit_dim 3] + $OFFY}]]D01*"
                        append string_g $a$C; append string_p $a$S;
                        set a "X[F2G [expr {[lindex $slit_dim 4] + $OFFX}]]"
                        append a "Y[F2G [expr {[lindex $slit_dim 5] + $OFFY}]]D01*"
                        append string_g $a$C; append string_p $a$S;
                        set a "X[F2G [expr {[lindex $slit_dim 6] + $OFFX}]]"
                        append a "Y[F2G [expr {[lindex $slit_dim 7] + $OFFY}]]D01*"
                        append string_g $a$C; append string_p $a$S;
                        set a "X[F2G [expr {[lindex $slit_dim 0] + $OFFX}]]"
                        append a "Y[F2G [expr {[lindex $slit_dim 1] + $OFFY}]]D01*"
                        append string_g $a$C; append string_p $a$S;
                    }
                    CIRCLE {
                        # we have a non-deleted circle aperture
                        if { [fors::MXU_SLIT::get_mmff] } {
                            # technical mask production,
                            # do not use OC values
                            set slit [C_mxus$i get_slit]
                        } else {
                            set slit [C_mxus$i get_oc_slit]
                        }
                        set lw [F2G [lindex $slit 3]]
                        # use flash command
                        set a "G04________SLIT$i CIRC $lw *"
                        append string_g $a$C; append string_p $a$S;
                        set a G54D[expr {$i + $ad_counter}]*
                        append string_g $a$C; append string_p $a$S;
                        set a "X[F2G [expr {[lindex $slit 0] + $OFFX}]]"
                        append a "Y[F2G [expr {[lindex $slit 1] + $OFFY}]]D03*"
                        append string_g $a$C; append string_p $a$S;
                    }
                    CURVED {
                        # we have a non-deleted curved slit
                        if { [fors::MXU_SLIT::get_mmff] } {
                            puts stdout "Error: CURVED in standard mask !!!"
                        }
                        set slit_dim [C_mxus$i get_b_oc]
                        # puts stdout ">$slit_dim<"
                        set lw [C_mxus$i cget -slit_width]
                        set a "G04________SLIT$i CURV $lw *"
                        append string_g $a$C; append string_p $a$S;
                        set a "G54D10*"
                        append string_g $a$C; append string_p $a$S;
                        #
                        # index of the six bezier points
                        #
                        #    0,1 +----+ 6,7
                        #         \    \
                        #          \    \
                        #           \    \
                        #           /    /  2,3 +     + 8,9
                        #      4,5 +----+ 10,11
                        #
                        # flash the first point
                        # convert 123.456789 to 123456
                        set a "X[F2G [expr {[lindex $slit_dim 0] + $OFFX}]]"
                        append a "Y[F2G [expr {[lindex $slit_dim 1] + $OFFY}]]D02*"
                        append string_g $a$C; append string_p $a$S;
                        # number of polygon steps for a curved slit
                        set res 200
                        #   1 pixel = 0.2 '' = 0.1 mm = 100 mu
                        # 200 pixel = 40''   = 20  mm = 20000 mu (maximum slit length)

                        # red Bezier
                        set x1 [lindex $slit_dim 0]
                        set y1 [lindex $slit_dim 1]
                        set x2 [lindex $slit_dim 2]
                        set y2 [lindex $slit_dim 3]
                        set x3 [lindex $slit_dim 4]
                        set y3 [lindex $slit_dim 5]

                        set x0 $x1
                        set y0 $y1
                        for { set j 0 } { $j <= $res } { incr j 1 } {
                            set t [expr {(1.0 * $j) / (1.0 * $res)}]
                            set xt [expr {(1 - $t) * (1 - $t) * $x1 + 2 * (1 - $t) * $t * $x2 + $t * $t * $x3}]
                            set yt [expr {(1 - $t) * (1 - $t) * $y1 + 2 * (1 - $t) * $t * $y2 + $t * $t * $y3}]
                            set a "X[F2G [expr {$xt + $OFFX}]]"
                            append a "Y[F2G [expr {$yt + $OFFY}]]D01*"
                            append string_g $a$C; append string_p $a$S;
                            set x0 $xt
                            set y0 $yt
                        }
                        # blue Bezier
                        # draw in opposite direction
                        set x1 [lindex $slit_dim 10]
                        set y1 [lindex $slit_dim 11]
                        set x2 [lindex $slit_dim 8]
                        set y2 [lindex $slit_dim 9]
                        set x3 [lindex $slit_dim 6]
                        set y3 [lindex $slit_dim 7]

                        # draw from red contour to blue contour
                        set a "X[F2G [expr {$x1 + $OFFX}]]"
                        append a "Y[F2G [expr {$y1 + $OFFY}]]D01*"
                        append string_g $a$C; append string_p $a$S;

                        set x0 $x1; set y0 $y1
                        for { set j 0 } { $j <= $res } { incr j 1 } {
                            set t [expr {(1.0 * $j) / (1.0 * $res)}]
                            set xt [expr {(1 - $t) * (1 - $t) * $x1 + 2 * (1 - $t) * $t * $x2 + $t * $t * $x3}]
                            set yt [expr {(1 - $t) * (1 - $t) * $y1 + 2 * (1 - $t) * $t * $y2 + $t * $t * $y3}]
                            set a "X[F2G [expr {$xt + $OFFX}]]"
                            append a "Y[F2G [expr {$yt + $OFFY}]]D01*"
                            append string_g $a$C; append string_p $a$S;
                            set x0 $xt; set y0 $yt
                        }
                        # draw blue conour back to red contour and end
                        set x1 [lindex $slit_dim 0]
                        set y1 [lindex $slit_dim 1]
                        set a "X[F2G [expr {$x1 + $OFFX}]]"
                        append a "Y[F2G [expr {$y1 + $OFFY}]]D01*"
                        append string_g $a$C; append string_p $a$S;
                    }
                }
            }
        }
    }

    # -------
    # mask id
    # -------
    #
    # W A R N I N G
    # -------------
    #
    # This part of the code exists as a copy in the
    # m2g script to insert the mask id to science masks
    # as part nof the science opartion on Paranal
    # any change in this part of the code must be
    # mirrored in the m2g script

    set maskid [lindex $PM 1]

    # new concept since 12/1999:
    # sky masks contain no mask id
    # technical masks contain mask id
    if { $maskid > 900000 && $maskid <= 900100 } {
        if { $noms > 1000 } {
            puts stdout "Error: number of apertures collides with maskid"
        }
        # this is a technical mask
        # create gerber file with mask id
        set a "G04______________MASK_ID_CUTTING $maskid*"
        append string_g $a$C; append string_p $a$S;
        # this defines a new layer for the LPKF SW
        # MASK ID will be cut faster
        set a "%LNBARCODE*%"
        append string_g $a$C; append string_p $a$S;

        # size of the digit
        set s  3.0;   # digit size = 2s x 4s
        set l  4.0;   # length of a bar, l < 2s !!
        set b  [expr {(2.0 * $s - $l) / 4.0}]; # width of a bar
        set ss [expr {$s * 2.0}]

        if { $l >= $ss } {
            puts stdout "ERROR: MASK_ID font size"
        }

        # for mmu working table switch l and b
        # set dummy $l;
        # set l $b;
        # set b $dummy

        puts "maskid = $maskid"
        puts -nonewline stdout "MASK_ID = "
        for { set k 0 } { $k <= 5 } { incr k 1 } {
            # central position of the digit in the focal field
            # set x0 [expr {-75 + $k * 15}]
            # set y0 -140
            # central position  on the mmu working table
            set x0 [expr {-140 + $OFFX}]
            set y0 [expr {75 - $k * 15 + $OFFY}]
            # select the digit
            set ziffer [string index $maskid $k]
            # puts stdout ">>$ziffer<<"
            # we paint digits directly in mmu working table coords
            switch -exact -- $ziffer {
                0 {
                    # puts "$k th number is a 0"
                    puts -nonewline stdout  0
                    set a "%AMZERO*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                                   21,1,\$2,\$1,\$3+$ss,\$4,0*\
                                   21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                                   21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                                   21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                                   21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
                    append string_g $a$C; append string_p $a$S;
                    set a "%ADD1000ZERO,$l\X$b\X0X0%"
                    append string_g $a$C; append string_p $a$S;
                    set a "G54D1000*"
                    append string_g $a$C; append string_p $a$S;
                    set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                    append string_g $a$C; append string_p $a$S;
                }
                1 {
                    # puts stdout "$k th number is a 1"
                    puts -nonewline stdout  1
                    set a "%AMONE*21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                                  21,1,\$1,\$2,\$3-$s,\$4-$s,0*%"
                    append string_g $a$C; append string_p $a$S;
                    set a "%ADD1001ONE,$l\X$b\X0X0%"
                    append string_g $a$C; append string_p $a$S;
                    set a "G54D1001*"
                    append string_g $a$C; append string_p $a$S;
                    set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                    append string_g $a$C; append string_p $a$S;
                }
                2 {
                    # puts stdout "$k th number is a 2"
                    puts -nonewline stdout  2
                    set a "%AMTWO*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                                  21,1,\$2,\$1,\$3+$ss,\$4,0*\
                                  21,1,\$2,\$1,\$3,\$4,0*\
                                  21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                                  21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
                    append string_g $a$C; append string_p $a$S;
                    set a "%ADD1002TWO,$l\X$b\X0X0%"
                    append string_g $a$C; append string_p $a$S;
                    set a "G54D1002*"
                    append string_g $a$C; append string_p $a$S;
                    set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                    append string_g $a$C; append string_p $a$S;
                }
                3 {
                    # puts stdout "$k th number is a 3"
                    puts -nonewline stdout  3
                    set a "%AMTHREE*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                                    21,1,\$2,\$1,\$3+$ss,\$4,0*\
                                    21,1,\$2,\$1,\$3,\$4,0*\
                                    21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                                    21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
                    append string_g $a$C; append string_p $a$S;
                    set a "%ADD1003THREE,$l\X$b\X0X0%"
                    append string_g $a$C; append string_p $a$S;
                    set a "G54D1003*"
                    append string_g $a$C; append string_p $a$S;
                    set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                    append string_g $a$C; append string_p $a$S;
                }
                4 {
                    # puts stdout "$k th number is a 4"
                    puts -nonewline stdout  4
                    set a "%AMFOUR*21,1,\$2,\$1,\$3,\$4,0*\
                                   21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                                   21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                                   21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
                    append string_g $a$C; append string_p $a$S;
                    set a "%ADD1004FOUR,$l\X$b\X0X0%"
                    append string_g $a$C; append string_p $a$S;
                    set a "G54D1004*"
                    append string_g $a$C; append string_p $a$S;
                    set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                    append string_g $a$C; append string_p $a$S;
                }
                5 {
                    # puts stdout "$k th number is a 5"
                    puts -nonewline stdout  5
                    set a "%AMFIVE*21,1,\$2,\$1,\$3,\$4,0*\
                                   21,1,\$2,\$1,\$3+$ss,\$4,0*\
                                   21,1,\$2,\$1,\$3-$ss,\$4,0*\
                                   21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                                   21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
                    append string_g $a$C; append string_p $a$S;
                    set a "%ADD1005FIVE,$l\X$b\X0X0%"
                    append string_g $a$C; append string_p $a$S;
                    set a "G54D1005*"
                    append string_g $a$C; append string_p $a$S;
                    set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                    append string_g $a$C; append string_p $a$S;
                }
                6 {
                    # puts stdout "$k th number is a 6"
                    puts -nonewline stdout  6
                    set a "%AMSIX*21,1,\$2,\$1,\$3,\$4,0*\
                                  21,1,\$2,\$1,\$3+$ss,\$4,0*\
                                  21,1,\$2,\$1,\$3-$ss,\$4,0*\
                                  21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                                  21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                                  21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
                    append string_g $a$C; append string_p $a$S;
                    set a "%ADD1006SIX,$l\X$b\X0X0%"
                    append string_g $a$C; append string_p $a$S;
                    set a "G54D1006*"
                    append string_g $a$C; append string_p $a$S;
                    set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                    append string_g $a$C; append string_p $a$S;
                }
                7 {
                    # puts stdout "$k th number is a 7"
                    puts -nonewline stdout  7
                    set a "%AMSEVEN*21,1,\$2,\$1,\$3+$ss,\$4,0*\
                                    21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                                    21,1,\$1,\$2,\$3-$s,\$4-$s,0*%"
                    append string_g $a$C; append string_p $a$S;
                    set a "%ADD1007SEVEN,$l\X$b\X0X0%"
                    append string_g $a$C; append string_p $a$S;
                    set a "G54D1007*"
                    append string_g $a$C; append string_p $a$S;
                    set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                    append string_g $a$C; append string_p $a$S;
                }
                8 {
                    # puts stdout "$k th number is a 8"
                    puts -nonewline stdout  8
                    set a "%AMEIGHT*21,1,\$2,\$1,\$3,\$4,0*\
                                    21,1,\$2,\$1,\$3+$ss,\$4,0*\
                                    21,1,\$2,\$1,\$3-$ss,\$4,0*\
                                    21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                                    21,1,\$1,\$2,\$3-$s,\$4+$s,0*\
                                    21,1,\$1,\$2,\$3+$s,\$4-$s,0*\
                                    21,1,\$1,\$2,\$3+$s,\$4+$s,0*%"
                    append string_g $a$C; append string_p $a$S;
                    set a "%ADD1008EIGHT,$l\X$b\X0X0%"
                    append string_g $a$C; append string_p $a$S;
                    set a "G54D1008*"
                    append string_g $a$C; append string_p $a$S;
                    set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                    append string_g $a$C; append string_p $a$S;
                }
                9 {
                    # puts stdout "$k th number is a 9"
                    puts -nonewline stdout  9
                    set a "%AMNINE*21,1,\$2,\$1,\$3-$ss,\$4,0*\
                                   21,1,\$2,\$1,\$3,\$4,0*\
                                   21,1,\$2,\$1,\$3+$ss,\$4,0*\
                                   21,1,\$1,\$2,\$3-$s,\$4-$s,0*\
                                   21,1,\$1,\$2,\$3+$s,\$4+$s,0*\
                                   21,1,\$1,\$2,\$3+$s,\$4-$s,0*%"
                    append string_g $a$C; append string_p $a$S;
                    set a "%ADD1009NINE,$l\X$b\X0X0%"
                    append string_g $a$C; append string_p $a$S;
                    set a "G54D1009*"
                    append string_g $a$C; append string_p $a$S;
                    set a "X[ F2G $x0 ]Y[ F2G $y0 ]D03*"
                    append string_g $a$C; append string_p $a$S;
                }
                A {
                    puts stdout "$k th number is a A, but is not yet supported"
                }
                default {
                    puts stdout "something wrong "
                }
            }
        }
    }

    puts stdout "\n"

    # -------------
    # mask contour
    # -------------
    # View A:
    # You will hold it in your hand  in such a way
    #
    #  (94.9,126)    o     (-95,126)
    #          +-----------+C               FF         MMU
    #         / D           \
    #       E+               +B              ^ Y          ^ X
    #        |        _      |               |            |
    #        |       |_      |o              |            |
    #        |       |       |               +---X   <----+
    #        |               |                        Y
    #        |               |o
    #        |---------------|    (-126)
    #        | 987654        |
    #       F+---------------+G  (-203)
    #                        |
    #                        +A = starting point for cutting
    #
    # o = the three frame pins
    #
    # View B: flip view A at the FG-axis
    # You prepare it with fims in an orientation as viewed
    # from bottom to top
    #
    #                        +A = starting point
    #                        |
    # (-203)F+---------------+G
    #        |68L924         |
    # (-126) |---------------|    (y=-126)
    #        |    North      |o
    #        |               |                X        Y
    #   East |       |_      |West        +--->    <---+
    #        |       |_      |            |            |
    #        |               |o           |            |
    #       E+               +B           V Y          V X
    #         \ D   South   /
    #          +-----------+C                 FF         MMU
    #  (-95,126)      o    (94.9,126)
    #
    #
    #  View C: = view A rotated by 90deg
    #  It will be cut on the MMU working table like this
    #
    #      F                E        ^           +---->Y
    #      +--+-------------+        |Y          |
    #      |_o|              \       |           |
    #      |oo|               +D     |           |X
    #      |  |    _ _        |      +---> X
    #      |  |     | |       |o
    #      |  |               |
    #      |  |               +C     MMU          FF
    #      |  |              /
    #   A+-+--+-------------+B
    #      G    o         o
    #
    # contours are hard-coded, fix-points at BC and AB
    # x=94.9mm  means 0.2 mm space for thermal contraction of the frame
    # focal field ends in -Y at C=-126mm but there is still about 75mm
    # space available in -Y for mask ID cutting

    set a "G04______________CONTOUR_CUTTING*"
    append string_g $a$C; append string_p $a$S;
    set a "%LNCONTOUR*%"
    append string_g $a$C; append string_p $a$S;
    set a "G04: G90*";                 # absolute
    append string_g $a$C; append string_p $a$S;
    set a "G54D10*";              # circle apertur
    append string_g $a$C; append string_p $a$S;
    # round corners
    # at B and E, we reduce the plate width by 2mm
    # on 2cm in y

    # in focal field coords
    # apply FF_X, FF_Y
    set a "X[F2G [expr { 95.0 + $MM_X}]]Y[F2G [expr {-164.0 + $MM_Y}]]D02*"; # A = starting point
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr { 95.0 + $MM_X}]]Y[F2G [expr { 106.0 + $MM_Y}]]D01*"; # B
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr { 93.0 + $MM_X}]]Y[F2G [expr { 126.0 + $MM_Y}]]D01*"; # C
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr {-92.9 + $MM_X}]]Y[F2G [expr { 126.0 + $MM_Y}]]D01*"; # D
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr {-94.9 + $MM_X}]]Y[F2G [expr { 106.0 + $MM_Y}]]D01*"; # E
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr {-94.9 + $MM_X}]]Y[F2G [expr {-162.0 + $MM_Y}]]D01*"; # F
    append string_g $a$C; append string_p $a$S;
    set a "X[F2G [expr { 95.0 + $MM_X}]]Y[F2G [expr {-162.0 + $MM_Y}]]D01*"; # G
    append string_g $a$C; append string_p $a$S;

    # in mmu working table coords, x=y, y=x
    # apply MM_X, MM_Y
    # set a "X[F2G [expr {-164.000 + $MM_X}]]Y[F2G [expr {-095.000 + $MM_Y}]]D02*"; # A
    # append string_g $a$C; append string_p $a$S;
    # set a "X[F2G [expr { 106.000 + $MM_X}]]Y[F2G [expr {-095.000 + $MM_Y}]]D01*"; # B
    # append string_g $a$C; append string_p $a$S;
    # set a "X[F2G [expr { 126.000 + $MM_X}]]Y[F2G [expr {-093.000 + $MM_Y}]]D01*"; # C
    # append string_g $a$C; append string_p $a$S;
    # set a "X[F2G [expr { 126.000 + $MM_X}]]Y[F2G [expr { 092.900 + $MM_Y}]]D01*"; # D
    # append string_g $a$C; append string_p $a$S;
    # set a "X[F2G [expr { 106.000 + $MM_X}]]Y[F2G [expr { 094.900 + $MM_Y}]]D01*"; # E
    # append string_g $a$C; append string_p $a$S;
    # set a "X[F2G [expr {-162.000 + $MM_X}]]Y[F2G [expr { 094.900 + $MM_Y}]]D01*"; # F
    # append string_g $a$C; append string_p $a$S;
    # set a "X[F2G [expr {-162.000 + $MM_X}]]Y[F2G [expr {-095.000 + $MM_Y}]]D01*"; # G
    # append string_g $a$C; append string_p $a$S;
    # end
    set a "D02*"
    append string_g $a$C; append string_p $a$S;
    set a "M02*"
    append string_g $a$C; append string_p $a$S;

    # SAVE  --- gerber format --- hitherto used for P2PP
    # set myfile $log_dir/$filename.p_gbr
    # set fileId [open $myfile w]
    # puts -nonewline $fileId $string_p
    # close $fileId
    # we will close this file soon

    # later at Paranal use UNIX command
    #
    # tr -s '?' '\012' < mxu2.60.test_0.p_gbr > mxu2.60.test_0.gbr
    #
    # to connvert '?' to CR again
    # conversion to DOS readable files:
    # insert a Ctrl-M, use Ctrl-V Ctrl-M in vi for the script:
    # sed -e 's/$/^M/' $1 > $2

    # SAVE  --- gerber format --- for LPKF-SW
    set myfile $log_dir/$filename
    set fileId [open $myfile w]
    puts -nonewline $fileId $string_g
    close $fileId
    catch { [exec pafchkCreate $myfile] } result
    tracer LEAVING
}

#=============================
private method wave_limit {} {
#=============================
# Sets the wavelength limits of the spectral range
# 'good' is the index of the dispersion table A_disp
# it was calculated by public variable filter and grism

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set ins_lmin $A_disp($good,min)
    set ins_lmax $A_disp($good,max)
    set ins_lamc $A_disp($good,cen)
    set ins_disp $A_disp($good,disp)
    set ins_ang  $A_disp($good,ang)

    # 'ins_lmax' and 'ins_lmin' are the wavelength limits
    # from the grism+filter combination, it is called the
    # instrumental wavelength range.
    # It is much larger than the CCD area
    # L_MIN and L_MAX determine the user-defined wavelength region
    # The maximum wavelength range which can be
    # matched by the CCD is c_min c_max

    set id00 [format ">> %5.1f < %5.1f < %5.1f < %5.1f < %5.1f\n" \
                       $ins_lmin $L_MIN $ins_lamc $L_MAX $ins_lmax]
    # Sorting order
    if { $ins_lmax < $ins_lmin } {
        set dum1 $ins_lmax
        set ins_lmax $ins_lmin
        set ins_lmin $dum1
    }

    # If the user defined range is inverted, exchange values
    if { $L_MAX < $L_MIN } {
        set dum1 $L_MAX
        set L_MAX $L_MIN
        set L_MIN $dum1
    }

    # If the user defined borders are more than 50% outside the instrumental range,
    # adjust to instrumental borders
    if { $L_MIN < $ins_lmin*0.5 } {
        set L_MIN $ins_lmin
    }
    if { $L_MAX > $ins_lmax*1.5 } {
        set L_MAX $ins_lmax
    }

    # Since the grism occurs in the optical path after the coll,
    # the dispersion on the CCD is the same in both coll. modes, but
    # 1) from SR->HR use the half slit width in order to retain
    #    the spectr. resolution
    # 2) the dispersion, projected into the MOS focal plane
    #    (thats the blue lines we draw)
    #    is different in both coll modes

    #
    # Spectrum lenght
    # From nm (wavelength) to mm (focalplane) (User Defined)
    set speclen [expr {abs($L_MAX - $L_MIN)/$ins_disp/$focscale}]
    set speclen [expr {$speclen/$PIX*$CCDscale_standard}]

    # From nm (wavelength) to mm (focalplane) (INStrumental)
    set ins_len [expr {abs($ins_lmax - $ins_lmin)/$ins_disp/$focscale}]
    set ins_len [expr {$ins_len/$PIX*$CCDscale_standard}]

    #
    # Spectrum dispersion from central wavelength
    # - min: to the BLUE side
    # - max: to the RED side
    # From nm (wavelength) to mm (focalplane) (User Defined)
    # 
    # Used as distances > 0 in mm
    set specmin [expr {($ins_lamc - $L_MIN)/$ins_disp/$focscale}]
    set specmin [expr {$specmin/$PIX*$CCDscale_standard}]

    set specmax [expr {($L_MAX - $ins_lamc)/$ins_disp/$focscale}]
    set specmax [expr {$specmax/$PIX*$CCDscale_standard}]

    set ccdm_specmin [expr {($ins_lamc - $L_MIN)/$ins_disp/$focscale}]
    set ccdm_specmin [expr {$ccdm_specmin/$PIX*$CCDscale_standard}]

    set ccdm_specmax [expr {($L_MAX - $ins_lamc)/$ins_disp/$focscale}]
    set ccdm_specmax [expr {$ccdm_specmax/$PIX*$CCDscale_standard}]

    set specminHR [expr {$specmin*$CCDscale_highres/$CCDscale_standard}]
    set specmaxHR [expr {$specmax*$CCDscale_highres/$CCDscale_standard}]

    set ccdm_specminHR [expr {$ccdm_specmin*$CCDscale_highres/$CCDscale_standard}]
    set ccdm_specmaxHR [expr {$ccdm_specmax*$CCDscale_highres/$CCDscale_standard}]
 
    # From nm (wavelength) to pixel (CCD) (User Defined)
    set dispmin [expr {($ins_lamc - $L_MIN)/$ins_disp/$PIX}]
    set dispmax [expr {($L_MAX - $ins_lamc)/$ins_disp/$PIX}]

    set dispminHR [expr {$dispmin*$CCDscale_highres/$CCDscale_standard}]
    set dispmaxHR [expr {$dispmax*$CCDscale_highres/$CCDscale_standard}]

    if { $debug && $verbose } {
        puts stdout         "wave_limit:"
        puts stdout         "  good     = $good"
        puts stdout         "  ins_lmin = $ins_lmin (nm)"
        puts stdout         "  ins_lmax = $ins_lmax (nm)"
        puts stdout         "  ins_lamc = $ins_lamc (nm)"
        puts stdout         "  ins_disp = $ins_disp (nm/mm)"
        puts stdout [format "  ins_len  = %8.2f (mm)" $ins_len]
        puts stdout         "  ins_ang  = $ins_ang (deg)"
        puts stdout         "  L_MIN    = $L_MIN (nm)"
        puts stdout         "  L_MAX    = $L_MAX (nm)"
        puts stdout [format "  speclen  = %8.2f (mm)" $speclen]
        puts stdout         "  SR:"
        puts stdout [format "  dispmin        = %8.2f (pix)" $dispmin]
        puts stdout [format "  dispmax        = %8.2f (pix)" $dispmax]
        puts stdout [format "  specmin        = %8.2f (mm)" $specmin]
        puts stdout [format "  specmax        = %8.2f (mm)" $specmax]
        puts stdout [format "  ccdm_specmin   = %8.2f (mm)" $ccdm_specmin]
        puts stdout [format "  ccdm_specmax   = %8.2f (mm)" $ccdm_specmax]
        puts stdout         "  HR:"
        puts stdout [format "  dispminHR      = %8.2f (pix)" $dispminHR]
        puts stdout [format "  dispmaxHR      = %8.2f (pix)" $dispmaxHR]
        puts stdout [format "  specminHR      = %8.2f (mm)" $specminHR]
        puts stdout [format "  specmaxHR      = %8.2f (mm)" $specmaxHR]
        puts stdout [format "  ccdm_specminHR = %8.2f (mm)" $ccdm_specminHR]
        puts stdout [format "  ccdm_specmaxHR = %8.2f (mm)" $ccdm_specmaxHR]
        puts stdout ""
    }

    # spec_len now in mm
    set id0 [format ">> %5.1f < %5.1f < %5.1f < %5.1f < %5.1f %s %s\n" \
                    $ins_lmin $L_MIN $ins_lamc $L_MAX $ins_lmax [$this cget -grism] [$this cget -filter]]

    set id0a [format ">>  DISP %5.1f A/mmCCD %5.1f A/pix %5.1f %5.1f A/'' %5.1f %5.1f A/mmfoc\n" \
                     [expr {$ins_disp*10.0}] \
                     [expr {$ins_disp*10.0*$PIX}] \
                     [expr {$ins_disp*10.0*$PIX/$CCDscale_standard}] \
                     [expr {$ins_disp*10.0*$PIX/$CCDscale_highres}] \
                     [expr {$ins_disp*10.0*$PIX/$CCDscale_standard/$focscale}] \
                     [expr {$ins_disp*10.0*$PIX/$CCDscale_highres/$focscale}]]

    set id1 [format ">>  BLU %6.1f A = %6.1f pixel = %6.1f '' = %6.3f ' =  %8.3f mmfoc\n" \
                    [expr {($ins_lamc - $L_MIN)*10.0}] \
                    [expr {($ins_lamc - $L_MIN)/$ins_disp/$PIX}] \
                    [expr {($ins_lamc - $L_MIN)/$ins_disp/$PIX*$CCDscale_standard}]  \
                    [expr {($ins_lamc - $L_MIN)/$ins_disp/$PIX*$CCDscale_standard/60.0}] \
                    $specmin]

    set id2 [format ">>  RED %6.1f A = %6.1f pixel = %6.1f '' = %6.3f ' =  %8.3f mmfoc\n" \
                    [expr {($L_MAX - $ins_lamc)*10.0}] \
                    [expr {($L_MAX - $ins_lamc)/$ins_disp/$PIX}] \
                    [expr {($L_MAX - $ins_lamc)/$ins_disp/$PIX*$CCDscale_standard}] \
                    [expr {($L_MAX - $ins_lamc)/$ins_disp/$PIX*$CCDscale_standard/60.0}] \
                    $specmax]

    set id3 [format ">>  USD %6.1f A = %6.1f pixel = %6.1f '' = %6.3f ' =  %8.3f mmfoc\n" \
                    [expr {($L_MAX - $L_MIN)*10.0}] \
                    [expr {($L_MAX - $L_MIN)/$ins_disp/$PIX}] \
                    [expr {($L_MAX - $L_MIN)/$ins_disp/$PIX*$CCDscale_standard}] \
                    [expr {($L_MAX - $L_MIN)/$ins_disp/$PIX*$CCDscale_standard/60.0}] \
                    [expr {($specmax + $specmin)}]]

    set id4 [format ">>  SPE %6.1f A = %6.1f pixel = %6.1f '' = %6.3f ' =  %8.3f mmfoc\n" \
                    [expr {($ins_lmax - $ins_lmin)*10.0}] \
                    [expr {($ins_lmax - $ins_lmin)/$ins_disp/$PIX}] \
                    [expr {($ins_lmax - $ins_lmin)/$ins_disp/$PIX*$CCDscale_standard}] \
                    [expr {($ins_lmax - $ins_lmin)/$ins_disp/$PIX*$CCDscale_standard/60.0}] \
                    $ins_len]

    set id5 [format ">> Field Stop total range = %6.1f A (SR) = %6.1f A (HR)\n"\
                    [expr {($XSRMAX - $XSRMIN)*$focscale*$PIX/$CCDscale_standard*$ins_disp*10.0}] \
                    [expr {($XHRMAX - $XHRMIN)*$focscale*$PIX/$CCDscale_highres*$ins_disp*10.0}]]

    set id6 [format ">> CCD total range        = %6.1f A (SR) = %6.1f A (HR)" \
                    [expr {($CCDMASTER_XSRMAX - $CCDMASTER_XSRMIN)*$focscale*$PIX/$CCDscale_standard*$ins_disp*10.0}] \
                    [expr {($CCDMASTER_XHRMAX - $CCDMASTER_XHRMIN)*$focscale*$PIX/$CCDscale_highres*$ins_disp*10.0}]]

    set mess " $id00 $id0 $id0a $id1 $id2 $id3 $id4 $id5 $id6"

    set binx 0
    catch { set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"] }
    if { $binx > 1 } {
        set binning binned
    } else {
        set binning unbinned
    }
    puts "Wave Limits:"
    puts " INS_DISP: $ins_disp (nm/mm)"
    puts " FOCSCALE: $focscale (arcsec/mm)"
    puts " PIX:      $PIX (mm/pix) $binning"
    puts " CCDscale_standard: $CCDscale_standard (arcsec/pix)"
    puts " CCDscale_highres:  $CCDscale_highres (arcsec/pix)"
    puts " "

    puts stdout " FORS CCD Frame"
    puts stdout " =============="
    puts stdout $mess
    puts stdout " ==============\n"

    if { $debug && $verbose } {
        puts stdout         "wave_limit: END"
    }
    tracer LEAVING
}

#=========================================================
private method calculate_focalfield_frame { {resol SR} } {
#=========================================================
# By P. Baksai
# Used by MXU and HIT-MS modes to calculate the focalfield
# and CCD frames.
# Method wave_limit MUST be called before this one.

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "calculate_focalfield_frame: BEGIN $resol"
    }

    # Select resolution parameters (mm)
    if { [string match *SR* $resol] } {
        # SR
        set ccd_scale $CCDscale_standard
        # Corner slots of the Frame Arrays used by plot_mask method
        lassign {a b c d} a b c d
        # Label placeholder slots
        lassign {stl sbl sel swl} tl bl el wl
        if { $debug && !$verbose } {
            puts stdout "calculate_focalfield_frame: Standard Resolution"
        }
    } else {
        # HR
        set ccd_scale $CCDscale_highres
        # Corner slots of the Frame Arrays used by plot_mask method
        lassign {e f g h} a b c d
        # Label placeholder slots
        lassign {htl hbl hel hwl} tl bl el wl
        if { $debug && !$verbose } {
            puts stdout "calculate_focalfield_frame: High Resolution"
        }
    }
    if { $debug && $verbose } {
        puts stdout "                            CCD Scale = $ccd_scale"
    }


    # Select focal field special limits (mm)
    # This overwrites values loaded from ISF according to each mode needs.
    set lmod [$this cget -modus]
    switch -exact -- $lmod {
        hitms {
            # The focal field is centered on the same pixel (image X coordinate)
            # specified by the multishiftClockPattern call in
            # FORS2_hitms_obs_exp.seq
            # The focal field width should be 10 arsec

        # PBG 20090803: This code is under development. BEGIN
        if { 0 } {
            # How many pixels are 10":
            set as2pix [expr {10/[lindex [$m_image wcsset] 2]}]
            # Focal field width in image pixels
            set pixmin [expr {850 - $as2pix/2}]
            set pixmax [expr {850 + $as2pix/2}]
            # Convert image pixels to WCS degrees
            catch { $m_image convert coords $ccx $ccy image wx1 wy1 "deg $EQX" }
            catch { $m_image convert coords $pixmin $ccy image wx2 wy2 "deg $EQX" }
            catch { $m_image convert coords $pixmax $ccy image wx3 wy3 "deg $EQX" }
            # Convert WCS degrees to focal field mm in the X axis
            set dx1 [expr {($wx1 - $wx2)/15*3600}]
            set dx2 [expr {($wx1 - $wx3)/15*3600}]
            # set xmin [expr {$dx1/$focscale}]
            set xmin [expr {$dx1/$PIX*$CCDscale_standard}]
            # set xmax [expr {$dx2/$focscale}]
            set xmax [expr {$dx2/$PIX*$CCDscale_standard}]
            set dx [expr {$xmax - $xmin}]
            if { $debug & $verbose } {
                puts stdout "            Focal Field center"
                puts stdout "       CCDscale_standard = $CCDscale_standard"
                puts stdout "        CCDscale_highres = $CCDscale_highres"
                puts stdout "                  as2pix = $as2pix"
                puts stdout "                  pixmin = $pixmin"
                puts stdout "                  pixmax = $pixmax"
                puts stdout "                     wx1 = $wx1"
                puts stdout "                     wy1 = $wy1"
                puts stdout "                     wx2 = $wx2"
                puts stdout "                     wy2 = $wy2"
                puts stdout "                     wx3 = $wx3"
                puts stdout "                     wy3 = $wy3"
                puts stdout "                    xmin = $xmin \[mm\]"
                puts stdout "                    xmax = $xmax \[mm\]"
                puts stdout "                      dx = $dx \[mm\]"
            }
        }
        # PBG 20090803: This code is under development. END

            if { $debug && $verbose } {
                puts stdout         "            SR X range from ISF adjusted for $lmod:"
                puts stdout [format "                  XSRMIN = %8.2f (mm)" $XSRMIN]
                puts stdout [format "                  XSRMAX = %8.2f (mm)" $XSRMAX]
                puts stdout [format "                  YSRMIN = %8.2f (mm)" $YSRMIN]
                puts stdout [format "                  YSRMAX = %8.2f (mm)" $YSRMAX]
            }
            # These were empirically determined by trial and error.
            set XSRMIN -82.4
            set XSRMAX -76.4
            # set XSRMAX -50.0
            set XHRMIN -55.0
            set XHRMAX -50.0
            if { $debug && $verbose } {
                puts stdout         " "
                puts stdout [format "             new  XSRMIN = %8.2f (mm)" $XSRMIN]
                puts stdout [format "             new  XSRMAX = %8.2f (mm)" $XSRMAX]
            }
        }
        hits {
        }
        img {
        }
        lss {
        }
        mos {
        }
        mxu {
            if { $debug && $verbose } {
                puts stdout         "            SR X range from ISF adjusted for $lmod:"
                puts stdout [format "                  XSRMIN = %8.2f (mm)" $XSRMIN]
                puts stdout [format "                  XSRMAX = %8.2f (mm)" $XSRMAX]
                puts stdout [format "                  YSRMIN = %8.2f (mm)" $YSRMIN]
                puts stdout [format "                  YSRMAX = %8.2f (mm)" $YSRMAX]
            }
            set XSRMIN -90.0
            set XSRMAX  90.0
            # set YSRMIN -115.0
            # set YSRMAX  115.0
            if { $debug && $verbose } {
                puts stdout         " "
                puts stdout [format "             new  XSRMIN = %8.2f (mm)" $XSRMIN]
                puts stdout [format "             new  XSRMAX = %8.2f (mm)" $XSRMAX]
            }
        }
        pmos {
        }
        default {
            if { $debug && $verbose } {
                puts stdout "            Warning, unknown modus >$lmod<"
            }
        }
    }

    lassign [calc_ctom FOC    $resol] ctom loc_m loc_m1 p1x p1y p2x p2y XMIN YMIN XMAX YMAX
    lassign [calc_ctom MASTER $resol] ccdm_ctom ccdm_loc_m ccdm_loc_m1 ccdm1x ccdm1y ccdm2x ccdm2y CCDM_XMIN CCDM_YMIN CCDM_XMAX CCDM_YMAX
    lassign [calc_ctom SLAVE  $resol] ccds_ctom ccds_loc_m ccds_loc_m1 ccds1x ccds1y ccds2x ccds2y CCDS_XMIN CCDS_YMIN CCDS_XMAX CCDS_YMAX

    if { $debug && $verbose } {
        puts stdout         "            Canvas to Millimeter conversions: $resol"
        puts stdout         "                                FOC   MASTER    SLAVE"
        puts stdout [format "                    CTOM = %8.2f %8.2f %8.2f" $ctom $ccdm_ctom $ccds_ctom]
        puts stdout [format "                   LOC_M = %8.2f %8.2f %8.2f" $loc_m $ccdm_loc_m $ccds_loc_m]
        puts stdout [format "                  LOC_M1 = %8.2f %8.2f %8.2f" $loc_m1 $ccdm_loc_m1 $ccds_loc_m1]
        puts stdout [format "                      1X = %8.2f %8.2f %8.2f" $p1x $ccdm1x $ccds1x]
        puts stdout [format "                      1Y = %8.2f %8.2f %8.2f" $p1y $ccdm1y $ccds1y]
        puts stdout [format "                      2X = %8.2f %8.2f %8.2f" $p2x $ccdm2x $ccds2x]
        puts stdout [format "                      2Y = %8.2f %8.2f %8.2f" $p2y $ccdm2y $ccds2y]
        puts stdout [format "                    XMIN = %8.2f %8.2f %8.2f" $XMIN $CCDM_XMIN $CCDS_XMIN]
        puts stdout [format "                    YMIN = %8.2f %8.2f %8.2f" $YMIN $CCDM_YMIN $CCDS_YMIN]
        puts stdout [format "                    XMAX = %8.2f %8.2f %8.2f" $XMAX $CCDM_XMAX $CCDS_XMAX]
        puts stdout [format "                    YMAX = %8.2f %8.2f %8.2f" $YMAX $CCDM_YMAX $CCDS_YMAX]
        puts stdout ""
    }

    # The slope of the mask can be infinity in canvas coords,
    # this case has to be handled differently
    # ne_flag = 1 means, image is not rotated,

    set ne_flag 0
    if { [expr {abs($p1x - $p2x)}] < 1e-4 } {
        # Image is not rotated
        set ne_flag 1
    }
    if { $debug && $verbose } {
        puts stdout [format "                   slope = %e" [expr {abs($p1x - $p2x)}]]
        puts stdout         "                 ne_flag = $ne_flag"
        if { $ne_flag } {
            puts stdout     "              frame || WCS -> not rotated"
        } else {
            puts stdout     "              frame \\/ WCS -> rotated"
        }
        puts stdout ""
    }
 
    # Factors for perpendicular case (pixel).
    # They are zero if rotated.
    # fits_x_scale is the sign of the X canvas scale towards positive Alpha
    set FMIN [expr {$fits_x_scale*$ctom*$XMIN}]
    set FMAX [expr {$fits_x_scale*$ctom*$XMAX}]
    set CCDM_FMIN [expr {$fits_x_scale*$ccdm_ctom*$CCDM_XMIN}]
    set CCDS_FMIN [expr {$fits_x_scale*$ccds_ctom*$CCDS_XMIN}]
    set CCDM_FMAX [expr {$fits_x_scale*$ccdm_ctom*$CCDM_XMAX}]
    set CCDS_FMAX [expr {$fits_x_scale*$ccds_ctom*$CCDS_XMAX}]

    if { $debug && $verbose } {
        puts stdout [format "                    FMIN = %8.2f %8.2f %8.2f" $FMIN $CCDM_FMIN $CCDS_FMIN]
        puts stdout [format "                    FMAX = %8.2f %8.2f %8.2f" $FMAX $CCDM_FMAX $CCDS_FMAX]
    }

    if { !$ne_flag } {
        # The coord axis are not perpendicular to the frame.
        # Factors for rotated case (pixel).
        # They are zero if perpendicular.
        # CRO is the sign of the rotator angle.
        set RMIN [expr {$FMIN*$CRO*$loc_m1}]
        set RMAX [expr {$FMAX*$CRO*$loc_m1}]
        set CCDM_RMIN [expr {$CCDM_FMIN*$CRO*$ccdm_loc_m1}]
        set CCDS_RMIN [expr {$CCDS_FMIN*$CRO*$ccds_loc_m1}]
        set CCDM_RMAX [expr {$CCDM_FMAX*$CRO*$ccdm_loc_m1}]
        set CCDS_RMAX [expr {$CCDS_FMAX*$CRO*$ccds_loc_m1}]
        # The F factors were included in the rotated R factors, now we zero them.
        set FMIN 0
        set FMAX 0
        set CCDM_FMIN 0
        set CCDS_FMIN 0
        set CCDM_FMAX 0
        set CCDS_FMAX 0
        if { $debug && $verbose } {
            puts stdout "               axis not perpendicular to frame"
        }
    } else {
        # The coord axis is perpendicular to the frame.
        # The slope and therefore the rotation factors are zero.
        set loc_m 0
        set ccdm_loc_m 0
        set ccds_loc_m 0
        set RMIN 0
        set RMAX 0
        set CCDM_RMIN 0
        set CCDS_RMIN 0
        set CCDM_RMAX 0
        set CCDS_RMAX 0
        if { $debug && $verbose } {
            puts stdout "               axis perpendicular to frame"
        }
    }
 
    if { $debug && $verbose } {
        puts stdout [format "                    RMIN = %8.2f %8.2f %8.2f" $RMIN $CCDM_RMIN $CCDS_RMIN]
        puts stdout [format "                    RMAX = %8.2f %8.2f %8.2f" $RMAX $CCDM_RMAX $CCDS_RMAX]
    }

    # Focal Field frame in the canvas (pixel)
    # The original orientation values are kept in the OF, OM, OS slots
    # - lower = south part
    set mos(OF,y$a) [expr {$p1y - $RMIN}]
    set mos(OF,x$a) [expr {$p1x + $FMIN - $loc_m*($mos(OF,y$a) - $p1y)}]
    set mos(OF,y$b) [expr {$p1y - $RMAX}]
    set mos(OF,x$b) [expr {$p1x + $FMAX - $loc_m*($mos(OF,y$b) - $p1y)}]
    # - upper = northern part
    set mos(OF,y$c) [expr {$p2y - $RMAX}]
    set mos(OF,x$c) [expr {$p2x + $FMAX - $loc_m*($mos(OF,y$c) - $p2y)}]
    set mos(OF,y$d) [expr {$p2y - $RMIN}]
    set mos(OF,x$d) [expr {$p2x + $FMIN - $loc_m*($mos(OF,y$d) - $p2y)}]

    # Master CCD frame in the canvas (pixel)
    # - lower = south part
    set ccd_frame(OM,y$a) [expr {$ccdm1y - $CCDM_RMIN}]
    set ccd_frame(OM,x$a) [expr {$ccdm1x + $CCDM_FMIN - $ccdm_loc_m*($ccd_frame(OM,y$a) - $ccdm1y)}]
    set ccd_frame(OM,y$b) [expr {$ccdm1y - $CCDM_RMAX}]
    set ccd_frame(OM,x$b) [expr {$ccdm1x + $CCDM_FMAX - $ccdm_loc_m*($ccd_frame(OM,y$b) - $ccdm1y)}]
    # - upper = northern part
    set ccd_frame(OM,y$c) [expr {$ccdm2y - $CCDM_RMAX}]
    set ccd_frame(OM,x$c) [expr {$ccdm2x + $CCDM_FMAX - $ccdm_loc_m*($ccd_frame(OM,y$c) - $ccdm2y)}]
    set ccd_frame(OM,y$d) [expr {$ccdm2y - $CCDM_RMIN}]
    set ccd_frame(OM,x$d) [expr {$ccdm2x + $CCDM_FMIN - $ccdm_loc_m*($ccd_frame(OM,y$d) - $ccdm2y)}]

    # Slave CCD frame in the canvas (pixel)
    # - lower = south part
    set ccd_frame(OS,y$a) [expr {$ccds1y - $CCDS_RMIN}]
    set ccd_frame(OS,x$a) [expr {$ccds1x + $CCDS_FMIN - $ccds_loc_m*($ccd_frame(OS,y$a) - $ccds1y)}]
    set ccd_frame(OS,y$b) [expr {$ccds1y - $CCDS_RMAX}]
    set ccd_frame(OS,x$b) [expr {$ccds1x + $CCDS_FMAX - $ccds_loc_m*($ccd_frame(OS,y$b) - $ccds1y)}]
    # - upper = northern part
    set ccd_frame(OS,y$c) [expr {$ccds2y - $CCDS_RMAX}]
    set ccd_frame(OS,x$c) [expr {$ccds2x + $CCDS_FMAX - $ccds_loc_m*($ccd_frame(OS,y$c) - $ccds2y)}]
    set ccd_frame(OS,y$d) [expr {$ccds2y - $CCDS_RMIN}]
    set ccd_frame(OS,x$d) [expr {$ccds2x + $CCDS_FMIN - $ccds_loc_m*($ccd_frame(OS,y$d) - $ccds2y)}]

    # Label placeholders
    set s [expr {-1.0*$fits_x_scale}]
    # Top label
    set mos(OF,x$tl) [expr {($mos(OF,x$a) + $mos(OF,x$b))/2.0}]
    set mos(OF,y$tl) [expr {($mos(OF,y$a) + $mos(OF,y$b))/2.0 + 10*$s}]
    set ccd_frame(OM,x$tl) [expr {($ccd_frame(OM,x$a) + $ccd_frame(OM,x$b))/2.0}]
    set ccd_frame(OM,y$tl) [expr {($ccd_frame(OM,y$a) + $ccd_frame(OM,y$b))/2.0 + 10*$s}]
    set ccd_frame(OS,x$tl) 0
    set ccd_frame(OS,y$tl) 0
    # Bottom label
    set mos(OF,x$bl) [expr {($mos(OF,x$c) + $mos(OF,x$d))/2.0}]
    set mos(OF,y$bl) [expr {($mos(OF,y$c) + $mos(OF,y$d))/2.0 - 10*$s}]
    set ccd_frame(OM,x$bl) 0
    set ccd_frame(OM,y$bl) 0
    set ccd_frame(OS,x$bl) [expr {($ccd_frame(OS,x$c) + $ccd_frame(OS,x$d))/2.0}]
    set ccd_frame(OS,y$bl) [expr {($ccd_frame(OS,y$c) + $ccd_frame(OS,y$d))/2.0 - 10*$s}]
    # East label
    set mos(OF,x$el) [expr {($mos(OF,x$a) + $mos(OF,x$d))/2.0 + 35*$s}]
    set mos(OF,y$el) [expr {($mos(OF,y$a) + $mos(OF,y$d))/2.0}]
    set ccd_frame(OM,x$el) 0
    set ccd_frame(OM,y$el) 0
    set ccd_frame(OS,x$el) 0
    set ccd_frame(OS,y$el) 0
    # West label
    set mos(OF,x$wl) [expr {($mos(OF,x$b) + $mos(OF,x$c))/2.0 - 35*$s}]
    set mos(OF,y$wl) [expr {($mos(OF,y$b) + $mos(OF,y$c))/2.0}]
    set ccd_frame(OM,x$wl) 0
    set ccd_frame(OM,y$wl) 0
    set ccd_frame(OS,x$wl) 0
    set ccd_frame(OS,y$wl) 0

    # The 0 and 1 slots are used by the plotting methods
    if { $mountangle == 180 } {
        if { $debug && $verbose } {
            puts stdout "               frame has been rotated 180 (deg) because of mount angle"
        }
        set cda [expr {cos(180*$deg2rad)}]
        set sda [expr {sin(180*$deg2rad)}]
        foreach k "$a $b $c $d $tl $bl $el $wl" {
            set d1 [expr {$ccx + ($mos(OF,x$k) - $ccx)*$cda - ($mos(OF,y$k) - $ccy)*$sda}]
            set d2 [expr {$ccy + ($mos(OF,x$k) - $ccx)*$sda + ($mos(OF,y$k) - $ccy)*$cda}]
            set mos(0,x$k) $d1
            set mos(0,y$k) $d2
            set ccdm_d1 [expr $ccx + ($ccd_frame(OM,x$k) - $ccx)*$cda - ($ccd_frame(OM,y$k) - $ccy)*$sda]
            set ccdm_d2 [expr $ccy + ($ccd_frame(OM,x$k) - $ccx)*$sda + ($ccd_frame(OM,y$k) - $ccy)*$cda]
            set ccd_frame(0,x$k) $ccdm_d1
            set ccd_frame(0,y$k) $ccdm_d2
            set ccds_d1 [expr $ccx + ($ccd_frame(OS,x$k) - $ccx)*$cda - ($ccd_frame(OS,y$k) - $ccy)*$sda]
            set ccds_d2 [expr $ccy + ($ccd_frame(OS,x$k) - $ccx)*$sda + ($ccd_frame(OS,y$k) - $ccy)*$cda]
            set ccd_frame(1,x$k) $ccds_d1
            set ccd_frame(1,y$k) $ccds_d2
        }
    } else {
        foreach k "$a $b $c $d $tl $bl $el $wl" {
            set mos(0,x$k) $mos(OF,x$k)
            set mos(0,y$k) $mos(OF,y$k)
            set ccd_frame(0,x$k) $ccd_frame(OM,x$k)
            set ccd_frame(0,y$k) $ccd_frame(OM,y$k)
            set ccd_frame(1,x$k) $ccd_frame(OS,x$k)
            set ccd_frame(1,y$k) $ccd_frame(OS,y$k)
        }
    }

    # The central green circle
    # mos(0,x/y) in canvas  coords,
    # ccx ccy current canvas coords
    set oldpos(x) $ccx
    set oldpos(y) $ccy
    set scal [expr {$atd * $focscale}]
    puts $lfId "New center ($ccx,$ccy) = ($wx,$wy)"
    # The radius of the green circle in world coords
    set mos(0,r) [expr {$scal*$XSRMAX/40}]
    if { $bonsai_flag } {
        # radius = 0.4 of a MOS stripe
        # 1 MOS stripe ~= 1/(18*sqrt(2)) of diagonal
        set dumx [expr {($mos(0,xa) - $mos(0,xc))/50.0}]
        set dumy [expr {($mos(0,ya) - $mos(0,yc))/50.0}]
        set mos(0,r) [expr {sqrt($dumx*$dumx + $dumy*$dumy)*0.4}]
    } else {
       set dummy 0
       catch { $m_image convert dist  $mos(0,r) 0 "deg $EQX" mos(0,r) dummy canvas }
       if { $mos(0,r) < $dummy } {
           set mos(0,r) $dummy
       }
    }

    if { $debug && $verbose } {
        puts stdout         " "
        puts stdout         "            Frame corners in the canvas (pixel)"
        puts stdout         "                      South (Geo)(MOS 1)"
        puts stdout [format "            FOC_%s:  a(%8.2f,%8.2f)   b(%8.2f, %8.2f)" $resol $mos(0,x$a) $mos(0,y$a) $mos(0,x$b) $mos(0,y$b)]
        puts stdout [format "            CCD_%s: Ma(%8.2f,%8.2f)  Mb(%8.2f, %8.2f)" $resol $ccd_frame(0,x$a) $ccd_frame(0,y$a) $ccd_frame(0,x$b) $ccd_frame(0,y$b)]
        puts stdout [format "       BLUE         Sa(%8.2f,%8.2f)  Sb(%8.2f, %8.2f)    RED" $ccd_frame(1,x$a) $ccd_frame(1,y$a) $ccd_frame(1,x$b) $ccd_frame(1,y$b)]
        puts stdout         " "
        puts stdout         "                      North (Geo)(MOS 19)"
        puts stdout [format "            FOC_%s:  d(%8.2f,%8.2f)   c(%8.2f, %8.2f)" $resol $mos(0,x$d) $mos(0,y$d) $mos(0,x$c) $mos(0,y$c)]
        puts stdout [format "            CCD_%s: Md(%8.2f,%8.2f)  Mc(%8.2f, %8.2f)" $resol $ccd_frame(0,x$d) $ccd_frame(0,y$d) $ccd_frame(0,x$c) $ccd_frame(0,y$c)]
        puts stdout [format "       BLUE         Sd(%8.2f,%8.2f)  Sc(%8.2f, %8.2f)    RED" $ccd_frame(1,x$d) $ccd_frame(1,y$d) $ccd_frame(1,x$c) $ccd_frame(1,y$c)]

        # If you need to see the corners plotted on the canvas:
        # set R [expr {0.3 * $mos(0,r)}]
        # catch {$m_canvas delete f_corner$resol}
        # foreach k "$a $b $c $d" {
            # Change resolution SR or HR according to the corners you need to see
            # if { [string compare $resol SR] == 0 } {
                # Choose between name only or name and variable or name and coordinates formats
                # set foc_text  [format "F_%s" ${resol}$k]
                # set foc_text  [format "F_%s(%s)" ${resol}$k "mos(0,x/y$k)"]
                # set foc_text  [format "F_%s(%7.2f,%7.2f)" ${resol}$k $mos(0,x$k) $mos(0,y$k)]

                # set ccdm_text [format "M_%s" ${resol}$k]
                # set ccdm_text [format "M_%s(%s)" ${resol}$k "ccd_frame(0,x/y$k)"]
                # set ccdm_text [format "M_%s(%7.2f,%7.2f)" ${resol}$k $ccd_frame(0,x$k) $ccd_frame(0,y$k)]

                # set ccds_text [format "S_%s" ${resol}$k]
                # set ccds_text [format "S_%s(%s)" ${resol}$k "ccd_frame(1,x/y$k)"]
                # set ccds_text [format "S_%s(%7.2f,%7.2f)" ${resol}$k $ccd_frame(1,x$k) $ccd_frame(1,y$k)]

                # Plot the corner dots
                # $m_canvas create oval [expr {$mos(0,x$k)-$R}] \
                                      [expr {$mos(0,y$k)-$R}] \
                                      [expr {$mos(0,x$k)+$R}] \
                                      [expr {$mos(0,y$k)+$R}] \
                                      -outline red \
                                      -fill yellow \
                                      -tag "r_corner$resol"
                # $m_canvas create oval [expr {$ccd_frame(0,x$k)-$R}] \
                                      [expr {$ccd_frame(0,y$k)-$R}] \
                                      [expr {$ccd_frame(0,x$k)+$R}] \
                                      [expr {$ccd_frame(0,y$k)+$R}] \
                                      -outline red \
                                      -fill yellow \
                                      -tag "r_corner$resol"
                # $m_canvas create oval [expr {$ccd_frame(1,x$k)-$R}] \
                                      [expr {$ccd_frame(1,y$k)-$R}] \
                                      [expr {$ccd_frame(1,x$k)+$R}] \
                                      [expr {$ccd_frame(1,y$k)+$R}] \
                                      -outline red \
                                      -fill yellow \
                                      -tag "r_corner$resol"
                # Plot the corner labels
                # $m_canvas create text $mos(0,x$k) $mos(0,y$k) -anchor n -fill yellow -text $foc_text -tag "f_corner$resol"
                # $m_canvas create text $ccd_frame(0,x$k) $ccd_frame(0,y$k) -anchor n -fill yellow -text $ccdm_text -tag "f_corner$resol"
                # $m_canvas create text $ccd_frame(1,x$k) $ccd_frame(1,y$k) -anchor n -fill yellow -text $ccds_text -tag "f_corner$resol"
            # }
        # }
    }

    if { $debug } {
        puts stdout         " "
        puts stdout         "            Frame center"
        puts stdout [format "            New Center: (%8.2f,%8.2f) \[pixel\] = (%8.2f,%8.2f) \[arcsec\]" $ccx $ccy $wx $wy]
        puts stdout [format "                Radius: mos(0,r) = %7.2f  \[pixel\]" $mos(0,r)]
    }

    
    # Initialize coordinate transformation matrices
    update_xform_coeff

    # The dispersion frame for spectroscopic modes
    calculate_dispersion_frame $resol

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "calculate_focalfield_frame: END $resol"
    }
    tracer LEAVING
}

#=========================================================
private method calculate_dispersion_frame { {resol SR} } {
#=========================================================
# By P. Baksai
# Used by MXU and HIT-MS modes to calculate the wavelength
# dispersion.
# Method wave_limit MUST be called before this one.

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "calculate_dispersion_frame: BEGIN $resol"
    }

    # Calculate canvas size in (mm) and the slope of the WCS axis in the canvas
    lassign [calc_ctom FOC    $resol] ctom loc_m loc_m1 p1x p1y p2x p2y XMIN YMIN XMAX YMAX
    lassign [calc_ctom MASTER $resol] ccdm_ctom ccdm_loc_m ccdm_loc_m1 ccdm1x ccdm1y ccdm2x ccdm2y CCDM_XMIN CCDM_YMIN CCDM_XMAX CCDM_YMAX
    lassign [calc_ctom SLAVE  $resol] ccds_ctom ccds_loc_m ccds_loc_m1 ccds1x ccds1y ccds2x ccds2y CCDS_XMIN CCDS_YMIN CCDS_XMAX CCDS_YMAX

    if { $debug && $verbose } {
        puts stdout         "            Canvas to Millimeter conversions: $resol"
        puts stdout         "                                FOC   MASTER    SLAVE"
        puts stdout [format "                    CTOM = %8.2f %8.2f %8.2f" $ctom $ccdm_ctom $ccds_ctom]
        puts stdout [format "                   LOC_M = %8.2f %8.2f %8.2f" $loc_m $ccdm_loc_m $ccds_loc_m]
        puts stdout [format "                  LOC_M1 = %8.2f %8.2f %8.2f" $loc_m1 $ccdm_loc_m1 $ccds_loc_m1]
        puts stdout [format "                      1X = %8.2f %8.2f %8.2f" $p1x $ccdm1x $ccds1x]
        puts stdout [format "                      1Y = %8.2f %8.2f %8.2f" $p1y $ccdm1y $ccds1y]
        puts stdout [format "                      2X = %8.2f %8.2f %8.2f" $p2x $ccdm2x $ccds2x]
        puts stdout [format "                      2Y = %8.2f %8.2f %8.2f" $p2y $ccdm2y $ccds2y]
        puts stdout [format "                    XMIN = %8.2f %8.2f %8.2f" $XMIN $CCDM_XMIN $CCDS_XMIN]
        puts stdout [format "                    YMIN = %8.2f %8.2f %8.2f" $YMIN $CCDM_YMIN $CCDS_YMIN]
        puts stdout [format "                    XMAX = %8.2f %8.2f %8.2f" $XMAX $CCDM_XMAX $CCDS_XMAX]
        puts stdout [format "                    YMAX = %8.2f %8.2f %8.2f" $YMAX $CCDM_YMAX $CCDS_YMAX]
        puts stdout ""
    }

    # Select resolution parameters
    if { [string match *SR* $resol] } {
        set resol SR
        # SR Spectrum range (mm): spectrum length from central wavelength to
        # blue (min or shorter) and red (max or longer) ends
        set smin $specmin
        set smax $specmax
        # Corner slots of the Frame Arrays used by plot_mask method
        # - Corners of the focal field (already calculated in calculate_focalfield_frame)
        lassign {a b c d} a b c d
        # - Corners of the reduced focal field (to be calculated here)
        lassign {ar br cr dr} ar br cr dr
        # Start and finish corner slots for big blue and red crosses.
        lassign {sb1s sb1f sb2s sb2f sr1s sr1f sr2s sr2f} b1s b1f b2s b2f r1s r1f r2s r2f
        # Label placeholder slots
        lassign {slbl slrl} lbl lrl
        if { $debug && !$verbose } {
            puts stdout "calculate_dispersion_frame: Standard Resolution"
        }
    } else {
        set resol HR
        # HR Spectrum range (mm)
        set smin $specminHR
        set smax $specmaxHR
        # Corner slots of the Frame Arrays used by plot_mask method
        lassign {e f g h} a b c d
        lassign {er fr gr hr} ar br cr dr
        # Start and finish corner slots for big blue and red crosses.
        lassign {hb1s hb1f hb2s hb2f hr1s hr1f hr2s hr2f} b1s b1f b2s b2f r1s r1f r2s r2f
        # Label placeholder slots
        lassign {hlbl hlrl} lbl lrl
        if { $debug && !$verbose } {
            puts stdout "calculate_dispersion_frame: High Resolution"
        }
    }

    # Select dispersion parameters
    set fits_scale 1
    if { $disp_ang == 0.0 || $disp_ang == -180.0 || $disp_ang == 180.0 } {
        # Normal dispersion along X axis on the focal plane
        # (grism angle = 0 degrees from X axis)
        set normal_disp 1
        #
        # For big crosses placeholders
        set blu1_s $ar
        set blu1_f $d
        set blu2_s $dr
        set blu2_f $a
        set red1_s $br
        set red1_f $c
        set red2_s $cr
        set red2_f $b
        # For lambda label placeholders
        set lb $d
        set lr $c
        #
        # Assume dispersion direction blue towards Blade-A,
        #                              red towards Blade-B
        if { $disp_ang == -180.0 || $disp_ang == 180.0 } {
            set fits_scale [expr {-$fits_scale}]
            # Invert big crosses finish points
            set blu1_f $c
            set blu2_f $b
            set red1_f $d
            set red2_f $a
            # Invert labels, blue is Blade-B
            set lb $c
            set lr $d
        }
        # The distance to the edges of the CCD that would cut the spectrum
        set MIN $CCDM_XMIN
        set MAX $CCDM_XMAX
        if { $debug && $verbose } {
            puts stdout "                            normal dispersion"
        }
    } else {
        # Cross dispersion along Y axis on the focal plane
        # (grism angle = -90 degrees counter clockwise from X axis)
        set normal_disp 0
        #
        # For big crosses placeholders
        set blu1_s $ar
        set blu1_f $c
        set blu2_s $dr
        set blu2_f $d
        set red1_s $br
        set red1_f $b
        set red2_s $cr
        set red2_f $a
        # For lambda label placeholders
        set lb $c
        set lr $b
        #
        # Assume dispersion direction blue towards Bottom slit 19,
        #                              red towards Top slit 1
        # Whatever angle comes from the ISF,
        # we take its sign and assume it to be 90 degrees
        set s [expr {$disp_ang/abs($disp_ang)}]
        if { $s == 1 } {
            set fits_scale [expr {-$fits_scale}]
            # Invert big crosses finish points
            set blu1_f $b
            set blu2_f $a
            set red1_f $c
            set red2_f $d
            # Invert labels, blue is Top
            set lb $b
            set lr $c
        }
        # The distance to the edges of the CCD that would cut the spectrum
        set MIN $CCDM_YMIN
        set MAX $CCDS_YMAX
        if { $debug && $verbose } {
            puts stdout "                            cross dispersion"
        }
    }

    # Select reduced focal field limits (mm)
    set lmod [$this cget -modus]
    switch -exact -- $lmod {
        hits {
            set FOC_LMIN  [expr {$htslitxmax($long_number) + $smax}]
            set FOC_LMAX  [expr {$htslitxmin($long_number) - $smin}]
        }
        lss {
            set FOC_LMIN  [expr {$longslitxmax($long_number) + $smax}]
            set FOC_LMAX  [expr {$longslitxmin($long_number) - $smin}]
        }
        hitms -
        mos -
        mxu -
        pmos {
            # Limit towards BLUE: is located at smin (mm) from the MIN CCD edge
            set FOC_LMIN  [expr {+$MIN + $smin}]
            # Limit towards RED: is located at smax (mm) from the MAX CCD edge
            set FOC_LMAX  [expr {+$MAX - $smax}]
        }
        default {
            set FOC_LMIN  $MIN
            set FOC_LMAX  $MAX
            if { $debug && $verbose } {
                puts stdout "calculate_dispersion_frame: Warning, unknown modus >$lmod<"
            }
        }
    }

    # foc_frame array: slot 0 for reduced frame position in (mm)
    if { $normal_disp } {
        # Dispersion distances (mm)
        set foc_frame(0,x$ar) [expr {$smin?[expr {$fits_scale*$FOC_LMIN}]:$CCDM_XMIN}]; set foc_frame(0,y$ar) $YMIN
        set foc_frame(0,x$br) [expr {$smax?[expr {$fits_scale*$FOC_LMAX}]:$CCDM_XMAX}]; set foc_frame(0,y$br) $YMIN
        set foc_frame(0,x$cr) [expr {$smax?[expr {$fits_scale*$FOC_LMAX}]:$CCDM_XMAX}]; set foc_frame(0,y$cr) $YMAX
        set foc_frame(0,x$dr) [expr {$smin?[expr {$fits_scale*$FOC_LMIN}]:$CCDM_XMIN}]; set foc_frame(0,y$dr) $YMAX
    } else {
        # Dispersion distances (mm)
        set foc_frame(0,x$ar) $XMIN; set foc_frame(0,y$ar) [expr {$smax?[expr {$fits_scale*$FOC_LMAX}]:$CCDM_YMIN}]
        set foc_frame(0,x$br) $XMIN; set foc_frame(0,y$br) [expr {$smin?[expr {$fits_scale*$FOC_LMIN}]:$CCDM_YMIN}]
        set foc_frame(0,x$cr) $XMAX; set foc_frame(0,y$cr) [expr {$smin?[expr {$fits_scale*$FOC_LMIN}]:$CCDS_YMAX}]
        set foc_frame(0,x$dr) $XMAX; set foc_frame(0,y$dr) [expr {$smax?[expr {$fits_scale*$FOC_LMAX}]:$CCDS_YMAX}]
    }

    if { $debug && $verbose } {
        puts stdout         "            Mask Slope:"
        puts stdout [format "                   slope = %e" [expr {abs($p1x - $p2x)}]]
        if { [expr {abs($p1x - $p2x)}] != 0.0 } {
            puts stdout     "              frame \\/  WCS -> rotated wrt canvas"
        } else {
            puts stdout     "              frame || WCS -> not rotated wrt canvas"
        }
        puts stdout         " "
        puts stdout         "            Focal Field $resol:"
        puts stdout         "              -Reduced Frame:"
        puts stdout [format "                    XMIN = %8.2f \[mm\]" $XMIN]
        puts stdout [format "                    YMIN = %8.2f \[mm\]" $YMIN]
        puts stdout [format "                    XMAX = %8.2f \[mm\]" $XMAX]
        puts stdout [format "                    YMAX = %8.2f \[mm\]" $YMAX]
        puts stdout         "              -CCD Master Frame:"
        puts stdout [format "               CCDM_XMIN = %8.2f \[mm\]" $CCDM_XMIN]
        puts stdout [format "               CCDM_YMIN = %8.2f \[mm\]" $CCDM_YMIN]
        puts stdout [format "               CCDM_XMAX = %8.2f \[mm\]" $CCDM_XMAX]
        puts stdout [format "               CCDM_YMAX = %8.2f \[mm\]" $CCDM_YMAX]
        puts stdout         "              -CCD Slave Frame:"
        puts stdout [format "               CCDS_XMIN = %8.2f \[mm\]" $CCDS_XMIN]
        puts stdout [format "               CCDS_YMIN = %8.2f \[mm\]" $CCDS_YMIN]
        puts stdout [format "               CCDS_XMAX = %8.2f \[mm\]" $CCDS_XMAX]
        puts stdout [format "               CCDS_YMAX = %8.2f \[mm\]" $CCDS_YMAX]
        puts stdout         " "
        puts stdout         "            Spectrum Range $resol:"
        puts stdout [format "                    smin = %8.2f \[mm\]" $smin]
        puts stdout [format "                    smax = %8.2f \[mm\]" $smax]
        puts stdout         " "
        puts stdout         "            Spectrum Dispersion Angle:"
        puts stdout [format "              fits_scale = %8.2f" $fits_scale]
        puts stdout [format "                disp_ang = %8.2f \[deg\]" $disp_ang]
        puts stdout [format "            disp_ang_rad = %8.2f \[rad\]" $disp_ang_rad]
        puts stdout         " "
        puts stdout         "            Focal Limits:"
        puts stdout [format "                     MIN = %8.2f \[mm\]" $MIN]
        puts stdout [format "                     MAX = %8.2f \[mm\]" $MAX]
        puts stdout [format "                FOC_LMIN = %8.2f \[mm\]" $FOC_LMIN]
        puts stdout [format "                FOC_LMAX = %8.2f \[mm\]" $FOC_LMAX]
        puts stdout         "            Dispersion Distances:"
        puts stdout [format "        foc_frame(0,x$ar) = %8.2f \[mm\]  foc_frame(0,y$ar) = %8.2f \[mm\]" $foc_frame(0,x$ar) $foc_frame(0,y$ar)]
        puts stdout [format "        foc_frame(0,x$br) = %8.2f \[mm\]  foc_frame(0,y$br) = %8.2f \[mm\]" $foc_frame(0,x$br) $foc_frame(0,y$br)]
        puts stdout [format "        foc_frame(0,x$cr) = %8.2f \[mm\]  foc_frame(0,y$cr) = %8.2f \[mm\]" $foc_frame(0,x$cr) $foc_frame(0,y$cr)]
        puts stdout [format "        foc_frame(0,x$dr) = %8.2f \[mm\]  foc_frame(0,y$dr) = %8.2f \[mm\]" $foc_frame(0,x$dr) $foc_frame(0,y$dr)]
    }

    # Dispersion Frame in the canvas (pixel)
    # mm focplane coords --> canvas coords
    C_mc coeff_mc $XMIN $YMIN $mos(OF,x$a) $mos(OF,y$a) \
                  $XMAX $YMIN $mos(OF,x$b) $mos(OF,y$b) \
                  $XMAX $YMAX $mos(OF,x$c) $mos(OF,y$c)
    # The original orientation values are kept in the OF, OM, OS slots
    # - lower = south part
    set buf [C_mc m2c $foc_frame(0,x$ar) $foc_frame(0,y$ar)]; set mos(OF,x$ar) [lindex $buf 0]; set mos(OF,y$ar) [lindex $buf 1]
    set buf [C_mc m2c $foc_frame(0,x$br) $foc_frame(0,y$br)]; set mos(OF,x$br) [lindex $buf 0]; set mos(OF,y$br) [lindex $buf 1]
    # - upper = northern part
    set buf [C_mc m2c $foc_frame(0,x$cr) $foc_frame(0,y$cr)]; set mos(OF,x$cr) [lindex $buf 0]; set mos(OF,y$cr) [lindex $buf 1]
    set buf [C_mc m2c $foc_frame(0,x$dr) $foc_frame(0,y$dr)]; set mos(OF,x$dr) [lindex $buf 0]; set mos(OF,y$dr) [lindex $buf 1]

    # Big crosses placeholders
    set mos(OF,x$b1s) $mos(OF,x$blu1_s)
    set mos(OF,y$b1s) $mos(OF,y$blu1_s)
    set mos(OF,x$b1f) $mos(OF,x$blu1_f)
    set mos(OF,y$b1f) $mos(OF,y$blu1_f)
    set mos(OF,x$b2s) $mos(OF,x$blu2_s)
    set mos(OF,y$b2s) $mos(OF,y$blu2_s)
    set mos(OF,x$b2f) $mos(OF,x$blu2_f)
    set mos(OF,y$b2f) $mos(OF,y$blu2_f)
    set mos(OF,x$r1s) $mos(OF,x$red1_s)
    set mos(OF,y$r1s) $mos(OF,y$red1_s)
    set mos(OF,x$r1f) $mos(OF,x$red1_f)
    set mos(OF,y$r1f) $mos(OF,y$red1_f)
    set mos(OF,x$r2s) $mos(OF,x$red2_s)
    set mos(OF,y$r2s) $mos(OF,y$red2_s)
    set mos(OF,x$r2f) $mos(OF,x$red2_f)
    set mos(OF,y$r2f) $mos(OF,y$red2_f)
    # Label placeholders
    # - Lambda Blue label
    set mos(OF,x$lbl) $mos(OF,x$lb)
    set mos(OF,y$lbl) $mos(OF,y$lb)
    # - Lambda Red label
    set mos(OF,x$lrl) $mos(OF,x$lr)
    set mos(OF,y$lrl) $mos(OF,y$lr)
    # This is for initialization of unused slots that will be 
    # later updated by methods rotate and shift.
    foreach k "$ar $br $cr $dr $b1s $b1f $b2s $b2f $r1s $r1f $r2s $r2f $lbl $lrl" {
        set ccd_frame(OM,x$k) 0
        set ccd_frame(OM,y$k) 0
        set ccd_frame(OS,x$k) 0
        set ccd_frame(OS,y$k) 0
        set ccd_frame(0,x$k)  0
        set ccd_frame(0,y$k)  0
        set ccd_frame(1,x$k)  0
        set ccd_frame(1,y$k)  0
    }

    # Shift reduced frame to the current center
    # The distance between initial and current
    # positions in canvas coordinates (pixel)
    if { $bonsai_flag } {
        set dum [C_wc w2c $wx $wy]
        set ocx [lindex $dum 0]
        set ocy [lindex $dum 1]
        if { $debug && $verbose } {
            puts stdout "            Bonsai matrix used"
        }
    } else {
        catch { $m_image convert coords $wx $wy "deg $EQX" ocx ocy canvas }
        if { $debug && $verbose } {
            puts stdout "            $m_image convert coords used"
        }
    }
    set shift_x [expr {$ccx - $ocx}]
    set shift_y [expr {$ccy - $ocy}]
    if { $debug && $verbose } {
        puts stdout "            frame has been shifted ($shift_x,$shift_y) (pixel)"
    }
    foreach k "$ar $br $cr $dr $b1s $b1f $b2s $b2f $r1s $r1f $r2s $r2f $lbl $lrl" {
        set mos(OF,x$k) [expr {$mos(OF,x$k) + $shift_x}]
        set mos(OF,y$k) [expr {$mos(OF,y$k) + $shift_y}]
    }

    # Rotate reduced frame to the current mask PA
    set cda [expr {cos($abs_ang_rad)}]
    set sda [expr {sin($abs_ang_rad)}]
    foreach k "$ar $br $cr $dr $b1s $b1f $b2s $b2f $r1s $r1f $r2s $r2f $lbl $lrl" {
        set d1 [expr {$ccx + ($mos(OF,x$k) - $ccx)*$cda - ($mos(OF,y$k) - $ccy)*$sda}]
        set d2 [expr {$ccy + ($mos(OF,x$k) - $ccx)*$sda + ($mos(OF,y$k) - $ccy)*$cda}]
        set mos(OF,x$k) $d1
        set mos(OF,y$k) $d2
    }
    if { $debug && $verbose } {
        puts stdout "            frame has been rotated $abs_ang_rad (rad) because of ROT angle"
    }

    # If it is a new mode or has been reset, hold the initial canvas scale
    # in order to correct future dispersion frames for zoom factor
    if { $refscale == 0 } {
        set refscale $canvscale
    }

    # Stretch if zoom level has changed with respect to
    # the first time the dispersion frame was calculated
    set stretch [expr (1.0*$canvscale)/(1.0*$refscale)]
    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "               refscale  = $refscale"
        puts stdout "               canvscale = $canvscale"
        puts stdout "               stretch   = $stretch"
    }

    if { $stretch != 1 } {
        # Rescale the size of the dispersion frame according to the current zoom
        foreach k "$ar $br $cr $dr $b1s $b1f $b2s $b2f $r1s $r1f $r2s $r2f $lbl $lrl" {
            set mos(OF,x$k) [expr {$mos(OF,x$k)*$stretch}]
            set mos(OF,y$k) [expr {$mos(OF,y$k)*$stretch}]
        }
        if { $debug && $verbose } {
            puts stdout "            frame has been stretched by $stretch () because of ZOOM"
        }
    }
 
    # The 0 and 1 slots are used by the plotting methods
    if { $mountangle == 180 } {
        set cda [expr {cos(180*$deg2rad)}]
        set sda [expr {sin(180*$deg2rad)}]
        foreach k "$ar $br $cr $dr $b1s $b1f $b2s $b2f $r1s $r1f $r2s $r2f $lbl $lrl" {
            set d1 [expr {$ccx + ($mos(OF,x$k) - $ccx)*$cda - ($mos(OF,y$k) - $ccy)*$sda}]
            set d2 [expr {$ccy + ($mos(OF,x$k) - $ccx)*$sda + ($mos(OF,y$k) - $ccy)*$cda}]
            set mos(0,x$k) $d1
            set mos(0,y$k) $d2
        }
        if { $debug && $verbose } {
            puts stdout "            frame has been rotated 180 (deg) because of mount angle"
        }
    } else {
        foreach k "$ar $br $cr $dr $b1s $b1f $b2s $b2f $r1s $r1f $r2s $r2f $lbl $lrl" {
            set mos(0,x$k) $mos(OF,x$k)
            set mos(0,y$k) $mos(OF,y$k)
        }
    }

    if { $debug && $verbose } {
        puts stdout         "            Frame corners in the canvas (pixel)"
        puts stdout         "                      South (Geo)(MOS 1)"
        puts stdout [format "     BLUE   FOC_%s:  ${ar}(%8.2f,%8.2f)   ${br}(%8.2f, %8.2f)   RED" $resol $mos(0,x$ar) $mos(0,y$ar) $mos(0,x$br) $mos(0,y$br)]
        puts stdout         " "
        puts stdout         "                      North (Geo)(MOS 19)"
        puts stdout [format "     BLUE   FOC_%s:  ${dr}(%8.2f,%8.2f)   ${cr}(%8.2f, %8.2f)   RED" $resol $mos(0,x$dr) $mos(0,y$dr) $mos(0,x$cr) $mos(0,y$cr)]
        # If you need to see the corners plotted on the canvas:
        # set R [expr {0.3 * $mos(0,r)}]
        # catch {$m_canvas delete r_corner$resol}
        # foreach k "$ar $br $cr $dr" {
            # Change resolution SR or HR according to the corners you need to see
            # if { [string compare $resol SR] == 0 } {
                # Choose between label formats:
                # - name only
                # set foc_text  [format "F_%s" ${resol}$k]
                # - name and variable
                # set foc_text  [format "F_%s(%s)" ${resol}$k "mos(0,x/y$k)"]
                # - name and coordinates in (pixels)
                # set foc_text  [format "F_%s(%7.2f,%7.2f)" ${resol}$k $mos(0,x$k) $mos(0,y$k)]
                # - name and coordinates in (mm)
                # set foc_text  [format "F_%s(%7.2f,%7.2f)" ${resol}$k $foc_frame(0,x$k) $foc_frame(0,y$k)]

                # Plot the corner dots
                # $m_canvas create oval [expr {$mos(0,x$k)-$R}] \
                                      [expr {$mos(0,y$k)-$R}] \
                                      [expr {$mos(0,x$k)+$R}] \
                                      [expr {$mos(0,y$k)+$R}] \
                                      -outline red \
                                      -fill yellow \
                                      -tag "r_corner$resol"
                # Plot the corner labels
                # $m_canvas create text $mos(0,x$k) $mos(0,y$k) -anchor n -fill yellow -text $foc_text -tag "r_corner$resol"
            # }
        # }
    }
    
    # Update coordinate transformation matrices
    update_xform_coeff

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "calculate_dispersion_frame: END $resol"
    }
    tracer LEAVING
}

#==============================================
private method calculate_slits { {resol SR} } {
#==============================================
# By P. Baksai

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Select scale parameters
    # numeric arcmin_to_degree atd = 2.7777778E-4
    # x [mm] * $focscale = y [arcsec]  (mtoas)
    # coord/mm * scal = coord/arcsec
    set scal [expr {$atd * $focscale}]
    # Park position beyond focal plane
    set mos(0,b) [format "%3.1f" [expr {6.0 * $focscale}]]

    # Select the initial slit width
    set lmod [$this cget -modus]
    switch -exact -- $lmod {
        echs {
            # Do nothing
        }
        hitms {
            # Do nothing
        }
        hits {
            # Do nothing
        }
        img {
            # For IMG, slit-width has the meaning of 'obscuration area'
            if { [string match *SR* $resol] } {
                # In img standard mode all slit blades are open
                #                                                        mm fp
                for { set i 1 } { $i < 20 } { incr i 1 } {
                    set mos($i,b) [expr {240 * $focscale}]
                }
            } else {
                # In IMG highres mode slit blades 1-5 and 15-20 are closed
                # and are used as a dekker to close the field beyond the
                # projected CCD field.
                # Slit blades 6-14 are open with respect to the highres scale
                #                                                       mm fp
                for { set i  1 } { $i <  5 } { incr i 1 } {
                    set mos($i,b) $mos(0,b)
                }
                for { set i  5 } { $i < 17 } { incr i 1 } {
                    set mos($i,b) [expr {140 * $focscale}]
                }
                for { set i 17 } { $i < 20 } { incr i 1 } {
                    set mos($i,b) $mos(0,b)
                }
            }
        }
        ipol {
            # In ipol standard mode all even slit blades are open,
            # all odd slit blades are closed, (10 is open)
            for { set i 2 } { $i < 20 } { incr i 2 } {
                 set mos($i,b) [expr {240 * $focscale}]
            }
            for { set i 1 } { $i < 20 } { incr i 2 } {
                 set mos($i,b) $mos(0,b)
            }
            # In ipol highres mode, all MOS-slit blades are open, and a
            # separate HIGHRES-IPOL-MASK is initialized
        }
        lss {
            # Do nothing
        }
        mos {
            # For MOS mode, slit-width has a spectroscopic meaning (resolution)
            for { set i 1 } { $i < 20 } { incr i 1 } {
                set mos($i,b) [$this cget -slit_width]
            }
        }
        mxu {
            # Do nothing
        }
        pmos {
            # For PMOS standard resolution collimator mode,
            # the even slit blades are used for spectroscopy,
            # odd slit blades are used as a dekker
            for { set i 1 } { $i < 20 } { incr i 1 } {
                set mos($i,b) [$this cget -slit_width]
            }
        }
        default {
            puts $lfId "calculate_slits: Warning, unknown modus >$lmod<"
        }
    }

    # Set world coords of points on the North-South line (arsec)
    switch -exact -- $mask {
        hitms {
            # Do nothing
        }
        hits {
            # Only 8 hits-slits available + 2 pinholes
            # 9 = big pinhole, 10 = small pinhole
            for { set i 1 } { $i < 10 } { incr i 1 } {
                # Reference point on NS
                set mos($i,xb) $wx
                # South side (i=1)
                set mos($i,yb) [expr {$wy + $scal*$htslitymin($i)}]
                set mos($i,xf) $wx
                # North side (i=10)
                set mos($i,yf) [expr {$wy + $scal*$htslitymax($i)}]
            }
            # Small pinhole in 10
            set mos(10,xb) $wx
            set mos(10,yb) [expr {$wy + $scal*$htslitymin(0)}]
            set mos(10,xf) $wx
            set mos(10,yf) [expr {$wy + $scal*$htslitymax(0)}]
        }
        lss {
            # Only 9 long-slits available
            for { set i 1 } { $i < 10 } { incr i 1 } {
                # Reference point on NS
                set mos($i,xb) $wx
                # South side (i=1)
                set mos($i,yb) [expr {$wy + $scal*$longslitymin($i)}]
                set mos($i,xf) $wx
                # North side (i=10)
                set mos($i,yf) [expr {$wy + $scal*$longslitymax($i)}]
            }
        }
        mos {
            # First set only the coords of points with
            # delta rectaszension = 0.
            # Those points are on the central north-south axis
            # 19 slit blades, unit is world coords
            # mos(1..19,bcfg) in WCS
            for { set i 1 } { $i < 20 } { incr i 1 } {
                # South side (i=1)
                set mos($i,xb) $wx
                set mos($i,yb) [expr {$wy + $scal*$mosslitmin($i)}]
                set mos($i,xc) $wx
                set mos($i,yc) $mos($i,yb)
                set mos($i,xf) $mos($i,xb)
                set mos($i,yf) [expr {$wy + $scal*$mosslitmax($i)}]
                set mos($i,xg) $mos($i,xf)
                set mos($i,yg) $mos($i,yf)
            }
            # We draw slit blade 19 and slit blade 1 not larger than CCD area
            # in order to avoid user-defined slit positions outside CCD frame
            # set mos(1,yb) [expr {$wy + $scal*$YSRMIN}]
            # set mos(1,yc) $mos(1,yb)
            # set mos(1,yf) [expr {$wy + $scal*$YSRMIN}]
            # set mos(1,yg) $mos(1,yf)

            # set mos(19,yb) [expr {$wy + $scal*$YSRMAX}]
            # set mos(19,yc) $mos(19,yb)
            # set mos(19,yf) [expr {$wy + $scal*$YSRMAX}]
            # set mos(19,yg) $mos(19,yf)
        }
        mxu {
            # Do nothing
            # There will be a particular mxu_create_slit
        }
        pol {
            # The high resolution polarimetric imaging mask
            # only 9 image stripes available
            for { set i 1 } { $i < 10 } { incr i 1 } {
                # Reference point on NS
                set mos($i,xb) $wx
                # South side
                set mos($i,yb) [expr {$wy + $scal*$polarimslitymin($i)}]
                set mos($i,xf) $wx
                # North side
                set mos($i,yf) [expr {$wy + $scal*$polarimslitymax($i)}]
            }
        }
        default {
            puts $lfId "calculate_slits: Warning, unknown mask >$mask<"
        }
    }

    # Transform points on North-South line from world to canvas coords (pixel)
    if { [string compare $mask "mos"] == 0 } {
        set local_max 20
        set local_set {b c f g}
    } elseif { [string compare $mask "mxu"] == 0 } {
        set local_max 0
        set local_set {none}
    } elseif { [string compare $mask "hitms"] == 0 } {
        set local_max 0
        set local_set {none}
    } else {
        set local_max 10
        set local_set {b f}
    }
    if { $debug && $verbose } {
        puts stdout "calculate_slits: "
        puts stdout "                 mask      = $mask"
        puts stdout "                 local_max = $local_max"
        puts stdout "                 local_set = $local_set"
    }
    if { $bonsai_flag } {
        for { set i 1 } { $i < $local_max } { incr i 1 } {
             foreach val $local_set {
                 set dum [C_wc w2c $mos($i,x$val) $mos($i,y$val)]
                 set mos($i,x$val) [lindex $dum 0]
                 set mos($i,y$val) [lindex $dum 1]
             }
        }
    } else {
        foreach val $local_set {
            for { set i 1 } { $i < $local_max } { incr i 1 } {
                 catch { $m_image convert coords $mos($i,x$val) $mos($i,y$val) "deg $EQX" \
                                                  mos($i,x$val)  mos($i,y$val) canvas
                 }
            }
        }
    }

    # Handle points on the field borders
    # MOS limit switches
    # Declination canvas coords
    # Note : m can be zero
    # Take the FOCFIELD values only to determine the
    # slope for all masks to lower south part

    lassign [calc_ctom FOC $resol] ctom loc_m loc_m1 p1x p1y p2x p2y XMIN YMIN XMAX YMAX

    # The slope of the mask can be infinity in canvas coords,
    # this case has to be handled differently
    # ne_flag = 1 means, image is not rotated,

    set ne_flag 0
    if { [expr {abs($p1x - $p2x)}] < 1e-4 } {
        set ne_flag 1
    }
    if { $debug && $verbose } {
        puts stdout         "calculate_slits:"
        puts stdout [format "    slope = %e" [expr {abs($p1x - $p2x)}]]
        puts stdout         "  ne_flag = $ne_flag"
        if { $ne_flag } {
            puts stdout     "   frame || WCS -> not rotated"
        } else {
            puts stdout     "   frame \\/ WCS -> rotated"
        }
    }

    # Factors for perpendicular case (pixel).
    # They are zero if rotated.
    # fits_x_scale is the sign of the X canvas scale towards positive Alpha
    set FS [expr {$fits_x_scale*$ctom}]

    if { !$ne_flag } {
        # The coord axis are not perpendicular to the frame.
        # Factors for rotated case (pixel).
        # They are zero if perpendicular.
        # CRO is the sign of the rotator angle.
        set RS [expr {$FS*$CRO*$loc_m1}]
        # The F factors were included in the rotated R factors, now we zero them.
        set FS 0
        if { $debug && $verbose } {
            puts stdout "calculate_slit: axis not perpendicular to frame"
        }
    } else {
        # The coord axis is perpendicular to the frame.
        # The slope and therefore the rotation factors are zero.
        set loc_m 0
        set ccdm_loc_m 0
        set ccds_loc_m 0
        set RS 0
        if { $debug && $verbose } {
            puts stdout "calculate_slit: axis perpendicular to frame"
        }
    }
 
    # These are the border points of the 19 slit blades
    for { set i 1 } { $i < $local_max } { incr i 1 } {
        switch -exact -- $mask {
            hitms {
                # Do nothing
            }
            hits {
                # chapter 3.2.2.0.0  mask hits
                # but here the hits-slit mask
                set mos($i,ya) [expr {$mos($i,yb) - $RS*$htslitxmin($i)}]
                set mos($i,xa) [expr {$mos($i,xb) + $FS*$htslitxmin($i) - $loc_m*($mos($i,ya) - $mos($i,yb))}]

                set mos($i,yd) [expr {$mos($i,yb) - $RS*$htslitxmax($i)}]
                set mos($i,xd) [expr {$mos($i,xb) + $FS*$htslitxmax($i) - $loc_m*($mos($i,yd) - $mos($i,yb))}]

                set mos($i,ye) [expr {$mos($i,yf) - $RS*$htslitxmin($i)}]
                set mos($i,xe) [expr {$mos($i,xf) + $FS*$htslitxmin($i) - $loc_m*($mos($i,ye) - $mos($i,yf))}]

                set mos($i,yh) [expr {$mos($i,yf) - $RS*$htslitxmax($i)}]
                set mos($i,xh) [expr {$mos($i,xf) + $FS*$htslitxmax($i) - $loc_m*($mos($i,yh) - $mos($i,yf))}]
            }
            lss {
                # chapter 3.2.1.0.0  mask lss
                # but here the long-slit mask
                set mos($i,ya) [expr {$mos($i,yb) - $RS*$longslitxmin($i)}]
                set mos($i,xa) [expr {$mos($i,xb) + $FS*$longslitxmin($i) - $loc_m*($mos($i,ya) - $mos($i,yb))}]

                set mos($i,yd) [expr {$mos($i,yb) - $RS*$longslitxmax($i)}]
                set mos($i,xd) [expr {$mos($i,xb) + $FS*$longslitxmax($i) - $loc_m*($mos($i,yd) - $mos($i,yb))}]

                set mos($i,ye) [expr {$mos($i,yf) - $RS*$longslitxmin($i)}]
                set mos($i,xe) [expr {$mos($i,xf) + $FS*$longslitxmin($i) - $loc_m*($mos($i,ye) - $mos($i,yf))}]

                set mos($i,yh) [expr {$mos($i,yf) - $RS*$longslitxmax($i)}]
                set mos($i,xh) [expr {$mos($i,xf) + $FS*$longslitxmax($i) - $loc_m*($mos($i,yh) - $mos($i,yf))}]
            }
            mos {
                # chapter 3.2.4.0.0  mask mos
                set mos($i,ya) [expr {$mos($i,yb) - $RS*$XBLADEA}]
                set mos($i,xa) [expr {$mos($i,xb) + $FS*$XBLADEA - $loc_m*($mos($i,ya) - $mos($i,yb))}]

                set mos($i,yd) [expr {$mos($i,yb) - $RS*$XBLADEB}]
                set mos($i,xd) [expr {$mos($i,xb) + $FS*$XBLADEB - $loc_m*($mos($i,yd) - $mos($i,yb))}]

                set mos($i,ye) [expr {$mos($i,yf) - $RS*$XBLADEA}]
                set mos($i,xe) [expr {$mos($i,xf) + $FS*$XBLADEA - $loc_m*($mos($i,ye) - $mos($i,yf))}]

                set mos($i,yh) [expr {$mos($i,yf) - $RS*$XBLADEB}]
                set mos($i,xh) [expr {$mos($i,xf) + $FS*$XBLADEB - $loc_m*($mos($i,yh) - $mos($i,yf))}]
            }
            mxu {
                # Do nothing
            }
            pol {
                # chapter 3.2.3.0.0  mask pol
                # IPOL HIGHRES
                set mos($i,ya) [expr {$mos($i,yb) - $RS*$polarimslitxmin($i)}]
                set mos($i,xa) [expr {$mos($i,xb) + $FS*$polarimslitxmin($i) - $loc_m*($mos($i,ya) - $mos($i,yb))}]

                set mos($i,yd) [expr {$mos($i,yb) - $RS*$polarimslitxmax($i)}]
                set mos($i,xd) [expr {$mos($i,xb) + $FS*$polarimslitxmax($i) - $loc_m*($mos($i,yd) - $mos($i,yb))}]

                set mos($i,ye) [expr {$mos($i,yf) - $RS*$polarimslitxmin($i)}]
                set mos($i,xe) [expr {$mos($i,xf) + $FS*$polarimslitxmin($i) - $loc_m*($mos($i,ye) - $mos($i,yf))}]

                set mos($i,yh) [expr {$mos($i,yf) - $RS*$polarimslitxmax($i)}]
                set mos($i,xh) [expr {$mos($i,xf) + $FS*$polarimslitxmax($i) - $loc_m*($mos($i,yh) - $mos($i,yf))}]
            }
            default {
                puts stdout "calculate_slits: Warning: unknown mask >$mask<"
            }
        }
    }

    # Now open the slit blades according to $mos($i,b)
    # mos(i,b) in arcsec, focscale = mm_to_as
    # mos(i,b)/mm_to_as/ctom = canvas
    # For MOS, all slits are still in the park position, with common slit width
    switch -exact -- $lmod {
        echs {
        }
        hitms {
            # Do nothing
        }
        hits {
            # Do nothing, since hits is a fixed mask, nothing to open
        }
        img {
            # Open all blades first
            for { set i 1 } { $i < 20 } { incr i 1 } {
                set dummy $mos($i,yb)
                set mos($i,yb) [expr {$dummy + $RS*$mos($i,b)/$focscale/2}]
                set mos($i,xb) [expr {$mos($i,xb) - $mos($i,b)/$focscale*$FS/2 - $loc_m*($mos($i,yb) - $dummy)}]
                set dummy $mos($i,yc)
                set mos($i,yc) [expr {$dummy - $RS*$mos($i,b)/$focscale/2}]
                set mos($i,xc) [expr {$mos($i,xc) + $mos($i,b)/$focscale*$FS/2 - $loc_m*($mos($i,yc) - $dummy)}]
                set dummy $mos($i,yf)
                set mos($i,yf) [expr {$dummy + $RS*$mos($i,b)/$focscale/2}]
                set mos($i,xf) [expr {$mos($i,xf) - $mos($i,b)/$focscale*$FS/2 - $loc_m*($mos($i,yf) - $dummy)}]
                set dummy $mos($i,yg)
                set mos($i,yg) [expr {$dummy - $RS*$mos($i,b)/$focscale/2}]
                set mos($i,xg) [expr {$mos($i,xg) + $mos($i,b)/$focscale*$FS/2 - $loc_m*($mos($i,yg) - $dummy)}]
            }
            if { [string match *HR* $resol] } {
                # Close the outside slit blades
                foreach i { 1 2 3 4 17 18 19 } {
                    set mos($i,yb) $mos($i,ya)
                    set mos($i,xb) $mos($i,xa)

                    set mos($i,yf) $mos($i,ye)
                    set mos($i,xf) $mos($i,xe)

                    set mos($i,yc) $mos($i,ya)
                    set mos($i,xc) $mos($i,xa)

                    set mos($i,yg) $mos($i,ye)
                    set mos($i,xg) $mos($i,xe)
                }
            }
        }
        ipol {
            if { [string match *SR* $resol] } {
            } else {
            }
        }
        lss {
            # Do nothing, since lss is a fixed mask, nothing to open
        }
        mos {
            if { $ne_flag } {
                # Perpendicular
                # Note: initial slitwidth not 6 mmm but 'slit_width' arcsec
                for { set i 1 } { $i < 20 } { incr i 1 } {
                    # All y-values are retained
                    # Open beyond the focal plane
                    set mos($i,xb) $mos($i,xa)
                    set mos($i,xc) [expr {$mos($i,xb) + $mos($i,b)/$focscale*$FS}]
                    set mos($i,xf) $mos($i,xe)
                    set mos($i,xg) [expr {$mos($i,xf) + $mos($i,b)/$focscale*$FS}]
                }
 
            } else {
                # Slope
                for { set i 1 } { $i < 20 } { incr i 1 } {
                    set dummy $mos($i,yb)
                    # mos slitlets beyond the focal plane
                    # This position can be used to unselect mos slits
                    set mos($i,xb) $mos($i,xa)
                    set mos($i,yb) $mos($i,ya)
                    set mos($i,yc) [expr {$mos($i,ya) + $fits_y_scale*$ctom*$mos($i,b)/$focscale*$loc_m1}]
                    set mos($i,xc) [expr {$mos($i,xa) - $loc_m*($mos($i,yc) - $dummy)}]
                    set mos($i,xf) $mos($i,xe)
                    set mos($i,yf) $mos($i,ye)
                    set mos($i,yg) [expr {$mos($i,ye) + $fits_y_scale*$ctom*$mos($i,b)/$focscale*$loc_m1}]
                    set mos($i,xg) [expr {$mos($i,xe) - $loc_m*($mos($i,yg) - $dummy)}]
                }
            }
        }
        mxu  {
            # Do nothing
        }
        pmos {
            if { $ne_flag } {
                # Perpendicular
                for { set i 2 } { $i < 20 } { incr i 2 } {
                    # All y-values are retained
                    set mos($i,xb) [expr {$mos($i,xb)-$mos($i,b)/2/$focscale*$fits_x_scale*$ctom}]
                    set mos($i,xc) [expr {$mos($i,xc)+$mos($i,b)/2/$focscale*$fits_x_scale*$ctom}]
                    set mos($i,xf) [expr {$mos($i,xf)-$mos($i,b)/2/$focscale*$fits_x_scale*$ctom}]
                    set mos($i,xg) [expr {$mos($i,xg)+$mos($i,b)/2/$focscale*$fits_x_scale*$ctom}]
                }
 
            } else {
                # Slope
                for { set i 2 } { $i < 20 } { incr i 2 } {
                    set dummy $mos($i,yb)
                    set mos($i,yb) [expr {$dummy+$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1}]
                    set mos($i,xb) [expr {$mos($i,xb)-$loc_m*($mos($i,yb)-$dummy)}]
                    set dummy $mos($i,yf)
                    set mos($i,yf) [expr {$dummy+$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1}]
                    set mos($i,xf) [expr {$mos($i,xf)-$loc_m*($mos($i,yf)-$dummy)}]
                    set dummy $mos($i,yc)
                    set mos($i,yc) [expr {$dummy-$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1}]
                    set mos($i,xc) [expr {$mos($i,xc)-$loc_m*($mos($i,yc)-$dummy)}]
                    set dummy $mos($i,yg)
                    set mos($i,yg) [expr {$dummy-$fits_x_scale*$ctom*$mos($i,b)/$focscale/2*$loc_m1}]
                    set mos($i,xg) [expr {$mos($i,xg)-$loc_m*($mos($i,yg)-$dummy)}]
                }
            }
            # Close the odd slit blades
            for { set i 1 } { $i < 20 } { incr i 2 } {
                set mos($i,yb) $mos($i,ya)
                set mos($i,xb) $mos($i,xa)
                set mos($i,yf) $mos($i,ye)
                set mos($i,xf) $mos($i,xe)
                set mos($i,yc) $mos($i,ya)
                set mos($i,xc) $mos($i,xa)
                set mos($i,yg) $mos($i,ye)
                set mos($i,xg) $mos($i,xe)
            }
        } 
        default {
            puts $lfId "calculate_slits: Warning, unknown modus >$lmod<"
        }
    }
 
    # Only for mask == mos
    # We initialize the world coords of the slit blade
    # positions, and the select_flag, as in mos mode
    if { [string compare $mask "mos"] == 0 } {
        for { set i 1 } { $i < 20 } { incr i 1 } {
            # default: MOS slit not set, not active, not positioned
            set targw($i,f) 0
            set targw($i,a) [expr ($mos($i,xb)+$mos($i,xc)+$mos($i,xe)+$mos($i,xf))/4.0]
            set targw($i,d) [expr ($mos($i,yb)+$mos($i,yc)+$mos($i,ye)+$mos($i,yf))/4.0]
            # v0.9.6 close the unused mos slit blades again
            # Only in mos/pmos; not in img/pimg
            # 14/04/99 also in echs
            if { [string match "*mos" $lmod] || [string match "*echs*" $lmod] } {
                set mos($i,xb) $mos($i,xa)
                set mos($i,yb) $mos($i,ya)
                set mos($i,xc) $mos($i,xa)
                set mos($i,yc) $mos($i,ya)
                set mos($i,xg) $mos($i,xe)
                set mos($i,yg) $mos($i,ye)
                set mos($i,xf) $mos($i,xe)
                set mos($i,yf) $mos($i,ye)
            }
        }
    }

    # The $i slots are used by the plotting methods
    if { $mountangle == 180 } {
        if { $debug && $verbose } {
            puts stdout "calculate_slits: slit frame has been rotated 180 (deg) because of mount angle"
        }
        set cda [expr {cos(180*$deg2rad)}]
        set sda [expr {sin(180*$deg2rad)}]
        foreach p "a b c d e f g h" {
            for { set i 1 } { $i < $local_max } { incr i 1 } {
                if { [info exists mos($i,x$p)] } {
                    set mos($i,x$p) [expr {$ccx + ($mos($i,x$p) - $ccx)*$cda - ($mos($i,y$p) - $ccy)*$sda}]
                }
                if { [info exists mos($i,y$p)] } {
                    set mos($i,y$p) [expr {$ccy + ($mos($i,x$p) - $ccx)*$sda + ($mos($i,y$p) - $ccy)*$cda}]
                }
            }
        }
    } else {
        if { $debug && $verbose } {
            puts stdout "calculate_slits: slit frame has not been rotated"
        }
    }
    tracer LEAVING
}

#==================================
private method set_scale { flag } {
#==================================
# new philosophy since 13/9/1999
# the focscale (''/mm_focalplane) is only dependent on the VLT unit
# $focscale is a constant for FORS as long as
# the instrument stays at the VLT UnitX.
# Moving the FORS camera (due to different filters,  temperature and
# seasonal air pressure at Paranal) changes the (''/pixel) scale,
# Hence changes the $CCDscale_standard and $CCDscale_highres

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $flag } {
        # take the ''/pixel scale from the FoRS fits image,
        # since the nominal scale might not be appropriate
        # the real scale is dependent on the instrument
        # temperature and the BB filter
        set fisc [lindex [$m_image wcsset] 2]
	if { "$fisc" == "" } {
            set fisc 0
        }
        # puts stdout ">>>$fisc<<<"
        set frac_s [expr {$fisc / $CCDscale_standard}]
        set frac_h [expr {$fisc / $CCDscale_highres}]
        if { abs($frac_s-1.0) < abs($frac_h-1.0) } {
            # its the scale of the standard resolution collimator
            set frac $frac_s
        } else {
            # its the scale of the highres collimator
            set frac $frac_h
        }

        # we have to think about this
        # set CCDscale_highres  [expr {$CCDscale_highres  * $frac}]
        # set CCDscale_standard [expr {$CCDscale_standard * $frac}]

        # set focscale [expr {$focscale * $frac}]

        # puts stdout [format "FITS SCALE : %8f''/p = %8e o/p \
        #                     %8f''/p = %8e o/p \
        #                     %8.6f''/mm ff %e" \
        #                     $CCDscale_standard [expr {$CCDscale_standard / 3600.0}] \
        #                     $CCDscale_highres  [expr {$CCDscale_highres  / 3600.0}] \
        #                     $focscale $frac]
    } else {
        # Fits image not from FORS
        # use the nominal values of ''/pixel
        # there is no nominal value of ''/mm
        # focscale is dependent on the telescope unit
        # the CCD_scale... is not so important, since it
        # does only manage the wavelength borders

        # Maintenence: If FORS1 is moved from UNIT -> UNIT4
        # edit the '1' branch
        # Default unbinned
        set binx 0
        switch -exact -- [$this cget -instrument] {
               FORS1 {
                   catch { set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"] }
                   if { $binx <= 1 } {
                       # unbinned
                       set PIX 0.015
                       set CCDscale_standard 0.125062
                       set CCDscale_highres 0.0625313
                   } else {
                       set PIX 0.030
                       set CCDscale_standard 0.250124
                       set CCDscale_highres  0.1250626
                   }
                   set focscale $focscale_UT2
                   puts stdout "FORS1 at UT2"
               }
               FORS2 {
                   catch { set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"] }
                   if { $binx <= 1 } {
                       # unbinned
                       set PIX 0.015
                       set CCDscale_standard 0.125062
                       set CCDscale_highres 0.0625313
                   } else {
                       set PIX 0.030
                       set CCDscale_standard 0.250124
                       set CCDscale_highres  0.1250626
                   }
                   set focscale $focscale_UT1
                   puts stdout "FORS2 at UT1"
               }
               FORS2+1 {
                   catch { set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"] }
                   if { $binx <= 1 } {
                       # unbinned
                       set PIX 0.015
                       set CCDscale_standard 0.125062
                       set CCDscale_highres 0.0625313
                   } else {
                       set PIX 0.030
                       set CCDscale_standard 0.250124
                       set CCDscale_highres  0.1250626
                   }
                   set focscale $focscale_UT1
                   puts stdout "FORS2 at UT1"
               }
               default {
                   puts stdout "Error in telescope focal length"
               }
        }
        puts stdout [format "NOMINAL SCALE : %8f''/p = %8e o/p (SR) \
                                             %8f''/p = %8e o/p (HR) \
                                             %8.6f''/mm ff (UT)" \
                             $CCDscale_standard [expr {$CCDscale_standard / 3600.0}] \
                             $CCDscale_highres  [expr {$CCDscale_highres  / 3600.0}] \
                             $focscale]
    }
    if { [string compare [$this cget -modus] hitms] == 0 ||
         [string compare [$this cget -modus] mxu] == 0 } {
        fors::MXU_SLIT::fc $focscale
    }
    tracer LEAVING
}

#==============================
public method update_scale {} {
#==============================
    # WCS - canvas coords
    # only for mxu

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $wcs_flag } {
        C_wc coeff_wc $wx $wy
    }

    switch -exact -- [$this cget -modus] {
        default {
            C_mc coeff_mc $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                          $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                          $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
        }
    }

    set ctom [lindex [eval calc_ctom FOC SR] 0]
    set ccdm_ctom [lindex [eval calc_ctom MASTER SR] 0]
    set ccds_ctom [lindex [eval calc_ctom SLAVE SR] 0]
    tracer LEAVING
}

#==============================================
public method wave_config_mask { {four ""} } {
#==============================================
# This is called by the wavelength EntryForm
# This is also called by public variables filter and grism

# Input is a list with up to four elements: lmin lmax who id
# lmin: lambda min
# lmax: lambda max
# who:  who called this method
#       - filter (called by public variable filter)
#       - grism (called by public variable grism)
#         update when a new grism has been selected
#         even the L_MIN and L_MAX look like not
#         having been updated
#       - user (called by the wavelength entry widget)
#         update only when the new user-defined wavelength
#         border differs from L_MIN, L_MAX
# id:   the component id in case who is filter or grism
#

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set flag 0
    lassign $four lmin lmax who id

    puts "\nWAVE CONFIG MASK by $who $id"
    if { $debug && $verbose } {
        puts stdout "wave_config_mask: four = $four"
        puts stdout "                  lmin = $lmin"
        puts stdout "                  lmax = $lmax"
        puts stdout "                   who = $who"
        puts stdout "                    id = $id"
    }
    
    # puts stdout "update Wave Length Border with >$lmin< >$lmax< >$who<"

    # Check if there are really changes with respect to the current
    # wavelength range setting
    if { [string compare $who "grism"] == 0 ||
         [string compare $who "filter"] == 0 } {
        if { $mask_initialized } {
            set flag 1
        }
        # puts stdout "called by $who, flag = $flag"
    } else {
        # If the widget called, the mask must be initialized already
        if { $L_MIN != $lmin } {
            set flag 1
            set L_MIN $lmin
        }
        if { $L_MAX != $lmax } {
            set flag 1
            set L_MAX $lmax
        }
        # puts stdout "called by Wave, flag = $flag"
        set L_MIN [expr {$L_MIN * 1.0}]; # int to float
        set L_MAX [expr {$L_MAX * 1.0}]; # int to float
        wave_limit;
    }

    # If so
    if { $flag } {
        # If you want to keep the slits after selecting a new
        # wavelength range you must code here something awful

        if { $mask_initialized } {
            update_xform_coeff
        }
        switch -exact -- [$this cget -modus] {
            hitms -
            hits -
            lss -
            mos -
            mxu -
            pmos {
                calculate_dispersion_frame [$this cget -collimat]
            }
        }

        if { $mask_initialized } {
            update_and_replot
        }
    }
    puts "WAVE CONFIG MASK END\n"
    tracer LEAVING
}

#==================================
public method init_mask_world2 {} {
#==================================
# The WCS command 'convert coords' does only work on
# frames but not on the canvas area beyond a FITS frame.
# Therefore the instrumental mask uses a separate coord
# conversion (a linear approach, defined via bonsai_matrix)
# CSO: Coordinate System Orientation: East to the right (+) or to the left (-)

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    #
    # This member function draws the instrumental mask on the canvas
    # with the correct size (~ 6.4' x 6.4') and position angle 0.
    # All coordinates are stored in the large array mos(,)
    # here we handle alp and del differently
    # canvas coordinates in canvas pixels
    # there is still a bug concerning south-pole
    # first at all : central position in canvas
    # units of mos: canvas coords
    # units of targw: canvas
    set wcsset [$m_image wcsset] 
    if { $debug } {
        set w $wcsset
        if { [string length [lindex $w 0]] == 0 } {
            set w {"00:00:00.000" "00:00:00.00" 0.0 0.0 0.0 2 2 0 2000 2000.0 "N/A"}
        }
        puts stdout "world2: BEGIN"
        puts stdout "          RA             DEC          asec/pix  PXcen  PYcen SizeX SizeY     Rot Equinox Epoch   Proj"
        puts stdout [format "        %14s %14s %8.6f %6.1f %6.1f %5d %5d %7.2f %7d %7.2f %4s\n" \
                            [lindex $w  0] \
                            [lindex $w  1] \
                            [lindex $w  2] \
                            [lindex $w  3] \
                            [lindex $w  4] \
                            [lindex $w  5] \
                            [lindex $w  6] \
                            [lindex $w  7] \
                            [lindex $w  8] \
                            [lindex $w  9] \
                            [lindex $w 10] \
                    ]
    }

    set icx [lindex $wcsset 3]
    set icy [lindex $wcsset 4]
    set EQX [lindex $wcsset 8]

    # Frameless mode
    if { $wx == "" || $wy == "" } {
        puts stdout "world2: Frameless mode"
        set frameless 1
        set w1 [DDMMSS2deg [lindex $wcsset 0]]
        set w2 [DDMMSS2deg [lindex $wcsset 1]]
        if { [catch {
                  $m_image convert coords $w1 $w2 "wcs $EQX" wx wy "deg $EQX"
              } result] } {
            if { $debug } {
                puts stdout "world2: Frameless reference: (w1,w2)"
                puts stdout "        Error converting ($w1,$w2) WCS to (wx,wy) \[deg\]"
                puts stdout "        $result"
                puts stdout "        Using world_x, wordl_y = >$world_x<,>$world_y<"
            }
            set wx $world_x
            set wy $world_y
        }
    } else {
        # Read the reference pixel from the fits header
        # ---------------------------------------------
        # This is independent on WCS notation
        set crp1 [$m_image fits get CRPIX1]
	set crp2 [$m_image fits get CRPIX2]
        set crv1 [$m_image fits get CRVAL1]
        set crv2 [$m_image fits get CRVAL2]
        if { [string length $crp1] } {
            puts stdout "world2: Frame mode"
        } elseif { $wx != 0 && $wy !=0 } {
            puts stdout "world2: Catalog mode"
        } else {
            puts stdout "world2: Dataless mode"
        }
        if { $debug && $verbose && [string length $crp1] } {
            puts stdout [format "world2: CRPIX1,2 (>%8.2f<,>%8.2f<)" \
                                $crp1 \
	                        $crp2 \
                        ] 
            puts stdout [format "        CRVAL1,2 (>%8.2f<,>%8.2f<)\n" \
                                $crv1 \
                                $crv2 \
                        ]
        }
        # Read the equinox
        if { $EQX == "" } {
            set EQX [$m_image fits get EQUINOX]
        }
        # if { $EQX == "" } {
        #     set EQX [$m_image fits get EPOCH]
        # }
        if { $EQX == "" } {
            puts stdout "world2: Error, EQUINOX not defined in FITS header!!!"
        }
    }

    # Reference pixel only used in the zemax Method
    if { [catch {
              $m_image convert coords $wx $wy "deg $EQX" ccx  ccy canvas
          } result] } {
        if { $debug } {
            puts stdout "             Reference pixel: (wx,wy)"
            puts stdout "             Error converting ($wx,$wy) \[deg\] to (ccx,ccy) canvas"
            puts stdout "             $result"
            puts stdout "             Using default ccx, ccy, icx, icy = 512, 512, 1024, 1024"
        }
        set ccx 512
        set ccy 512
        set icx 1024
        set icy 1024
    }

    if { $debug && $verbose } {
        puts stdout         "world2: reference pixel"
        puts stdout         "        EQX = $EQX"
        puts stdout [format "        wx  = %8.4f  wy  = %8.4f\n" $wx $wy]
        puts stdout [format "        ccx = %8.4f  ccy = %8.4f\n" $ccx $ccy]
    }
    set mos(0,x0) $ccx
    set mos(0,y0) $ccy
    set ccd_frame(0,x0) $ccx
    set ccd_frame(0,y0) $ccy
    set ccd_frame(1,x0) $ccx
    set ccd_frame(1,y0) $ccy

    puts stdout [format "world2: reference pixel %8.4f %8.4f %8.4f %8.4f" \
                        $wx $wy $ccx $ccy \
                ]

    # Retrieving the actual zooming parameter
    set canvscale [$m_image scale]
    set canvscale [lindex $canvscale 0]
    if { $canvscale < 0 } {
        set canvscale [expr {(-1.0) / $canvscale}]
    }

    # Initialized for calculate_dispersion_frame
    set refscale 0

    # Now the (signum of CROTA2 = CRO), needed for init_mask
    # first test if fits header contains CD,
    # then test if fits header contains PC
    # then test if fits header contains CROTA (WCS keywords)
    # - WCS not yet defined by default
    set wcs_flag 0
    set d11 [$m_image fits get CD1_1]
    if { $d11 != "" } {
        # Everything is still ok
        set wcs_flag 1
        if { $debug && $verbose } {
            puts stdout "world2: try CD notation ..."
        }
        set d12 [$m_image fits get CD1_2]
        set d21 [$m_image fits get CD2_1]
        set d22 [$m_image fits get CD2_2]
        if { $d12 == "" } {
            set wcs_flag 0
            if { $debug && $verbose } {
                puts stdout "world2: CD1_2 not found"
            }
        }
        if { $d21 == "" } {
            set wcs_flag 0
            if { $debug && $verbose } {
                puts stdout "world2: CD2_1 not found"
            }
        }
        if { $d22 == "" } {
            set wcs_flag 0
            if { $debug && $verbose } {
                puts stdout "world2: CD2_2 not found"
            }
        }
        if { !$wcs_flag } {
            puts stdout "world2: Error in FITS Header. \
                         Incomplete CD-WCS keywords, try PC notation ...\n"
        } else {
            # Everything correctly found up to now
            set d30 [expr {$d11 * $d22 - $d12 * $d21}]
            set CSO [expr {$d30 / abs($d30)}]; # = sign(CDELT1*CDELT2)
            if { $CSO == 1 } {
                if { $debug } {
                    puts stdout "world2: WCS right-hand coords --> East to right"
                }
                # By convention if CSO < 0 then CDELT1 < 0
                set fits_x_scale 1.0
                set fits_y_scale 1.0
            } else {
                if { $debug } {
                    puts stdout "world2: WCS left-hand coords <-- East to left"
                }
                # By convention if CSO < 0 then CDELT1 < 0
                set fits_x_scale -1.0
                set fits_y_scale 1.0
            }
            if { $debug && $verbose } {
                puts stdout "world2: d30 = ($d11 * $d22 - $d12 * $d21) = $d30"
                puts stdout "        CSO = (sign of d30) = $CSO"
                puts stdout "        fits_x_scale = $fits_x_scale"
                puts stdout "        fits_y_scale = $fits_y_scale"
            }
            set CRO [expr {$rad2deg * atan2($d30*$d12,$d22)}]
            if { $debug && $verbose } {
                puts stdout "world2: CRO = (rad2deg * atan2(d30*d12,d22))"
                puts stdout "              ($rad2deg * atan2($d30*$d12,$d22)) = $CRO"
            }
            # Sign of CROTA2
            if { $CRO == 0.0 } {
                set CRO 1.0
            } else {
                set CRO [expr {$CRO / abs($CRO)}]
            }
            if { $debug && $verbose } {
                puts stdout "world2: CROTA = $CRO"
            }
            if { $debug } {
                puts stdout "wordl2: ...got values from CD"
                puts stdout [format \
                                 "     CD1_1,CD1_2 = %10e %10e" \
                                 $d11 \
                                 $d12 \
                            ]
                puts stdout [format \
                                 "     CD2_1,CD2_2 = %10e %10e" \
                                 $d21 \
                                 $d22 \
                            ]
            }
        }
    } else {
        if { $debug && $verbose } {
            puts stdout "world2: CD1_1 not found"
        }
    }

    if { !$wcs_flag } {
        # CD notation either incomplete or not available
        set pc11 [$m_image fits get PC001001]
        if { $pc11 != "" } {
            set wcs_flag 1
            if { $debug && $verbose } {
                puts stdout "world2: try PC notation ..."
            }
            set cd1 [$m_image fits get CDELT1]
            set cd2 [$m_image fits get CDELT2]
            set pc12 [$m_image fits get PC001002]
            set pc21 [$m_image fits get PC002001]
            set pc22 [$m_image fits get PC002002]
            if { $cd1 == "" } {
                set wcs_flag 0
                if { $debug && $verbose } {
                    puts stdout "world2: CDELT1 not found"
                }
            }
            if { $cd2 == "" } {
                set wcs_flag 0
                if { $debug && $verbose } {
                    puts stdout "world2: CDELT2 not found"
                }
            }
            if { $pc12 == "" } {
                set wcs_flag 0
                if { $debug && $verbose } {
                    puts stdout "world2: PC001002 not found"
                }
            }
            if { $pc22 == "" } {
                set wcs_flag 0
                if { $debug && $verbose } {
                    puts stdout "world2: PC002002 not found"
                }
            }
            if { !$wcs_flag } {
                puts stdout "world2: Error in FITS Header. \
                             Incomplete PC-WCS keywords, try CROTA notation ...\n"
            } else {
                set fits_x_scale [expr {$cd1 / abs($cd1)}]
                set fits_y_scale [expr {$cd2 / abs($cd2)}]
                set CSO [expr {$fits_x_scale * $fits_y_scale}]
                if { $debug } {
                    if { $CSO == 1 } {
                        puts stdout "world2: WCS right-hand coords --> East to right"
                    } else {
                        puts stdout "world2: WCS left-hand coords <-- East to left"
                    }
                }
                if { $debug && $verbose } {
                    puts stdout "        fits_x_scale = (sign of CDELT1) = $fits_x_scale"
                    puts stdout "        fits_y_scale = (sign of CDELT2) = $fits_y_scale"
                    puts stdout "        CSO = (fits_x_scale * fits_y_scale) = $CSO"
                }
                set CRO [expr {$rad2deg * atan2($pc12*abs($cd1),$pc22*abs($cd2))}]
                if { $debug && $verbose } {
                    puts stdout "world2: CRO = (rad2deg * atan2(pc12*abs(cd1),pc22*abs(cd2))"
                    puts stdout "              ($rad2deg * atan2($pc12*abs($cd1),$pc22*abs($cd2)) = $CRO"
                }
                # Sign of CROTA2
                if { $CRO == 0.0 } {
                    set CRO 1.0
                } else {
                    set CRO [expr {$CRO / abs($CRO)}]
                }
                if { $debug && $verbose } {
                    puts stdout "world2: CROTA = $CRO"
                }
                if { $debug } {
                    puts stdout "wordl2: ...got values from PC"
                    puts stdout [format \
                                     "      PC11,PC12  = %10e %10e" \
                                     $pc11 \
                                     $pc12 \
                                ]
                    puts stdout [format \
                                     "      PC21,PC22  = %10e %10e" \
                                     $pc21 \
                                     $pc22 \
                                ]
                    puts stdout [format \
                                     "   CDELT1,CDELT2 = %10e %10e" \
                                     $cd1 \
                                     $cd2 \
                                ]
                }
            }
        } else {
            if { $debug && $verbose } {
                puts stdout "world2: PC001001 not found"
            }
        }
    }

    if { !$wcs_flag } {
        # PC notation either incomplete or not available
        set cd1 [$m_image fits get CDELT1]
        if { $cd1 != "" } {
            # Everything ok up to here
            set wcs_flag 1
            if { $debug && $verbose } {
                puts stdout "world2: try CDELT notation ..."
            }
            set cd2 [$m_image fits get CDELT2]
            set cro1 [$m_image fits get CROTA1]
            set cro2 [$m_image fits get CROTA2]
            if { $cd2 == "" } {
                set wcs_flag 0
                if { $debug && $verbose } {
                    puts stdout "world2: CDELT2 not found"
                }
            }
            if { $cro1 == "" } {
                if { $debug && $verbose } {
                    puts stdout "world2: CROTA1 not found"
                }
            }
            if { $cro2 == "" } {
                if { $debug && $verbose } {
                    puts stdout "world2: CROTA2 not found"
                }
            }
            # At least one of CROTA should be defined
            if { $cro1 == "" && $cro2 == "" } {
                if { $debug && $verbose } {
                    puts stdout "world2: At least one CROTA1 or CROTA2 needed. None found."
                }
                set wcs_flag 0
            }
            if { !$wcs_flag } {
                puts stdout "world2: Error in FITS Header. \
                             Incomplete CR-WCS keywords, try SkyCat ... \n"
            } else {
                set fits_x_scale [expr {$cd1 / abs($cd1)}]
                set fits_y_scale [expr {$cd2 / abs($cd2)}]
                set CSO [expr {$fits_x_scale * $fits_y_scale}]
                if { $debug } {
                    if { $CSO == 1 } {
                        puts stdout "world2: WCS: right-hand coords --> East to right"
                    } else {
                        puts stdout "world2: WCS: left-hand coords <-- East to left"
                    }
                }
                if { $debug && $verbose } {
                    puts stdout "        fits_x_scale = (sign of CDELT1) = $fits_x_scale"
                    puts stdout "        fits_y_scale = (sign of CDELT2) = $fits_y_scale"
                    puts stdout "        CSO = (fits_x_scale * fits_y_scale) = $CSO"
                }
                if { $cro1 == "" } {
                    set CRO $cro2
                    set n 2
                } else {
                    set CRO $cro1
                    set n 1
                }
                if { $debug && $verbose } {
                    puts stdout "world2: CRO = cro$n = $CRO"
                }
                # Sign of CROTA2
                if { $CRO == 0.0 } {
                   set CRO 1.0
                } else {
                   set CRO [expr {$CRO / abs($CRO)}]
                }
                if { $debug && $verbose } {
                    puts stdout "world2: CROTA = $CRO"
                }
                if { $debug } {
                    puts stdout "wordl2: ...got values from CDELT"
                    puts stdout [format \
                                     "        CDELT1,2 = %10e %10e" \
                                     $cd1 \
	                             $cd2 \
                                ]
                    puts stdout [format \
                                     "        CROTA1,2 = %6.2f %6.2f sign(CROTA) = %4.1f" \
                                     $cro1 \
                                     $cro2 \
                                     $CRO \
                                ]
                }
            }
        } else {
            if { $debug && $verbose } {
                puts stdout "world2: CDELT1 not found"
            }
        }
    }

    if { !$wcs_flag } {
        if { $debug && $verbose } {
            puts stdout "world2: try SkyCat ..."
        }
        # Try skycat WCS as a last chance
        set CRO [lindex $wcsset 7]
        if { $CRO == "" } {
            if { $debug && $verbose } {
                puts stdout "world2: CRO not found in skycat"
            }
        }
        set CSO -1
        set fits_x_scale -1.0
        set fits_y_scale 1.0
        if { $debug && $verbose } {
            puts stdout "        fits_x_scale = $fits_x_scale"
            puts stdout "        fits_y_scale = $fits_y_scale"
            puts stdout "        CSO = $CSO"
        }
        if { $debug && $verbose } {
            puts stdout "world2: CRO = wcsset 7 = $CRO"
        }
        # Sign of CROTA2
        if { $CRO == 0.0 } {
            set CRO 1.0
        } else {
            set CRO [expr {$CRO / abs($CRO)}]
        }
        if { $debug && $verbose } {
            puts stdout "world2: CROTA = $CRO"
        }
        if { $debug } {
            puts stdout "wordl2: ...got values from SkyCat"
        }
        # Everything ok finally
        set wcs_flag 1
    }

    # PBG 20090620: PPRS-23944, 24729:
    # This is the patch for the mask rotation problem when CROTA2 = +-180
    # A more thorough analysis should be done
    # PBG 20111118: PPRS-24729, 36459:
    # Using $CRO in next expresion is wrong. Changed to PA and now it works.
    # Patch moved down below all the attempts to calculate CSO, CRO.
    set PA [lindex $wcsset 7]
    if { $PA == "" } {
        set PA 0.0
        if { $debug && $verbose } {
            puts stdout "world2: PA not found in skycat. Set to 0.0"
        }
    }
    if { [expr {abs(abs($PA) - 180.0)}] < 1e-4 } {
        set fits_x_scale [expr {-1.0*$fits_x_scale}]
        set fits_y_scale [expr {-1.0*$fits_y_scale}]
        if { $debug && $verbose } {
            puts stdout "world2: fits_<x|y>_scale inverted:"
            puts stdout "                  PA = $PA"
            puts stdout "    new fits_x_scale = $fits_x_scale"
            puts stdout "    new fits_y_scale = $fits_y_scale"
        }
    }

    # SkyCat image flip buttons
    # --------------------------
    # State (if buttons are pressed)
    set flipr [$m_image rotate];   # [()]  button
    set flipx [$m_image flip x];   # [==]  button
    set flipy [$m_image flip y];   # [||]  button

    if { $flipr } {
        if { $debug && $verbose } {
            puts stdout "flipr"
        }
        if { $CRO == -1 } {
            set fits_x_scale [expr {-1.0*$fits_x_scale}]
        }
        set fits_y_scale [expr {-1.0*$fits_y_scale}]
        set CSO [expr {-1.0*$CSO}]
        set CRO [expr {-1.0*$CRO}]
    }
    if { $flipx } {
        if { $debug && $verbose } {
            puts stdout "flipx"
        }
        set fits_x_scale [expr {(-1) * $fits_x_scale}]
        set CSO [expr {-1.0*$CSO}]
        set CRO [expr {-1.0*$CRO}]
    }
    if { $flipy } {
        if { $debug && $verbose } {
            puts stdout "flipy"
        }
        set fits_y_scale [expr {-1.0*$fits_y_scale}]
        set CSO [expr {-1.0*$CSO}]
        set CRO [expr {-1.0*$CRO}]
    }

    # CSO = 1 -> mouse button 2 (B2) mask rotates left -> decreasing
    #            position angle (PA)
    # CSO =-1 -> mouse button 2 (B2) mask rotates left -> increasing
    #            position angle (PA)

    # coord/mm * scal = coord/arcsec
    set scal [expr {$atd * $focscale}]
    #----------------------------------------
    set trigo [expr {1 / cos($wy*$deg2rad)}]
    #----------------------------------------
    # trigo is for declination dependent hour-angle scale

    if { $debug } {
        puts stdout "world2:  << coord \[mm\] * scal = coord \[arcsec\] >>"
        puts stdout "        fits_x_scale = $fits_x_scale"
        puts stdout "        fits_y_scale = $fits_y_scale"
        puts stdout "            focscale = $focscale"
        puts stdout "           canvscale = $canvscale"
        puts stdout "                scal = $scal (arc_to_deg)"
        puts stdout "                 atd = $atd"
        puts stdout "               trigo = $trigo"
        puts stdout "                  wx = $wx"
        puts stdout "                  wy = $wy"
        puts stdout "                  PA = $PA"
        puts stdout "                 CRO = $CRO (sign(CROTA2))"
        puts stdout "                 CSO = $CSO (sign(CDELT1*CDELT2))"
    }
    # Since mos(,) in world coordinates, canvscale not necessary
    puts $lfId "Init_Mask_World: canvscale = $canvscale wx = $wx wy = $wy trigo = $trigo"
    puts $lfId "world2: focscale = $focscale scal = $scal arc_to_deg = $atd"

    # Absolute alp/del coordinates
    # 19 pairs of slit blades

    # Initial mode-dependent bindings maybe be overridden by
    # slitless spectro(polari)metry
    # hits, lss and ipol(hr) use only bind_frame

    switch -exact -- [$this cget -modus] {
        echs {
            bind_align
            bind_echs
        }
        hitms {
            bind_hitms
            bind_align
        }
        hits {
            bind_align
        }
        img {
            bind_img
        }
        ipol {
            if { [string compare $mask "pol"] } {
                bind_ipol
            }
            # bind_frame already in constr.
            # bind_img called by bind_ipol
        }
        lss {
            bind_align
        }
        mos {
            bind_mos
        }
        mxu {
            bind_mxu
            bind_align
        }
        pmos {
            bind_mos
        }
        default {
            # ipol(hr)
        }
    }

    # There is a slit-less MOS mode with $slit_width=open
    # In this submode of MOS blades can be positioned as in IMG
    # We handle this by a mode change.
    # This means mos->img; pmos->ipol
    if { [string compare [$this cget -slit_width] open] == 0 } {
        if { [string compare [$this cget -modus] mos] == 0 } {
            # slitwidth = open in mos : handle as IMG
            $this configure -modus img
            $this bind_img
            # puts stdout "SLS mos -> img | [$this cget -modus] $mask"
        } elseif { [string compare [$this cget -modus] img] == 0 } {
            $this configure -modus mos
            $this bind_mos
            # puts stdout "SLS img -> mos | [$this cget -modus] $mask"
        } elseif { [string compare [$this cget -modus] pmos] == 0 } {
            $this configure -modus ipol
            if { [string match *HR* [$this cget -collimat]] } {
                set mask pol
                $this bind_ipol
                # puts stdout "SLS pmos -> ipol | [$this cget -modus] $mask"
            } else {
                set mask mos
                $this bind_img
                # puts stdout "SLS pmos -> ipol | [$this cget -modus] $mask"
            }
        } elseif { [string compare [$this cget -modus] ipol] == 0 } {
            $this configure -modus pmos
            set mask mos
            $this bind_mos
            # puts stdout "SLS ipol -> pmos | [$this cget -modus] $mask"
        }
    }

    # For small frames, "deg $EQX -> canvas" does not work
    # In this case we have to create our own scale for mask points
    # outside the frame.
    # It is implemented in the Bonsai Matrix (see conv_wc class)
    set bonsai_flag 0

    # Check bonsai effect in the focal field
    set a_w $wx
    set b_w [expr {$wy + $scal*$YSRMAX}]
    set a_c $a_w
    set b_c $b_w
    catch {$m_image convert coords $a_w $b_w "deg $EQX" a_c b_c canvas} result
    if { ![expr {$a_c - $a_w}] } {
        if { $debug && $verbose } {
            puts stdout "world2: bonsai_flag raised at 1 with $result"
        }
        set bonsai_flag 1
    }

    set a_w $wx
    set b_w [expr {$wy + $scal*$YSRMIN}]
    set a_c $a_w
    set b_c $b_w
    catch {$m_image convert coords $a_w $b_w "deg $EQX" a_c b_c canvas} result
    if { ![expr {$a_c - $a_w}] } {
        if { $debug && $verbose } {
            puts stdout "world2: bonsai_flag raised at 2 with $result"
        }
        set bonsai_flag 1
    }

    # Check bonsai effect for CCD dimmensions
    set m1x $wx
    set m1y [expr {$wy + $scal*$CCDMASTER_YSRMIN}]
    set s1x $wx
    set s1y [expr {$wy + $scal*$CCDSLAVE_YSRMIN}]
    set m2x $wx
    set m2y [expr {$wy + $scal*$CCDMASTER_YSRMAX}]
    set s2x $wx
    set s2y [expr {$wy + $scal*$CCDSLAVE_YSRMAX}]
    if { [catch {$m_image convert coords $m1x $m1y "deg $EQX" m1x m1y canvas} result] } {
        if { $debug && $verbose } {
            puts stdout "world2: bonsai_flag raised at 3"
        }
        set bonsai_flag 1
    }
    if { [catch {$m_image convert coords $m2x $m2y "deg $EQX" m2x m2y canvas} result] } {
        if { $debug && $verbose } {
            puts stdout "world2: bonsai_flag raised at 4 with $result"
        }
        set bonsai_flag 1
    }
    if { [catch {$m_image convert coords $s1x $s1y "deg $EQX" s1x s1y canvas} result] } {
        if { $debug && $verbose } {
            puts stdout "world2: bonsai_flag raised at 5 with $result"
        }
        set bonsai_flag 1
    }
    if { [catch {$m_image convert coords $s2x $s2y "deg $EQX" s2x s2y canvas} result] } {
        if { $debug && $verbose } {
            puts stdout "world2: bonsai_flag raised at 6 with $result"
        }
        set bonsai_flag 1
    }

    if { $bonsai_flag } {
        puts stdout "world2: BONSAI ON"
        puts $lfId "world2: BONSAI ON"
        # Calculate the matrix coefficients
        C_wc coeff_wc $wx $wy
    } else {
        # The frame is large enough for the whole mask:
        puts stdout "world2: BONSAI OFF"
        puts $lfId "world2: BONSAI OFF"
    }

    # Select resolution parameters
    set resol [$this cget -collimat]
    if { [string match *SR* $resol] } {
        # SR
        set resol SR
        if { $debug } {
            puts stdout "world2: Standard Resolution"
        }
    } else {
        # HR
        set resol HR
        if { $debug } {
            puts stdout "world2: High Resolution"
        }
    }

    # Select current instrumental modus
    set lmod [$this cget -modus]

    calculate_slits $resol

    #++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # After having initialized slit-less spectroscopy,
    # switch back to original mode
    if { [string compare [$this cget -slit_width] open] == 0 } {
        if { [string compare [$this cget -modus] img] == 0 } {
            $this configure -modus mos
            # puts stdout "SLS img -> mos | [$this cget -modus] $mask"
        }
    }
    if { [string compare [$this cget -slit_width] open] == 0 } {
        if { [string compare [$this cget -modus] ipol] == 0 } {
            $this configure -modus pmos
            # puts stdout "SLS ipol-> pmos | [$this cget -modus] $mask"
        }
    }

    #++++++++++++++++++++++++++++++++++++++++++++++++++++++
    #--------------------
    # FRAME
    #--------------------
    # chapter 5.0.0.0.0  frame corners
    calculate_focalfield_frame SR
    calculate_focalfield_frame HR

    # Some debug tests
    if { $debug } {
        catch { $m_image convert coords $wx $wy "deg $EQX" p1 p2 canvas }
        catch { puts -nonewline $lfId "world2: ($wx,$wy) = canvas($p1,$p2)" }
        catch { $m_image convert coords $wx $wy "deg $EQX" p1 p2 image }
        catch { puts -nonewline $lfId "= image($p1,$p2)" }
        catch { $m_image convert coords $wx $wy "deg $EQX" p1 p2 screen }
        catch { puts -nonewline $lfId " = screen($p1,$p2)" }
        catch { $m_image convert coords $wx $wy "deg $EQX" p1 p2 "wcs $EQX" }
        catch { puts $lfId " = wcs($p1,$p2)" }
    }

    # The angle between mask and pixel/canvas/screen system
    set abs_ang_rad 0.0
    # The angle between mask and WCS, depends on the orientation
    # of the WCS, see CSO
    set PAM 0.0

    # 23/SEP/98 : a very special event
    # FORS1 is mounted on UT1/VLT in the correct way
    # but the sky field orientation in the FORS1 focal field is
    # different to the Basic Telescope Definiton
    # in contradiction to the FORS-Reference-System manual
    # Solution: the zero point of the rotator corresponds to
    # 180 degrees position angle

    if { $mountangle == 180 } {
        if { $debug } {
            puts stdout "world2: Mountangle = $mountangle, frame has been rotated 180 (deg)"
        }
    }

    # Configure grism dispersion for interference filters
    # puts stdout "MXU with interference filters"
    # puts stdout "-----------------------------"
    # puts stdout "lamcen(grism)=$A_disp($good,cen) nm "

    # Grism dispersion from [nm/mm(CCD)] to [mm(ff)/nm]
    # Last check 15/09/99 hummel

    # When order separation filters are used instead of interference
    # filters, mxu_class will indicate the dispersion stripe until
    # the rim of the CCD otherwise the FWHM of the interference filter
    # is used to indicate the predicted length of the spectrum.
    # But there is one exception: the low dispersion grisms
    # GRIS_150I+27 and GRIS_200I+28.
    # therefore for order seperation filters we will calculate the
    # broad band FWHM and the wavelength of the OS filter center
    # from the disperion table
    # Hummel: 08.08.2000 No, broad band filter extra branch
    # no longer supported for targ2

    if { [string match "*hits*" $lmod] } {
        set corr_flag [$this cget -fors_frame]
        fix_mask_slit hits $corr_flag
    }
    if { [string match "*lss*" $lmod] } {
        set corr_flag [$this cget -fors_frame]
        fix_mask_slit lss $corr_flag
    }
    # if { [string match "*mxu*" $lmod] } {
    #     puts "Introduce HERE the loading of FAD file"
    #     load_mxu_FAD $set_dir/mxu2.009.standard.fad
    # }
    set mask_initialized 1
    if { $debug } {
        puts stdout "world2: END"
    }
    tracer LEAVING
}

#======================================
private method align_mask { w sx sy } {
#======================================
# For echs, lss, mxu (standard maks mode), hitms or hits :
# Move the mask to the center/gauss result
# w = canvas widget
# sx, sy is cursor pos in screen coords
# on==1  : select, on==0 : un-select, give slit free
# stop==1: do nothing (when selecting a reference object)
# stop==0: do the normal procedure

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $stop } {
        tracer LEAVING STOP
        return
    }
    # puts stdout "in align_mask"
    set my 1
    set l_mod [$this cget -modus]

    catch { $m_image convert coords $sx $sy screen x y canvas }

    # Old (current mask center)
    if { $bonsai_flag } {
        set dum [C_wc w2c $world_x $world_y]
        catch { $m_image convert coords [lindex $dum 0] [lindex $dum 1] canvas ocx ocy screen }
        catch { $m_image convert coords $world_x $world_y "deg $EQX" a b "wcs $EQX" }
    } else {
        catch { $m_image convert coords $world_x $world_y "deg $EQX" ocx ocy screen }
        catch { $m_image convert coords $world_x $world_y "deg $EQX" a b "wcs $EQX" }
    }

    puts $lfId "AliMask: jump slit to $a $b"
    # Calculate new mask position in screen coords
    # new_mask = old_mask + (center/gauss - current slit )
    # sx sy screen are cursor or center/gauss
    # current slit is either MOS 10 (echs) or HITS ? (hits) or pinhole

    #-------------------
    # center-slit option
    #-------------------
    if { [$this cget -pics] } {
        # sx,sy are approximate coords,
        # use RtdImagePick to center slit on image target
        catch { $m_image convert coords $sx $sy screen a0 d0 image }
        # puts stdout "Cursor canvas=$x $y screen=$sx $sy pixel=$a0 $d0"
        set picked {}
        pick_star
        # set FWHM [lindex $picked 5]
        set FWHM [expr {$FWHM_X + $FWHM_Y}]
        if { $FWHM > 0  } {
            catch { $m_image convert coords $star_ix $star_iy image sx sy screen }
            catch { $m_image convert coords $star_ix $star_iy image  x  y canvas }
            # set CCD pixel of target
            # set targw($my,x) $star_ix
            # set targw($my,y) $star_iy
            # puts stdout [format "targw(%2d,x) = %8.3f, targw(%2d,y) = %8.3f" \
            #                      $my $targw($my,x) $my  $targw($my,y)]
            # puts stdout "center_slit from ($a0,$d0) on center_gauss result=\
            #              ($star_ix,$star_iy) pixel"
        } else {
            # puts stdout "center_slit failed, use cursor at ($a0,$d0) pixel"
            # If we are gauss centering it is not desirable to let the mask be
            # moved to the cursor pixel when pick failed.
            set sx 0.0
            set sy 0.0
            set x 0.0
            set y 0.0
        }
    } else {
        tracer LEAVING NO_PICS
        return
    }

    # MXU, HIT-MS
    # we use the first created mxu slit
    if { [string compare $l_mod hitms] == 0 ||
         [string compare $l_mod mxu] == 0 } {
        fors::MXU_SLIT::coeff $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                              $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                              $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
        # x,y -> xc,yc -> ra,dec in mxu
        C_mxus1 M2W
        set targw(1,a) [C_mxus1 get_RA]
        set targw(1,d) [C_mxus1 get_DEC]
        catch { $m_image convert coords $targw(1,a) $targw(1,d) "wcs $EQX" ssx ssy screen }
    }

    # ECHS --------------------
    # we use the center of MOS 10
    if { [string match $l_mod echs] } {
        set X [expr {($mos(10,xb) + $mos(10,xc) + $mos(10,xf) + $mos(10,xg))*0.25}]
        set Y [expr {($mos(10,yb) + $mos(10,yc) + $mos(10,yf) + $mos(10,yg))*0.25}]
        catch { $m_image convert coords $X $Y canvas ssx ssy screen }
    }

    # LSS- --------------------
    # we use the center of the long slit in the mask
    # which is not necessarily at Y=0 in the focal field
    if { [string match $l_mod lss] } {
        set X [expr {($mos($long_number,xa) + $mos($long_number,xd) + $mos($long_number,xe) + $mos($long_number,xh))*0.25}]
        set Y [expr {($mos($long_number,ya) + $mos($long_number,yd) + $mos($long_number,ye) + $mos($long_number,yh))*0.25}]
        catch { $m_image convert coords $X $Y canvas ssx ssy screen }
    }

    # HITS --------------------
    if { [string match $l_mod hits] } {
        if { $long_number == 1 } {
            # --------------- PINHOLE ALIGNMENT
            # highres collimator with hits -> pinhole
            # where is it ? in mos(9,adeh) (long_number==1)
            # square points, we assume long_number==1
            set X [expr {($mos(9,xa) + $mos(9,xd) + $mos(9,xe) + $mos(9,xh))/4}]
            set Y [expr {($mos(9,ya) + $mos(9,yd) + $mos(9,ye) + $mos(9,yh))/4}]
            catch { $m_image convert coords $X $Y canvas ssx ssy screen }
            puts stdout "current BPH position is $ssx $ssy"
        } else {
            # --------------- HT-SLIT ALIGNMENT
            # some parts of the ht slit is outside the CCD.
            # consider only that part of the htslit inside
            # the focalfield (=CCD). We dont put the
            # center/gauss-target on the center of the
            # focal field part of the htslit but very close
            # to the rim of the slit. Make the ht-slit length
            # smaller until there is a square (slit-width==slit_length).
            # put target on center of this square.

            # any of the available slits, not the pinhole !!!
            # slit width in canvas
            set dx [expr {$mos($long_number,xa) - $mos($long_number,xd)}]
            set dy [expr {$mos($long_number,ya) - $mos($long_number,yd)}]
            set A [expr {sqrt($dx*$dx + $dy*$dy)}]

            # full slit length in canvas
            set dx [expr {$mos($long_number,xd) - $mos($long_number,xh)}]
            set dy [expr {$mos($long_number,yd) - $mos($long_number,yh)}]
            set B [expr {sqrt($dx*$dx + $dy*$dy)}]

            # square points
            set xhn [expr {$mos($long_number,xd) + ($mos($long_number,xh) - $mos($long_number,xd))*$A/$B}]
            set yhn [expr {$mos($long_number,yd) + ($mos($long_number,yh) - $mos($long_number,yd))*$A/$B}]
            set xen [expr {$mos($long_number,xa) + ($mos($long_number,xe) - $mos($long_number,xa))*$A/$B}]
            set yen [expr {$mos($long_number,ya) + ($mos($long_number,ye) - $mos($long_number,ya))*$A/$B}]

            set X [expr {($mos($long_number,xa) + $mos($long_number,xd) + $xhn+$xen)/4}]
            set Y [expr {($mos($long_number,ya) + $mos($long_number,yd) + $yhn+$yen)/4}]
            catch { $m_image convert coords $X $Y canvas ssx ssy screen }
        }
    }

    set newx [expr {$ocx + $sx - $ssx}]
    set newy [expr {$ocy + $sy - $ssy}]

    # before we shift, update the oldpos in mark
    set oldpos(x) $mos(0,x0)
    set oldpos(y) $mos(0,y0)
    # shift mask to new screen coords
    shift $w $newx $newy

    if { [string match $l_mod echs] } {
        echs_targ
    }

    # now we apply the correction for optical distortion
    # of FORS images and differential refraction
    set corr_flag [$this cget -fors_frame]

    # <><><><><><><><><><><><><><><><><><><><><>
    # workaround 13. Sept 2000 WH
    # FORS1 images are aliens for MXU
    # if { [string match $l_mod mxu] } {
    #     if { [string compare [$m_image fits get INSTRUME] "FORS1"] == 0 } {
    #         set corr_flag 0
    #     }
    # }
    # <><><><><><><><><><><><><><><><><><><><><>

    fix_mask_slit $l_mod $corr_flag
    tracer LEAVING
}

#============================
public method reset_mask {} {
#============================
    tracer ENTERING
    global mw_list

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Reset the mask
    # Telescope pointing in [canvas]
    # puts $lfId "Reset_Mask:  m_image=>>$m_image<< \

    # Destroy slit parameter editor panels left around this mask
    if { [llength mw_list] > 0 } {
        foreach mw $mw_list {
            if { [info exists mw] } {
                catch { destroy $mw }
            }
        }
        set mw_list ""
    }

    # Update mask value
    # Update the mask mode
    if { [string compare [$this cget -modus] "ipol"] == 0 } {
        if { [string match *HR* [$this cget -collimat]] } {
            set mask pol
            # modus=ipol mask=pol
        } else {
            set mask mos
            # modus=ipol mask=mos
        }
    }

    # In case a new image is loaded
    # mask center x_ff=0,y_ff=0 is center of frame
    set world_x [$m_image fits get CRVAL1]
    set world_y [$m_image fits get CRVAL2]
    if { $debug && $verbose } {
        puts "reset_mask: image frame mode"
        puts "            world_x = >$world_x<"
        puts "            world_y = >$world_y<"
        puts ""
    }

    # Frameless mode
    # maybe better a string length operation
    if { [string length $world_x] == 0 || [string length $world_y] == 0 } {
        set wx [lindex [$m_image wcsset] 0]
        set wy [lindex [$m_image wcsset] 1]
        set world_x 0
        set wordl_y 0
        if { [catch { $m_image convert coords $wx $wy "wcs $EQX" world_x world_y "deg $EQX" } result] } {
            if { $debug && $verbose } {
                puts "reset_mask: Frameless reference (wx,wy)"
                puts "            Error converting ($wx,$wy) WCS to (world_x,world_y) \[deg\] coord."
                puts "            $result" 
                puts ""
            }
        }
        if { $debug && $verbose } {
            puts "reset_mask: frameless mode"
            puts "            wx      = >$wx<"
            puts "            wy      = >$wy<"
            puts "            world_x = >$world_x<"
            puts "            world_y = >$world_y<"
            puts ""
        }
    }

    # write Warning if FORS image is processed by reduction software
    # for status panel
    set sp_instrume [$m_image fits get INSTRUME]
    set sp_origin   [$m_image fits get ORIGIN]

    # if { [string match *FORS* $sp_instrume] } {
    #     if { [string compare ESO $sp_origin] } {
    #         puts stdout "Warning: FITS file observed with $sp_instrume \
    #                      but processed by $sp_origin"
    #     }
    # }

    set wx $world_x
    set wy $world_y
    set ccx [$m_image fits get CRPIX1]
    set ccy [$m_image fits get CRPIX2]
    set PAM 0
    set tcs_pam 0
    # puts stdout [format "reset_mask : CCD REF_PIX %10e %10e %7.2f %7.2f" \
    #                      $wx $wy $ccx $ccy]

    # Delete all instances of MXU slit
    mxu_delete_all_slits

    init_mask_world2

    catch {$m_canvas delete A}
    catch {$m_canvas delete D}
    plot_mask

    # call echs_pos and echs targ after mask had been
    # plotted
    if { [string compare [$this cget -modus] echs] == 0 } {
        echs_pos
        echs_targ
    }

    # badpixelmap
    if { $badpixelmap } {
        update_BPM
    }
    # Do not load refslit for hitms
    if { [string match "*mxu*" [$this cget -modus]] } {
        mxu_create_refslit
    }
    # Added this to recover the wave limits
    eval configure { -filter [$this cget -filter] -grism [$this cget -grism] }

    set AutoposMethod ""

    set last_action "Reset Mode"
    catch {SP update}
    tracer LEAVING
}

#=============================
public method update_mask {} {
#=============================
# What happens if you zoom the image or
# shift screen versus canvas ?
# 1. rescale the mask according to the new image zoom
#    update the matrix mos(..,..)
# 2. world coordinates retain

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Check on new scale, only used for trace
    set newscale [lindex [$m_image scale] 0]
    if { $newscale < 0 } {
        set newscale [expr -1.0/$newscale]
    }
    set stretch [expr (1.0*$newscale)/(1.0*$canvscale)]
    if { $debug && $verbose } {
        # puts $lfId "NewScale = $newscale, OldScale = $canvscale"
        puts stdout "update_mask: newscale  = $newscale"
        puts stdout "             canvscale = $canvscale"
        puts stdout "             stretch   = $stretch"
    }

    # Check on shift between screen and canvas
    # puts $lfId "$ccx $ccy"
    # $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
    #                         nwx nwy "deg $EQX"
    # puts $lfId "OLD: $canvscale $mos(0,x0) $mos(0,y0) $nwx $nwy \
    #                  $world_x $world_y"
    # Do not update mos(0,x0), mos(0,y0) alone !!!
    # $m_image convert coords $world_x $world_y "deg $EQX" \
    #                         ccx ccy canvas
    set canvscale $newscale

    # 1. Determine new canvas postion using retained world coords
    #    Do this in all cases
    set dum [C_wc w2c $world_x $world_y]
    set x [lindex $dum 0]
    set y [lindex $dum 1]
    # puts stdout " bonsai x y $x $y"
    # puts stdout [$m_image convert coords $world_x $world_y "deg $EQX" \
    #                                      x y canvas]
    # puts stdout " convertx y $x $y"
    # The distance between old and new position in canvas coordinates
    # This is not the mark position but the mask center
    # puts stdout [format "->   %6.1f == %6.1f %6.1f == %6.1f" \
    #                      $x $ccx $y $ccy]
    set shift_x [expr $x - $ccx]
    set shift_y [expr $y - $ccy]

    # Set new center coordinates
    set ccx $x
    set ccy $y
    # puts stdout [format "->   shift = %6.1f %6.1f" $shift_x $shift_y]
    # puts stdout [format "->   %6.1f == %6.1f %6.1f == %6.1f" \
    #                     $x $ccx $y $ccy]

    puts $lfId "update_mask: scale = $newscale stretch = $stretch world = ($world_x, $world_y)"

    # Shift the mos to the new canvas but old world position
    # 19 slit blades
    switch -exact -- $mask  {
        mos {
            for { set i 1 } { $i < 20 } { incr i 1 } {
                foreach val { a b c d e f g h } {
                    set mos($i,x$val) [expr $mos($i,x$val)+$shift_x]
                    set mos($i,y$val) [expr $mos($i,y$val)+$shift_y]
                }
	    }
        }
        hits -
        lss {
            for { set i 1 } { $i < 10 } { incr i 1 } {
                foreach val { a b d e f h } {
                    set mos($i,x$val) [expr $mos($i,x$val)+$shift_x]
                    set mos($i,y$val) [expr $mos($i,y$val)+$shift_y]
                }
            }
        }
        pol {
            for { set i 1 } { $i < 10 } { incr i 1 } {
                foreach val { a b d e f h } {
                    set mos($i,x$val) [expr $mos($i,x$val)+$shift_x]
                    set mos($i,y$val) [expr $mos($i,y$val)+$shift_y]
                }
            }
        }
        hitms -
        mxu {
        }
        default {
            puts stdout "Warning: mask update_mask"
        }
    }

    if { $shift_x != 0 || $shift_y != 0 } {
        # The frame and the central point
        # -------------------------------
        foreach val { 0 a b c d e f g h ar br cr dr er fr gr hr stl sbl sel swl htl hbl hel hwl slbl slrl hlbl hlrl sb1s sb1f sb2s sb2f sr1s sr1f sr2s sr2f hb1s hb1f hb2s hb2f hr1s hr1f hr2s hr2f} {
            set mos(0,x$val) [expr $mos(0,x$val)+$shift_x]
            set mos(0,y$val) [expr $mos(0,y$val)+$shift_y]
            set ccd_frame(0,x$val) [expr $ccd_frame(0,x$val)+$shift_x]
            set ccd_frame(0,y$val) [expr $ccd_frame(0,y$val)+$shift_y]
            set ccd_frame(1,x$val) [expr $ccd_frame(1,x$val)+$shift_x]
            set ccd_frame(1,y$val) [expr $ccd_frame(1,y$val)+$shift_y]
        }
    }

    if { $stretch != 1 } {
        # Rescale the size of the mos according to the new zoom
        # The frame and the central point
        # -------------------------------
        foreach val { 0 a b c d e f g h ar br cr dr er fr gr hr stl sbl sel swl htl hbl hel hwl slbl slrl hlbl hlrl sb1s sb1f sb2s sb2f sr1s sr1f sr2s sr2f hb1s hb1f hb2s hb2f hr1s hr1f hr2s hr2f} {
            set mos(0,x$val) [expr $mos(0,x0)+($mos(0,x$val)-$mos(0,x0))*$stretch]
            set mos(0,y$val) [expr $mos(0,y0)+($mos(0,y$val)-$mos(0,y0))*$stretch]
            set ccd_frame(0,x$val) [expr $ccd_frame(0,x0)+($ccd_frame(0,x$val)-$ccd_frame(0,x0))*$stretch]
	    set ccd_frame(0,y$val) [expr $ccd_frame(0,y0)+($ccd_frame(0,y$val)-$ccd_frame(0,y0))*$stretch]
            set ccd_frame(1,x$val) [expr $ccd_frame(1,x0)+($ccd_frame(1,x$val)-$ccd_frame(1,x0))*$stretch]
            set ccd_frame(1,y$val) [expr $ccd_frame(1,y0)+($ccd_frame(1,y$val)-$ccd_frame(1,y0))*$stretch]
        }
        # The green circle (radius is a distance)
        set mos(0,r) [expr $mos(0,r)*$stretch]
    }

    switch -exact -- $mask {
        mos {
            for { set i 1 } { $i < 20 } { incr i 1 } {
                foreach val { a b c d e f g h } {
                    set mos($i,x$val) [expr $mos(0,x0)+($mos($i,x$val)-$mos(0,x0))*$stretch]
                    set mos($i,y$val) [expr $mos(0,y0)+($mos($i,y$val)-$mos(0,y0))*$stretch]
                }
            }
        }
        pol {
            for { set i 1 } { $i < 10 } { incr i 1 } {
                foreach val { a b d e f h } {
                    set mos($i,x$val) [expr $mos(0,x0)+($mos($i,x$val)-$mos(0,x0))*$stretch]
                    set mos($i,y$val) [expr $mos(0,y0)+($mos($i,y$val)-$mos(0,y0))*$stretch]
                }
            }
        }
        hits -
        lss {
            for { set i 1 } { $i < 10 } { incr i 1 } {
                foreach val { a b d e f h } {
                    set mos($i,x$val) [expr $mos(0,x0)+($mos($i,x$val)-$mos(0,x0))*$stretch]
                    set mos($i,y$val) [expr $mos(0,y0)+($mos($i,y$val)-$mos(0,y0))*$stretch]
                }
            }
        }
        hitms -
        mxu {
            fors::MXU_SLIT::coeff $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                                  $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                                  $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
        }
        default {
            puts stdout "Warning: mask update_mask"
        }
    }

    # The reference stars
    # -------------------
    for { set i 1 } { $i <= 10 } { incr i 1 } {
        catch {
            set ref($i,x) [expr $mos(0,x0)+($ref($i,x)-$mos(0,x0))*$stretch]
            set ref($i,y) [expr $mos(0,y0)+($ref($i,y)-$mos(0,y0))*$stretch]
        }
    }

    # Set the index according to long_width
    switch -exact -- [$this cget -long_width] {
        all {set long_number 0 }
        0.3 {set long_number 1 }
        0.4 {set long_number 2 }
        0.5 {set long_number 3 }
        0.7 {set long_number 4 }
        1.0 {set long_number 5 }
        1.3 {set long_number 6 }
        1.6 {set long_number 7 }
        2.0 {set long_number 8 }
        2.5 {set long_number 9 }
        default {
            puts stdout "Warning: LSS slit"
        }
    }
    tracer LEAVING
}

#=================================
private method update_mask_na {} {
#=================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set na M[format "%03d" $exposure_counter][$this cget -file_name]
    set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
    catch { fors::MXU_SLIT::set_MASK_NAID "$na [lindex $PM 1]" }
    catch {SP update}
    tracer LEAVING
}


#=================================
private method update_mask_id {} {
#=================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set old_id [exec date -u +%H%M%S]
    set l [string length $old_id]
    set id "9"
    for { set i 1 } { $i < $l } { incr i 1 } {
        append id [string index $old_id $i]
    }
    if { $id < 900100 } {
        # oh, its accidently a (forbidden) technical mask number
        set id [expr {$id + 1000}]
    }
    set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
    catch { fors::MXU_SLIT::set_MASK_NAID "[lindex $PM 0] $id" }
    catch {SP update}
    tracer LEAVING
}


#===================================
private method update_mask_naid {} {
#===================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # this is called when a project name is specified
    set na M[format "%03d" $exposure_counter][$this cget -file_name]
    set old_id [exec date -u +%H%M%S]
    set l [string length $old_id]
    set id "9"
    for { set i 1 } { $i < $l } { incr i 1 } {
        append id [string index $old_id $i]
    }
    if { $id < 900100 } {
        # oh, its accidently a (forbidden) technical mask number
        set id [expr {$id + 1000}]
    }
    catch { fors::MXU_SLIT::set_MASK_NAID "$na $id" }
    catch {SP update}
    tracer LEAVING
}

#=================================
private method update_labels {} {
#=================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { [$this cget -label] } {
        catch { $m_canvas delete D }
        plot_tcs_label
        plot_ins_label
    }
    tracer LEAVING
}

#===================================
public method update_and_replot {} {
#===================================
# Does not reset the mask but replots it at
# the current position with current configuration
# options but with different image zoom, flip option ...
# and plot_mask it
# Scratch the whole old mask and plot_mask

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    update_scale
    update_mask
    catch {$m_canvas delete A}
    catch {$m_canvas delete D}
    # badpixelmap
    if { $badpixelmap } {
        update_BPM
    }
    set reply [plot_mask]
    tracer LEAVING
    return $reply
}

#==============================
public method update_color {} {
#==============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    switch -exact -- [$this cget -colorset] {
        Black/White   {set color1 black;   set color2 white}
        Blue/White    {set color1 blue;    set color2 white}
        Red/Yellow    {set color1 red;     set color2 yellow}
        Yellow/Blue   {set color1 yellow;  set color2 blue}
        Yellow/White  {set color1 yellow;  set color2 white}
        Orange/Yellow {set color1 orange;  set color2 yellow}
        Purple/Orange {set color1 purple;  set color2 orange}
        Bright        {set color1 white;   set color2 yellow}
        Dark          {set color1 #000000; set color2 #2f2f2f}
        Gray          {set color1 #9f9f9f; set color2 #6f6f6f}
        default       {puts $lfId "UpdateColor: Warning ColorSet"}
    }
    set color3 $color1
    set color4 $color2

    if { $debug && $verbose } {
        puts stdout "update_color: "
        puts stdout "              Color 1: $color1"
        puts stdout "              Color 2: $color2"
        puts stdout "              Color 3: $color3"
        puts stdout "              Color 4: $color4"
    }

    # Transparency
    switch -glob -- [$this cget -graytype] {
              0 { 
                  # Look through
                  set color1 ""
                  set color2 ""
                  # For green circle
                  set trapa gray25
                }
             ?? {
                  set trapa gray[$this cget -graytype]
                }
            100 {
                  # Background obscured
                  set trapa ""
                }
              1 {
                  set trapa gray25
                }
        default {
                  puts $lfId "UpdateColor: Warning, GrayType = [$this cget -graytype]"
                }
    }

    if { $debug && $verbose } {
        puts stdout "update_color: "
        puts stdout "              Trapa:   $trapa"
        puts stdout "              Color 1: $color1"
        puts stdout "              Color 2: $color2"
        puts stdout "              Color 3: $color3"
        puts stdout "              Color 4: $color4"
    }
    tracer LEAVING
}

#==================================================
public method update_xform_coeff { {of_whom ""} } {
#==================================================
# Updates coefficients of the coordinate transformation matrices in
# C_mc, C_wc, and C_mxu'i' objects

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Select resolution parameters (mm)
    set resol [$this cget -collimat]
    if { [string match *SR* $resol] } {
        # SR
        set XMIN $XSRMIN
        set YMIN $YSRMIN
        set XMAX $XSRMAX
        set YMAX $YSRMAX
        # Corner slots of the Frame Arrays
        lassign {a b c d} a b c d
    } else {
        # HR
        set XMIN $XHRMIN
        set YMIN $YHRMIN
        set XMAX $XHRMAX
        set YMAX $YHRMAX
        # Corner slots of the Frame Arrays
        lassign {e f g h} a b c d
    }

    # For the mask
    # radec world coords --> canvas coords
    C_wc coeff_wc $wx $wy
    # mm focplane coords --> canvas coords
    C_mc coeff_mc $XMIN $YMIN $mos(0,x$a) $mos(0,y$a) \
                  $XMAX $YMIN $mos(0,x$b) $mos(0,y$b) \
                  $XMAX $YMAX $mos(0,x$c) $mos(0,y$c)
    # For the MXU slits
    if { [string compare [$this cget -modus] hitms] == 0 ||
         [string compare [$this cget -modus] mxu] == 0 } {
        fors::MXU_SLIT::coeff $XMIN $YMIN $mos(0,x$a) $mos(0,y$a) \
                              $XMAX $YMIN $mos(0,x$b) $mos(0,y$b) \
                              $XMAX $YMAX $mos(0,x$c) $mos(0,y$c) \
                              $of_whom
    }
    tracer LEAVING
}

#===================================
private method update_bez_oc { i } {
#===================================
# Copy bezier points out of mxu class
# Apply optical distortion (or not)
# Copy values back to mxu class
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set dummy [C_mxus$i get_curve_ca]
    set targw(1,a) [C_mxus$i get_RA]
    set targw(1,d) [C_mxus$i get_DEC]

    $m_image convert coords [lindex $dummy 0] [lindex $dummy 1] canvas \
                            targw(1,x) targw(1,y) image
    calc_zemax 1 $targw(1,x) $targw(1,y)
    C_mxus$i put_b_oc 1 [list $targw(1,cx) $targw(1,cy)]

    $m_image convert coords [lindex $dummy 2] [lindex $dummy 3] canvas \
                            targw(1,x) targw(1,y) image
    calc_zemax 1 $targw(1,x) $targw(1,y)
    C_mxus$i put_b_oc 2 [list $targw(1,cx) $targw(1,cy)]

    $m_image convert coords [lindex $dummy 4] [lindex $dummy 5] canvas \
                            targw(1,x) targw(1,y) image
    calc_zemax 1 $targw(1,x) $targw(1,y)
    C_mxus$i put_b_oc 3 [list $targw(1,cx) $targw(1,cy)]

    $m_image convert coords [lindex $dummy 6] [lindex $dummy 7] canvas \
                            targw(1,x) targw(1,y) image
    calc_zemax 1 $targw(1,x) $targw(1,y)
    C_mxus$i put_b_oc 4 [list $targw(1,cx) $targw(1,cy)]

    $m_image convert coords [lindex $dummy 8] [lindex $dummy 9] canvas \
                            targw(1,x) targw(1,y) image
    calc_zemax 1 $targw(1,x) $targw(1,y)
    C_mxus$i put_b_oc 5 [list $targw(1,cx) $targw(1,cy)]

    $m_image convert coords [lindex $dummy 10] [lindex $dummy 11] canvas \
                            targw(1,x) targw(1,y) image
    calc_zemax 1 $targw(1,x) $targw(1,y)
    C_mxus$i put_b_oc 6 [list $targw(1,cx) $targw(1,cy)]
    tracer LEAVING
}

#==============================
private method update_BPM { } {
#==============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # convert badpixelmask from mm ff to canvas
    for { set i 1 } { $i <= $BPM_counter } { incr i 1 } {
        set a [C_mc m2c $BPM($i,xmin) $BPM($i,ymin)]
        set BPM($i,x1) [lindex $a 0]; set BPM($i,y1) [lindex $a 1]

        set a [C_mc m2c $BPM($i,xmin) $BPM($i,ymax)]
        set BPM($i,x2) [lindex $a 0]; set BPM($i,y2) [lindex $a 1]

        set a [C_mc m2c $BPM($i,xmax) $BPM($i,ymax)]
        set BPM($i,x3) [lindex $a 0]; set BPM($i,y3) [lindex $a 1]

        set a [C_mc m2c $BPM($i,xmax) $BPM($i,ymin)]
        set BPM($i,x4) [lindex $a 0]; set BPM($i,y4) [lindex $a 1]
    }
    plot_BPM
    tracer LEAVING
}

#==========================
private method get_cat {} {
#==========================
# Gets all AstroCat widgets, sends the
# list to select_cat and returns the selected
# catalog

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set test1 [itcl::find objects]
    set a [split $test1 " "]
    set b [lsearch -glob $a *AstroCat*]
    set a [lindex $a $b]
    set b [split $a "::"]
    set a [llength $b]
    set b [lindex $b [expr {$a - 1}]]
    set a [split $b "."]
    set gc ".[lindex $a 1].[lindex $a 2]"
    set gc [itcl::find objects -class ::skycat::SkySearch]
    set gc [split $gc " "]
    set ll [llength $gc]

    if { $ll > 1 } {
        set gc [select_cat $gc]
    }
    tracer LEAVING
    return $gc
}

#===============================
private method get_ec { mode } {
#===============================
# Gets exposure counter (ec)
# Scans setupfile directory with mode filter
# and sets ec to the preceeding value

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set local_dir "$set_dir/"
    if { [catch { set a [exec ls -1 $local_dir | grep $mode] } result] } {
        puts "$result\nNo setup files found; set expo-counter 1"
        set exposure_counter 1
    } else {
        set a [split $a \n]
        set exposure_counter 0
        for { set i 0 } { $i < [llength $a] } { incr i 1 } {
            # puts stdout ">[lindex $a $i]<"
            set part [split [lindex $a $i] .]
            # PBG: 20071129: Test only part 1 of file name, since file names are
            #                created always with ec after the first dot.
            # for { set j 0 } { $j < [llength $part] } { incr j 1 } {
                # set test [lindex $part $j]
                set test [lindex $part 1]
                if { [scan $test {%d} nr] } {
                    if { $nr > $exposure_counter } {
                        set exposure_counter $nr
                    }
                }
            # }
        }
        incr exposure_counter 1
    }
    tracer LEAVING
    return $exposure_counter
}

#=================================
private method get_fits_name { } {
#=================================
# Gets the 6 first chars of the fits file
# as an initial project name

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set name [$m_image cget -file]
    if { [string length $name] == 0 } {
        set name "no_frame"
    }
    
    return [file rootname [file tail $name]]
    tracer LEAVING
}

#=================================
private method extra_lss { a b } {
#=================================
# split the result string of PickObject
# and copy values to globals for further
# processing

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set b [split $b " "]
    switch -exact -- $a {
        1 {
            set lss_slit_ra [lindex $b 0]
            set lss_slit_de [lindex $b 1]
        }
        2 {
            set lss_ref_ra [lindex $b 0]
            set lss_ref_de [lindex $b 1]
        }
    }
    tracer LEAVING
}

#=============================
private method pick_star { } {
#=============================
    tracer ENTERING
    set io [$m_skycat component image]
    tracer LEAVING
    return [$io pick_dialog [code $this star_picked]]
}

#=================================
private method star_picked { b } {
#=================================
# split the result string of PickObject
# and copy values to globals for further
# processing
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 1}]

    if { $debug && $verbose } {
        puts stdout "star_picked: Data received from PickObject dialog:"
        puts stdout "             $b"
    }
    set picked [split $b " "]
    if { [lindex $picked 5] > 0 } {
        # Successfull center gauss
        set star_ix [lindex $picked 0]
        set star_iy [lindex $picked 1]
        set star_ra [lindex $picked 2]
        set star_de [lindex $picked 3]
        set EQUINOX [lindex $picked 4]
        set FWHM_X  [lindex $picked 5]
        set FWHM_Y  [lindex $picked 6]
        if { $debug && $verbose } {
            puts stdout "             Success:"
            puts stdout "                   star_ix = >$star_ix<"
            puts stdout "                   star_iy = >$star_iy<"
            puts stdout "                    FWHM_X = >$FWHM_X<"
            puts stdout "                    FWHM_Y = >$FWHM_Y<"
        }
    } else {
        set star_ix 0.0
        set star_iy 0.0
        set star_ra 00:00:00.00
        set star_de 00:00:00.00
        set EQUINOX 0
        set FWHM_X  0.0
        set FWHM_Y  0.0
        if { $debug && $verbose } {
            puts stdout "             Center Slit failed !"
        }
    }
    if { $debug && $verbose } {
        puts stdout "star_picked: END"
    }
    tracer LEAVING
}

#============================
private method echs_pos { } {
#============================
# Position 1 or 2 ECHS slits after mask rotation
# and after reset_mask
# we handle both ECHS slits, built by MOS as
# a fixed mask

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set y1   0.0;  # hardcoded center of MOS 10
    set y2 -10.0;  # hardcoded center of  MOS 9
    # set y2  10;    # hardcoded center of MOS 11, no longer used

    # we have three hardcoded x-positions,
    # dependent on blaze angle positions of XGRISM
    # default value is -80
    # puts stdout ">$echelle_range<"
    set x -80.0
    foreach val $echelle_range {
        # puts stdout ">[$this cget -grism]<"
        # puts stdout ">$val<"
        if {[string match [$this cget -grism] $val]} {
            # a constant mos position for all EGRIS-XGRIS-OSfilter combinations
            set x 0.0
        }
    }
    if { $x == -80.0 } {
        puts "Warning >>> XGRIS not correctly defined"
        foreach val $echelle_range {
            # puts -nonewline stdout ">$val<"
        }
        # puts stdout ""
        for { set i 1 } { $i <= 4 } { incr i 1 } {
            # puts -nonewline stdout ">[$this cget -grism]<"
        }
        # puts stdout ""
    } else {
        # XGRIS is correctly defined
        # 2 transformations mm->canvas->screen
        # MOS blade pair 10
        set a [C_mc m2c $x $y1]
        catch { $m_image convert coords [lindex $a 0] [lindex $a 1] canvas sx sy screen }
        # some diagnostics
        set loca 0
        set locd 0
        catch { $m_image convert coords [lindex $a 0] [lindex $a 1] canvas loca locd "wcs $EQX" }
        puts stdout "echs_pos10: desired position = $loca $locd"

        set tx [lindex $a 0]; set ty [lindex $a 1]

        set a [$m_canvas find overlapping \
                 [expr $tx-1] [expr $ty-1] \
                 [expr $tx+1] [expr $ty+1]]
        # foreach c $a {
        #     set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
        #     puts stdout $local_tags
        # }

        $m_canvas delete ali
        if { [catch { $this move_slit $m_canvas $sx $sy 1 0 } result] } {
            puts stdout "echs_pos10: result = $result, grism,x,y= [$this cget -grism] $x $y1"
            puts stdout [format \
                              ">>targw(10,x,y,cx,cy)= %8.4f %8.4f %8.4f %8.4f" \
                         $targw(10,x) $targw(10,y) $targw(10,cx) $targw(10,cy)]
            puts stdout [format \
                              ">>targw(10,a,d,ca,cd)= %8.4f %8.4f %8.4f %8.4f" \
                          $targw(10,a) $targw(10,d) $targw(10,ca) $targw(10,cd)]
        } else {
            puts stdout "MOS \#10 positioned"
        }

        if { $echs_flag } {
            # MOS blade pair 9 is also active
            set a [C_mc m2c $x $y2]
            catch { $m_image convert coords [lindex $a 0] [lindex $a 1] canvas sx sy screen }
            # some diagnostics
            set loca 0
            set locd 0
            catch { $m_image convert coords [lindex $a 0] [lindex $a 1] canvas loca locd "wcs $EQX" }
            puts stdout "echs_pos 9: desired position = $loca $locd"
            set tx [lindex $a 0]; set ty [lindex $a 1]
            set a [$m_canvas find overlapping \
                  [expr $tx-1] [expr $ty-1] \
                  [expr $tx+1] [expr $ty+1]]
            # foreach c $a {
            #     set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
            #     puts stdout $local_tags
            # }
            $m_canvas delete ali
            if { [catch { $this move_slit $m_canvas $sx $sy 1 0 } result] } {
                puts stdout "echs_pos 9: result = $result, grism,x,y= [$this cget -grism] $x $y2"
                puts stdout [format \
                               ">>targw(9,x,y,cx,cy)= %8.4f %8.4f %8.4f %8.4f" \
                             $targw(9,x) $targw(9,y) $targw(9,cx) $targw(9,cy)]
                puts stdout [format \
                               ">>targw(9,a,d,ca,cd)= %8.4f %8.4f %8.4f %8.4f" \
                             $targw(9,a) $targw(9,d) $targw(9,ca) $targw(9,cd)]
            } else {
                puts stdout "MOS \#9 positioned"
            }
        } else {
            # puts stdout " MOS 9 not active "
        }

    }
    puts stdout "echs_pos end: [$this cget -grism]"
    # return = grism error
    if { $x == -80.0 } {
        tracer LEAVING 1
        return 1
    } else {
        tracer LEAVING 0
        return 0
    }
}

#============================
private method echs_targ {} {
#============================
    # update one or two echs targw values,
    #                        -----
    # after the mask has been shifted

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if {$echs_flag} {
        set start  9
        set end 10
    } else {
        set start 10
        set end 10
    }

    for {set my $start} {$my <=$end} { incr my 1} {
        # canvas slit center
        set xc [expr ($mos($my,xb)+$mos($my,xc)+$mos($my,xf)+$mos($my,xg) )/4]
        set yc [expr ($mos($my,yb)+$mos($my,yc)+$mos($my,yf)+$mos($my,yg) )/4]

        # canvas -> wcs of MOS 10 or (MOS 10 and MOS 9)
        if { [catch {$m_image convert coords $xc $yc canvas \
                             targw($my,a) targw($my,d) "wcs $EQX"} result] } {
            puts stdout "echs_targ $my: $result"
        }

        # canvas -> pixel of MOS 10 or (MOS 10 and MOS 9)
        catch { $m_image convert coords $xc $yc canvas targw($my,x) targw($my,y) image }
    }

    # now we apply the correction for optical distortion
    # of FORS images and differential refraction
    set corr_flag [$this cget -fors_frame]
    fix_mask_slit echs $corr_flag
    set last_action "Move Mask"
    catch {SP update}
    tracer LEAVING
}

#===========================
public method plot_mask {} {
#===========================
# Draws the whole mask once more with the new settings
# Draw the odd yellow slit blades
# tags are mm, m1, m3, .. m19
# The mos(,) array contains coordinates in canvas coords,
# ready for direct printing no coords transformations
# have to be done
#
# tag A  = all elements
# tag C  = collimator/CCD-window
# tag P2 = white circle, rot+shift, dither 1/2 in IPOL
# tag P3 = grey  circle, rot+shift, dither 1/3 in IPOL
# tag m  = green circle, rotation and shift point
# tag sw = slit width label
# tag out = slit number rim label
# tag D  = display, dont move
# For imaging, each slilet needs a mosa and a mosb tag
# tag mm   = all mos slit blades (ma$i, mb$i, ms$i, m$i)
# tag m$i  = one of the 19 slit blade pairs (ma$i and mb$i sw out)
# tag ma$i = one of the 19 sky-west mosA slit blade only
# tag mb$i = one of the 19 sky-east mosB slit blade only
# tag ms$i = one of the 19 mos slits (invisible, transparent)

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $debug && $verbose } {
        puts stdout "plot_mask: BEGIN"
    }

    set lmod [$this cget -modus]
    set lcol [$this cget -collimat]
    set lcam [$this cget -camera]

    # Used in ipol/img/mos(open)
    # in mos all slits are closed by default (slitrim=9)
    # in img all mos blades are out of the focal field by default (slitrim=1)
    # in ims all mos blades are out of the focal field by default (slitrim=1)
    #
    set slitrim 0
    if { [string compare $lmod ipol] == 0 ||
         [string compare $lmod img]  == 0 ||
         [string compare [$this cget -slit_width] open] == 0 } {
        set slitrim 1
    }

    #-------------------------------------------------
    # plot the 19 MOS slit blades or
    # plot one of the 9 lss slits or
    # plot the 9 ipol stripes or
    # plot one of the 8 hits windows and 2 pinholes or
    # plot one of the 2 hitms windows
    #-------------------------------------------------

    # General tasks
    # plot the focal field rim (blue frame and tag play grounds)
    # plot ccd field (green frame)
    # plot the slits, the contents of the focal field
    # plot the labels (slit labels and focf labels)

    if { $debug && $verbose } {
    }

    # Select resolution parameters
    if { [string match *SR* $lcol] } {
        # Focal field rim width
        # Frame corners
        lassign {a b c d} a b c d
        # Reduced frame corners
        lassign {ar br cr dr} ar br cr dr
        # Label placeholders
        lassign {stl sbl sel swl} tl bl el wl
    } else {
        lassign {e f g h} a b c d
        lassign {er fr gr hr} ar br cr dr
        lassign {htl hbl hel hwl} tl bl el wl
    }

    #---------------------------
    # Plot the focal field (rim)
    #---------------------------
    # Focal field rim always plotted, as a thin line for HR, thicker for SR
    $m_canvas create line $mos(0,xa) $mos(0,ya) $mos(0,xb) $mos(0,yb) \
                          $mos(0,xc) $mos(0,yc) $mos(0,xd) $mos(0,yd) \
                          $mos(0,xa) $mos(0,ya) \
                          -fill blue \
                          -width 1 \
                          -tag { A C }
    $m_canvas create line $mos(0,x$a) $mos(0,y$a) $mos(0,x$b) $mos(0,y$b) \
                          $mos(0,x$c) $mos(0,y$c) $mos(0,x$d) $mos(0,y$d) \
                          $mos(0,x$a) $mos(0,y$a) \
                          -fill blue \
                          -width 3 \
                          -tag { A C }

    # CCD frame
    #- Master section -#
    $m_canvas create line $ccd_frame(0,x$a) $ccd_frame(0,y$a) \
                          $ccd_frame(0,x$b) $ccd_frame(0,y$b) \
                          $ccd_frame(0,x$c) $ccd_frame(0,y$c) \
                          $ccd_frame(0,x$d) $ccd_frame(0,y$d) \
                          $ccd_frame(0,x$a) $ccd_frame(0,y$a) \
                          -fill green \
                          -width 1 \
                          -tag { A C }
    #- Slave section -#
    $m_canvas create line $ccd_frame(1,x$a) $ccd_frame(1,y$a) \
                          $ccd_frame(1,x$b) $ccd_frame(1,y$b) \
                          $ccd_frame(1,x$c) $ccd_frame(1,y$c) \
                          $ccd_frame(1,x$d) $ccd_frame(1,y$d) \
                          $ccd_frame(1,x$a) $ccd_frame(1,y$a) \
                          -fill green \
                          -width 1 \
                          -tag { A C }

    # Reduced frame only in spectroscopic modes (user-defined wavelength region)
    # hitms, hits, lss, mos, mxu, pmos
    # but not for echs, img, ipol
    switch -exact -- $lmod  {
        hitms -
        hits -
        lss -
        mos -
        mxu -
        pmos {
            $m_canvas create line $mos(0,x$ar) $mos(0,y$ar) \
                                  $mos(0,x$dr) $mos(0,y$dr) \
                                  -fill violet \
                                  -width 1 \
                                  -stipple "" \
                                  -tag { A C }
            $m_canvas create line $mos(0,x$br) $mos(0,y$br) \
                                  $mos(0,x$cr) $mos(0,y$cr) \
                                  -fill red \
                                  -width 1 \
                                  -stipple "" \
                                  -tag { A C }
        }
    }

    # Tag play grounds (invisible)
    switch -exact -- $lmod {
        echs {
            # For slit positioning with center/gauss
            # echs_ref_targ is used by reference target verify
            # ali is used by center/slit alignment
            set ltag echs_ref_targ
            # If the center slit option is ON, draw a cyan center cross
            if { $pics } {
                $m_canvas create line [expr {($mos(10,xb) + $mos(10,xf))*0.5}] \
                                      [expr {($mos(10,yb) + $mos(10,yf))*0.5}] \
                                      [expr {($mos(10,xc) + $mos(10,xg))*0.5}] \
                                      [expr {($mos(10,yc) + $mos(10,yg))*0.5}] \
                                      -fill "cyan" \
                                      -smooth false \
                                      -tag { A C }
                $m_canvas create line [expr {($mos(10,xb) + $mos(10,xc))*0.5}] \
                                      [expr {($mos(10,yb) + $mos(10,yc))*0.5}] \
                                      [expr {($mos(10,xf) + $mos(10,xg))*0.5}] \
                                      [expr {($mos(10,yf) + $mos(10,yg))*0.5}] \
                                      -fill "cyan" \
                                      -smooth false \
                                      -tag { A C }
            }
        }
        mxu -
        hitms {
            # For hitms, mxu alignment with center/gauss
            # ali us ised by center/slit alignment
            set ltag mxu
            # No cyan center cross in slit
        }
        hits {
            # For pinhole alignment with center/gauss
            # hits_ref_targ is used by reference target verify
            # ali us ised by center/slit alignment
            set ltag hits_ref_targ
            # No cyan center cross in slit
        }
        lss {
            # For lss alignment with center/gauss
            # lss_ref_trag is used by refernce target verify
            # ali is used by center/slit alignment
            set ltag lss_ref_targ
            # If the center slit option is ON, draw a cyan center cross
            if { $pics } {
                $m_canvas create line \
                     [expr {($mos($long_number,xa) + $mos($long_number,xd))*0.5}] \
                     [expr {($mos($long_number,ya) + $mos($long_number,yd))*0.5}] \
                     [expr {($mos($long_number,xe) + $mos($long_number,xh))*0.5}] \
                     [expr {($mos($long_number,ye) + $mos($long_number,yh))*0.5}] \
                     -fill "cyan" \
                     -smooth false \
                     -tag { A C }
                $m_canvas create line \
                     [expr {($mos($long_number,xa) + $mos($long_number,xe))*0.5}] \
                     [expr {($mos($long_number,ya) + $mos($long_number,ye))*0.5}] \
                     [expr {($mos($long_number,xd) + $mos($long_number,xh))*0.5}] \
                     [expr {($mos($long_number,yd) + $mos($long_number,yh))*0.5}] \
                     -fill "cyan" \
                     -smooth false \
                     -tag { A C }
            }
        }
        default {
            set ltag ""
        }
    }
    $m_canvas create poly $mos(0,x$a) $mos(0,y$a) $mos(0,x$b) $mos(0,y$b) \
                          $mos(0,x$c) $mos(0,y$c) $mos(0,x$d) $mos(0,y$d) \
                          -fill "" \
                          -stipple "" \
                          -smooth false \
                          -tag " A C $ltag "
    # If center slit option is ON, add the 'ali' tag
    if { $pics } {
        if { [string compare $lmod hitms] == 0 ||
             [string compare $lmod mxu]   == 0 } {
            # If we are in standard mask mode
            if { [fors::MXU_SLIT::get_mmff] } {
                $m_canvas addtag ali withtag mxu
                $m_canvas addtag PG  withtag mxu
                $m_canvas dtag mxu
            } else {
                $m_canvas addtag mxu withtag ali
                $m_canvas dtag ali
                $m_canvas dtag PG
            }
        } else {
            $m_canvas addtag ali withtag $ltag
        }
    }
    
    #---------------
    # Plot the SLITS
    #---------------
    switch -exact -- $mask  {
        mxu -
        hitms {
            # Update the color for all slits
            update_color
            fors::MXU_SLIT::set_col [list $color1 $color2 $color3 $color4 $trapa]
            if { $debug && $verbose } {
                puts stdout "plot_mask: "
                puts stdout "           Color 1: $color1"
                puts stdout "           Color 2: $color2"
                puts stdout "           Color 3: $color3"
                puts stdout "           Color 4: $color4"
            }
            # Number of mxu slits
            set noms [fors::MXU_SLIT::nos]
            if { $noms > 0 } {
                for { set i 1 } { $i <= $noms } { incr i 1 } {
                    C_mxus$i update_slit
                }
            }
        }
        hits {
            # The current ht-slit
            # The 0.3'' and 0.4'' slit is not supported
            switch -exact -- $long_number {
                3 -
                4 -
                5 -
                6 -
                7 -
                8 {
                    $m_canvas create polygon $mos($long_number,xa) \
                                             $mos($long_number,ya) \
                                             $mos($long_number,xd) \
                                             $mos($long_number,yd) \
                                             $mos($long_number,xh) \
                                             $mos($long_number,yh) \
                                             $mos($long_number,xe) \
                                             $mos($long_number,ye) \
                                             -outline $color4 \
                                             -fill "" \
                                             -stipple "" \
                                             -tag A
                  }
                1 {
                    # Draw the pinhole
                    $m_canvas create polygon $mos(9,xa) $mos(9,ya) \
                                             $mos(9,xd) $mos(9,yd) \
                                             $mos(9,xh) $mos(9,yh) \
                                             $mos(9,xe) $mos(9,ye) \
                                             -outline $color4 \
                                             -fill "" \
                                             -stipple "" \
                                             -tag A
                    # tcltk too stupid, cannot draw oval with angle
                    set x [expr {$mos(9,xa) - $mos(9,xh)}]
                    set y [expr {$mos(9,ya) - $mos(9,yh)}]
                    set r [expr {sqrt($x*$x + $y*$y)/2.0/sqrt(2.0)}]
                    set x [expr {($mos(9,xa) + $mos(9,xh))/2.0}]
                    set y [expr {($mos(9,ya) + $mos(9,yh))/2.0}]
                    $m_canvas create oval [expr {$x - $r}] [expr {$y - $r}] \
                                          [expr {$x + $r}] [expr {$y + $r}] \
                                          -fill "" \
                                          -stipple "" \
                                          -outline $color4 \
                                          -tag A
                  }
                0 {
                    # long_number ==0
                    # 'show-all 7 slits'  modus
                    for { set i 3 } { $i < 10 } { incr i 1 } {
                         $m_canvas create polygon $mos($i,xa) $mos($i,ya) \
                                                  $mos($i,xd) $mos($i,yd) \
                                                  $mos($i,xh) $mos($i,yh) \
                                                  $mos($i,xe) $mos($i,ye) \
                                                  -outline $color4 \
                                                  -fill "" \
                                                  -stipple "" \
                                                  -tag A
                    }
                    # Draw the oval pinhole
                    # tcltk too stupid, cannot draw oval with angle
                    set x [expr {$mos(9,xa) - $mos(9,xh)}]
                    set y [expr {$mos(9,ya) - $mos(9,yh)}]
                    set r [expr {sqrt($x*$x + $y*$y)/2.0/sqrt(2.0)}]
                    set x [expr {($mos(9,xa) + $mos(9,xh))/2.0}]
                    set y [expr {($mos(9,ya) + $mos(9,yh))/2.0}]
                    $m_canvas create oval [expr {$x - $r}] [expr {$y - $r}] \
                                          [expr {$x + $r}] [expr {$y + $r}] \
                                          -fill "" \
                                          -stipple "" \
                                          -outline $color4 \
                                          -tag A
                }
            }
        }
        lss {
            # The current long-slit
            if { $long_number > 0 } {
                $m_canvas create polygon $mos($long_number,xa) \
                                         $mos($long_number,ya) \
                                         $mos($long_number,xd) \
                                         $mos($long_number,yd) \
                                         $mos($long_number,xh) \
                                         $mos($long_number,yh) \
                                         $mos($long_number,xe) \
                                         $mos($long_number,ye) \
                                         -outline $color4 \
                                         -fill "" \
                                         -stipple "" \
                                         -tag A
            } else {
                for { set i 1 } { $i < 10 } { incr i 1 } {
                     $m_canvas create polygon $mos($i,xa) $mos($i,ya) \
                                              $mos($i,xd) $mos($i,yd) \
                                              $mos($i,xh) $mos($i,yh) \
                                              $mos($i,xe) $mos($i,ye) \
                                              -outline $color4 \
                                              -fill "" \
                                              -stipple "" \
                                              -tag A
                }
            }
        }
        mos {
            # mos and pmos, img, ipol
            # Default slit blade number mos
            set start 1
            set end 19
            set bb mm
            if { [string compare $lmod "echs"] == 0 } {
                set bb ee; # to avoid MOS/IMG bindings in ECHS
                if { $echs_flag } {
                    set start 9
                    set end 9
                } else {
                    set start 0
                    set end -1
                }
            }

            # ODD
            for { set i $start } { $i <= $end } { incr i 2 } {
                # Odd blades
	        # Plot modified slit blades shaded
                # In IPOL plot odd slit blades shaded
	        if { [$this cget -graytype] } {
                    if { [string compare $targw($i,f) "0"] == 0 } {
                        set color1 ""
                        set color2 ""
                    } else {
                        set trapa gray25
                        set color1 $color3
                        set color2 $color4
                    }
                    if { [string match ipol $lmod] ||
                         [string match pmos $lmod] } {
                        set trapa gray25
                        set color1 $color3
                        set color2 $color4
                    }
                }
                $m_canvas create polygon $mos($i,xa) $mos($i,ya) $mos($i,xb) $mos($i,yb) \
                                         $mos($i,xf) $mos($i,yf) $mos($i,xe) $mos($i,ye) \
                                         -outline $color3 \
                                         -fill $color1 \
                                         -stipple $trapa \
                                         -tag " A $bb m$i ma$i "
                $m_canvas create polygon $mos($i,xd) $mos($i,yd) $mos($i,xc) $mos($i,yc) \
                                         $mos($i,xg) $mos($i,yg) $mos($i,xh) $mos($i,yh) \
                                         -outline $color3 \
                                         -fill $color1 \
                                         -stipple $trapa \
                                         -tag " A $bb m$i mb$i "

                # Create invisible objects, the slits. They can be called by
                # their tag to open and close slit blades in imaging mode
                if { $slitrim } {
                    $m_canvas create polygon $mos($i,xb) $mos($i,yb) \
                                             $mos($i,xc) $mos($i,yc) \
                                             $mos($i,xg) $mos($i,yg) \
                                             $mos($i,xf) $mos($i,yf) \
                                             -outline "" \
                                             -fill "" \
                                             -stipple "" \
                                             -tag " A $bb m$i ms$i "
                }
            }
            
            # Draw the even $color2 slit blades
            # tags are mm, m2, .. m18
            # Default slit blade numbers for mos
            set start 2
            set end 18
            if { [string compare $lmod "echs"] == 0 } {
                set start 10
                set end 10
            }

            # EVEN
            for { set i $start } { $i <= $end } { incr i 2 } {
                if { [$this cget -graytype] } {
                    if { [string compare $targw($i,f) "0"] == 0 } {
                        set color1 ""
                        set color2 ""
                    } else {
                        set trapa gray25
                        set color1 $color3
                        set color2 $color4
                    }
                }
                $m_canvas create polygon $mos($i,xa) $mos($i,ya) $mos($i,xb) \
                                         $mos($i,yb) $mos($i,xf) $mos($i,yf) \
                                         $mos($i,xe) $mos($i,ye) \
                                         -outline $color4 \
                                         -fill $color2 \
                                         -stipple $trapa \
                                         -tag " A $bb m$i ma$i "
                $m_canvas create polygon $mos($i,xd) $mos($i,yd) $mos($i,xc) \
                                         $mos($i,yc) $mos($i,xg) $mos($i,yg) \
                                         $mos($i,xh) $mos($i,yh) \
                                         -outline $color4 \
                                         -fill $color2 \
                                         -stipple $trapa \
                                         -tag " A $bb m$i mb$i "
                if { $slitrim } {
                    $m_canvas create polygon $mos($i,xb) $mos($i,yb) \
                                             $mos($i,xc) $mos($i,yc) \
                                             $mos($i,xg) $mos($i,yg) \
                                             $mos($i,xf) $mos($i,yf) \
                                             -outline "" \
                                             -fill "" \
                                             -stipple "" \
                                             -tag " A $bb m$i ms$i "
                }
            }
        }
        pol {
            # Polarimetric imaging mask for high res
            # First fill the mask, not the stripes
            $m_canvas create polygon $mos(0,xa) $mos(0,ya) $mos(0,xb) \
                                     $mos(0,yb) $mos(0,xc) $mos(0,yc) \
                                     $mos(0,xd) $mos(0,yd) $mos(9,xe) \
                                     $mos(9,ye) $mos(9,xh) $mos(9,yh) \
                                     $mos(1,xd) $mos(1,yd) $mos(1,xa) \
                                     $mos(1,ya) $mos(9,xe) $mos(9,ye) \
                                     $mos(0,xd) $mos(0,yd) $mos(0,xa) \
                                     $mos(0,ya) \
                                     -outline "" \
                                     -fill $color1 \
                                     -stipple $trapa \
                                     -width 1 \
                                     -tag A
            for { set i 1 } { $i < 9 } { incr i 1 } {
                set j [expr {$i + 1}]
                $m_canvas create polygon $mos($i,xe) $mos($i,ye) $mos($i,xh) \
                                         $mos($i,yh) $mos($j,xd) $mos($j,yd) \
                                         $mos($j,xa) $mos($j,ya) $mos($i,xe) \
                                         $mos($i,ye) \
                                         -outline "" \
                                         -fill $color1 \
                                         -stipple $trapa \
                                         -width 1 \
                                         -tag A
            }
            # And the open stripes
            for { set i 1 } { $i < 10 } { incr i 1 } {
                $m_canvas create polygon $mos($i,xa) $mos($i,ya) $mos($i,xd) \
                                         $mos($i,yd) $mos($i,xh) $mos($i,yh) \
                                         $mos($i,xe) $mos($i,ye) \
                                         -outline $color4 \
                                         -fill ""  \
                                         -stipple ""  \
                                         -tag A
            }
        }
        default {
            puts stdout "plot_mask: Warning, unknown mask >$mask<"
        }
    }

    #----------------------
    # Draw the green circle
    #----------------------
    # Showing the ccx ccy position = rotation axis
    # tag = m, m10
    # puts stdout "plot_mask: green point = \
    #              [expr $mos(0,x0)-$mos(0,r)]"
    $m_canvas create oval [expr {$mos(0,x0) - $mos(0,r)}] [expr {$mos(0,y0) - $mos(0,r)}] \
                          [expr {$mos(0,x0) + $mos(0,r)}] [expr {$mos(0,y0) + $mos(0,r)}] \
                          -fill green \
                          -stipple $trapa \
                          -tag { A m m10 }
    $m_canvas create oval [expr {$mos(0,x0) - $mos(0,r)/20}] [expr {$mos(0,y0) - $mos(0,r)/20}] \
                          [expr {$mos(0,x0) + $mos(0,r)/20}] [expr {$mos(0,y0) + $mos(0,r)/20}] \
                          -fill "" \
                          -stipple "" \
                          -tag { A m m10 }


    #----------------
    # Plot the LABELS
    #----------------
    if { [$this cget -label] } {
        # Select labels according to camera
        switch $lcam {
            B {
                set cam " Blue "
            }
            R {
                set cam " Red "
            }
            default {
                set cam " "
            }
        }

        #--------------------
        # 1.0 CCD chip labels
        #--------------------
        #- Master label -#
        $m_canvas create text $ccd_frame(0,x$tl) \
                              $ccd_frame(0,y$tl) \
                              -anchor c \
                              -fill green \
                              -text "$cam CCD Master" \
                              -tag { A C m }
        #- Slave label -#
        $m_canvas create text $ccd_frame(1,x$bl) \
                              $ccd_frame(1,y$bl) \
                              -anchor c \
                              -fill green \
                              -text "$cam CCD Slave" \
                              -tag { A C m }

        #----------------------------------
        # 2.0 FocalField Orientation labels
        #----------------------------------
        # Label orientation: which side of the text is anchored
        # for labels near blade A, blade B, slit 1, and slit 19
        # depending on the focal field orientation.
        if { $fits_x_scale ==  1.0 } {
            set ori1 n
            set ori19 s
            set oriA w
            set oriB e
        }
        if { $fits_x_scale == -1.0 } {
            set ori1 s
            set ori19 n
            set oriA e
            set oriB w
        }
        #>> set oriA e
        #>> set oriB w
        # Text for wavecolor and direction labels
        set col1 ""
        set col19 ""
        set colA ""
        set colB ""
        if { $disp_ang == 0.0 } {
            set colA Blue
            set colB Red
        } elseif { $disp_ang == 180.0 || $disp_ang == -180.0 } {
            set colA Red
            set colB Blue
        } elseif { $disp_ang < 0.0 } {
            set col1 Blue
            set col19 Red
        } else {
            set col1 Red
            set col19 Blue
        }
        set dirA W
        set dirB E
        set dir1 S
        set dir19 N
        if { $mountangle == 180 } {
            # Swap the wavecolor and direction labels
            # 20111117:
            # This was wrong: If the whole instrument is rotated 180 deg in the
            # adaptor, color labels stick to blade labels, but
            # direction labels swap.
            # set dummy $col1
            # set col1 $col19
            # set col19 $dummy
            # set dummy $colA
            # set colA $colB
            # set colB $dummy
            set dummy $dirA
            set dirA $dirB
            set dirB $dummy
            set dummy $dir1
            set dir1 $dir19
            set dir19 $dummy
        }

        catch { $m_image convert coords $world_x $world_y "deg $EQX" lwx lwy "wcs $EQX" }

        if { $debug && $verbose } {
	    puts stdout         "plot_mask: FITS scale X = $fits_x_scale"
	    puts stdout         "           FITS scale Y = $fits_y_scale"
	    puts stdout         "           Mountangle   = $mountangle \[deg\]"
            puts stdout         ""
            puts stdout         "               Label Orientation"
            puts stdout         "                  $ori1 $dir1  1 $col1"
            puts stdout         ""
            puts stdout [format "           %1s %1s A %4s  +  %1s %1s B %4s" $oriA $dirA $colA $oriB $dirB $colB]
            puts stdout         ""
            puts stdout         "                  $ori19 $dir19 19 $col19"
            puts stdout         ""
        }

        $m_canvas create text $mos(0,x$tl) \
                              $mos(0,y$tl) \
                              -anchor c \
                              -fill $color4 \
                              -text "$dir1  1  $col1" \
                              -tag { A C m }
        $m_canvas create text $mos(0,x$bl) \
                              $mos(0,y$bl) \
                              -anchor c \
                              -fill $color4 \
                              -text "$dir19  19  $col19" \
                              -tag { A C m }
        $m_canvas create text $mos(0,x$el) \
                              $mos(0,y$el) \
                              -anchor c \
                              -fill $color4 \
                              -text "$dirA  A  $colA" \
                              -tag { A C m }
        $m_canvas create text $mos(0,x$wl) \
                              $mos(0,y$wl) \
                              -anchor c \
                              -fill $color4 \
                              -text "$dirB  B  $colB" \
                              -tag { A C m }

        #-------------------------------------------
        # 3.0 TCS and INS labels (upper left corner)
        #-----------------------------------------1-
        update_labels

        #----------
        # 4.0 Modes
        #----------
        # Some text to help orientation
        switch -exact -- $mask {
            hits {
                #--------------
                # 4.1 HITS-mask
                #--------------
                # 4.1.1 cyan move button
                switch -exact -- $long_number {
                    3 -
                    4 -
                    5 -
                    6 -
                    7 -
                    8 {
                        # 4.1.2 extra move button
                        $m_canvas create oval [expr $mos($long_number,xe)-$mos(0,r)/2.0] \
                                              [expr $mos($long_number,ye)-$mos(0,r)/2.0] \
                                              [expr $mos($long_number,xe)+$mos(0,r)/2.0] \
                                              [expr $mos($long_number,ye)+$mos(0,r)/2.0] \
                                              -fill cyan \
                                              -tag {A m}
                        # 4.1.3 slit label
                        $m_canvas create text $mos($long_number,xh) \
                                              $mos($long_number,yh) \
                                              -anchor $oriB \
                                              -fill $color4 \
                                              -text "  [$this cget -long_width]''  "  \
                                              -tag " A "
                    }
                    1 {
                        $m_canvas create text $mos(9,xa) $mos(9,ya) \
                                              -anchor $oriA \
                                              -fill $color4 \
                                              -text "----(big pinhole)----"  \
                                              -tag " A "
                    }
                }

                # 4.1.4 wavelength range labels
                if { [string match *HR* $lcol] } {
                    $m_canvas create text $ccd_frame(0,xhr) \
                                          $ccd_frame(0,yhr) \
                                          -anchor $oriB \
                                          -fill $color4 \
                                          -text [format "%4.0fA" \
                                                       [expr $L_MAX*10.0] \
                                                ] \
                                          -tag " A"
                    $m_canvas create text $ccd_frame(0,xgr) \
                                          $ccd_frame(0,ygr) \
                                          -anchor $oriA \
                                          -fill $color4 \
                                          -text [format "%4.0fA" \
                                                        [expr $L_MIN*10.0] \
                                                ] \
                                          -tag " A"
                } else {
                    $m_canvas create text $mos(0,xdr) $mos(0,ydr) \
                                          -anchor $oriB \
                                          -fill $color4 \
                                          -text [format "%4.0fA" \
                                                        [expr $L_MAX*10.0] \
                                                ] \
                                          -tag " A"
                    $m_canvas create text $mos(0,xcr) $mos(0,ycr) \
                                          -anchor $oriA \
                                          -fill $color4 \
                                          -text [format "%4.0fA" \
                                                        [expr $L_MIN*10.0] \
                                                ] \
                                          -tag " A"
                }
            }
            lss {
                #-------------
                # 4.2 LSS-mask
                #-------------
                # 4.2.1 slit label
                $m_canvas create text $mos($long_number,xh) \
                                      $mos($long_number,yh) \
                                      -anchor $oriB \
                                      -fill $color4 \
                                      -text "  [$this cget -long_width]''  "  \
                                      -tag " A "

                # 4.2.2 extra move-button
                # $m_canvas create oval [expr $mos($long_number,xe) - $mos(0,r)/2.0] \
                                      # [expr $mos($long_number,ye) - $mos(0,r)/2.0] \
                                      # [expr $mos($long_number,xe) + $mos(0,r)/2.0] \
                                      # [expr $mos($long_number,ye) + $mos(0,r)/2.0] \
                                      # -fill cyan \
                                      # -tag {A m}
                # $m_canvas create oval [expr $mos(1,xa)-0.1*$mos(0,r)] \
                                      # [expr $mos(1,ya)-0.1*$mos(0,r)] \
                                      # [expr $mos(1,xa)+0.1*$mos(0,r)] \
                                      # [expr $mos(1,ya)+0.1*$mos(0,r)] \
                                      # -outline red \
                                      # -tag A
                # $m_canvas create oval [expr $mos(1,xd)-0.1*$mos(0,r)] \
                                      # [expr $mos(1,yd)-0.1*$mos(0,r)] \
                                      # [expr $mos(1,xd)+0.1*$mos(0,r)] \
                                      # [expr $mos(1,yd)+0.1*$mos(0,r)] \
                                      # -outline yellow \
                                      # -tag A

                # 4.2.3 wavelength range labels
                if { [string match *HR* $lcol] } {
                    $m_canvas create text $ccd_frame(0,xgr) \
                                          $ccd_frame(0,ygr) \
                                          -anchor $oriA \
                                          -fill $color4 \
                                          -text [format "%4.0fA" \
                                                        [expr $L_MIN*10.0] \
                                                ] \
                                          -tag " A"
                    $m_canvas create text $ccd_frame(0,xhr) \
                                          $ccd_frame(0,yhr) \
                                          -anchor $oriB \
                                          -fill $color4 \
                                          -text [format "%4.0fA" \
                                                        [expr $L_MAX*10.0] \
                                                ] \
                                          -tag " A"
                }
            }
            mos {
                #----------------------------------
                # 4.3 MOS-mask  (mos,pmos,img,ipol)
                #----------------------------------
                # 4.3.1 cyan circle
                # These are only for the progammer for diagnostic purposes
                # mos blade 10 A (center west)
                # set dum_x [expr ($mos(10,xa)+$mos(10,xb) + $mos(10,xe) + $mos(10,xf))/4.0]
                # set dum_y [expr ($mos(10,ya)+$mos(10,yb) + $mos(10,ye) + $mos(10,yf))/4.0]
                # $m_canvas create oval [expr $dum_x + $mos(0,r)] \
                                      # [expr $dum_y + $mos(0,r)] \
                                      # -fill cyan \
                                      # -tag A
                # $m_canvas create oval [expr $mos(1,xa)-2.05*$mos(0,r)] \
                                      # [expr $mos(1,ya)-2.05*$mos(0,r)] \
                                      # [expr $mos(1,xa)+2.05*$mos(0,r)] \
                                      # [expr $mos(1,ya)+2.05*$mos(0,r)] \
                                      # -outline red \
                                      # -tag A
                # $m_canvas create oval [expr $mos(1,xb)-0.05*$mos(0,r)] \
                                      # [expr $mos(1,yb)-0.05*$mos(0,r)] \
                                      # [expr $mos(1,xb)+0.05*$mos(0,r)] \
                                      # [expr $mos(1,yb)+0.05*$mos(0,r)] \
                                      # -outline yellow \
                                      # -tag A

                # 4.3.2 blade labels
                # ODD
                set start 1
                set end 19
                if { [string compare $lmod "echs"] == 0 } {
                    if { $echs_flag } {
                        set start 9
                        set end 9
                    } else {
                        set start 0
                        set end -1
                    }
                }

                for { set i $start } { $i <= $end } { incr i 2 } {
                    # slit labels at the target positions
                    # puts stdout "$i $targw($i,f)"
                    # puts stdout "> [format " %2.1f'' " $mos($i,b)] <"
                     if { $targw($i,f) != 0 } {
                         # Slit number label
                         $m_canvas create text [expr ($mos($i,xb) + $mos($i,xf))/2] \
                                               [expr ($mos($i,yb) + $mos($i,yf))/2] \
                                               -anchor $oriA \
                                               -fill $color4 \
                                               -text " #$i " \
                                               -tag "A lb m$i"
                         # Slit width label
                         $m_canvas create text [expr ($mos($i,xc) + $mos($i,xg))/2] \
                                               [expr ($mos($i,yc) + $mos($i,yg))/2] \
                                                     -anchor $oriB \
                                                     -fill $color4 \
                                                     -text [format " %2.1f'' " $mos($i,b)] \
                                                     -tag "A sw m$i"
                     }
                     # At both rims
                     # we give these labels the individual slit tag, since
                     # it could be that the labels are in the slit itself
                     # East drive A
                     $m_canvas create text [expr ($mos($i,xa) + $mos($i,xe))/2] \
                                           [expr ($mos($i,ya) + $mos($i,ye))/2] \
                                           -anchor $oriA \
                                           -fill $color4 \
                                           -text "#$i" \
                                           -tag "A out m$i ma$i"
                     # West drive B
                     $m_canvas create text [expr ($mos($i,xd) + $mos($i,xh))/2] \
                                           [expr ($mos($i,yd) + $mos($i,yh))/2] \
                                           -anchor $oriB  \
                                           -fill $color4 \
                                           -text "#$i" \
                                           -tag "A out m$i mb$i"
                     # Make the mos slits visible
                     if { [string match *mos $lmod] ||
                          [string match echs $lmod] } {
                           $m_canvas create polygon $mos($i,xb) $mos($i,yb) \
                                                    $mos($i,xc) $mos($i,yc) \
                                                    $mos($i,xg) $mos($i,yg) \
                                                    $mos($i,xf) $mos($i,yf) \
                                                    -outline $color4 \
                                                    -fill "" \
                                                    -tag "A $bb m$i ms$i "
                     }
                }

                # EVEN
                set start 2
                set end 18
                if { [string compare $lmod "echs"] == 0 } {
                    set start 10
                    set end 10
                }

                for { set i $start } { $i <= $end } { incr i 2 } {
                    # slit labels
                    # at the target positions (only if have been set)
                    # puts stdout "$i $targw($i,f)"
                    if { $targw($i,f) != 0 } {
                        # Slit number label
                        $m_canvas create text [expr ($mos($i,xb) + $mos($i,xf))/2] \
                                              [expr ($mos($i,yb) + $mos($i,yf))/2] \
                                              -anchor $oriA \
                                              -fill $color3 \
                                              -text " #$i " \
                                              -tag "A lb m$i"
                        # Slit width label
                        $m_canvas create text [expr ($mos($i,xc) + $mos($i,xg))/2] \
                                              [expr ($mos($i,yc) + $mos($i,yg))/2] \
                                              -anchor $oriB \
                                              -fill $color3 \
                                              -text [format " %2.1f'' " $mos($i,b)] \
                                              -tag "A sw m$i"
                     }
                     # At the both frame rims
                     # East drive A
                     $m_canvas create text [expr ($mos($i,xa) + $mos($i,xe))/2] \
                                           [expr ($mos($i,ya) + $mos($i,ye))/2] \
                                           -anchor $oriA \
                                           -fill $color3 \
                                           -text "#$i" \
                                           -tag " A out m$i ma$i"
                     # West drive B
                     $m_canvas create text [expr ($mos($i,xd) + $mos($i,xh))/2] \
                                           [expr ($mos($i,yd) + $mos($i,yh))/2] \
                                           -anchor $oriB \
                                           -fill $color3 \
                                           -text "#$i" \
                                           -tag " A out m$i mb$i"
                     # Make the mos slits visible
                     if { [string match *mos $lmod] ||
                          [string match echs $lmod]} {
                         $m_canvas create polygon $mos($i,xb) $mos($i,yb) \
                                                  $mos($i,xc) $mos($i,yc) \
                                                  $mos($i,xg) $mos($i,yg) \
                                                  $mos($i,xf) $mos($i,yf) \
                                                  -outline $color3 \
                                                  -fill "" \
                                                  -tag " A $bb m$i ms$i "
                     }
                     if { [string match echs $lmod] } {
                         $m_canvas create oval [expr $mos(10,xg) - $mos(0,r)/2] \
                                               [expr $mos(10,yg) - $mos(0,r)/2] \
                                               [expr $mos(10,xg) + $mos(0,r)/2] \
                                               [expr $mos(10,yg) + $mos(0,r)/2] \
                                               -fill cyan \
                                               -stipple $trapa \
                                               -tag { A m PE }
                     }
                }
                # 4.3.3 wavelength range labels
                # indicate wavelength range only for mos, pmos
                # not for IMG and IPOL-SR (they also have mask = mos)
                # not for ECHS (they also have mask = mos)
                if { [string match "*mos" $lmod] } {
                    plot_mos_dispersion_label $oriA $oriB $ori1 $ori19
                }
            }
            hitms -
            mxu {
                #-------------
                # 4.4 MXU-mask
                #-------------
                plot_mos_dispersion_label $oriA $oriB $ori1 $ori19

                # Number of mxu slits
                if { $noms > 0 } {
                    if { [fors::MXU_SLIT::get_mmff] } {
                        # If technical mask mode
                        # draw a cyan move circle at the first of many
                        # technical slits

                        # Get x y width length in mm focal plane
                        set first_slit [C_mxus1 get_slit]
                        # first_slit 0,1 = x,y center of slit

                        # Draw cyan circle 3 times a slit width 
                        # away from the slit center
                        # Take the smallest (either width or length)
                        # Note: the radius of the green move-circle is about 8'' or ~4mm
                        # The cyan circle should be at least 16'' or 8 mm away
                        # from the green circle
                        if { [lindex $first_slit 2] < [lindex $first_slit 3] } {
                            set shift [lindex $first_slit 2]
                        } else {
                            set shift [lindex $first_slit 3]
                        }
                        if { $shift < 4 } {
                            set shift 4
                        }

                        set px [expr [lindex $first_slit 0]+$shift*3.0]
                        set py [expr [lindex $first_slit 1]-$shift*3.0]

                        # Slit center in canvas to plot
                        set a [C_mc m2c $px $py]
                        set loc_rad [expr $mos(0,r)/2.0]

                        $m_canvas create oval [expr [lindex $a 0] - $loc_rad] \
                                              [expr [lindex $a 1] - $loc_rad] \
                                              [expr [lindex $a 0] + $loc_rad] \
                                              [expr [lindex $a 1] + $loc_rad] \
                                              -fill cyan \
                                              -tag {A m}
                    }
                }
                # puts stdout "plot_mask: mxu label ok"
            }
            pol {
                #-------------
                # 4.5 POL-mask
                #-------------
                # $m_canvas create oval [expr {$mos(1,xa) - $mos(0,r)}] [expr {$mos(1,ya) - $mos(0,r)}] \
                #                       [expr {$mos(1,xa) + $mos(0,r)}] [expr {$mos(1,ya) + $mos(0,r)}] \
                #                       -outline red \
                #                       -tag A
                # $m_canvas create oval [expr {$mos(1,xd) - $mos(0,r)}] [expr {$mos(1,yd) - $mos(0,r)}] \
                #                       [expr {$mos(1,xd) + $mos(0,r)}] [expr {$mos(1,yd) + $mos(0,r)}] \
                #                       -outline yellow \
                #                       -tag A
            }
        }


        #--------------
        # 5.0 all masks
        #--------------
        if { $mountangle == 180 } {
            # The white circle for (x>0,y>0 = NE) in focal field coord system
            $m_canvas create oval [expr $mos(0,x$a)-$mos(0,r)] \
                                  [expr $mos(0,y$a)-$mos(0,r)] \
                                  [expr $mos(0,x$a)+$mos(0,r)] \
                                  [expr $mos(0,y$a)+$mos(0,r)] \
                                  -fill white \
                                  -tag { A P2 m }
            # Gray circle for CCD reference pixel = (1,1 = SE)
            $m_canvas create oval [expr $mos(0,x$d)-$mos(0,r)] \
                                  [expr $mos(0,y$d)-$mos(0,r)] \
                                  [expr $mos(0,x$d)+$mos(0,r)] \
                                  [expr $mos(0,y$d)+$mos(0,r)] \
                                  -fill gray \
                                  -tag { A P3 m }
	    if { [string compare ipol [$this cget -modus]] } {
                # Its not IPOL
	        $m_canvas dtag P2
	        $m_canvas dtag P3
	    }
        } else {
            # The white circle for (x>0,y>0 = NE) in focal field coord system
            $m_canvas create oval [expr $mos(0,x$c)-$mos(0,r)] \
                                  [expr $mos(0,y$c)-$mos(0,r)] \
                                  [expr $mos(0,x$c)+$mos(0,r)] \
                                  [expr $mos(0,y$c)+$mos(0,r)] \
                                  -fill white \
                                  -tag { A P2 m }
            # Gray circle for CCD reference pixel = (1,1 = SE)
            $m_canvas create oval [expr $mos(0,x$b)-$mos(0,r)] \
                                  [expr $mos(0,y$b)-$mos(0,r)] \
                                  [expr $mos(0,x$b)+$mos(0,r)] \
                                  [expr $mos(0,y$b)+$mos(0,r)] \
                                  -fill gray \
                                  -tag { A P3 m }
        }
    }

    if { $reftarg_zone } {
        if { [C_ref_targ cget -mos10ali] == 0 } {
            plot_ref_star_area 0
        } else {
            plot_ref_star_area 1
        }
    }

    # AAG - PPRS 10127
    ::fors::FORS::my_mask edit_ref_targ plot

    if { $debug && $verbose } {
        puts stdout "plot_mask: END"
    }
    tracer LEAVING
    return 0
}

#=================================
private method plot_tcs_label {} {
#=================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Extra coords and PA in the upper left corner
    catch { $m_image convert coords $world_x $world_y "deg $EQX" lwx lwy "wcs $EQX" }
    catch { $m_image convert coords 10 20 screen px20 py20 canvas }
    $m_canvas create text $px20 $py20 -fill $color4 \
                                      -text "ROT = $tcs_pam" \
                                      -anchor w \
                                      -tag {D}
    $m_canvas create text $px20 [expr $py20 +20] -fill $color4 \
                                                 -text "  RA = $lwx" \
                                                 -anchor w \
                                                 -tag {D}
    $m_canvas create text $px20 [expr $py20 +40] -fill $color4 \
                                                 -text "DEC = $lwy" \
                                                 -anchor w \
                                                 -tag {D}
    if { $pics } {
        $m_canvas create text $px20 [expr $py20 +60] -fill $color4 \
                                                     -text ">CENTER<" \
                                                     -anchor w \
                                                     -tag {D}
    }
    tracer LEAVING
}

#=================================
private method plot_ins_label {} {
#=================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Instrument optical setup at the top of the screen
    catch { $m_image convert coords 200 20 screen px200 py200 canvas }
    if { [string match *SR* [$this cget -collimat]] } {
        set resol SR
    } else {
        set resol HR
    }
    $m_canvas create text $px200 $py200 \
                          -fill $color4 \
                          -text "MODE = [string toupper [$this cget -modus]]  FORS2  $resol" \
                          -anchor w \
                          -tag {D}
    $m_canvas create text $px200 [expr $py200 +20] \
                          -fill $color4 \
                          -text "   FILT = [$this cget -filter]" \
                          -anchor w \
                          -tag {D}
    $m_canvas create text $px200 [expr $py200 +40] \
                          -fill $color4 \
                          -text "  GRIS = [$this cget -grism]" \
                          -anchor w \
                          -tag {D}
    tracer LEAVING
}

#=================================================================
private method plot_mos_dispersion_label { oriA oriB ori1 ori19} {
#=================================================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Select slots according to resolution
    if { [string match *SR* [$this cget -collimat]] } {
        lassign {a b c d} a b c d
        lassign {ar br cr dr} ar br cr dr
        lassign {sb1s sb1f sb2s sb2f sr1s sr1f sr2s sr2f} b1s b1f b2s b2f r1s r1f r2s r2f
        lassign {slbl slrl} lbl lrl
    } else {
        lassign {e f g h} a b c d
        lassign {er fr gr hr} ar br cr dr
        lassign {hb1s hb1f hb2s hb2f hr1s hr1f hr2s hr2f} b1s b1f b2s b2f r1s r1f r2s r2f
        lassign {hlbl hlrl} lbl lrl
    }

    # Draw the big red and blue crosses.
    $m_canvas create line $mos(0,x$b1s) $mos(0,y$b1s) \
                          $mos(0,x$b1f) $mos(0,y$b1f) \
                          -fill violet \
                          -width 1 \
                          -stipple "" \
                          -tag {A C}
    $m_canvas create line $mos(0,x$b2s) $mos(0,y$b2s) \
                          $mos(0,x$b2f) $mos(0,y$b2f) \
                          -fill violet \
                          -width 1 \
                          -stipple "" \
                          -tag {A C}
    $m_canvas create line $mos(0,x$r1s) $mos(0,y$r1s) \
                          $mos(0,x$r1f) $mos(0,y$r1f) \
                          -fill red \
                          -width 1 \
                          -stipple "" \
                          -tag {A C}
    $m_canvas create line $mos(0,x$r2s) $mos(0,y$r2s) \
                          $mos(0,x$r2f) $mos(0,y$r2f) \
                          -fill red \
                          -width 1 \
                          -stipple "" \
                          -tag {A C}

    # Draw the wavelength labels (A)
    $m_canvas create text $mos(0,x$lbl) $mos(0,y$lbl) \
                          -anchor $ori19 \
                          -fill violet \
                          -text [format "%4.0fA " [expr {$L_MIN*10.0}]] \
                          -tag " A"
    $m_canvas create text $mos(0,x$lrl) $mos(0,y$lrl) \
                          -anchor $ori19 \
                          -fill red \
                          -text [format " %4.0fA" [expr {$L_MAX*10.0}]] \
                          -tag " A"
    $m_canvas create text $mos(0,x$ar) $mos(0,y$ar) \
                          -anchor $ori1 \
                          -fill yellow \
                          -text [format "%4.0fA" [expr {$A_disp($good,cen)*10.0}]] \
                          -tag " A"
    $m_canvas create text $mos(0,x$br) $mos(0,y$br) \
                          -anchor $ori1 \
                          -fill yellow \
                          -text [format "%4.0fA" [expr {$A_disp($good,cen)*10.0}]] \
                          -tag " A"
    tracer LEAVING
}

#============================
private method plot_BPM { } {
#============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # plot BPM
    $m_canvas delete -tag BPM
    for { set i 1 } { $i <= $BPM_counter } { incr i 1 } {
         $m_canvas create poly $BPM($i,x1) $BPM($i,y1) \
                               $BPM($i,x2) $BPM($i,y2) \
                               $BPM($i,x3) $BPM($i,y3) \
                               $BPM($i,x4) $BPM($i,y4) \
                               $BPM($i,x1) $BPM($i,y1) \
                               -outline white \
                               -width 1 \
                               -fill "" \
                               -smooth false \
                               -stipple "" \
                               -tag { A BPM }
                               # tag A to move with the mask
    }
    tracer LEAVING
}

#==========================================
private method plot_ref_star_area_ccd2 {} {
#==========================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Select resolution parameters (mm)
    set resol [$this cget -collimat]
    if { [string match *SR* $resol] } {
        set fa [expr {10.0/396.0}]
        # Corner slots of the Frame Arrays
        lassign {a b c d} a b c d
        if { $debug && $verbose } {
            puts stdout "plot_ref_star_area_ccd2: Standard Resolution"
        }
    } else {
	set fa [expr {10.0/198.0}]
        lassign {e f g h} a b c d
        if { $debug && $verbose } {
            puts stdout "plot_ref_star_area_ccd2: High Resolution"
        }
    }
    #- Slave part plus gap -#
    $m_canvas create polygon $ccd_frame(0,x$c) $ccd_frame(0,y$c) \
                             $ccd_frame(0,x$d) $ccd_frame(0,y$d) \
                             $ccd_frame(1,x$d) $ccd_frame(1,y$d) \
                             $ccd_frame(1,x$c) $ccd_frame(1,y$c) \
                             -outline cyan \
                             -fill cyan \
                             -stipple "" \
                             -tag " A mm rtz"
    if { [string match *hitms* $mask] } {
        tracer LEAVING HITMS
        return 0
    }

    if { [string match *SR* $resol] } {
        #- It should be forbidden between field stop and master CCD -#
	#- Upper area -#
	set xua  $ccd_frame(0,xa)
	set yua  $ccd_frame(0,ya)
        set xub  $ccd_frame(0,xb)
        set yub  $ccd_frame(0,yb)
        set xuc  $mos(0,xb)
        set yuc  $mos(0,yb)
        set xud  $mos(0,xa)
        set yud  $mos(0,ya)
        $m_canvas create polygon $xua $yua $xub $yub $xuc $yuc $xud $yud \
                                 -outline cyan \
                                 -fill cyan \
                                 -stipple "" \
                                 -tag " A mm rtz"
        #- Left area  -#
        set xla  $ccd_frame(0,xa)
	set yla  $ccd_frame(0,ya)
        set xlb  $mos(0,xa)
        set ylb  $mos(0,ya)
        set xlc  $mos(0,xd)
        set ylc  $mos(0,yd)
        set xld  $ccd_frame(1,xd)
        set yld  $ccd_frame(1,yd)
        $m_canvas create polygon $xla $yla $xlb $ylb $xlc $ylc $xld $yld \
                                 -outline cyan \
                                 -fill cyan \
                                 -stipple "" \
                                 -tag " A mm rtz"
        #- Right area -#
        set xra  $mos(0,xb)
	set yra  $mos(0,yb)
        set xrb  $ccd_frame(0,xb)
        set yrb  $ccd_frame(0,yb)
        set xrc  $ccd_frame(1,xc)
        set yrc  $ccd_frame(1,yc)
        set xrd  $mos(0,xc)
        set yrd  $mos(0,yc)
        $m_canvas create polygon $xra $yra $xrb $yrb $xrc $yrc $xrd $yrd \
                                 -outline cyan \
                                 -fill cyan \
                                 -stipple "" \
                                 -tag " A mm rtz"
    }
    tracer LEAVING
}

#================================================
private method plot_ref_star_area { mos10flag } {
#================================================
# Plots some cyan areas which are forbidden
# for the reference stars.
# All objects have tag : rfz

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    plot_ref_star_area_ccd2

    if { $mos10flag } {
        # Draw full slit blade 10
        $m_canvas create polygon $mos(9,xe)  $mos(9,ye) \
                                 $mos(9,xh)  $mos(9,yh) \
                                 $mos(11,xd) $mos(11,yd) \
                                 $mos(11,xa) $mos(11,ya) \
                                 -outline cyan \
                                 -fill cyan \
                                 -stipple "" \
                                 -tag " A mm rtz"
        if { $debug && $verbose } {
            puts stdout "plot_ref_star_area: MOS10 ON"
        }
    }

    # Select resolution parameters (mm)
    set resol [$this cget -collimat]
    if { [string match *SR* $resol] } {
        set fa [expr {10.0/396.0}]
        # Corner slots of the Frame Arrays
        lassign {a b c d} a b c d
        if { $debug && $verbose } {
            puts stdout "plot_ref_star_area: Standard Resolution"
        }
    } else {
	set fa [expr {10.0/198.0}]
        lassign {e f g h} a b c d
        if { $debug && $verbose } {
            puts stdout "plot_ref_star_area: High Resolution"
        }
    }

    # Draw the cyan forbidden rim region
    # Note : 10.0/396.0 means 10 arcsec of 396 arcsec

    if { [string match *mos* $mask] } {
        # mos, echs, pmos, img, ipol/sr
        if { [string match *HR* $resol] } {
	    set fa [expr 10.0/198.0]
            # AB-bar
            set ax1 [expr $ccd_frame(0,xe)  + \
                         ($ccd_frame(0,xh)  - \
                          $ccd_frame(0,xe)) * $fa]
            set ay1 [expr $ccd_frame(0,ye)  + \
                         ($ccd_frame(0,yh)  - \
                          $ccd_frame(0,ye)) * $fa]
            set bx1 [expr $ccd_frame(0,xf)  + \
                         ($ccd_frame(0,xg)  - \
                          $ccd_frame(0,xf)) * $fa]
            set by1 [expr $ccd_frame(0,yf)  + \
                         ($ccd_frame(0,yg)  - \
                          $ccd_frame(0,yf)) * $fa]
            $m_canvas create polygon $mos(1,xa) $mos(1,ya) \
                                     $mos(1,xd) $mos(1,yd) \
                                     $bx1 $by1 $ax1 $ay1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # DC-bar
            set dx1 [expr $ccd_frame(1,xh)  + \
                         ($ccd_frame(1,xe)  - \
                          $ccd_frame(1,xh)) * $fa]
            set dy1 [expr $ccd_frame(1,yh)  + \
                         ($ccd_frame(0,ye)  - \
                          $ccd_frame(1,yh)) * $fa]
            set cx1 [expr $ccd_frame(1,xg)  + \
                         ($ccd_frame(1,xf)  - \
                          $ccd_frame(1,xg)) * $fa]
            set cy1 [expr $ccd_frame(1,yg)  + \
                         ($ccd_frame(0,yf)  - \
                          $ccd_frame(1,yg)) * $fa]
            $m_canvas create polygon $mos(19,xe) $mos(19,ye) \
                                     $mos(19,xh) $mos(19,yh) \
                                     $cx1 $cy1 $dx1 $dy1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # AD-bar
            set ax1 [expr $ccd_frame(0,xe)  + \
                         ($ccd_frame(0,xf)  - \
                          $ccd_frame(0,xe)) * $fa]
            set ay1 [expr $ccd_frame(0,ye)  + \
                         ($ccd_frame(0,yf)  - \
                          $ccd_frame(0,ye)) * $fa]
            set dx1 [expr $ccd_frame(1,xh)  + \
                         ($ccd_frame(1,xg)  - \
                          $ccd_frame(1,xh)) * $fa]
            set dy1 [expr $ccd_frame(1,yh)  + \
                         ($ccd_frame(1,yg)  - \
                          $ccd_frame(1,yh)) * $fa]
            $m_canvas create polygon $mos(1,xa)  $mos(1,ya) \
                                     $mos(19,xe) $mos(19,ye) \
                                     $dx1 $dy1 $ax1 $ay1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # BC-bar
            set bx1 [expr $ccd_frame(0,xf)  + \
                         ($ccd_frame(0,xe)  - \
                          $ccd_frame(0,xf)) * $fa]
            set by1 [expr $ccd_frame(0,yf)  + \
                         ($ccd_frame(0,ye)  - \
                          $ccd_frame(0,yf)) * $fa]
            set cx1 [expr $ccd_frame(1,xg)  + \
                         ($ccd_frame(1,xh)  - \
                          $ccd_frame(1,xg)) * $fa]
            set cy1 [expr $ccd_frame(1,yg)  + \
                         ($ccd_frame(1,yh)  - \
                          $ccd_frame(1,yg)) * $fa]
            $m_canvas create polygon $mos(1,xd)  $mos(1,yd) \
                                     $mos(19,xh) $mos(19,yh) \
                                     $cx1 $cy1 $bx1 $by1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
        } else {
            set fa [expr 10.0/396.0]
            # AB-bar
            set ax1 [expr $mos(0,xa) + ($mos(0,xd) - $mos(0,xa)) * $fa]
            set ay1 [expr $mos(0,ya) + ($mos(0,yd) - $mos(0,ya)) * $fa]
            set bx1 [expr $mos(0,xb) + ($mos(0,xc) - $mos(0,xb)) * $fa]
            set by1 [expr $mos(0,yb) + ($mos(0,yc) - $mos(0,yb)) * $fa]
            $m_canvas create polygon $mos(1,xa) $mos(1,ya) \
                                     $mos(1,xd) $mos(1,yd) \
                                     $bx1 $by1 $ax1 $ay1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # DC-bar
            set dx1 [expr $mos(0,xd) + ($mos(0,xa) - $mos(0,xd)) * $fa]
            set dy1 [expr $mos(0,yd) + ($mos(0,ya) - $mos(0,yd)) * $fa]
            set cx1 [expr $mos(0,xc) + ($mos(0,xb) - $mos(0,xc)) * $fa]
            set cy1 [expr $mos(0,yc) + ($mos(0,yb) - $mos(0,yc)) * $fa]
            $m_canvas create polygon $mos(19,xe) $mos(19,ye) \
                                     $mos(19,xh) $mos(19,yh) \
                                     $cx1 $cy1 $dx1 $dy1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # AD-bar
            set ax1 [expr $mos(0,xa) + ($mos(0,xb) - $mos(0,xa)) * $fa]
            set ay1 [expr $mos(0,ya) + ($mos(0,yb) - $mos(0,ya)) * $fa]
            set dx1 [expr $mos(0,xd) + ($mos(0,xc) - $mos(0,xd)) * $fa]
            set dy1 [expr $mos(0,yd) + ($mos(0,yc) - $mos(0,yd)) * $fa]
            $m_canvas create polygon $mos(1,xa)  $mos(1,ya) \
                                     $mos(19,xe) $mos(19,ye) \
                                     $dx1 $dy1 $ax1 $ay1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # BC-bar
            set bx1 [expr $mos(0,xb) + ($mos(0,xa) - $mos(0,xb)) * $fa]
            set by1 [expr $mos(0,yb) + ($mos(0,ya) - $mos(0,yb)) * $fa]
            set cx1 [expr $mos(0,xc) + ($mos(0,xd) - $mos(0,xc)) * $fa]
            set cy1 [expr $mos(0,yc) + ($mos(0,yd) - $mos(0,yc)) * $fa]
            $m_canvas create polygon $mos(1,xd)  $mos(1,yd) \
                                     $mos(19,xh) $mos(19,yh) \
                                     $cx1 $cy1 $bx1 $by1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
        }
    } elseif { [string match *hitms* $mask] } {
        # AB-bar
        set ax1 [expr {$ccd_frame(0,x$a) + ($ccd_frame(0,x$d) - $ccd_frame(0,x$a))*$fa}]
        set ay1 [expr {$ccd_frame(0,y$a) + ($ccd_frame(0,y$d) - $ccd_frame(0,y$a))*$fa}]
        set bx1 [expr {$ccd_frame(0,x$b) + ($ccd_frame(0,x$c) - $ccd_frame(0,x$b))*$fa}]
        set by1 [expr {$ccd_frame(0,y$b) + ($ccd_frame(0,y$c) - $ccd_frame(0,y$b))*$fa}]
        $m_canvas create polygon $ccd_frame(0,xa) $ccd_frame(0,ya) \
                                 $ccd_frame(0,xb) $ccd_frame(0,yb) \
                                 $bx1 $by1 $ax1 $ay1 \
                                 -outline cyan  \
                                 -fill cyan \
                                 -stipple "" \
                                 -tag " A mm rtz"
        # DC-bar
        set dx1 [expr {$ccd_frame(0,x$d) + ($ccd_frame(0,x$a) - $ccd_frame(0,x$d))*$fa}]
        set dy1 [expr {$ccd_frame(0,y$d) + ($ccd_frame(0,y$a) - $ccd_frame(0,y$d))*$fa}]
        set cx1 [expr {$ccd_frame(0,x$c) + ($ccd_frame(0,x$b) - $ccd_frame(0,x$c))*$fa}]
        set cy1 [expr {$ccd_frame(0,y$c) + ($ccd_frame(0,y$b) - $ccd_frame(0,y$c))*$fa}]
        $m_canvas create polygon $ccd_frame(0,xd) $ccd_frame(0,yd) \
                                 $ccd_frame(0,xc) $ccd_frame(0,yc) \
                                 $cx1 $cy1 $dx1 $dy1 \
                                 -outline cyan  \
                                 -fill cyan \
                                 -stipple "" \
                                 -tag " A mm rtz"
        # AD-bar
        set ax1 [expr {$ccd_frame(0,x$a) + ($ccd_frame(0,x$b) - $ccd_frame(0,x$a))*$fa}]
        set ay1 [expr {$ccd_frame(0,y$a) + ($ccd_frame(0,y$b) - $ccd_frame(0,y$a))*$fa}]
        set dx1 [expr {$ccd_frame(0,x$d) + ($ccd_frame(0,x$c) - $ccd_frame(0,x$d))*$fa}]
        set dy1 [expr {$ccd_frame(0,y$d) + ($ccd_frame(0,y$c) - $ccd_frame(0,y$d))*$fa}]
        $m_canvas create polygon $ccd_frame(0,xa) $ccd_frame(0,ya) \
                                 $ccd_frame(0,xd) $ccd_frame(0,yd) \
                                 $dx1 $dy1 $ax1 $ay1 \
                                 -outline cyan  \
                                 -fill cyan \
                                 -stipple "" \
                                 -tag " A mm rtz"
        # BC-bar
        set bx1 [expr {$ccd_frame(0,x$b) + ($ccd_frame(0,x$a) - $ccd_frame(0,x$b))*$fa}]
        set by1 [expr {$ccd_frame(0,y$b) + ($ccd_frame(0,y$a) - $ccd_frame(0,y$b))*$fa}]
        set cx1 [expr {$ccd_frame(0,x$c) + ($ccd_frame(0,x$d) - $ccd_frame(0,x$c))*$fa}]
        set cy1 [expr {$ccd_frame(0,y$c) + ($ccd_frame(0,y$d) - $ccd_frame(0,y$c))*$fa}]
        $m_canvas create polygon $ccd_frame(0,xb) $ccd_frame(0,yb) \
                                 $ccd_frame(0,xc) $ccd_frame(0,yc) \
                                 $cx1 $cy1 $bx1 $by1 \
                                 -outline cyan  \
                                 -fill cyan \
                                 -stipple "" \
                                 -tag " A mm rtz"
    } else {
        #  hitms, hits, ipol/hr, lss, mxu
        if { [string match *HR* $resol] } {
            set fa [expr 10.0/198.0]
            # AB-bar
            set ax1 [expr $mos(0,xe) + ($mos(0,xh)-$mos(0,xe))*$fa]
            set ay1 [expr $mos(0,ye) + ($mos(0,yh)-$mos(0,ye))*$fa]
            set bx1 [expr $mos(0,xf) + ($mos(0,xg)-$mos(0,xf))*$fa]
            set by1 [expr $mos(0,yf) + ($mos(0,yg)-$mos(0,yf))*$fa]
            $m_canvas create polygon $mos(0,xa) $mos(0,ya) \
                                     $mos(0,xb) $mos(0,yb) \
                                     $bx1 $by1 $ax1 $ay1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # DC-bar
            set dx1 [expr $mos(0,xh) + ($mos(0,xe)-$mos(0,xh))*$fa]
            set dy1 [expr $mos(0,yh) + ($mos(0,ye)-$mos(0,yh))*$fa]
            set cx1 [expr $mos(0,xg) + ($mos(0,xf)-$mos(0,xg))*$fa]
            set cy1 [expr $mos(0,yg) + ($mos(0,yf)-$mos(0,yg))*$fa]
            $m_canvas create polygon $mos(0,xd) $mos(0,yd) \
                                     $mos(0,xc) $mos(0,yc) \
                                     $cx1 $cy1 $dx1 $dy1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # AD-bar
            set ax1 [expr $mos(0,xe) + ($mos(0,xf)-$mos(0,xe))*$fa]
            set ay1 [expr $mos(0,ye) + ($mos(0,yf)-$mos(0,ye))*$fa]
            set dx1 [expr $mos(0,xh) + ($mos(0,xg)-$mos(0,xh))*$fa]
            set dy1 [expr $mos(0,yh) + ($mos(0,yg)-$mos(0,yh))*$fa]
            $m_canvas create polygon $mos(0,xa) $mos(0,ya) \
                                     $mos(0,xd) $mos(0,yd) \
                                     $dx1 $dy1 $ax1 $ay1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # BC-bar
            set bx1 [expr $mos(0,xf) + ($mos(0,xe)-$mos(0,xf))*$fa]
            set by1 [expr $mos(0,yf) + ($mos(0,ye)-$mos(0,yf))*$fa]
            set cx1 [expr $mos(0,xg) + ($mos(0,xh)-$mos(0,xg))*$fa]
            set cy1 [expr $mos(0,yg) + ($mos(0,yh)-$mos(0,yg))*$fa]
            $m_canvas create polygon $mos(0,xb) $mos(0,yb) \
                                     $mos(0,xc) $mos(0,yc) \
                                     $cx1 $cy1 $bx1 $by1 \
                                     -outline cyan \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
        } else {
            set fa [expr 10.0/396.0]
            # AB-bar
            set ax1 [expr $mos(0,xa) + ($mos(0,xd)-$mos(0,xa))*$fa]
            set ay1 [expr $mos(0,ya) + ($mos(0,yd)-$mos(0,ya))*$fa]
            set bx1 [expr $mos(0,xb) + ($mos(0,xc)-$mos(0,xb))*$fa]
            set by1 [expr $mos(0,yb) + ($mos(0,yc)-$mos(0,yb))*$fa]
            $m_canvas create polygon $mos(0,xa) $mos(0,ya) \
                                     $mos(0,xb) $mos(0,yb) \
                                     $bx1 $by1 $ax1 $ay1 \
                                     -outline cyan  \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # DC-bar
            set dx1 [expr $mos(0,xd) + ($mos(0,xa)-$mos(0,xd))*$fa]
            set dy1 [expr $mos(0,yd) + ($mos(0,ya)-$mos(0,yd))*$fa]
            set cx1 [expr $mos(0,xc) + ($mos(0,xb)-$mos(0,xc))*$fa]
            set cy1 [expr $mos(0,yc) + ($mos(0,yb)-$mos(0,yc))*$fa]
            $m_canvas create polygon $mos(0,xd) $mos(0,yd) \
                                     $mos(0,xc) $mos(0,yc) \
                                     $cx1 $cy1 $dx1 $dy1 \
                                     -outline cyan  \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # AD-bar
            set ax1 [expr $mos(0,xa) + ($mos(0,xb)-$mos(0,xa))*$fa]
            set ay1 [expr $mos(0,ya) + ($mos(0,yb)-$mos(0,ya))*$fa]
            set dx1 [expr $mos(0,xd) + ($mos(0,xc)-$mos(0,xd))*$fa]
            set dy1 [expr $mos(0,yd) + ($mos(0,yc)-$mos(0,yd))*$fa]
            $m_canvas create polygon $mos(0,xa) $mos(0,ya) \
                                     $mos(0,xd) $mos(0,yd) \
                                     $dx1 $dy1 $ax1 $ay1 \
                                     -outline cyan  \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
            # BC-bar
            set bx1 [expr $mos(0,xb) + ($mos(0,xa)-$mos(0,xb))*$fa]
            set by1 [expr $mos(0,yb) + ($mos(0,ya)-$mos(0,yb))*$fa]
            set cx1 [expr $mos(0,xc) + ($mos(0,xd)-$mos(0,xc))*$fa]
            set cy1 [expr $mos(0,yc) + ($mos(0,yd)-$mos(0,yc))*$fa]
            $m_canvas create polygon $mos(0,xb) $mos(0,yb) \
                                     $mos(0,xc) $mos(0,yc) \
                                     $cx1 $cy1 $bx1 $by1 \
                                     -outline cyan  \
                                     -fill cyan \
                                     -stipple "" \
                                     -tag " A mm rtz"
        }

    }

    set reftarg_zone 1
    tracer LEAVING
}

#==================================
public method edit_ref_targ { c } {
#==================================
# Wrapper to allow external access to C_ref_targ
# called by menu bars, defined in fims_menu.tcl

# statuspanel: update statuspanel only for 'add' and 'purge' and 'reset'

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set res 0

    if { $debug } {
        puts stdout "EDIT_REF_TARG...$c"
    }

    if { [string match hitms [$this cget -modus]] } {
        # Allow RT outside the focal field
        C_ref_targ configure -outside 1
    }
    switch -exact -- $c {
        show {
            C_ref_targ show
        }
        reset {
            C_ref_targ reset
            set sp_nort 0
	    set last_action "Reset Reference Targets"
            catch {SP update}
        }
        add {
            set stop 1
            set slit_flag 0
            puts "Adding a reference target..."
            if {[C_ref_targ cget -mos10ali]} {
                set local ON
            } else {
                set local OFF
            }
            if { [string match *mos [$this cget -modus]] ||
                 [string match echs [$this cget -modus]] } {
                #set parent [utilNamespaceTail $this]
                set wid .skycat1.corr
                catch {destroy $wid}
                set w [util::DialogWidget $wid \
                       -title "Warning" \
                       -text "MOS 10 slit alignment : $local" \
                       -bitmap warning \
                       -buttons {ON OFF}]
                if { [$w activate] } {
                    C_ref_targ configure -mos10ali 0
                    set slit_flag 0
                } else {
                    C_ref_targ configure -mos10ali 1
                    set slit_flag 1
                }
                # for status panel
                set sp_mtali [C_ref_targ cget -mos10ali]
                catch {SP update}
            }
            plot_ref_star_area $slit_flag
            C_ref_targ add
            # for statuspanel
            set sp_nort [C_ref_targ nort]
	    set last_action "Add Reference Target"
            catch {SP update}
            set stop 0
            C_ref_targ plot $mos(0,r)
            $m_canvas delete rtz
            set reftarg_zone 0
        }
        purge {
            C_ref_targ purge
            C_ref_targ plot $mos(0,r)
            # for status panel
            set sp_nort  [C_ref_targ nort]
	    set last_action "Purge Reference Targets"
            catch {SP update}
        }
        verify {
            # plot the forbidden zone
            if { [C_ref_targ cget -mos10ali] } {
	        set slit_flag 1
	    } else {
	        set slit_flag 0
	    }
            plot_ref_star_area $slit_flag
            # reftarg_zone is set to 1 in plot_ref_star_area
            # call reftarg verify
            set res [C_ref_targ verify]
            # delete the forbidden zone
	    $m_canvas delete rtz
	    set reftarg_zone 0
	    # for status panel
	    set sp_nort [C_ref_targ nort]
	    catch {SP update}
        }
        verify_load {
            # plot the forbidden zone
            if { [C_ref_targ cget -mos10ali] } {
	        set slit_flag 1
	    } else {
	        set slit_flag 0
	    }
            plot_ref_star_area $slit_flag
            # reftarg_zone is set to 1 in plot_ref_star_area
            # call reftarg verify
            set res [C_ref_targ verify 1]
            # delete the forbidden zone
	    $m_canvas delete rtz
	    set reftarg_zone 0
	    # for status panel
	    set sp_nort [C_ref_targ nort]
	    catch {SP update}
        }
        plot {
            C_ref_targ plot $mos(0,r)
        }
        mtali  {
            # switch the mos10ali flag
            set sp_mtali [C_ref_targ cget -mos10ali]
            if { $sp_mtali } {
                set new_stat 0
            }
            if { !$sp_mtali } {
                set new_stat 1
            }
            C_ref_targ configure -mos10ali $new_stat
            # for status panel
            set sp_mtali [C_ref_targ cget -mos10ali]
            catch {SP update}
        }
    }
    C_ref_targ configure -outside 0
    if { $debug } {
        puts stdout "EDIT_REF_TARG...END"
    }
    tracer LEAVING
    return $res
}

#======================
public method help {} {
#======================
# Prints minihelp on the screen
# no longer needed 2.12.97

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    puts $lfId "FORS Simulator V0.9"
    puts $lfId "------------------ >> Mini Help <<"
    puts $lfId "(B1=Mouse Button-1 = left Mous button)"
    puts $lfId "B1 on green circle = Shift Mask"
    puts $lfId "B2 on green circle = Rotate Mask anti-clock"
    puts $lfId "B3 on green circle = Rotate Mask clock"

    puts $lfId "B1 on slit blade   = Move Slitlet"
    puts $lfId "B2 on slit blade   = this info"
    puts $lfId "B3 on CCD          = update"
    puts $lfId "B2 on CCD          = reset"
    puts $lfId "--------------------------------------------"
    puts $lfId $this
    tracer LEAVING
}

#=========================================
private method fix_mask_slit  { mod cf } {
#=========================================
# Calculates zemax correction for
# slit (either echs, lss, hits, pinh or
#  mxu-standard-mask-mode ) but apply
# correction to pointing (mask center)
# mod must be hits, lss  or echs
# puts stdout "in fix_mask_slit"

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # mask center in pixel
    catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x y image }

    # MXU, HIT-MS
    if { [string compare $mod hitms] == 0 ||
         [string compare $mod mxu] == 0 } {
        # puts "correct a fixed mask = MXU"
        # TCS
        # world_new = world_old - (targ_corrected - targ_initial)
        catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
        targw(2,a) targw(2,d) "wcs $EQX" }
        catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
        targw(2,x) targw(2,y) image }
        if { $cf } {
            calc_zemax 2 $targw(2,x) $targw(2,y)
            if { $dire_flag } {
                calc_diff_refrac 2 $targw(2,ca) $targw(2,cd)
                puts stdout "old pointing $targw(2,a) $targw(2,d)"
                puts stdout "new pointing $targw(2,ca) $targw(2,cd)"
            }
        }

        # slit
        # slit center pf C_mxus1
        # this is probably not necessary since it will be repeated in
        # part prepare of save_mxu_TPX ?
        # x,y -> xc,yc -> ra,dec in mxu
        C_mxus1 M2W
        set targw(10,a) [C_mxus1 get_RA]
        set targw(10,d) [C_mxus1 get_DEC]
        catch { $m_image convert coords $targw(10,a) $targw(10,d) "wcs $EQX" \
                                        targw(10,x)  targw(10,y) image
        }
        if { $cf } {
            calc_zemax 10 $targw(10,x) $targw(10,y)
            if { $dire_flag } {
                calc_diff_refrac 10 $targw(10,ca) $targw(10,cd)
                # puts stdout "old mos10    $targw(10,a) $targw(10,d)"
                # puts stdout "new mos10    $targw(10,ca) $targw(10,cd)"
            }
        }
        # puts stdout "$targw(2,x) $targw(2,y) $targw(2,cx) $targw(2,cy)"
        # puts stdout "$targw(2,a) $targw(2,d) $targw(2,ca) $targw(2,cd)"
        # puts stdout "$targw(10,x) $targw(10,y) $targw(10,cx) $targw(10,cy)"
        # puts stdout "$targw(10,a) $targw(10,d) $targw(10,ca) $targw(10,cd)"
    }

    # ECHS
    if { [string compare $mod echs] == 0 } {
        # puts "correct a fixed mask = ECHS"
        # TCS
        # world_new = world_old - (targ_corrected - targ_initial)
        if { [catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
                                              targw(2,a) targw(2,d) "wcs $EQX"
              } result] } {
            puts stdout "fix_mask ECHS: $result"
        }

        if { [catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
                                              targw(2,x) targw(2,y) image
              } result] } {
            puts stdout "fix_mask ECHS: $result"
        }

        if { $cf } {
            calc_zemax 2 $targw(2,x) $targw(2,y)
            if { $dire_flag } {
                calc_diff_refrac 2 $targw(2,ca) $targw(2,cd)
                puts stdout "old pointing $targw(2,a) $targw(2,d)"
                puts stdout "new pointing $targw(2,ca) $targw(2,cd)"
            }
            # slit
            # mask center = mos 10 in pixel
            calc_zemax 10 $targw(10,x) $targw(10,y)
            if { $dire_flag } {
                calc_diff_refrac 10 $targw(10,ca) $targw(10,cd)
                # puts stdout "old mos10    $targw(10,a) $targw(10,d)"
                # puts stdout "new mos10    $targw(10,ca) $targw(10,cd)"
            }
        }
        # puts stdout "$targw(2,x) $targw(2,y) $targw(2,cx) $targw(2,cy)"
        # puts stdout "$targw(2,a) $targw(2,d) $targw(2,ca) $targw(2,cd)"
        # puts stdout "$targw(10,x) $targw(10,y) $targw(10,cx) $targw(10,cy)"
        # puts stdout "$targw(10,a) $targw(10,d) $targw(10,ca) $targw(10,cd)"
    }

    # LSS
    if { [string compare $mod lss] == 0 } {
        # puts "correct a fixed mask = LSS"
        # TCS
        # world_new = world_old - (targ_corrected - targ_initial)

        if { [catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
                                              targw(2,a) targw(2,d) "wcs $EQX"
              } result] } {
            puts stdout "fix_mask:LSS: $result"
        }

        catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
                                        targw(2,x) targw(2,y) image
        }
        if { $cf } {
            calc_zemax 2 $targw(2,x) $targw(2,y)
            if { $dire_flag } {
                calc_diff_refrac 2 $targw(2,ca) $targw(2,cd)
                puts stdout "old pointing $targw(2,a) $targw(2,d)"
                puts stdout "new pointing $targw(2,ca) $targw(2,cd)"
            }
        }

        # SLIT (=science target)
        set dx [expr $mos($long_number,xa)-$mos($long_number,xd)]
        set dy [expr $mos($long_number,ya)-$mos($long_number,yd)]
        set A [expr sqrt($dx*$dx+$dy*$dy)]
        # Where is lss slit center? in mos($long_number,adeh)
        set X [expr ($mos($long_number,xa)+$mos($long_number,xd)+ \
                     $mos($long_number,xe)+$mos($long_number,xh))/4]
        set Y [expr ($mos($long_number,ya)+$mos($long_number,yd)+ \
                     $mos($long_number,ye)+$mos($long_number,yh))/4]
        catch { $m_image convert coords $X $Y canvas \
                                        targw(10,x) targw(10,y) image
        }
        if { [catch { $m_image convert coords $X $Y canvas \
                                       targw(10,a) targw(10,d) "wcs $EQX"
              } result] } {
            puts stdout "fix_mask:LSS: $result"
        }

        # puts stdout "current LSS position is $targw(10,a) $targw(10,d)"

        if { $cf } {
            calc_zemax 10 $targw(10,x) $targw(10,y)
        }

    }

    # HITS
    if { [string compare $mod hits] == 0 } {
        # puts "correct a fixed mask = HITS"
        # TCS
        # world_new = world_old - (targ_corrected - targ_initial)
        if { $bonsai_flag } {
            set dum [C_wc c2w $mos(0,x0) $mos(0,y0)]
            catch { $m_image convert coords [lindex $dum 0] [lindex $dum 1] "deg $EQX" \
                                            targw(2,a) targw(2,d) "wcs $EQX"
            }
        } else {
            if { [catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
                                            targw(2,a) targw(2,d) "wcs $EQX"
                  } result] } {
                puts stdout "fix_max:HITS:TCS  $result"
            }
        }
        if { [catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
                                              targw(2,x) targw(2,y) image
              } result] } {
             puts stdout "fix_mask:HITS:TCS $result"
        } else {
            if { $cf } {
                if { [calc_zemax 2 $targw(2,x) $targw(2,y)] } {
                    puts stdout "No Distortion Correction Applied to TEL !!"
                }
                if { $dire_flag } {
                    calc_diff_refrac 2 $targw(2,ca) $targw(2,cd)
                    puts stdout "old pointing $targw(2,a) $targw(2,d)"
                    puts stdout "new pointing $targw(2,ca) $targw(2,cd)"
                }
            }
        }
        # slit
        # similar to center-slit in hits
        # not necessary, since slit and TCS must be inside
        # frame
        # set dum [C_wc w2c $p1x $p1y]
        # set p1x [lindex $dum 0]
        # set p1y [lindex $dum 1]

        # --- SR ---
        if { $long_number != 1 } {
            # any of the available slits, not the pinhole !!!
            # slit width in canvas
            set dx [expr $mos($long_number,xa)-$mos($long_number,xd)]
            set dy [expr $mos($long_number,ya)-$mos($long_number,yd)]
            set A [expr sqrt($dx*$dx+$dy*$dy)]

            # full slit length in canvas
            set dx [expr $mos($long_number,xd)-$mos($long_number,xh)]
            set dy [expr $mos($long_number,yd)-$mos($long_number,yh)]
            set B [expr sqrt($dx*$dx+$dy*$dy)]

            # square points
            set xhn [expr $mos($long_number,xd)+($mos($long_number,xh)-$mos($long_number,xd))*$A/$B]
            set yhn [expr $mos($long_number,yd)+($mos($long_number,yh)-$mos($long_number,yd))*$A/$B]
            set xen [expr $mos($long_number,xa)+($mos($long_number,xe)-$mos($long_number,xa))*$A/$B]
            set yen [expr $mos($long_number,ya)+($mos($long_number,ye)-$mos($long_number,ya))*$A/$B]

            set X [expr ($mos($long_number,xa)+$mos($long_number,xd)+$xhn+$xen)/4]
            set Y [expr ($mos($long_number,ya)+$mos($long_number,yd)+$yhn+$yen)/4]
            if { $bonsai_flag } {
                # if underlying frame is small
                set dum [C_wc c2w $X $Y]
                catch { $m_image convert coords [lindex $dum 0] [lindex $dum 1] "deg $EQX" \
                                                targw(10,a) targw(10,d) "wcs $EQX"
                }
            } else {
                # underlying frame is large enough
                if { [catch {$m_image convert coords $X $Y canvas \
                                                     targw(10,a) targw(10,d) "wcs $EQX"
                      } result] } {
                    puts stdout "fix_mask:HITS:SLIT: $result"
                }
            }
            if { [catch { $m_image convert coords $X $Y canvas \
                                                  targw(10,x) targw(10,y) image
                  } result] } {
                puts stdout "fix_mask:HITS:SLIT: $result"
            }
            # puts stdout "current HITS Slit at $targw(10,a) $targw(10,d)"
        } else {
            # --- PINHOLE ---
            # highres collimator with hits -> pinhole
            # where is it ? in mos(9,adeh) (long_number==1)
            # square points, we assume long_number==1
            set X [expr ($mos(9,xa)+$mos(9,xd)+$mos(9,xe)+$mos(9,xh))/4]
            set Y [expr ($mos(9,ya)+$mos(9,yd)+$mos(9,ye)+$mos(9,yh))/4]
            catch { $m_image convert coords $X $Y canvas targw(10,x) targw(10,y) image }
            catch { $m_image convert coords $X $Y canvas targw(10,a) targw(10,d) "wcs $EQX" }
            puts stdout "current BPH position is $targw(10,a) $targw(10,d)"
        }
        if { $cf } {
            if { [calc_zemax 10 $targw(10,x) $targw(10,y)] } {
                puts stdout "fix_mask:HITS:SLIT: Distortion not corrected for TARG !!"
            }
            if { $dire_flag } {
                calc_diff_refrac 10 $targw(10,ca) $targw(10,cd)
                puts stdout "old slit     $targw(10,a) $targw(10,d)"
                puts stdout "new slit     $targw(10,ca) $targw(10,cd)"
            }
        }
    }

    # In any case the new correct pointing will be stored
    # to MOS 2
    # corrected telescope pointing
    if { $cf } {
        catch { $m_image convert coords $targw(2,ca) $targw(2,cd) "wcs $EQX" ta td "deg $EQX" }
        # for slit difference
        catch { $m_image convert coords $targw(10,a) $targw(10,d) "wcs $EQX" a d "deg $EQX" }
        catch { $m_image convert coords $targw(10,ca) $targw(10,cd) "wcs $EQX" ca cd "deg $EQX" }
        set ta [expr {$ta - $ca + $a}]
        set td [expr {$td - $cd + $d}]
        catch { $m_image convert coords $ta $td "deg $EQX" targw(2,ca) targw(2,cd) "wcs $EQX" }
    }
    puts $lfId "FixMaskSlit to $targw(10,a) $targw(10,d)"
    tracer LEAVING
}

#==============================
private method reset_slits {} {
#==============================
# Called by button1-release,
# after mask shift.
# Resets slit positions.

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Update the 19 slit-positions of the mos
    if {[string compare $mask "mos"]==0} {
        # This means mos,pmos,img,pimg-SR,echs

        # v0.9.6 Close the unused mos slit blades again
        # Only in mos/pmos, not in img/pimg
        # Exception (p)mos slit-les spectroscopy
        if {[string match "*mos" [$this cget -modus]]} {
            if {[string compare open [$this cget -slit_width]]} {
                # Its mos or pmos, but not slitless spectroscopy (IMS), (PIMS)
                for { set i 1} { $i < 20 } { incr i 1} {
                    # Close all 19 mos slit blades
                    set mos($i,xb) $mos($i,xa)
                    set mos($i,yb) $mos($i,ya)
                    set mos($i,xc) $mos($i,xa); # + some value to open to 6 mm
                    set mos($i,yc) $mos($i,ya)
                    set mos($i,xf) $mos($i,xe)
                    set mos($i,yf) $mos($i,ye)
                    set mos($i,xg) $mos($i,xe); # + some value to open to 6 mm
                    set mos($i,yg) $mos($i,ye)
                    set targw($i,f) 0;          # deselect mos slit
                }
            }
        }

        # Take the mean
        for { set i 1} { $i < 20 } { incr i 1} {
            # Update the slit postion values
            # Obsolete in mos, pmos, since f=0
            set targw($i,a) [expr ($mos($i,xc)+$mos($i,xb)+\
                            $mos($i,xf)+$mos($i,xg))/4.0]
            set targw($i,d) [expr ($mos($i,yc)+$mos($i,yb)+\
                            $mos($i,yf)+$mos($i,yg))/4.0]
        }
    }

    # MXU, HIT-MS specials delete all instances of MXU
    if { [string compare [$this cget -modus] hitms] == 0 ||
         [string compare [$this cget -modus] mxu] == 0 } {
        if { [fors::MXU_SLIT::get_mmff] } {
            # Do nothing, retain the standard mask as a fixed mask
        } else {
            # Delete all slits but keep reference slits in MXU
            # PBG 20100422 mxu_delete_all_slits 1
        }
    }

    # Now we apply the correction for optical distortion
    # of FORS images and differential refraction
    if { [string compare [$this cget -modus] hits] == 0 } {
        set corr_flag [$this cget -fors_frame]
        fix_mask_slit hits $corr_flag
    }

    update_and_replot
    
    set AutoposMethod ""
    
    set last_action {Move Mask}
    catch {SP update}
    tracer LEAVING
}

#======================================================
private method move_slit { w sx sy {on 1} {click 1} } {
#======================================================
# moves the MOS slit (MOS, PMOS)
# w = canvas widget
# sx, sy is cursor pos in screen coords
# on==1  : select, on==0 : un-select, give slit free
# click==1 : called by binding (click), moved manually
# click==0 : called by private member functions (e.g. load_mask),
#            do not use center gauss
#            do not use optical correction
# stop==1: do nothing (e.g. when selecting a reference object)
# stop==0: do the normal procedure

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $stop } {
        tracer LEAVING 0
        return
    }

    catch { $m_image convert coords $sx $sy screen x y canvas }
    set pmos_flag [expr [string compare [$this cget -modus] "pmos"] == 0]

    # find the tag of the slit blade belonging to the target object
    # puts $lfId $curtag
    # puts $lfId " coords of $curtag : [$w coords $curtag]"
    # puts $lfId "$mos(12,xa) $mos(12,ya) $mos(12,xb)"
    set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
    set my [string range $curtag  1 end]
    # puts stdout ">[$w find closest $x $y]< \
    #              >[$w gettags [$w find closest $x $y]]< \
    #              curtag= >$curtag<  my= >$my<"

    if { $on == 0 } {
        if { $my != "" } {
            if { $click } {
                set wid .skycat1.save
                catch { destroy $wid }
                set w [DialogWidget $wid \
                                    -text "You are De-Selecting MOS slit #$my" \
                                    -bitmap warning \
                                    -default -1 \
                                    -buttons {OK Cancel}
                      ]
                if { [$w activate] } {
                    tracer LEAVING 1
                    return
                }
            }
            #  we have to deselect
            set targw($my,f) 0;       # deselect slit
            set mos($my,b) $mos(0,b); # set slit width to park position slit width
            if { [expr abs($mos(0,xa)-$mos(0,xb))] < 0.01 } {
                # puts stdout "deselect perpendicular"
                set mos($my,xb) $mos($my,xa)
                set mos($my,xc) [expr $mos($my,xb) + $mos($my,b)/$focscale*$fits_x_scale*$ctom]
                set mos($my,xf) $mos($my,xe)
                set mos($my,xg) [expr $mos($my,xf) + $mos($my,b)/$focscale*$fits_x_scale*$ctom]
            } else {
                # slope
                # puts stdout "deselect slope"
                set mos($my,xb) $mos($my,xa)
                set mos($my,yb) $mos($my,ya)
                set mos($my,xc) [expr $mos($my,xa) + ($mos($my,xd)-$mos($my,xa))/80.0]
                set mos($my,yc) [expr $mos($my,ya) + ($mos($my,yd)-$mos($my,ya))/80.0]

                set mos($my,xf) $mos($my,xe)
                set mos($my,yf) $mos($my,ye)
                set mos($my,xg) [expr $mos($my,xe) + ($mos($my,xh)-$mos($my,xe))/80.0]
                set mos($my,yg) [expr $mos($my,ye) + ($mos($my,yh)-$mos($my,ye))/80.0]
            }
            # set slit width  back again
            set mos($my,b) [$this cget -slit_width]
        }
        # scratch the whole old mask and plot_mask it
        catch {$w delete A}
        #------------------------
        update_and_replot
        #------------------------
    } else {
        # no deselect
        if { [string length $my] == 0 } {
            # if the slit was deselected before
            set mos($my,b) [$this cget -slit_width]
            catch { $m_image convert coords $sx $sy screen nx ny "deg $EQX" }
            catch { $m_image convert coords $sx $sy screen tx ty "wcs $EQX" }
            puts stdout "Warning cannot move #$my to ($nx,$ny)=($tx,$ty)"
            puts $lfId "MoveSlit: Warning cannot move #$my to ($nx,$ny)=($tx,$ty)"
        } elseif { [expr $my-($my/2)*2] && $pmos_flag } {
            catch { $m_image convert coords $sx $sy screen nx ny "deg $EQX" }
            catch { $m_image convert coords $sx $sy screen tx ty "wcs $EQX" }
            puts $lfId "MoveSlit:  Warning cannot move #$my to ($nx,$ny)=($tx,$ty)"
           puts stdout "Warning cannot move #$my to ($nx,$ny)=($tx,$ty)"
        } else {
            # from here on : everything is ok : carry out the move
            #-------------------
            # center-slit option
            #-------------------
            if { $click } {
                if { [$this cget -pics] } {
                   # sx,sy are approximate coords,
                   # use RtdImagePick to center slit on image target
                   catch { $m_image convert coords $sx $sy screen a0 d0 image }
                   # puts stdout "Button-Press canvas=$x $y screen=$sx $sy pixel=$a0 $d0"
                   set picked {}
                   pick_star
                   # set FWHM [lindex $picked 5]
                   set FWHM [expr {$FWHM_X + $FWHM_Y}]
                   if { $FWHM > 0  } {
                       catch { $m_image convert coords $star_ix $star_iy image sx sy screen }
                       catch { $m_image convert coords $star_ix $star_iy image  x  y canvas }
                       # puts stdout "[$w gettags current]"
                       set my [string range [lindex [$w gettags current] 2] 1 end]
                       if { $my == "" } {
                           puts stdout "Cursor not on any MOS slit"
                           tracer LEAVING 2
                           return
                       }
                       # set CCD pixel of target
                       set targw($my,x) $star_ix
                       set targw($my,y) $star_iy
                       puts stdout [format "targw(%2d,x) = %8.3f, targw(%2d,y) = %8.3f" \
                                           $my $targw($my,x) $my $targw($my,y)
                                   ]
                       # puts stdout "pick_star from ($a0,$d0) on center_gauss result = \
                       #             ($star_ix,$star_iy) pixel"
                   } else {
                       # there are two posibilities
                       # a) : return, do not specify any position
                       # b) : use the Button-Release coords
                       pick_technique
                       tracer LEAVING 3
                       return
                   }
                }
            }
            #-----------------------
            # center-slit option end
            #-----------------------
            # puts $lfId "number = >[string range $curtag  1 end]<"
            catch { $m_image convert coords $sx $sy screen a1 d1 image }
            catch { $m_image convert coords $sx $sy screen a2 d2 "deg $EQX" }
            catch { $m_image convert coords $sx $sy screen a3 d3 "wcs $EQX" }
            # CCD pixel of target
            set targw($my,x) $a1
            set targw($my,y) $d1
            # puts stdout "targw($my,x) = $targw($my,x), targw($my,y) = $targw($my,y)"
            # puts stdout [format "MS: in=(%8.3f,%8.3f)screen (%8.3f,%8.3f)ima (%s,%s)wcs" \
            #                      $sx $sy $a1 $d1 $a3 $d3
            #             ]
            # targw in  wcs
            # -------------
            catch { $m_image convert coords $x $y canvas targw($my,a) targw($my,d) "wcs $EQX" }
            set targw($my,f) 1
            # $m_image convert dist   $mos($my,b) dummy canvas \
            #                "deg $EQX" wid dummy
            set l_focscale 1.89393939; # nominal focscale
            set scal [expr $atd * $l_focscale]

            # a2-world_x is difference delta alpha
            # since x-axis points to the west and y-axis points to the south
            # there is (delta alpha > 0) -> (delta x < 0)
            # and      (delta decli > 0) -> (delta y < 0)
            # for wrong instrument mounting
            # if { $mountangle == 180 } {
            #     set xd_mm [expr ($world_x-$a2)/$trigo/$scal]
            #     set yd_mm [expr ($world_y-$d2)/$scal]
            # } else {
            #     set xd_mm [expr ($a2-$world_x)/$trigo/$scal]
            #     set yd_mm [expr ($d2-$world_y)/$scal]
            # }
            # puts stdout "ang=$abs_ang_rad $xd_mm $yd_mm"
            # set x_mm [expr $xd_mm*cos($abs_ang_rad)-$yd_mm*sin($abs_ang_rad)]
            # set y_mm [expr $xd_mm*sin($abs_ang_rad)+$yd_mm*cos($abs_ang_rad)]

            # we calculate x in mm focplane of the new slit position
            # first the slit-center at x==0 in canvas
            set mid_x [expr ($mos($my,xa)+$mos($my,xd)+$mos($my,xe)+$mos($my,xh))/4.0]
            set mid_y [expr ($mos($my,ya)+$mos($my,yd)+$mos($my,ye)+$mos($my,yh))/4.0]
            # second the slit position in canvas (=x,y) already known
            # third the distance between both points as an approx.
            set mid_x [expr $mid_x - $x]
            set mid_y [expr $mid_y - $y]
            set mid_x [expr $mid_x * $mid_x]
            set mid_y [expr $mid_y * $mid_y]
            set x_mm  [expr sqrt($mid_x+$mid_y) / $ctom]
            # check the absolute value (+ or - x)
            set mid_x [expr $mos($my,xa)-$x]
            set mid_y [expr $mos($my,ya)-$y]
            set mad_x [expr $mos($my,xd)-$x]
            set mad_y [expr $mos($my,yd)-$y]
            set mid_x [expr sqrt($mid_x*$mid_x+$mid_y*$mid_y)]
            set mad_x [expr sqrt($mad_x*$mad_x+$mad_y*$mad_y)]
            if { $mid_x < $mad_x } {
                set x_mm [expr $x_mm * -1.0]
            }
            puts $lfId [format "Slit #%2d to (%s,%s)=(%6.3f,%6.3f) mm(%6.3f)" \
                               $my $a3 $d3 $a2 $d2 $x_mm
                       ]
            puts stdout [format "Move #%2d to (%s,%s)=(%6.3f,%6.3f) mm(%6.3f) %6.3f %6.3f" \
                                $my $a3 $d3 $a2 $d2 $x_mm $x $y
                        ]
            calc_zemax $my $a1 $d1;   # pixel coords
            # apply optical correction in any case
            #                   deg     wcs
            # calc_oc $x_mm $a1 $my $a2 $d2 $a3 $d3

            # apply correction of differential refraction (10.5.99)
            # input slit position, corrected for optical distortion
            if { $dire_flag } {
                calc_diff_refrac $my $targw($my,ca) $targw($my,cd)
            }
            # we calculate x in mm focplane of the slit position
            # here once more with optical correction
            # first the slit-center at x==0 in canvas
            set mid_x [expr ($mos($my,xa)+$mos($my,xd)+$mos($my,xe)+$mos($my,xh))/4.0]
            set mid_y [expr ($mos($my,ya)+$mos($my,yd)+$mos($my,ye)+$mos($my,yh))/4.0]
            # second the slit position in canvas (=x,y) already known
            # but not with the optical correction value
            catch { $m_image convert coords $targw($my,ca) $targw($my,cd) "wcs $EQX" x5 y5 canvas }
            # third the distance between both points as an approx.
            set mid_x [expr $mid_x - $x5]; set mid_y [expr $mid_y - $y5]
            set mid_x [expr $mid_x * $mid_x]; set mid_y [expr $mid_y * $mid_y]
            set x_mm [expr sqrt($mid_x+$mid_y) / $ctom]
            # check the absolute value (+ or - x)
            set mid_x [expr $mos($my,xa)-$x5]
            set mid_y [expr $mos($my,ya)-$y5]
            set mad_x [expr $mos($my,xd)-$x5]
            set mad_y [expr $mos($my,yd)-$y5]
            set mid_x [expr sqrt($mid_x*$mid_x+$mid_y*$mid_y)]
            set mad_x [expr sqrt($mad_x*$mad_x+$mad_y*$mad_y)]
            if { $mid_x < $mad_x } {
                set x_mm [expr $x_mm * -1.0]
            }
            # show optical correction
            # puts $lfId  [format "Slit #%2d to (%s,%s)=(%6.3f,%6.3f) mm(%6.3f)" \
            #                      $my $targw($my,ca) $targw($my,cd) 0.0  0.0 $x_mm]
            # puts stdout [format "Move #%2d to (%s,%s)=(%6.3f,%6.3f) mm(%6.3f) %6.3f %6.3f" \
            #                      $my $targw($my,ca) $targw($my,cd) 0.0 0.0  $x_mm $x5 $y5]

            # set ctom [eval calc_ctom]
            # puts stdout $ctom
            # now the graphical part: perpendicular
            if { [expr abs($mos(0,xa)-$mos(0,xb))] < 0.01 } {
                # slit blade move direction is up-down
                # in order to avoid deviding by zero, special treatment
                puts $lfId "MoveSlit:  >> special up-down <<"
                if { $mountangle == 180 } {
                    set mos($my,yc) [expr $y - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,yg) [expr $y - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,yb) [expr $y + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,yf) [expr $y + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                } else {
                    set mos($my,yc) [expr $y + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,yg) [expr $y + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,yb) [expr $y - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,yf) [expr $y - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                }
                # the rest coordinates remain unchanged
                # for slit-less spectroscopy, the slit may be moved out of the
                # focal field. In this case draw the slit smaller;
                # next move_slit inside focal field will draw the full slit again
                # in original size mos(b)
                set lam [expr ($mos($my,yf)-$mos($my,ye))/($mos($my,yh)-$mos($my,ye))]
                # puts stdout "0 < $lam < 1"
                if { $lam < 0 } {
                    set mos($my,xf) $mos($my,xe)
                    set mos($my,yf) $mos($my,ye)
                    set mos($my,xb) $mos($my,xa)
                    set mos($my,yb) $mos($my,ya)
                }
                set lam [expr ($mos($my,yg)-$mos($my,ye))/($mos($my,yh)-$mos($my,ye))]
                if { $lam > 1 } {
                    set mos($my,xg) $mos($my,xh)
                    set mos($my,yg) $mos($my,yh)
                    set mos($my,xc) $mos($my,xd)
                    set mos($my,yc) $mos($my,yd)
                }
            } elseif { [expr abs($mos(0,ya)-$mos(0,yb))] < 1e-2 } {
                # in order to avoid deviding by zero, special treatment
                # slit blade move direction is left-right
                # puts stdout "move slit in NE direction"
                puts $lfId "MoveSlit: >> special left-right <<"
                if { $mountangle == 180 } {
                    set mos($my,xb) [expr $x + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,xf) [expr $x + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,xc) [expr $x - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,xg) [expr $x - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                } else {
                    set mos($my,xb) [expr $x - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,xf) [expr $x - $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,xc) [expr $x + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                    set mos($my,xg) [expr $x + $mos($my,b)/2/$focscale*$fits_x_scale*$ctom]
                }
                # for slitless spectroscopy, the slit may be moved out of the
                # focal field. In this case draw the slit smaller;
                # next move_slit inside focal field will draw the full slit again
                # in original size mos(b)
                set lam [expr ($mos($my,xf)-$mos($my,xe))/($mos($my,xh)-$mos($my,xe))]
                # puts stdout "0 < $lam < 1"
                if { $lam < 0 } {
                    set mos($my,xf) $mos($my,xe)
                    set mos($my,yf) $mos($my,ye)
                    set mos($my,xb) $mos($my,xa)
                    set mos($my,yb) $mos($my,ya)
                }
                set lam [expr ($mos($my,xg)-$mos($my,xe))/($mos($my,xh)-$mos($my,xe))]
                if { $lam > 1 } {
                    set mos($my,xg) $mos($my,xh)
                    set mos($my,yg) $mos($my,yh)
                    set mos($my,xc) $mos($my,xd)
                    set mos($my,yc) $mos($my,yd)
                }
            } else {
                # slope
                # now the standard case
                # deviding by zero does no more occur
                # analytical expressions to adjust the slit blade in such a
                # way that the slit center matches the position of the target
                # star, even if the target star is not on the
                # slilet central line
                puts $lfId "MoveSlit: >> usual shear <<"
                set m1 [expr ($mos($my,ya)-$mos($my,yd))/($mos($my,xa)-$mos($my,xd))]
                set m2 [expr ($mos($my,ye)-$mos($my,yh))/($mos($my,xe)-$mos($my,xh))]
                set b1 [expr ($mos($my,ya)*$mos($my,xd)-$mos($my,xa)*$mos($my,yd))]
                set b1 [expr $b1/($mos($my,xd)-$mos($my,xa))]
                set b2 [expr ($mos($my,ye)*$mos($my,xh)-$mos($my,xe)*$mos($my,yh))]
                set b2 [expr $b2/($mos($my,xh)-$mos($my,xe))]
                # puts $lfId "m1 = $m1  m2 = $m2 b1=$b1 b2=$b2"
                set h2 [expr $m1+1/$m1]
                if { $mountangle == 180 } {
                    set h1 [expr  $h2*$fits_x_scale*$ctom*$mos($my,b)/$focscale/2/sqrt($m1*$m1+1)]
                } else {
                    set h1 [expr -$h2*$fits_x_scale*$ctom*$mos($my,b)/$focscale/2/sqrt($m1*$m1+1)]
                }
                set bl [expr $y + $x/$m1 + $h1]
                set br [expr $y + $x/$m1 - $h1]
                #puts $lfId "h1 = $h1  h2 = $h2  br = $br  bl = $bl"
                set x1 [expr ($bl-$b1)/$h2]
                set x2 [expr ($br-$b1)/$h2]
                set x3 [expr ($bl-$b2)/$h2]
                set x4 [expr ($br-$b2)/$h2]
                set y1 [expr $m1*$x1+$b1]
                set y2 [expr $m1*$x2+$b1]
                set y3 [expr $m1*$x3+$b2]
                set y4 [expr $m1*$x4+$b2]
                # puts $lfId "$x1 $y1, $x2 $y2, $x3 $y3, $x4 $y4"
                # index1=c 2=b 3=g 4=f
                # set mos($my,xc) $x1
                # set mos($my,yc) $y1
                # set mos($my,xb) $x2
                # set mos($my,yb) $y2
                # set mos($my,xg) $x3
                # set mos($my,yg) $y3
                # set mos($my,xf) $x4
                # set mos($my,yf) $y4
                set mos($my,xb) $x1
                set mos($my,yb) $y1
                set mos($my,xc) $x2
                set mos($my,yc) $y2
                set mos($my,xf) $x3
                set mos($my,yf) $y3
                set mos($my,xg) $x4
                set mos($my,yg) $y4

                # for slit-les spectroscopy, the slit may be moved out of the
                # focal field. In this case draw the slit smaller;
                # next move_slit inside focal field will draw the full slit again
                # in original size mos(b)
                set lam [expr ($mos($my,xf)-$mos($my,xe))/($mos($my,xh)-$mos($my,xe))]
                # puts stdout "0 < $lam < 1"
                if { $lam < 0 } {
                    set mos($my,xf) $mos($my,xe)
                    set mos($my,yf) $mos($my,ye)
                    set mos($my,xb) $mos($my,xa)
                    set mos($my,yb) $mos($my,ya)
                }
                set lam [expr ($mos($my,xg)-$mos($my,xe))/($mos($my,xh)-$mos($my,xe))]
                if { $lam > 1 } {
                    set mos($my,xg) $mos($my,xh)
                    set mos($my,yg) $mos($my,yh)
                    set mos($my,xc) $mos($my,xd)
                    set mos($my,yc) $mos($my,yd)
                }
            }
        }
        # scratch the whole old mask and plot_mask it
        catch {$w delete A}
        #--------
        plot_mask
        #--------
        if { $badpixelmap } {
            plot_BPM
        }
    }
    set last_action "Move MOS \#$my"
    catch {SP update}
    tracer LEAVING 4
    return $my
}

#===============================================
private method move_slit_blade { w sx sy but } {
#===============================================
# Moves only one slit blade of the pair
# For IMS, PIMS, IMG, IPOL-SR
# sx,sy screen coords, but = mouse button
# but = 0 1 out in
#       out : of the focal field, delete blade setting
#         1 : move blade A (if clicked inside the slit)
#         0 : move blade B (if clicked inside the slit)
# stop = 1: do nothing (when selecting a reference object)
# stop = 0: do the normal procedure

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $stop } {
        tracer LEAVING STOP
        return
    }

    # puts stdout "Move Slitlet : $w sx=$sx sy=$sy but=$but"
    # puts stdout " coords of m1 : [$w coords m1]"
    # puts stdout [$w gettags m1]
    # puts stdout [$w find closest $sx $sy]
    # puts stdout [$w gettags [$w find closest $sx $sy]]
    # shows the current tag clicked on
    # puts stdout [lindex [$w gettags [$w find closest $sx $sy]] 1]
    catch { $m_image convert coords $sx $sy screen x y canvas }

    # find the tag of the slit blade belonging to the cursor position
    # puts stdout "[$w gettags [$w find closest $x $y]]"
    set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
    set sidtag [lindex [$w gettags [$w find closest $x $y]] 3]
    # puts stdout " coords of $curtag : [$w coords $curtag]"
    # puts $lfId "$mos(12,xa) $mos(12,ya) $mos(12,xb)"
    set c1 [expr [string compare [$this cget -modus] "ipol"] == 0]
    set c2 [expr [string compare [$this cget -modus] "pmos"] == 0 && \
                 [string compare [$this cget -slit_width] open] == 0]
    set ipol_flag [expr $c1 || $c2]
    # puts stdout "ipol_flag = $ipol_flag"
    # set ipol_flag [expr [string compare [$this cget -modus] "ipol"]==0]

    set my [string range $curtag  1 end]
    set sid [string range $sidtag  1 1]

    # puts stdout ">$curtag< >$sidtag< >$my< >$sid<"

    if { [string compare $but out] == 0 } {
        if { ![expr {$my - ($my/2)*2}] && $ipol_flag || !$ipol_flag } {
           set mos($my,b) $mos(0,b)
           set mos($my,xb) $mos($my,xa)
           set mos($my,yb) $mos($my,ya)
           set mos($my,xc) $mos($my,xd)
           set mos($my,yc) $mos($my,yd)
           set mos($my,xf) $mos($my,xe)
           set mos($my,yf) $mos($my,ye)
           set mos($my,xg) $mos($my,xh)
           set mos($my,yg) $mos($my,yh)
           set targw($my,f) 0
        }
        catch {$w delete A}
        plot_mask
        tracer LEAVING 1
        return;
    }

    if { [string length $my] == 0 } {
        # puts stdout "Warning cannot move #$my to ($x,$y)"
    } elseif { [expr {$my - ($my/2)*2}] && $ipol_flag } {
        # puts stdout "Warning cannot move #$my to ($x,$y)" } else {
        # puts stdout "number = >[string range $curtag  1 end]<"
        # puts stdout "side   = >[string range $sidtag  1 1]<"
        # [expr $sid-($sid/2)*2]
        # this is a odd number
        catch { $m_image convert coords $sx $sy screen a1 d1 image }

        if { [catch { $m_image convert coords $sx $sy screen a2 d2 "deg $EQX" } result] } {
            puts stdout "move_slit_blade: $result"
        }

        if { [catch { $m_image convert coords $sx $sy screen a3 d3 "wcs $EQX" } result] } {
            puts stdout "move_slit_blade: $result"
        }

        set targw($my,f) 1
        # $m_image convert dist   $mos($my,b) dummy canvas \
        #                "deg $EQX" wid dummy
        set scal [expr $atd * $focscale]
        # a2-world_x is difference delta alpha
        # since x-axis points to east and y-axis points to north
        # there is (delta alpha > 0) -> (delta x > 0)
        # and      (delta decli > 0) -> (delta y > 0)
        set x_mm [expr ($a2-$world_x)/$trigo/$scal]
        set y_mm [expr ($d2-$world_y)/$scal]
        puts $lfId  [format "blade #%2d%s to (%s,%s)=(%6.3f,%6.3f) mm(%6.3f,%6.3f)" \
                            $my $sid $a3 $d3 $a2 $d2 $x_mm $y_mm]
        puts stdout [format "blade #%2d%1s to (%s,%s)=(%6.3f,%6.3f) mm(%6.3f,%6.3f)" \
                            $my $sid $a3 $d3 $a2 $d2 $x_mm $y_mm]
        # b=[expr $wid*$atod] arcsec" \
        #   c($x,$y) s($sx,$sy) i($a1,$d1)"

        if { [expr {abs($mos(0,xa)-$mos(0,xb))}] < 0.01 } {
            # slit blade move direction is up-down
            # in order to avoid deviding by zero, special treatment
            puts $lfId "MoveBlade: >> special up-down <<"
            # ERROR
            switch -exact -- $sid {
                a {
                    set mos($my,yb) $y
                    set mos($my,yf) $y
                }
                b {
                    set mos($my,yc) $y
                    set mos($my,yg) $y
                }
                s {
                    # which point is closer ?
                    switch -exact -- $but {
                        1 {
                            # take MOS A
                            set mos($my,yb) $y
                            set mos($my,yf) $y
                        }
                        0 {
                            # take MOS B
                            set mos($my,yc) $y
                            set mos($my,yg) $y
                        }
                    }
                }
                default {
                    puts $lfId "MoveBlade: Warning "
                }
            }
            # the rest coordinated remain unchanged
        } elseif { [expr {abs($mos(0,ya) - $mos(0,yb))}] < 1e-2 } {
            # in order to avoid deviding by zero, special treatment
            # slit blade move direction is left-right
            # puts $lfId ">> special left-right <<"
            # ERROR
            switch -exact -- $sid {
                a {
                    set mos($my,xb) $x
                    set mos($my,xf) $x
                }
                b {
                    set mos($my,xc) $x
                    set mos($my,xg) $x
                }
                s {
                    # which point is closer ?
                    switch -exact -- $but {
                        1 {
                            # take MOS A
                            set mos($my,xb) $x
                            set mos($my,xf) $x
                        }
                        0 {
                            # take MOS B
                            set mos($my,xc) $x
                            set mos($my,xg) $x
                        }
                    }
                }
                default {
                    puts $lfId "MoveBlade: Warning shift single slit blade"
                }
            }
        } else {
            # now the standard case
            # deviding by zero does no more occur
            # analytical expressions to adjust the slit blade in such a
            # way that the slit center matches the position of the target
            # star, even if the target star is not on the
            # slilet central line
            # puts $lfId ">> usual shear <<"
            # index 1 line ABCD
            # index 2 line EFGH
            set m1 [expr ($mos($my,ya)-$mos($my,yd))/($mos($my,xa)-$mos($my,xd))]
            set m2 [expr ($mos($my,ye)-$mos($my,yh))/($mos($my,xe)-$mos($my,xh))]
            set b1 [expr ($mos($my,ya)*$mos($my,xd)-$mos($my,xa)*$mos($my,yd))]
            set b1 [expr $b1/($mos($my,xd)-$mos($my,xa))]
            set b2 [expr ($mos($my,ye)*$mos($my,xh)-$mos($my,xe)*$mos($my,yh))]
            set b2 [expr $b2/($mos($my,xh)-$mos($my,xe))]
            #puts $lfId "m1 = $m1  m2 = $m2 b1=$b1 b2=$b2"
            set h2 [expr $m1+1/$m1]
            set b3 [expr $y + $x/$m1]
            #puts $lfId "h1 = $h1  h2 = $h2  br = $br  bl = $bl"
            set x1 [expr ($b3-$b1)/$h2]
            set x2 [expr ($b3-$b2)/$h2]
            set y1 [expr $m1*$x1+$b1]
            set y2 [expr $m1*$x2+$b2]
            switch -exact -- $sid {
                a {
                    set mos($my,xb) $x1; set mos($my,yb) $y1
                    set mos($my,xf) $x2; set mos($my,yf) $y2
                }
                b {
                    set mos($my,xc) $x1; set mos($my,yc) $y1
                    set mos($my,xg) $x2; set mos($my,yg) $y2
                }
                s {
                    # which point is closer ?
                    # set da [expr $mos($my,xb)-$x1]; set da [expr $da*$da]
                    # set d1 [expr $mos($my,yb)-$y1]; set d1 [expr $d1*$d1]
                    # set da [expr $da+$d1]
                    # set db [expr $mos($my,xc)-$x1]; set db [expr $db*$db]
                    # set d1 [expr $mos($my,yc)-$y1]; set d1 [expr $d1*$d1]
                    # set db [expr $db+$d1]
                    # if {$da<$db}
                    switch -exact -- $but {
                        1 {
                            # take MOS A
                            set mos($my,xb) $x1; set mos($my,yb) $y1
                            set mos($my,xf) $x2; set mos($my,yf) $y2
	                    set sid a
                        }
                        0 {
                           # take MOS B
                            set mos($my,xc) $x1; set mos($my,yc) $y1
                            set mos($my,xg) $x2; set mos($my,yg) $y2
	                    set sid b
                        }
                    }
                }
                default {
                    puts $lfId "MoveBlade: Warning"
                }
            }

            # set mos($my,xb) $x1; set mos($my,yb) $y1
            # set mos($my,xc) $x2; set mos($my,yc) $y2
            # set mos($my,xf) $x3; set mos($my,yf) $y3
            # set mos($my,xg) $x4; set mos($my,yg) $y4
        }
        # the new slitwidth
        set xdum [expr $mos($my,xb) - $mos($my,xc)]
        set ydum [expr $mos($my,yb) - $mos($my,yc)]
        $m_image convert dist $xdum $ydum canvas x y "deg $EQX"
        set mos($my,b) [format "%3.1f" [expr sqrt($x*$x*1.296e7+$y*$y*1.296e7)]]
        # puts stdout "Width #$my = $mos($my,b) arcsec   \
        #         = [expr $mos($my,b) * $focscale] mm"
        set targw($my,a) [expr ($mos($my,xc)+$mos($my,xb)+$mos($my,xf)+$mos($my,xg))/4.0]
        set targw($my,d) [expr ($mos($my,yc)+$mos($my,yb)+$mos($my,yf)+$mos($my,yg))/4.0]
        # targw in wcs
        # ------------
        catch { $m_image convert coords $targw($my,a) $targw($my,d) canvas targw($my,a) targw($my,d) "wcs $EQX" }
        # scratch the whole old mask and plot_mask it
        # tcs-label needs not to be updated
        catch {$w delete A}
        plot_mask
        if { $badpixelmap } {
            plot_BPM
        }
        set last_action "Move MOS \#$my[string toupper  $sid]"
        catch {SP update}
    }
    tracer LEAVING
}

#=========================================
public method open_blades { {resol SR} } {
#=========================================
# By P. Baksai

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # mos(i,b) in arcsec, focscale = mm_to_as
    # mos(i,b)/mm_to_as/ctom = canvas
    # For MOS, all slits are still in the park position, with common slit width
    for { set i 1 } { $i < 20 } { incr i 1 } {
        set mos($i,yb) $mos($i,ya)
        set mos($i,xb) $mos($i,xa)
        set mos($i,yf) $mos($i,ye)
        set mos($i,xf) $mos($i,xe)
        set mos($i,yc) $mos($i,yd)
        set mos($i,xc) $mos($i,xd)
        set mos($i,yg) $mos($i,yh)
        set mos($i,xg) $mos($i,xh)
    }

    set lmod [$this cget -modus]
    switch -exact -- $lmod {
        echs {
        }
        hitms {
            # Do nothing
        }
        hits {
            # Do nothing, since hits is a fixed mask, nothing to open
        }
        img {
            if { [string match *HR* $resol] } {
                # Close the outside slit blades
                foreach i { 1 2 3 4 17 18 19 } {
                    set mos($i,yc) $mos($i,ya)
                    set mos($i,xc) $mos($i,xa)
                    set mos($i,yg) $mos($i,ye)
                    set mos($i,xg) $mos($i,xe)
                }
            }
        }
        ipol {
            if { [string match *SR* $resol] } {
            } else {
            }
        }
        lss {
            # Do nothing, since lss is a fixed mask, nothing to open
        }
        mos {
            if { [string match *SR* $resol] } {
            } else {
            }
        }
        mxu  {
            # Do nothing
        }
        pmos {
            # Close the odd slit blades
            for { set i 1 } { $i < 20 } { incr i 2 } {
                set mos($i,yb) $mos($i,ya)
                set mos($i,xb) $mos($i,xa)
                set mos($i,yf) $mos($i,ye)
                set mos($i,xf) $mos($i,xe)
                set mos($i,yc) $mos($i,ya)
                set mos($i,xc) $mos($i,xa)
                set mos($i,yg) $mos($i,ye)
                set mos($i,xg) $mos($i,xe)
            }
        } 
        default {
            puts $lfId "open_blades: Warning, unknown modus >$lmod<"
        }
    }
    erase
    plot_mask
    tracer LEAVING
}

#==========================================
public method close_blades { {resol SR} } {
#==========================================
# By P. Baksai

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    for { set i 1 } { $i < 20 } { incr i 1 } {
        set mos($i,yb) $mos($i,ya)
        set mos($i,xb) $mos($i,xa)
        set mos($i,yf) $mos($i,ye)
        set mos($i,xf) $mos($i,xe)
        set mos($i,yc) $mos($i,ya)
        set mos($i,xc) $mos($i,xa)
        set mos($i,yg) $mos($i,ye)
        set mos($i,xg) $mos($i,xe)
    }
    erase
    plot_mask
    tracer LEAVING
}

#====================================
public method adjust_slit_length {} {
#====================================
# Reduces the mos slit length by s_slit in mm
# Call this after load_instrument but before init_mask_world2
# transpose slit_length from '' to mm and adjust mosslit...
# Don't call this when -slit_length = real

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    if { [string compare [$this cget -modus] "mos"] == 0 } {
        if { [string compare $slit_length "real"] } {
            # mos, not real
            for { set i 2 } { $i < 19 } { incr i 1 } {
                set delta [expr {($mosslitmax($i) - $mosslitmin($i)  \
                                  - $slit_length / $focscale) / 2}]
                set mosslitmin($i) [expr {$mosslitmin($i) + $delta}]
                set mosslitmax($i) [expr {$mosslitmax($i) - $delta}]
            }
            # Special handling of the rim blades
            set delta [expr {($mosslitmax(1) - $mosslitmin(1)  \
                       - ($slit_length-12.0) / $focscale) / 2}]
            set mosslitmin(1)  [expr {$mosslitmin(1) + $delta}]
            set mosslitmax(1)  [expr {$mosslitmax(1) - $delta}]
            set delta [expr {($mosslitmax(19) - $mosslitmin(19)  \
                        - ($slit_length-12.0) / $focscale) / 2}]
            set mosslitmin(19) [expr {$mosslitmin(19) + $delta}]
            set mosslitmax(19) [expr {$mosslitmax(19) - $delta}]
        } else {
            puts stdout "adjust_slit_length 1: Error, 'real' cannot be handled"
        }
    } elseif { [string compare [$this cget -modus] "pmos"] == 0 } {
        # We assume this can only be modus pmos
        if { [string compare $slit_length "real"] } {
            # pmos, not real
            for { set i 2 } { $i < 20 } { incr i 2 } {
                set delta [expr {($mosslitmax($i) - $mosslitmin($i)  \
                                  - $slit_length / $focscale) / 2}]
                set mosslitmin($i) [expr {$mosslitmin($i) + $delta}]
                set mosslitmax($i) [expr {$mosslitmax($i) - $delta}]
            }
            # The rim blades are closed, no special handling
        } else {
            puts stdout "adjust_slit_length 2: Error, 'real' cannot be handled"
        }
    } else {
        # PBG: nothing...for the time being
    }
    tracer LEAVING
}

#=======================
public method noars {} {
#=======================
# Returns the number of all reference mxu slits

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set rs_c [fors::MXU_SLIT::noars]
    tracer LEAVING
    return $rs_c
}

#======================
public method noas {} {
#======================
# Returns the number of all non-deleted mxu slits

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set s_c [fors::MXU_SLIT::noas]
    tracer LEAVING
    return $s_c
}

#========================
private method erase {} {
#========================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    $m_canvas delete $this -tag A
    $m_canvas delete $this -tag D
    $m_canvas delete $this -tag REFTARG
    tracer LEAVING
}

#================================
private method mark { w xs ys } {
#================================
# Marks mask pointing

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    catch { $m_image convert coords $xs $ys screen oldpos(x) oldpos(y) canvas }
    tracer LEAVING
}

#====================================
private method rotate { w x y dir } {
#====================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $stop } {
        tracer LEAVING STOP
        return
    }

    # Rotate the whole mask with all slits
    # del_ang is incremental rotation of the object
    # default + or - 5 [deg] rotangle

    if { $dir < 0 } {
        set sign "-"
    } else {
        set sign "+"
    }
    # rotangle in deg, del_ang in rad
    set del_ang [expr $dir*$rotangle*$deg2rad]
    # puts $lfId "RotAng >>$rotangle<< DelAng >>$del_ang<<"
    # abs_ang_rad is the current rot-angle with respect to the
    # initial object, = mask-wcs angle from north to east
    set abs_ang_rad [expr $abs_ang_rad + $del_ang]
    set PAM [expr $PAM + $dir*$rotangle*$CSO]
    set PAM [expr fmod($PAM,360)]
    set tcs_pam [format "%6.2f" [expr $PAM * -1]]
    # set myob($w,obj) [$w find closest $x $y]

    puts stdout ""
    puts stdout "Rotate at ($world_x, $world_y) by $sign$rotangle \[deg\] to PA = $tcs_pam"
    puts $lfId "rotate at $world_x,$world_y to PA = $tcs_pam"

    # New coordinates
    # We rotate the all coordinates around the axis
    # located in the center of the figure = ccx, ccy
    # The angle step is del_angle
    # The new total angle = abs_ang_rad
    if { [string compare $mask "hitms"] == 0 } {
        set local_max 0
        set local_set {a b d e f h}
        # like mxu
        # later we will use the mxu internal counter
    } elseif { [string compare $mask "mos"] == 0 } {
        set local_max 20
        set local_set {a b c d e f g h }
    } elseif { [string compare $mask "mxu"] == 0 } {
        set local_max 0
        set local_set {a b d e f h}
        # later we will use the mxu internal counter
    } else {
        set local_max 10
        set local_set {a b d e f h}
    }

    #---------------------------
    # Rotate the mos-slit blades
    #---------------------------
    set cda [expr {cos($del_ang)}]
    set sda [expr {sin($del_ang)}]
    # Rotation via matrix multiplication
    for { set i 1 } { $i < $local_max } { incr i 1 } {
        foreach val $local_set {
            set d1 [expr {$ccx + ($mos($i,x$val) - $ccx)*$cda - ($mos($i,y$val) - $ccy)*$sda}]
            set d2 [expr {$ccy + ($mos($i,x$val) - $ccx)*$sda + ($mos($i,y$val) - $ccy)*$cda}]
            set mos($i,x$val) $d1
            set mos($i,y$val) $d2
        }
    }

    # Update the 19 slit-positions of the mos
    if { [string compare $mask "mos"] == 0 } {
        for { set i 1 } { $i < $local_max } { incr i 1 } {
            # close the unused mos slit blades again in MOS/PMOS
            # keep them open in IMG IPOL ECHS (mask=mos)
            # exception : MOS slit-les spectroscopy
            if { [string match "*mos" [$this cget -modus]] } {
                if { [string compare "open" [$this cget -slit_width]] } {
                    # Deselect all mos slits
                    set targw($i,f) 0
                    set mos($i,xb) $mos($i,xa)
                    set mos($i,yb) $mos($i,ya)
                    set mos($i,xc) $mos($i,xa)
                    set mos($i,yc) $mos($i,ya)
                    set mos($i,xg) $mos($i,xe)
                    set mos($i,yg) $mos($i,ye)
                    set mos($i,xf) $mos($i,xe)
                    set mos($i,yf) $mos($i,ye)
                }
            } else {
                # img, ipol, echs
                # mean value
                set A [expr ($mos($i,xc)+$mos($i,xb)+$mos($i,xf)+$mos($i,xg))/4.0]
                set B [expr ($mos($i,yc)+$mos($i,yb)+$mos($i,yf)+$mos($i,yg))/4.0]
                if {$targw($i,f) != 0 } {
                    catch { $m_image convert coords $A $B canvas targw($i,a) targw($i,d) "wcs $EQX" }
                }
            }
        }
    }

    # Rotate the frames
    foreach k {a b c d e f g h ar br cr dr er fr gr hr stl sbl sel swl htl hbl hel hwl slbl slrl hlbl hlrl sb1s sb1f sb2s sb2f sr1s sr1f sr2s sr2f hb1s hb1f hb2s hb2f hr1s hr1f hr2s hr2f} {
        set d1 [expr {$ccx + ($mos(0,x$k) - $ccx)*$cda - ($mos(0,y$k) - $ccy)*$sda}]
        set d2 [expr {$ccy + ($mos(0,x$k) - $ccx)*$sda + ($mos(0,y$k) - $ccy)*$cda}]
        set mos(0,x$k) $d1
        set mos(0,y$k) $d2

        set ccdm_d1 [expr $ccx + ($ccd_frame(0,x$k) - $ccx)*$cda - ($ccd_frame(0,y$k) - $ccy)*$sda]
        set ccdm_d2 [expr $ccy + ($ccd_frame(0,x$k) - $ccx)*$sda + ($ccd_frame(0,y$k) - $ccy)*$cda]
        set ccd_frame(0,x$k) $ccdm_d1
        set ccd_frame(0,y$k) $ccdm_d2

        set ccds_d1 [expr $ccx + ($ccd_frame(1,x$k) - $ccx)*$cda - ($ccd_frame(1,y$k) - $ccy)*$sda]
        set ccds_d2 [expr $ccy + ($ccd_frame(1,x$k) - $ccx)*$sda + ($ccd_frame(1,y$k) - $ccy)*$cda]
        set ccd_frame(1,x$k) $ccds_d1
        set ccd_frame(1,y$k) $ccds_d2
    }

    # Update echs slits targw
    # if { [string compare echs [$this cget -modus]] == 0 } {
    #     $m_image convert coords $A $B canvas targw(10,a) targw(10,d) "wcs $EQX"
    # }

    if { [string compare $mask "hitms"] == 0 ||
         [string compare $mask "mxu"] == 0 } {
        # Update mxu focal field <-> canvas transformation matrix
        fors::MXU_SLIT::coeff $XSRMIN $YSRMIN $mos(0,xa) $mos(0,ya) \
                              $XSRMAX $YSRMIN $mos(0,xb) $mos(0,yb) \
                              $XSRMAX $YSRMAX $mos(0,xc) $mos(0,yc)
        if { [fors::MXU_SLIT::get_mmff] } {
            # Do nothing for standard masks
        } else {
            # Delete all hitms, mxu slits if there are any,
            # and keep reference slits in MXU
            # PBG 20090731 mxu_delete_all_slits 1
        }
        fors::MXU_SLIT::set_mmff 0
    }

    # Delete the object and plot_mask it
    catch {$w delete A}
    catch {$w delete D}
    plot_mask

    if { $badpixelmap } {
        update_BPM
    }

    # puts stdout [format "Rotate Mask around %5.1f  %5.1f from \
    #                      %3.1f by %3.1f to %3.1f  (degr)" \
    #                     $ccx $ccy \
    #                     [expr $abs_ang_rad/$deg2rad-$dir*$rotangle] \
    #                     [expr $dir*$rotangle] \
    #                     [expr $abs_ang_rad/$deg2rad]]

    #  puts stdout [format "Rotate Mask around %5.1f  %5.1f from \
    #                       %5.1f by %5.1f to %5.1f  (PA )" \
    #                      $ccx $ccy \
    #                      [expr $PAM-$dir*$rotangle*$CSO] \
    #                      [expr $dir*$rotangle*$CSO] \
    #                      [expr $PAM]]

    set last_action "Rotate Mask"
    catch {SP update}
    tracer LEAVING
}

#=================================
private method shift { w xs ys } {
#=================================
# A) shifts the mask via tcl move command
# B) updates the mos-array
# xs,ys are the screen coords where to be shifted
#
# Comments:
# declare bord as soon as possible as global
# not yet moved outside frame

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $stop } {
        tracer LEAVING STOP
        return
    }
    set over 0

    # Convert last drag coord to pixels
    catch { $m_image convert coords $xs $ys screen xi yi image }

    # puts stdout "pixel ($xi,$yi) screen ($xs,$ys) ($width,$height)"
    # Border in image coords
    if { $bonsai_flag} {
        set over 1
    }
    if { $xi < 1 || $xi >= $width } {
        set over 1
    }
    if { $yi < 1 || $yi >= $height } {
        set over 1
    }

    # Screen to canvas
    catch { $m_image convert coords $xs $ys screen x y canvas }

    # mark and shift position != mask center
    # The distance between old and new position in canvas coordinates
    set shift_x [expr {$x - $oldpos(x)}]
    set shift_y [expr {$y - $oldpos(y)}]

    # Moves the mask on the screen without recreating it via plot_mask
    $w move A  $shift_x $shift_y

    # Update the old-position for permanent shifts
    set oldpos(x) $x
    set oldpos(y) $y

    # Apply shift mask center = telescope pointing (canvas)
    set ccx [expr {$ccx + $shift_x}]
    set ccy [expr {$ccy + $shift_y}]

    # Update world coords of mask center
    # If the frame is smaller than the mask or the mask is
    # shifted outside the frame use bonsai-wcs
    if { $over } {
        set dum [C_wc c2w $ccx $ccy]
        set world_x [lindex $dum 0]
        set world_y [lindex $dum 1]
    } else {
        if { [catch { $m_image convert coords $ccx $ccy canvas world_x world_y "deg $EQX" } result] } {
            puts stdout "shift: Big Error!!!!!!!!!!!!! $result"
        }

    }

    # Since the fors instrumental mask can be rotated by
    # a minimum amount of 1 deg, we have to keep an internal
    # array containing all mask coords

    # Update the mos
    # 19 slit blades
    set local_max 10
    set local_set {a b d e f h}
    if { [string compare $mask "mos"] == 0 } {
        set local_max 20
        set local_set {a b c d e f g h}
    }
    if { [string compare $mask "hitms"] == 0 ||
         [string compare $mask "mxu"] == 0 } {
        set local_max 0
        # later we will ask for the hitms or mxu local slit counter
    }

    # Update the MOS slit blades
    for { set i 1 } { $i < $local_max } { incr i 1 } {
        foreach val $local_set {
            set mos($i,x$val) [expr {$mos($i,x$val) + $shift_x}]
            set mos($i,y$val) [expr {$mos($i,y$val) + $shift_y}]
        }
    }

    # Update the ccd center
    set mos(0,x0) [expr {$mos(0,x0) + $shift_x}]
    set mos(0,y0) [expr {$mos(0,y0) + $shift_y}]
    set ccd_frame(0,x0) [expr {$ccd_frame(0,x0) + $shift_x}]
    set ccd_frame(0,y0) [expr {$ccd_frame(0,y0) + $shift_y}]
    set ccd_frame(1,x0) [expr {$ccd_frame(1,x0) + $shift_x}]
    set ccd_frame(1,y0) [expr {$ccd_frame(1,y0) + $shift_y}]
    # Update the ccd frame including collimator and wavelength borders
    foreach k {a b c d e f g h ar br cr dr er fr gr hr stl sbl sel swl htl hbl hel hwl slbl slrl hlbl hlrl sb1s sb1f sb2s sb2f sr1s sr1f sr2s sr2f hb1s hb1f hb2s hb2f hr1s hr1f hr2s hr2f} {
        set mos(0,x$k) [expr {$mos(0,x$k) + $shift_x}]
        set mos(0,y$k) [expr {$mos(0,y$k) + $shift_y}]
        set ccd_frame(0,x$k) [expr {$ccd_frame(0,x$k) + $shift_x}]
        set ccd_frame(0,y$k) [expr {$ccd_frame(0,y$k) + $shift_y}]
        set ccd_frame(1,x$k) [expr {$ccd_frame(1,x$k) + $shift_x}]
        set ccd_frame(1,y$k) [expr {$ccd_frame(1,y$k) + $shift_y}]
    }

    # This must be correct, since we can put up the mask
    # not only in the center of the green point, but also
    # at some biased position.
    # Cursor position, note that canvas-wcs canvas-deg does not work
    # outside frame
    # $m_image convert coords $xs $ys screen lwx lwy "wcs $EQX"

    # Cursor position, not TCS pointing
    catch { $m_image convert coords $xs $ys screen xc yc canvas }
    set dum [C_wc c2w $xc $yc]
    set xw [lindex $dum 0]
    set yw [lindex $dum 1]

    if { [catch { catch { $m_image convert coords $xw $yw "deg $EQX" lcx lcy "wcs $EQX" }
          } result] } {
        puts stdout "shift: $result"
    }

    # mask-tcs position (= green point center)
    # $m_image convert coords $world_x $world_y "deg $EQX" lwx lwy "wcs $EQX"
    # puts stdout [format "Shift Mask to c(%10.3f,%10.3f) (%10.3f,%10.3f)" \
    #                     $mos(0,x0) $mos(0,y0) $world_x $world_y]
    # puts stdout [format "T%10s %10s C%10s %10s c%6.1f %6.1f s%6.1f %6.1f" \
    #                     $lwx $lwy $lcx $lcy $x $y $xs $ys]

    # Renew tcs and ins labels
    update_labels

    # And the badpixelmap
    if { $badpixelmap } {
        update_BPM
    }
    tracer LEAVING
}

#=======================
private method warn {} {
#=======================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set number [random 7]
    switch -exact -- $number {
        0 -
        1 {
            set result 1
            while { $result == 0 } {
                set wid .skycat1.save
                catch {destroy $wid}
                set w [DialogWidget $wid \
                                    -text "Are you Really Shure ?" \
                                    -bitmap warning \
                                    -default 0  \
                                    -buttons {YES NO}\
                      ]
                set result [$w activate]
            }
        }
        2 {
            info_dialog "You have pressed \[Ok\]\n\nPlease verify" .skycat1
            info_dialog "You have pressed \[Ok\]\n\nPlease verify" .skycat1
            info_dialog "\[Ok\] Accepted" .skycat1
            info_dialog "\[Ok\] Not Accepted" .skycat1
            info_dialog "You have pressed \[Ok\]\n\nPlease verify" .skycat1
        }
        3 {
            expire_date
        }
        4 {
            info_dialog "Not Accepted\n\n\Press again \[YES\]" .skycat1
            info_dialog "Not Accepted\n\n\Press again \[NO\]" .skycat1
            info_dialog "Not Accepted\n\n\Press again \[OK\]" .skycat1
        }
        5 {
            info_dialog "No Save Permission for\n\n\User: [exec whoami]\n\n\Press \[NO\]" .skycat1
            info_dialog "No Save Permission for\n\n\User: [exec whoami]\n\n\Press \[NO\]" .skycat1
        }
        6 {
            for { set j 1 } { $j < 5 } { incr j 1 } {
                set text "Agreement:\n\n\
                          This setup file\n\
                          remains a property of ESO\n\
                          even if never scheduled."
                set result 1
                set haha [random 3]
                while { $result != $haha } {
                    set wid .skycat1.save
                    catch {destroy $wid}
                    set w [DialogWidget $wid \
                                        -text $text  \
                                        -bitmap warning \
                                        -default 0 \
                                        -buttons {YES OK Accept Thanks} \
                          ]
                    set result [$w activate]
                }
            }
        }
    }
    tracer LEAVING
}

#===============================
private method catalog { a_c } {
#===============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # 1) get the target list from the Astrocat widget
    # 2) compare entries with the list of slit positions
    #    just selected
    # 3) delete the selected entries in the target list
    # 4) put reduced target list back to AstroCat widget
    # use two times the MOS slit width
    # puts stdout "in catalog with $a_c"
    if {[string compare [$this cget -slit_width] "open"]==0} {
        set error_box 22.7
    } else {
        set error_box [expr [$this cget -slit_width] * 2.0]
    }
    set error_box [expr $error_box * 2.7777778e-4]
    puts $lfId "CatalogInterface: ErrorBox =  \
                [expr $error_box / 2.7777778e-4] ''"
    # puts stdout "----------------------------"
    # if you use more than one catalog

    # puts stdout "###->$a_c info class    = [$a_c info class]"
    # puts stdout "###->$a_c info inherit  = [$a_c info inherit]"
    # puts stdout "###->$a_c info heritage = [$a_c info heritage]"
    #----list the functions
    # set f [$a_c info function]
    # set f [split $f " "]
    # set l [llength $f]
    # for { set i 0 } { $i < $l } { incr i 1 } {
    #     if { [string match [lindex $f $i] *rem*] == 1 } {
    #         puts stdout [lindex $f $i]
    #     }
    # }
    #---list the variables
    # set f [$a_c info variable]
    # set f [split $f " "]
    # set l [llength $f]
    # for { set i 0 } { $i < $l } { incr i 1 } {
    #     if { [string length [$a_c info variable [lindex $f $i]]] < 100 } {
    #          puts stdout [$a_c info variable [lindex $f $i]]
    #     }
    # }
    # puts stdout "###->$a_c configure = [$a_c configure]"
    # puts stdout "###->$a_c itcl::find objects -class AstroCat = \
    #             [$a_c itcl::find objects -class AstroCat]"
    # puts stdout "###->$a_c info variable n_instances_ = \
    #             [$a_c info variable n_instances_]"
    # puts stdout "###->$a_c info variable instances_ = \
    #             [$a_c info variable instances_]"
    # puts stdout "###->$a_c info variable astrocat_ = \
    #             [$a_c info variable astrocat_]"
    # puts stdout "###->$a_c info variable instance_idx_ = \
    #             [$a_c info variable instance_idx_]"
    # puts stdout "###->$a_c info variable catalog_menu_info_ = \
    #             [$a_c info variable count_]"
    # puts stdout "###->$a_c info variable current_instance_ = \
    #             [$a_c info variable current_instance_]"
    #

    # puts stdout "###->$a_c cget              = [$a_c cget]"
    # puts stdout "###->$a_c cget -image       = [$a_c cget -image]"
    # puts stdout "###->$a_c cget -debug       = [$a_c cget -debug]"
    # puts stdout "###->$a_c cget -catalog     = [$a_c cget -catalog]"
    # puts stdout "###->$a_c cget -catalogtype = [$a_c cget -catalogtype]"
    # puts stdout "###->$a_c cget -tcs         = [$a_c cget -tcs]"
    # puts stdout "###->$a_c cget -transient   = [$a_c cget -transient]"
    # puts stdout "###->$a_c cget -center      = [$a_c cget -center]"
    # puts stdout "###->$a_c info function remove_FORS = \
    #             [$a_c info function remove_FORS]"
    # puts stdout "###->$a_c info function save_as = \
    #             [$a_c info function save_as]"
    # puts stdout "###->$a_c info variable info_ = \
    #             [$a_c info variable info_]"
    # puts stdout "###->$a_c  = [$a_c get_equinox]"
    # puts stdout "###->$a_c  = [$a_c select_columns]"
    # puts stdout "###->$a_c  = [$a_c select_result_row]"

    # there is no way to get the 'highlight objects from a catalog
    # maybe we have to plugin something to overwrite a SkyCat Method

    # 1) get the target list from the AstroCat widget
    # default, the first created AstroCat instance
    # puts stdout [set ac [itcl::find objects -class ::skycat::SkyCat]]
    # search the current AstroCat instance

    set cur [$a_c info variable current_instance_]
    set hea [$a_c info variable headings_]
    set hea [lindex $hea 4]
    # puts stdout "cur = >>$cur<<"
    # puts stdout "hea = >>$hea<<"
    # set cur [lindex $cur 4]
    # puts stdout "cur = >>$cur<<"
    # set current AstroCat instance to the last
    # cerated AstroCat instance
    # this could be chnaged by a user-defined instance
    # selected by a menu button ??
    # set a_c cat::AstroCat::$cur
    set filename [lindex [$a_c info variable instance_idx_] 4]
    set filename [split $filename ","]
    set filename [lindex $filename 0]

    # puts stdout $filename
    # puts stdout "###->$a_c cget -catalog     = [$a_c cget -catalog]"
    # puts stdout "###->CatalogInterface: catname=$a_c"
    # puts stdout "###->CatalogInterface: filename= \
    # [lindex [$a_c info variable instance_idx_] 4]"

    puts $lfId "CatalogInterface: catname  = $a_c"
    puts $lfId "CatalogInterface: filename = $filename"

    # puts stdout "CatalogInterface: catname  = $a_c"
    # puts stdout "CatalogInterface: filename = $filename"
    set CatFileName $filename
    set loc_list [$a_c info variable info_]
    if { $debug } {
        puts stdout ">>$loc_list<<"
    }
    set loc_list [lindex $loc_list 4]
    # puts stdout ">>$loc_list<<"

    # scratch those targets in the catalog
    # which are in mos

    # 2) compare the target list with the just selected slit positions
    # ----------------------------------------------------------------
    if { $debug } {
        puts stdout "CatalogInterface: search in list ... "
    }
    set found_list ""
    set rest_list $loc_list
    set work_list $loc_list
    set found_counter 0
    # puts stdout "LOC_LIST  = >>$loc_list<<"
    # puts stdout "FOUND_LIST= >>$found_list<<"
    # puts stdout "REST_LIST = >>$rest_list<<"

    if { [string compare [$this cget -modus] hitms] == 0 ||
         [string compare [$this cget -modus] mxu] == 0 } {
        set max [fors::MXU_SLIT::nos]
    } else {
        # this should be MOS
        set max 19
    }

    for { set i 1 } { $i <= $max } { incr i 1 } {
        # all 19 slit blades, or all hitms, mxu-slits
        if { [string compare [$this cget -modus] hitms] == 0 ||
             [string compare [$this cget -modus] mxu] == 0 } {
            set on [C_mxus$i cget -ON]
        } else {
            # this should be MOS
            set on  $targw($i,f)
        }

        if { $on } {
            if { $debug } {
                puts stdout "search for \#$i with $on"
            }
            # if the slit is explicitely selected
            # random coincidence with unused slits is not accepted
            if { [string compare [$this cget -modus] hitms] == 0 ||
                 [string compare [$this cget -modus] mxu] == 0 } {
                set loc_RA  [C_mxus$i get_RA]
                set loc_DEC [C_mxus$i get_DEC]
            } else {
                # this should be MOS
                set loc_RA  $targw($i,a)
                set loc_DEC $targw($i,d)
            }
            catch { $m_image convert coords $loc_RA  $loc_DEC "wcs $EQX" aa dd "deg $EQX" }
            # aa dd are RA DEC of selected slit in deg
            if { $debug } {
                puts stdout "\#$i $loc_RA $loc_DEC -> ($aa,$dd)"
            }
            # now search in the target list for coord coincidence
            set list_counter 0
            set free 1
            foreach star $work_list {
                if { $free } {
                    if { $debug } {
                       puts stdout $star
                       puts stdout "[lindex $star 1] [lindex $star 2]"
                    }
                    catch { $m_image convert coords [lindex $star 1] [lindex $star 2] \
                                                    "wcs $EQX" alp dec "deg $EQX"
                    }
                    if { $debug } {
                         puts stdout "a=$alp d=$dec"
                    }
                    # alp dec are RA DEC of current test target
                    # calculate distance between target and cat entry
                    # assume simple linear scale, and constant cos(dec)
                    set dec [expr $dec - $dd]
                    set alp [expr ($alp - $aa)*cos($dd)]
                    if { $debug } {
                        puts stdout "Da= [expr $alp*3600]''  Dd=[expr $dec*3600]''"
                    }
                    set dec [expr $dec*$dec]
                    set alp [expr $alp*$alp]
                    set dec [expr sqrt($dec+$alp)]
                    if { $dec < $error_box } {
                       # coord coincidence !!!!!!!!!!!!
                       puts $lfId "CI: \#$i ($loc_RA,$loc_DEC) ($aa,$dd) with \
                                   [lindex $star 0] ([lindex $star 1],[lindex $star 2]) \
                                   within [expr $dec*3600] ''"
                       set rest_list [lreplace $work_list $list_counter $list_counter]
                       if { $debug } {
                           puts stdout "[llength $work_list] - 1 = [llength $rest_list]"
                       }
                       #  element $list_c replaced by nothing --> reduced list
                       # transfer the target name
                       if { [string compare [$this cget -modus] hitms] == 0 ||
                            [string compare [$this cget -modus] mxu] == 0 } {
                           C_mxus$i configure -target_name [lindex $star 0]
                       } else {
                           set targw($i,f) [lindex $star 0]
                           set targw($i,f) [string_exchange $targw($i,f) " " "_"]
                       }
                       # puts stdout $targw($i,f)
                       # and add to found list
                       lappend found_list [lindex $work_list $list_counter]
                       incr found_counter 1
                       set free 0
                       # dont assign further names to slit
                    } ; # end coincidence
                    incr list_counter 1
                } ; # free
            } ; # list loop (foreach star)
            # foreach star
            # Update the working list, reduce it by 1
            set work_list $rest_list
            # keep this for later purposes
            # set index [lsearch $star high]
            # if { $index == -1 } {
            #     set index [lsearch $star low]
            # }
            # if { $index > 1 } {
            #     set prio [lindex $star $index]
            # }
        }
        if { [string compare [$this cget -modus] hitms] == 0 ||
             [string compare [$this cget -modus] mxu] == 0 } {
            puts stdout [format "CatalogInterface: Slit \#%2d %s" \
                                $i \
                                [C_mxus$i cget -target_name] \
                        ]
        } else {
            puts stdout [format "CatalogInterface: Slit \#%2d %s" \
                                $i \
                                $targw($i,f) \
                        ]
        }
    }
    # puts stdout [$a_c component results remove_selected]
    # puts stdout [$a_c search]
    # puts stdout "LOC_LIST  = >>$loc_list<<"
    # puts stdout "FOUND_LIST= >>$found_list<<"
    # puts stdout "REST_LIST = >>$rest_list<<"
    puts stdout "CatalogInterface: initial : [llength $loc_list], \
                 found-and-purged : [llength $found_list], \
                 remaining : [llength $rest_list]"
    puts $lfId "CatalogInterface: $found_counter objetcs identified"
    # puts stdout "[llength $loc_list] - [llength $found_list] + [llength $rest_list]"

    # 5 delete from the AstroCat current local catalog
    # puts stdout "a_c = $a_c"
    # puts stdout "###->$a_c info class    = [$a_c info class]"
    # puts stdout "###->$a_c info inherit  = [$a_c info inherit]"
    # puts stdout "###->$a_c info heritage = [$a_c info heritage]"
    # puts stdout "###->$a_c info function = [$a_c info function]"
    # puts stdout "###->$a_c info variable = [$a_c info variable]"
    # puts stdout "###->$a_c itcl::find objects -class AstroCat = \
    #              [$a_c itcl::find objects -class AstroCat]"

    # 4) delete, the found_list from the target_list
    set a [$a_c query_done "" $hea $rest_list 0]

    # puts stdout [set ll [llength $rest_list]]
    # for { set i 0 } { $i < $ll } { incr i 1 } {
    #     puts stdout [lindex $rest_list $i]
    # }

    puts stdout "CatalogInterface: end"
    tracer LEAVING
}

#====================================================
private method calc_ctom { {frame FOC} {resol SR} } {
#====================================================
# Re-implemented by P. Baksai 20090622

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "calc_ctom: BEGIN $frame $resol"
    }
    # Select area (mm)
    switch -exact -- $frame {
        FOC {
            set XSMIN $XSRMIN
            set YSMIN $YSRMIN
            set XSMAX $XSRMAX
            set YSMAX $YSRMAX
            set XHMIN $XHRMIN
            set YHMIN $YHRMIN
            set XHMAX $XHRMAX
            set YHMAX $YHRMAX
        }
        MASTER {
            set XSMIN $CCDMASTER_XSRMIN
            set YSMIN $CCDMASTER_YSRMIN
            set XSMAX $CCDMASTER_XSRMAX
            set YSMAX $CCDMASTER_YSRMAX
            set XHMIN $CCDMASTER_XHRMIN
            set YHMIN $CCDMASTER_YHRMIN
            set XHMAX $CCDMASTER_XHRMAX
            set YHMAX $CCDMASTER_YHRMAX
        }
        SLAVE {
            set XSMIN $CCDSLAVE_XSRMIN
            set YSMIN $CCDSLAVE_YSRMIN
            set XSMAX $CCDSLAVE_XSRMAX
            set YSMAX $CCDSLAVE_YSRMAX
            set XHMIN $CCDSLAVE_XHRMIN
            set YHMIN $CCDSLAVE_YHRMIN
            set XHMAX $CCDSLAVE_XHRMAX
            set YHMAX $CCDSLAVE_YHRMAX
        }
    }

    # Select resolution parameters (mm)
    if { [string match *SR* $resol] } {
        # SR
        set XMIN $XSMIN
        set YMIN $YSMIN
        set XMAX $XSMAX
        set YMAX $YSMAX
        if { $debug && $verbose } {
            puts stdout "calc_ctom: Standard Resolution $frame"
        }
    } else {
        # HR
        set XMIN $XHMIN
        set YMIN $YHMIN
        set XMAX $XHMAX
        set YMAX $YHMAX
        if { $debug && $verbose } {
            puts stdout "calc_ctom: High Resolution $frame"
        }
    }
    if { $debug && $verbose } {
        puts stdout "           XMIN = $XMIN"
        puts stdout "           YMIN = $YMIN"
        puts stdout "           XMAX = $XMAX"
        puts stdout "           YMAX = $YMAX"
    }

    # Select scale parameters
    # numeric arcmin_to_degree atd = 2.7777778E-4
    # x [mm] * $focscale = y [arcsec]  (mtoas)
    # coord/mm * scal = coord/arcsec
    set scal [expr {$atd * $focscale}]
    if { $debug && $verbose } {
        puts stdout "           scal = atd * focscale = $scal"
        puts stdout "       focscale = $focscale"
        puts stdout "            atd = $atd"
    }

    # Select reference points p1 and p2 (arcsec)
    # on the central north-south line (geographical reference):
    # p1 lies between MOS blades A and B at x=0 but south
    # p2 lies between MOS blades A and B at x=0 but north
    # Lower southern point (near MOS 1, north of sky in focal field)
    set p1x $wx
    set p1y [expr {$wy + $scal*$YMIN}]
    # Upper northern point (near MOS 19, south of sky in focal field)
    set p2x $wx
    set p2y [expr {$wy + $scal*$YMAX}]

    if { $debug && $verbose } {
        puts stdout         "           WCS coords of points in South-North geographical line"
        puts stdout         "                       South                          North"
        puts stdout [format "           %6s_1%s(%8.2f,%8.2f)  %6s_2%s(%8.2f, %8.2f)" $frame $resol $p1x $p1y $frame $resol $p2x $p2y]
    }

    # Transform these points into canvas coords (pixel)
    if { $bonsai_flag } {
        set dum [C_wc w2c $p1x $p1y]
        set p1x [lindex $dum 0]
        set p1y [lindex $dum 1]
        set dum [C_wc w2c $p2x $p2y]
        set p2x [lindex $dum 0]
        set p2y [lindex $dum 1]
        if { $debug && $verbose } {
            puts stdout "calc_ctom: Bonsai matrix used"
        }
    } else {
        catch { $m_image convert coords $p1x $p1y "deg $EQX" p1x p1y canvas }
        catch { $m_image convert coords $p2x $p2y "deg $EQX" p2x p2y canvas }
        if { $debug && $verbose } {
            puts stdout "calc_ctom: m_image convert coords used"
        }
    }

    if { $debug && $verbose } {
        puts stdout         "           CANVAS coords of points in South-North geographical line"
        puts stdout         "                       South                          North"
        puts stdout [format "           %6s_1%s(%8.2f,%8.2f)  %6s_2%s(%8.2f, %8.2f)" $frame $resol $p1x $p1y $frame $resol $p2x $p2y]
        # If you need to see the points plotted on the canvas:
        # catch {$m_canvas delete point${frame}${resol}}
        # foreach k "1 2" {
        #     set p_var p$k
        #     set p_text  [format "%6s_%s(%7.2f,%7.2f)" $frame ${k}$resol [set ${p_var}x] [set ${p_var}y]]
        #     $m_canvas create text [set ${p_var}x] [set ${p_var}y] -anchor n -fill yellow -text $p_text -tag "point${frame}${resol}"
        # }
    }

    # In canvas (pixels)
    set dx [expr {$p1x - $p2x}]
    set dy [expr {$p1y - $p2y}]

    # In (mm)
    set DM [expr {$YMAX - $YMIN}]

    # Since MIN and MAX numbers come from the ISF file,
    # we better check that delta is not zero:
    if { $DM != 0 } {
        set c2m [expr {sqrt($dx*$dx + $dy*$dy) / $DM}]
    } else {
        if { $debug && $verbose} {
            puts stdout "           Denominator DM is zero!"
        }
        set c2m 0
    }

    # Is the image rotated wrt the frame?
    if { [expr {abs($p1x - $p2x)}] < 1e-4 } {
        # No
        # The coord axis is perpendicular to the frame.
        # The slope and therefore the rotation factors are zero.
        set loc_m 0
        set loc_m1 0
    } else {
        # Yes
        # The coord axis is not perpendicular to the frame,
        # therefore the dx should not be zero.
        # Slope of wcs on the canvas (non dimensional):
        set loc_m [expr {$dy / $dx}]
        set loc_m1 [expr {1.0 / sqrt($loc_m*$loc_m + 1.0)}]
    }
    if { $debug && $verbose } {
        puts stdout         "calc_ctom: New Scale of $frame $resol"
        puts stdout [format " %14s = %8.2f" ${frame}_dx $dx]
        puts stdout [format " %14s = %8.2f" ${frame}_dy $dy]
        puts stdout [format " %14s = %8.2f" ${frame}_DM $DM]
        puts stdout [format " %14s = %8.2f" ${frame}_ctom $c2m]
        puts stdout [format " %14s = %8.2f" ${frame}_loc_m $loc_m]
        puts stdout [format " %14s = %8.2f" ${frame}_loc_m1 $loc_m1]
        puts stdout ""
    }
    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "calc_ctom: END $frame $resol"
    }
    tracer LEAVING
    return "$c2m $loc_m $loc_m1 $p1x $p1y $p2x $p2y $XMIN $YMIN $XMAX $YMAX"
}

#=====================================
private method calc_zemax { my x y } {
#=====================================
# optical correction based on zemax data
# we correct the underlying FORS image
# my = MOS slit nr
# x and y are CCD pixel coords
#
# if fr_x="" then frameless mode
# there is no underlying FORS image to be corrected

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set go 0
    if { [string match *FORS* [$m_image fits get INSTRUME]] } {
        # if the underlying image is from FORS1 or from FORS2
        set go 1
    }

    if { ($x == $fr_x) && ($y == $fr_y) } {
        # Avoid divide by zero
        set go 0
    }

    # W.H 13.Sept 2000 , workaround
    # FORS1 images are aliens for MXU
    # if { [string compare [$this cget -modus] mxu] == 0 } {
    #     if { [string compare [$m_image fits get INSTRUME] "FORS1"] == 0 } {
    #         set go 0
    #     }
    # }

    if { $go } {
        set x_r  [expr {$x - $fr_x}]
        set y_r  [expr {$y - $fr_y}]
        puts stdout "zemax dist $x_r    $y_r"
        set r [expr {sqrt($x_r * $x_r + $y_r * $y_r)}]
        # read scale from fits header, 0.1=HR 0.2=SR
        set fisc [lindex [$m_image wcsset] 2]
        if { $fisc > 0.15 } {
            set ab [ablage_SR $r]
            puts stdout [format "zemax_SR %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f" \
                                 $r \
                                 $ab \
                                 [expr {($r-$ab)/$r}] \
                                 [expr {($r+$ab)/$r}] \
                                 $x_r \
                                 $y_r \
                        ]
        } else {
            set ab [ablage_HR  $r]
            puts stdout [format "zemax_HR %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f" \
                                 $r \
                                 $ab \
                                 [expr {($r-$ab)/$r}] \
                                 [expr {($r+$ab)/$r}] \
                                 $x_r \
                                 $y_r \
                        ]
        }
        # radius must be made smaller
        if { $r < 1e-10 } {
            set x_new $fr_x
            set y_new $fr_y
        } else {
            set x_new [expr {($r-$ab)/$r*$x_r + $fr_x}]
            set y_new [expr {($r-$ab)/$r*$y_r + $fr_y}]
        }
        puts stdout [format "ZEMAX = X: %5.3f%+5.3f = %5.3f  |  Y: %5.3f%+5.3f = %5.3f" \
                             $x \
                             [expr {$x_new-$x}] \
                             $x_new \
                             $y \
                             [expr {$y_new-$y}] \
                             $y_new \
                    ]
        set targw($my,cx) $x_new
        set targw($my,cy) $y_new
        # puts stdout "zemax $targw($my,ca) $targw($my,cd)"
    } else {
        # in frameless mode or image not from FORS1/2
        # do not apply optical correction
        set targw($my,cx) $targw($my,x)
        set targw($my,cy) $targw($my,y)
    }

    # in both cases again
    set reply [catch { $m_image convert coords $targw($my,cx) $targw($my,cy) image \
                                               targw($my,ca) targw($my,cd) "wcs $EQX"
               } result]
    if { $reply != 0 } {
        set targw($my,ca) $targw($my,a)
        set targw($my,cd) $targw($my,d)
        puts stdout "calc_zemax: $result"
        tracer LEAVING 1
        return 1
    } else {
        tracer LEAVING 2
        return 0
    }
}

#======================================
private method calc_zemax_ref { a d } {
#======================================
# optical correction based on zemax data
# we correct the underlying FORS image
# puts stdout "zemax input $x $y"
# x and y are CCD pixel coords, my = MOS slit nr
# input a d in wcs
#
# if fr_x="" then frameless mode
# there is no underlying FORS image to be corrected

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set go 0

    if { [string match *FORS* [$m_image fits get INSTRUME]] } {
        # if the underlying image is from FORS1 or from FORS2
        set go 1
    }

    # W.H 14.Sept 2000 , workaround
    # FORS1 images are aliens for MXU
    # if { [string compare [$this cget -modus] mxu] == 0 } {
    #     if { [string compare [$m_image fits get INSTRUME] "FORS1"] == 0 } {
    #         set go 0
    #     }
    # }

    if { $go } {
        catch { $m_image convert coords $a $d "wcs $EQX" x y image }
        set x_r [expr {$x - $fr_x}]
        set y_r [expr {$y - $fr_y}]
        # puts stdout "using $a $d $x $y $fr_x $fr_y"
        # puts stdout "zemax dist $x_r $y_r"
        set r [expr {sqrt($x_r * $x_r + $y_r * $y_r)}]
        # read scale from fits header, 0.1=HR 0.2=SR
        set fisc [lindex [$m_image wcsset] 2]
        if { $fisc > 0.15 } {
            set ab [ablage_SR $r]
            # puts stdout [format "zemax_SR %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f" \
            #                      $r \
            #                      $ab \
            #                      [expr {($r-$ab)/$r}] \
            #                      [expr {($r+$ab)/$r}] \
            #                      $x_r \
            #                      $y_r \
            #             ]
        } else {
            set ab [ablage_HR  $r]
            # puts stdout [format "zemax_HR %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f" \
            #                      $r \
            #                      $ab \
            #                      [expr {($r-$ab)/$r}] \
            #                      [expr {($r+$ab)/$r}] \
            #                      $x_r \
            #                      $y_r \
            #             ]
        }
        # |radius| must be made smaller
        set x_new [expr {($r-$ab)/$r*$x_r + $fr_x}]
        set y_new [expr {($r-$ab)/$r*$y_r + $fr_y}]

        # puts stdout [format "ZEMAX_REF_TARG = X: %5.3f %+5.3f = %5.3f Y: %5.3f %+5.3f = %5.3f" \
        #                      $x \
        #                      [expr {$x_new-$x}] \
        #                      $x_new \
        #                      $y \
        #                      [expr {$y_new-$y}] \
        #                      $y_new \
        #             ]
        catch { $m_image convert coords $x_new $y_new image a d "wcs $EQX" }
        # puts stdout "zemax_ref_targ $a $d"
        tracer LEAVING 1
        return "$a $d"
    } else {
        # frameless mode, or underlying image not from FORS1 or FORS2
        # do not apply image distortion correction
        tracer LEAVING 2
        return "$a $d"
    }

}

#=====================================================
private method calc_oc { x_mm x_pix my a2 d2 a3 d3 } {
#=====================================================
# -------------------
# optical correction
# -------------------
# x_mm   = slit position in mm focal plane
# x_pix  = slit position in pixel
# my     = slit number [1..19]
# a2 d2  = slit position RA DEC  / deg
# a3 d3  = slit position RA DEC  / wcs
        
    # The distortion is corrected via two terms
    # 1. The absolute distortion at the 10th slit in pixel

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set pix [expr {$x_mm - 7.7225}]
    set pix [expr {$pix * $pix}]
    set pix [expr {$pix * $pix}]
    set pix [expr {9.6542e-9 * $pix}]
    if { $x_mm > 0 } {
        set pix [expr {$pix * -1}]
    }
    # 2. Relative distortion of slit y with respect to slit 10
    set rel_x_pix [expr {abs($x_pix - 1020.0)}]; # distance from x = 0
    set corr_x [expr {$rel_x_pix*([od_F $x_mm $my] - 1.0)}]; # relative correction
    if { $x_mm > 0 } {
        set corr_x [expr {$corr_x * -1}]
    }

    puts stdout "abs = $pix   ||  rel = $corr_x"
    set pix [expr {$pix + $corr_x}];       # total in pixel
    set as  [expr {$pix*0.2}];             # in arscsec
    set del_x [expr {$pix*0.2/$focscale}]; # in mm
    set new_x [expr {$x_mm + $del_x}]
    set da [expr {$as*$CSO*cos($abs_ang_rad)}]
    set dd [expr {$as*$CSO*sin($abs_ang_rad)}]
    # These are the relative distortions with respect to slit nr 10
    puts stdout [format "Move #%2d from x=%5.3f by %5.3f to %5.3f =%5.3f \
                         pixel = %5.3f'' Dalp=%6.4f'' Ddel=%6.4f'' rel=%6.4f" \
                        $my $x_mm $del_x $new_x $pix $as $da $dd  $corr_x \
                ]

    # Now we have to copy these values to the targw array
    set deg_da [expr {$da*15.0/3600.0*$trigo}]
    set deg_dd [expr {$dd/3600.0}]
    set deg_da [expr {$deg_da + $a2}]
    set deg_dd [expr {$deg_dd + $d2}]
    catch { $m_image convert coords $deg_da $deg_dd "deg $EQX" targw($my,ca) targw($my,cd) "wcs $EQX" }
    set targw($my,cx) [expr {$targw($my,x) + $pix}]
    puts stdout "calc_oc:"
    puts stdout "  old   $a3 $d3"
    puts stdout "  new   $targw($my,ca) $targw($my,cd)\n"
    tracer LEAVING
}

#======================================================
private method calc_diff_refrac { my aw dw } {
#======================================================
# Calculates differential correction
#  aw dw = RA DEC in wcs of target
#  my is MOS slit number, but
#  my = 0 means reference target

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # puts stdout "reference pixel = $fr_x $fr_y"
    # puts stdout "reference wcs   = $wx $wy"
    set h [$m_image fits get "HIERARCH ESO TEL ALT"]
    # puts stdout "reference height= $h"
    set p [$m_image fits get "HIERARCH ESO TEL PARANG START"]
    # puts stdout "HIERARCH ESO TEL PARANG START >$p<  deg"

    # images without ALT or PARANG keywords
    #
    if { $h == "" || $p == "" } {
        if { $my != 0 } {
            tracer LEAVING 0
            return
        } else {
            # Reference target
            tracer LEAVING 1
            return [list $aw $dw]
        }
    }

    set p [expr {$p*$deg2rad}]

    catch { $m_image convert coords $aw $dw "wcs $EQX" a d "deg $EQX" }
    # puts stdout "ref  $wx $wy target  $a $d"

    # distance between target and ref pixel in deg
    set del_alp [expr {$a - $wx}]
    set del_del [expr {$d - $wy}]
    # puts stdout "delta RA = $del_alp deg  delta DEC = $del_del deg "

    # and in arcsec
    # set xx [expr $del_alp * 3600.0 * cos($d)];
    # set yy [expr $del_del * 3600.0]
    # set rr [expr sqrt($xx*$xx+$yy*$yy)]
    # puts stdout "delta RA = $xx ''  delta DEC = $yy '' r=$rr"

    # transform RA DEC into Azi Alti using parallactic angle p
    set del_azi [expr {-cos($p)*$del_alp*cos($d) - sin($p)*$del_del}]
    set del_hei [expr {-sin($p)*$del_alp*cos($d) + cos($p)*$del_del}]
    # puts stdout "delta A = $del_azi deg delta H = $del_hei deg"

    # show results in arcsec
    # set xx [expr $del_azi * 3600.0]
    # set yy [expr $del_hei * 3600.0]
    # set rr [expr sqrt($xx*$xx+$yy*$yy)]
    # puts stdout "a_targ - a_ref = $xx '' h_targ - h_ref = $yy '' r=$rr"

    # set z1 [expr (90.0 - $h)]
    # puts stdout "h = $h deg    z = $z1 deg "
    set z [expr {(90.0 - $h)*$deg2rad}]
    # formula for differential refraction
    set refrac [expr {(57.717-3*0.066*tan($z)*tan($z)) / (cos($z)*cos($z))}]
    set refrac [expr {$refrac*$del_hei*$deg2rad}]
    # puts stdout "refrac at z=($z1^0+$del_hei deg) = $refrac ''"
    set refrac [expr {$refrac / 3600.0}]
    # puts stdout "refrac at z=($z1^0+$del_hei deg) = $refrac deg"

    set aa [expr {$refrac / cos($d) * sin($p)}]; # RA component
    set a1 [expr {$a - $aa}];                    # correction
    # puts stdout "$a + $aa = $a1"

    set dd [expr $refrac * cos($p)];           # DEC component
    set d1 [expr $dd + $d];                    # correction
    # puts stdout "$d + $dd = $d1"

    catch { $m_image convert coords $a1 $d1 "deg $EQX" a d "wcs $EQX" }
    # puts stdout "RA  $targw($my,ca) ---> $a"
    # puts stdout "DEC $targw($my,cd) ---> $d"

    # Update corrected coordinates
    if { $my != 0 } {
        # MOS targets
        set targw($my,ca) $a
        set targw($my,cd) $d
    } else {
        # Reference targets
        tracer LEAVING 2
        return "$a $d"
    }
    tracer LEAVING
}

#======================================
private method calc_dither { c frac } {
#======================================
# For polarimetric imaging IPOL
# Moves the mask one stripe further
# highres  = 11.36''
# standard = 22.73''
# c =+-1 = up down
# frac = 1        : 2 exposures, shift = '1' *  22.73 ''
# frac = 0.666666 : 3 exposures, shift = '2/3' *22.73'

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Take the half for highres collimator
    if { [string match *HR* [$this cget -collimat]] } {
        set dith [expr {22.73 / 3600.0 / 2.0 * $frac}]
    } else {
        set dith [expr {22.73 / 3600.0 * $frac}]
    }
    # puts stdout "Dither : shift by $dith deg, PAM = $PAM  deg"

    # deg to rad for sin/cos
    set pa [expr {$PAM * $deg2rad}]
    if { $c < 0 } {
        set pa [expr {($PAM + 180.0) * $deg2rad}]
    }

    # puts stdout [format \
    # "w-%8.3f %8.3f c-%8.3f %8.3f m-%8.3f %8.3f o-%8.3f %8.3f" \
    #  $world_x $world_y $ccx $ccy $mos(0,x0) $mos(0,y0) $oldpos(x) $oldpos(y)]

    catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas a0 d0 "wcs $EQX" }
    catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x1 y1 "deg $EQX" }
    # $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x2  y2 canvas
    # $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x3  y3 screen
    # $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x4  y4 image

    puts stdout [format "Di old: %10s  %10s %8.5f %8.5f" $a0 $d0 $x1 $y1]

    # Calculation, linear scale
    catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas a0 d0 "deg $EQX" }
    set a1 [expr {$a0 + $dith * sin($pa) / cos($d0 * $deg2rad)}]
    set d1 [expr {$d0 + $dith * cos($pa)}]
    #
    set dither_offset_a [expr {$dith * sin($pa) / cos($d0 * $deg2rad)}]
    set dither_offset_d [expr {$dith * cos($pa)}]
    # puts stdout "offset RA  = $dither_offset_a "
    # puts stdout "offset DEC = $dither_offset_d"
    # offset from deg to arcsec
    set dither_offset_a [expr {$dither_offset_a * 3600.0 / 15.0 / cos($d0 * $deg2rad)}]
    set dither_offset_d [expr {$dither_offset_d * 3600.0}]

    catch { $m_image convert coords $a1 $d1 "deg $EQX" a0 d0 "wcs $EQX" }
    puts stdout [format "Di new: %10s  %10s %8.5f %8.5f" $a0 $d0 $a1 $d1]
    # $m_image convert coords $a1 $d1 "deg $EQX" x2 y2 canvas
    # $m_image convert coords $a1 $d1 "deg $EQX" x3 y3 screen
    # $m_image convert coords $a1 $d1 "deg $EQX" x4 y4 image

    puts stdout [format "SEQ.OFF.ALPHA  = %8.5f''"  $dither_offset_a]
    puts stdout [format "SEQ.OFF.DELTA  = %8.5f''"  $dither_offset_d]
    if { $frac < 0.9 } {
        set number 2
    } else {
        set number 1
    }
    puts stdout [format "SEQ.OFF.NUMBER =  %1d"  $number]

    # puts stdout [format "new TCS : %10s  %10s %8.5f %8.5f" $a0 $d0 $a1 $d1]
    tracer LEAVING
    return "$a1 $d1 $dither_offset_a $dither_offset_d"
}

#=======================================
private method dither_pol { w c frac } {
#=======================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # do not dither when specifying reference targets
    if { $stop } {
        tracer LEAVING 1
        return
    }

    # for polarimetric imaging IPOL
    # this moves the mask one stripe further
    # highres  = 11.36''
    # standard = 22.73''
    # frac = 1        : 2 exposures, shift = '1' * 22.73''
    # frac = 0.666666 : 3 exposures, shift = '2/3' *22.73''

    #--------------------------------
    set result [calc_dither $c $frac]
    #--------------------------------
    if { $frac < 0.9 } {
        set dith_num 2
    } else {
        set dith_num 1
    }
    # puts stdout ">$result<"
    set result [split $result " "]
    set a0 [lindex $result 0];      # wcs
    set d0 [lindex $result 1];      # wcs
    # set off_a [lindex $result 2];   # arcsec
    # set off_d [lindex $result 3];   # arcsec
    # puts stdout "result = >$a0< >$d0< >$off_a< >$off_d<"

    # we shift the mask without mark
    set oldpos(x) $mos(0,x0)
    set oldpos(y) $mos(0,y0)

    # $m_image convert coords $a0 $d0 "deg $EQX" a0 d0 "wcs $EQX"
    # $m_image convert coords $a0 $d0 "deg $EQX" x2 y2 canvas
    catch { $m_image convert coords $a0 $d0 "deg $EQX" x3 y3 screen }
    # $m_image convert coords $a0 $d0 "deg $EQX" x4 y4 image

    # puts stdout [format \
    # "w-%8.3f %8.3f c-%8.3f %8.3f m-%8.3f %8.3f o-%8.3f %8.3f" \
    # $world_x $world_y $ccx $ccy $mos(0,x0) $mos(0,y0) $oldpos(x) $oldpos(y)]
    # puts stdout "   --- shift ---"
    #-----------------------------
    [code $this shift $w $x3 $y3]
    shift $w $x3 $y3
    #-----------------------------

    # puts stdout [format \
    # "w-%8.3f %8.3f c-%8.3f %8.3f m-%8.3f %8.3f o-%8.3f %8.3f" \
    # $world_x $world_y $ccx $ccy $mos(0,x0) $mos(0,y0) $oldpos(x) $oldpos(y)]

    #----------------
    update_and_replot
    #----------------

    # puts stdout [format "%8.3f  %8.3f -- %8.3f  %8.3f -- %8.3f %8.3f" \
    #                      $world_x $world_y $ccx $ccy $mos(0,x0) $mos(0,y0)]
    set last_action [format "Dither Y %5.3f" [expr $c * $frac]]
    catch {SP update}
    tracer LEAVING
}

#==============================
private method bind_align { } {
#==============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # for hits, lss and echs we want to shift the mask
    # that the slit is on the center/gauss - result
    # in case the center/slit option is on
    $m_canvas bind ali <Button-1> [code $this align_mask %W %x %y]
    tracer LEAVING
}

#==============================
private method bind_frame { } {
#==============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # tag m = green circle, tag C = blue CCD window
    # if mouse in m (green central circle) then mark
    $m_canvas bind m <Button-1>  [code $this mark %W %x %y]
    $m_canvas bind m <B1-Motion> [code $this shift %W %x %y]

    if { [string compare [$this cget -modus] echs] == 0 } {
        $m_canvas bind m <ButtonRelease-1> [code $this echs_targ]
    } else {
        $m_canvas bind m <ButtonRelease-1> [code $this reset_slits]
    }

    # if mouse is in mc and clicked, rotate clockwise
    $m_canvas bind m <Button-2>       [code $this rotate %W %x %y -1]
    # if mouse is in mc and shift-b2 clicked, rotate anti-clock-wise
    $m_canvas bind m <Shift-Button-2> [code $this rotate %W %x %y 1]

    puts $lfId "$this bind frame"
    tracer LEAVING
}

#=============================
private method bind_echs { } {
#=============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # the cyan circle is used to (de)-activate the MOS 9 slit
    $m_canvas bind PE <Shift-Button-1> [code $this flip_echs]
    $m_canvas bind PE <Shift-Button-1> [code $this flip_echs]
    tracer LEAVING
}

#==============================
private method bind_hitms { } {
#==============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    $m_canvas bind mxu <Button-1> [code $this hitms_create_slit %W %x %y]

    # Click on the slit width label (.e.g  '1.3"') to change individual
    # HIT-MS slit configuration
    $m_canvas bind sw <Button-1> [code $this hitms_config_slit %W %x %y]

    # Click on slit label (e.g. '#15') at the rim of the focal field to
    # delete HIT-MS slit
    $m_canvas bind kill <Button-1> [code $this hitms_delete_slit %W %x %y]

    # Deselect HIT-MS slit
    $m_canvas bind rs <Button-1> [code $this hitms_delete_slit %W %x %y]

    # Unbind move slit from previous MOS or IMG modes.
    $m_canvas bind mm <Button-1>  [code $this unbind_hitms]
    $m_canvas bind mm <Button-2>  [code $this unbind_hitms]

    # For the bezier points
    $m_canvas bind door <Button-1>        [code $this mark_cms %W %x %y]
    $m_canvas bind door <B1-Motion>       [code $this shift_cms %W %x %y]
    $m_canvas bind door <ButtonRelease-1> [code $this end_cms %W %x %y]
    tracer LEAVING
}

#================================
private method unbind_hitms { } {
#================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    puts stdout "HIT-MS mask cannot be repositioned by clicking."
    tracer LEAVING
}

#============================
private method bind_img { } {
#============================
    # if mouse is on any object
    # the last arg for mosa or mosb tag

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # B1 inside slit, move blade A, otherwise default blade
    $m_canvas bind mm  <Button-1> [code $this move_slit_blade %W %x %y 1]

    # B2 inside slit, move blade B, otherwise default blade
    $m_canvas bind mm  <Button-2> [code $this move_slit_blade %W %x %y 0]

    # B1 on slit rim label or B2 on slit label : out of focal field
    $m_canvas bind out <Button-1> [code $this move_slit_blade %W %x %y out]
    $m_canvas bind lb  <Button-2> [code $this move_slit_blade %W %x %y out]

    # individual slit width
    $m_canvas bind sw  <Button-1> [code $this mos_config_slit %W %x %y]

    puts $lfId "$this bind img"
    tracer LEAVING
}

#=============================
private method bind_ipol { } {
#=============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    bind_img
    # white  circle : dither 1 MOS slit width
    $m_canvas bind P2 <Button-1>       [code $this dither_pol %W  1 1.0]
    $m_canvas bind P2 <Shift-Button-1> [code $this dither_pol %W -1 1.0]

    # grey  circle : dither 2/3 MOS slit width
    $m_canvas bind P3 <Button-1> [code $this dither_pol %W  1 [expr {2.0/3.0}]]
    $m_canvas bind P3 <Shift-Button-1> \
                                 [code $this dither_pol %W -1 [expr {2.0/3.0}]]
    puts $lfId "$this bind ipol"
    tracer LEAVING
}

#============================
private method bind_mos { } {
#============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # if mouse is on any object
    # set
    $m_canvas bind mm <Button-1>  [code $this move_slit %W %x %y 1 1]

    # scratch
    $m_canvas bind lb  <Button-1> [code $this move_slit %W %x %y 0 1]
    $m_canvas bind out <Button-1> [code $this move_slit %W %x %y 0 1]

    # change individual MOS slit width
    $m_canvas bind sw <Button-1>  [code $this mos_config_slit %W %x %y]
    tracer LEAVING
}

#============================
private method bind_mxu { } {
#============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    $m_canvas bind mxu <Button-1> [code $this mxu_create_slit %W %x %y]

    # click on the slit width label (.e.g  '1.3"') to change individual
    # MXU slit width
    $m_canvas bind sw <Button-1> [code $this mxu_config_slit %W %x %y]

    # click on slit label (e.g. '#15') at the rim of the focal field to
    # delete mxu slit
    $m_canvas bind kill <Button-1> [code $this mxu_delete_slit %W %x %y]

    # click on the slit label (e.g. '#15') at the slit to assign slit as
    # reference slit for 'Fast Alignment'
    # $m_canvas bind rs <Button-1> [code $this mxu_ref_slit %W %x %y 1]
    # $m_canvas bind rs <Button-2> [code $this mxu_ref_slit %W %x %y 0]

    # scratch
    # AAG, De-Select mxu slit
    $m_canvas bind rs <Button-1> [code $this mxu_delete_slit %W %x %y]

    # Unbind move slit from previous MOS or IMG modes.
    $m_canvas bind mm <Button-1>  [code $this unbind_mxu]
    $m_canvas bind mm <Button-2>  [code $this unbind_mxu]

    # for the bezier points
    $m_canvas bind door <Button-1>        [code $this mark_cms %W %x %y]
    $m_canvas bind door <B1-Motion>       [code $this shift_cms %W %x %y]
    $m_canvas bind door <ButtonRelease-1> [code $this end_cms %W %x %y]
    tracer LEAVING
}

#==============================
private method unbind_mxu { } {
#==============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    puts stdout "MXU mask cannot be repositioned by clicking."
    tracer LEAVING
}

#=============================
private method flip_echs { } {
#=============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # ECHS uses either MOS slit 10 alone (echs_flag==0)
    # or MOS 10 and 9 together (echs_flag==1)
    # puts stdout "flip_echs: [$this cget -grism]"
    if { $echs_flag } {
        set echs_flag 0
        set targw(9,f) 0
    } else {
        set echs_flag 1
        set targw(9,f) 1
    }

    # for status panel
    catch {SP update}

    update_and_replot
    # plot first echs slit 9, then position
    if { $echs_flag } {
        echs_pos
    }
    tracer LEAVING
}

#===============================
public method AutoPos { kind } {
#===============================
# 1) Create file AutoPos.asc
# 2) Call FIMSROOT/bin/AP

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Kind may be static, shift, rotate, all
    puts stdout "AutoPos: Method = $kind"

    # Select one of the local catalogs
    set a_c [get_cat]

    if { $a_c == "" } {
        info_dialog "AutoPos: No catalog specified" .skycat1
        tracer LEAVING NO_CAT
        return
    }

    # Test flag
    set p 0

    set loc_list [lindex [$a_c info variable info_] 4]
    # Do not process empty list
    if { $loc_list == "" } {
        set p 1
        if { $debug } {
            puts stdout "AutoPos: Error, catalog empty"
        }
    } else {
        if { $debug } {
            puts stdout "AutoPos: Hurray!, catalog not empty"
        }
    }

    #--SET Priority-Mag-Flag

    # Process headings; what's available
    set loc_head [$a_c info variable headings_]
    set loc_head [lindex $loc_head 4]
    if { $debug && $verbose } {
        puts stdout "AutoPos: catalog head = >$loc_head<"
    }

    # Search in table header mandatory keywords
    # Several tries for each one
    set i_ra [lsearch -glob  $loc_head *ra*]
    if { $i_ra == -1 } {
        set i_ra [lsearch -glob  $loc_head *RA*]
    }
    if { $i_ra == -1 } {
        set i_ra [lsearch -glob  $loc_head *Ra*]
    }
    if { $i_ra == -1 } {
        puts stdout "AutoPos: Error, catalog header RA not found"
        set p 1
    }

    set i_dec [lsearch -glob  $loc_head *dec*]
    if { $i_dec == -1 } {
        set i_dec [lsearch -glob  $loc_head *DEC*]
    }
    if { $i_dec == -1 } {
        set i_dec [lsearch -glob  $loc_head *Dec*]
    }
    if { $i_dec == -1 } {
        puts stdout "AutoPos: Error, catalog header DEC not found"
        set p 1
    }
    if { $debug } {
        puts stdout "AutoPos: i_ra, i_dec = >$i_ra, $i_dec<"
    }

    set i_id  [lsearch -glob  $loc_head *id*]
    if { $i_id == -1 } {
        set i_id [lsearch -glob  $loc_head *ID*]
    }
    if { $i_id == -1 } {
        set i_id  [lsearch -glob  $loc_head *Id*]
    }
    if { $i_id == -1 } {
        set i_id [lsearch -glob  $loc_head *name*]
    }
    if { $i_id == -1 } {
        set i_id [lsearch -glob  $loc_head *target*]
    }
    if { $i_id == -1 } {
        set i_id [lsearch -glob  $loc_head *star*]
    }
    if { $i_id == -1 } {
        set i_id [lsearch -glob  $loc_head *object*]
    }
    if { $i_id == -1 } {
        puts stdout "AutoPos: Error, catalog header ID not found"
        set i_id 0; # assuming the first entry is the ID
        set p 1
    }
    if { $debug } {
        if { !$p } {
            puts stdout "AutoPos: Hurray!, catalog header accepted"
        }
    }

    # optional, keywords
    set p 0
    set i_mag  [lsearch -glob  $loc_head *mag*]
    if { $i_mag == -1 } {
        set i_mag [lsearch -glob  $loc_head *MAG*]
    }
    if { $i_mag == -1 } {
        set i_mag [lsearch -glob  $loc_head *Mag*]
    }
    if { $i_mag == -1 } {
        puts stdout "AutoPos: Warning, catalog header MAG not found"
        set p 1
    }

    set i_prio [lsearch -glob  $loc_head *pri*]
    if { $i_prio == -1 } {
        set i_prio [lsearch -glob  $loc_head *PRI*]
    }
    if { $i_prio == -1 } {
        set i_prio [lsearch -glob  $loc_head *Pri*]
    }
    if { $i_prio == -1 } {
        puts stdout "AutoPos: Warning, catalog header PRIO not found"
        set p 1
    }

    if { $debug } {
        if { !$p } {
            puts stdout "AutoPos: MAG and PRIO found in catalog header"
        }
    }

    if { $i_prio >= 0 } {
        set mp_header "1"
    } else {
        set mp_header "0"
    }
    if { $i_mag >= 0 } {
        set mp_header 1$mp_header
    } else {
        set mp_header 0$mp_header
    }

    if { $debug && $verbose } {
        puts stdout "AutoPos: Magnitude header present: $i_mag, Priority header present: $i_prio, mp_header = $mp_header"
    }

    # -- Frame Rotation -- Scale
    # WP = angle between Frame (pixel-coords) and WCS = CROTA2
    # WP > 0 means north counterclock, independent on E-axis
    # WP * CSO = mathematical rotation angle
    #
    # Note: Position angle (PA) is a direction in the WCS
    #       it is measured from N to E
    #
    # angle between mask and wcs = abs_ang_rad =  PAM
    # = position angle of the mask with respect to the WCS
    # Since the WCS can be right or left oriented we need
    # CSO in order to determine PAM
    # PAM (position angle of the mask)
    #
    # fits_scale= ++ || --           -------------
    # angle between mask and pixel = MP = WP + PAM
    #                                -------------
    # fits_scale= +- || -+           -------------
    # angle between mask and pixel = MP = WP - PAM
    #                                -------------
    # WP and PAM are given to Autopos
    # Autopos returns the position angle = PAM  in
    # the target acquisition file opt1.targ

    set fitsheader [$m_image wcsset]
    if { $debug } {
        puts stdout "AutoPos: fits header = $fitsheader"
        puts stdout "         scale = [lindex $fitsheader  2] arcsec/pixel"
        puts stdout "         rot   = [lindex $fitsheader  7] deg"
        puts stdout "         Proj  = [lindex $fitsheader 10]"
    }

    # CROTA2 = World-Pixel
    set WP [lindex $fitsheader 7]

    # MP = mask-pixel angle
    # PAM = mask-wcs angle  (=position angle of instrument)
    # WP = wcs-pixel angle (=fits keyword CROTA2)
    # give AutoPos neg MW, since AutoPos uses mathemat. angles
    # set MP [expr -$WP*$CSO - $PAM]
    set MP [expr $WP - $PAM*$CSO]; # mathemat mask angle

    if { $debug } {
        puts stdout "         WP    = $WP fixed"
        puts stdout "         PAM   = $PAM"
        puts stdout "         MP    = $MP"
    }

    set myfile $log_dir/AutoPos.asc
    puts stdout "AutoPos: >$myfile<"
    set fileId [open $myfile w]
    puts stdout "AutoPos: open $myfile"

    # We need to know the collimator, here as
    # high-res collimator flag
    if { [string match *HR* [$this cget -collimat]] } {
        set sc 1
    } else {
        set sc 0
    }

    # CCD -- Window

    # Furthermore the CCD-window del_ra, del_dec
    if { [string match *HR* [$this cget -collimat]] } {
        set del_dec [expr ($YHRMAX-$YHRMIN)*$focscale]
    } else {
        set del_dec [expr ($YSRMAX-$YSRMIN)*$focscale]
    }

    if { [string match *HR* [$this cget -collimat]] } {
        # Assume dispersion direction BLUE=E=B=x>0, RED=W=A=x<0
        set ra_min [expr ($XHRMIN+$specmax/2)*$focscale]
        set ra_max [expr ($XHRMAX-$specmin/2)*$focscale]
        set hrcolli 1
        # Assume dispersion direction RED=E=B=x>0, BLUE=W=A=x<0
        # set ra_min [expr ($XHRMIN+$specmin/2)*$focscale]
        # set ra_max [expr ($XHRMAX-$specmax/2)*$focscale]
    } else {
        # Assume dispersion direction BLUE=E=B=x>0, RED=W=A=x<0
        set ra_min [expr ($XSRMIN+$specmax)*$focscale]
        set ra_max [expr ($XSRMAX-$specmin)*$focscale]
        set hrcolli 0
        # Assume dispersion direction RED=E=B=x>0, BLUE=W=A=x<0
        # set ra_min [expr ($XSRMIN+$specmin)*$focscale]
        # set ra_max [expr ($XSRMAX-$specmax)*$focscale]
    }

    # Convert the current mask position in wcs and pixel
    catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
                                    lwx lwy "wcs $EQX" }
    set lwx [expr {15*[DDMMSS2deg $lwx]}]
    set lwy [DDMMSS2deg $lwy]
    catch { $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
                                    px py image }

    if { $debug } {
        puts stdout "AutoPos:"
        puts stdout "  MOS(0,x0) canv = $mos(0,x0)"
        puts stdout "  MOS(0,y0) canv = $mos(0,y0)"
        puts stdout "  LWX \[deg\]    = $lwx"
        puts stdout "  LWY \[deg\]    = $lwy"
        puts stdout "  PX  \[pix\]    = $px"
        puts stdout "  PY  \[pix\]    = $py"
    }

    # Control parameter

    # AAG - PPRS 10127
    # Added 'simple' option

    switch -exact -- $kind {
        static {
            set co 0
        }
        shift {
            set co 1
        }
        rotat {
            set co 2
        }
        all {
            set co 3
        }
        simple {
            set co 4
        }
    }

    switch -exact -- [$this cget -modus] {
        mos {
            set mosp 0
        }
        pmos {
            set mosp 1
        }
        mxu {
            set mosp 2
        }
        hitms {
            set mosp 3
        }
        default {
            puts stdout "AutoPos: invalid mode"
        }
    }

    # puts stdout "modus = [$this cget -modus], mosp = $mosp"
    # puts stdout "FORS  = $insId"

    # mask_frame angle

    set big_string [format "%1d %1d %2d %1d %8.6f %1d %1d %6.3f %6.3f " \
                           $exposure_counter \
                           $insId \
                           $mp_header \
                           $co \
                           $focscale \
                           $mosp \
                           $hrcolli \
                           [lindex $fitsheader 2] \
                           $WP \
                   ]

    if { $debug } {
        puts stdout "AutoPos:"
        puts stdout "  big_string = $big_string"
    }

    set app_string [format "%6.3f %10.8f %10.8f %6.3f %6.3f %6.3f " \
                           $PAM $lwx $lwy $px $py $del_dec \
                   ]

    if { $debug } {
        puts stdout "  app_string = $app_string"
    }

    # In MOS default slit length is 'real', although slit_length is not
    # used for MOS, only in MXU, where real is omitted
    if { [string match *real* [$this cget -slit_length]] } {
           set sl 22.0
    } else {
        set sl [$this cget -slit_length]
    }

    # puts stdout "sw, sl = [$this cget -slit_width], $sl"

    set ap2_string [format "%6.3f %6.3f %4.1f %4.1f %6.3f %6.3f" \
                           $ra_min \
                           $ra_max \
                           $fits_x_scale \
                           $fits_y_scale \
                           [$this cget -slit_width] \
                           $sl \
                   ]

    if { $debug } {
        puts stdout "  ap2_string = $ap2_string"
        puts stdout "  ecmp K B P C   scal   CROT    PA     wx    wy       px    py    dec      mi/ma/ra ori"
        puts stdout "  $big_string $app_string $ap2_string"
    }
    puts $fileId "$big_string $app_string $ap2_string"

    set counter 0
    if { $debug && $verbose } {
        puts stdout "  loc_list = >>$loc_list<< \n\n"
    }
    foreach star $loc_list {
        incr counter 1
        # puts stdout "$star >[lindex $star $i_ra]< >[lindex $star $i_dec]<"
        if { $debug } {
            puts stdout "  $counter: $star"
            puts stdout "  RA  = [lindex $star $i_ra]"
            puts stdout "  DEC = [lindex $star $i_dec]"
        }
        set px 0
        set py 0
        set alp 0
        set dec 0
        set alp [expr {15*[DDMMSS2deg [lindex $star $i_ra]]}]
        set dec [DDMMSS2deg [lindex $star $i_dec]]
        # catch { $m_image convert coords [lindex $star $i_ra][lindex $star $i_dec] "wcs $EQX" \
        #                                 alp dec "deg $EQX"
        # }
        catch { $m_image convert coords [lindex $star $i_ra] [lindex $star $i_dec] "wcs $EQX" \
                                        px py image
        }
        if { $debug } {
            puts stdout "  ALP \[deg\] = $alp"
            puts stdout "  DEL \[deg\] = $dec"
            puts stdout "  Px  \[pix\] = $px"
            puts stdout "  Py  \[pix\] = $py"
        }

        set id [lindex $star $i_id]
        # Target name strings must not contain blanks, hence fill them with "_"
        set id [string_exchange $id " " "_"]
        set plotstring [format "%s %3.6f %3.6f %6.2f %6.2f" $id $alp $dec $px $py]
        if { $i_mag >= 0 } {
            set plotstring "$plotstring [lindex $star $i_mag]"
        }
        if { $i_prio >= 0 } {
            set plotstring "$plotstring [lindex $star $i_prio]"
        }

        # Now all together
        if { $debug } {
            puts stdout "  $counter: $plotstring\n\n"
        }
        # Consider only targets with correctly converted coords
        if { $px > 0 } {
            puts $fileId $plotstring
        }
    }
    close $fileId

    puts stdout "AutoPos: close $myfile"

    set exfile $int_dir
    append exfile "/bin/AP"

    set source_file "AutoPos.asc"

    puts stdout "AutoPos: start external $exfile $source_file ..."
    set result [exec $exfile $source_file]
    puts stdout "AutoPos: receive result : $result"
    puts stdout "AutoPos: load setupfile ... end"

    switch -exact -- [$this cget -modus] {
        pmos {
            load_mos_TPX  ""
        }
        mos {
            load_mos_TPX  ""
        }
        hitms {
            load_hitms_TPX  ""
        }
        mxu {
            load_mxu_TPX  ""
        }
    }

    # autpos has created a mos/mxu1/2.$expo_counter.auto_pos.fims file
    # which should not be overwritten by the next autopos try

    append AutoposMethod "$kind "
    incr exposure_counter 1
    if { $debug } {
        puts stdout "AutoPos: EC = $exposure_counter"
        puts stdout "AutoPos: END\n\n"
    }
    tracer LEAVING
}

#=============================
public method set_wcs { in } {
#=============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    puts stdout "set_wcs  $in"
    set in [split $in " "]
    puts stdout "[lindex $in 0] [lindex $in 1] \
                 [lindex $in 2] [lindex $in 3] \
                 [lindex $in 4] [lindex $in 5] \
                 [lindex $in 6] [lindex $in 7] \
                 2000 1998.5 -TAN"
    $m_image wcsset [lindex $in 0] [lindex $in 1] \
                    [lindex $in 2] [lindex $in 3] \
                    [lindex $in 4] [lindex $in 5] \
                    [lindex $in 6] [lindex $in 7] \
                    2000 1998.5 -TAN
    puts stdout [$m_image wcsset]
    tracer LEAVING
}

#==========================
public method new_wcs { } {
#==========================
# External configuration of the wavelength range
# Cancel button -> destroy

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    global .ew
    puts stdout "new_wcs"

    if { [catch {wm deiconify .ew}] } {
        EntryForm .ew \
           -title "Define WCS (no frame)" \
           -labels {ra dec secpix xrefpix yrefpix nxpix nypix rotate equinox} \
           -command "fors::FORS::my_mask set_wcs" \
           -values {{01:01:00}{-72:01:00}{2.0}{100}{100}{400}{400}{0.0}{2000.0}}
    } else {
        .ew configure
           -values {{01:00:00}{-72:00:00}{2.0}{100}{100}{400}{400}{0.0}{2000.0}}
    }
    tkwait window .ew
    tracer LEAVING
}

#=====================================
private method shift_cms { w sx sy } {
#=====================================
# External configuration of an individual MXU slit : CURVED

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    catch { $m_image convert coords $sx $sy screen x y canvas }
    set particle [$w find closest $x $y]
    set alltags [$w gettags $particle]
    set curtag [lindex $alltags 2]
    set beztag [lindex $alltags 3]
    set my [string range $curtag 3 end]

    if { $debug && $verbose } {
        puts stdout "shift_cms: "
        puts stdout "           sx       = $sx"
        puts stdout "           sy       = $sy"
        puts stdout "           x        = $x"
        puts stdout "           y        = $y"
        puts stdout "           particle = $particle"
        puts stdout "           alltags  = $alltags"
        puts stdout "           curtag   = $curtag"
        puts stdout "           beztag   = $beztag"
        puts stdout "           my       = >$my<"
    }

    if { ![string match B* $beztag] } {
        if { $debug && $verbose } {
            puts stdout "           Invalid tag $beztag. Not a bezier point."
        }
        tracer LEAVING BAD_TAG
        return
    }

    if { $my == "" } {
        info_dialog "Dragging too fast !!!\n\
                     Try with Slit Width = 20''\n\
                     to have bigger dots to drag.\n\
                     When finished, set final width." .skycat1
    } else {
        # puts stdout ">[$w find closest $x $y]< \
        #              >[$w gettags [$w find closest $x $y]]< \
        #              curtag= >$curtag<  beztag= >$beztag< my= >$my<"

        # The shift distance in canvas units
        set shift_x [expr {$x - $op(x)}]
        set shift_y [expr {$y - $op(y)}]

        # puts stdout "$my $beztag $x +- $shift_x $y +- $shift_y"
        # Modify position of one bezier point
        C_mxus$my mod_bezier $beztag $shift_x $shift_y

        # Plot the slit contour of curve $my
        C_mxus$my plot_slit
        # Delete and replot the two non-moved bez-points
        C_mxus$my plot_bezier_dots $beztag

        # Shift the selected bezier point on the canvas
        $w move $beztag $shift_x $shift_y

        # Update cursor oldpos for permanent shifts
        set op(x) $x
        set op(y) $y

    }
    tracer LEAVING
}

#====================================
private method mark_cms { w xs ys } {
#====================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    update_xform_coeff
    catch { $m_image convert coords $xs $ys screen op(x) op(y) canvas }
    tracer LEAVING
}

#===================================
private method end_cms { w sx sy } {
#===================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # Update the curved mxu slit
    catch { $m_image convert coords $sx $sy screen x y canvas }
    set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
    set my [string range $curtag  3 end]

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "end_cms: "
        puts stdout "         sx       = $sx"
        puts stdout "         sy       = $sy"
        puts stdout "         x        = $x"
        puts stdout "         y        = $y"
        puts stdout "         curtag   = $curtag"
        puts stdout "         my       = >$my<"
    }

    if { $my == "" } {
        info_dialog "Dragging too fast !!!\n\
                     Try with Slit Width = 20''\n\
                     to have bigger dots to drag.\n\
                     When finished, set final width." .skycat1
    } else {
        # Delete the mxc tags
        $m_canvas delete mxc$my

        if { $label } {
            C_mxus$my plot_label e w
        }

        # Update the optical correction for all three bezier points
        # Get out the values of class mxu, apply correction and copy them
        # into class mxu again
        if { [string match *CURVED* [C_mxus$my cget -shape]] } {
            update_bez_oc $my
        }

        # set a [C_mxus$my get_curve_ca]
        # set targw(1,a) [C_mxus$my get_RA]
        # set targw(1,d) [C_mxus$my get_DEC]
        #
        # $m_image convert coords [lindex $a 0] [lindex $a 1] canvas \
        #                         targw(1,x) targw(1,y) image
        # calc_zemax 1 $targw(1,x) $targw(1,y)
        # C_mxus$my put_b_oc 1 [list $targw(1,cx) $targw(1,cy)]
        #
        # $m_image convert coords [lindex $a 2] [lindex $a 3] canvas \
        #                         targw(1,x) targw(1,y) image
        # calc_zemax 1 $targw(1,x) $targw(1,y)
        # C_mxus$my put_b_oc 2 [list $targw(1,cx) $targw(1,cy)]
        #
        # $m_image convert coords [lindex $a 4] [lindex $a 5] canvas \
        #                         targw(1,x) targw(1,y) image
        # calc_zemax 1 $targw(1,x) $targw(1,y)
        # C_mxus$my put_b_oc 3 [list $targw(1,cx) $targw(1,cy)]
        #
        # $m_image convert coords [lindex $a 6] [lindex $a 7] canvas \
        #                         targw(1,x) targw(1,y) image
        # calc_zemax 1 $targw(1,x) $targw(1,y)
        # C_mxus$my put_b_oc 4 [list $targw(1,cx) $targw(1,cy)]
        #
        # $m_image convert coords [lindex $a 8] [lindex $a 9] canvas \
        #                         targw(1,x) targw(1,y) image
        # calc_zemax 1 $targw(1,x) $targw(1,y)
        # C_mxus$my put_b_oc 5 [list $targw(1,cx) $targw(1,cy)]
        #
        # $m_image convert coords [lindex $a 10] [lindex $a 11] canvas \
        #                         targw(1,x) targw(1,y) image
        # calc_zemax 1 $targw(1,x) $targw(1,y)
        # C_mxus$my put_b_oc 6 [list $targw(1,cx) $targw(1,cy)]
        #
        # C_mxus$my status

        # apply correction of differential refraction (10.5.99)
        # input slit position, corrected for optical distortion
        # if { $dire_flag } {
        #     calc_diff_refrac 1 $targw(1,ca) $targw(1,cd)
        # }
        # C_mxus$noms put_OC [list $targw(1,ca) $targw(1,cd)]

        set a [size_CS $my]
    }
    tracer LEAVING
}

#==============================
private method size_CS { my } {
#==============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # WH 14.Sept 2000
    # finally we test if the slit is small enough
    set c [C_mxus$my get_b_oc_mmu]
             # index of the six bezier points
             #
             #    0,1 +----+ 6,7
             #         \    \
             #          \    \
             #           \    \
             #           /    /  2,3 +     + 8,9
             #      4,5 +----+ 10,11
             #
             # flash the first point
    set p1x [expr ([lindex $c 0] + [lindex $c 6])  /2.0]
    set p1y [expr ([lindex $c 1] + [lindex $c 7])  /2.0]
    set p3x [expr ([lindex $c 4] + [lindex $c 10]) /2.0]
    set p3y [expr ([lindex $c 5] + [lindex $c 11]) /2.0]
    set hx  [expr [lindex $c 0] - [lindex $c 6]]
    set hy  [expr [lindex $c 1] - [lindex $c 7]]
    set w   [expr sqrt($hx*$hx +$hy*$hy)]
    # puts stdout $c
    puts stdout [format "width = %5.3f  mm" $w]
    set size1 [expr sqrt( ($p1x-$p3x)*($p1x-$p3x) + ($p1y-$p3y)*($p1y-$p3y) ) +$w]
    # oversize flag
    set CtL 0

    # first test
    if { $size1 > 20.0 } {
        puts stdout [format "Warning: CURVED slit %1d size = %5.3fmm > 20mm" \
                            $my $size1 \
                    ]
        set CtL 1
    } else {
        # puts stdout "GO on!! $size mm <= 20mm"
    }

    # Since the bezier curve does not match P2, we use the half length
    set p2x [expr ([lindex $c 2] + [lindex $c 8]) /2.0]
    set p2y [expr ([lindex $c 3] + [lindex $c 9]) /2.0]
    set size2 [expr sqrt(($p2x-$p3x)*($p2x-$p3x)+($p2y-$p3y)*($p2y-$p3y))/2.0+$w]
    # second test
    if { $size2 > 20.0 } {
        puts stdout [format "Warning: CURVED slit %1d size = %5.3fmm > 20mm" \
                           $my $size2 \
                    ]
        set CtL 1
    } else {
        # puts stdout "GO on!! $size mm <= 20mm"
    }
    set size3 [expr sqrt(($p2x-$p1x)*($p2x-$p1x)+($p2y-$p1y)*($p2y-$p1y))/2.0+$w]
    # third test test
    if { $size3 > 20.0 } {
        puts stdout [format "Warning: CURVED slit %1d size = %5.3fmm > 20mm" \
                            $my $size3 \
                    ]
        set CtL 1
    } else {
        # puts stdout "GO on!! $size mm <= 20mm"
    }

    puts stdout [format "curve (1-3/1-2/2-3) = (%5.3f/%5.3f/%5.3f) mm" $size1 $size2 $size3]

    if { $CtL } {
        info_dialog "CURVED slit \#$my\n too large !\n\n Shrink \#$my !\n" .skycat1
    }
    tracer LEAVING
    return $CtL
}

#==================================================================================================
private method mxu_new_slit { click x y px py wid len off pa shape name ff max {ref 0} {mmff 0} } {
#==================================================================================================
# Creates a new mxu slit
# click: If 1, the slit is being created by clicking mouse button 1
# x: Canvas X coordinate
# y: Canvas Y coordinate
# px: Image pixel X coordinate
# py: Image pixel Y coordinate
# wid: Slit width [arcsec]
# len: Slit length [arcsec]
# off: Slit offset [arcsec]
# pa: Slit position angle [deg]
# shape: Slit shape
# name: Target name
# ff: Focal field limits [mm]
# max: Maximum number of slits that can be created
# ref: If 1, slit is a reference slit
# mmff: If 1, slit dimensions are in [mm]

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 1}]

    set noms [fors::MXU_SLIT::nos]
    set noas [fors::MXU_SLIT::noas]
    if { $noms >= $mnomxus || $noas >= $max } {
        puts stdout "mxu_new_slit: Sorry, cannot handle more than $max [string toupper [$this cget -modus]] slits. Skipped."
        tracer LEAVING MAX_MXUs
        return
    }

    if { $debug && $verbose } {
        puts stdout "mxu_new_slit: Parameters"
        puts stdout "              click = >$click<"
        puts stdout "                  x = >$x<"
        puts stdout "                  y = >$y<"
        puts stdout "                 px = >$px<"
        puts stdout "                 py = >$py<"
        puts stdout "                wid = >$wid<"
        puts stdout "                len = >$len<"
        puts stdout "                off = >$off<"
        puts stdout "                 pa = >$pa<"
        puts stdout "              shape = >$shape<"
        puts stdout "               name = >$name<"
        puts stdout "                 ff = >$ff<"
        puts stdout "                max = >$max<"
        puts stdout "                ref = >$ref<"
        puts stdout "               mmff = >$mmff<"
        puts stdout " "
    }
    
    #-------------------
    # Center slit option
    #-------------------
    set slit_x $x
    set slit_y $y
    if { $click } {
        if { [$this cget -pics] } {
            # Click gave approximate screen coords.
            # Use RtdImagePick to center slit on image target and get
            # precise image pixel coordinates of its center (px,py)
            set picked {}
            pick_star
            # set FWHM [lindex $picked 5]
            set FWHM [expr {$FWHM_X + $FWHM_Y}]
            if { $debug && $verbose } {
                puts stdout ""
                puts stdout "             Picked = $picked"
                puts stdout "               FWHM = $FWHM"
            }
            if { $FWHM > 0 } {
                if { [catch {
                          $m_image convert coords $star_ix $star_iy image cx cy canvas
                      } result] } {
                    if { $debug } {
                        puts stdout "mxu_new_slit: Slit center (star_ix,star_iy)"
                        puts stdout "              Error converting ($star_ix,$star_iy) image to (cx,cy) canvas"
                        puts stdout "              $result"
                    }
                    set cx 0.0
                    set cy 0.0
                } else {
                    if { $debug && $verbose } {
                        puts stdout "            star_ix = >$star_ix< pick image X"
                        puts stdout "            star_iy = >$star_iy< pick image Y"
                        puts stdout "                 cx = >$cx< canvas pixel"
                        puts stdout "                 cy = >$cy< canvas pixel"
                    }
                }
                set slit_x $cx
                set slit_y $cy
                set px $star_ix
                set py $star_iy
                if { $debug && $verbose } {
                    puts stdout "             slit_x = >$slit_x<"
                    puts stdout "             slit_y = >$slit_y<"
                }
                if { $debug } {
                    puts stdout "mxu_new_slit:    Picked Object            | ($slit_x, $slit_y) canvas | ($px, $py) pixel"
                }
            } else {
                if { $debug } {
                    puts stdout "mxu_new_slit: Pick Object failed, no slit created!."
                }
                pick_technique
                set slit_x 0.0
                set slit_y 0.0
            }
        }
    }

    if { $slit_x == 0.0 && $slit_y == 0.0 } {
        # Do not create slit if pick failed.
        tracer LEAVING FAILED
        return
    }
    if { $mmff } {
        set cur_mmff [fors::MXU_SLIT::get_mmff]
        # this is the flag which determines
        # science mask (0) or standard mask mode (1)
        # sky mask (0) or technical mask mode (1)
        fors::MXU_SLIT::set_mmff $mmff
    }
    # Instances are C_mxus1, C_mxus2 ..., C_mxus<noms>
    set a [fors::MXU_SLIT C_mxus[incr noms 1] $m_canvas \
                                              $m_image  \
                                              $slit_x \
                                              $slit_y \
                                              -eqx $EQX \
                                              -slit_width $wid \
                                              -slit_length $len \
                                              -slit_offset $off \
                                              -pos_ang $pa \
                                              -shape $shape \
                                              -target_name $name \
                                              -ff $ff \
                                              -ref_slit $ref \
                                              -ON 1 \
                                              -child $this \
                                              -debug [$this cget -debug] \
                                              -verbosity [$this cget -verbosity] \
          ]
    if { $mmff } {
        fors::MXU_SLIT::set_mmff $cur_mmff
    }

    if { [string compare $name aperture] != 0 &&
         [string compare $name refslit]  != 0 } {
        # This is necessary for calc_zemax
        set targw(1,f) 1
        set targw(1,x) $px
        set targw(1,y) $py
        set targw(1,a) [C_mxus$noms get_RA]
        set targw(1,d) [C_mxus$noms get_DEC]
        if { $debug } {
            puts stdout "mxu_new_slit: C_mxus$noms, RA: $targw(1,a) |  DEC: $targw(1,d)"
        }
        switch -exact -- $shape {
          STRAIGHT -
            CIRCLE {
                # Optical correction of just created slit or circle,
                # We use targw 1
                calc_zemax 1 $targw(1,x) $targw(1,y)
                # targw(1,cx) and targw(1,cy) is not used
                C_mxus$noms put_OC  [list $targw(1,ca) $targw(1,cd)]
                # Apply correction of differential refraction (10.5.99)
                # Input slit position, corrected for optical distortion
                if { $dire_flag } {
                    calc_diff_refrac 1 $targw(1,ca) $targw(1,cd)
                }
                C_mxus$noms put_OC  [list $targw(1,ca) $targw(1,cd)]
            }
            CURVED {
                # Special note for CURVED
                # The click-point/center-gauss-point is not on the curved slit.
                # We determine the curved slitcenter, apply correction of
                # optical distortion and save correct values to mxu slit.
                # This point has only meaning for DRS
                set dummy [C_mxus$noms update_curved_center]
                catch { $m_image convert coords [lindex $dummy 0] [lindex $dummy 1] canvas \
                                                targw(1,x) targw(1,y) image }
                calc_zemax 1 $targw(1,x) $targw(1,y)
                # targw(1,cx) and targw(1,cy) is not used
                C_mxus$noms put_OC [list $targw(1,ca) $targw(1,cd)]
            }
        }
    }
    if { $debug && $verbose } {
        puts stdout "mxu_new_slit: END"
    }
    tracer LEAVING
}

#=============================================
private method hitms_create_slit { w sx sy } {
#=============================================
# Creates a new slit for the HIT-MS, a mode based on MXU
# w = skycat image
# sx,sy = screen coords of slit center
    global ::m_i

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 1}]

    if { $debug && $verbose } {
        puts stdout "hitms_create_slit: Parameters"
        puts stdout "                   w = >$w<"
        puts stdout "                  sx = >$sx<"
        puts stdout "                  sy = >$sy<"
    }

    # stop = 1: do nothing (when selecting a reference object)
    # stop = 0: do the normal procedure
    if { $stop } {
        tracer LEAVING STOP
        return
    }

    # Adapt mos slit length
    set mos_sl  [$this cget -slit_length]
    if { [string compare $mos_sl real] == 0 } {
        set mxu_sl 22.0
    } else {
        set mxu_sl $mos_sl
    }
    if { [string match *CIRCLE* [$this cget -slit_shape]] } {
        set mxu_sl [$this cget -slit_width]
    }

    # Adapt collimator
    set ff [list $CCDMASTER_XSRMIN $CCDMASTER_XSRMAX $CCDMASTER_YSRMIN $CCDSLAVE_YSRMAX]
    if { [string match *HR* [$this cget -collimat]] } {
        set ff [list $CCDMASTER_XHRMIN $CCDMASTER_XHRMAX $CCDMASTER_YHRMIN $CCDSLAVE_YHRMAX]
    }

    # The number of hitms slits can be 1 or 2
    # depending on the slit length.
    set sl [$this cget -slit_length]
    switch $sl {
        5.0 {
            set hmslit_max 2
        }
        10.0 {
            set hmslit_max 1
        }
        default {
            set hmslit_max 1
        }
    }

    if { [$this noas] >= $hmslit_max } {
        info_dialog " Sorry, cannot have more than $hmslit_max \n HIT-MS slit(s) of $sl'' length." .skycat1
        tracer LEAVING MAX_SLIT
        return
    }

    if { [catch {
              $m_image convert coords $sx $sy screen x y canvas
          } result] } {
        if { $debug } {
            puts stdout "              Slit center (sx,sy)"
            puts stdout "              Error converting ($sx,$sy) screen to (x,y) canvas"
            puts stdout "              $result"
        }
        set x 0.0
        set y 0.0
    } else {
        if { $debug && $verbose } {
            puts stdout "                  x = >$x< canvas pixel"
            puts stdout "                  y = >$y< canvas pixel"
        }
    }
    if { [catch {
              $m_image convert coords $sx $sy screen px py image
          } result] } {
        if { $debug } {
            puts stdout "              Slit center (sx,sy)"
            puts stdout "              Error converting ($sx,$sy) screen to (px,py) image"
            puts stdout "              $result"
        }
        set px 0.0
        set py 0.0
    } else {
        if { $debug && $verbose } {
            puts stdout "                 px = >$px< image pixel"
            puts stdout "                 py = >$py< image pixel"
        }
    }
    if { $debug } {
        puts stdout "hitms_create_slit: Cursor ($sx, $sy) screen | ($x, $y) canvas | ($px, $py) pixel"
    }
    # Create one instance of MXU SLIT
    mxu_new_slit 1 $x $y $px $py [$this cget -slit_width] $mxu_sl [$this cget -slit_offset] 0.0 [$this cget -slit_shape] NN $ff $hmslit_max 0 0

    if { [$this noas] >= $hmslit_max } {
        $m_mb.layout.sl entryconfigure $m_i(Layout_SlitLength_10_0) -state disabled
    }

    # Update slit positioning method list
    if { [string first manual $AutoposMethod] == -1 } {
        append AutoposMethod "manual "
    }

    # For status panel
    set last_action "Create HIT-MS \#[fors::MXU_SLIT::nos]"
    catch {SP update}
    if { $debug } {
        puts stdout "hitms_create_slit: END"
    }
    tracer LEAVING
}

#===========================================
private method mxu_create_slit { w sx sy } {
#===========================================
# Creates a new slit for the FORS2-MXU
# w = skycat image
# sx,sy = screen coords of slit center
# Removed unused parameters:
# but = fourth parameter was mouse button, but removed because it was always
#       zero and not used.
# on = fifth parameter's purpose was unknown, always 1 and not used
# click = sixth parameter was useless since this method is bound to mouse
#         button-1 click.

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 1}]

    if { $debug && $verbose } {
        puts stdout "mxu_create_slit: Parameters"
        puts stdout "                 w = >$w<"
        puts stdout "                sx = >$sx< click screen X"
        puts stdout "                sy = >$sy< click screen Y"
    }

    # stop = 1: do nothing (when selecting a reference object)
    # stop = 0: do the normal procedure
    if { $stop } {
        tracer LEAVING STOP
        return
    }

    # Adapt mos slit length
    set mos_sl [$this cget -slit_length]
    if { [string compare $mos_sl real] == 0 } {
        set mxu_sl 22.0
    } else {
        set mxu_sl $mos_sl
    }
    if { [string match *CIRCLE* [$this cget -slit_shape]] } {
        set mxu_sl [$this cget -slit_width]
    }

    # Adapt collimator
    set ff [list $CCDMASTER_XSRMIN $CCDMASTER_XSRMAX $CCDMASTER_YSRMIN $CCDSLAVE_YSRMAX]
    if { [string match *HR* [$this cget -collimat]] } {
        set ff [list $CCDMASTER_XHRMIN $CCDMASTER_XHRMAX $CCDMASTER_YHRMIN $CCDSLAVE_YHRMAX]
    }

    # The number of mxu slits...
    if { [fors::MXU_SLIT::nos] >= $mnomxus } {
        info_dialog " Sorry, cannot handle more than\n $mnomxus MXU slits." .skycat1
        tracer LEAVING MAX_MXUs
        return
    }

    if { [catch {
              $m_image convert coords $sx $sy screen x y canvas
          } result] } {
        if { $debug } {
            puts stdout "              Slit center (sx,sy)"
            puts stdout "              Error converting ($sx,$sy) screen to (x,y) canvas"
            puts stdout "              $result"
        }
        set x 0.0
        set y 0.0
    } else {
        if { $debug && $verbose } {
            puts stdout "                  x = >$x< canvas pixel"
            puts stdout "                  y = >$y< canvas pixel"
        }
    }
    if { [catch {
              $m_image convert coords $sx $sy screen px py image
          } result] } {
        if { $debug } {
            puts stdout "              Slit center (sx,sy)"
            puts stdout "              Error converting ($sx,$sy) screen to (px,py) image"
            puts stdout "              $result"
        }
        set px 0.0
        set py 0.0
    } else {
        if { $debug && $verbose } {
            puts stdout "                 px = >$px< image pixel"
            puts stdout "                 py = >$py< image pixel"
        }
    }
    if { $debug } {
        puts stdout "mxu_create_slit: Cursor ($sx, $sy) screen | ($x, $y) canvas | ($px, $py) pixel"
    }
    # Create one instance of MXU SLIT
    mxu_new_slit 1 $x $y $px $py [$this cget -slit_width] $mxu_sl [$this cget -slit_offset] 0.0 [$this cget -slit_shape] NN $ff $mnomxus 0 0

    # Update slit positioning method list
    if { [string first manual $AutoposMethod] == -1 } {
        append AutoposMethod "manual "
    }

    # For status panel
    set last_action "Create MXU \#[fors::MXU_SLIT::nos]"
    catch {SP update}
    if { $debug } {
        puts stdout "mxu_create_slit: END"
    }
    tracer LEAVING
}

#=====================================
private method mxu_create_refslit {} {
#=====================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set ff [list $XSRMIN $XSRMAX $YSRMIN $YSRMAX]
    if { [string match *HR* [$this cget -collimat]] } {
        set ff [list $XHRMIN $XHRMAX $YHRMIN $YHRMAX]
    }
    for { set i 0 } { $i < $nf_refslit } { incr i } {
        mxu_new_slit 0 $mxurefsx($i) $mxurefsy($i) $mxurefsx($i) $mxurefsy($i) $mxurefwd($i) $mxurefle($i) $mxurefof($i) $mxurefro($i) $mxureftp($i) refslit $ff $mnomxus 1 1
    }
    set nf_refslit [fors::MXU_SLIT::nos]
    tracer LEAVING
}

#============================================
public method hitms_delete_slit { w sx sy } {
#============================================
# Deletes an individual HIT-MS slit

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    global ::m_i
    catch { $m_image convert coords $sx $sy screen x y canvas }
    set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
    set my [string range $curtag  3 end]
    # puts stdout ">[$w find closest $x $y]< \
    #              >[$w gettags [$w find closest $x $y]]< \
    #              curtag= >$curtag<  my= >$my<"

    set wid .skycat1.mxu_save
    catch {destroy $wid}
    set w [DialogWidget $wid \
                        -text "You are deleting HIT-MS slit #$my" \
                        -bitmap warning \
                        -default -1 \
                        -buttons {OK Cancel} \
          ]
    if { [$w activate] } {
        tracer LEAVING ACTIVATE
        return
    }

    C_mxus$my erase
    if { [$this noas] < $hmslit_max } {
        $m_mb.layout.sl entryconfigure $m_i(Layout_SlitLength_10_0) -state normal
    }

    # For statuspanel
    set last_action "Delete HIT-MS \#$my"
    catch {SP update}
    tracer LEAVING
}

#==========================================
public method mxu_delete_slit { w sx sy } {
#==========================================
# Deletes an individual MXU slit

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    catch { $m_image convert coords $sx $sy screen x y canvas }
    set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
    set my [string range $curtag  3 end]
    # puts stdout ">[$w find closest $x $y]< \
    #              >[$w gettags [$w find closest $x $y]]< \
    #              curtag= >$curtag<  my= >$my<"

    #- There fix ref slits that cannot be deleted -#
    if { $my <= $nf_refslit } {
        puts "Fixed mxu referece slit ($my): Cannot be deleted !"
        tracer LEAVING 1
        return
    }

    set wid .skycat1.mxu_save
    catch {destroy $wid}
    set w [DialogWidget $wid \
                        -text "You are deleting MXU slit #$my" \
                        -bitmap warning \
                        -default -1 \
                        -buttons {OK Cancel} \
          ]
    if { [$w activate] } {
        tracer LEAVING 2
        return
    }

    C_mxus$my erase

    # For statuspanel
    set last_action "Delete MXU \#$my"
    catch {SP update}
    tracer LEAVING
}

#===================================================
public method mxu_delete_all_slits { {keep_rs 0} } {
#===================================================
# Deletes all MXU slits from the canvas
# keep_rs: if 1, do not delete reference slits

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { [string compare [$this cget -modus] mxu] == 0 ||
         [string compare [$this cget -modus] hitms] == 0 } {
        set ns [fors::MXU_SLIT::nos]
        if { $ns > 0 } {
            if { $debug } {
                puts stdout "mxu_delete_all_slits: deleting all MXU slits..."
            }
            for { set i 1 } { $i <= $ns } { incr i 1 } {
                set keep [expr { $keep_rs && $i <= $nf_refslit && [string compare [$this cget -modus] mxu] == 0 }]
                if { !$keep } {
                    delete object C_mxus$i
                    if { $debug && $verbose } {
                        puts stdout "                      deleted C_mxus$i"
                    }
                }
            }
            if { $debug } {
                puts stdout "                      done."
            }
            # For statuspanel
            set last_action "Delete ALL MXUs "
            catch {SP update}
        }
        fors::MXU_SLIT::set_mmff 0
        if { $debug && $verbose } {
            puts stdout "  MXU ns   = [fors::MXU_SLIT::nos]"
            puts stdout "  MXU nrs  = [fors::MXU_SLIT::nor]"
            puts stdout "  MXU nas  = [fors::MXU_SLIT::noas]"
            puts stdout "  MXU nars = [fors::MXU_SLIT::noars]"
            puts stdout "  MXU mmff = [fors::MXU_SLIT::get_mmff]"
            puts stdout "  MXU mid  = [fors::MXU_SLIT::get_MASK_NAID]"
        }
    }
    tracer LEAVING
}

#==========================================
public method mxu_ref_slit { w sx sy on } {
#==========================================
# Assigns reference MXU slit
# Unused method, deactivated in bind_mxu

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    catch { $m_image convert coords $sx $sy screen x y canvas }
    set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
    set my [string range $curtag  3 end]

    # Circle slits are no longer permitted to be used as
    # reference slits (WH 8/2000)

    # a) it must be a straight slit
    if { [string match *STRAIGHT* [C_mxus$my cget -shape]] } {
        # b) not more than 10 ref-slits allowed
        # set nors [fors::MXU_SLIT::nor]
        # the ::nor does not work, since we have to delete
        # refslits when noms = 0, meaning when the mask is
        # reset
        set nors [$this noars]
        if { $nors >= 10 && $on } {
            puts stdout "Warning: Not more than 10 ref-slits allowed"
        } else {
            # Do not try to config the slit as a ref-0slit in case
            # it is already a refslit
            set already [C_mxus$my cget -ref_slit]
            if { $already == $on } {
                puts stdout "Warning: slit $my is already a reference"
            } else {
                C_mxus$my configure -ref_slit $on
                C_mxus$my plot_label e w
            }
        }

        # For status panel
        if { $on } {
            set last_action "Reference Slits + \#$my"
            incr sp_rsc 1
        } else {
            set last_action "Reference Slits - \#$my"
            incr sp_rsc -1
        }
        catch {SP update}
    } else {
        info_dialog "CURVED and CIRCLE slits \n cannot be reference slits" .skycat1
        puts stdout "CURVED and CIRCLE slits cannot be reference slits" .skycat1
    }
    tracer LEAVING
}

#============================================
public method hitms_config_slit { w sx sy } {
#============================================
# External configuration of an individual HIT-MS slit
#  straight: width, length offset and posang
#  circle  : radius
#  curved  : ???

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    catch { $m_image convert coords $sx $sy screen x y canvas }
    set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
    set my [string range $curtag  3 end]
    set shape [C_mxus$my cget -shape]

    global .mxw_$my mw_list

    switch -exact [C_mxus$my cget -shape] {
        STRAIGHT {
            if { [catch {wm deiconify .mxw_$my}] } {
                # PBG: Slit offset deactivated
                # EntryForm .mxw_$my -title "Edit HIT-MS Slit #$my for [C_mxus$my cget -target_name]" \
                #                    -labels {Width/'' Offset/'' Inclin/} \
                #                    -command "fors::FORS::my_mask config_hitms $my" \
                #                    -values  "[C_mxus$my cget -slit_width] \
                #                              [C_mxus$my cget -slit_offset] \
                #                              [C_mxus$my cget -pos_ang]"
                EntryForm .mxw_$my -title "Edit HIT-MS Slit #$my for [C_mxus$my cget -target_name]" \
                                   -labels {Width/'' Inclin/} \
                                   -command "fors::FORS::my_mask config_hitms $my" \
                                   -values  "[C_mxus$my cget -slit_width] \
                                             [C_mxus$my cget -pos_ang]"
                append mw_list " .mxw_$my"
            } else {
                catch { wm iconify .mxw_$my }
                catch { wm deiconify .mxw_$my }
                # PBG: Slit offset deactivated
                # .mxw_$my configure \
                #                -values  "[C_mxus$my cget -slit_width] \
                #                          [C_mxus$my cget -slit_offset] \
                #                          [C_mxus$my cget -pos_ang]"
                .mxw_$my configure \
                               -values  "[C_mxus$my cget -slit_width] \
                                         [C_mxus$my cget -pos_ang]"
            }
        }
        CURVED {
            if { [catch {wm deiconify .mxw_$my}] } {
                EntryForm .mxw_$my -title "Edit HIT-MS Curve #$my for [C_mxus$my cget -target_name]" \
                                   -labels {Width/''} \
                                   -command "fors::FORS::my_mask config_mxu $my" \
                                   -values  "[C_mxus$my cget -slit_width]"
                append mw_list " .mxw_$my"
            } else {
                catch { wm iconify .mxw_$my }
                catch { wm deiconify .mxw_$my }
                .mxw_$my configure \
                               -values  "[C_mxus$my cget -slit_width]"
            }
            # Move the Bezier point like the mask
            # 1. make slit $my active (enable )
            # 2. move and update simulatneously
            # 3. button release (disable)
        }
        CIRCLE {
            if { [catch {wm deiconify .mxw_$my}] } {
                EntryForm .mxw_$my -title "Edit HIT-MS Circle #$my for [C_mxus$my cget -target_name]" \
                                   -labels {Diameter/''} \
                                   -command "fors::FORS::my_mask config_mxu $my" \
                                   -values  "[C_mxus$my cget -slit_width]"
                append mw_list " .mxw_$my"
            } else {
                catch { wm iconify .mxw_$my }
                catch { wm deiconify .mxw_$my }
                .mxw_$my configure \
                               -values  "[C_mxus$my cget -slit_width]"
            }
            # Move the Bezier point like the mask
        }
    }
    set last_action "Configure HIT-MS  \#$my"
    catch {SP update}
    tracer LEAVING
}

#============================================
public method mos_config_slit { w sx sy } {
#============================================
# External configuration of an individual MOS slit width

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    catch { $m_image convert coords $sx $sy screen x y canvas }
    set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
    set my [string range $curtag  1 end]
    # puts stdout ">[$w find closest $x $y]< \
    #           >[$w gettags [$w find closest $x $y]]< \
    #          curtag= >$curtag<  my= >$my<"

    global .msw_$my mw_list

    if [catch {wm deiconify .msw_$my}] {
        EntryForm .msw_$my -title "Slit Width of MOS #$my in arcsec" \
                           -labels {Width} \
                           -command "fors::FORS::my_mask config_mos $my" \
                           -values  "$mos($my,b)"
        append mw_list " .msw_$my"
    } else {
        catch { wm iconify .msw_$my }
        catch { wm deiconify .msw_$my }
        .msw_$my configure -values "$mos($my,b)"
    }
    set last_action "Set Slit Width \#$my"
    catch {SP update}
    tracer LEAVING
}

#==========================================
public method mxu_config_slit { w sx sy } {
#==========================================
# External configuration of an individual MXU slit
#  straight: width, length offset and posang
#  circle  : radius
#  curved  : ???

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    catch { $m_image convert coords $sx $sy screen x y canvas }
    set curtag [lindex [$w gettags [$w find closest $x $y]] 2]
    set my [string range $curtag  3 end]
    set shape [C_mxus$my cget -shape]

    if { $my <= $nf_refslit } {
        puts "Fixed mxu referece slit ($my): Cannot be changed !"
        tracer LEAVING 1
        return
    }

    global .mxw_$my mw_list

    switch -exact [C_mxus$my cget -shape] {
        STRAIGHT {
            if { [catch {wm deiconify .mxw_$my}] } {
                # PBG: Slit offset deactivated
                # EntryForm .mxw_$my -title "Edit MXU Slit #$my for [C_mxus$my cget -target_name]" \
                #                    -labels {Width/'' Length/'' Offset/'' Inclin/} \
                #                    -command "fors::FORS::my_mask config_mxu $my" \
                #                    -values  "[C_mxus$my cget -slit_width] \
                #                              [C_mxus$my cget -slit_length] \
                #                              [C_mxus$my cget -slit_offset] \
                #                              [C_mxus$my cget -pos_ang]"
                EntryForm .mxw_$my -title "Edit MXU Slit #$my for [C_mxus$my cget -target_name]" \
                                   -labels {Width/'' Length/'' Inclin/} \
                                   -command "fors::FORS::my_mask config_mxu $my" \
                                   -values  "[C_mxus$my cget -slit_width] \
                                             [C_mxus$my cget -slit_length] \
                                             [C_mxus$my cget -pos_ang]"
                append mw_list " .mxw_$my"
            } else {
                catch { wm iconify .mxw_$my }
                catch { wm deiconify .mxw_$my }
                # PBG: Slit offset deactivated
                # .mxw_$my configure \
                #                -values  "[C_mxus$my cget -slit_width] \
                #                          [C_mxus$my cget -slit_length] \
                #                          [C_mxus$my cget -slit_offset] \
                #                          [C_mxus$my cget -pos_ang]"
                .mxw_$my configure \
                               -values  "[C_mxus$my cget -slit_width] \
                                         [C_mxus$my cget -slit_length] \
                                         [C_mxus$my cget -pos_ang]"
            }
        }
        CURVED {
            if { [catch {wm deiconify .mxw_$my}] } {
                EntryForm .mxw_$my -title "Edit MXU Curve #$my for [C_mxus$my cget -target_name]" \
                                   -labels {Width/''} \
                                   -command "fors::FORS::my_mask config_mxu $my" \
                                   -values  "[C_mxus$my cget -slit_width]"
                append mw_list " .mxw_$my"
            } else {
                catch { wm iconify .mxw_$my }
                catch { wm deiconify .mxw_$my }
                .mxw_$my configure \
                               -values  "[C_mxus$my cget -slit_width]"
            }
            # Move the Bezier point like the mask
            # 1. make slit $my active (enable )
            # 2. move and update simulatneously
            # 3. button release (disable)
        }
        CIRCLE {
            if { [catch {wm deiconify .mxw_$my}] } {
                EntryForm .mxw_$my -title "Edit MXU Circle #$my for [C_mxus$my cget -target_name]" \
                                   -labels {Diameter/''} \
                                   -command "fors::FORS::my_mask config_mxu $my" \
                                   -values  "[C_mxus$my cget -slit_width]"
                append mw_list " .mxw_$my"
            } else {
                catch { wm iconify .mxw_$my }
                catch { wm deiconify .mxw_$my }
                .mxw_$my configure \
                               -values  "[C_mxus$my cget -slit_width]"
            }
            # Move the Bezier point like the mask
        }
    }
    set last_action "Configure MXU  \#$my"
    catch {SP update}
    tracer LEAVING
}

#=========================================
public method config_hitms { my wiofpo } {
#=========================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # my = current hitms slit number
    # wiofpo = list of width/'' offset/'' slitposang/deg
    set wi [lindex $wiofpo 0]
    # PBG: Slit offset deactivated
    # set of [lindex $wiofpo 1]
    # set po [lindex $wiofpo 2]
    set of 0.0
    set po [lindex $wiofpo 1]

    # in case of circle diameter
    # puts stdout ">$wi< >$le<"
    if {$of==""} { set of 0.0 }
    if {$po==""} { set po 0.0 }

    # If a negative got through
    set wi [expr {abs($wi)}]

    set le [$this cget -slit_length]

    set l_size [expr sqrt($wi*$wi+$le*$le)  / cos($po/180.0*3.1415)]

    if {$l_size > 40.0} {
        # 20mm is the upper limit to automatic mask manufacuring
        # see VIRMOS-MMU documents
        puts stdout [format "Error: Slit size %5.3f'' \
                             exceeds 20mm (40'') in focf" $l_size]
        puts stdout "HITMS slit setup Canceled"
        tracer LEAVING 1
        return
    }

    # Maximum offset from center of slit
    # PBG: Slit offset deactivated
    # set maxof [expr { abs($le)/2 }]
    # if { [expr { abs($of) }] > $maxof } {
    #     tracer MSG "[tracer TAB  ]Offset is too much $of, maximum is -+$maxof"
    #     set of [expr { $maxof * $of / abs($of) }]
    # }

    switch -exact -- [C_mxus$my cget -shape] {
        CIRCLE {
            C_mxus$my configure -slit_width  $wi
            C_mxus$my configure -slit_length $le
        }
        STRAIGHT {
            C_mxus$my configure -slit_width  $wi
            C_mxus$my configure -slit_length $le
            # PBG: Slit offset deactivated
            # C_mxus$my configure -slit_offset $of
            C_mxus$my configure -pos_ang     $po
        }
        CURVED {
            C_mxus$my configure -slit_width  $wi
        }
    }

    C_mxus$my plot_slit
    if { $label } {
        C_mxus$my plot_label e w
    }

    if { [string match *CURVED* [C_mxus$my cget -shape]] } {
        update_bez_oc $my
        set a [size_CS $my]
    }
    tracer LEAVING
}

#======================================
public method config_mos { my width } {
#======================================
# Apply only if MOS slit width is
# larger than 0.2 arcsec = 0.1 mm focal plane = 1 pixel

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $width >= 0.2 } {
        # store new slit width
        set mos($my,b) [format "%3.1f" $width]
        # set mos array
        catch { $m_image convert coords $targw($my,a) $targw($my,d) "wcs $EQX" \
                                sx sy screen }
        catch { $this move_slit $m_canvas $sx $sy 1 0 }
    } else {
        puts stdout "Error: slit width not accepted"
    }
    tracer LEAVING
}

#=========================================
public method config_mxu { my wileofpo } {
#=========================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # my = current mxu slit number
    # wileofpo = list of width/'' length/'' offset/'' slitposang/deg
    set wi [lindex $wileofpo 0]
    set le [lindex $wileofpo 1]
    # PBG: Slit offset deactivated
    # set of [lindex $wileofpo 2]
    # set po [lindex $wileofpo 3]
    set of 0.0
    set po [lindex $wileofpo 2]

    # in case of circle diameter
    # puts stdout ">$wi< >$le<"
    if {$of==""} { set of 0.0 }
    if {$po==""} { set po 0.0 }
    if {$le==""} { set le $wi }

    # If a negative got through
    set wi [expr {abs($wi)}]
    set le [expr {abs($le)}]

    set l_size [expr sqrt($wi*$wi+$le*$le)  / cos($po/180.0*3.1415)]

    if {$l_size > 40.0} {
        # 20mm is the upper limit to automatic mask manufacuring
        # see VIRMOS-MMU documents
        puts stdout [format "Error: Slit size %5.3f'' \
                             exceeds 20mm (40'') in focf" $l_size]
        puts stdout "MXU slit setup Canceled"
        tracer LEAVING 1
        return
    }

    # Maximum offset from center of slit
    # PBG: Slit offset deactivated
    # set maxof [expr { abs($le)/2 }]
    # if { [expr { abs($of) }] > $maxof } {
    #     tracer MSG "[tracer TAB  ]Offset is too much $of, maximum is -+$maxof"
    #     set of [expr { $maxof * $of / abs($of) }]
    # }

    switch -exact -- [C_mxus$my cget -shape] {
        CIRCLE {
            C_mxus$my configure -slit_width  $wi
            C_mxus$my configure -slit_length $le
        }
        STRAIGHT {
            C_mxus$my configure -slit_width  $wi
            C_mxus$my configure -slit_length $le
            # PBG: Slit offset deactivated
            # C_mxus$my configure -slit_offset $of
            C_mxus$my configure -pos_ang     $po
        }
        CURVED {
            C_mxus$my configure -slit_width  $wi
        }
    }

    C_mxus$my plot_slit
    if { $label } {
        C_mxus$my plot_label e w
    }

    if { [string match *CURVED* [C_mxus$my cget -shape]] } {
        update_bez_oc $my
        set a [size_CS $my]
    }
    tracer LEAVING
}

#==============================
public method wave_length { } {
#==============================
# External configuration of the wavelength range
# Cancel button -> destroy

    tracer ENTERING
    global .wal

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { [catch {wm deiconify .wal}] } {
        EntryForm .wal -title "grism : [$this cget -grism] \n \
                               filter: [$this cget -filter]\n \
                               range : $A_disp($good,min) nm < \
                               $A_disp($good,cen) nm < $A_disp($good,max) nm\n \
                               Enter New Wavelength Interval :" \
                       -labels {Min Max} \
                       -command "fors::FORS::my_mask wave_length_CB" \
                       -values "$L_MIN $L_MAX"
    } else {
        .wal configure -values "$L_MIN $L_MAX"
    }
    tracer LEAVING
}

#============================================
public method wave_length_CB  { range } {
#============================================
    tracer ENTERING
    wave_config_mask [concat $range user]
    tracer LEAVING
}

#===================================
public method select_mxu_name  { } {
#===================================
# Select MASK_ID and proposal ID fro MXU
# Cancel button -> destroy

    tracer ENTERING
    global .mas

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { [catch {wm deiconify .mas}] } {
        EntryForm .mas -title "MASK NA --- MASK ID" \
                       -labels {MASK_NA MASK_ID} \
                       -command "fors::MXU_SLIT::set_MASK_NAID" \
                       -values [fors::MXU_SLIT::get_MASK_NAID]
    } else {
        .mas configure -values [fors::MXU_SLIT::get_MASK_NAID]
    }
    tkwait window .mas

    set reply [fors::MXU_SLIT::get_MASK_NAID]
    tracer LEAVING
    return $reply
}

#==================================
public method select_set_dir  { } {
#==================================
# External configuration set_dir
# Cancel button -> destroy

    tracer ENTERING
    global .sd

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    puts stdout "old set_dir = >$set_dir<"

    if { [catch {wm deiconify .sd}] } {
        EntryForm .sd -title "Directory of setup files" \
                      -labels {Directory} \
                      -command "$this configure -set_dir" \
                      -values $set_dir
    } else {
        .sd configure -values $set_dir
    }
    tkwait window .sd

    set last_action "Set SetUpDir"
    catch {SP update}
    tracer LEAVING
}

#===================================
public method select_filename  { } {
#===================================
# External configuration of the project name

    tracer ENTERING
    global .waf

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # puts stdout "new filename = [$this cget -file_name]"

    if {[catch {wm deiconify .waf}]} {
        EntryForm .waf -title "ProjectName (8 letters)" \
                       -labels {ProjectName} \
                       -command  "$this configure -file_name" \
                       -values [$this cget -file_name]
    } else {
        catch { wm iconify .waf }
        catch { wm deiconify .waf }
        .waf configure -values [$this cget -file_name]
    }
    tkwait window .waf

    # puts stdout "new filename = [$this cget -file_name]"
    set last_action "Set ProjectName"
    catch {SP update}
    tracer LEAVING
}

#=================================
private method select_cat { sc } {
#=================================
# show the filenames instead of the internal
# widget names

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set allnames ""
    foreach w $sc {
        set filename [lindex [$w info variable instance_idx_] 4]
        set filename [split $filename ","]
        set filename [lindex $filename 0]
        if { [string match */* $filename] } {
            set filename [split $filename "/"]
            set ll [llength $filename]
            set filename [lindex $filename [expr $ll-1]]
        } else {
            set filename "{$filename}"
        }
         set allnames "$allnames $filename"
    }

    # show the List dialog

    set wid .skycat1.ld
    catch { destroy $wid }
    set ld_flag [ListDialog $wid -text "Select catalog :" \
                                 -choice $allnames \
                                 -modal 0]
    #choice should be the tail of the file name

    # and return the internal widget name again
    set s [$ld_flag activate]
    puts stdout "selected : >$s<"
    foreach w $sc {
       set test [lindex [$w info variable instance_idx_] 4]
       if { [string match *$s* $test] } {
           tracer LEAVING 1
           return $w
       }
    }
    tracer LEAVING
}

#====================================
public method get_calib_passwd  { } {
#====================================
# External configuration set_dir
# Cancel button -> destroy

    tracer ENTERING
    global .cpwd

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set passwd cloud

    # science masks or sometimes called sky masks
    # are user defined masks van be saved as gerber
    # format to be manufactored by the mmu of the
    # VIRMOS consortium. Every fims user is permitted  to
    # save his own created mask in gerber format in order
    # to be manufacured by the mmu.
    # standard masks or sometimes called technical masks
    # have a longerlifetime than a science maks, they
    # can be used more often by different astromomers.
    # To provide the fims-maintainer
    # the exclusive permission to
    # a) define new standard masks
    # b) save standard masks in gerber format
    # c) manufacture new standard masks
    # d) declare the new standard mask in the
    #    fims/config/standard_mask_list
    # we introduce the fims maintenance password
    # fims setups with 900000<maskid<900100 require
    # the password to be saved in gerber format

    # on odd days the password is different
    append passwd s
    set cu_day [exec date -u +%2d]
    if { [expr {($cu_day/2) * 2}] != $cu_day } {
        # today is unfortunately an odd day
        # we change the password
        set passwd hokuspokus
    }

    if { [catch {wm deiconify .cpwd}] } {
        EntryForm .cpwd -title "Maintenance Password" \
                        -labels {Password} \
                        -command  "$this configure -calib_passwd" \
                        -values  ""
    } else {
        .cpwd configure -values ""

    }
    tkwait window .cpwd

    # puts stdout "user input= >$calib_passwd< passwd= >$passwd<"
    tracer LEAVING
    return [string match $passwd $calib_passwd]
}

#=========================================
public method test_optic_distor_FORS { } {
#=========================================
# Handles empirical optical distortion of FORS
# polynomic functions

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    puts stdout "test optical distortion"
    puts stdout "5 input distortion polynoms"
    puts stdout "slit p0 p1 p2 p3 p4"
    for { set i 1 } { $i <= 19 } { incr i 1 } {
        set y [expr $i * 1.0]
        puts stdout [format "%2d %7.6f %7.6f %7.6f %7.6f %7.6f"  \
                             $i [p0 $y] [p1 $y] [p2 $y] [p3 $y] [p4 $y]]
    }

    puts stdout "\n\n"
    puts stdout "slit -100 -75 -50  -25 0   25   50   75   100"
    for { set i 1 } { $i <= 19 } { incr i 1 } {
        set y [expr {$i * 1.0}]; # slit number
        puts stdout [format "%2d %7.6f %7.6f %7.6f %7.6f %7.6f %7.6f %7.6f %7.6f %7.6f"  \
                             $i [od_F -100 $y] \
                                [od_F -75  $y] \
                                [od_F -50  $y] \
                                [od_F -25  $y] \
                                [od_F   0  $y] \
                                [od_F  25  $y] \
                                [od_F  50  $y] \
                                [od_F  75  $y] \
                                [od_F 100  $y]]
    }

    # for { set i -120 } { $i <= 120 } { incr i 10 } {
    #     set x  expr $i * 1.0]
    #     set y 1.0
    #     set a [$this optic_distor_FORS $x $y]
    # }

    puts stdout "y=  -100  -80   -60   ...   60   80   100"
    for { set i -100 } { $i <= 100 } { incr i 50 } {
        set x [expr {1.0 * $i}]
        set plot_string  [format " %5.2f" $x]
        for { set j 1 } { $j <=19 } { incr j 1 } {
            set y [expr {1.0 * $j}]
            set test($j) [$this od_F $x $y]
            # puts stdout $test($j)
            append plot_string [format " %8.4f" $test($j)]
        }
        puts stdout $plot_string
    }
    puts stdout "\n\n"

    puts stdout "y=  -100  -80   -60   ...   60   80   100"
    for { set i 40 } { $i <= 60 } { incr i 1 } {
        set x [expr {1.0 * $i}]
        set plot_string  [format " %5.2f" $x]
        for { set j 40 } { $j < 60 } { incr j 1 } {
            set y [expr {1.0 * $j}]
            set test($j) [$this od_F $x $y]
            # puts stdout $test($j)
            append plot_string [format " %6.3f" $test($j)]
        }
        puts stdout $plot_string
    }

    # puts stdout "y=  -120  -110  -100  ...  +100  +110  +120"
    # for { set i -120 } { $i <= 120 } { incr i 10 } {
    #     set x [expr {1.0 * $i}]
    #     set plot_string  [format " %5.2f" $x]
    #     for { set j -5 } { $j < 6 } { incr j 1 } {
    #         set y [expr {20.0 * $j}]
    #         set test($j) [$this optic_distor_FORS $x $y]
    #         # puts stdout $test($j)
    #         append plot_string [format " %6.3f" $test($j)]
    #     }
    #     puts stdout $plot_string
    # }

    test_ablage
    puts stdout " ablage_SR (1462.32)= 4.40 = [ablage_SR 1462.32]"
    puts stdout " ablage_HR (1460.60)= 22.35 = [ablage_HR 1460.60]"
    puts stdout "distortion finished"
    tracer LEAVING
}

#============================
private method od_F { x y } {
#============================
# handles empirical optical distortion of FORS1
# polynomic functions
# p0 at x = -100
# p1 at x = -50
# p2 at x = 0
# p3 at x = +50
# p4 at x = +100
# function  argument is the y-value (slit-number)
# x in mm focal plane
# y in slit number [1..19]

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set ip 5; set im 4
    set xp 100; set xm 50
    set flag 1
    for { set i 0 } { $i<5 } { incr i 1 } {
        set xt [expr -100.0 + $i * 50.0]
        if { $flag } {
            if { $x < $xt } {
                set ip $i; set im [expr $i-1]
                set xp $xt; set xm [expr $xp-50.0]
                set flag 0
            }
        }
    }
    # puts stdout "$xm < $x $xp, $im < $ip"

    switch -exact -- $ip  {
        0 {
            #extrapolate to minimum
            set pp [p1 $y]
            set pm [p0 $y]
            set p [expr ($pm*($xp-$x)-$pp*($xm-$x))/($xp-$xm)]
            #puts stdout "v x=$x,y=$y,pa=$pa,pb=$pb,xi=$xi,p=$p"
        }
        5 {
            # extrapolate to maximum
            set pp [p4 $y]
            set pm [p3 $y]
            set p [expr $pm + ($pp-$pm)*($x-$xm)/($xp-$xm)]
            #puts stdout "^ x=$x,y=$y,pa=$pa,pb=$pb,xi=$i,p=$p"
        }
        1 -
        2 -
        3 -
        4 {
            # interpolate
            set pp [p$ip $y]
            set pm [p$im $y]
            set p [expr $pm + ($pp-$pm)*($x-$xm)/($xp-$xm)]
            #puts stdout "I x=$x y=$y index=$index ,\
                          xa=$xa, pb=$pb pa=$pa p=$p,xi=$xi"
        }
    }
    tracer LEAVING
    return $p
}

#========================
private method p0 { y } {
#========================
# at x=-100

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    tracer LEAVING
    return [expr (-952.965+0.516767*$y-0.027334*$y*$y)/(-950.522)]
}

#========================
private method p1 { y } {
#========================
# at x = -50; y is slit number !!!

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    tracer LEAVING
    return [expr (-475.824+0.259736*$y-0.013703*$y*$y)/(-474.593)]
}

#========================
private method p2 { y } {
#========================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    tracer LEAVING
    return 1.0
}

#========================
private method p3 { y } {
#========================
# at x = +50

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    tracer LEAVING
    return [expr (475.543-0.228404*$y+0.012263*$y*$y)/474.479]
}

#========================
private method p4 { y } {
#========================
# at x = +100

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    tracer LEAVING
    return [expr (951.932-0.476773*$y+0.026347*$y*$y)/949.775]
}

#===============================
private method ablage_SR { r } {
#===============================
# ZEMAX radial optical correction
# r = radius in pixel from reference pixel

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set pszx [$m_image fits get "HIERARCH ESO DET CHIP1 PSZX"]
    if { $pszx == "" } {
        set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"]
        if { $binx == 1 } {
            set pszx 15
        } elseif { $binx == 2 } {
            set pszx 30
        } else {
            set pszx 15
        }
    } else {
        set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"]
        set pszx [expr {$pszx * $binx}]
    }

    puts "PSZX: $pszx"
    set r [expr {($r*$pszx)/24}]

    # AAG - PPRS 10127
    set dist1 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR1"]
    set dist2 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR2"]
    set dist3 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR3"]
    set chipName [$m_image fits get "HIERARCH ESO DET CHIP1 ID"]
    if { ($dist1 != "") && ($dist2 != "") && ($dist3 != "") } {
        puts stdout "DISTORTION: $chipName ccd, using fits keyword values"
        tracer LEAVING FITS
        return [expr {$dist1*$r + $dist2*$r*$r + $dist3*$r*$r*$r}]
    } else {
        if { [string compare [$this cget -camera] R] == 0 } {
            puts stdout "DISTORTION: $chipName  ccd, using FORS2 hardcoded values"
            tracer LEAVING R
            return [expr {$r*$r*$r*2.113310E-9 - 2.157780E-6*$r*$r + 5.367470E-4*$r}]
        } elseif { [string compare [$this cget -camera] B] == 0 } {
            puts stdout "DISTORTION: $chipName  ccd, using FORS1 hardcoded values"
            tracer LEAVING B
            return [expr {$r*$r*$r*2.113310E-9 - 2.157780E-6*$r*$r + 5.367470E-4*$r}]
        } else {
            puts stdout "DISTORTION: $chipName ccd, unknown camera, returning 0"
            tracer LEAVING U
            return 0
        }
    }
    puts "ablage_SR"
    tracer LEAVING
}

#===============================
private method ablage_HR { r } {
#===============================
# ZEMAX radial optical correction
# r = radius in pixel from reference pixel

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set pszx [$m_image fits get "HIERARCH ESO DET CHIP1 PSZX"]
    puts "PSZX: $pszx"

    if { $pszx == "" } {
        set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"]
        if { $binx == 1 } {
            set pszx 15
        } elseif { $binx == 2 } {
            set pszx 30
        } else {
            set pszx 15
        }
    } else {
        set binx [$m_image fits get "HIERARCH ESO DET WIN1 BINX"]
        set pszx [expr {$pszx * $binx}]
    }

    set r [expr {($r*$pszx)/24}]

    # AAG - PPRS 10127
    set dist1 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR1"]
    set dist2 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR2"]
    set dist3 [$m_image fits get "HIERARCH ESO INS IMAGE DISTOR3"]
    set chipName [$m_image fits get "HIERARCH ESO DET CHIP1 ID"]
    if { ($dist1 != "") && ($dist2 != "") && ($dist3 != "") } {
        puts stdout "DISTORTION: $chipName ccd, using fits keyword values"
        tracer LEAVING FITS
        return [expr {$dist1*$r + $dist2*$r*$r + $dist3*$r*$r*$r}]
    } else {
        if { [string compare [$this cget -camera] R] == 0 } {
            puts stdout "DISTORTION: $chipName ccd, using FORS2 hardcoded values"
            tracer LEAVING R
            return [expr {8.540E-9*$r*$r*$r - 2.464E-6*$r*$r + 6.815E-4*$r}]
        } elseif { [string compare [$this cget -camera] B] == 0 } {
            puts stdout "DISTORTION: $chipName ccd, using FORS1 hardcoded values"
            tracer LEAVING B
            return [expr {8.540E-9*$r*$r*$r - 2.464E-6*$r*$r + 6.815E-4*$r}]
        } else {
            puts stdout "DISTORTION: $chipName ccd, unknown camera, returning 0"
            tracer LEAVING U
            return 0
        }
    }
    puts "ablage_HR"
    tracer LEAVING
}

#================================
private method test_ablage {  } {
#================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    puts stdout "\n\n reference pixel at $fr_x $fr_y\ntest ablage SR"
    for { set j 0 } { $j <= 10 } { incr j 1 } {
        set y [expr $j*200]
        set y_r  [expr {($y-$fr_y)*($y-$fr_y)}]
        set strang ""
        for { set i 0 } { $i <= 10 } { incr i 1 } {
            set x [expr {$i*200}]
            set x_r  [expr {($x-$fr_x)*($x-$fr_x)}]
            set r [expr {sqrt($x_r+$y_r)}]
            set ab [ablage_SR  $r]
            set delx [expr {$ab/sqrt(1.0+$y_r/$x_r)}]

            append strang [format "%5.2f " $delx]
        }
        puts stdout $strang
    }
    puts stdout "\n\ntest ablage HR"
    for { set j 0 } { $j <= 10 } { incr j 1 } {
        set y [expr {$j*200}]
        set y_r  [expr {($y-$fr_y)*($y-$fr_y)}]
        set strang ""
        for { set i 0 } { $i <= 10 } { incr i 1 } {
            set x [expr {$i*200}]
            set x_r  [expr {($x-$fr_x)*($x-$fr_x)}]

            set r [expr {sqrt($x_r+$y_r)}]
            set ab [ablage_HR  $r]
            set delx [expr {$ab/sqrt(1.0+$y_r/$x_r)}]

            append strang [format "%5.2f " $delx]
        }
        puts stdout $strang
    }
    tracer LEAVING
}

#=============================
public method show_status {} {
#=============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    catch {SP show}
    tracer LEAVING
}

#=====================================
private method small_status_panel {} {
#=====================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set text ""
    # basic instrumental setup
    append text [format "% 9s : % 10s\n" "INST"   [$this cget -instrument]]
    append text [format "% 9s : % 10s\n" "MODE"   [$this cget -modus]]
    append text [format "% 9s : % 10s\n" "MASK" $mask]
    append text [format "% 9s : % 10s\n" "COLL"   [$this cget -collimat]]
    append text [format "% 9s : % 10s\n" "GRISM"  [$this cget -grism]]
    append text [format "% 9s : % 10s\n" "FILTER" [$this cget -filter]]
    append text [format "% 9s : % 10s\n" "WIDTH"  [$this cget -slit_width]]
    append text [format "% 9s : % 10s\n" "LENGTH" [$this cget -slit_length]]
    append text [format "% 9s : % 10s\n" "OFFSET" [$this cget -slit_offset]]

    if { [string match *hitms* [$this cget -modus]] ||
         [string match *mxu* [$this cget -modus]] } {
        set noms [fors::MXU_SLIT::nos]
        if { $noms > 0 } {
            set sp_rsc [$this noars]
            set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]
            append text [format "% 9s : % 10s\n" "MASK_NA"  [lindex $PM 0]]
            append text [format "% 9s : % 10s\n" "MASK_ID"  [lindex $PM 1]]
            append text [format "% 9s : % 10s\n" "TechMask" [fors::MXU_SLIT::get_mmff]]
            append text [format "% 9s : % 10d\n" "X_REFSLIT" $sp_rsc]
        }
        append text [format "% 9s : % 10s\n" "X_TYPE"  [$this cget -slit_shape]]
        append text [format "% 9s : % 10d\n" "X_NUMBER" $noms]
    }

    # other flags

    append text  [format "% 9s : % 10s\n" "CEN/SLIT" [$this cget -pics]]

    if { [C_ref_targ cget -mos10ali] } {
        set local ON
    } else {
        set local OFF
    }

    append text [format "% 9s : % 2d\n" "MOS10ALI" $local]
    append text [format "% 9s : % 10s\n" "RefTarg"  [C_ref_targ nort]]
    append text [format "% 9s : % 10s\n" "PROGRAM"  [$this cget -file_name]]
    append text [format "% 9s : % 10s\n" "FITS"  [$this cget -file_name]]
    append text [format "% 9s : % 10s\n" "loaded" $last_load_fn]
    append text [format "% 9s : % 10s\n" "saved"  $last_save_fn]

    info_dialog $text .skycat1
    tracer LEAVING
}

#===========================
private method open_log {} {
#===========================
# close the log file for the constructor and open
# a new log-file for the user commands

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    close $lfId
    set mod [$this cget -modus]${insId}
    set log_file $log_dir
    append log_file "/$mod.$exposure_counter.log"
    set lfId [open $log_file w]
    tracer LEAVING
}

# this migth become an external library

#============================================
private method ldelete_all { w_list value } {
#============================================
# modified ldelete
# delete all list elements by value (single character)

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set ix [lsearch -exact $w_list $value]
    set new $w_list
    while { $ix >= 0 } {
        set new [lreplace $w_list $ix $ix]
        set ix [lsearch -exact $new $value]
        set w_list $new
    }
    tracer LEAVING
    return $new
}

#======================================
private method ldelete { list value } {
#======================================
# taken from Welch, p34
# delete one list element by value

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set ix [lsearch -exact $list $value]
    if { $ix >= 0 } {
        tracer LEAVING 1
        return [lreplace $list $ix $ix]
    } else {
        tracer LEAVING 2
        return $list
    }
}

#=================================================
private method string_exchange { input old new } {
#=================================================
# exchange all characters in string 'old' by 'new'
# works only correctly if old and new are single chars

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set output ""
    set l [string length $input]
    for { set i 0 } { $i < $l } { incr i 1 } {
        set c [string index $input $i]
        if { $c != $old } {
            append output $c
        } else {
            append output $new
        }
    }
    tracer LEAVING
    return $output
}

#=============================================
private method string_delete { input value } {
#=============================================
# delete all characters of 'value' in string 'input'
# works only correctly if old and new are single chars

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set output ""
    set l [string length $input]
    for { set i 0 } { $i < $l } { incr i 1 } {
        set c [string index $input $i]
        if { $c != $value } {
            append output $c
        }
    }
    tracer LEAVING
    return $output
}

#=====================================
private method DDMMSS2deg { ddmmss } {
#=====================================
# RA,DEC format conversion
# from  +/-ddmmss.xxx or +/-dd:mm:ss.xxx to +/-ddd.xxx
# like -703514.123 or -70:35:14.123 (means -70 deg 35 arcmin 14.123 arcsec)
# to -70.5872563
# This routine works also for hour angles but the factor of 15 must be
# corrected outside this proc
# This proc is very primitive, it works char by char

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $debug && $verbose } {
        puts stdout "DDMMSS2deg: ddmmss = >$ddmmss<"
    }
    
    if { [string length $ddmmss] == 0 } {
        tracer LEAVING EMPTY
        return 0.0
    }
    # Get the sign
    if { [string compare [string index $ddmmss 0] "-"] == 0 } {
        set sign -1.0
    } else {
        set sign 1.0
    }
    # Get rid of eventual ':' and/or '-' '+' signs
    set buf [string_delete $ddmmss :]
    set buf [string_delete $buf +]
    set buf [string_delete $buf -]
    # Split by floating point and get rid of eventual - sign
    set a [split $buf "."]

    # Check if it is already in the desired ddd.xxx format
    set a1 [lindex $a 0]
    set a2 .[lindex $a 1]
    set l [string length $a1]
    # Minimum length of valid ddmmss is 5, i.e. dmmss
    if { $l < 5 } {
        tracer LEAVING OK
        return $ddmmss
    }

    if { $debug && $verbose } {
        puts stdout "              sign = >$sign<"
        puts stdout "               buf = >$buf<"
        puts stdout "                 a = >$a<"
        puts stdout "                a1 = >$a1<"
        puts stdout "                a2 = >$a2<"
    }

    set a2 [expr {$a2 / 3600.0}]

    set a11 [string index $a1 [expr {$l - 1}]]
    set a12 [string index $a1 [expr {$l - 2}]]
    if { $debug && $verbose } {
        puts stdout "               a11 = >$a11<"
        puts stdout "               a12 = >$a12<"
    }
    set a11 [expr {($a11 + 10.0 * $a12) / 3600.0}]
    if { $debug && $verbose } {
        puts stdout "               A11 = >$a11<"
    }

    set a21 [string index $a1 [expr {$l - 3}]]
    set a22 [string index $a1 [expr {$l - 4}]]
    if { $debug && $verbose } {
        puts stdout "               a21 = >$a21<"
        puts stdout "               a22 = >$a22<"
    }
    set a21 [expr {($a21 + 10.0 * $a22) / 60.0}]
    if { $debug && $verbose } {
        puts stdout "               A21 = >$a21<"
    }

    set a31 [string index $a1 [expr {$l - 5}]]
    set a32 [string index $a1 [expr {$l - 6}]]
    if { $debug && $verbose } {
        puts stdout "               a31 = >$a31<"
        puts stdout "               a32 = >$a32<"
    }
    set a31 [expr {($a31 + 10.0 * $a32)}]
    if { $debug && $verbose } {
        puts stdout "               A31 = >$a31<"
    }
    set a31 [expr {$sign * ($a31 + $a21 + $a11 + $a2)}]
    if { $debug && $verbose } {
        puts stdout "                 A = >$a31<"
    }
    tracer LEAVING
    return $a31
}

#=============================
private method F2G { value } {
#=============================
# Format conversion for floating point
# from  xxx.yyy to xxxyyy to be used in MXU
# mode for gerber (rs274x) format
# X33Y33

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set value [format "%3.3f" $value]
    set value [split $value "."]
    set value "[lindex $value 0][lindex $value 1]"
    # puts stdout ">$value<"
    tracer LEAVING
    return $value
}

#=================================
private method pick_technique {} {
#=================================
# Explains the pick object technique for center slit option.
    tracer ENTERING
    if { [$this cget -interactive] && $naggme } {
        set text "If you are not getting slits: \
                \n-Click mouse B-1 and hold it down\
                \n-drag cursor around the star \
                \n-close in to star center \
                \n-release B-1. \
                \n\nDon't bother clicking on Pick Object \
                \nbutton of Pick Object widget."
        set wid .skycat1.pictech
        catch { destroy $wid }
        set w [DialogWidget $wid \
                            -title "Pick Technique" \
                            -text $text \
                            -bitmap warning \
                            -default 1 \
                            -buttons { Ok GotIt! } \
              ]
        set asnw [$w activate]
        if { $asnw == 1 } {
            set naggme 0
        }
    }
    tracer LEAVING
}

#======================
public method demo {} {
#======================
# A short demo, but should be updated
# reset the mask
# there must be already a mask

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set w .skycat1.dm
    catch {destroy $w}
    set dw [DialogWidget $w \
           -text "Demo ? \n  1-5 min" \
           -bitmap warning \
           -default -1 \
           -buttons {START STOP}]
    if { [$dw activate] } {
        tracer LEAVING 1
        return
    }

    puts stdout "Demo Start ..."
    if { [string match echs [$this cget -modus]] } {
        $this configure -grism  XGRIS_600B+92.EGRIS_110B+98.FILT_465_250+82
        # $m_mb.layout.gr invoke 10
        # puts stdout ">grims>>>>"
    }

    # delete everything and reset current mask
    catch {$m_canvas delete A}
    reset_mask
    puts stdout "after reset"

    # FORS1 f2=0, FORS2 f2=1
    set f2 [expr {$insId - 1}]

    $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x y screen
    # move slits needs screen coords
    # first step move slit blades in MOS and IMG

    puts stdout "Demo 1 all : set collimator"
    #--------------------------------------
    if { $f2 } {
        $this configure -collimat COLL_SR+6
    } else {
        $this configure -collimat COLL_SR+1
    }

    puts stdout "after collimator"

    puts stdout "Demo 2 set grism"
    # -----------------------------
    switch -exact -- [$this cget -modus] {
        mos -
        pmos -
        lss -
        hits -
        hitms -
        mxu {
            if { $f2 } {
                $this configure -grism GRIS_600B+22
                $this configure -filter none
                wave_config_mask [list 360.0 450.0 demo]
            } else {
                $this configure -grism GRIS_600B+12
                $this configure -filter none
                wave_config_mask [list 360.0 450.0 demo]
            }
        }
        echs {
            # was already set before reset_mask
        }
    }

    switch -exact -- [$this cget -modus] {
        pmos -
        mos {
            puts stdout "Demo 1 mos : move slits"
            #----------------------------------
            for { set i 1 } { $i < 25 } { incr i 1 } {
                set xn [expr {$x + [random 250]}]
                set yn [expr {$y + [random 250]}]
                catch { move_slit $m_canvas $xn $yn 1 0 }
                update
                after 1
                set xn [expr {$x - [random 250]}]
                set yn [expr {$y - [random 250]}]
                catch { move_slit $m_canvas $xn $yn 1 0 }
                update
                after 1
                set xn [expr {$x + [random 250]}]
                set yn [expr {$y - [random 250]}]
                catch { move_slit $m_canvas $xn $yn 1 0 }
                update
                after 1
                set xn [expr {$x - [random 250]}]
                set yn [expr {$y + [random 250]}]
                catch { move_slit $m_canvas $xn $yn 1 0 }
                update
                after 1
            }
        }
        mxu {
            puts stdout "Demo 1 mxu : create slits"
            #----------------------------------
            $this configure -slit_length 6
            $this configure -slit_width 2
            $this configure -slit_shape STRAIGHT
            for { set i 1 } { $i < 60 } { incr i 1 } {
                set xn [expr {$x + [random 210]}]
                set yn [expr {$y + [random 250]}]
                $m_image convert coords $xn $yn screen  xc yc canvas
                set a [$m_canvas find overlapping \
                      [expr {$xc - 1}] [expr {$yc - 10}] \
                      [expr {$xc + 1}] [expr {$yc + 10}]]
                set go 1
                foreach c $a {
                    set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
                    if { [string match *mm* $local_tags] } {
                        # position occupied
                        set go 0
                    }
                }
                if { $go } {
                    catch { mxu_create_slit $m_canvas $xn $yn }
                    update
                    after 1
                }
            }
            for { set i 1 } { $i < 60 } { incr i 1 } {
                set xn [expr {$x + [random 210]}]
                set yn [expr {$y - [random 250]}]
                $m_image convert coords $xn $yn screen  xc yc canvas
                set a [$m_canvas find overlapping \
                      [expr {$xc - 1}] [expr {$yc - 10}] \
                      [expr {$xc + 1}] [expr {$yc + 10}]]
                set go 1
                foreach c $a {
                    set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
                    if { [string match *mm* $local_tags] } {
                        # position occupied
                        set go 0
                    }
                }
                if { $go } {
                    catch { mxu_create_slit $m_canvas $xn $yn }
                    update
                    after 1
                }
            }
            $this configure -slit_shape CIRCLE
            for { set i 1 } { $i < 80 } { incr i 1 } {
                set xn [expr {$x - [random 210]}]
                set yn [expr {$y - [random 250]}]
                $m_image convert coords $xn $yn screen  xc yc canvas
                set a [$m_canvas find overlapping \
                      [expr {$xc - 1}] [expr {$yc - 1}] \
                      [expr {$xc + 1}] [expr {$yc + 1}]]
                set go 1
                foreach c $a {
                    set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
                    if { [string match *mm* $local_tags] } {
                        # position occupied
                        set go 0
                    }
                }
                if { $go } {
                    catch { mxu_create_slit $m_canvas $xn $yn }
                    update
                    after 1
                }
            }
            for { set i 1 } { $i < 20 } { incr i 1 } {
                set xn [expr {$x - [random 210]}]
                set yn [expr {$y + [random 250]}]
                $m_image convert coords $xn $yn screen  xc yc canvas
                set a [$m_canvas find overlapping \
                      [expr {$xc - 1}] [expr {$yc - 1}] \
                      [expr {$xc + 1}] [expr {$yc + 1}]]
                set go 1
                foreach c $a {
                    set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
                    if { [string match *mm* $local_tags] } {
                        # position occupied
                        set go 0
                    }
                }
                if { $go } {
                    catch { mxu_create_slit $m_canvas $xn $yn }
                    update
                    after 1
                }
            }
        }
        ipol -
        img {
            puts stdout "Demo 1 img : move slits"
            #-----------------------------------
            for { set i 1 } { $i < 60 } { incr i 1 } {
                set xn [expr {$x + [random 500]}]
                set yn [expr {$y + [random 500]}]
                catch { move_slit_blade $m_canvas $xn $yn [random 2] }
                update
                after 10
                set xn [expr {$x - [random 250]}]
                set yn [expr {$y - [random 250]}]
                catch { move_slit_blade $m_canvas $xn $yn [random 2] }
                update
                after 10
            }
        }
        hits -
        lss -
        mxu -
        echs -
        default {
            puts stdout "Demo 1 [$this cget -modus]: no mos slits move"
        }
    }

    puts stdout "Demo 2 all : colors and tranparency:"
    #-------------------------------------------------
    $this configure -label 1;     # 10
    $this configure -graytype 0;  # 10

    switch -exact -- [$this cget -modus] {
        pmos -
        img -
        echs -
        mos {
            foreach col {Red/Yellow Black/White Violet/Orange \
                         Gray Green/Red Bright Dark Red/Yellow} {
                $this configure -colorset $col
                if { [string match echs [$this cget -modus]] } {
                    flip_echs
                }
                update_and_replot
                update 
                after 1000
                # graytype
                $this configure -graytype 1 ;          # 11
                update_and_replot; update; after 1000
                $this configure -label 0 ;             # 01
                update_and_replot; update; after 1000
                $this configure -graytype 0 ;          # 00
                update_and_replot; update; after 1000
                $this configure -label 1;              # 10
                update_and_replot
                update
                after 1000
            }
        }
        default {
            # all modi
            foreach col {Red/Yellow Black/White Violet/Orange \
                         Gray Green/Red Bright Dark Red/Yellow} {
                $this configure -colorset $col
                update_and_replot; update; after 1000
                $this configure -label 0
                update_and_replot; update; after 1000
                $this configure -label 1
                update_and_replot
                update
                after 1000
            }
        }
    }

    if { [string match echs [$this cget -modus]] } {
        flip_echs
    }

    #-----------
    # SLIT WIDTH
    #-----------
    switch -exact -- [$this cget -modus] {
        pmos -
        echs -
        mxu -
        mos {
            puts stdout "Demo 3 mos : slit widths"
            #------------------------------------
            set xn [expr {$x + 20}]
            set yn $y
            # MOS slit_width only in MOS or PMOS mode
            # puts stdout "Slit Width"
            for { set i 51 } { $i >= 1 } { incr i -5 } {
                # puts stdout "slit width = [expr {(1.0 * $i)}]''"
                $this configure -slit_width [expr {(1.0 * $i)}]
                # move_slit $m_canvas $xn $yn 1
                update
                after 1
            }
            for { set i 1 } { $i <= 10 } { incr i 1 } {
                # puts stdout "slit width = [expr {(0.1 * $i)}]''"
                $this configure -slit_width [expr {(0.1 * $i)}]
                # update_and_replot
                # move_slit $m_canvas $xn $yn 1 0
                update
                after 1
            }
            $this configure -slit_width  5
            update_and_replot
            # puts stdout "..end"
            if { [string compare echs [$this cget -modus]] } {
                # no slit length demo for ECHS
                puts stdout "Demo 3 mos : slit length"
                #------------------------------------
                # slit_length  only in MOS mode
                for { set i 20 } { $i >= 5 } { incr i -5 } {
                    $this configure -slit_length $i
                    reset_mask
                    update 
                    after 1
                }
                for { set i 5 } { $i <= 20 } { incr i 5 } {
                    $this configure -slit_length $i
                    reset_mask
                    update
                    after 1
                }
                $this configure -slit_length 22
                reset_mask
            }
        }
        lss {
            puts stdout "Demo 3 lss : slit width"
            #------------------------------------
            foreach i {all 2.5 1.6 1.0 0.5 0.3 0.4 0.7 1.3 2.0 all 2.5} {
                $this configure -long_width $i
                switch -exact -- [$this cget -long_width] {
                    all {
                        set long_number 0
                    }
                    0.3 {
                        set long_number 1
                    }
                    0.4 {
                        set long_number 2
                    }
                    0.5 {
                        set long_number 3
                    }
                    0.7 {
                        set long_number 4
                    }
                    1.0 {
                        set long_number 5
                    }
                    1.3 {
                        set long_number 6
                    }
                    1.6 {
                        set long_number 7
                    }
                    2.0 {
                        set long_number 8
                    }
                    2.5 {
                        set long_number 9
                    }
                    default {
                        puts stdout "Warning: LSS slit"
                    }
                }
                # puts stdout "$long_number $i [$this cget -long_width] "
                reset_mask
                update
                after 2000
            }
        }
        hits {
            puts stdout "Demo 3 hits: slit width"
            #-----------------------------------
            foreach i {all 1.6 1.0 0.5 PH 0.7 1.3 2.0 all PH} {
                $this configure -long_width $i
                switch -exact -- [$this cget -long_width] {
                    all {
                        set long_number 0
                    }
                    0.3 {
                        set long_number 1
                    }
                    0.4 {
                        set long_number 2
                    }
                    0.5 {
                        set long_number 3
                    }
                    0.7 {
                        set long_number 4
                    }
                    1.0 {
                        set long_number 5
                    }
                    1.3 {
                        set long_number 6
                    }
                    1.6 {
                        set long_number 7
                    }
                    2.0 {
                        set long_number 8
                    }
                    default {
                        puts stdout "Warning: HITS slit"
                    }
                }
                # puts stdout "$long_number $i [$this cget -long_width] "
                reset_mask
                update
                after 2000
            }
        }
        img {
            puts stdout "Demo 3 img : slit width TBD"
            #---------------------------------------
            # move_slit_blade $w sx sy 1
        }
        ipol {
            if { [string match *SR* [$this cget -collimat]] } {
                puts stdout "Demo 3 ipol : slit width TBD"
                #----------------------------------------
            }
        }
        mxu -
        default {
            puts stdout "Demo 3 [$this cget -modus]: no mos slits width"
        }
    }

    puts stdout "Demo 4 all : rotate mask "
    # rotate mask

    if { [string match img  [$this cget -modus]] } {
        reset_mask
    }
    if { [string match ipol [$this cget -modus]] } {
        reset_mask
    }

    # rotate mask
    $this configure -rotangle 5
    for { set i 1 } { $i < 14 } { incr i 1 } {
         $this rotate $m_canvas $mos(0,x0) $mos(0,y0) -1
         update
         after 1
    }
    $this configure -rotangle 1
    for { set i 1 } { $i < 20 } { incr i 1 } {
         $this rotate $m_canvas $mos(0,x0) $mos(0,y0) 1
         update
         after 1
    }
    $this configure -rotangle 5

    puts stdout "Demo 5 all : move mask "
    #  move mask
    $m_image convert coords $mos(0,x0) $mos(0,y0) canvas x y screen
    mark $m_canvas $x $y
    for { set i 1 } { $i < 90 } { incr i 4 } {
         set a [expr {1.0 * $i}]
         update
         after 1
         set xn [expr {$x + 40 * cos($a*$deg2rad)}]
         set yn [expr {$y + 40 * sin($a*$deg2rad)}]
         catch { $this shift $m_canvas $xn $yn }
    }
    $this shift $m_canvas $x $y
    update_and_replot

    # PPRS-29765: Collimator change demo does not work. Taken out
    # puts stdout "Demo 6 all : change collimator "
    # change collimator
    # if { [string compare "pmos" [$this cget -modus]] != 0 } {
    #     for { set i 1 } { $i < 3 } { incr i 1 } {
    #         $this configure -collimat [lindex [split $co_range " "] 1]
    #         switch -exact -- [$this cget -modus] {
    #             mxu -
    #             hits -
    #             echs -
    #             lss -
    #             mos {
    #                 update_and_replot
    #                 update
    #                 after 1000
    #             }
    #             pmos -
    #             img -
    #             ipol {
    #                 reset_mask
    #                 update_and_replot
    #                 update
    #                 after 1000
    #             }
    #             default {
    #                 puts stdout "Warning: modus"
    #             }
    #         }
    #         $this configure -collimat [lindex [split $co_range " "] 0]
    #         switch -exact -- [$this cget -modus] {
    #             mxu -
    #             hits -
    #             echs -
    #             lss -
    #             mos {
    #                 update_and_replot
    #                 update
    #                 after 1000
    #             }
    #             pmos -
    #             img -
    #             ipol {
    #                 reset_mask
    #                 update_and_replot
    #                 update
    #                 after 1000
    #             }
    #             default {
    #                 puts stdout "Warning: modus"
    #             }
    #         }
    #     }
    #     # puts stdout "ok"
    # }
    $this configure -graytype 0
    update
    after 1000
    puts stdout "Demo finished"
    tracer LEAVING 2
    return 0
}

#===================================================================
proc FORSInit { skycat_ canvas_ image_ menu_ instrument camera modus \
                rot \
                col \
                tra \
                BPM \
                lbl \
                cnt \
                dbg \
                vbs \
                ctrl \
              } {
#===================================================================
# Some remarks on the arguments
# Private:
# rtd_   : Rtd pointer
# skycat_: SkyCat pointer
# canvas_: canvas name
# image_ : image name
# menu_  : widget to control FORS menu buttons
#
# Public:
# instrument: FORS1
#             FORS2
#             FORS2+1
# camera    : Camera option (B: Blue or R: Red)
# modus     : hitms hits img ipol lss mos mxu pmos
# rot       : rotation angle step
# col       : color set
# tra       : 0|1 use transparency
# BPM       : Bad Pixel Map option
# lbl       : 0|1 use labels (super user privilege)
# cnt       : 0|1 use center slit (pick object)
# dbg       : 0|1 print debug messages on stdout
# vbs       : 0|>=1 print verbose messages on stdout
#
# Local:
# ctrl: 0 delete current instance
#       1 create the only instance

    tracer ENTERING
    if { $ctrl & !$inst_counter } {
        # Case A create the only instance
        # -instrument either FORS1, FORS2 or FORS2+1
        # Create an instance of FORS
        # global ::my_mask
        fors::FORS my_mask $skycat_ $canvas_ $image_ $menu_ \
                          -instrument $instrument \
                          -camera $camera \
                          -modus $modus \
                          -width [$image_ width] \
                          -height [$image_ height] \
                          -rotangle $rot \
                          -colorset $col \
                          -graytype $tra \
                          -badpixelmap $BPM \
                          -label $lbl \
                          -pics $cnt \
                          -debug $dbg \
                          -verbosity $vbs

        incr inst_counter 1

        # puts stdout "Number of instances: $inst_counter"
        # puts stdout "my_mask isa FORS = [my_mask isa FORS]"
        # puts stdout "my_mask info class    = [my_mask info class]"
        # puts stdout "my_mask info inherit  = [my_mask info inherit]"
        # puts stdout "my_mask info heritage = [my_mask info heritage]"
        # puts stdout "my_mask info function = [my_mask info function]"
        # puts stdout "my_mask info variable = [my_mask info variable]"
        # puts stdout "my_mask itcl::find objects -class FORS = \
        #              [my_mask itcl::find objects -class FORS]"
        # puts stdout "my_mask info function help = \
        #              [my_mask info function help]"
        # puts stdout "my_mask info variable ccx = \
        #              [my_mask info variable ccx]"
        # puts stdout "my_mask info variable exposure_counter  = \
        #              [my_mask info variable exposure_counter]"
        # puts stdout "my_mask info variable inst_counter  = \
        #              [my_mask info variable inst_counter]"
        # puts stdout "my_mask cget -collimat = \
        #              [my_mask cget -collimat]"

    } elseif { !$ctrl & $inst_counter } {
        # Case B delete the only instance
        if { [string compare [itcl::find objects -class FORS] my_mask] == 0 } {
            # puts stdout "my_mask does exist"
        } else {
            # puts stdout "my_mask does not exist"
        }

        delete object my_mask
        incr inst_counter -1

        if { [string compare [itcl::find objects -class FORS] my_mask] == 0 } {
            # puts stdout "my_mask still exists"
        } else {
            # puts stdout "my_mask does not exist anymore"
        }

        # puts stdout "Number of instances: $inst_counter"
        # unset my_mask
    } else {
        # Case C create more than one instance. Not supported yet.
        if { $ctrl } {
            # puts stdout "Warning : cannot create more than one Mask"
        } else {
            # puts stdout "Warning : cannot delete $inst_counter Masks"
        }
    }
    tracer LEAVING
}

#-----------------
# common variables
#-----------------
    # number of class instances, only one (mask) permitted
    common inst_counter 0

#-----------------
# public variables
#-----------------

    # Public variables configured in this class should behave like state variables.
    # If one variable is configured from a menu radio, or check button, the state
    # change in the menu buttons.
    # It should work also the other way around: when the variable is configured
    # from the methods, it should align the menu buttons to the new value.
    #---------------------------------
    public variable instrument FORS2 {
    #---------------------------------
        tracer ENTERING " variable instrument"
        global ::m_i
        # Maintenance: If FORS1 is moved from UT2 -> UT3
        # edit the 'FORS1' branch with focscale_UT3
        switch -exact -- [$this cget -instrument] {
            FORS1 {
                set insId 1
                set isf_file "$isf_dir/FORS1.isf"
                set focscale $focscale_UT2
                puts stdout "for FORS1 at UT2"
            }
            FORS2 {
                set insId 2
                set isf_file "$isf_dir/FORS2.isf"
                set focscale $focscale_UT1
                puts stdout "for FORS2 at UT1"
            }
            FORS2+1 {
                set insId 2
                set isf_file "$isf_dir/FORS2.isf"
                set focscale $focscale_UT1
                puts stdout "for FORS2+1 at UT1"
            }
            default {
                set insId 0
                set isf_file "$isf_dir/INS.isf"
                set focscale 9999
                puts stdout "Error in instrument identifier: [$this cget -instrument]"
            }
        }
        if { $debug } {
            puts stdout "instrument var: [$this cget -instrument]"
        }
        build_fors_menu_EnterMode $m_skycat [$this cget -instrument]
        tracer LEAVING "end of variable instrument"
    }

    #--------------------------
    public variable modus mos {
    #--------------------------
        tracer ENTERING " variable modus"
        global ::m_i
        set mask mos; # default mask for default modus
        set coll [$this cget -collimat]
        set new_coll $coll

        # mode can be : ipol, img, hits, hitms, lss, mos, mxu, pmos

        # PMOS in highres collimator physically not possible
        if { [string compare [$this cget -modus] "pmos"] == 0 } {
            if { [string compare [$this cget -slit_width] "open"] } {
                if { [string match *HR* $coll] } {
                    set new_coll [lindex [split $co_range " "] 0]
                }
            }
        }

        # HITS in highres collimator not sensful
        #  note : 9/7/5/3/BigPH/2/4/6/8
        # use BigPH when central (smallest) slit is open
        # BigPH can be used with highres collimator
        if { [string compare [$this cget -modus] "hits"] == 0 } {
            if { [string compare [$this cget -long_width] "0.3"] } {
                if { [string match *HR* $coll] } {
                    set new_coll [lindex [split $co_range " "] 0]
                }
            }
        }

        # update
        if { [string compare $coll $new_coll] } {
            $this configure -collimat $new_coll
        }

        # mask can be : mos, pol, lss, hits, hitms, mxu
        if { [string compare [$this cget -modus] "ipol"] == 0 } {
            if { [string match *HR* [$this cget -collimat]] } {
                set mask pol
            }
        }

        # slit less spectroscopy in pmos
        if { [string compare [$this cget -modus] "pmos"] == 0 } {
            if { [string match *open* [$this cget -slit_width]] } {
                if { [string match *HR* $coll] } {
                    set mask pol
                }
            }
        }

        if { [string compare [$this cget -modus] "hits"] == 0 } {
            set mask hits
        }

        if { [string compare [$this cget -modus] "hitms"] == 0 } {
            set mask hitms
        }

        if { [string compare [$this cget -modus] "lss"] == 0 } {
            set mask lss
        }

        if { [string compare [$this cget -modus] "mxu"] == 0 } {
            set mask mxu
        }
        if { $debug } {
            puts stdout "modus var: [$this cget -modus] with mask $mask"
        }
        puts $lfId   "modus: [$this cget -modus] with mask $mask"
	set last_action "Select Modus"
        catch {SP update}
        tracer LEAVING "end of variable modus"
    }

    #-------------------------------
    public variable long_width 0.5 {
    #-------------------------------
        tracer ENTERING " variable long_width"
        global ::m_i
        set doit 0
        set lm [$this cget -modus]
        if { $mask_initialized } {
            if { [string compare $lm  "lss"] == 0 ||
                 [string compare $lm "hits"] == 0 } {
                set doit 1
            }
        } else {
            set doit 1
        }

        if { $doit } {
            # long_number is internal slit counter
            switch -exact -- $long_width {
                all {set long_number 0 }
                PH  {set long_number 1 }
                0.3 {set long_number 1 }
                0.4 {set long_number 2 }
                0.5 {set long_number 3 }
                0.7 {set long_number 4 }
                1.0 {set long_number 5 }
                1.3 {set long_number 6 }
                1.6 {set long_number 7 }
                2.0 {set long_number 8 }
                2.5 {set long_number 9 }
                default {
                    puts stdout "Warning: LSS $long_width"
                }
            }

            # allow coll=highres for hits-pinhole
            if { [string compare $lm "hits"] == 0 } {
                if { $long_number == 1 } {
                    # pinhole in use
                    # enable collimator
                    $m_mb.layout.sh entryconfigure $m_i(Layout_Collimator_HR) -state normal
                } else {
                    # no pinhole in use
                    $m_mb.layout.sh entryconfigure $m_i(Layout_Collimator_HR) -state disabled
                }
            }
            if { $mask_initialized } {
                reset_mask
            }
        }
        if { $debug } {
            puts stdout "long_width var: [$this cget -long_width]"
        }
        puts $lfId  "lslit: [$this cget -long_width]"
        set last_action "Select LongSlit"
        catch {SP update}
        tracer LEAVING "end of variable long_width"
    }

    #-------------------------------
    # Radio button in Slit Width submenu
    # in layout.sw menu
    public variable slit_width 1.0 {
    #-------------------------------
        tracer ENTERING " variable slit_width"
        global ::m_i
        set swid [string map {. _} [$this cget -slit_width]]
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_SlitWidth_1_0)] } {
            set m_var [$m_mb.layout.sw entrycget $m_i(Layout_SlitWidth_1_0) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [string compare $m_value [$this cget -slit_width]] != 0 } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_SlitWidth_$swid)] } {
                    $m_mb.layout.sw invoke $m_i(Layout_SlitWidth_$swid)
                }
            }
        }

        if { $mask_initialized } {
            # we need to know if the slit_width changed from
            # open to numerical or vice versa
            set new_open_flag [string match *open* $slit_width]
            # puts stdout "old_open_flag = $old_open_flag"
            # puts stdout "new_open_flag = $new_open_flag"

            # puts stdout "mask_init = $mask_initialized"
            if { [string compare [$this cget -modus] pmos] == 0 } {
                if { [string match *SR* [$this cget -collimat]] } {
                    $m_mb.layout entryconfigure $m_i(Layout_SlitWidth) -state normal
                } else {
                    $m_mb.layout entryconfigure $m_i(Layout_SlitWidth) -state disabled
                }
                if { [string compare [$this cget -slit_width] open] == 0 } {
                    # allow collimator change
                    $m_mb.layout entryconfigure $m_i(Layout_Collimator) -state normal
                } else {
                    # disable collimator change
                    $m_mb.layout entryconfigure $m_i(Layout_Collimator) -state disabled
                }
            }

            # the Save button
            if { [string compare [$this cget -slit_width] open] == 0 } {
                if { [string compare [$this cget -modus] pmos] == 0 } {
                    $m_mb entryconfigure $m_i(SaveSetup) -label "Save 'pims.$exposure_counter.targ'"
                } else {
                    $m_mb entryconfigure $m_i(SaveSetup) -label "Save 'ims.$exposure_counter.targ'"
                }
            } else {
                $m_mb entryconfigure $m_i(SaveSetup) -label "Save '[$this cget -modus].$exposure_counter.targ'"
            }

            if { $new_open_flag != $old_open_flag } {
                puts stdout "change open/numeric: reset mask"
                reset_mask
            } else {
                switch -exact -- [$this cget -modus] {
                    echs -
                    mos -
                    pmos {
                        for { set i 1 } { $i < 20 } { incr i 1 } {
                            # set the new slit width
                            set mos($i,b) [$this cget -slit_width]
                            # if slit is positioned
                            if { $targw($i,f) != 0 } {
                                catch { $m_image convert coords $targw($i,a) $targw($i,d) "wcs $EQX" sx sy screen }
                                # moves the slit again to the same position with the
                                # new slit width
                                catch { $this move_slit $m_canvas $sx $sy 1 0 }
                            }
                        }
                    }
                    hits -
                    lss {
                        eval configure { -long_width [$this cget -slit_width] }
                    }
                    hitms {
                        fors::MXU_SLIT::config_slits 0 sw [$this cget -slit_width]
                    }
                    mxu {
                        fors::MXU_SLIT::config_slits $nf_refslit sw [$this cget -slit_width]
                    }
                    default {
                        reset_mask
                    }
                }

            }
            set old_open_flag $new_open_flag
            # puts stdout "old_open_flag = $old_open_flag"
            # puts stdout "new_open_flag = $new_open_flag"
        }
        if { $debug } {
            puts stdout "slit_width var: [$this cget -slit_width]"
        }
        puts $lfId "width: [$this cget -slit_width]"
	set last_action "Select Slit Width"
        catch {SP update}
        tracer LEAVING "end of variable slit_width"
    }

    #---------------------------------
    # Radio button in Slit Length submenu
    # in layout.sl menu
    public variable slit_length real {
    #---------------------------------
        tracer ENTERING " variable slit_length"
        global ::m_i
        set slen [string map {. _} [$this cget -slit_length]]
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_SlitLength_10_0)] } {
            set m_var [$m_mb.layout.sl entrycget $m_i(Layout_SlitLength_10_0) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [string compare $m_value [$this cget -slit_length]] != 0 } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_SlitLength_$slen)] } {
                    $m_mb.layout.sl invoke $m_i(Layout_SlitLength_$slen)
                }
            }
        }

        switch -exact -- [$this cget -modus] {
            mos -
            pmos {
                if { [string compare [$this cget -slit_length] "real"] } {
                    # configure non-real MOS slit_length
                    adjust_slit_length
                    reset_mask
                } else {
                    # configure 'real' MOS slit_length
                    if { $mask_initialized } {
                        # not called by constructor
                        load_instrument; # again
                        # balde should not exceed focal field
                        set mosslitmin(1)  $YSRMIN
                        set mosslitmax(19) $YSRMAX
                        reset_mask
                    }
                }
            }
            hitms {
                if { $mask_initialized } {
                    # global .ldw
                    # if { [catch {wm deiconify .ldw}] } {
                    #     EntryForm .ldw -title "Change slit length starting from " \
                    #                    -labels {Slit_Number Par Length} \
                    #                    -command fors::MXU_SLIT::config_slits \
                    #                    -values "0 sl $slit_length"
                    # }
                    # if { [info exists .ldw] } {
                    #     catch { destroy .ldw }
                    # }
                    fors::MXU_SLIT::config_slits 0 sl [$this cget -slit_length]
                }
            }
            mxu {
                if { $mask_initialized } {
                    # global .ldw
                    # if { [catch {wm deiconify .ldw}] } {
                    #     EntryForm .ldw -title "Change slit length starting from " /
                    #                    -labels {Slit_Number Par Length} /
                    #                    -command fors::MXU_SLIT::config_slits /
                    #                    -values "$nf_refslit sl $slit_length"
                    # }
                    # if { [info exists .ldw] } {
                    #     catch { destroy .ldw }
                    # }
                    fors::MXU_SLIT::config_slits $nf_refslit sl [$this cget -slit_length]
                }
            }
        }
        if { $debug } {
            puts stdout "slit_length var: [$this cget -slit_length]"
        }
        puts $lfId   "length: [$this cget -slit_length]"
        set last_action "Select Slit Length"
        catch {SP update}
        tracer LEAVING "end of variable slit_length"
    }

    #------------------------------
    public variable slit_offset 0 {
    #------------------------------
        tracer ENTERING " variable slit_offset"
        switch -exact -- [$this cget -modus] {
            mos -
            pmos {
                puts stdout "offset: [$this cget -slit_offset]"
            }
            hitms {
                if { $mask_initialized } {
                    fors::MXU_SLIT::config_slits 0 so [$this cget -slit_offset]
                }
            }
            mxu {
                if { $mask_initialized } {
                    fors::MXU_SLIT::config_slits $nf_refslit so [$this cget -slit_offset]
                }
            }
        }
        if { $debug } {
            puts stdout "slit_offset var: [$this cget -slit_offset]"
        }
        set last_action "Select Slit Offset"
        catch {SP update}
        tracer LEAVING "end of variable slit_offset"
    }

    #-----------------------------------
    # Radio button in Slit Shape submenu
    # in layout.ss menu
    public variable slit_shape STRAIGHT {
    #-----------------------------------
        tracer ENTERING " variable slit_shape"
        global ::m_i
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_SlitShape_none)] } {
            set m_var [$m_mb.layout.ss entrycget $m_i(Layout_SlitShape_none) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [string compare $m_value [$this cget -slit_shape]] != 0 } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_SlitShape_[$this cget -slit_shape])] } {
                    $m_mb.layout.ss invoke $m_i(Layout_SlitShape_[$this cget -slit_shape])
                }
            }
        }
        if { $debug } {
            puts stdout "slit_shape var: [$this cget -slit_shape]"
        }
        puts $lfId "MXU aperture type: [$this cget -slit_shape]"
	set last_action "Select MXU Slit Type"
        catch {SP update}
        tracer LEAVING "end of variable slit_shape"
    }

    #------------------------------
    # Radio button in Collimator submenu
    # in layout.sh menu
    public variable collimat none {
    #------------------------------
        tracer ENTERING " variable collimat"
        global ::m_i
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_Collimator_SR)] } {
            set m_var [$m_mb.layout.sh entrycget $m_i(Layout_Collimator_SR) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [string compare $m_value [$this cget -collimat]] != 0 } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_Collimator_SR)] } {
                    $m_mb.layout.sh invoke *[$this cget -collimat]*
                }
            }
        }
        if { $mask_initialized } {
            if { $badpixelmap } {
                load_BPM
                update_BPM
            }

            # change mask for ipol/hr
            if { [string compare [$this cget -modus] "ipol"] == 0 } {
                if { [string match *HR* [$this cget -collimat]] } {
                    set mask pol
                    puts stdout "from MASK=mos to MASK=$mask"
                } else {
                    set mask mos
                    puts stdout "from MASK=pol to MASK=$mask"
                }
                reset_mask
            } else {
                # modus != ipol
                # 24/01/2000 special request by hboehnha
                # hboehnha wants this only for SR/HR-change
                if { [string match *img* [$this cget -modus]] ||
                     [string match *mos* [$this cget -modus]] } {
                    reset_mask
                } else {
                    erase
                    plot_mask
                }
            }

            # enable hr for pinhole in hits
            if { [string compare [$this cget -modus] "hits"] == 0 } {
                if { [string match *SR* [$this cget -collimat]] } {
                    $m_mb.layout entryconfigure $m_i(Layout_SlitWidth) -state normal
                } else {
                    $m_mb.layout entryconfigure $m_i(Layout_SlitWidth) -state disabled
                }
            }

            # pmos/open -> ipol mask
            if { [string compare [$this cget -modus] "pmos"] == 0 } {
                if { [string compare [$this cget -slit_width] "open"] == 0 } {
                    if { [string match *HR* [$this cget -collimat]] } {
                        $m_mb.layout entryconfigure $m_i(Layout_SlitWidth) -state disabled
                        $m_mb.layout entryconfigure $m_i(Layout_SlitLength) -state disabled
                        reset_mask
                    } else {
                        $m_mb.layout entryconfigure $m_i(Layout_SlitWidth) -state normal
                        $m_mb.layout entryconfigure $m_i(Layout_SlitLength) -state normal
                        reset_mask
                    }
                }
            }
        } else {
            # mask not initialized = default coll by constructor
        }
        if { $debug } {
            puts stdout "collimat var: [$this cget -collimat]"
        }
        puts $lfId   "colli: [$this cget -collimat]"
        set last_action "Select Collimator"
        catch {SP update}
        tracer LEAVING "end of variable collimat"
    }

    #----------------------------
    # Radio button in OS Filter submenu
    # in layout.fi menu
    public variable filter none {
    #----------------------------
        tracer ENTERING " variable filter"
        global ::m_i
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_Filter_m0_0)] } {
            set m_var [$m_mb.layout.fi entrycget $m_i(Layout_Filter_m0_0) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [string compare $m_value [$this cget -filter]] != 0 } {
                # The option was modified, need to align menu button
                set mii [lsearch -exact $filt_range [$this cget -filter]]
                # Filter decade 0,1...4,5...
                set sm [expr {$mii / 10}]
                # Index from 0 to 9 for each menu item of the current decade
                set smi [expr {$mii % 10}]
                if { [info exists m_i(Layout_Filter_m${sm}_$smi)] } {
                    if { $sm == 0 } {
                        $m_mb.layout.fi invoke $m_i(Layout_Filter_m${sm}_$smi)
                    } else {
                        $m_mb.layout.fi.m$sm invoke $m_i(Layout_Filter_m${sm}_$smi)
                    }
                }
            }
        }

        puts stdout ""
        puts stdout "filter: [$this cget -filter]"
        puts $lfId  "filter: [$this cget -filter]"

        # This matches echs lss mos pmos hits hitms
        if { [string match *s* [$this cget -modus]]          ||
             [string compare [$this cget -modus] mxu] == 0 } {
            # Search in the dispersion table
            set prio_flag 0
            set good 0
            # 0 = not filter, 1 = a filter, 2 = best filter
            for { set i 1 } { $i <= $A_disp(0) } { incr i 1 } {
                if { [string compare $grism $A_disp($i,grism)] == 0 } {
                    if { $prio_flag == 0 } {
                        set good $i
                        # lassign $pat filter disp lamc ins_lmin ins_lmax
                        if { [string compare $filter $A_disp($i,filter)] == 0 } {
                            set prio_flag 2
                        } else {
                            set prio_flag 1
                        }
                        # puts "$i $good $prio_flag"
                    }
                    # If there are further filters in table
                    # select
                    if { $prio_flag == 1 } {
                        # filter 'none' matches better than the wrong filter
                        if { [string compare $A_disp($i,filter) "none"] == 0 } {
                            set good $i
                            # lassign $pat filter disp lamc ins_lmin ins_lmax
                            set prio_flag 1
                            # puts "$i $good $prio_flag"
                        }
                        # If the filter matches -> overwrite
                        if { [string compare $A_disp($i,filter) $filter] == 0 } {
                            set good $i
                            # lassign $pat filter disp lamc ins_lmin ins_lmax
                            set prio_flag 2
                            # puts "$i $good $prio_flag"
                        }
                    }
                    # flag==2 is already the best case, do not overwrite
                }
            }

            if { $prio_flag < 2 } {
                set warn ">> OS filter not found <<"
            } else {
                set warn ">> OS filter found ok <<"
            }
            puts stdout [format " filter: %12s, good = %2d, prio = %2d %20s" [$this cget -filter] $good $prio_flag $warn]

            set disp_ang $A_disp($good,ang)
            set disp_ang_rad [expr {$disp_ang*$deg2rad}]
            if { [string compare [$this cget -modus] hitms] == 0 ||
                 [string compare [$this cget -modus] mxu] == 0 } {
                set pos [lsearch -exact $filt_range [$this cget -filter]]
                set l_disp [expr $CCDscale_standard/$focscale/$A_disp($good,disp)/$PIX]
                set l_lam0 [lindex $lamcent_range $pos]
                set l_lam0 [expr {[string length $l_lam0]?$l_lam0:0}]
                set l_fwhm [lindex $fwhm_range $pos]
                set l_fwhm [expr {[string length $l_fwhm]?$l_fwhm:0}]

                # grism_disp[mm/nm]  grism_lam0[nm]  filt_lam0[nm]  filt_fwhm[nm] grism_disp_angle[rad]
                set 5val [list $l_disp [expr $A_disp($good,cen)*1.0] $l_lam0 $l_fwhm $disp_ang_rad]
                fors::MXU_SLIT::disp $5val [$this cget -debug]
                if { [$this cget -debug] == 1 } {
                    puts stdout         "filter: good = $good"
                    puts stdout         "        A_disp($good,disp) = $A_disp($good,disp)"
                    puts stdout         "        pos = $pos"
                    puts stdout         "filter: grism_disp\[mm/nm\] grism_lam0\[nm\] filt_lam0\[nm\] filt_fwhm\[nm\] grism_disp_angle\[rad\]"
                    puts stdout [format "        %17.2f %14.2f %13.2f %13.2f %+21.2f" $l_disp [expr {$A_disp($good,cen)*1.0}] $l_lam0 $l_fwhm $disp_ang_rad]
                }
            }
            # Specify the wavelength limits
            set L_MIN [expr {($A_disp($good,cen) + $A_disp($good,min)) * 0.5}]
            set L_MAX [expr {($A_disp($good,cen) + $A_disp($good,max)) * 0.5}]
            wave_limit
            wave_config_mask [list $L_MIN $L_MAX filter [$this cget -filter]]
        }

        if { $mask_initialized } {
            # if { [$this cget -label] } {
            #     update_labels
            # }
        }
	set last_action "Select Filter"
        catch {SP update}
        tracer LEAVING "end of variable filter"
    }

    #---------------------------
    # Radio button in Grism submenu
    # in layout.gr menu
    public variable grism none {
    #---------------------------
        tracer ENTERING " variable grism"
        global ::m_i
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_Grism_none)] } {
            set m_var [$m_mb.layout.gr entrycget $m_i(Layout_Grism_none) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [string compare $m_value [$this cget -grism]] != 0 } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_Grism_[$this cget -grism])] } {
                    $m_mb.layout.gr invoke $m_i(Layout_Grism_[$this cget -grism])
                }
            }
        }

        puts stdout ""
        puts stdout "grism: [$this cget -grism]"
        puts $lfId  "grism: [$this cget -grism]"

        switch -exact -- [$this cget -modus] {
            echs {
                # Do not call echs_pos when mask does not yet
                # exist (not when called from constructor)
                if { $mask_initialized } {
                    # puts stdout ">>!!! echs_pos called from config -grism"
                    echs_pos
                }
            }
             mos -
            pmos -
             lss -
            hits -
           hitms -
             mxu {
                # Search in the dispersion table
                set prio_flag 0
                set good 0
                # 0 = not filter, 1 = a filter, 2 = best filter
                for { set i 1 } { $i <= $A_disp(0) } { incr i 1 } {
                    if { [string compare $grism $A_disp($i,grism)] == 0 } {
                        if { $prio_flag == 0 } {
                            set good $i
                            # lassign $pat filter disp lamc ins_lmin ins_lmax
                            if { [string compare $filter $A_disp($i,filter)] == 0 } {
                                set prio_flag 2
                            } else {
                                set prio_flag 1
                            }
                            # puts "$i $good $prio_flag"
                        }
                        # If there are further filters in table
                        # select
                        if { $prio_flag == 1 } {
                            # filter 'none' matches better than the wrong filter
                            if { [string compare $A_disp($i,filter) "none"] == 0 } {
                                set good $i
                                # lassign $pat filter disp lamc ins_lmin ins_lmax
                                set prio_flag 1
                                # puts "$i $good $prio_flag"
                            }
                            # If the filter matches -> overwrite
                            if { [string compare $A_disp($i,filter) $filter] == 0 } {
                                set good $i
                                # lassign $pat filter disp lamc ins_lmin ins_lmax
                                set prio_flag 2
                                # puts "$i $good $prio_flag"
                            }
                        }
                        # flag==2 is already the best case, do not overwrite
                    }
                }
                switch -exact -- $prio_flag {
                    0 { puts stdout " Error  : Filter-Grism mismatch"       }
                    1 { puts stdout " Warning: Filter does not match Grism" }
                    2 { puts stdout " Ok     : Filter does match Grism"     }
                    default {
                        puts stdout " Error  : Filter-Grism mismatch"
                    }
                }
                if { $prio_flag < 2 } {
                    set warn ">> filter not found <<"
                } else {
                    set warn ""
                }
                puts stdout [format " grism: %12s, good = %2d, prio = %2d %20s" [$this cget -grism] $good $prio_flag $warn]

                set disp_ang $A_disp($good,ang)
                set disp_ang_rad [expr {$disp_ang*$deg2rad}]
                if { [string compare [$this cget -modus] hitms] == 0 ||
                     [string compare [$this cget -modus] mxu] == 0 } {
                    set pos [lsearch -exact $filt_range [$this cget -filter]]
                    set l_disp [expr $CCDscale_standard/$focscale/$A_disp($good,disp)/$PIX]
                    set l_lam0 [lindex $lamcent_range $pos]
                    set l_lam0 [expr {[string length $l_lam0]?$l_lam0:0}]
                    set l_fwhm [lindex $fwhm_range $pos]
                    set l_fwhm [expr {[string length $l_fwhm]?$l_fwhm:0}]

                    # grism_disp[mm/nm]  grism_lam0[nm]  filt_lam0[nm]  filt_fwhm[nm] grism_disp_angle[rad]
                    set 5val [list $l_disp [expr $A_disp($good,cen)*1.0] $l_lam0 $l_fwhm $disp_ang_rad]
                    fors::MXU_SLIT::disp $5val [$this cget -debug]
                    if { [$this cget -debug] == 1 } {
                        puts stdout         "grism:  good = $good"
                        puts stdout         "        A_disp($good,disp) = $A_disp($good,disp)"
                        puts stdout         "        pos = $pos"
                        puts stdout         "grism:  grism_disp\[mm/nm\] grism_lam0\[nm\] filt_lam0\[nm\] filt_fwhm\[nm\] grism_disp_angle\[rad\]"
                        puts stdout [format "        %17.2f %14.2f %13.2f %13.2f %+21.2f" $l_disp [expr {$A_disp($good,cen)*1.0}] $l_lam0 $l_fwhm $disp_ang_rad]
                    }
                }
                # Specify the wavelength limits
                set L_MIN [expr ($A_disp($good,cen) + $A_disp($good,min)) * 0.5]
                set L_MAX [expr ($A_disp($good,cen) + $A_disp($good,max)) * 0.5]
                wave_limit
                wave_config_mask [list $L_MIN $L_MAX grism [$this cget -grism]]
            }
        }

        if { $mask_initialized } {
            if { [string match *GRIS_1400V+18* [$this cget -grism]] && !$note1 } {
                set text "Note: Spectra of  GRIS_1400V+18\n\
                          are offset by 111.4 pixel towards +Y\n\
                          (toward lower slit numbers=TOP)\n"
                if { [$this cget -interactive] } {
                    info_dialog $text .skycat1
                } else {
                    puts stdout $text
                }
                set note1 1
            }
            if { [string match *GRIS_600RI+19* [$this cget -grism]] && !$note2 } {
                set text "Note: Spectra of  GRIS_600RI+19\n\
                          are offset by 272.4 pixel toward +Y\n\
                          (toward lower slit numbers=TOP)\n"
                if { [$this cget -interactive] } {
                    info_dialog $text .skycat1
                } else {
                    puts stdout $text
                }
                set note2 1
            }
            if { [string match *XGRIS* [$this cget -grism]] && !$note3 } {
                set text "Note: Spectra of  [$this cget -grism]\n\
                          are dispersed along Y\n\
                          (vertical)\n"
                if { [$this cget -interactive] } {
                    info_dialog $text .skycat1
                } else {
                    puts stdout $text
                }
                set note3 1
            }
        }
        set last_action "Select Grism"
        catch {SP update}
        tracer LEAVING "end of variable grism"
    }

    #-------------------------
    # Radio button in Camera submenu
    # in layout.cm menu
    public variable camera U {
    #-------------------------
        tracer ENTERING " variable camera"
        global ::m_i
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_Camera_B)] } {
            set m_var [$m_mb.layout.cm entrycget $m_i(Layout_Camera_B) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [string compare $m_value [$this cget -camera]] != 0 } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_Camera_[$this cget -camera])] } {
                    # FORS1 and FORS2 have fixed camera option
                    if { [string compare [$this cget -instrument] "FORS1"] == 0 } {
                        $m_mb.layout.cm invoke $m_i(Layout_Camera_B)
                    } elseif { [string compare [$this cget -instrument] "FORS2"] == 0 } {
                        $m_mb.layout.cm invoke $m_i(Layout_Camera_R)
                    } else {
                        $m_mb.layout.cm invoke $m_i(Layout_Camera_[$this cget -camera])
                    }
                }
            }
        }
        if { $mask_initialized } {
            catch { load_ccd_window }
            catch { reset_mask }
        }
        if { $debug } {
            puts stdout "camera var: [$this cget -camera]"
        }
        set last_action "Select Camera"
        catch {SP update}
        tracer LEAVING "end of variable camera"
    }

    #---------------------------
    # Radio button in Rotation Step submenu
    # in layout.rs menu
    public variable rotangle 5 {
    #---------------------------
        tracer ENTERING " variable rotangle"
        global ::m_i
        set rota [string map {. _} [$this cget -rotangle]]
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_RotationStep_90)] } {
            set m_var [$m_mb.layout.rs entrycget $m_i(Layout_RotationStep_90) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [string compare $m_value [$this cget -rotangle]] != 0 } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_RotationStep_$rota)] } {
                    $m_mb.layout.rs invoke $m_i(Layout_RotationStep_$rota)
                }
            }
        }
        if { $debug } {
            puts stdout "rotangle var: [$this cget -rotangle]"
        }
        tracer LEAVING "end of variable rotangle"
    }

    #------------------------------------
    # Radio button in Color submenu
    # in layout.cs menu
    public variable colorset Red/Yellow {
    #------------------------------------
        tracer ENTERING " variable colorset"
        global ::m_i
        set color [string map {/ _} [$this cget -colorset]]
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_Color_Dark)] } {
            set m_var [$m_mb.layout.cs entrycget $m_i(Layout_Color_Dark) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [string compare $m_value [$this cget -colorset]] != 0 } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_Color_$color)] } {
                    $m_mb.layout.cs invoke $m_i(Layout_Color_$color)
                }
            }
        }
        update_color
        if { $mask_initialized } {
            update_and_replot
        }
        if { $debug } {
            puts stdout "colorset var: [$this cget -colorset]"
        }
        tracer LEAVING "end of variable colorset"
    }

    #---------------------------
    # Transparency check button
    # in layout menu
    public variable graytype 0 {
    #---------------------------
        tracer ENTERING " variable graytype"
        global ::m_i
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_Transparency)] } {
            set m_var [$m_mb.layout entrycget $m_i(Layout_Transparency) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [$this cget -graytype] != $m_value } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_Transparency)] } {
                    # For a check button, invoke toggles it
                    $m_mb.layout invoke $m_i(Layout_Transparency)
                }
            }
        }
        update_color
        if { $mask_initialized } {
            if { [string compare [$this cget -modus] hitms] == 0 ||
                 [string compare [$this cget -modus] mxu] == 0 } {
                fors::MXU_SLIT::transparency [$this cget -graytype]
            }
            update_and_replot
        }
        if { $debug } {
            puts stdout "graytype var: [$this cget -graytype]"
        }
        puts $lfId  "transparency: [$this cget -graytype]"
        tracer LEAVING "end of variable graytype"
    }

    #------------------------------
    # Bad Pixel Map check button
    # in layout menu
    public variable badpixelmap 0 {
    #------------------------------
        tracer ENTERING " variable badpixelmap"
        global ::m_i
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_BadPixelMap)] } {
            set m_var [$m_mb.layout entrycget $m_i(Layout_BadPixelMap) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [$this cget -badpixelmap] != $m_value } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_BadPixelMap)] } {
                    # For a check button, invoke toggles it
                    $m_mb.layout invoke $m_i(Layout_BadPixelMap)
                }
            }
        }

        if { $mask_initialized } {
            if { $badpixelmap } {
                # Load and update BPM
                load_BPM
                update_BPM
            } else {
                # Purge BPM
                $m_canvas delete -tag BPM
            }
        } else {
            # Load BPM
            load_BPM
        }
        if { $debug } {
            puts stdout "badpixelmap var: [$this cget -badpixelmap]"
        }
        puts $lfId  "BPM: [$this cget -badpixelmap]"
        tracer LEAVING "end of variable badpixelmap"
    }

    #------------------------
    # Labels check button
    # in layout menu
    public variable label 1 {
    #------------------------
        tracer ENTERING " variable label"
        global ::m_i
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_Labels)] } {
            set m_var [$m_mb.layout entrycget $m_i(Layout_Labels) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [$this cget -label] != $m_value } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_Labels)] } {
                    # For a check button, invoke toggles it
                    $m_mb.layout invoke $m_i(Layout_Labels)
                }
            }
        }
        if { $mask_initialized } {
            if { [$this cget -label] } {
                update_labels
            }
            if { [string compare [$this cget -modus] hitms] == 0 ||
                 [string compare [$this cget -modus] mxu] == 0 } {
                fors::MXU_SLIT::label [$this cget -label]
            }
            erase
            plot_mask
            if { $badpixelmap } {
                update_BPM
            }
            # because of the brutal erase
        }
        if { $debug } {
            puts stdout "label var: [$this cget -label]"
        }
        puts $lfId  "Label: [$this cget -label]"
        tracer LEAVING "end of variable label"
    }

    #-----------------------
    # Center Slit check button
    # in layout menu
    public variable pics 0 {
    #-----------------------
        tracer ENTERING " variable pics"
        global ::m_i
        # Align option value with corresponding menu button
        if { [info exists m_i(Layout_CenterSlit)] } {
            set m_var [$m_mb.layout entrycget $m_i(Layout_CenterSlit) -variable]
            global $m_var
            set doit [concat set m_value $$m_var]
            eval $doit
            if { [$this cget -pics] != $m_value } {
                # The option was modified, need to align menu button
                if { [info exists m_i(Layout_CenterSlit)] } {
                    # For a check button, invoke toggles it
                    $m_mb.layout invoke $m_i(Layout_CenterSlit)
                }
            }
        }
        catch {SP update}
        # the center-slit option calls the
        # pickObject widget
        if { $pics == 1 } {
            if { !$center && $mask_initialized } {
                pick_star
            }
            set center 1
        } else {
            if { $center && $mask_initialized } {
                set w [$m_skycat component image]
                $w.pick close
            }
            set center 0
        }
        if { $debug } {
            puts stdout "pics var: [$this cget -pics]"
        }
        puts $lfId   "center/slit flag : [$this cget -pics]"
        tracer LEAVING "end of variable pics"
    }

    #------------------------
    public variable debug 0 {
    #------------------------
        if { [$this cget -debug] == 1 } {
            puts "debug var: $debug"
        } else {
            puts "DEBUG OFF"
        }
        catch {C_ref_targ configure -debug [$this cget -debug]}
    }

    #----------------------------
    public variable verbosity 0 {
    #----------------------------
        if { [$this cget -debug] == 1 } {
            puts "verbosity var: $verbosity"
        }
        catch {C_ref_targ configure -verbosity [$this cget -verbosity]}
    }

    #------------------------------
    # If configured to 0 then most
    # dialogues are omited.
    public variable interactive 1 {
    #------------------------------
        if { [$this cget -debug] == 1 } {
            puts "interactive var: $interactive"
        }
        catch {C_ref_targ configure -interactive [$this cget -interactive]}
    }

    #-------------------------
    # check this and throw out
    public variable width  200
    public variable height 200
    #-------------------------

    #-----------------------------
    public variable fors_frame 1 {
    #-----------------------------
        tracer ENTERING " variable fors_frame"
        if { $mask_initialized } {
            set_scale $fors_frame
            reset_mask
        } else {
            # do not reset when called by the constructor
            set_scale $fors_frame
        }
        catch {SP update}
        if { $debug } {
            puts stdout "fors_frame var: [$this cget -fors_frame]"
        }
        puts $lfId   "FORS frame flag: [$this cget -fors_frame]"
        tracer LEAVING "end of variable fors_frame"
    }

    #-----------------------------
    public variable file_name "" {
    #-----------------------------
        tracer ENTERING " variable file_name"
        set file_name [string_delete $file_name " "]
        set file_name [string_delete $file_name "`"]
        set file_name [string_delete $file_name "~"]
        set file_name [string_delete $file_name "!"]
        set file_name [string_delete $file_name "@"]
        set file_name [string_delete $file_name "#"]
        set file_name [string_delete $file_name "\$"]
        set file_name [string_delete $file_name "%"]
        set file_name [string_delete $file_name "^"]
        set file_name [string_delete $file_name "&"]
        set file_name [string_delete $file_name "*"]
        set file_name [string_delete $file_name "-"]
        # set file_name [string_delete $file_name "_"]
        set file_name [string_delete $file_name "+"]
        set file_name [string_delete $file_name "="]
        set file_name [string_delete $file_name "|"]
        set file_name [string_delete $file_name ":"]
        set file_name [string_delete $file_name ";"]
        set file_name [string_delete $file_name "'"]
        set file_name [string_delete $file_name "\""]
        set file_name [string_delete $file_name ","]
        set file_name [string_delete $file_name "."]
        set file_name [string_delete $file_name "/"]
        set file_name [string_delete $file_name "?"]
        set file_name [string_delete $file_name ")"]
        set file_name [string_delete $file_name "("]
        set file_name [string_delete $file_name "\["]
        set file_name [string_delete $file_name "\]"]
        set file_name [string_delete $file_name "\{"]
        set file_name [string_delete $file_name "\}"]
        set file_name [string_delete $file_name "\<"]
        set file_name [string_delete $file_name "\>"]
        set file_name [string trimleft $file_name "0123456789"]
        set file_name [string range $file_name 0 7]
        if { [string compare [$this cget -modus] hitms] == 0 ||
             [string compare [$this cget -modus] mxu] == 0 } {
            if { ![fors::MXU_SLIT::get_mmff] } {
                update_mask_na
            }
        }
        if { $debug } {
            puts stdout "file_name var: [$this cget -file_name]"
        }
        puts $lfId "new file name $file_name"
        tracer LEAVING "end of variable file_name"
    }

    #--------------------------------
    public variable calib_passwd "" {
    #--------------------------------
        tracer ENTERING " variable calib_passwd"
        if { $debug } {
            puts stdout "calib_passwd var: [$this cget -calib_passwd]"
        }
        tracer LEAVING "end of variable calib_passwd"
    }

    #--------------------------------------------------------------------
    public variable int_dir  $env(FIMSROOT); # fims install dir
    public variable isf_dir  $env(ISF_DIR);  # load ISF
    public variable log_dir  $env(LOG_DIR);  # write log files
    public variable set_dir  $env(SET_DIR);  # write setup files
    public variable prep_dir $env(PREP_DIR); # load fits files
    public variable isf_file INS.isf;        # ISF file name
    #--------------------------------------------------------------------


#------------------
# private variables
#------------------

    # versions
    private variable f_v
    private variable s_v
    # logfile name
    private variable lfId
    private variable insId 0
    private variable lochost ""
    private variable F_N mycopy.targ
    # current catalog name and auto slit method
    private variable CatFileName no_catalog
    private variable AutoposMethod ""
    # mask local name for skycat, canvas and image
    private variable m_skycat
    private variable m_canvas
    private variable m_image
    # mask local name for menu bar and menu item index array
    private variable m_mb
    private variable m_mi
    # scale (zoom) of the world/canvas
    private variable canvscale
    # holds the first canvscale used for dispersion frame calc
    private variable refscale 0
    # equinox
    private variable EQX
    private variable EQUINOX
    private variable FWHM_X
    private variable FWHM_Y
    # boundary errors in slits_lengths  [mm]
    private variable s_slit 4
    # the actual position of the mask in world units
    private variable world_x
    private variable world_y
    # the initial position of the mask in world units
    private variable wx
    private variable wy
    # the cordinates of the MOS mask, a big matrix
    private variable mos
    # the BadPixelMap
    private variable BPM
    private variable BPM_counter
    # the cordinates of the reference objects
    private variable ref
    # the grism-OS-filter-dispersion-array
    private variable A_disp
    private variable good 0
    # the world coordinates of the 19 positioned slits
    private variable targw
    # absolute angle of the mask in rad with respect to WCS
    private variable abs_ang_rad 0.0
    # size of the ccd
    private variable ccdsize
    # these coordinates remain constant for both collimators
    private variable x_ccd 300
    # the reference pixel of the image
    private variable icx
    private variable icy
    # the  center of the mask (telescope pointing position on canvas)
    private variable ccx
    private variable ccy
    # initial canvas position of mask = canvas center
    private variable fr_x
    private variable fr_y
    # scale pixel/''
    # here a very preliminary and incorrect formular
    private variable worldscale 1.7
    # old position for shifting
    private variable oldpos
    private variable op

    # the length of the spectrum on the CCD in arcsec
    private variable speclen 50.0
    private variable specmin 50.0
    private variable specmax 50.0
    private variable ccdm_specmin 50.0
    private variable ccdm_specmax 50.0
    private variable specminHR 25.0
    private variable specmaxHR 25.0
    private variable ccdm_specminHR 25.0
    private variable ccdm_specmaxHR 25.0

    # length of the spectrum in pixels
    private variable dispmin 0.0
    private variable dispmax 0.0
    private variable dispminHR 0.0
    private variable dispmaxHR 0.0

    # user defined wavelength range
    private variable L_MIN 400.0
    private variable L_MAX 600.0

    # FORS configuration constants
    private variable XSRMAX
    private variable YSRMAX
    private variable XHRMAX
    private variable YHRMAX
    private variable XSRMIN
    private variable YSRMIN
    private variable XHRMIN
    private variable YHRMIN
    private variable XBLADEA
    private variable XBLADEB

    private variable htslitxmin
    private variable htslitymin
    private variable htslitxmax
    private variable htslitymax

    private variable longslitxmin
    private variable longslitymin
    private variable longslitxmax
    private variable longslitymax

    private variable mosslitmin
    private variable mosslitmax

    private variable polarimslitxmin
    private variable polarimslitymin
    private variable polarimslitxmax
    private variable polarimslitymax

    private variable mxureftp
    private variable mxurefsx
    private variable mxurefsy
    private variable mxurefwd
    private variable mxurefle
    private variable mxurefof
    private variable mxurefro

    private variable pinhxcent
    private variable pinhycent

    private variable long_number
    private variable mask
    private variable slit_coll
    # Number of fixed reference slits that cannot be deleted or changed
    private variable nf_refslit 
    private variable co_range
    private variable cam_range
    private variable gris_range
    private variable filt_range
    private variable osfi_range
    private variable fwhm_range
    private variable lamcent_range
    private variable slit_range
    private variable hmslit_max 0
    private variable htslit_range
    private variable mxu_refslit_range
    private variable ph_range
    private variable echelle_range
    private variable exposure_counter 1
    private variable trapa gray25; # transparency slit blades
    private variable disp_string
    private variable NOG

    # VLT-U1 focal scale in arcsec/mm
    # there is rho=0.5280 theta -1.1288E-6 theta*theta
    # we use  1/0.5280 [arcsec/mm]
    # x [mm] *$focscale = y [arcsec]  (mtoas)
    private variable focscale

    # The values, received from Paranal
    # the focal length of UT1 = 108827.0mm
    # the focal length of UT2 = 108518.8mm
    # focscale = arctan(1/fl)* 180/pi * 3600

    private variable focscale_UT1 1.901679
    private variable focscale_UT2 1.9007288
    # still the nominal values corresponding
    # to focal length of 108827.0mm
    private variable focscale_UT3 1.9007288
    private variable focscale_UT4 1.9007288

    # scale on the CCD  [mm/arcsec]
    # y [arcsec] * CCDscale = x [mm] (astom)
    # this has to be modified:
    # take the Focus value from FORS1.isf
    private variable CCDscale_standard 0.2
    private variable CCDscale_highres  0.1

    # CCD dimensions
    private variable CCDMASTER_XSRMIN
    private variable CCDMASTER_XSRMAX
    private variable CCDSLAVE_XSRMIN
    private variable CCDSLAVE_XSRMAX
    private variable CCDMASTER_YSRMIN
    private variable CCDMASTER_YSRMAX
    private variable CCDSLAVE_YSRMIN
    private variable CCDSLAVE_YSRMAX
    private variable CCDGAP_SR
    private variable CCDMASTER_XHRMIN
    private variable CCDMASTER_XHRMAX
    private variable CCDSLAVE_XHRMIN
    private variable CCDSLAVE_XHRMAX
    private variable CCDMASTER_YHRMIN
    private variable CCDMASTER_YHRMAX
    private variable CCDSLAVE_YHRMIN
    private variable CCDSLAVE_YHRMAX
    private variable CCDGAP_HR
    private variable FIELDSTOP_XMIN
    private variable FIELDSTOP_XMAX
    private variable FIELDSTOP_YMIN
    private variable FIELDSTOP_YMAX
    private variable ccd_frame
    private variable foc_frame


    # pixelsize in mm
    private variable PIX 0.024
    # degrees to radians conversion constant
    private variable deg2rad 0.017453292519943295
    # radians to degrees conversion constant
    private variable rad2deg 57.295779513082320876
    # numeric arcmin_to_degree
    private variable atd 2.7777778E-4
    private variable trigo 1
    # numeric canvas_to_mm
    private variable ctom
    private variable ccdm_ctom
    private variable ccds_ctom
    # fill colors
    private variable color1 red
    private variable color2 yellow
    # outline colors
    private variable color3 red
    private variable color4 yellow

    # maximum permitted number of mxu slits
    # AAG PPRS 10127
    private variable mnomxus 1000

    private variable naggme 1
    private variable center 00
    private variable picked
    private variable lss_ref_ra
    private variable lss_ref_de
    private variable lss_slit_ra
    private variable lss_slit_de
    private variable star_ra
    private variable star_de
    private variable star_ix
    private variable star_iy
    private variable fits_x_scale
    private variable fits_y_scale
    private variable WP
    private variable man_WP 0.0
    private variable CSO
    private variable CRO
    private variable PAM 0
    private variable tcs_pam 0
    private variable reftarg_zone 0
    private variable disp_ang 0
    private variable disp_ang_rad 0
    # private variable mountangle 0
    private variable mountangle 180
    private variable stop 0
    private variable dith_num 1

    private variable echs_flag 0
    private variable dire_flag 0
    private variable wcs_flag 0
    private variable new_open_flag 0
    private variable old_open_flag 0
    private variable warn_flag 0
    # bonsai matrix elements for mm-wcs
    private variable frameless 0
    private variable bonsai_flag 0
    private variable mask_initialized 0

    private variable last_action  ""
    private variable last_save_fn ""
    private variable last_load_fn ""

    private variable sp_mtali 0;     # MOS10ali       only for statuspanel
    private variable sp_nort "";     # NumOfRefTarg   only for statuspanel
    private variable sp_rsc 0;       # RefSlitCounter only for statuspanel
    private variable sp_mid "";      # MaskId         only for statuspanel
    private variable sp_instrume ""; # underlying fits image
    private variable sp_origin   ""; # underlying fits image

    # Used to display popup just once
    private variable note1 0
    private variable note2 0
    private variable note3 0

    # For setup files
    private variable file_header
    private variable paf_header
    }
    # End of the class FORS
}
# End of the namespace

#
#___oOo___
